Package: Gviz
Title: Plotting data and annotation information along genomic
        coordinates
Version: 1.57.1
Authors@R: c(person("Florian", "Hahne", role="aut"),
	 person("Steffen", "Durinck", role="aut"),
	 person("Robert", "Ivanek", role=c("aut", "cre"), 
	        email="robert.ivanek@unibas.ch", 
	        comment=c(ORCID="0000-0002-8403-056X")),
	 person("Arne", "Mueller", role="aut"),
	 person("Steve", "Lianoglou", role="aut"),
	 person("Ge", "Tan", role="aut"),
	 person("Lance", "Parsons", role="aut"),
	 person("Shraddha", "Pai", role="aut"),
	 person("Thomas", "McCarthy", role="ctb"),
	 person("Felix", "Ernst", role="ctb"),
	 person("Mike", "Smith", role="ctb"))
Description: Genomic data analyses requires integrated visualization of
        known genomic information and new experimental data. Gviz uses
        the biomaRt and the rtracklayer packages to perform live
        annotation queries to Ensembl and UCSC and translates this to
        e.g. gene/transcript structures in viewports of the grid
        graphics package. This results in genomic information plotted
        together with your data.
LazyLoad: yes
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
Depends: R (>= 4.3), methods, S4Vectors (>= 0.9.25), IRanges (>=
        1.99.18), GenomicRanges (>= 1.61.1), grid
Imports: XVector (>= 0.5.7), rtracklayer (>= 1.69.1), lattice,
        RColorBrewer, biomaRt (>= 2.11.0), AnnotationDbi (>= 1.27.5),
        Biobase (>= 2.15.3), GenomicFeatures (>= 1.61.4), ensembldb (>=
        2.11.3), BSgenome (>= 1.77.1), Biostrings (>= 2.77.2),
        biovizBase (>= 1.13.8), Rsamtools (>= 2.25.1), latticeExtra (>=
        0.6-26), matrixStats (>= 0.8.14), GenomicAlignments (>=
        1.45.1), cigarillo, Seqinfo, GenomeInfoDb, BiocGenerics (>=
        0.11.3), digest(>= 0.6.8), graphics, grDevices, stats, utils
Suggests: BSgenome.Hsapiens.UCSC.hg19, xml2, BiocStyle, knitr,
        rmarkdown, testthat
biocViews: Visualization, Microarray, Sequencing
URL: https://github.com/ivanek/Gviz
BugReports: https://github.com/ivanek/Gviz/issues
VignetteBuilder: knitr
License: Artistic-2.0
Collate: 'utils.R' 'ImageMap-class.R' 'DisplayPars-class.R'
        'GdObject-class.R' 'ReferenceTrack-class.R'
        'SequenceTrack-class.R' 'RangeTrack-class.R'
        'StackedTrack-class.R' 'NumericTrack-class.R'
        'DataTrack-class.R' 'AlignmentsTrack-class.R'
        'AnnotationTrack-class.R' 'GeneRegionTrack-class.R'
        'BiomartGeneRegionTrack-class.R' 'CustomTrack-class.R'
        'GenomeAxisTrack-class.R' 'Gviz-defunct.R' 'Gviz-deprecated.R'
        'HighlightTrack-class.R' 'IdeogramTrack-class.R'
        'OverlayTrack-class.R' 'UcscTrack.R' 'collapsing.R'
        'datasets.R' 'exportTracks.R' 'grouping.R' 'plotTracks.R'
        'settings.R'
Config/roxygen2/version: 8.0.0
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev libjpeg-dev
        liblzma-dev libpng-dev libuv1-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-07-29 16:11:37 UTC
RemoteUrl: https://github.com/bioc/Gviz
RemoteRef: HEAD
RemoteSha: 2132e60faee98057a4ff849b6738b44e104060ea
NeedsCompilation: no
Packaged: 2026-07-29 21:44:35 UTC; root
Author: Florian Hahne [aut],
  Steffen Durinck [aut],
  Robert Ivanek [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8403-056X>),
  Arne Mueller [aut],
  Steve Lianoglou [aut],
  Ge Tan [aut],
  Lance Parsons [aut],
  Shraddha Pai [aut],
  Thomas McCarthy [ctb],
  Felix Ernst [ctb],
  Mike Smith [ctb]
Maintainer: Robert Ivanek <robert.ivanek@unibas.ch>
Built: R 4.6.1; ; 2026-07-29 21:48:10 UTC; unix

# Generated by roxygen2: do not edit by hand

export(.DisplayPars)
export(.chrName)
export(.doCache)
export(AlignmentsTrack)
export(AnnotationTrack)
export(BiomartGeneRegionTrack)
export(CustomTrack)
export(DataTrack)
export(DetailsAnnotationTrack)
export(DisplayPars)
export(GeneRegionTrack)
export(GenomeAxisTrack)
export(HighlightTrack)
export(IdeogramTrack)
export(OverlayTrack)
export(RNASequenceTrack)
export(SequenceTrack)
export(UcscTrack)
export(addScheme)
export(availableDefaultMapping)
export(availableDisplayPars)
export(clearSessionCache)
export(drawAxis)
export(drawGD)
export(exportTracks)
export(getScheme)
export(plotTracks)
exportClasses(AlignmentsTrack)
exportClasses(AnnotationTrack)
exportClasses(BiomartGeneRegionTrack)
exportClasses(CustomTrack)
exportClasses(DataTrack)
exportClasses(DetailsAnnotationTrack)
exportClasses(DisplayPars)
exportClasses(GdObject)
exportClasses(GeneRegionTrack)
exportClasses(GenomeAxisTrack)
exportClasses(HighlightTrack)
exportClasses(IdeogramTrack)
exportClasses(ImageMap)
exportClasses(NumericTrack)
exportClasses(OverlayTrack)
exportClasses(RangeTrack)
exportClasses(ReferenceAlignmentsTrack)
exportClasses(ReferenceAnnotationTrack)
exportClasses(ReferenceDataTrack)
exportClasses(ReferenceGeneRegionTrack)
exportClasses(ReferenceSequenceTrack)
exportClasses(ReferenceTrack)
exportClasses(SequenceBSgenomeTrack)
exportClasses(SequenceDNAStringSetTrack)
exportClasses(SequenceRNAStringSetTrack)
exportClasses(SequenceTrack)
exportClasses(StackedTrack)
exportMethods("[")
exportMethods("chromosome<-")
exportMethods("displayPars<-")
exportMethods("end<-")
exportMethods("exon<-")
exportMethods("feature<-")
exportMethods("gene<-")
exportMethods("genome<-")
exportMethods("group<-")
exportMethods("identifier<-")
exportMethods("imageMap<-")
exportMethods("stacking<-")
exportMethods("start<-")
exportMethods("strand<-")
exportMethods("symbol<-")
exportMethods("transcript<-")
exportMethods("values<-")
exportMethods("width<-")
exportMethods(as.list)
exportMethods(chromosome)
exportMethods(consolidateTrack)
exportMethods(coords)
exportMethods(displayPars)
exportMethods(drawAxis)
exportMethods(drawGD)
exportMethods(end)
exportMethods(exon)
exportMethods(feature)
exportMethods(gene)
exportMethods(genome)
exportMethods(getPar)
exportMethods(group)
exportMethods(identifier)
exportMethods(imageMap)
exportMethods(initialize)
exportMethods(length)
exportMethods(max)
exportMethods(min)
exportMethods(position)
exportMethods(range)
exportMethods(seqinfo)
exportMethods(seqlevels)
exportMethods(seqnames)
exportMethods(setCoverage)
exportMethods(setPar)
exportMethods(setStacks)
exportMethods(show)
exportMethods(split)
exportMethods(stacking)
exportMethods(stacks)
exportMethods(start)
exportMethods(strand)
exportMethods(subset)
exportMethods(symbol)
exportMethods(tags)
exportMethods(transcript)
exportMethods(values)
exportMethods(width)
import(BiocGenerics)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(Seqinfo)
import(XVector)
import(grid)
import(methods)
importClassesFrom(BSgenome,BSgenome)
importClassesFrom(BSgenome,MaskedBSgenome)
importClassesFrom(Biostrings,BString)
importClassesFrom(Biostrings,BStringSet)
importClassesFrom(Biostrings,DNAString)
importClassesFrom(Biostrings,DNAStringSet)
importClassesFrom(Biostrings,RNAString)
importClassesFrom(Biostrings,RNAStringSet)
importClassesFrom(GenomicFeatures,TxDb)
importClassesFrom(biomaRt,Mart)
importClassesFrom(ensembldb,EnsDb)
importClassesFrom(rtracklayer,UCSCData)
importFrom(BSgenome,bsapply)
importFrom(Biobase,listLen)
importFrom(Biobase,rowMax)
importFrom(Biobase,rowMin)
importFrom(Biostrings,BString)
importFrom(Biostrings,BStringSet)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,DNA_ALPHABET)
importFrom(Biostrings,RNAString)
importFrom(Biostrings,RNAStringSet)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,stackStrings)
importFrom(GenomicAlignments,GAlignments)
importFrom(GenomicAlignments,extractAlignmentRangesOnReference)
importFrom(GenomicAlignments,summarizeJunctions)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rsamtools,BamFile)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,bamWhich)
importFrom(Rsamtools,scanBam)
importFrom(Rsamtools,scanBamFlag)
importFrom(Rsamtools,scanBamHeader)
importFrom(Rsamtools,scanBamWhat)
importFrom(Rsamtools,scanFa)
importFrom(Rsamtools,scanFaIndex)
importFrom(biomaRt,getBM)
importFrom(biomaRt,listAttributes)
importFrom(biomaRt,listDatasets)
importFrom(biomaRt,listFilters)
importFrom(biomaRt,useEnsembl)
importFrom(biomaRt,useMart)
importFrom(biovizBase,getBioColor)
importFrom(cigarillo,cigars_as_ranges_along_ref)
importFrom(cigarillo,project_sequences)
importFrom(digest,digest)
importFrom(grDevices,boxplot.stats)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(grDevices,extendrange)
importFrom(grDevices,hsv)
importFrom(grDevices,rgb2hsv)
importFrom(graphics,par)
importFrom(lattice,current.panel.limits)
importFrom(lattice,panel.abline)
importFrom(lattice,panel.grid)
importFrom(lattice,panel.lines)
importFrom(lattice,panel.points)
importFrom(lattice,panel.polygon)
importFrom(lattice,panel.segments)
importFrom(lattice,panel.text)
importFrom(lattice,panel.xyplot)
importFrom(lattice,trellis.par.get)
importFrom(latticeExtra,panel.horizonplot)
importFrom(matrixStats,colMaxs)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowMins)
importFrom(rtracklayer,BigWigFile)
importFrom(rtracklayer,FastaFile)
importFrom(rtracklayer,GenomicData)
importFrom(rtracklayer,TwoBitFile)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,ucscGenomes)
importFrom(stats,loess.smooth)
importFrom(stats,setNames)
importFrom(utils,assignInNamespace)
importFrom(utils,modifyList)
importFrom(utils,type.convert)
importFrom(utils,write.table)
importMethodsFrom(AnnotationDbi,colnames)
importMethodsFrom(AnnotationDbi,get)
importMethodsFrom(AnnotationDbi,ls)
importMethodsFrom(AnnotationDbi,mget)
importMethodsFrom(AnnotationDbi,tail)
importMethodsFrom(BSgenome,providerVersion)
importMethodsFrom(BSgenome,seqnames)
importMethodsFrom(Biostrings,"seqtype<-")
importMethodsFrom(Biostrings,complement)
importMethodsFrom(Biostrings,consensusMatrix)
importMethodsFrom(Biostrings,consensusString)
importMethodsFrom(Biostrings,seqtype)
importMethodsFrom(Biostrings,unmasked)
importMethodsFrom(GenomicFeatures,"isActiveSeq<-")
importMethodsFrom(GenomicFeatures,cdsBy)
importMethodsFrom(GenomicFeatures,exonsBy)
importMethodsFrom(GenomicFeatures,fiveUTRsByTranscript)
importMethodsFrom(GenomicFeatures,isActiveSeq)
importMethodsFrom(GenomicFeatures,threeUTRsByTranscript)
importMethodsFrom(GenomicFeatures,transcripts)
importMethodsFrom(GenomicFeatures,transcriptsBy)
importMethodsFrom(ensembldb,cdsBy)
importMethodsFrom(ensembldb,exonsBy)
importMethodsFrom(ensembldb,fiveUTRsByTranscript)
importMethodsFrom(ensembldb,threeUTRsByTranscript)
importMethodsFrom(ensembldb,transcripts)
importMethodsFrom(ensembldb,transcriptsBy)
importMethodsFrom(rtracklayer,"tableName<-")
importMethodsFrom(rtracklayer,chrom)
importMethodsFrom(rtracklayer,close)
importMethodsFrom(rtracklayer,getTable)
importMethodsFrom(rtracklayer,import)
importMethodsFrom(rtracklayer,import.2bit)
importMethodsFrom(rtracklayer,import.bed)
importMethodsFrom(rtracklayer,import.bed15)
importMethodsFrom(rtracklayer,import.bedGraph)
importMethodsFrom(rtracklayer,import.bw)
importMethodsFrom(rtracklayer,import.chain)
importMethodsFrom(rtracklayer,import.gff)
importMethodsFrom(rtracklayer,import.gff1)
importMethodsFrom(rtracklayer,import.gff2)
importMethodsFrom(rtracklayer,import.gff3)
importMethodsFrom(rtracklayer,import.ucsc)
importMethodsFrom(rtracklayer,import.wig)
importMethodsFrom(rtracklayer,seqinfo)
importMethodsFrom(rtracklayer,tableNames)
importMethodsFrom(rtracklayer,track)
importMethodsFrom(rtracklayer,trackNames)
importMethodsFrom(rtracklayer,ucscTableQuery)

<img src="vignettes/Gviz-logo.png" align="right" alt="" width="120" />

# _Gviz_ - Plotting data and annotation information along genomic coordinates

## Software status

<!-- badges: start -->
[![Project Status: Active - The project has reached a stable, usable state and is being actively developed.](http://www.repostatus.org/badges/latest/active.svg)](http://www.repostatus.org/#active)
[![How long since the package was first in a released Bioconductor version](https://bioconductor.org/shields/years-in-bioc/Gviz.svg)](https://bioconductor.org/packages/Gviz) 
[![Bioconductor-devel Downloads](https://bioconductor.org/shields/downloads/devel/Gviz.svg)](https://bioconductor.org/packages/stats/bioc/Gviz/)
[![Support site activity in last 6 months: agged questions/avg. answers per question/avg. comments per question/accepted answers, or 0 if no tagged posts](https://bioconductor.org/shields/posts/Gviz.svg)](https://support.bioconductor.org/t/gviz/)
[![License:
Artistic-2.0](https://img.shields.io/badge/License-Artistic%202.0-0298c3.svg)](https://opensource.org/licenses/Artistic-2.0)
[![GitHub issues](https://img.shields.io/github/issues/ivanek/Gviz)](https://github.com/ivanek/Gviz/issues)
<!-- badges: end -->

&nbsp;

|                     | Bioc ([release](https://bioconductor.org/packages/release/bioc/html/Gviz.html)) | Bioc ([devel](https://bioconductor.org/packages/devel/bioc/html/Gviz.html)) |
|:--------------------|----------------------------------------------------------------------------:|--------------------------------------------------------------------------------:|
| OS                  | [![Platforms](https://bioconductor.org/shields/availability/release/Gviz.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/Gviz/) | [![Platforms](https://bioconductor.org/shields/availability/devel/Gviz.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/Gviz/) |
| Bioc Last Update    | [![Bioconductor-release Last Commit](https://bioconductor.org/shields/lastcommit/release/bioc/Gviz.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/Gviz/) | [![Bioconductor-devel Last Commit](https://bioconductor.org/shields/lastcommit/devel/bioc/Gviz.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/Gviz/) |
| Bioc Status         | [![Bioconductor-release Build Status](https://bioconductor.org/shields/build/release/bioc/Gviz.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/Gviz) | [![Bioconductor-devel Build Status](https://bioconductor.org/shields/build/devel/bioc/Gviz.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/Gviz) |
| GitHub Last Commit  | [![GitHub last commit (Bioconductor-release)](https://img.shields.io/github/last-commit/ivanek/Gviz/RELEASE_3_17)](https://github.com/ivanek/Gviz/tree/RELEASE_3_17) | [![GitHub last commit (Bioconductor-devel)](https://img.shields.io/github/last-commit/ivanek/Gviz/devel)](https://github.com/ivanek/Gviz/tree/devel/) |
| GitHub Actions      | [![R build status](https://github.com/ivanek/Gviz/workflows/R-CMD-check-bioc/badge.svg?branch=RELEASE_3_17)](https://github.com/ivanek/Gviz/actions) | [![R build status](https://github.com/ivanek/Gviz/workflows/R-CMD-check-bioc/badge.svg?branch=devel)](https://github.com/ivanek/Gviz/actions) |
| Coverage            | [![Codecov.io (Bioconductor-release)](https://codecov.io/github/ivanek/Gviz/coverage.svg?branch=RELEASE_3_17)](https://codecov.io/gh/ivanek/Gviz/branch/RELEASE_3_17) | [![Codecov.io (Bioconductor-devel)](https://codecov.io/github/ivanek/Gviz/coverage.svg?branch=devel)](https://codecov.io/github/ivanek/Gviz) |

## Authors

- Florian Hahne
- Steffen Durinck
- Robert Ivanek
- Arne Mueller
- Steve Lianoglou
- Ge Tan 
- Lance Parsons
- Shraddha Pai

## Overview

![Gviz UCSC like screenshot](vignettes/Gviz-example.png)

Genomic data analyses requires integrated visualization of known genomic information and new experimental data. Gviz uses the [biomaRt](https://bioconductor.org/packages/biomaRt/) and the [rtracklayer](https://bioconductor.org/packages/rtracklayer/) packages to perform live annotation queries to [Ensembl](https://www.ensembl.org/) and [UCSC](https://genome.ucsc.edu) and translates this to e.g. gene/transcript structures in viewports of the grid graphics package. This results in genomic information plotted together with your data.

## Installation

#### Release version

```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("Gviz", version = "release")
```

#### Developmental version

```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("Gviz", version = "devel")
```

#### Github

```
if (!requireNamespace("remotes", quietly = TRUE))
    install.packages("remotes")
remotes::install_github("ivanek/Gviz")
```
## Usage

For detailed instructions check the package vignette 
([release](https://bioconductor.org/packages/release/bioc/vignettes/Gviz/inst/doc/Gviz.html) 
or 
[developmental](https://bioconductor.org/packages/devel/bioc/vignettes/Gviz/inst/doc/Gviz.html) 
version). Or check the [GitHub.io page](https://ivanek.github.io/Gviz/).

## Citation 

- Hahne F, Ivanek R (2016). "Statistical Genomics: Methods and Protocols." In Mathé E, Davis S (eds.), chapter Visualizing Genomic Data Using Gviz and Bioconductor, 335–351. Springer New York, New York, NY. ISBN 978-1-4939-3578-9, doi: [10.1007/978-1-4939-3578-9_16](https://dx.doi.org/10.1007/978-1-4939-3578-9_16).

## ----init, echo=FALSE, results='hide'-----------------------------------------
library(knitr)

## 1. issue with too large vignette, maybe optimizing png?
## unfortunately it requires additional system libs -> not feasible
##           `pngquant=""`, `optipng = "-o7"` both in `opts_chunk`
## SOLUTION: might be to use `fig.retina = 1` and only set it to 2 (default)
##           for figures which requires higher resolution
##           or even decrease res `dpi = 72`, also both in `opts_chunk`
## 
## knit_hooks$set(optipng = hook_optipng,
##                pngquant = hook_pngquant)
##
## 2. crop=NULL or FALSE =>  fix vignette rendering based on yihui/knitr#1796
## added also error=FALSE to include_graphics

opts_chunk$set(
  collapse = TRUE,
  tidy = FALSE,
  fig.retina = 1,
  ## comment = "#>",
  error = FALSE,
  warning = FALSE,
  message = FALSE,
  crop = NULL                           
)

## check the output type
out_type <- opts_knit$get("rmarkdown.pandoc.to")
if (is.null(out_type))
    out_type <- "html"

## add styling
if (out_type == "html") {
    BiocStyle::markdown()
} else if (out_type == "latex") {
    BiocStyle::latex()
}

## ----vignetteGvizSetup, echo=FALSE, results='hide'----------------------------
source(system.file("scripts/documentation.R", package="Gviz"))
xtabDetails <- details

addParTable <- function(xtabDetails, class, 
                        skip=c("showTitle", "size", "background.title"), 
                        add=NULL, out_type="html") {
  Parameters <- data.frame("Display Parameter"=names(xtabDetails[[class]]),
                           "Description"=xtabDetails[[class]], 
                           check.names=FALSE)
  if(!is.null(add)) {
    Parameters <- cbind(Parameters, add)
  }
  Parameters <- Parameters[order(Parameters[,1]),]  
  sel <- !Parameters[,1] %in% skip
  Parameters <- Parameters[sel,]
  Parameters[,2] <- gsub("\\\\\\code\\{(.*?)\\}", "`\\1`", Parameters[,2])
  Parameters[,2] <- gsub("\\\\\\link[sS]4[Cc]lass\\{(.*?)\\}", "\\1", 
                         Parameters[,2])
  Parameters[,2] <- gsub("\\\\\\link\\{(.*?)\\}", "\\1", Parameters[,2])
  rownames(Parameters) <- NULL
  if (out_type == "html") {
     out <- kable(Parameters)
  } else if (out_type == "latex") {
     out <- kable(Parameters, format="latex", booktabs=TRUE, longtable=TRUE)
  }
  print(out)
  return(invisible())
}

## hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent=TRUE), "try-error")
hasUcscConnection <- FALSE
## oto <- options(timeout=5)
hasBiomartConnection <- FALSE
## hasBiomartConnection <- (!is(try(download.file("http://www.biomart.org", tempfile(), quiet=TRUE)), "try-error") &&
##                          !is(try(biomaRt::listMarts(), silent=TRUE), "try-error"))
## options(timeout=oto)

## ## clear BiomaRt cache
## biomaRt::biomartCacheClear()

## ## Uncommenting this helps when the UCSC server has a hickup but still lets you connect:
## hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent=TRUE), "try-error") && 
##   !is(try(IdeogramTrack(genome="hg19", chromosome=7), silent=TRUE), "try-error")

## ----loadPackage, cache=FALSE-------------------------------------------------
library(Gviz)

## ----AnnotationTrack----------------------------------------------------------
library(GenomicRanges)
data(cpgIslands)
class(cpgIslands)
chr <- as.character(unique(seqnames(cpgIslands)))
gen <- genome(cpgIslands)
atrack <- AnnotationTrack(cpgIslands, name = "CpG")

## ----plotAnnotationTrack, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(atrack)

## ----GenomeAxisTrack----------------------------------------------------------
gtrack <- GenomeAxisTrack()

## ----plotGenomeAxisTrack, fig.width=7.5, fig.height=1.1-----------------------
plotTracks(list(gtrack, atrack))

## ----showIdeogramTrack, eval=FALSE--------------------------------------------
# itrack <- IdeogramTrack(genome = gen, chromosome = chr)

## ----doIdeogramTrack, echo=FALSE, results='hide'------------------------------
if(hasUcscConnection) {
  itrack <- IdeogramTrack(genome = gen, chromosome = chr)
} else{
  data(itrack)
}

## ----plotIdeogramTrack, fig.width=7.5, fig.height=1.5-------------------------
plotTracks(list(itrack, gtrack, atrack))

## ----GeneRegionTrack, fig.width=7.5, fig.height=3-----------------------------
data(geneModels)
grtrack <- GeneRegionTrack(geneModels, genome = gen,
                           chromosome = chr, name = "Gene Model")
plotTracks(list(itrack, gtrack, atrack, grtrack))

## ----zooming, fig.width=7.5, fig.height=2.2-----------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack),
           from = 26700000, to = 26750000)

## ----zooming2, fig.width=7.5, fig.height=3------------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack),
           extend.left = 0.5, extend.right = 1000000)

## ----zooming3, fig.width=7.5, fig.height=3------------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack), 
           extend.left = 0.5, extend.right = 1000000, col = NULL)

## ----zooming4, fig.width=7.5, fig.height=3.1----------------------------------
library(BSgenome.Hsapiens.UCSC.hg19)
strack <- SequenceTrack(Hsapiens, chromosome = chr)
plotTracks(list(itrack, gtrack, atrack, grtrack, strack), 
           from = 26591822, to = 26591852, cex = 0.8)

## ----DataTrack, fig.width=7.5, fig.height=4-----------------------------------
set.seed(255)
lim <- c(26700000, 26750000)
coords <- sort(c(lim[1], 
                 sample(seq(from = lim[1], to = lim[2]), 99), 
                 lim[2]))
dat <- runif(100, min = -10, max = 10)
dtrack <- DataTrack(data = dat, start = coords[-length(coords)],
                    end = coords[-1], chromosome = chr, genome = gen, 
                    name = "Uniform")
plotTracks(list(itrack, gtrack, atrack, grtrack, dtrack), 
           from = lim[1], to = lim[2])

## ----DataTrackHist, fig.width=7.5, fig.height=4-------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack, dtrack), 
           from = lim[1], to = lim[2], type = "histogram")

## ----displayPars1f, fig.width=7.5, fig.height=3-------------------------------
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr, 
                           name = "Gene Model",
                           transcriptAnnotation = "symbol",
                           background.title = "brown")
head(displayPars(grtrack))
displayPars(grtrack) <- list(background.panel = "#FFFEDB", col = NULL)
head(displayPars(grtrack))

plotTracks(list(itrack, gtrack, atrack, grtrack))

## ----displayPars2f, fig.width=7.5, fig.height=3-------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack), 
           background.panel = "#FFFEDB", background.title = "darkblue")

## ----displayPars3-------------------------------------------------------------
dp <- availableDisplayPars(grtrack)
tail(dp)

## ----displayPars4, fig.width=7.5, fig.height=1.5------------------------------
getOption("Gviz.scheme")
scheme <- getScheme()
scheme$GeneRegionTrack$fill <- "salmon"
scheme$GeneRegionTrack$col <- NULL
scheme$GeneRegionTrack$transcriptAnnotation <- "transcript"
addScheme(scheme, "myScheme")
options(Gviz.scheme = "myScheme")
grtrack <- GeneRegionTrack(geneModels, genome = gen,
                           chromosome = chr, name = "Gene Model")
plotTracks(grtrack)
options(Gviz.scheme="default")
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr,
                           name = "Gene Model",
                           transcriptAnnotation = "symbol")

## ----schemes, eval=FALSE------------------------------------------------------
# .GvizSchemes <- list(myScheme = list(
#   GeneRegionTrack=list(fill = "salmon", col = NULL,
#                        transcriptAnnotation = "transcript")))

## ----plottingdirections, fig.width=7.5, fig.height=3--------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack), reverseStrand = TRUE)

## ----GenomeAxisTrackClass1, fig.width=7.5, fig.height=0.75--------------------
axisTrack <- GenomeAxisTrack()
plotTracks(axisTrack, from = 1e6, to = 9e6)

## ----GenomeAxisTrackClass2, fig.width=7.5, fig.height=0.75--------------------
axisTrack <- GenomeAxisTrack(range=IRanges(start = c(2e6, 4e6),
                                           end = c(3e6, 7e6),
                                           names = rep("N-stretch", 2)))
plotTracks(axisTrack, from = 1e6, to = 9e6)

## ----GenomeAxisTrackClass2a, fig.width=7.5, fig.height=0.75-------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, showId = TRUE)

## ----GenomeAxisTrackClass3, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, add53 = TRUE, add35 = TRUE)

## ----GenomeAxisTrackClass4, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, add53 = TRUE,
           add35 = TRUE, littleTicks = TRUE)

## ----GenomeAxisTrackClass5, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, exponent = 4)

## ----GenomeAxisTrackClass6, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, labelPos = "below")

## ----GenomeAxisTrackClass7, fig.width=7.5, fig.height=0.5---------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, scale = 0.5)

## ----GenomeAxisTrackClass8, fig.width=7.5, fig.height=0.5---------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, scale = 0.5, 
           labelPos = "below")

## ----GenomeAxisTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"GenomeAxisTrack", out_type = out_type)

## ----IdeogramTrackClass1Show, eval=FALSE--------------------------------------
# ideoTrack <- IdeogramTrack(genome = "hg19", chromosome = "chrX")
# plotTracks(ideoTrack, from = 85e6, to = 129e6)

## ----IdeogramTrackClass1Do, fig.width=7.5, fig.height=0.5, echo=FALSE, results='hide'----
if(hasUcscConnection) {
  ideoTrack <- IdeogramTrack(genome = "hg19", chromosome = "chrX")
} else{
  data(ideoTrack)
}
plotTracks(ideoTrack, from = 85e6, to = 129e6)

## ----IdeogramTrackClass2, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE)

## ----IdeogramTrackClass3, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE, 
           showBandId = TRUE, cex.bands = 0.5)

## ----IdeogramTrackClass4, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE,
           centromereShape = "circle")

## ----IdeogramTrackClassTable, echo=FALSE, results='asis'----------------------
addParTable(xtabDetails, "IdeogramTrack", out_type = out_type)

## ----DataClass1, fig.width=7.5, fig.height=1.5--------------------------------
data(twoGroups)
dTrack <- DataTrack(twoGroups, name = "uniform")
plotTracks(dTrack)

## ----types, echo=FALSE, results='asis'----------------------------------------
types <- data.frame(Value=c("p", "l", "b", "a", "s", "S", "g", "r", "h", "confint", "smooth", "histogram", "mountain", "polygon", "boxplot", "gradient", "heatmap", "horizon"),
                    Type=c("dot plot", "lines plot", "dot and lines plot", "lines plot of average (i.e., mean) values", "stair steps (horizontal first)",
                           "stair steps (vertical first)", "add grid lines", "add linear regression line", "histogram lines", "confidence intervals for average values", "add loess curve",
                           "histogram (bar width equal to range with)", "'mountain-type' plot relative to a baseline",
                           "'polygon-type' plot relative to a baseline", "box and whisker plot",
                           "false color image of the summarized values", "false color image of the individual values",
                           "Horizon plot indicating magnitude and direction of a change relative to a baseline"))

if (out_type == "html") {
  kable(types)
} else if (out_type == "latex") {
  kable(types, "latex", booktabs = TRUE, longtable = TRUE)
}

## ----typePlots, fig.width=7.5, fig.height=8.5, echo=FALSE, results='hide'-----
pushViewport(viewport(layout = grid.layout(nrow = 9, ncol = 2)))
i <- 1
for(t in types$Value) {
  pushViewport(viewport(layout.pos.col = ((i - 1) %% 2) + 1,
                        layout.pos.row = ((i - 1) %/% 2)+1))
    if(t != "horizon"){
        names(dTrack) <- t
        plotTracks(dTrack, type = t, add = TRUE, 
                   cex.title = 0.8, margin = 0.5)
    } else {
        data(dtHoriz)
        names(dtHoriz) <- "horizon *"
        plotTracks(dtHoriz[8, ], type = "horizon", add = TRUE, 
                   cex.title = 0.8, margin = 0.5, 
                   showAxis = FALSE, horizon.origin = 0.7)
    }
    i <- i + 1
    popViewport(1)
}
popViewport(1)
names(dTrack) <- "uniform"

## ----mutitype, results='hide', fig.width=7.5, fig.height=1.5------------------
plotTracks(dTrack, type = c("boxplot", "a", "g"))

## ----sampNames, fig.width=7.5, fig.height=1.5---------------------------------
colnames(mcols(twoGroups))
plotTracks(dTrack, type = c("heatmap"), showSampleNames = TRUE, 
           cex.sampleNames = 0.6)

## ----grouping, results='hide', fig.width=7.5, fig.height=1.5------------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3),
           type = c("a", "p", "confint"))

## ----typeGroupedPlots, fig.width=7.5, fig.height=6, echo=FALSE, results='hide'----
pushViewport(viewport(layout=grid.layout(nrow=9, ncol=1)))
i <- 1
for(t in c("a", "s", "confint", "smooth", "histogram", "boxplot", "heatmap", "horizon", "hor. type")) {
  pushViewport(viewport(layout.pos.col=((i-1)%%1)+1, layout.pos.row=((i-1)%/%1)+1))
  names(dTrack) <- t
  if(!t %in% c("horizon", "hor. type")) {
    plotTracks(dTrack, type=t, add=TRUE, cex.title=0.8, groups=rep(1:2, each=3), margin=0.5)
  } else if (t == "horizon") {
    plotTracks(dtHoriz[c(1,8),], type="horizon", add=TRUE, cex.title=0.8, margin=0.5, showAxis=FALSE, horizon.origin=0.3,
               groups=1:2)
  } else {
    plotTracks(dTrack, type="histogram", stackedBars=FALSE, add=TRUE, cex.title=0.8, groups=rep(1:2, each=3), margin=0.5)
  }
  i <- i+1
  popViewport(1)
}
popViewport(1)
names(dTrack) <- "uniform"

## ----groupingLegend, results='hide', fig.width=7.5, fig.height=1.5------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3),
           type = c("a", "p"), legend = TRUE)

## ----horizLegend, results='hide', fig.width=7.5, fig.height=1.5---------------
data(dtHoriz)
dtHoriz <- dtHoriz[1:6, ]
plotTracks(dtHoriz, type = "horiz", groups = rownames(values(dtHoriz)),
           showSampleNames = TRUE, cex.sampleNames = 0.6, separator = 1)

## ----filedt1, fig.width=7.5, fig.height=1-------------------------------------
bgFile <- system.file("extdata/test.bedGraph", package = "Gviz")
dTrack2 <- DataTrack(range = bgFile, genome = "hg19", type = "l", 
                     chromosome = "chr19", name = "bedGraph")
class(dTrack2)
plotTracks(dTrack2)

## ----filedt2------------------------------------------------------------------
library(rtracklayer)
dTrack3 <-  DataTrack(range = bgFile, genome = "hg19", type = "l", 
                      chromosome = "chr19", name = "bedGraph",
                      importFunction = function(file) import(con=file))
identical(dTrack2, dTrack3)

## ----filedt3, fig.width=7.5, fig.height=1-------------------------------------
bamFile <- system.file("extdata/test.bam", package = "Gviz")
dTrack4 <- DataTrack(range = bamFile, genome = "hg19", type = "l", 
                     name = "Coverage", window = -1, 
                     chromosome = "chr1")
class(dTrack4)
dTrack4
plotTracks(dTrack4, from = 189990000, to = 190000000)

## ----filedt4, fig.width=7.5, fig.height=1-------------------------------------
plotTracks(dTrack4, chromosome = "chr1", from = 189891483, to = 190087517)

## ----filedt5------------------------------------------------------------------
myImportFun <- function(file, selection){
    ## do something here
}
DataTrack(range = bamFile, genome = "hg19", type = "l",
          name = "Coverage", window = -1, chromosome = "chr1",
          importFunction = myImportFun, stream = TRUE)

## ----biggerdata, results='hide', fig.width=7.5, fig.height=1.5----------------
dat <- sin(seq(pi, 10*pi, len=500))
dTrack.big <- DataTrack(start = seq(1, 100000, len = 500), width = 15, 
                        chromosome = "chrX", genome = "hg19", 
                        name = "sinus",
                        data = sin(seq(pi, 5 * pi, len = 500)) * 
                          runif(500, 0.5, 1.5))
plotTracks(dTrack.big, type="hist")

## ----aggregation, results='hide', fig.width=7.5, fig.height=1.5---------------
plotTracks(dTrack.big, type = "hist", window = 50)

## ----aggregation2, results='hide', fig.width=7.5, fig.height=1.5--------------
plotTracks(dTrack.big, type = "hist", window = -1, windowSize = 2500)

## ----transformation, results='hide', fig.width=7.5, fig.height=1.5------------
plotTracks(dTrack.big, type = "l", 
           transformation = function(x) { x[x < 0] <- 0; x })

## ----groupingAv1, results='hide', fig.width=7.5, fig.height=1.5---------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3), 
           type = c("b"), aggregateGroups = TRUE)

## ----groupingAv2, results='hide', fig.width=7.5, fig.height=1.5---------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3), 
           type = c("b"), aggregateGroups = TRUE, aggregation = "max")

## ----DataTrackClassTable, echo=FALSE, results='asis'--------------------------
addParTable(xtabDetails,"DataTrack", out_type = out_type)

## ----anntrack1, results='hide', fig.width=7.5, fig.height=0.5-----------------
aTrack <- AnnotationTrack(start = c(10, 40, 120), width = 15, 
                          chromosome = "chrX", 
                          strand = c("+", "*", "-"),
                          id = c("Huey", "Dewey", "Louie"), 
                          genome = "hg19", name = "foo")
plotTracks(aTrack)

## ----anntrack2, results='hide', fig.width=7.5, fig.height=0.5-----------------
plotTracks(aTrack, shape = "box", featureAnnotation = "id")

## ----anntrack3, results='hide', fig.width=7.5, fig.height=0.5-----------------
plotTracks(aTrack, shape = "ellipse", featureAnnotation = "id", 
           fontcolor.feature = "darkblue")

## ----anntrack4f, results='hide', fig.width=7.5, fig.height=0.5----------------
aTrack.groups <- AnnotationTrack(start = c(50, 180, 260, 460, 860, 1240), 
                                 width = c(15, 20, 40, 100, 200, 20),
                                 chromosome = "chrX",
                                 strand = rep(c("+", "*", "-"), 
                                              c(1, 3, 2)),
                                 group = rep(c("Huey", "Dewey", "Louie"), 
                                             c(1, 3, 2)),
                                 genome = "hg19", name = "foo")
plotTracks(aTrack.groups, groupAnnotation = "group")

## ----anntrack4af, results='hide', fig.width=7.5, fig.height=0.5---------------
plotTracks(aTrack.groups, groupAnnotation = "group", 
           just.group = "right")

## ----anntrack4bf, results='hide', fig.width=7.5, fig.height=0.5---------------
plotTracks(aTrack.groups, groupAnnotation = "group", 
           just.group = "above")

## ----stacking1, results='hide', fig.width=7.5, fig.height=0.5-----------------
aTrack.stacked <- AnnotationTrack(start = c(50, 180, 260, 800, 600, 1240),
                                  width = c(15, 20, 40, 100, 500, 20),
                                  chromosome = "chrX",
                                  strand = "*",
                                  group = rep(c("Huey", "Dewey", "Louie"), 
                                              c(1, 3, 2)),
                                  genome = "hg19", name = "foo")
plotTracks(aTrack.stacked, groupAnnotation="group")

## ----stacking2, results='hide', fig.width=7.5, fig.height=0.5-----------------
plotTracks(aTrack.stacked, stacking = "dense")

## ----features-----------------------------------------------------------------
feature(aTrack.stacked)
feature(aTrack.stacked) <- c("foo", "bar", "bar", "bar", "no", "no")

## ----featuresIdPlot, results='hide', fig.width=7.5, fig.height=0.5------------
plotTracks(aTrack.stacked, featureAnnotation = "feature", 
           groupAnnotation = "feature", fontcolor.feature = 1, 
           cex.feature = 0.7)

## ----featuresPlotf, results='hide', fig.width=7.5, fig.height=0.5-------------
plotTracks(aTrack.stacked, groupAnnotation = "group", 
           foo = "darkred", bar = "darkgreen")

## ----overplotting, results='hide', fig.width=7.5, fig.height=0.75-------------
data("denseAnnTrack")
plotTracks(denseAnnTrack, showOverplotting = TRUE)

## ----collapse1f, results='hide', fig.width=7.5, fig.height=0.85---------------
data(collapseTrack)
plotTracks(ctrack)

## ----collapse2f, results='hide', fig.width=7.5, fig.height=0.85---------------
plotTracks(ctrack, min.width = 1)

## ----collapse3f, results='hide', fig.width=7.5, fig.height=0.85---------------
plotTracks(ctrack, min.width = 1, collapse = TRUE)

## ----collapse4f, results='hide', fig.width=7.5, fig.height=0.85---------------
plotTracks(ctrack, min.width = 3, min.distance = 5, collapse = TRUE)

## ----collapse5f, results='hide', fig.width=7.5, fig.height=0.65---------------
plotTracks(ctrack, min.width = 3, min.distance = 5, collapse = TRUE,
           mergeGroups = TRUE, extend.left = 0.1)

## ----fileat1, fig.width=7.5, fig.height=1-------------------------------------
aTrack2 <- AnnotationTrack(range = bamFile, genome = "hg19", 
                           name = "Reads", chromosome = "chr1")
class(aTrack2)
aTrack2
plotTracks(aTrack2, from = 189995000, to = 190000000)

## ----fileat2, fig.width=7.5, fig.height=1-------------------------------------
aTrack3 <- AnnotationTrack(range = bamFile, genome = "hg19", 
                           name = "Reads", chromosome = "chr1", 
                           group = "id")
aTrack3
plotTracks(aTrack3, from = 189995000, to = 190000000)

## ----fileat3------------------------------------------------------------------
availableDefaultMapping(bamFile, "AnnotationTrack")

## ----fileat4, fig.width=7.5, fig.height=2-------------------------------------
plotTracks(list(dTrack4, aTrack2), from = 189990000, to = 190000000)

## ----AnnotationTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"AnnotationTrack", out_type = out_type)

## ----generegtrackf, results='hide', fig.width=7.5, fig.height=1.5-------------
data(geneModels)
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr, 
                           name = "foo")
head(gene(grtrack))
head(transcript(grtrack))
head(exon(grtrack))
head(symbol(grtrack))
plotTracks(grtrack)

## ----generegtrack2af, results='hide', fig.width=7.5, fig.height=2.5-----------
plotTracks(grtrack, transcriptAnnotation = "symbol")

## ----generegtrack2bf, results='hide', fig.width=7.5, fig.height=2.5-----------
plotTracks(grtrack, transcriptAnnotation = "transcript")

## ----generegtrack2cf, results='hide', fig.width=7.5, fig.height=1-------------
plotTracks(grtrack, exonAnnotation = "exon", extend.left = -0.8, 
           fontcolor.exon = 1)

## ----generegtrack3f, results='hide', fig.width=7.5, fig.height=1--------------
plotTracks(grtrack, collapseTranscripts = TRUE, shape = "arrow", 
           transcriptAnnotation = "symbol")

## ----generegtrack3g, results='hide', fig.width=7.5, fig.height=1--------------
plotTracks(grtrack, collapseTranscripts = "longest", shape = "arrow", 
           transcriptAnnotation = "symbol")

## ----generegtrack3h, results='hide', fig.width=7.5, fig.height=1--------------
plotTracks(grtrack, collapseTranscripts = "meta", shape = "arrow", 
           transcriptAnnotation = "symbol")

## ----tdb2grt1-----------------------------------------------------------------
library(GenomicFeatures)
samplefile <- system.file("extdata", "hg19_knownGene_sample.sqlite", 
                          package = "GenomicFeatures")
txdb <- loadDb(samplefile)
GeneRegionTrack(txdb)

## ----tdb2grt2-----------------------------------------------------------------
txTr <- GeneRegionTrack(txdb, chromosome = "chr6", 
                        start = 35000000,  end = 40000000)

## ----generegtrack4f, results='hide', fig.width=7.5, fig.height=1--------------
feature(txTr)
plotTracks(txTr)

## ----generegtrack4g, eval=FALSE, echo=FALSE, results='hide', fig.width=7.5, fig.height=1----
# library(org.Hs.eg.db)
# symbol(txTr) <- mapIds(org.Hs.eg.db,
#                        keys=sub("\\.\\d+$", "", gene(txTr)),
#                        keytype="ENSEMBL", column="SYMBOL")
# symbol(txTr) <- ifelse(is.na(symbol(txTr)), gene(txTr), symbol(txTr))
# plotTracks(txTr, transcriptAnnotation = "symbol")

## ----ensDb, eval=FALSE--------------------------------------------------------
# library(ensembldb)
# library(EnsDb.Hsapiens.v75)
# 
# edb <- EnsDb.Hsapiens.v75
# seqlevelsStyle(edb) <- "UCSC"
# 
# eTrack <- GeneRegionTrack(edb, chromosome = "chr6",
#                           start = 35000000, end = 40000000)
# 
# plotTracks(eTrack)

## ----GeneRegionTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"GeneRegionTrack", out_type = out_type)

## ----BiomartGeneRegionTrackShow, eval=FALSE-----------------------------------
# library(biomaRt)
# bm <- useEnsembl(host = "https://grch37.ensembl.org",
#               biomart = "ENSEMBL_MART_ENSEMBL",
#               dataset = "hsapiens_gene_ensembl")
# biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr,
#                                     start = 20e6, end = 21e6,
#                                     name = "ENSEMBL", biomart = bm)
# plotTracks(biomTrack)

## ----BiomartGeneRegionTrackDo, echo=FALSE, results='hide', fig.width=7.5, fig.height=1.25----
library(biomaRt)
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org", 
                biomart = "ENSEMBL_MART_ENSEMBL", 
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                      start = 20e6, end = 21e6,
                                      name = "ENSEMBL", biomart = bm)
} else {
    data(biomTrack)
    biomTrack <- as(biomTrack, "GeneRegionTrack")
}
plotTracks(biomTrack)

## ----BiomartGeneRegionTrackCol, fig.width=7.5, fig.height=1.25----------------
plotTracks(biomTrack, col.line = NULL, col = NULL)

## ----BiomartGeneRegionTrackHeight, fig.width=7.5, fig.height=1.25-------------
plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)

## ----BiomartGeneRegionTrackFilterShow, eval=FALSE-----------------------------
# bm <- useEnsembl(host = "https://grch37.ensembl.org",
#               biomart = "ENSEMBL_MART_ENSEMBL",
#               dataset = "hsapiens_gene_ensembl")
# biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr,
#                                     start = 20e6, end = 21e6,
#                                     name = "ENSEMBL",
#                                     filter = list(with_refseq_mrna=TRUE),
#                                     biomart = bm)
# plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)

## ----BiomartGeneRegionTrackFilterDo, fig.width=7.5, fig.height=1.25, echo=FALSE, results='hide'----
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org",
                biomart = "ENSEMBL_MART_ENSEMBL",
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                      start = 20e6, end = 21e6,
                                      name = "ENSEMBL",
                                      filter = list(with_refseq_mrna=TRUE), 
                                      biomart = bm)
  plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)
} else {
  data(biomTrack2)
  biomTrack2 <- as(biomTrack2, "GeneRegionTrack")
  plotTracks(biomTrack2, col.line = NULL, col = NULL, stackHeight = 0.3)
}

## ----BiomartGeneRegionTrackSymbolShow, eval=FALSE-----------------------------
# bm <- useEnsembl(host = "https://grch37.ensembl.org",
#               biomart = "ENSEMBL_MART_ENSEMBL",
#               dataset = "hsapiens_gene_ensembl")
# biomTrack <- BiomartGeneRegionTrack(genome = "hg19", name = "ENSEMBL",
#                                     symbol = "ABCB5", biomart = bm)
# plotTracks(biomTrack, transcriptAnnotation = "symbol")

## ----BiomartGeneRegionTrackSymbolDo, fig.width=7.5, fig.height=1.25, echo=FALSE, results='hide'----
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org", 
                biomart = "ENSEMBL_MART_ENSEMBL", 
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", name = "ENSEMBL", 
                                      symbol = "ABCB5", biomart = bm)
} else {
    ranges(biomTrack) <- ranges(biomTrack)[symbol(biomTrack) == "ABCB5"]
}
plotTracks(biomTrack, transcriptAnnotation="symbol")

## ----BiomartGeneRegionTrackCustom, fig.width=7.5, fig.height=1.25, eval=FALSE, echo=FALSE----
# library(biomaRt)
# bm <- useEnsembl(host="dec2012.archive.ensembl.org", biomart="ENSEMBL_MART_ENSEMBL",
#               dataset="hsapiens_gene_ensembl")
# fm <- Gviz:::.getBMFeatureMap()
# fm[["symbol"]] <- "external_gene_id"
# biomTrack <- BiomartGeneRegionTrack(genome="hg19", chromosome="chr7", start=20e6, end=21e6,name="ENSEMBL",
#                                     featureMap=fm, biomart=bm)
# plotTracks(biomTrack, col.line=NULL, col=NULL, stackHeight=0.3)

## ----BiomartGeneRegionTrackClassTable, echo=FALSE, results='asis'-------------
addInfo <- t(data.frame(displayPars(biomTrack, names(details[["BiomartGeneRegionTrack"]]))))
colnames(addInfo) <- "Color"
addParTable(xtabDetails,"BiomartGeneRegionTrack", add=addInfo, out_type = out_type)

## ----DetailsAnnotationTrack1--------------------------------------------------
library(GenomicRanges)
probes <- GRanges(seqnames = "chr7", ranges = IRanges(
      start = c(2000000, 2070000, 2100000, 2160000),
      end = c(2050000, 2130000, 2150000, 2170000)), 
  strand = c("-", "+", "-", "-"))

## ----DetailsAnnotationTrack2--------------------------------------------------
methylation <- matrix(c(rgamma(400, 1)),
                  ncol = 100,
                  dimnames = list(paste("probe", 1:4, sep = ""), NULL))
methylation[, 51:100] <- methylation[, 51:100] + 0:3
sgroups <- rep(c("grp1", "grp2"), each = 50)

## ----DetailsAnnotationTrack3--------------------------------------------------
library(lattice)
details <- function(identifier, ...) {
  d <- data.frame(signal = methylation[identifier, ], group = sgroups)
  print(densityplot(~signal, group = group, data = d, 
                    main = list(label = identifier, cex = 0.7),
                    scales = list(draw = FALSE, x = list(draw = TRUE)), 
                    ylab = "", xlab = ""), 
        newpage = FALSE, prefix = "plot")
}

## ----DetailsAnnotationTrack4, results='hide', fig.width=7.5, fig.height=5-----
deTrack <- AnnotationTrack(range = probes, genome = "hg19", 
                           chromosome = 7, id = rownames(methylation),
                           name = "probe details", stacking = "squish",
                           fun = details)
plotTracks(deTrack)

## ----DetailsAnnotationTrack5--------------------------------------------------
selFun <- function(identifier, start, end, track, GdObject, ...){
    gcount <- table(group(GdObject))
    ## This computes the width of 2 pixels in genomic coordinates
    pxRange <- Gviz:::.pxResolution(min.width = 20, coord = "x")
    return((end - start) < pxRange && gcount[identifier] == 1)
}

## ----DetailsAnnotationTrack6--------------------------------------------------
detFun <- function(identifier, GdObject.original, ...){
  plotTracks(list(GenomeAxisTrack(scale = 0.3, size = 0.2, cex = 0.7), 
        GdObject.original[group(GdObject.original) == identifier]),
        add = TRUE, showTitle = FALSE)
}

## ----DetailsAnnotationTrack7f, results='hide', fig.width=7.5, fig.height=2----
data(geneDetails)
deTrack2 <- AnnotationTrack(geneDetails, fun = detFun, 
                            selectFun = selFun,
                            groupDetails = TRUE, details.size = 0.5, 
                            detailsConnector.cex = 0.5, 
                            detailsConnector.lty = "dotted",
                            shape = c("smallArrow", "arrow"), 
                            groupAnnotation = "group")
plotTracks(deTrack2, extend.left = 90000)

## ----DetailsAnnotationTrack8, results='hide', fig.width=7.5, fig.height=3-----
plotTracks(deTrack, details.size = 0.75, detailsConnector.pch = NA, 
           detailsConnector.col = "darkred", 
           detailsBorder.fill = "#FFE3BF", 
           detailsBorder.col = "darkred", shape ="box", 
           detailsConnector.lty = "dotted")

## ----DetailsAnnotationTrackClassTableSec, echo=FALSE, results='asis'----------
addParTable(xtabDetails,"DetailsAnnotationTrack", out_type = out_type)

## ----SequenceTrack1-----------------------------------------------------------
library(BSgenome.Hsapiens.UCSC.hg19)
sTrack <- SequenceTrack(Hsapiens)
sTrack

## ----SequenceTrack2, results='hide', fig.width=7.5, fig.height=0.5------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050)

## ----SequenceTrack3, results='hide', fig.width=7.5, fig.height=0.5------------
fcol <- c(A="darkgray", C="darkgray", T="darkgray", G="darkgray")
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050, 
           fontcolor = fcol)

## ----SequenceTrack4, results='hide', fig.width=7.5, fig.height=0.5------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050,
           add53 = TRUE)

## ----SequenceTrack5, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050, 
           add53 = TRUE, complement = TRUE)

## ----SequenceTrack6, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20100)

## ----SequenceTrack7, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 201000)

## ----SequenceTrack8, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20100, cex = 0.5)

## ----SequenceTrackClassTableSec, echo=FALSE, results='asis'-------------------
addParTable(xtabDetails,"SequenceTrack", out_type = out_type)

## ----alignmentstrack_1_do, echo=FALSE, results='hide'-------------------------
afrom <- 2960000
ato <- 3160000
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "gapped.bam"), 
  isPaired = TRUE)
data(alTrackGenes)

## ----alignmentstrack_1_show, eval=FALSE---------------------------------------
# afrom <- 2960000
# ato <- 3160000
# alTrack <- AlignmentsTrack(
#   system.file(package = "Gviz", "extdata", "gapped.bam"),
#   isPaired = TRUE)
# bmt <- BiomartGeneRegionTrack(genome = "hg19", chromosome = "chr12",
#                               start = afrom, end = ato,
#                               filter = list(with_ox_refseq_mrna = TRUE),
#                               stacking = "dense")

## ----alignmentstrack_2, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom, to = ato, 
           chromosome = "chr12")

## ----alignmentstrack_3, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom, to = ato,
           chromosome = "chr12", min.height = 0, 
           coverageHeight = 0.08, minCoverageHeight = 0)

## ----alignmentstrack_4, results='hide', fig.width=7.5, fig.height=2-----------
plotTracks(c(alTrack, bmt), from = afrom, to = ato, 
           chromosome = "chr12", type = "coverage")

## ----alignmentstrack_5, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom + 12700, 
           to = afrom + 15200, chromosome = "chr12")

## ----alignmentstrack_5_1, results='hide', fig.width=7.5, fig.height=2---------
plotTracks(c(bmt, alTrack), from = afrom + 12700, 
           to = afrom + 15200, chromosome = "chr12",
           type = c("coverage", "sashimi"))

## ----alignmentstrack_5_2, results='hide', fig.width=7.5, fig.height=2---------
introns <- GRanges("chr12", IRanges(start = c(2973662, 2973919),
                                    end = c(2973848, 2974520)))
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12", type = c("coverage", "sashimi"), 
           sashimiFilter = introns)

## ----alignmentstrack_5_3, results='hide', fig.width=7.5, fig.height=2---------
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12", type = c("coverage", "sashimi"), 
           sashimiFilter = introns, sashimiFilterTolerance = 5L)

## ----alignmentstrack_6, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200,
           chromosome = "chr12", reverseStacking = TRUE,
           col.mates = "purple", col.gap = "orange", type = "pileup")

## ----alignmentstrack_6_1, results='hide', fig.width=7.5, fig.height=5---------
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "gapped.bam"), 
  isPaired = FALSE)
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12")

## ----alignmentstrack_7, results='hide', fig.width=7.5, fig.height=5-----------
afrom <- 44945200
ato <- 44947200
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "snps.bam"), isPaired = TRUE)
plotTracks(alTrack, chromosome = "chr21", from = afrom, to = ato)

## ----alignmentstrack_8, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(alTrack, sTrack), chromosome = "chr21", 
           from = afrom, to = ato)

## ----alignmentstrack_9, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(alTrack, sTrack), chromosome = "chr21",
           from = 44946590, to = 44946660)

## ----alignmentstrack_10, results='hide', fig.width=7.5, fig.height=5----------
plotTracks(c(alTrack, sTrack), chromosome = "chr21", from = 44946590, 
           to = 44946660, cex = 0.5, min.height = 8)

## ----alignmentstrack_11, results='hide', fig.width=7.5, fig.height=5----------
indelTrack1 <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "indels.bam"), 
  name = "Standard")
indelTrack2 <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "indels.bam"), 
  showIndels=TRUE, name="Indels")
plotTracks(c(indelTrack1, indelTrack2), 
           chromosome = "chr2", from = 126442000, to = 126453000)

## ----AlignmentsTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"AlignmentsTrack", out_type = out_type)

## ----UCSC1, echo=FALSE,fig.cap="A screen shot of a UCSC genome browser view around the FMR1 locus on the mouse chromosome."----
include_graphics("ucsc1.png")

## ----UCSC2, echo=FALSE, fig.cap="A screen shot of a UCSC table browser view on the UCSC Known Genes track."----
include_graphics("ucsc2.png")

## ----ucscTrack1, eval=FALSE---------------------------------------------------
# from <- 65921878
# to <- 65980988
# knownGenes <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                         track = "UCSC Genes", table="knownGene",
#                         from = from, to = to,
#                         trackType = "GeneRegionTrack",
#                         rstarts = "exonStarts", rends = "exonEnds",
#                         gene = "name", symbol = "name",
#                         transcript = "name", strand = "strand",
#                         fill = "#8282d2", name = "UCSC Genes")

## ----ucscTrack2, eval=FALSE---------------------------------------------------
# refGenes <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                       track="Other RefSeq", table = "xenoRefGene",
#                       from = from, to = to,
#                       trackType = "GeneRegionTrack",
#                       rstarts = "exonStarts", rends = "exonEnds",
#                       gene = "name",  symbol = "name2",
#                       transcript = "name", strand = "strand",
#                       fill = "#8282d2", stacking = "dense",
#                       name = "Other RefSeq")
# 
# ensGenes <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                       track="Ensembl Genes", table = "ensGene",
#                       from = from, to = to,
#                       trackType = "GeneRegionTrack",
#                       rstarts = "exonStarts", rends = "exonEnds",
#                       gene = "name", symbol = "name2",
#                       transcript = "name", strand = "strand",
#                       fill = "#960000", name = "Ensembl Genes")

## ----ucscTrack3, eval=FALSE---------------------------------------------------
# cpgIslands <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                         track="CpG Islands", table = "cpgIslandExt",
#                         from = from, to = to,
#                         trackType = "AnnotationTrack",
#                         start = "chromStart", end = "chromEnd",
#                         id = "name", shape = "box", fill = "#006400",
#                         name = "CpG Islands")
# 
# snpLocations <-  UcscTrack(genome = "mm9", chromosome = "chrX",
#                            track="SNPs (128)", table = "snp128",
#                            from = from, to = to,
#                            trackType = "AnnotationTrack",
#                            start = "chromStart", end = "chromEnd",
#                            id = "name", feature = "func",
#                            strand = "strand", shape = "box",
#                            stacking = "dense", fill = "black",
#                            name = "SNPs")

## ----ucscTrack4, eval=FALSE---------------------------------------------------
# conservation <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                           track = "Conservation",
#                           table = "phyloP30wayPlacental",
#                           from = from, to = to, trackType = "DataTrack",
#                           start = "start", end = "end", data = "value",
#                           type = "hist", window = "auto",
#                           col.histogram = "darkblue",
#                           fill.histogram = "darkblue",
#                           ylim = c(-3.7, 4), name = "Conservation")
# 
# gcContent <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                        track = "GC Percent", table = "gc5Base",
#                        from = from, to = to, trackType = "DataTrack",
#                        start = "start", end = "end", data = "value",
#                        type = "hist", window = -1, windowSize = 1500,
#                        fill.histogram = "black", col.histogram = "black",
#                        ylim = c(30, 70), name = "GC Percent")

## ----ucscTrack5, eval=FALSE---------------------------------------------------
# axTrack <- GenomeAxisTrack()
# idxTrack <- IdeogramTrack(genome="mm9", chromosome="chrX")

## ----ucscTrackLoad, echo=FALSE, results='hide'--------------------------------
data(ucscItems)

## ----ucscTrack6, results='hide', fig.width=7.5, fig.height=5.5----------------
plotTracks(list(idxTrack, axTrack, knownGenes, refGenes, ensGenes, 
                cpgIslands, gcContent, conservation, snpLocations), 
           from = from, to = to, showTitle = FALSE)

## ----HighlightTrack, fig.width=7.5, fig.height=4------------------------------
ht <- HighlightTrack(trackList = list(atrack, grtrack, dtrack),
                     start = c(26705000, 26720000), width = 7000,
                     chromosome = 7)
plotTracks(list(itrack, gtrack, ht), from = lim[1], to = lim[2])

## ----HighlightTrack2, fig.width=7.5, fig.height=4-----------------------------
ht1 <- HighlightTrack(trackList=list(itrack, gtrack, atrack), 
                      start = c(26705000, 26720000), width = 7000, 
                      chromosome = 7)
ht2 <- HighlightTrack(trackList = dtrack, start = c(26705000, 26720000),
                      width = 7000, chromosome = 7)
plotTracks(list(ht1, grtrack, ht2), from = lim[1], to = lim[2])

## ----HighlightTrackClassTable, echo=FALSE, results='asis'---------------------
addParTable(xtabDetails,"HighlightTrack", out_type = out_type)

## ----OverlayTrack, fig.width=7.5, fig.height=3--------------------------------
dat <- runif(100, min = -2, max = 22)
dtrack2 <- DataTrack(data = dat, start = coords[-length(coords)], 
                end = coords[-1], chromosome = chr, genome = gen, 
                name = "Uniform2",  groups = factor("sample 2", 
                  levels = c("sample 1", "sample 2")), legend = TRUE)
displayPars(dtrack) <- list(groups = factor("sample 1", 
              levels = c("sample 1", "sample 2")), legend = TRUE)
ot <- OverlayTrack(trackList=list(dtrack2, dtrack))
ylims <- extendrange(range(c(values(dtrack), values(dtrack2))))
plotTracks(list(itrack, gtrack, ot), from = lim[1], to = lim[2], 
           ylim = ylims, type = c("smooth", "p"))

## ----OverlayTrack2, fig.width=7.5, fig.height=3-------------------------------
displayPars(dtrack) <- list(alpha.title = 1, alpha = 0.5)
displayPars(dtrack2) <- list(alpha.title = 1, alpha = 0.5)
ot <- OverlayTrack(trackList = list(dtrack, dtrack2))
plotTracks(list(itrack, gtrack, ot), from = lim[1], to = lim[2], 
           ylim = ylims, type = c("hist"), window = 30)

## ----multPlot1----------------------------------------------------------------
chroms <- c("chr1", "chr2", "chr3", "chr4")
maTrack <- AnnotationTrack(range=GRanges(seqnames = chroms, 
    ranges = IRanges(start = 1,  width = c(100, 400, 200,1000)),
    strand = c("+", "+", "-", "+")), genome = "mm9", 
    chromosome = "chr1", name = "foo")

mdTrack <- DataTrack(
  range = GRanges(seqnames = rep(chroms, c(10, 40, 20, 100)),
                  ranges = IRanges(start = c(seq(1, 100, len = 10),
                                             seq(1, 400, len = 40), 
                                             seq(1, 200, len = 20),
                                             seq(1, 1000, len = 100)), 
                                   width = 9), values = runif(170)),
  data = "values", chromosome = "chr1", genome = "mm9", name = "bar")

## ----multPlot2----------------------------------------------------------------
mgTrack <- GenomeAxisTrack(scale = 50, labelPos = "below", exponent = 3)
chromosome(itrack) <- "chr1"

## ----multPlot3, results='hide', fig.width=7.5, fig.height=4-------------------
ncols <- 2
nrows <- length(chroms) %/% ncols
grid.newpage()
pushViewport(viewport(layout = grid.layout(nrows, ncols)))
for(i in seq_along(chroms)) {
  pushViewport(viewport(layout.pos.col = ((i - 1) %% ncols) + 1,
                        layout.pos.row = (((i) - 1) %/% ncols) + 1))
    plotTracks(list(itrack, maTrack, mdTrack, mgTrack), 
               chromosome = chroms[i], add = TRUE)
    popViewport(1)
}

## ----multPlot4, results='hide', fig.width=7.5, fig.height=4-------------------
library(lattice)
chroms <- data.frame(chromosome = chroms)
xyplot(1 ~ chromosome | chromosome, data = chroms, panel = function(x) {
  plotTracks(list(itrack , maTrack, mdTrack, mgTrack), 
             chromosome = x, add = TRUE, showId = FALSE) },
  scales = list(draw = FALSE), xlab = NULL, ylab = NULL)

## ----biocStruct1, echo=FALSE, results='asis'----------------------------------
dt <- data.frame(`Gviz class` = rep(c("AnnotationTrack","GeneRegionTrack","DataTrack","SequenceTrack"), c(4,5,3,2)),
                 `Bioconductor class`=c("data.frame","IRanges","GRanges","GRangesList",
                                        "data.frame","IRanges","GRanges","GRangesList","TxDb",
                                        "data.frame","IRanges","GRanges",
                                        "DNAStringSet", "BSgenome"),
                 `Method`=c("Constructor","Constructor + additional arguments","Constructor or setAs method, additional data in metadata columns","Constructor or setAs method",
                            "Constructor","Constructor + additional arguments","Constructor or setAs method, additional data in metadata columns","Constructor or setAs method, additional data in metadata columns", "Constructor or setAs method",
                             "Constructor","Constructor + additional data matrix","Constructor or setAs method, numeric data in metadata columns",
                            "DNAStringSet & Constructor", "Constructor"), 
                 stringsAsFactors = FALSE, check.names=FALSE)
dt[duplicated(dt[,1]),1] <- "" 
if (out_type == "html") {
  kable(dt)
} else if (out_type == "latex") {
  kable(dt, "latex", booktabs=TRUE, longtable=TRUE)
}

## ----biocStruct2, echo=FALSE, results='asis'----------------------------------
dt <- data.frame(`Gviz class`=rep(c("AnnotationTrack","GeneRegionTrack","DataTrack","SequenceTrack","AlignmentsTrack"), c(5, 4, 4, 2, 1)),
                 `File type`=c("BED","GFF","GFF2","GFF3","BAM",
                               "GTF","GFF","GFF2","GFF3",
                               "BedGraph","WIG","BigWig","BAM",
                               "FASTA","2Bit",
                               "BAM"),
                 `Extension`=c(".bed",".gff, .gff1, ",".gff2",".gff3",".bam",
                               ".gtf",".gff, .gff1",".gff2",".gff3",
                               ".bedGraph",".wig",".bigWig",".bam",
                               ".fa, .fasta",".2bit",
                               ".bam"),
                 `Streaming`=c("no","no","no","no","YES",
                               "no","no","no","no",
                               "no","no","YES","YES",
                               "YES","YES",
                               "YES"),
                 `Details`=c("Genomic locations from the mandatory `chrom`, `chromStart` and `chromEnd` fields, and optionally the strand from the strand field. If present, the information in the field is mapped to track item ids, and is mapped to track item feature type. All other fields are currently ignored.", "Only the following basic GFF fields are recognized: `seqname`, `start`, `end`, `strand`, (mapped to track item feature type) and `group` (to allow for track item grouping).", "Same as above, but feature grouping information may be provided either as `Group` or `Parent` attribute. Feature ids are mapped to one of the ID, Name or Alias attributes.", "Same as above, but feature grouping information has to be provided as the `Parent` attribute.", "Only start and end locations as well as the strand information for the reads are used. Read identifiers are used for track item grouping.", 
                             "A somewhat looser format definition for `gtf` files is applied here where gene, transcript and exon identifiers and names can be parsed from the `gene_id`, `gene_name`, `transcript_id`, `transcript_name`, `exon_id`, or `exon_id attributes`", "This only supports very limited item grouping and thus complete gene models can not be properly encoded.", "In most instances this is identical to the `GTF` standard and it could make sense to rename the file accordingly.", "The gene-to-transcript and transcript-to- exon relationships are encoded in the parent and `type` attributes and the parser tries to accommodate most of the existing `GFF3` variants.",
                             "", "", "", "Read coverage only is extracted from the bam file.",
                             "Streaming only possible if an index file is found in the same directory as the original fasta file.", "",
                             "Always needs an index file is found in the same directory as the original `BAM` file."),
                 stringsAsFactors = FALSE, check.names=FALSE)
dt[duplicated(dt[,1]),1] <- "" 
if (out_type == "html") {
  kable(dt)
} else if (out_type == "latex") {
  kable(dt, "latex", booktabs=TRUE, longtable=TRUE)
}

## ----session-info-------------------------------------------------------------
sessionInfo()

## Write all tracks in a list of tracks into a single BED file.
# FIXME: Need to support WIG exports as well...

#' Export Gviz tracks into an annotation file representation.
#'
#' This function is still a bit experimental. Write all tracks provided as a
#' list `tracks` into a single BED file. So far only BED export is supported.
#'
#' @name exportTracks
#'
#' @param tracks A list of annotation track objects to be exported into a
#' single BED file.
#' @param range A numeric vector or length 2. The genomic range to display when
#' opening the file in a browser.
#' @param chromosome The chromosome to display when opening the file in a browser.
#' @param file Character, the path to the file to write into.
#'
#' @return The function is called for its side effect of writing to a file.
#'
#' @author Florian Hahne
#'
#' @examples
#' ## export AnnotationTrack to BED file
#' at <- AnnotationTrack(start = seq(1, 10), width = 1, chromosome = "chr1")
#' exportTracks(list(at),
#'     range = c(1, 10), chromosome = "chr1",
#'     file = paste0(tempfile(), ".bed")
#' )
#' @importClassesFrom rtracklayer UCSCData
#' @importFrom utils write.table
#' @importFrom grDevices col2rgb
#'
#' @export exportTracks
exportTracks <- function(tracks, range, chromosome, file) {
    if (missing(file)) {
        file <- "customTracks.bed"
    }
    con <- file(file, open = "wt")
    writeLines(
        sprintf("browser position %s:%i-%i", chromosome, range[1], range[2]),
        con
    )
    writeLines("browser hide all", con)
    for (t in seq_along(tracks))
    {
        track <- tracks[[t]]
        if (length(track) > 0 && (is(track, "AnnotationTrack") || is(track, "GeneRegion"))) {
            track <- as(track, "UCSCData")
            writeLines(as(track@trackLine, "character"), con)
            ## nextMet <- selectMethod("export.bed", c("RangedData", "characterORconnection"))
            ## nextMet(as(track, "GRanhes"), con)
            .expBed(as(track, "GRanges"), con)
        }
    }
    close(con)
}

## This function is broken in the rtracklayer package
.expBed <- function(object, con, variant = c("base", "bedGraph", "bed15"), color, append) {
    variant <- match.arg(variant)
    name <- strand <- thickStart <- thickEnd <- color <- NULL
    blockCount <- blockSizes <- blockStarts <- NULL
    df <- data.frame(as.character(seqnames(object)), start(object) - 1, end(object))
    score <- score(object)
    if (!is.null(score)) {
        if (!is.numeric(score) || any(is.na(score))) {
            stop("Scores must be non-NA numeric values")
        }
    }
    if (variant == "bedGraph") {
        if (is.null(score)) {
            score <- 0
        }
        df$score <- score
    } else {
        blockSizes <- object$blockSizes
        blockStarts <- object$blockStarts
        if (variant == "bed15" && is.null(blockSizes)) {
            blockStarts <- blockSizes <- ""
        }
        if (!is.null(blockSizes) || !is.null(blockStarts)) {
            if (is.null(blockSizes)) {
                stop("'blockStarts' specified without 'blockSizes'")
            }
            if (is.null(blockStarts)) {
                stop("'blockSizes' specified without 'blockStarts'")
            }
            lastBlock <- function(x) sub(".*,", "", x)
            lastSize <- lastBlock(blockSizes)
            lastStart <- lastBlock(blockStarts)
            if (any(df[[2]] + as.integer(lastSize) + as.integer(lastStart) != df[[3]]) ||
                any(sub(",.*", "", blockStarts) != 0)) {
                stop("blocks must span entire feature")
            }
            blockCount <- vapply(strsplit(blockSizes, ","), length, FUN.VALUE = numeric(1L))
        }
        if (is.null(color)) {
            color <- object$itemRgb
        }
        if (is.null(color) && !is.null(blockCount)) {
            color <- "0"
        } else if (!is.null(color)) {
            nacol <- is.na(color)
            colmat <- col2rgb(color)
            color <- paste(colmat[1, ], colmat[2, ], colmat[3, ], sep = ",")
            color[nacol] <- "0"
        }
        thickStart <- object$thickStart
        thickEnd <- object$thickEnd
        if (is.null(thickStart) && !is.null(color)) {
            thickStart <- start(object)
            thickEnd <- end(object)
        }
        strand <- object$strand
        if (!is.null(thickStart) && is.null(strand)) {
            strand <- rep(NA, length(object))
        }
        if (!is.null(strand) && is.null(score)) {
            score <- 0
        }
        name <- object$name
        if (is.null(name)) {
            name <- rownames(object)
        }
        if (!is.null(score) && is.null(name)) {
            name <- rep(NA, length(object))
        }
        df$name <- name
        df$score <- score
        df$strand <- strand
        df$thickStart <- thickStart
        df$thickEnd <- thickEnd
        df$itemRgb <- color
        df$blockCount <- blockCount
        df$blockSizes <- blockSizes
        df$blockStarts <- blockStarts
        if (variant == "bed15") {
            df$expCount <- object$expCount
            df$expIds <- object$expIds
            df$expScores <- object$expScores
        }
    }
    scipen <- getOption("scipen")
    options(scipen = 100)
    on.exit(options(scipen = scipen))
    write.table(df, con,
        sep = "\t", col.names = FALSE, row.names = FALSE,
        quote = FALSE, na = ".", append = append
    )
}

#' @include RangeTrack-class.R
NULL

## OverlayTrack Class --------------------------------------------------------

#' OverlayTrack class and methods
#'
#'
#' A container for other track objects from the Gviz package that allows for
#' overlays of their content on the same region of the plot.
#'
#'
#' A track to conceptionally group other Gviz track objects into a meta track
#' in order to merge them into a single overlay visualization. Only the first
#' track in the supplied list will be inferred when setting up the track title
#' and axis, for all the other tracks only the panel content is plotted.
#'
#' @name OverlayTrack-class
#'
#' @param trackList A list of Gviz track objects that all have to inherit from
#' class \code{GdObject}.
#' @param name Character scalar of the track's name. This is not really used
#' and only exists fro completeness.
#' @param \dots All additional parameters are ignored.
#' @return
#'
#' The return value of the constructor function is a new object of class
#' \code{OverlayTrack}.
#' @section Objects from the Class:
#'
#' Objects can be created using the constructor function \code{OverlayTrack}.
#'
#' @author Florian Hahne
#' @inherit GdObject-class seealso
#' @examples
#'
#'
#' ## Object construction:
#' set.seed(123)
#' dat <- runif(100, min = -2, max = 22)
#' dt1 <- DataTrack(data = dat, start = sort(sample(200, 100)), width = 1, genome = "hg19")
#' dt2 <- DataTrack(data = dat, start = sort(sample(200, 100)), width = 1, genome = "hg19")
#' ot <- OverlayTrack(trackList = list(dt1, dt2))
#' @exportClass OverlayTrack
setClass("OverlayTrack",
    representation = representation(trackList = "list"),
    contains = c("GdObject"),
    prototype = prototype(dp = DisplayPars())
)

## Initialize ----------------------------------------------------------------

#' @describeIn OverlayTrack-class Initialize.
#' @export
setMethod("initialize", "OverlayTrack", function(.Object, trackList, ...) {
    .Object <- .updatePars(.Object, "OverlayTrack")
    .Object@trackList <- trackList
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

#' @describeIn OverlayTrack-class Constructor function for `OverlayTrack-class`.
#' @export
OverlayTrack <- function(trackList = list(), name = "OverlayTrack", ...) {
    if (!is.list(trackList)) {
        trackList <- list(trackList)
    }
    if (!all(vapply(trackList, is, class2 = "GdObject", FUN.VALUE = logical(1)))) {
        stop("All elements in 'trackList' must inherit from 'GdObject'")
    }
    return(new("OverlayTrack", trackList = trackList, name = name, ...))
}

## General accessors ---------------------------------------------------------

#' @describeIn OverlayTrack-class set display parameters using the values of
#' the named list in value. See \code{\link{settings}} for details on
#' display parameters and customization.
#' @export
setReplaceMethod("displayPars", signature("OverlayTrack", "list"), function(x, recursive = FALSE, value) {
    x <- setPar(x, value, interactive = FALSE)
    if (recursive) {
        x@trackList <- lapply(x@trackList, function(y) {
            displayPars(y) <- value
            return(y)
        })
    }
    return(x)
})

#' @describeIn OverlayTrack-class return the number of subtracks.
#' @export
setMethod("length", "OverlayTrack", function(x) length(x@trackList))

#' @describeIn OverlayTrack-class return the chromosome for which the track
#' is defined.
#' @export
setMethod("chromosome", "OverlayTrack", function(GdObject) {
    unique(unlist(lapply(GdObject@trackList, chromosome)))
})

#' @describeIn OverlayTrack-class replace the value of the track's chromosome.
#' This has to be a valid UCSC chromosome identifier or an integer or character
#' scalar that can be reasonably coerced into one.
#' @export
setReplaceMethod("chromosome", "OverlayTrack", function(GdObject, value) {
    GdObject@trackList <- lapply(GdObject@trackList, function(x) {
        chromosome(x) <- value
        x
    })
    return(GdObject)
})


## Annotation Accessors ------------------------------------------------------
## Stacking ------------------------------------------------------------------

#' @describeIn OverlayTrack-class recompute the stacks based on the available
#' space and on the object's track items and stacking settings.
#' This really just calls the setStacks methods for the contained
#' tracks and only exists for dispatching reasons.
#' @export
setMethod("setStacks", "OverlayTrack", function(GdObject, ...) {
    GdObject@trackList <- lapply(GdObject@trackList, setStacks, ...)
    return(GdObject)
})

## Consolidate ---------------------------------------------------------------

#' @describeIn OverlayTrack-class #' For a `OverlayTrack` apply the method on
#' each of the subtracks in the `trackList` slot
#' @keywords internal
#' @export
setMethod("consolidateTrack", signature(GdObject = "OverlayTrack"), function(GdObject, chromosome, ...) {
    GdObject@trackList <- lapply(GdObject@trackList, consolidateTrack, chromosome = chromosome, ...)
    return(GdObject)
})

## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------

#' @describeIn OverlayTrack-class plot subset all the contained tracks in an
#' `OverlayTrack` by coordinates and sort if necessary.
#' @export
setMethod("subset", signature(x = "OverlayTrack"), function(x, ...) {
    x@trackList <- lapply(x@trackList, subset, ...)
    return(x)
})

## Position ------------------------------------------------------------------
## DrawGrid ------------------------------------------------------------------
## DrawAxis ------------------------------------------------------------------
## DrawGD --------------------------------------------------------------------

#' @describeIn OverlayTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @export
setMethod("drawGD", signature("OverlayTrack"), function(GdObject, ...) {
    GdObject@trackList <- lapply(GdObject@trackList, drawGD, ...)
    return(invisible(GdObject))
})

## SetAs ---------------------------------------------------------------------
## Show ----------------------------------------------------------------------

#' @describeIn OverlayTrack-class Show method.
#' @export
setMethod("show", signature(object = "OverlayTrack"), function(object) {
    cat(sprintf(
        "OverlayTrack '%s' containing %i subtrack%s\n", names(object), length(object),
        ifelse(length(object) == 1, "", "s")
    ))
})

#' @include DisplayPars-class.R
NULL

###  GdObject-class ----------------------------------------------------------

#' GdObject class and methods
#'
#' The virtual parent class for all track items in the Gviz package.  This
#' class definition contains all the common entities that are needed for a
#' track to be plotted. During object instantiation for any of the sub-classes
#' inheriting from `GdObject`, this class' global initializer has to be
#' called in order to assure that all necessary settings are present.
#'
#' @template DisplayParams_list
#'
#' @template ImageMap-class_param
#' @template GdObject-class_param
#'
#' @template GdObject-class_slot
#'
#' @name GdObject-class
#'
#' @return A virtual class: No objects may be created from it.
#'
#' @author Florian Hahne
#'
#' @seealso
#' \code{\linkS4class{DisplayPars}}
#'
#' \code{\linkS4class{GdObject}}
#'
#' \code{\linkS4class{GRanges}}
#'
#' \code{\linkS4class{HighlightTrack}}
#'
#' \code{\linkS4class{ImageMap}}
#'
#' \code{\linkS4class{IRanges}}
#'
#' \code{\linkS4class{RangeTrack}}
#'
#' \code{\linkS4class{DataTrack}}
#'
#' \code{\link{collapsing}}
#'
#' \code{\link{grouping}}
#'
#' \code{\link{panel.grid}}
#'
#' \code{\link{plotTracks}}
#'
#' \code{\link{settings}}
#'
#' @examples
#' ## This is a reference class therefore we show below
#' ## an example from AnnotationTrack:
#'
#' ## An empty object
#' AnnotationTrack()
#'
#' ## Construct from individual arguments
#' st <- c(2000000, 2070000, 2100000, 2160000)
#' ed <- c(2050000, 2130000, 2150000, 2170000)
#' str <- c("-", "+", "-", "-")
#' gr <- c("Group1", "Group2", "Group1", "Group3")
#'
#' annTrack <- AnnotationTrack(
#'     start = st, end = ed, strand = str, chromosome = 7,
#'     genome = "hg19", feature = "test", group = gr,
#'     id = paste("annTrack item", 1:4),
#'     name = "generic annotation", stacking = "squish"
#' )
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(annTrack)
#' @exportClass GdObject
setClass("GdObject",
    representation = representation("VIRTUAL",
        dp = "DisplayPars",
        name = "character",
        imageMap = "ImageMapOrNULL"
    ),
    prototype = prototype(
        dp = DisplayPars(
            alpha = 1,
            alpha.title = NULL,
            background.panel = "transparent",
            background.title = "lightgray",
            background.legend = "transparent",
            cex.axis = NULL,
            cex.title = NULL,
            cex = 1,
            col.axis = "white",
            col.border.title = "white",
            col.frame = "lightgray",
            col.grid = .DEFAULT_SHADED_COL,
            col.line = NULL,
            col.symbol = NULL,
            col.title = "white",
            col = .DEFAULT_SYMBOL_COL,
            collapse = TRUE,
            fill = .DEFAULT_FILL_COL,
            fontcolour = "black",
            fontface.title = 2,
            fontface = 1,
            fontfamily.title = "sans",
            fontfamily = "sans",
            fontsize = 12,
            frame = FALSE,
            grid = FALSE,
            h = -1,
            lineheight = 1,
            lty.grid = "solid",
            lty = "solid",
            lwd.border.title = 1,
            lwd.title = 1,
            lwd.grid = 1,
            lwd = 1,
            min.distance = 1,
            min.height = 3,
            min.width = 1,
            reverseStrand = FALSE,
            rotation.title = 90,
            rotation = 0,
            showAxis = TRUE,
            showTitle = TRUE,
            size = 1,
            v = -1
        ),
        name = "GdObject",
        imageMap = NULL
    )
)

##  GdObject Constructor -----------------------------------------------------

## We add everything that hasn't been clobbered up so far as
## additional DisplayParameters.  Also, the `dp` slot must be
## re-initiated here in order to get a fresh environment for each
## instance of the class.

#' @describeIn GdObject-class Initialize the object. This involves setting up a
#' new environment for the display parameters and filling it up with the current
#' settings. All arguments that have not been clobbered up by one of the
#' sub-class initializers are considered to be additional display parameters
#' and are also added to the environment. See `settings` for details on setting
#' graphical parameters for tracks.
#' @keywords internal
setMethod("initialize", "GdObject", function(.Object, name, ...) {
    ## update the default parameters first
    .makeParMapping()
    .Object <- .updatePars(.Object, "GdObject")
    ## now rebuild the slot to get a new environment
    pars <- getPar(.Object, hideInternal = FALSE)
    .Object@dp <- DisplayPars()
    .Object <- setPar(.Object, pars, interactive = FALSE)
    if (!missing(name)) {
        .Object@name <- if (is.null(name)) "" else name
    }
    ## Finally clobber up everything that's left
    .Object <- setPar(.Object, list(...), interactive = FALSE)
    return(.Object)
})

##  GdObject Methods Setters -------------------------------------------------

## We need to set and query DisplayPars in the the initializer, hence
## the appropriate methods have to be defined here first.
#' @describeIn GdObject-class set the single display parameter name to value.
#' Note that display parameters in the `GdObject-class` are pass-by-reference,
#' so no re-assignment to the symbol `obj` is necessary. See settings for
#' details on display parameters and customization.
setMethod("setPar", signature("GdObject", "character"), function(x, name, value, interactive = TRUE) {
    newDp <- setPar(x@dp, name, value, interactive = interactive)
    x@dp <- newDp
    return(x)
})

#' @describeIn GdObject-class set display parameters by the values of the named
#' list in value. Note that display parameters in the `GdObject-class` are
#' pass-by-reference, so no re-assignment to the symbol `obj` is necessary.
#' See settings for details on display parameters and customization.
setMethod("setPar", signature("GdObject", "list"), function(x, value, interactive = TRUE) {
    newDp <- setPar(x@dp, value, interactive = interactive)
    x@dp <- newDp
    return(x)
})

#' @describeIn GdObject-class set display parameters using the values of the
#' named list in `value`. See `settings` for details on display parameters
#' and customization.
setReplaceMethod("displayPars", signature("GdObject", "list"), function(x, recursive = FALSE, value) {
    x <- setPar(x, value, interactive = FALSE)
    return(x)
})

##  GdObject Methods Getters -------------------------------------------------

#' @describeIn GdObject-class alias for the `displayPars` method.
#' See `settings` for details on display parameters and customization.
setMethod("getPar", c("GdObject", "character"), function(x, name, asIs = FALSE) getPar(x@dp, name, asIs = asIs))

#' @describeIn GdObject-class alias for the `displayPars` method.
#' See `settings` for details on display parameters and customization.
setMethod("getPar", c("GdObject", "missing"), function(x, hideInternal = TRUE) getPar(x@dp, hideInternal = hideInternal))

#' @describeIn GdObject-class list the value of the display parameter name.
#' See `settings` for details on display parameters and customization.
setMethod("displayPars", c("GdObject", "character"), function(x, name) getPar(x, name))

#' @describeIn GdObject-class list the value of all available display parameters.
#' See `settings` for details on display parameters and customization.
setMethod("displayPars", c("GdObject", "missing"), function(x, hideInternal = TRUE) getPar(x, hideInternal = hideInternal))

##  GdObject Methods Coord and Tags  -----------------------------------------

#' @describeIn GdObject-class return the coordinates from the internal image map.
setMethod("coords", "GdObject", function(ImageMap) coords(imageMap(ImageMap)))

#' @describeIn GdObject-class return the tags from the internal image map.
setMethod("tags", "GdObject", function(ImageMap) tags(imageMap(ImageMap)))

##  GdObject Methods Subset --------------------------------------------------

#' @describeIn GdObject-class subset a `GdObject` by coordinates.
#' Most of the respective sub-classes inheriting from `GdObject` overwrite this
#' method, the default is to return the unaltered input object.
setMethod("subset", signature(x = "GdObject"), function(x, ...) x)

##  GdObject Methods Name ----------------------------------------------------

#' @describeIn GdObject-class return the value of the `name` slot.
setMethod("names", "GdObject", function(x) x@name)

#' @describeIn GdObject-class set the value of the `name` slot.
setReplaceMethod(
    "names", signature("GdObject", "character"),
    function(x, value) {
        x@name <- value[1]
        return(x)
    }
)

##  GdObject Methods Group ---------------------------------------------------

#' @describeIn GdObject-class Generics for `group`.
#' @exportMethod group
#' @keywords internal
setGeneric("group", function(GdObject, ...) standardGeneric("group"))

#' @exportMethod "group<-"
#' @describeIn GdObject-class Generics for `group<-`.
#' @keywords internal
setGeneric("group<-", function(GdObject, value) standardGeneric("group<-"))

#' @describeIn GdObject-class return grouping information for the individual
#' items in the track. Unless overwritten in one of the sub-classes,
#' this usually returns `NULL`.
setMethod("group", "GdObject", function(GdObject) NULL)

##  GdObject Methods ImageMap ------------------------------------------------

#' @exportMethod imageMap
#' @describeIn GdObject-class Generics for `imageMap`.
#' @keywords internal
setGeneric("imageMap", function(GdObject, ...) standardGeneric("imageMap"))

#' @describeIn  GdObject-class Extract the content of the `imageMap` slot.
#' @keywords internal
#' @export
setMethod("imageMap", "GdObject", function(GdObject) GdObject@imageMap)

#' @exportMethod "imageMap<-"
#' @describeIn GdObject-class Generics for `imageMap<-`.
#' @keywords internal
setGeneric("imageMap<-", function(GdObject, value) standardGeneric("imageMap<-"))

#' @describeIn  GdObject-class Replace the content of the `imageMap` slot.
#' @keywords internal
#' @export
setReplaceMethod(
    "imageMap", signature("GdObject", "ImageMapOrNULL"),
    function(GdObject, value) {
        GdObject@imageMap <- value
        return(GdObject)
    }
)

##  GdObject Methods drawAxis ------------------------------------------------

#' @describeIn GdObject-class Generics for `drawAxis`.
#' @export
setGeneric("drawAxis", function(GdObject, ...) standardGeneric("drawAxis"))

#' @describeIn GdObject-class add a y-axis to the title panel of a track if
#' necessary. Unless overwritten in one of the sub-classes this usually
#' does not plot anything and returns `NULL`.
#' @export
setMethod("drawAxis", signature(GdObject = "GdObject"), function(GdObject, ...) {
    return(NULL)
})

##  GdObject Methods drawGrid ------------------------------------------------

#' @describeIn GdObject-class Generics for `drawGrid`.
setGeneric("drawGrid", function(GdObject, ...) standardGeneric("drawGrid"))

# #' @describeIn GdObject-class superpose a grid on top of a track if necessary.
# #' Unless overwritten in one of the sub-classes this usually does not plot
# #' anything and returns `NULL`.
#' @keywords internal
setMethod("drawGrid", signature(GdObject = "GdObject"), function(GdObject, ...) {
    return(NULL)
})

##  GdObject Methods drawGd --------------------------------------------------

#' @describeIn GdObject-class Generics for `drawGD`.
#' @export
setGeneric("drawGD", function(GdObject, ...) standardGeneric("drawGD"))

##  GdObject Methods Annotation accessors ------------------------------------

.getAnn <- function(GdObject, type) {
    return(as.character(values(GdObject)[[type]]))
}
.setAnn <- function(GdObject, value, type) {
    v <- values(GdObject)
    if (length(value) > 1 && length(value) != nrow(v)) {
        stop(
            "The length of the replacement value for the '", type, "' annotation does not match the number ",
            "of features in the track."
        )
    }
    v[[type]] <- value
    mcols(GdObject@range) <- v
    return(GdObject)
}

#' @exportMethod gene
#' @describeIn GdObject-class Generics for `gene`.
#' @keywords internal
setGeneric("gene", function(GdObject, ...) standardGeneric("gene"))

#' @exportMethod "gene<-"
#' @describeIn GdObject-class Generics for `gene<-`.
#' @keywords internal
setGeneric("gene<-", function(GdObject, value) standardGeneric("gene<-"))

#' @exportMethod symbol
#' @describeIn GdObject-class Generics for `symbol`.
#' @keywords internal
setGeneric("symbol", function(GdObject, ...) standardGeneric("symbol"))

#' @exportMethod "symbol<-"
#' @describeIn GdObject-class Generics for `symbol<-`.
#' @keywords internal
setGeneric("symbol<-", function(GdObject, value) standardGeneric("symbol<-"))

#' @exportMethod transcript
#' @describeIn GdObject-class Generics for `transcript`.
#' @keywords internal
setGeneric("transcript", function(GdObject, ...) standardGeneric("transcript"))

#' @exportMethod "transcript<-"
#' @describeIn GdObject-class Generics for `transcript<-`.
#' @keywords internal
setGeneric("transcript<-", function(GdObject, value) standardGeneric("transcript<-"))

#' @exportMethod exon
#' @describeIn GdObject-class Generics for `exon`.
#' @keywords internal
setGeneric("exon", function(GdObject, ...) standardGeneric("exon"))

#' @exportMethod "exon<-"
#' @describeIn GdObject-class Generics for `exon<-`.
#' @keywords internal
setGeneric("exon<-", function(GdObject, value) standardGeneric("exon<-"))

#' @exportMethod feature
#' @describeIn GdObject-class Generics for `feature`.
#' @keywords internal
setGeneric("feature", function(GdObject, ...) standardGeneric("feature"))

#' @exportMethod "feature<-"
#' @describeIn GdObject-class Generics for `feature<-`.
#' @keywords internal
setGeneric("feature<-", function(GdObject, value) standardGeneric("feature<-"))

#' @exportMethod identifier
#' @describeIn GdObject-class Generics for `identifier`.
#' @keywords internal
setGeneric("identifier", function(GdObject, ...) standardGeneric("identifier"))

#' @exportMethod "identifier<-"
#' @describeIn GdObject-class Generics for `identifier<-`.
#' @keywords internal
setGeneric("identifier<-", function(GdObject, value) standardGeneric("identifier<-"))


##  GdObject Methods  General accessors --------------------------------------

#' @exportMethod chromosome
#' @describeIn GdObject-class Generics for `chromosome`.
#' @keywords internal
setGeneric("chromosome", function(GdObject, ...) standardGeneric("chromosome"))

#' @describeIn GdObject-class return the chromosome for which the track is defined.
#' @export
setMethod("chromosome", "GdObject", function(GdObject) {
    return(NULL)
})

#' @exportMethod "chromosome<-"
#' @describeIn GdObject-class Generics for `chromosome`.
#' @keywords internal
setGeneric("chromosome<-", function(GdObject, value) standardGeneric("chromosome<-"))

#' @describeIn GdObject-class replace the value of the track's chromosome. This
#' has to be a valid UCSC chromosome identifier or an integer or character
#' scalar that can be reasonably coerced into one.
#' @export
setReplaceMethod("chromosome", "GdObject", function(GdObject, value) {
    return(GdObject)
})

#' @exportMethod position
#' @describeIn GdObject-class Generics for `position`.
#' @keywords internal
setGeneric("position", function(GdObject, ...) standardGeneric("position"))

#' @describeIn GdObject-class return the track's genome.
#' @export
setMethod("genome", "GdObject", function(x) NULL)

#' @describeIn GdObject-class set the track's genome. Usually this has to be a
#' valid UCSC identifier, however this is not formally enforced here.
#' @export
setReplaceMethod("genome", "GdObject", function(x, value) {
    return(x)
})

##  GdObject Methods  Prepare tracks for plotting ----------------------------

#' @exportMethod consolidateTrack
#' @describeIn GdObject-class Generics for `consolidateTrack`.
#' @keywords internal
setGeneric("consolidateTrack", function(GdObject, ...) standardGeneric("consolidateTrack"))

#' @describeIn GdObject-class Consolidate.
#' Determine whether there is `alpha` settings or not, and add this information
#' as the internal display parameter `.__hasAlphaSupport`.
#' @export
# #' @keywords internal
setMethod("consolidateTrack", signature(GdObject = "GdObject"), function(GdObject, alpha, ...) {
    pars <- list(...)
    pars <- pars[names(pars) != ""]
    pars[[".__hasAlphaSupport"]] <- alpha
    displayPars(GdObject) <- pars
    return(GdObject)
})

#' @noRd
#' @keywords internal
setGeneric("collapseTrack", function(GdObject, ...) standardGeneric("collapseTrack"))

#' @exportMethod stacking
#' @describeIn GdObject-class Generics for `stacking`.
#' @keywords internal
setGeneric("stacking", function(GdObject, ...) standardGeneric("stacking"))

#' @exportMethod "stacking<-"
#' @describeIn GdObject-class Generics for `stacking<-`.
#' @keywords internal
setGeneric("stacking<-", function(GdObject, value) standardGeneric("stacking<-"))

#' @exportMethod stacks
#' @describeIn GdObject-class Generics for `stacks`.
#' @keywords internal
setGeneric("stacks", function(GdObject, ...) standardGeneric("stacks"))

#' @exportMethod setStacks
#' @describeIn GdObject-class Generics for ``.
#' @keywords internal
setGeneric("setStacks", function(GdObject, ...) standardGeneric("setStacks"))

#' @describeIn GdObject-class set stacks.
#' @export
setMethod("setStacks", "GdObject", function(GdObject, ...) GdObject)

#' @exportMethod setCoverage
#' @describeIn GdObject-class Generics for ``.
#' @keywords internal
setGeneric("setCoverage", function(GdObject, ...) standardGeneric("setCoverage"))

##  GdObject Methods Internal methods ----------------------------------------

#' @noRd
#' @keywords internal
setGeneric(".buildRange", function(range, start, end, width, ...) standardGeneric(".buildRange"))

#' Grouping of annotation features
#'
#' Many annotation tracks are actually composed of a number of grouped
#' sub-features, for instance exons in a gene model. This man page highlights
#' the use of grouping information to build informative annotation plots.
#'
#'
#' All track objects that inherit from class
#' \code{\linkS4class{AnnotationTrack}} support the grouping feature. The
#' information is usually passed on to the constructor function (for
#' \code{AnnotationTrack} via the \code{groups} argument and for
#' \code{\linkS4class{GeneRegionTrack}} objects via the \code{exon} argument)
#' or automatically downloaded from an online annotation repository
#' (\code{\linkS4class{BiomartGeneRegionTrack}}). Group membership is specified
#' by a factor vector with as many items as there are annotation items in the
#' track (i.e., the value of \code{length(track)}. Upon plotting, the grouped
#' annotation features are displayed together and will not be separated in the
#' stacking of track items.
#'
#' @name grouping
#'
#' @author Florian Hahne
#' @seealso
#'
#' \code{\linkS4class{AnnotationTrack}}
#'
#' \code{\linkS4class{BiomartGeneRegionTrack}}
#'
#' \code{\linkS4class{GeneRegionTrack}}
NULL

## All class definitions and constructors/initializers for the
## package. Note that some methods have to be defined in here as well
## because they are used as part of the initializers.  Constructors
## and definitions that are exported in the name space are marked with
## (N)





##----------------------------------------------------------------------------------------------------------------------
## Some usefull class unions to allow for NULL values in certain slots
##----------------------------------------------------------------------------------------------------------------------
setClassUnion("DfOrNULL", c("data.frame", "NULL"))
setClassUnion("MartOrNULL", c("Mart", "NULL"))
setClassUnion("FactorOrCharacterOrNULL", c("factor", "character", "NULL"))
setClassUnion("NumericOrNULL", c("numeric", "NULL"))
setClassUnion("ListOrEnv", c("list", "environment"))
setClassUnion("GRangesOrIRanges", c("GRanges", "IRanges"))
setClassUnion("NULLOrMissing", c("NULL", "missing"))
setClassUnion("BSgenomeOrNULL", c("BSgenome", "NULL"))
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## ImageMap:
##
## A class to hold HTML image map information
## Slots :
##    o coords: a numeric matrix of rectangular image map cordinates, in the order x bl, y bl, x tr, y tr. Rownames are
##       mandatory for the matrix and have to be unique
##    o tags: a list of tags that are to be added to the <map> tag, where the name of the list item is used as the
##      tagname. The value of each list item has to be a named character vector, where the names must match back into
##      the rownames of the 'coords' matrix
##----------------------------------------------------------------------------------------------------------------------
setClass("ImageMap", representation(coords="matrix", tags="list"),
         prototype=prototype(coords=matrix(1, ncol=4, nrow=0), tags=list()))

## Constructor
ImageMap <- function(coords, tags)
{
    if(!(is.matrix(coords) && is.numeric(coords) && ncol(coords)==4))
        stop("'coords' must be a numeric matrix with 4 columns")
    rn <- rownames(coords)
    if(is.null(rn))
        stop("Rownames must be set for the matrix in 'coords'")
    if(!is.list(tags) || is.null(names(tags)) || any(names(tags)=="") || !all(sapply(tags, is.character)))
        stop("'tags' must be a named list with character vector items.")
    n <- unique(unlist(sapply(tags, names)))
    if(is.null(n) || any(n==""))
        stop("All items in the 'tags' list must be named character vectors.")
    m <- n %in% rn
    if(!all(m))
        stop("The following values in the 'tags' list could not be mapped to the 'coords' matrix:\n",
             paste(n[!m], sep="", collapse=", "))
    new("ImageMap", coords=coords, tags=tags)
}

## Allow for NULL value slots
setClassUnion("ImageMapOrNULL", c("ImageMap", "NULL"))
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## DisplayPars:
##
## A class to control the plotting parameters for GdObjects
## Slots :
##    o pars: an environment or a list containing parameter key value pairs
##       The initial idea was for the class to uses pass by reference semantic, allowing to update the object
##       without reassigning to a symbol. However this turned out to be problematic whenever a GdObject was
##       copied and to avoid confusion with the users about unwanted side-effects we decided to deprecate this
##       feature.
##----------------------------------------------------------------------------------------------------------------------
setClass("DisplayPars", representation(pars="ListOrEnv"))

## The initializer needs to create the environment, we can't take this
## directly from the class prototype since this would result in using
## the same environment all the time
setMethod("initialize", "DisplayPars", function(.Object, ...) {
  e = new.env(hash=TRUE)
  args <- list(...)
  n <- names(args)
  if(any(n == ""))
    stop("All supplied arguments must be named.")
  .Object@pars <- args
  return(.Object)
})

## Constructor, all supplied arguments are added to the environment.
DisplayPars <- function(...)
{
  return(new("DisplayPars", ...))
}

## Update function and deprecation message to show that an old environment-based
## DisplayParameter object has been updated to a list-based object.
.updateDp <- function(x, interactive=TRUE)
{
    if(interactive)
        message("Note that the behaviour of the 'setPar' method has changed. You need to reassign the result to an ",
                "object for the side effects to happen. Pass-by-reference semantic is no longer supported.")
    if(class(x@pars)=="environment")
    {
        x@pars <- as.list(x@pars)
        message("The DisplayPars object has been updated to a list-based representation.")
    }
    return(x)
}

## The accessor methods for the DisplayPars class. (they need to be in
## here because they are being called in the prototypes). For the
## setter method, the input can either be a named list, or a single
## keyword/value pair. The DisplayPars class was first implemented as
## an environment, so we essentially had pass by reference semantic here,
## and the object could be modified without the need to reassign to a
## symbol. However this somewhat broke the the R paradigm, and 'setPar'
## has been deprecated in favour of the more standard 'DisplayPars<-'
## replacement method. The getter methods either return the list
## of all parameters, or a subset of parameters if their names are
## provided as a character vector.  Please note that for convenience
## the result is unlisted if only a single parameter is queried.
## Since version 1.7.3 we have introduced an alias table for display
## parameters in order to harmomize parameter names without loosing
## backwards compatability.
setMethod("setPar", signature("DisplayPars", "list"),
          function(x, value, interactive=TRUE) {
              x <- .updateDp(x, interactive)
              aliasRes <- .dpAliasReverseTable[names(value)]
              new <- is.na(aliasRes)
              aliasRes[new] <- names(value)[new]
              x@pars[aliasRes] <- value
              return(x)
          })

setMethod("setPar", signature("DisplayPars", "character"),
          function(x, name, value, interactive=TRUE) {
              if(!(length(name) == 1 && is.null(value)) && length(name) != length(value))
                  stop("'name' and 'value' must be of equal length")
              x <- .updateDp(x, interactive)
              for(i in seq_along(name)){
                  aliasRes <- .dpAliasReverseTable[name[i]]
                  if(is.na(aliasRes))
                      aliasRes <- name[i]
                  x@pars[[aliasRes]] <- value[[i]]
              }
              return(x)
          })

setReplaceMethod("displayPars", signature("DisplayPars", "list"), function(x, recursive=FALSE, value) {
  x <- setPar(x, value, interactive=FALSE)
  return(x)
})

setMethod("getPar", c("DisplayPars", "character"),
          function(x, name, asIs=FALSE){
              aliasRes <- .dpAliasReverseTable[name]
              new <- is.na(aliasRes)
              aliasRes[new] <- name[new]
              if(class(x@pars)=="environment")
              {
                  name <- intersect(aliasRes, base::ls(x@pars, all.names=TRUE))
                  tmp <- mget(name, x@pars)
              }else{
                  name <- intersect(aliasRes, names(x@pars))
                  tmp <- x@pars[name]
              }
              if(!asIs)
                  tmp <- if(is.list(tmp) && length(tmp)==1) tmp[[1]] else tmp
              return(if(length(tmp)) tmp else NULL)
          })

setMethod("getPar", c("DisplayPars", "missing"), function(x, hideInternal=TRUE){
    pars <- as.list(x@pars)
    if(hideInternal)
        pars <- pars[!grepl("^\\.__", names(pars))]
    return(pars)
})

setMethod("displayPars", c("DisplayPars", "missing"), function(x, hideInternal=TRUE) getPar(x, hideInternal=hideInternal))

setMethod("displayPars", c("DisplayPars", "character"), function(x, name) getPar(x, name))

setMethod("as.list", "DisplayPars", function(x) as(x, "list"))

setAs("DisplayPars", "list", function(from, to) if(!is.null(from)) as.list(from@pars) else list())

## An alias table to define display parameter synonyms. Essentially this is a simple named list,
## where the element names represent the preferred parameter name under which the actual value is
## stored, and the element content is a character vector of synonyms. Please note that this structure
## is also turned into a reverse lookup table for fast access.
.dpAliasTable <- list(
                      "fontcolor.title"="col.title",
                      "rotation.title"=c("rot.title", "rotate.title"),
                      "fontcolor.group"="col.group",
                      "fontcolor.item"="col.item",
                      "just.group"=c("labelJust", "labelJustification"),
                      "transcriptAnnotation"="geneSymbols",
                      "fontcolor.item"=c("fontcolor.exon", "fontcolor.feature"),
                      "fontsize.item"=c("fontsize.exon", "fontsize.feature"),
                      "fontfamily.item"=c("fontfamily.exon", "fontfamily.feature"),
                      "fontface.item"=c("fontface.exon", "fontface.feature"),
                      "lineheight.item"=c("lineheight.exon", "lineheight.feature"),
                      "alpha.item"=c("alpha.exon", "alpha.feature"),
                      "cex.item"=c("cex.exon", "cex.feature"),
                      "col.mate"=c("col.mate", "col.mates", "col.pair", "col.pairs"),
                      "lty.mate"=c("lty.mate", "lty.mates", "lty.pair", "lty.pairs"),
                      "lwd.mate"=c("lwd.mate", "lwd.mates", "lwd.pair", "lwd.pairs"),
                      "alpha.mate"=c("alpha.mate", "alpha.mates", "alpha.pair", "alpha.pairs"),
                      "col.gap"="col.gaps",
                      "lwd.gap"="lwd.gaps",
                      "lty.gap"="lty.gaps",
                      "alpha.gap"="alpha.gaps"
                      )
.dpAliasReverseTable <- character()
.dpAliasReverseTable[names(.dpAliasTable)] <- names(.dpAliasTable)
.dpAliasReverseTable[unlist(.dpAliasTable, use.names=FALSE)] <- rep(names(.dpAliasTable), listLen(.dpAliasTable))
.dpAliasReverseTable <- .dpAliasReverseTable[order(names(.dpAliasReverseTable))]
##----------------------------------------------------------------------------------------------------------------------


##----------------------------------------------------------------------------------------------------------------------
## InferredDisplayPars:
##
## A class to allow for querrying of available display parameters. Essentially this is a normal list with
## a bit of a fancyfied show method.
## Slots :
##    o name: the name of the class
##    o inheritance: a character vector indicating the inheritance structure
##----------------------------------------------------------------------------------------------------------------------
setClass("InferredDisplayPars", representation(name="character", inheritance="character"), contains="list")

setMethod("as.list", "InferredDisplayPars", function(x) as(x, "list"))

setAs("InferredDisplayPars", "list",
          function(from, to) {ll <- from@.Data; names(ll) <- names(from); ll})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
##  GdObject:
##
## The GdObject is the parent of all Gviz objects in the system.
## This class is virtual.
## Slots:
##    o dp: object of class DisplayPars to fine-tune the drawing.
##    o name: the object name, used in the title panel if needed.
##    o imageMap: an ImageMap object holding relevant image map information, or NULL
## A bunch of DisplayPars are set during object instantiation:
##    o fontface, fontcolor, fontsize, fontfamily, lineheight, cex: default settings for
##       all text unless specified someplace else.
##    o col, lwd, lty, fill: default settings for all plotting elements unless specified
##       somewhere else.
##    o col.line, col.symbol: default colors for plot symbols and plot lines. The default is
##       to take the value of the global col parameter.
##    o col.frame: the color of the panel frame, if frame==TRUE
##    o col.grid, lwd.grid, lty.grid, v and h: the default parameters for the grid plotting,
##       both when type=="g" in DataTracks and when grid==TRUE
##    o alpha: the transparancy for all track items.
##    o background.title: the fill color for the title panel. Defaults to lightgray.
##    o col.title: the font color for the title panel. Defaults to white.
##    o cex.title: the expansion factor for the title panel. This effects the fontsize
##       of both the title and the axis, if any. Defaults to NULL, which means that the
##       text size is automatically adjusted to the available space.
##    o fontfamily.title, fontface.title: the font family and font face for the title panel.
##       Defaults to bold sans serif.
##    o col.axis: the font and line color for the y axis, if any. Defaults to white.
##    o cex.axis: the expansion factor for the axis annotation. Defaults to NULL, in which case
##       it is computed based on the available space.
##    o background.panel: the background color of the content panel. Defaults to transparent.
##    o showTitle: boolean, controlling whether to plot a title panel. Although this can be
##       set individually for each track, in multi-track plots there will still
##       be an empty placeholder in case any of the other tracks include a title. The
##       same holds true for axes. Note that the background color could be set to
##       transparent in order to completely hide the title/axis panel.
##    o showAxis: boolean, controlling whether to plot a y axis (only track types where axes
##	 are implemented).
##    o grid: boolean, switching on/off the plotting of a grid.
##    o collapse: collapse the content of the track to accomodate the minimum current
##	 device resolution
##    o min.width, min.height: the minimum width and height in pixels to display. All ranges are expanded
##       to this size in order to avoid rendering issues.
##    o min.distance: the minimum pixel distance before collapsing range items, only if collapse==TRUE
##    o frame: draw a frame around the track
##    o size: the relative size of the track
##    o ...: additional DisplayPars are allowed. Unless specified in one of the subclasses,
##       those should take the value of a valid R color descriptors. The parameter names will
##	 later be matched to optional track item types as defined in the 'feature' range
##	 attribute, and all tracks of the matched types are colored accordingly. See the
##	 documentation of the 'GeneRegion' and 'AnnotationTrack' classes for details.
##----------------------------------------------------------------------------------------------------------------------
setClass("GdObject",
         representation=representation("VIRTUAL",
                                       dp="DisplayPars",
                                       name="character",
                                       imageMap="ImageMapOrNULL"),
         prototype=prototype(dp=DisplayPars(alpha=1,
                                            alpha.title=NULL,
                                            background.panel="transparent",
                                            background.title="lightgray",
                                            background.legend="transparent",
                                            cex.axis=NULL,
                                            cex.title=NULL,
                                            cex=1,
                                            col.axis="white",
                                            col.border.title="white",
                                            col.frame="lightgray",
                                            col.grid=.DEFAULT_SHADED_COL,
                                            col.line=NULL,
                                            col.symbol=NULL,
                                            col.title="white",
                                            col=.DEFAULT_SYMBOL_COL,
                                            collapse=TRUE,
                                            fill=.DEFAULT_FILL_COL,
                                            fontcolor="black",
                                            fontface.title=2,
                                            fontface=1,
                                            fontfamily.title="sans",
                                            fontfamily="sans",
                                            fontsize=12,
                                            frame=FALSE,
                                            grid=FALSE,
                                            h=-1,
                                            lineheight=1,
                                            lty.grid="solid",
                                            lty="solid",
                                            lwd.border.title=1,
                                            lwd.title=1,
                                            lwd.grid=1,
                                            lwd=1,
                                            min.distance=1,
                                            min.height=3,
                                            min.width=1,
                                            reverseStrand=FALSE,
                                            rotation.title=90,
                                            rotation=0,
                                            showAxis=TRUE,
                                            showTitle=TRUE,
                                            size=1,
                                            v=-1),
                             name="GdObject",
                             imageMap=NULL))

## We need to set and query DisplayPars in the the initializer, hence
## the appropriate methods have to be defined here first.
setMethod("setPar", signature("GdObject", "character"), function(x, name, value, interactive=TRUE) {
    newDp <- setPar(x@dp, name, value, interactive=interactive)
    x@dp <- newDp
    return(x)
})

setMethod("setPar", signature("GdObject", "list"), function(x, value, interactive=TRUE) {
    newDp <- setPar(x@dp, value, interactive=interactive)
    x@dp <- newDp
    return(x)
})

setReplaceMethod("displayPars", signature("GdObject", "list"), function(x, recursive=FALSE, value) {
    x <- setPar(x, value, interactive=FALSE)
    return(x)
})

setMethod("getPar", c("GdObject", "character"), function(x, name, asIs=FALSE) getPar(x@dp, name, asIs=asIs))

setMethod("getPar", c("GdObject", "missing"), function(x, hideInternal=TRUE) getPar(x@dp, hideInternal=hideInternal))

setMethod("displayPars", c("GdObject", "character"), function(x, name) getPar(x, name))

setMethod("displayPars", c("GdObject", "missing"), function(x, hideInternal=TRUE) getPar(x, hideInternal=hideInternal))

## We add everything that hasn't been clobbered up so far as
## additional DisplayParameters.  Also, the dp slot must be
## re-initiated here in order to get a fresh environment for each
## instance of the class.
setMethod("initialize", "GdObject", function(.Object, name, ...) {
    ## update the default parameters first
    .makeParMapping()
    .Object <- .updatePars(.Object, "GdObject")
    ## now rebuild the slot to get a new environment
    pars <- getPar(.Object, hideInternal=FALSE)
    .Object@dp <- DisplayPars()
    .Object <- setPar(.Object, pars, interactive=FALSE)
    if(!missing(name))
    .Object@name <- if(is.null(name)) "" else name
    ## Finally clobber up everything that's left
    .Object <- setPar(.Object, list(...), interactive=FALSE)
    return(.Object)
})
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## RangeTrack:
##
## Parent class for all region-like annotation tracks, storing information
## about start, end, strand, chromosome and the associated genome.
## Slots:
##    o range: object of class GRangesOrIRanges containing all the necessary information
##       for plotting. The content of the metadata columns may vary between
##	 subclasses. The strand information may be provided in the form '+' for
##	 the Watson strand, '-' for the Crick strand or '*' for any of the two.
##    o chromosome: a character vector giving the active chromosome for which the
##	 track is defined. Valid chromosome names are:
##          - a single numeric character
##	    - a string, starting with 'chr', followed by any additional characters
##    o genome: character giving the reference genome for which the track is defined.
##----------------------------------------------------------------------------------------------------------------------
setClass("RangeTrack",
         representation=representation("VIRTUAL",
                                       range="GRangesOrIRanges",
                                       chromosome="character",
                                       genome="character"),
         contains="GdObject",
         prototype=prototype(chromosome="chr1",
                             dp=DisplayPars(),
                             genome="ANY",
                             name="RangeTrack",
                             range=GRanges()))

## Coercing all input to the appropriate form
setMethod("initialize", "RangeTrack", function(.Object, range, chromosome, genome, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "RangeTrack")
    if(!missing(chromosome) && !is.null(chromosome)){
        .Object@chromosome <- .chrName(chromosome)[1]
    }
    if(!missing(genome) && !is.null(genome)){
        .Object@genome <- genome
    }
    if(!missing(range) && is(range, "GRanges")){
        .Object@range <- range
    }
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## ReferenceTrack:
##
## Parent class for all tracks that provide a reference to data somewhere on the file system. This class is virtual
## and only exists for the purpose of dispatching
## Slots:
##   o stream: the import function to stream data off the disk. Needs to be able to handle the two mandatory arguments
##      'file' (a character containing a valid file path) and 'selection' (a GRanges object with the genomic region to plot)
##   o reference: the path to the file containing the data
##   o mapping: a default mapping between the metadata columns of the returned GRanges object from the import function
##      and the metadata columns that make up the final track object
##   o args: a list with the passed in constructor arguments during object instantiation. Those will be needed when
##     fetching the data in order to fill all necessary slots
##   o defaults: a list with the relevant default values to be used when neither 'mapping' nor 'args' provides the
##     necessary information
setClass("ReferenceTrack",  representation=representation("VIRTUAL",
                                                          stream="function",
                                                          reference="character",
                                                          mapping="list",
                                                          args="list",
                                                          defaults="list"),
         prototype=prototype(stream=function(x, selection){},
                             reference="~",
                             mapping=list()),
         validity=function(object){
             msg <- NULL
             if(!all(c("file", "selection") %in% names(formals(object@stream))))
                 msg <- "The streaming function in the 'stream' slot needs to define two arguments, 'file' and 'selection'"
             if(!file.exists(object@reference))
                 msg <- c(msg, sprintf("The referenced file '%s' does not exist", object@reference))
             return(if(is.null(msg)) TRUE else msg)
         })
setMethod("initialize", "ReferenceTrack", function(.Object, stream, reference, mapping=list(),
                                                   args=list(), defaults=list()) {
    .Object@stream <- stream
    .Object@reference <- reference
    .Object@mapping <- mapping
    .Object@args <- args
    .Object@defaults <- defaults
    validObject(.Object)
    return(.Object)
})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## NumericTrack:
##
## Parent class for all annotation tracks that include numeric values. This is for
## dispatching purpose only.
##----------------------------------------------------------------------------------------------------------------------
setClass("NumericTrack",
         representation=representation("VIRTUAL"),
         prototype=prototype(name="NumericTrack",
                             dp=DisplayPars()),
         contains="RangeTrack")
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## StackedTrack
##
## Parent class for all tracks that involve stacking. This is for
## dispatching purpose only.
## Slots:
##    o stacking: character controlling the stacking of overlapping items on the final plot.
##       One in 'hide', 'dense', 'squish', 'pack' or 'full'.
##    o stacks: a numeric vector holding the current stacking information. Not usually set by the user.
## This is part of the prototype only
##    o stackingValues: possible values of the stacking slot
##	 [c("hide", "dense", "squish", "pack", "full")]
##----------------------------------------------------------------------------------------------------------------------
setClass("StackedTrack",
         representation=representation("VIRTUAL",
                                       stacking="character",
                                       stacks="numeric"),
         prototype=prototype(name="StackedTrack",
                             stacking="squish",
                             stackingValues=c("hide", "dense", "squish", "pack", "full"),
                             dp=DisplayPars(stackHeight=0.75,
                                            reverseStacking=FALSE)),
         contains="RangeTrack")

## Need to fill the stacks slot here, don't want to recompute all the time
setMethod("initialize", "StackedTrack", function(.Object, stacking, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "StackedTrack")
    pt <- getClass("StackedTrack")@prototype
    if(!missing(stacking))
    {
        if(!all(stacking %in% pt@stackingValues))
            stop("Problem initializing AnnotationTrack need the following values for 'stacking':",
                 paste(pt@stackingValues, collpase=", "), "\n")
        .Object@stacking <- stacking
        r <- list(...)$range
        ##stacks <- if(length(r)>0) disjointBins(ranges(r)) else 0
        ##.Object@stacks <- stacks
        .Object@stacks <- numeric()
    }
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## AnnotationTrack:
##
## A generic Annotation track, storing information about start, end, chromosome,
## strand and the associated genome.
## Slots: no additional formal slots are defined, but the following are part of the prototype
##    o columns: column names that are allowed as part of the internal GRanges object in
##       the ranges slot [c("feature", "group")]
##    o featureColumnName: the column in the internal RangedData object identifying the
##       feature type ["feature"]. If a display parameter of the same name is specified
##	 the software will use its value for the coloring.
## A bunch of DisplayPars are set during object instantiation:
##    o fill: the fill color for untyped items. Defaults to lightblue.
##    o col: the border color for all track items. This is also used to connect grouped items.
##    o lty, lwd: the line type and width for all track items. This is also used to connect grouped items.
##    o lex: the line expansion factor
##    o fontsize, fontfamily, fontface, fontcolor: the face, size, family and color for the
##       annotation text (i.e., the track item IDs)
##    o cex: the font expansion factor.
##    o size: the relative size of the track, if not explicitely set in the plotTracks function.
##    o lineheight: the text lineheight
##    o showId: boolean controlling whether to plot track item identifiers
##    o showFeatureId: boolean controlling whether to annotate individual exons
##    o col.group, cex.group=0.7, fontface.group=2: the font color, size and face for the
##       group-level annotation
##    o shape: the shape used for the annotation items. Currently only 'box' and 'arrow' are implemented.
##    o rotation: the rotation of the item annotation in degrees.
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("AnnotationTrack",
         contains="StackedTrack",
         prototype=prototype(columns=c("feature", "group", "id"),
                             stacking="squish",
                             name="AnnotationTrack",
                             dp=DisplayPars(arrowHeadWidth=30,
                                            arrowHeadMaxWidth=40,
                                            cex.group=0.6,
                                            cex=1,
                                            col.line="darkgray",
                                            col="transparent",
                                            featureAnnotation=NULL,
                                            fill="lightblue",
                                            fontfamily.group="sans",
                                            fontcolor.group=.DEFAULT_SHADED_COL,
                                            fontcolor.item="white",
                                            fontface.group=2,
                                            fontsize.group=12,
                                            groupAnnotation=NULL,
                                            just.group="left",
                                            lex=1,
                                            lineheight=1,
                                            lty="solid",
                                            lwd=1,
                                            mergeGroups=FALSE,
                                            min.height=3,
                                            min.width=1,
                                            rotation=0,
                                            rotation.group=0,
                                            rotation.item=0,
                                            shape="arrow",
                                            showFeatureId=FALSE,
                                            showId=FALSE,
                                            showOverplotting=FALSE,
                                            size=1)))

## Essentially we just check for the correct GRanges columns here
setMethod("initialize", "AnnotationTrack", function(.Object, ...) {
    if(is.null(list(...)$range) && is.null(list(...)$genome) && is.null(list(...)$chromosome))
        return(.Object)
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "AnnotationTrack")
    range <- list(...)$range
    if(!is.null(range) && length(.Object))
    {
        if(!all(.Object@columns %in% colnames(values(range))))
            stop(paste("Problem initializing AnnotationTrack need the following columns:",
                     paste(.Object@columns, collpase = ", ")), "\n")
        grp <- if(is(.Object, "GeneRegionTrack")) values(range)$transcript else values(range)$group
        if(any(sapply(split(as.character(strand(range)), grp), function(x) length(unique(x))) != 1))
            stop("Grouped elments of a RangeTrack can not be on opposing strands")
    }
    .Object <- callNextMethod()
    return(.Object)
})

## The file-based version of the AnnotationTrack class. This will mainly provide a means to dispatch to
## a special 'subset' method which should stream the necessary data from disk.
setClass("ReferenceAnnotationTrack", contains=c("AnnotationTrack", "ReferenceTrack"))

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritence has some strange features with regards to method selection
setMethod("initialize", "ReferenceAnnotationTrack", function(.Object, stream, reference, mapping=list(),
                                                             args=list(), defaults=list(), ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object=.Object, reference=reference, stream=stream,
                                                            mapping=mapping, args=args, defaults=defaults)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor. The following arguments are supported:
##    o range: a data.frame or a GRanges object containing the information
##       about the track items. If a data.frame, it needs to be coerceable
##	 to a GRanges object, i.e., it needs at least the mandatory 'start', 'stop' and
##	 'strand' columns. Additional optional columns are:
##	    - feature: the type of the item. Can be mapped to colors via the DisplayPars.
##	    - group: a grouping factor to connect track items.
##	    - id: a unique identifier for a feature. This will be plotted if showId==TRUE
##             Note that internally we use the value of ID as the seqnames slot in the
##             internal GRanges object. Defaults for all missing columns are generated.
##	Instead of using the 'range' parameter, all these values can also be passed as
##	individual vectors, in which case they need to be of similar length.
##    o start, end, width: numeric vectors of the item start and end coordinates
##    o strand: the strand information may be provided in the form '+' for
##       the Watson strand, '-' for the Crick strand or '*' for any of the two.
##    o feature, group, id: individual vectors of equal length as described above.
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o stacking: character controlling the stacking of overlapping items. One in 'hide',
##       'dense', 'squish', 'pack' or 'full'.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as further DisplayParameters
## (N)
AnnotationTrack <- function(range=NULL, start=NULL, end=NULL, width=NULL, feature, group, id, strand, chromosome,
                            genome, stacking="squish", name="AnnotationTrack", fun, selectFun, importFunction,
                            stream=FALSE, ...)
{
    ## Some defaults
    covars <- .getCovars(range)
    isStream <- FALSE
    if(!is.character(range)){
        n <- max(c(length(start), length(end), length(width)), nrow(covars))
        if(is.null(covars[["feature"]]) && missing(feature))
            feature <- rep("unknown", n)
        if(is.null(covars[["id"]]) && missing(id))
            id <- make.unique(rep(if(!is.null(feature)) as.character(feature) else covars[["feature"]], n)[1:n])
        if(is.null(covars[["group"]]) && missing(group))
            group <- seq_len(n)
    }
    ## Build a GRanges object from the inputs
    .missingToNull(c("feature", "group", "id", "strand", "chromosome", "importFunction", "genome"))
    args <- list(feature=feature, group=group, id=id, strand=strand, chromosome=chromosome, genome=genome)
    defs <- list(feature="unknown", group="unknown", id="unknown", strand="*", density=1, chromosome="chrNA", genome=NA)
    range <- .buildRange(range=range, groupId="group", start=start, end=end, width=width,
                         args=args, defaults=defs, chromosome=chromosome, trackType="AnnotationTrack",
                         importFun=importFunction, stream=stream)
    if(is.list(range)){
        isStream <- TRUE
        slist <- range
        range <- GRanges()
    }
    ## Pipes have a special meaning for merged groups, so we can't have them in the initial group vector
    mcols(range)[["group"]] <- gsub("|", "", mcols(range)[["group"]], fixed=TRUE)
    ## If no chromosome was explicitely asked for we just take the first one in the GRanges object
    if(missing(chromosome) || is.null(chromosome))
        chromosome <- if(length(range)>0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    ## And finally the object instantiation, we have to distinguish between DetailsAnnotationTracks and normal ones
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    if(missing(fun))
    {
        if(!isStream){
            return(new("AnnotationTrack", chromosome=as.character(chromosome[1]), range=range,
                       name=name, genome=genome, stacking=stacking, ...))
        }else{
            ## A bit hackish but for some functions we may want to know which track type we need but at the
            ## same time we do not want to enforce this as an additional argument
            e <- new.env()
            e[["._trackType"]] <- "AnnotationTrack"
            environment(slist[["stream"]]) <- e
            return(new("ReferenceAnnotationTrack", chromosome=as.character(chromosome[1]), range=range,
                       name=name, genome=genome, stacking=stacking, stream=slist[["stream"]], reference=slist[["reference"]],
                       mapping=slist[["mapping"]], args=args, defaults=defs, ...))
        }
    }else{
        if(!is.function(fun))
            stop("'fun' must be a function")
        if(missing(selectFun))
            selectFun <- function(...) return(TRUE)
        if(!is.function(selectFun))
            stop("'selectFun' must be a function")
        return(new("DetailsAnnotationTrack", chromosome=as.character(chromosome[1]), range=range,
                   name=name, genome=genome, stacking=stacking, fun=fun, selectFun=selectFun, ...))
    }
}
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## DetailsAnnotationTrack is an AnnotationTrack for which each fearture (annotation)
## has a detail plot (e.g. a scatter plot e.g. using lattice). The details plot
## is generated by a user defined funnction that gets called internally with
## arguments start, end and chromosome for the feature. The details plots are placed
## on top of the AnnotationTrack. As details plots can take relatively much space
## compared to features of an AnnotationTrack it only makse sense to prove details
## for an AnnotationTrack with few features. The details plots are distributed evenly
## on top of the annotation and can be connected to their corresponding featurs (they're
## providing details for) with lines for better readability.
##
## The function must have the '...' arguments. All items from the list in
## detailsFunArgs get added as arguments to the function call as well as the
## arguments, start, end, chromosome and identifier.
##
## foo = function(...) {
##    plot(densityplot(rnorm(1000), xlab=NA, ylab=NA), newpage=FALSE, prefix="foo")
## }
##
## Note, use plot with newpage=FALSE and an explicit prefix (otherwise plotting
## get more and more slow the more trellis plots you create!). See '?plot.trellis'
## for details.
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("DetailsAnnotationTrack",
         contains="AnnotationTrack",
         representation=representation(fun="function", selectFun="function"),
         prototype=prototype(fun=function(...){},
                             selectFun=function(...){return(TRUE)},
                             dp=DisplayPars(details.minWidth=100,
                                            details.ratio=Inf,
                                            details.size=0.5,
                                            detailsBorder.col="darkgray",
                                            detailsBorder.fill="transparent",
                                            detailsBorder.lty="solid",
                                            detailsBorder.lwd=1,
                                            detailsConnector.cex=1,
                                            detailsConnector.col="darkgray",
                                            detailsConnector.lty="dashed",
                                            detailsConnector.lwd=1,
                                            detailsConnector.pch=20,
                                            detailsFunArgs=list(),
                                            groupDetails=FALSE)))

DetailsAnnotationTrack <- function(...) AnnotationTrack(...)

setMethod("initialize", "DetailsAnnotationTrack", function(.Object, fun, selectFun, ...) {
			## the diplay parameter defaults
			.Object <- .updatePars(.Object, "DetailsAnnotationTrack")
			.makeParMapping()
			.Object@fun <- fun
                        .Object@selectFun <- selectFun
			.Object <- callNextMethod(.Object, ...)
			return(.Object)
		})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## GeneRegionTrack:
##
## A track containing all genes in a particular region. The data are usually fetched dynamially
## from an online data store. Of course it would
## also be possible to manully construct an object from local data. Particular data sources
## should be implemented as sub-classes, this is just the commone denominator that is being
## used for plotting later on. There are several levels of data associated to a GeneRegionTrack:
##   - exon level: identifiers are stored in the exon column of the GRanges object. Data may be extracted
##          using the 'exons' method.
##   - transcript level: identifiers are stored in the transcript column of the GRanges object.
##          Data may be extracted using the 'transcripts' method.
##   - gene level: identifiers are stored in the gene column of the GRanges object, more human-readable
##          versions in the symbol column. Data may be extracted using the 'genes' or the 'symbols' methods.
##   - transcript-type level: information is stored in the feature column of the GRanges object.
##      If a display parameter of the same name is specified the software will use its value for the coloring.
## Slots: no additional formal slots are defined, but the following are part of the prototype
##    o columns: column names that are allowed as part of the internal GRanges object in
##       the ranges slot [c("feature", "transcript", "symbol", "gene")]
## A bunch of DisplayPars are set during object instantiation:
##    o fill: the fill color for untyped items. Defaults to lightblue.
##    o col: the border color for all track items. This is also used to connect grouped items.
##    o alpha: the transparancy for all track items.
##    o lty, lwd: the line type and width for all track items. This is also used to connect grouped items.
##    o lex: the line expansion factor
##    o fontsize, fontfamily, fontface, fontcolor: the face, size, family and color for the
##       annotation text (i.e., the track item IDs)
##    o cex: the font expansion factor.
##    o lineheight: the text lineheight
##    o showId: boolean controlling whether to plot group identifiers, e.g, gene symbols or transcript IDs.
##    o shape: the shape used for the annotation items. Currently only 'box' and 'arrow' are implemented.
##    o rotation: the rotation of the exon annotation in degrees.
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("GeneRegionTrack",
         contains="AnnotationTrack",
         representation=representation(start="NumericOrNULL", end="NumericOrNULL"),
         prototype=prototype(columns=c("feature", "transcript", "symbol", "gene", "exon"),
                             stacking="squish",
                             stacks=0,
                             start=0,
                             end=0,
                             name="GeneRegionTrack",
                             dp=DisplayPars(arrowHeadWidth=10,
                                            arrowHeadMaxWidth=20,
                                            col=NULL,
                                            collapseTranscripts=FALSE,
                                            exonAnnotation=NULL,
                                            fill="orange",
                                            min.distance=0,
                                            shape=c("smallArrow", "box"),
                                            showExonId=NULL,
                                            thinBoxFeature=.THIN_BOX_FEATURES,
                                            transcriptAnnotation=NULL)))


## Making sure all the display parameter defaults are being set
setMethod("initialize", "GeneRegionTrack", function(.Object, start, end, ...){
    if(is.null(list(...)$range) && is.null(list(...)$genome) && is.null(list(...)$chromosome))
        return(.Object)
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "GeneRegionTrack")
    .Object@start <- ifelse(is.null(start), 0 , start)
    .Object@end <- ifelse(is.null(end), 0 , end)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## The file-based version of the GeneRegionTrack class. This will mainly provide a means to dispatch to
## a special 'subset' method which should stream the necessary data from disk.
setClass("ReferenceGeneRegionTrack", contains=c("GeneRegionTrack", "ReferenceTrack"))

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritence has some strange features with regards to method selection
setMethod("initialize", "ReferenceGeneRegionTrack", function(.Object, stream, reference, mapping=list(),
                                                             args=list(), defaults=list(), ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object=.Object, reference=reference, stream=stream,
                                                            mapping=mapping, args=args, defaults=defaults)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor. The following arguments are supported:
##    o range: one in a whole number of differerent potential inputs upon which the .buildRanges method will dispatch
##    o start, end: numeric vectors of the track start and end coordinates.
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o rstarts, rends, rwidths: integer vectors of exon start and end locations or widths, or a character vector of
##      comma-delimited exon locations, one vector element for each transcript
##    o strand, feature, exon, transcript, gene, symbol, chromosome: vectors of equal length containing
##       the exon strand, biotype, exon id, transcript id, gene id, human-readable gene symboland chromosome information
##    o stacking: character controlling the stacking of overlapping items. One in 'hide',
##       'dense', 'squish', 'pack' or 'full'.
##    o name: the name of the track. This will be used for the title panel.
##    o exonList: boolean, causing the values in starts, rends or rwidths to be interpreted as delim-deparated
##       lists that have to be exploded. All other annotation arguments will be repeated accordingly.
##    o delim: the delimiter if coordinates are in a list
## All additional items in ... are being treated as DisplayParameters
## (N)
GeneRegionTrack <- function(range=NULL, rstarts=NULL, rends=NULL, rwidths=NULL, strand, feature, exon,
                            transcript, gene, symbol, chromosome, genome, stacking="squish",
                            name="GeneRegionTrack", start=NULL, end=NULL, importFunction, stream=FALSE, ...)
{
    ## Some defaults
    covars <- if(is.data.frame(range)) range else if(is(range, "GRanges")) as.data.frame(mcols(range)) else data.frame()
    isStream <- FALSE
    if(!is.character(range)){
        n <- if(is.null(range)) max(c(length(start), length(end), length(width))) else if(is(range, "data.frame")) nrow(range) else length(range)
        if(is.null(covars[["feature"]]) && missing(feature))
            feature <- paste("exon", 1:n, sep="_")
        if(is.null(covars[["exon"]]) && missing(exon))
            exon <- make.unique(rep(if(!missing(feature) && !is.null(feature)) as.character(feature) else covars[["feature"]], n)[1:n])
        if(is.null(covars[["transcript"]]) && missing(transcript))
            transcript <- paste("transcript", seq_len(n), sep="_")
        if(is.null(covars[["gene"]]) && missing(gene))
            gene <- paste("gene", seq_len(n), sep="_")
    }
    ## Build a GRanges object from the inputs
    .missingToNull(c("feature", "exon", "transcript", "gene", "symbol", "strand", "chromosome", "importFunction", "genome"))
    args=list(feature=feature, id=exon, exon=exon, transcript=transcript, gene=gene, symbol=symbol, strand=strand,
              chromosome=chromosome, genome=genome)
    defs <- list(feature="unknown", id="unknown", exon="unknown", transcript="unknown", genome=NA,
                 gene="unknown", symbol="unknown", strand="*", density=1, chromosome="chrNA")
    range <- .buildRange(range=range, groupId="transcript", start=rstarts, end=rends, width=rwidths, args=args, defaults=defs,
                         chromosome=chromosome, tstart=start, tend=end, trackType="GeneRegionTrack", importFun=importFunction,
                         genome=genome)
    if(is.list(range)){
        isStream <- TRUE
        slist <- range
        range <- GRanges()
    }
    if(is.null(start))
        start <- if(!length(range)) NULL else min(start(range))
    if(is.null(end))
        end <- if(!length(range)) NULL else max(end(range))
    if(missing(chromosome) || is.null(chromosome))
        chromosome <- if(length(range)>0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    if(!isStream){
        return(new("GeneRegionTrack", start=start, end=end, chromosome=chromosome[1], range=range,
                   name=name, genome=genome, stacking=stacking, ...))
    }else{
        return(new("ReferenceGeneRegionTrack", start=start, end=end, chromosome=chromosome[1], range=range,
                   name=name, genome=genome, stacking=stacking, stream=slist[["stream"]],
                   reference=slist[["reference"]], mapping=slist[["mapping"]], args=args, defaults=defs, ...))
    }
}
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## BiomartGeneRegionTrack:
##
## A GeneRegionTrack that fetches its information from Biomart
## Slots:
##    o biomart: a biomaRt object providing the connection to the data source.
##        currently we assume that this connects to a esembl_gene source for
##        a particular organism.
##    o filter: a named list of additional filters that are passed on to the Biomart query.
## This is defined only in the prototype:
##    o columns: column names that are allowed as part of the internal GRanges object in
##        the ranges slot [c("feature", "transcript", "symbol", "gene")]
## A bunch of DisplayPars are set during object instantiation, all of which are
## transcript types as returned from Biomart in the 'gene_biotype' field.
## This class mainly exists for dispatching purpose and to keep things as flexible
## as possible, the actual plottable information is all contained in the parent
## class definition.
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("BiomartGeneRegionTrack",
         contains="GeneRegionTrack",
         representation=representation(biomart="MartOrNULL", filter="list"),
         prototype=prototype(biomart=NULL,
                             filter=list(),
                             columns=c("feature", "transcript", "symbol", "gene", "rank"),
                             name="BiomartGeneRegionTrack",
                             dp=DisplayPars(C_segment="burlywood4",
                                            D_segment="lightblue",
                                            J_segment="dodgerblue2",
                                            Mt_rRNA="yellow",
                                            Mt_tRNA="darkgoldenrod",
                                            Mt_tRNA_pseudogene="darkgoldenrod1",
                                            V_segment="aquamarine",
                                            miRNA="cornflowerblue",
                                            miRNA_pseudogene="cornsilk",
                                            misc_RNA="cornsilk3",
                                            misc_RNA_pseudogene="cornsilk4",
                                            protein_coding="#FFD58A",
                                            pseudogene="brown1",
                                            rRNA="darkolivegreen1",
                                            rRNA_pseudogene="darkolivegreen" ,
                                            retrotransposed="blueviolet",
                                            scRNA="gold4",
                                            scRNA_pseudogene="darkorange2",
                                            snRNA="coral",
                                            snRNA_pseudogene="coral3",
                                            snoRNA="cyan",
                                            snoRNA_pseudogene="cyan2",
                                            tRNA_pseudogene="antiquewhite3",
                                            utr3="#FFD58A",
                                            utr5="#FFD58A",
                                            verbose=FALSE)))

## Helper to return the default biomart to feature mapping
.getBMFeatureMap <- function(){
    return(list(gene_id="ensembl_gene_id",transcript_id="ensembl_transcript_id", exon_id="ensembl_exon_id",
             start="exon_chrom_start", end="exon_chrom_end", rank="rank", strand="strand",
             symbol=c("external_gene_name", "external_gene_id"), feature="gene_biotype", chromosome="chromosome_name",
             u5s="5_utr_start", u5e="5_utr_end", u3s="3_utr_start", u3e="3_utr_end", cdsl=c("cds_length", "cds_start"),
             phase="phase"))
}

## Helper to do the actual fetching of data from Biomart
.fetchBMData <- function(object, chromosome_name=NULL, staged=FALSE){
    if(!is.null(chromosome_name)){
        chromosome_name <- gsub("^chr", "", chromosome_name)
    }
    ## The map between Biomart DB fields and annotation features. The individual values can be vectors for cases where there is
    ## ambiguity between different marts. This will be dynamically evaluated against available filters.
    origFeatureMap <- list(gene_id="ensembl_gene_id",
                           transcript_id="ensembl_transcript_id",
                           exon_id="ensembl_exon_id",
                           start="exon_chrom_start",
                           end="exon_chrom_end",
                           rank="rank",
                           strand="strand",
                           symbol=c("external_gene_name", "external_gene_id"),
                           feature="gene_biotype",
                           chromosome="chromosome_name",
                           u5s="5_utr_start",
                           u5e="5_utr_end",
                           u3s="3_utr_start",
                           u3e="3_utr_end",
                           cdsl=c("cds_length", "cds_start"),
                           phase="phase")
    featureMap <- modifyList(origFeatureMap, as.list(.dpOrDefault(object, ".__featureMap", list())))
    needed <- c("gene_id","transcript_id", "exon_id", "start", "end", "rank", "strand", "symbol", "feature",
                "chromosome", "u5s", "u5e", "u3s", "u3e", "cdsl", "phase")
    if(!all(needed %in% names(featureMap)))
        stop("'featureMap' needs to include items '", paste(setdiff(needed, names(featureMap)), collapse=", "), "'")
    avail <- listAttributes(object@biomart)[,1]
    ambig <- names(featureMap)[listLen(featureMap) > 1]
    for(i in ambig){
        mt <- match(featureMap[[i]], avail)
        if(!all(is.na(mt))){
            featureMap[[i]] <- featureMap[[i]][min(which(!is.na(mt)))]
        } else {
            featureMap[[i]] <- NA
        }
    }
    featureMap <- unlist(featureMap)
    ## Deal with the filters
    filterValues <- as.list(object@filter)
    start <- object@start
    end <- object@end
    for(i in c("start", "end", "chromosome_name")){
        if(!is.null(get(i)) && length(get(i)) > 0){
            filterValues[[i]] <- get(i)
        }
    }
    ens <- getBM(as.vector(featureMap), filters=names(filterValues),
                 values=filterValues, bmHeader=FALSE,
                 mart=object@biomart, uniqueRows=TRUE)
    colnames(ens) <- names(featureMap)
    if(staged && nrow(ens)>0){
        filterValues <- list(start=min(ens$start), end=max(ens$end), chromosome_name=ens[1, "chromosome"])
        ens <- getBM(as.vector(featureMap), filters=names(filterValues),
                     values=filterValues, bmHeader=FALSE,
                     mart=object@biomart, uniqueRows=TRUE)
        colnames(ens) <- names(featureMap)
    }
    ## Only those transcripts that have a CDS length will be considered protein_coding
    ens$feature <- ifelse(is.na(ens$cdsl) & ens$feature == "protein_coding", "non_coding", ens$feature)
    ## We may have to split exons if they contain UTRs
    hasUtr <- !is.na(ens$u5s) | !is.na(ens$u3s)
    ensUtr <- ens[hasUtr,, drop=FALSE]
    ensUtr$ffeature <- ifelse(is.na(ensUtr$u5s), "utr3", "utr5")
    ensUtr$us <- ifelse(ensUtr$ffeature=="utr3", ensUtr$u3s, ensUtr$u5s)
    ensUtr$ue <- ifelse(ensUtr$ffeature=="utr3", ensUtr$u3e, ensUtr$u5e)
    ensUtr$u5e <- ensUtr$u5s <- ensUtr$u3e <- ensUtr$u3s <- NULL
    allUtr <- ensUtr$us == ensUtr$start & ensUtr$ue == ensUtr$end
    utrFinal <- ensUtr[allUtr,, drop=FALSE]
    ensUtr <- ensUtr[!allUtr,, drop=FALSE]
    ensUtrS <- split(ensUtr, ifelse(ensUtr$start==ensUtr$us, "left", "right"))
    utrFinal <- rbind(utrFinal, do.call(rbind, lapply(names(ensUtrS), function(i){
        y <- ensUtrS[[i]]
        if(nrow(y)==0)
            return(NULL)
        yy <- y[rep(1:nrow(y), each=2),]
        sel <- seq(1, nrow(yy), by=2)
        yy[sel, "end"] <- if(i=="left") yy[sel, "ue"] else yy[sel, "us"]-1
        yy[sel, "ffeature"] <-  yy[sel, ifelse(i=="left", "ffeature", "feature")]
        yy[sel, "phase"] <-  if(i=="left") -1 else 0
        sel <- seq(2, nrow(yy), by=2)
        yy[sel, "start"] <- if(i=="left") yy[sel, "ue"]+1 else yy[sel, "us"]
        yy[sel, "ffeature"] <-  yy[sel, ifelse(i=="left", "feature", "ffeature")]
        yy[sel, "phase"] <- if(i=="left") yy[sel, "phase"] else -1
        yy
    })))
    utrFinal$feature <- utrFinal$ffeature
    keep <-  c("gene_id","transcript_id","exon_id","start",
               "end", "rank", "strand", "symbol", "feature",
               "chromosome", "phase")
    ens <- rbind(ens[!hasUtr,keep, drop=FALSE], utrFinal[,keep])
    ens$chromosome <- .chrName(ens$chromosome, force=TRUE)
    range <- GRanges(seqnames=ens$chromosome,
                     ranges=IRanges(start=ens$start, end=ens$end),
                     strand=ens$strand, feature=as.character(ens$feature),
                     gene=as.character(ens$gene_id), exon=as.character(ens$exon_id),
                     transcript=as.character(ens$transcript_id), symbol=as.character(ens$symbol),
                     rank=as.numeric(ens$rank), phase=as.integer(ens$phase))
    suppressWarnings(genome(range) <- unname(genome(object)[1]))
    range <- sort(range)
    return(range)
}


## Create an MD5 hash for a BiomartGeneRegion track taking into account the Biomart details for caching
.bmGuid <- function(bmtrack){
    digest(list(genome=genome(bmtrack), host=bmtrack@biomart@host, mart=bmtrack@biomart@biomart, schema=bmtrack@biomart@vschema,
                dataset=bmtrack@biomart@dataset, filters=bmtrack@filter))
}


## Retrieving information from Biomart.
setMethod("initialize", "BiomartGeneRegionTrack", function(.Object, start=NULL, end=NULL, biomart, filter=list(), range, genome=NULL, chromosome=NULL, strand=NULL,
                                                           featureMap=NULL, symbol=NULL, gene=NULL, transcript=NULL, entrez=NULL, ...){
    if((missing(range) || is.null(range)) && is.null(genome) && is.null(chromosome))
        return(.Object)
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "BiomartGeneRegionTrack")
    verb <- list(...)$verbose
    displayPars(.Object) <- list(!is.null(verb) && verb == TRUE)
    ## preparing filters
    strand <- .strandName(strand, extended=TRUE)
    if(strand %in% 0:1){
        filter$strand <- c(1, -1)[strand+1]
    }
    filterOrig <- filter
    idFilters <- list(symbol=c("external_gene_name", "external_gene_id", "hgnc_symbol", "wikigene_name", "dbass3_name"),
                      gene="ensembl_gene_id",
                      transcript="ensembl_transcript_id",
                      entrez="entrezgene")
    staged <- FALSE
    for(i in names(idFilters)){
        if(!is.null(get(i))){
            mt <- match(idFilters[[i]], listFilters(biomart)[,1])
            sfilt <- listFilters(biomart)[mt[!is.na(mt)],1]
            if(length(sfilt) > 0){
                staged <- TRUE
                filter[[sfilt[1]]] <- get(i)
                if(!is.null(filter$strand)){
                    warning(sprintf("Cannot combine %s filter with a strand filter. Strand filtering is ignored.", i))
                    filter$strand <- NULL
                }
                if(!is.null(start) || !is.null(end) || !is.null(chromosome)){
                    warning(sprintf("Cannot combine %s filter with a range restriction. Ignoring start and end coordinates.", i))
                    start <- end <- chromosome <- NULL
                }
            }else{
                stop(sprintf("Unable to automatically map the %s filter. Manually provide adequate filter list.", i))
            }
        }
    }
    ## We can't have only one in start or end
     if(!is.null(end) && is.null(start)){
        start <- 1
    }else if(!is.null(start) && is.null(end)){
        end <- 10e8
    }
    ## filling slots
    .Object@filter <- filter
    .Object@biomart <- biomart
    ## Extending start and end positions to capture genes on the edges. We also need both coordinates set, or none.
    extend <- if(is.null(start) || is.null(end)) 10000 else max(10000, abs(diff(c(end, start))))
    .Object@start <-  if(!is.null(start)) max(1, start - extend) else NULL
    .Object@end <- if(!is.null(end)) end + extend else NULL
    displayPars(.Object) <- list(".__featureMap"=featureMap)
    genome(.Object) <- ifelse(is.null(genome), "ANY", genome)
    ## fetching data from Biomart
    range <- if(!is.null(.Object@biomart) && (!is.null(.Object@start) || !is.null(.Object@end) || length(.Object@filter) != 0)){
        .cacheMartData(.Object, chromosome, staged)
    }else{
        tmp <- GRanges()
        values(tmp) <- DataFrame(feature="a", gene="a", exon="a", transcript="a", symbol="a", rank=0, phase=as.integer(1))[0,]
        suppressWarnings(genome(tmp) <- unname(genome[1]))
        displayPars(.Object) <- list(".__streamOnly"=TRUE)
        tmp
    }
    .Object@filter <- filterOrig
    if(length(range)==0){
        .Object <- setPar(.Object, "size", 0, interactive=FALSE)
    }else{
        chromosome <- if(is.null(chromosome)) seqlevels(range)[1] else chromosome
        rr <- range(range, ignore.strand=TRUE)
        s <- start(rr[seqnames(rr) == chromosome])
        e <- end(rr[seqnames(rr) == chromosome])
        start <- min(s, start)
        end <- max(e, end)

    }
    .Object <- callNextMethod(.Object=.Object, range=range, start=start, end=end,
                              genome=genome, chromosome=chromosome, strand=strand, ...)
    ## We want to warn if searching was performed based on an identifier and now values have been returned
    if((!is.null(symbol) || !is.null(gene) || !is.null(transcript) || !is.null(entrez)) && length(range)==0){
        warning("Search by identifier did not yield any values", call.=FALSE)
    }
    return(.Object)
})

## Constructor. The following arguments are supported:
##    o start, end: numeric vectors of the item start and end coordinates
##    o biomart: a biomaRt object used to to query for gene annotations
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o strand: character, search for gene models on the plus strand ("+" or 0), the
##       minus strand ("-" or 1) or both strands ("+-" or "-+" or 2)
##    o stacking: character controlling the stacking of overlapping items. One in 'hide',
##       'dense', 'squish', 'pack' or 'full'.
##    o filters: list of additional filters for the biomaRt query, where the item names
##     	 are the filter identifiers and the item values are the filter values.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters
## (N)
BiomartGeneRegionTrack <- function(start=NULL, end=NULL, biomart, chromosome=NULL, strand, genome=NULL,
                                   stacking="squish", filters=list(), featureMap=NULL, name="BiomartGeneRegionTrack",
                                   symbol=NULL, gene=NULL, entrez=NULL, transcript=NULL, ...)
{
    ## Some default checking
    .missingToNull(c("genome"))
    if(missing(strand))
        strand <- "*"
    if((!is.null(start) || !is.null(end)) && is.null(chromosome)){
        stop("Also need to specify a chromsome when initializing a BiomartGeneRegionTrack with start or end coordinates")
    }
    if(!is.null(chromosome)){
        chromosome <- .chrName(chromosome)[1]
    }
    if(missing(biomart))
    {
        if(is.null(genome))
            stop("Need either a valid Mart connection object as 'biomart' argument or a UCSC genome identifier as the 'genome' argument.")
        biomart <- .genome2Dataset(genome)
    }else if(is.null(genome)){
        genome <- biomart@dataset
    }
    new("BiomartGeneRegionTrack", start=start, end=end, chromosome=chromosome, strand=strand,
        biomart=biomart, name=name, genome=genome, stacking=stacking, filter=filters, featureMap=featureMap,
        symbol=symbol, gene=gene, transcript=transcript, entrez=entrez, ...)
}


## This filters for genes with refseq IDs only
.refseqFilter <- list("with_refseq_dna"=TRUE)
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## GenomeAxisTrack:
##
## A track for the genome axis
## Slots:
##    o range: an object of class GRanges containing ranges to be highlighted along the axis.
## A bunch of DisplayPars are set during object instantiation:
##    o col: the color for the axis text, lines and tickmarks
##    o fill.range: the fill color for the optional range annotation
##    o col.range: the border color for the optional range annotation
##    o fontsize: the font size for the axis annotation
##    o showTitle: boolean, show or hide the title panel text
##    o background.title: the background color of the title panel
##    o cex: the cex value for the axis text
##    o exponent: the exponent for the axis coordinates. E.g., 3 means mb, 6 means gb, etc.
##    o distFromAxis: numeric, distance of text from the axis
##    o labelPos: character giving the position of axis labels, one in "alternating", "revAlternating",
##       "above" or "below"
##    o add53: boolean, add a 5'->3' indicator
##    o add35: boolean, add a 3'->5' indicator
##    o littleTicks: boolean, add second level of smaller tick marks
##    o size: the relative size of the track
##    o col.id, cex.id: text settings for the optional range annotation
##    o showId: boolean, show range annotation
##    o scale: numeric, if not NULL a small scale is drawn instead of the full axis,
##      if between 0 and 1 it is interpreted as a fraction of the region (otherwise absolute).
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("GenomeAxisTrack",
         contains="GdObject",
         representation=representation(range="GRanges"),
         prototype(range=GRanges(),
                   name="GenomeAxisTrack",
                   dp=DisplayPars(add35=FALSE,
                                  add53=FALSE,
                                  background.title="transparent",
                                  cex.id=0.7,
                                  cex=0.8,
                                  col.border.title="transparent",
                                  lwd.border.title=1,
                                  col.id="white",
                                  col.range="cornsilk4",
                                  distFromAxis=1,
                                  exponent=NULL,
                                  fill.range="cornsilk3",
                                  fontcolor="#808080",
                                  fontsize=10,
                                  labelPos="alternating",
                                  littleTicks=FALSE,
                                  lwd=2,
                                  scale=NULL,
                                  showId=FALSE,
                                  showTitle=FALSE,
                                  ticksAt=NULL,
                                  size=NULL,
                                  col="darkgray")))

## Only pass on the stuff to the GdObject initializer
setMethod("initialize", "GenomeAxisTrack", function(.Object, range, ids, ...){
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "GenomeAxisTrack")
    if(missing(range) || is.null(range))
        range <- GRanges()
    if(is(range, "IRanges"))
        range <- GRanges(ranges=range, seqnames="dummy", id=ids)
    .Object@range <- range
    .Object<- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor. The following arguments are supported:
##    o range: an object of class 'GRanges' containing regions to be highlighted on the axis by colored boxes
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters
## (N)
GenomeAxisTrack <- function(range=NULL, name="Axis", id, ...)
{
    if(missing(id))
        id <- names(range)
    new("GenomeAxisTrack", name=name, range=range, id=id, ...)
}
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## DataTrack:
##
## A track for all sorts of quantitative values
## Slots:
##    o data: a numeric matrix containing the data values
## A bunch of DisplayPars are set during object instantiation:
##    o jitter.x, jitter.y, amount, factor: parameters controlling the jittering in xy-type plots.
##    o span, degree, family, evaluation: parameters controlling the loess calculation.
##    o col.mountain, fill.mountain, col.baseline, lwd.baseline, lty.baseline, baseline:
##       parameters controlling the colors and the baseline position in mountain-type plots.
##    o fill.histogram, col.histogram: fill and line colors for the histogram-type plots
##    o box.ratio, box.width, varwidth, notch, notch.frac, levels.fos, stats, coef, do.out:
##       parameters controlling the boxplot appearance
##    o size: the relative size of the track
##    o type: the plot type, one or several in c("p", "l", "b", "a", "s", "g", "r", "S", "smooth",
##       "histogram", "mountain", "h", "boxplot", "gradient", "heatmap", "polygon")
##    o cex: the default pixel size
##    o ncolor, gradient: the number of colors and the base colors for the gradient type
##    o collpase: collapse overlapping ranges
##    o min.distance: the mimimum distance in pixel below which to collapse
##    o window: average the rows of the data matrix to 'window' slices on the data range
##    o separator: number of pixels used to separate individual samples in heatmap-type plots
##    o transformation: a function applied on the data matrix prior to plotting. The function
##         should accept exactly one argument and the return value needs to be a numeric vector
##         which can be coerced back into a data matrix of identical dimensionality as the input
##         data.
##    o aggregation: a function to aggregate values in windows or for collapsed items. Either a
##      function that collapses a numeric vector into a single number, or one of the predefined
##      options "mean", "median", "sum" "min", "max" or "extreme". Defaults to "mean"
##    o stackedBars: logical, draw stacked histograms if groups!=NULL. Else show grouped data as
##         side-by-side bars.
##    o na.rm: remove NA values before plotting
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("DataTrack",
         contains="NumericTrack",
         representation=representation(data="matrix", strand="character"),
         prototype=prototype(columns=c("score"),
                             name="DataTrack",
                             dp=DisplayPars(aggregateGroups=FALSE,
                                            aggregation="mean",
                                            alpha.confint=0.3,
                                            amount=NULL,
                                            baseline=NULL,
                                            box.legend=FALSE,
                                            box.ratio=1,
                                            box.width=NULL,
                                            grid=FALSE,
                                            cex.legend=0.8,
                                            cex.sampleNames=NULL,
                                            cex=0.7,
                                            coef=1.5,
                                            col.baseline=NULL,
                                            col.confint=NA,
                                            col.histogram=.DEFAULT_SHADED_COL,
                                            col.horizon=NA,
                                            col.mountain=NULL,
                                            col.sampleNames="white",
                                            col=trellis.par.get("superpose.line")[["col"]],
                                            collapse=FALSE,
                                            degree=1,
                                            do.out=TRUE,
                                            evaluation=50,
                                            factor=0.5,
                                            family="symmetric",
                                            fill.confint=NULL,
                                            fill.histogram=NULL,
                                            fill.horizon=c("#B41414", "#E03231", "#F7A99C", "#9FC8DC", "#468CC8", "#0165B3"),
                                            fill.mountain=c("#CCFFFF", "#FFCCFF"),
                                            fontface.legend=NULL,
                                            fontfamily.legend=NULL,
                                            fontsize.legend=NULL,
                                            fontcolor.legend=.DEFAULT_SHADED_COL,
                                            gradient=brewer.pal(9, "Blues"),
                                            groups=NULL,
                                            horizon.origin=0,
                                            horizon.scale=NULL,
                                            jitter.x=FALSE,
                                            jitter.y=FALSE,
                                            levels.fos=NULL,
                                            legend=TRUE,
                                            lineheight.legend=NULL,
                                            lty.baseline=NULL,
                                            lty.mountain=NULL,
                                            lwd.baseline=NULL,
                                            lwd.mountain=NULL,
                                            min.distance=0,
                                            na.rm=FALSE,
                                            ncolor=100,
                                            notch.frac=0.5,
                                            notch=FALSE,
                                            pch=20,
                                            separator=0,
                                            showColorBar=TRUE,
                                            showSampleNames=FALSE,
                                            size=NULL,
                                            span=1/5,
                                            stackedBars=TRUE,
                                            stats=boxplot.stats,
                                            transformation=NULL,
                                            type="p",
                                            varwidth=FALSE,
                                            window=NULL,
                                            windowSize=NULL,
                                            ylim=NULL)))

## Only pass on the stuff to the GdObject initializer
setMethod("initialize", "DataTrack", function(.Object, data=matrix(), strand, ...){
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "DataTrack")
    .Object@data <- data
    if(!missing(strand))
        .Object@strand <- unique(strand)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## The file-based version of the DataTrack class. This will mainly provide a means to dispatch to
## a special 'subset' method which should stream the necessary data from disk.
setClass("ReferenceDataTrack", contains=c("DataTrack", "ReferenceTrack"))

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritence has some strange features with regards to method selection
setMethod("initialize", "ReferenceDataTrack", function(.Object, stream, reference, mapping=list(),
                                                       args=list(), defaults=list(), ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object=.Object, reference=reference, stream=stream,
                                                            mapping=mapping, args=args, defaults=defaults)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor. The following arguments are supported:
##    o range: an object of class 'GRanges' containing the data coordinates, or an object of class data.frame
##       with the two mandatroy columns 'start' and 'end'. The coordinates (i.e., the length of the GRanges object)
##       have to match the columns of the data matrix (see below).
##    o data: a numeric matrix of data points with number of columns equal to the number of coordinates in 'range',
##      or a numeric vector of appropriate length that will be coerced in such a one-row matrix.
##    o start, end, width: numeric vectors of the item start and end coordinates, have to match the columns in 'data'
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o strand: character, display data on the plus strand ("+" or 0), the minus strand ("-" or 1) or both
##      strands ("+-" or "-+" or 2). Currently has to be unique for the whole track.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters
## (N)
DataTrack <- function(range=NULL, start=NULL, end=NULL, width=NULL, data, chromosome, strand, genome,
                      name="DataTrack", importFunction, stream=FALSE, ...)
{
    ## Build a GRanges object from the inputs
    wasGR <- is(range, "GRanges") || is.character(range)
    fromFile <- is.character(range)
    isStream <- FALSE
    .missingToNull(c("strand", "chromosome", "importFunction", "genome"))
    args <- list(strand=strand, chromosome=chromosome, genome=genome)
    defs <- list(strand="*", chromosome="chrNA")
    range <- .buildRange(range=range, start=start, end=end, width=width, args=args, defaults=defs,
                         asIRanges=FALSE, chromosome=chromosome, genome=NA, trackType="DataTrack",
                         importFun=importFunction, stream=stream)
    if(is.list(range)){
        isStream <- TRUE
        slist <- range
        range <- GRanges()
    }
    ## Some default checking
    if(length(unique(strand(range)))>1)
        stop("The strand has to be unique for all ranges in a DataTrack object.")
    if(!missing(data) && length(range) > 0)
    {
        if(is.character(data))
        {
            if(!wasGR)
                stop("Columns indices for the data section are only allowed when 'range' is of class 'GRanges'")
            mt <- is.na(match(data, colnames(values(range))))
            if(any(mt))
                warning("Unable to match data columns: ", paste(data[mt], collapse=","))
            data <- as.data.frame(values(range)[,data[!mt], drop=FALSE])
        }
        if(is.null(dim(data)))
            dim(data) <- c(1, length(data))
        if(is.matrix(data))
            data <- as.data.frame(t(data))
    } else {
        data <- if(ncol(values(range))) as.data.frame(values(range)) else matrix(nrow=0, ncol=0)
    }
    data <- .prepareDtData(data, len=length(range))
    if(missing(chromosome) || is.null(chromosome))
        chromosome <- if(length(range)>0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    values(range) <- NULL
    if(!isStream){
        return(new("DataTrack", chromosome=chromosome, strand=as.character(strand(range)), range=range,
                   name=name, genome=genome, data=data, ...))
    }else{
        ## A bit hackish but for some functions we may want to know which track type we need but at the
        ## same time we do not want to enforce this as an additional argument
        e <- new.env()
        e[["._trackType"]] <- "DataTrack"
        environment(slist[["stream"]]) <- e
        return(new("ReferenceDataTrack", chromosome=chromosome, strand=as.character(strand(range)), range=range,
                   name=name, genome=genome, data=data, stream=slist[["stream"]], reference=slist[["reference"]],
                   mapping=slist[["mapping"]], args=args, defaults=defs, ...))
    }
}
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## IdeogramTrack:
##
## A track for chromosome ideograms
## A bunch of DisplayPars are set during object instantiation:
##    o size: the relative size of the track
##    o col, fill: the border and fill color used for the highlighted currently displayed region on the chromosome
##    o fontcolor, fontface, fontfamily, fonsize, lineheight, cex : the color, family, face, size, lineheight and
##       expansion factor for the chromosome name text
##    o showId: indicate the chromosome name next to the ideogram
##    o showBandId: show the identifiers of the chromosome bands
##    o cex.bands: character expansion factor for the chromosome band information
##    o bevel: the amount of beveling at the ends of the ideogram. A number between 0 and 1.
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("IdeogramTrack", contains = "RangeTrack",
         representation=representation(bandTable="data.frame"),
         prototype=prototype(name="IdeogramTrack",
                             bandTable=data.frame(),
                             dp=DisplayPars(background.title="transparent",
                                            bevel=0.45,
                                            cex.bands=0.7,
                                            cex=0.8,
                                            col="red",
                                            col.border.title="transparent",
                                            lwd.border.title=1,
                                            fill="#FFE3E6",
                                            fontface=1,
                                            fontfamily="sans",
                                            fontcolor=.DEFAULT_SHADED_COL,
                                            fontsize=10,
                                            outline=FALSE,
                                            showBandId=FALSE,
                                            lty=1,
                                            lwd=1,
                                            showId=TRUE,
                                            showTitle=FALSE,
                                            size=NULL)))

## Grab the chromosome band and length information from UCSC and fill the ranges slot.
setMethod("initialize", "IdeogramTrack", function(.Object, genome, chromosome, bands, name, ...){
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "IdeogramTrack")
    if(missing(bands))
       bands <- NULL
    if(is.null(bands) && (missing(genome) || missing(chromosome)))
        return(callNextMethod(.Object=.Object, range=GRanges(), genome=NULL, chromosome=NULL, ...))
    if(is.null(bands)){
        sessionInfo <- .cacheGenomes(genome=genome)
        .Object@bandTable <- sessionInfo$bands
        bands <- sessionInfo$bands
    }else{
        .checkClass(bands, "data.frame")
        cols <- c("chrom", "chromStart", "chromEnd", "name", "gieStain")
        miss <- ! cols %in% colnames(bands)
        if(any(miss))
            stop(sprintf("The following column%s missing from the bands table: %s",
                         ifelse(sum(miss)>1, "s are", " is"), paste(cols[miss], collapse=", ")))
        .Object@bandTable <- bands
    }
    chromosome <- if(is.null(chromosome)) as.character(bands[1, "chrom"]) else .chrName(chromosome)[1]
    bands <- bands[bands$chrom==chromosome,]
    if(nrow(bands)==0)
        stop("Chromosome '", chromosome, "' does not exist on UCSC genome '", genome, "'")
    if(is.null(name))
        name <- .chrName(chromosome)[1]
    bnames <- as.character(bands$name)
    sel <- is.na(bnames)
    if(any(sel))
        bnames[sel] <- paste("band", seq_len(sum(sel)), sep="_")
    if(any(bnames == ""))
        bnames[bnames == ""] <- sprintf("band_%i", which(bnames == ""))
    ranges <- GRanges(seqnames=bnames, ranges=IRanges(start=bands$chromStart, end=bands$chromEnd),
                      name=bnames, type=as.character(bands$gieStain))
    .Object <- callNextMethod(.Object=.Object, range=ranges, genome=genome, chromosome=chromosome, name=name, ...)
    return(.Object)
})

## Constructor. The following arguments are supported:
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters
## (N)
IdeogramTrack <- function(chromosome=NULL, genome, name=NULL, bands=NULL, ...){
    if(missing(genome)) stop("Need to specify genome for creating an IdeogramTrack")
    new("IdeogramTrack", chromosome=chromosome, genome=genome, name=name, bands=bands, ...)
}
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## UcscTrack:
##
## Strictly speaking this is not a class, but rather some sort of meta-constructor for several of the previously
## defined track types directly from UCSC data. It will fetch online data from a particular track (or a sub-table of
## a track) and feed it to one of the original constructors with user-provided argument mappings.
##----------------------------------------------------------------------------------------------------------------------
## Constructor. The following arguments are supported:
##    o track: a character of one of the available UCSC tracks
##    o table: a character of one one of the sub-tables of the track, or NULL to fetch all
##    o trackType: a character giving the name of the constructor to pass the data to, one in
##       c("AnnotationTrack", "GeneRegionTrack", "DataTrack", "GenomeAxisTrack")
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o from: the starting and end coordinates of the track data
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters

## A simple caching mechanism for UCSC session information. The overhead for establishing a connection to UCSC is
## quite significant and we can shave off 5 to 10 seconds here by caching sessions and associated information
## for a particular genome and chromosome.
.ucscCache <- new.env()
.ensemblCache <- new.env()
.martCache <- new.env()
.doCache <- function(token, expression, env, callEnv=environment())
{
     if(!token %in% base::ls(env))
     {
         res <- eval(expression, envir=callEnv)
         assign(x=token, value=res, envir=env)
         res
     } else env[[token]]
}
.cacheTracks <- function(genome, chromosome, track, env=.ucscCache)
{
    genomes <- .doCache("availableGenomes", expression(ucscGenomes()), env)
    if(!genome %in% as.character(genomes[,"db"]))
        stop("'", genome, "' is not a valid UCSC genome.")
    sessionToken <- paste("session", genome, sep="_")
    tracksToken <- paste("tracks", genome, sep="_")
    tablesToken <- paste("tables", track, genome, sep="_")
    cenv <- environment()
    session <- .doCache(sessionToken,
                        expression({myUcscUrl <- getOption("Gviz.ucscUrl")
                                    tmp <- if(is.null(myUcscUrl)) browserSession() else browserSession(url=myUcscUrl)
                                    genome(tmp) <- genome
                                    tmp}), env, cenv)
    availTracks <- .doCache(tracksToken, expression(trackNames(ucscTableQuery(session))), env, cenv)
    track <- match.arg(track, sort(c(availTracks, names(availTracks))))
    if(!is.na(availTracks[track]))
        track <- availTracks[track]
    availTables <- .doCache(tablesToken, expression({query <- ucscTableQuery(session, track)
                                                     sort(tableNames(query))}), env, cenv)
    chrInfo <- seqlengths(session)
    return(list(session=session, availTracks=availTracks, availTables=availTables, track=track,
                chrInfo=chrInfo))
}
.cacheGenomes <- function(genome=NULL, env=.ucscCache)
{
    availToken <- "availableGenomes"
    genomesToken <- paste("genomeBands", genome, sep="_")
    genomes <- .doCache(availToken, expression(ucscGenomes()), env)
    bands <- NULL
    if(!is.null(genome))
    {
        cenv <- environment()
        bands <- .doCache(genomesToken, expression({
            if(!genome %in% as.character(genomes[,"db"]))
                stop("'", genome, "' is not a valid UCSC genome.")
            sessionToken <- paste("session", genome, sep="_")
            session <- .doCache(sessionToken,
                                expression({myUcscUrl <- getOption("Gviz.ucscUrl")
                                            tmp <- if(is.null(myUcscUrl)) browserSession() else browserSession(url=myUcscUrl)
                                            genome(tmp) <- genome
                                            tmp}), env, cenv)
            query <-  tryCatch(ucscTableQuery(session, "cytoBandIdeo"), error=function(e) {
                               warning("There doesn't seem to be any cytoband data available for genome '", genome,
                                       "' at UCSC or the service is temporarily down. Trying to fetch the chromosome length data.")
                               tryCatch(ucscTableQuery(session, table="chromInfo"), error=function(e)
                                            stop("There doesn't seem to be any chromosome length data available for genome '", genome,
                                                 "' at UCSC or the service is temporarily down."))
                           })
            out <- getTable(query)
            if (all(c("chrom","size") %in% colnames(out))) {
                out <- data.frame(chrom=out$chrom, chromStart=0, chromEnd=out$size, name="",  gieStain="gneg", stringsAsFactors=F)
            }
            out
        }), env, cenv)
    }

    return(list(availableGenomes=genomes, bands=bands))
}
.cacheMartData <- function(bmtrack, chromosome=NULL, staged=FALSE){
    uid <- .bmGuid(bmtrack)
    req <- if(!is.null(bmtrack@start) && !is.null(bmtrack@end)) GRanges(seqnames=chromosome[1], IRanges(start=bmtrack@start, bmtrack@end)) else NULL
    if(is.null(chromosome) || is.null(.martCache[[uid]])){
        data <- .fetchBMData(bmtrack, chromosome, staged)
        if(!is.null(req)){
            .martCache[[uid]] <- list(data=data, ranges=req)
        }else{
            req <- range(data)
        }
        if(length(data) && .dpOrDefault(bmtrack, "verbose", FALSE))
            message("Loaded data from Biomart for region ", paste(sprintf("%s:%i-%i(%s)", seqnames(req), start(req), end(req), strand(req)), collapse=" and "))
    }else{
        rr <- .martCache[[uid]][["ranges"]]
        dd <- .martCache[[uid]][["data"]]
        if(!is.null(req) && suppressWarnings(req %within% rr)){
            genes <- unique(subsetByOverlaps(dd, req)$gene)
            data <- dd[seqnames(dd) == chromosome[1] & dd$gene %in% genes]
            if(.dpOrDefault(bmtrack, "verbose", FALSE))
                message(sprintf("Retrieved data from cache for region %s:%i-%i(%s)", chromosome, start(req), end(req), strand(req)))
        }else{
            data <- .fetchBMData(bmtrack, chromosome, staged)
            if(is.null(req)){
                req <- range(data)
            }
            .martCache[[uid]][["data"]] <- suppressWarnings(c(.martCache[[uid]][["data"]], data[!(seqnames(data) == chromosome[1] & data$gene %in% dd$gene)]))
            .martCache[[uid]][["ranges"]] <- suppressWarnings(union(rr, req))
            if(length(req) && .dpOrDefault(bmtrack, "verbose", FALSE))
                message("Loaded data from Biomart for region ", paste(sprintf("%s:%i-%i(%s)", seqnames(req), start(req), end(req), strand(req)), collapse=" and "))
        }
    }
    return(data)
}

## empty the session cache
clearSessionCache <- function(){
    assignInNamespace(".ucscCache", new.env(), ns="Gviz")
    assignInNamespace(".ensemblCache", new.env(), ns="Gviz")
    assignInNamespace(".martCache", new.env(), ns="Gviz")
}



## (N)
UcscTrack <- function(track, table=NULL, trackType=c("AnnotationTrack", "GeneRegionTrack",
                                           "DataTrack", "GenomeAxisTrack"),
                      genome, chromosome, name=NULL, from, to, ...)
{
    trackType <- match.arg(trackType)
    if(missing(genome) || !isSingleString(genome)) stop("Need to specify genome for creating a UcscTrack")
    if(missing(chromosome)) stop("Need to specify chromosome for creating a UcscTrack")
    chromosome <- .chrName(chromosome)[1]
    sessionInfo <- .cacheTracks(genome=genome, chromosome=chromosome, track=track, env=.ucscCache)
    if(missing(from))
        from <- 1
    if(missing(to))
        to <- sessionInfo$chrInfo[chromosome]
    gr <- GRanges(ranges=IRanges(start=from, end=to), seqnames=chromosome)
    suppressWarnings(genome(gr) <- unname(genome))[1]
    query <- ucscTableQuery(sessionInfo$session, sessionInfo$track, gr)
    if(!is.null(table))
    {
        table <- match.arg(table, sessionInfo$availTables)
        tableName(query) <- table
    }
    if(is.null(name))
      name <- if(is.null(table)) track else paste(sessionInfo$track, table)
    tableDat <- if(trackType=="DataTrack"){
        tmp <- try(track(query), silent=TRUE)
        if(is(tmp, "try-error")){
            warning(tmp)
            data.frame()
        } else as.data.frame(tmp)} else {
            tmp <- try(getTable(query), silent=TRUE)
            if(is(tmp, "try-error")){
                warning(tmp)
                data.frame()
            } else tmp}
    if(is(tmp, "try-error") && nrow(tableDat)==0)
        stop("Error fetching data from UCSC")
    args <- lapply(list(...), function(x) if(is.character(x) && length(x)==1)
                   if(!x %in% colnames(tableDat)) x else tableDat[,x] else x)
    if(trackType=="GeneRegionTrack")
    {
        args$start <- from
        args$end <- to
    }
    args <- lapply(args, function(x) if(!length(x))  NULL else x)
    trackObject <- do.call(trackType, args=c(list(chromosome=chromosome, genome=genome, name=name), args))
    return(trackObject)
  }
##----------------------------------------------------------------------------------------------------------------------




##----------------------------------------------------------------------------------------------------------------------
## AlignedReadTrack:
##
## A track to visualize sequence reads, as typically produced by NGS experiments
## Slots: no additional formal slots are defined
## A bunch of DisplayPars are set during object instantiation:
##    o detail: the amount of plotting details to show for the aligned reads, one in c("reads", "coverage")
##----------------------------------------------------------------------------------------------------------------------
## (N)
setClass("AlignedReadTrack",
         representation=representation(coverage="list",
                                       coverageOnly="logical"),
         contains="StackedTrack",
         prototype=prototype(stacking="squish",
                             name="AlignedReadTrack",
                             coverageOnly=FALSE,
                             dp=DisplayPars(detail="coverage",
                                            type="histogram",
                                            fill="#0080ff",
                                            size=NULL,
                                            collapse=FALSE)))

## Recompute coverage on plus and minus strand and combined strands for AlignedRead tracks
## and update the respective slot to hold this information
setMethod("setCoverage", signature("AlignedReadTrack"),
          definition=function(GdObject){
              if(length(GdObject))
              {
                  str <- factor(strand(GdObject), levels=c("+", "-"))
                  gdSplit <- as.list(split(range(GdObject), str))
                  covs <- lapply(gdSplit, coverage)
                  covs[["*"]] <- coverage(range(GdObject))
                  GdObject@coverage <- covs
              }
              return(GdObject)
          })

## Essentially we just update the display parameters here and precompute the coverage
setMethod("initialize", "AlignedReadTrack", function(.Object, coverageOnly=FALSE, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "AlignedReadTrack")
    .Object <- callNextMethod(.Object, ...)
    .Object <- setCoverage(.Object)
    if(coverageOnly)
    {
      ## from <- min(unlist(lapply(.Object@coverage, function(y) if(length(y)) min(start(y)))))
      ## to <- max(unlist(lapply(.Object@coverage, function(y) if(length(y)) max(end(y)))))
      from <- min(start(range(.Object)))
      to <- max(end(range(.Object)))
        .Object@range <- GRanges(ranges=IRanges(start=from, end=to),
                                 strand=names(.Object@coverage), seqnames=.Object@chromosome)
        .Object@coverageOnly <- coverageOnly
    }
    return(.Object)
})

## Constructor. The following arguments are supported:
##    o range: a data.frame or a GRanges object containing the information
##       about the track items. If a data.frame, it needs to be coerceable
##	 to a GRanges object, i.e., it needs at least the mandatory 'start', 'stop' and
##	 'strand' columns.
##	Instead of using the 'range' parameter, all these values can also be passed as
##	individual vectors, in which case they need to be of similar length.
##    o start, end, width: numeric vectors of the item start and end coordinates, or their widths
##    o strand: the strand information needs to be provided in the form '+' for
##       the Watson strand, '-' for the Crick strand.
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o stacking: character controlling the stacking of overlapping items. One in 'hide',
##       'dense', 'squish', 'pack' or 'full'.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as further DisplayParameters
## (N)
AlignedReadTrack <- function(range=NULL, start=NULL, end=NULL, width=NULL, chromosome, strand, genome, stacking="squish",
                             name="AlignedReadTrack", coverageOnly=FALSE, ...)
{
    .missingToNull(c("strand", "chromosome", "genome"))
    ## Build a GRanges object from the inputs
    range <- .buildRange(range=range, start=start, end=end, width=width,
                         args=list(strand=strand, genome=genome, chromosome=chromosome),
                         defaults=list(strand="+", genome=NA, chromosome="chrNA"), chromosome=chromosome,
                         trackType="AlignedReadTrack")
    str <- unique(as.character(GenomicRanges::strand(range)))
    if("*" %in% str)
        stop("Only '+' and '-' strand information is allowed for AlignedReadTrack objects.")
    if(missing(chromosome) || is.null(chromosome))
        chromosome <- if(length(range)>0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    ## And finally the object instantiation
    return(new("AlignedReadTrack", chromosome=chromosome, range=range, name=name, genome=genome(range)[1],
               stacking=stacking, coverageOnly=coverageOnly, ...))
}





##----------------------------------------------------------------------------------------------------------------------





##----------------------------------------------------------------------------------------------------------------------
## SequenceTrack:
##
## A generic track to visualize nucleotide sequences. This class is virtual.
## Slots:
##    o chromosome: a character vector giving the active chromosome for which the
##	 track is defined. Valid chromosome names are:
##          - a single numeric character
##	    - a string, starting with 'chr', followed by any additional characters
##    o genome: character giving the reference genome for which the track is defined.
## A bunch of DisplayPars are set during object instantiation:
##    o foo: bar
setClass("SequenceTrack",
         representation=representation("VIRTUAL",
                                       chromosome="character",
                                       genome="character"),
         contains="GdObject",
         prototype=prototype(name="Sequence",
                             dp=DisplayPars(add53=FALSE,
                                            background.title="transparent",
                                            col="darkgray",
                                            complement=FALSE,
                                            fontcolor=getBioColor("DNA_BASES_N"),
                                            fontface=2,
                                            fontsize=10,
                                            lwd=2,
                                            min.width=2,
                                            noLetters=FALSE,
                                            showTitle=FALSE,
                                            size=NULL),
                             genome=as.character(NA),
                             chromosome="chrNA"))

## Essentially we just update the display parameters here and set the chromosome and the genome
setMethod("initialize", "SequenceTrack", function(.Object, chromosome, genome, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "SequenceTrack")
     if(!missing(chromosome) && !is.null(chromosome)){
        .Object@chromosome <- .chrName(chromosome)[1]
    }
    if(missing(genome) || is.null(genome))
        genome <- as.character(NA)
    .Object@genome <- genome
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## We want the following behaviour in the constructor:
##   a) sequence is missing (NULL) => build SequenceDNAStringSetTrack with chromosome NA and genome as supplied or NA if missing
##   b) sequence is DNAStringSet => build SequenceDNAStringSetTrack where chromosome is names(sequence)[1] or the supplied
##      chromosome if available, and genome as supplied or NA if missing
##   c) sequence is BSgenome => build SequenceBSgenomeTrack where chromosome is seqnames(sequence)[1] or the supplied
##      chromosome if available, and genome is the supplied genome or the one extracted from the BSgenome object
SequenceTrack <- function(sequence, chromosome, genome, name="SequenceTrack", importFunction, stream=FALSE, ...){
    .missingToNull(c("chromosome", "genome", "sequence"))
    if(is.null(sequence)){
        return(new("SequenceDNAStringSetTrack", chromosome=chromosome, genome=genome, name=name, ...))
    }
    if(is(sequence, "BSgenome")){
        if(is.null(genome))
            genome <- providerVersion(sequence)
        if(is.null(chromosome))
            chromosome <- seqnames(sequence)[1]
        obj <- new("SequenceBSgenomeTrack", sequence=sequence, chromosome=chromosome, genome=genome, name=name, ...)
    }else if(is(sequence, "DNAStringSet")){
        if(is.null(names(sequence)))
            stop("The sequences in the DNAStringSet must be named")
        if(any(duplicated(names(sequence))))
            stop("The sequence names in the DNAStringSet must be unique")
        if(is.null(chromosome))
            chromosome <- names(sequence)[1]
        obj <- new("SequenceDNAStringSetTrack", sequence=sequence, chromosome=chromosome, genome=genome, name=name, ...)
    } else if(is.character(sequence)){
        sequence <- sequence[1]
        if(!file.exists(sequence))
            stop(sprintf("'%s' is not a valid file.", sequence))
        ext <- .fileExtension(sequence)
        obj <- if(missing(importFunction) && ext %in% c("fa", "fasta")){
            if(!file.exists(paste(sequence, "fai", sep="."))){
                new("SequenceDNAStringSetTrack", sequence=readDNAStringSet(sequence), chromosome=chromosome,
                    genome=genome, name=name, ...)
            }else{
                new("ReferenceSequenceTrack", chromosome=chromosome, genome=genome, name=name,
                    stream=.import.fasta, reference=path.expand(sequence), ...)
            }
        }else if(missing(importFunction) && ext == "2bit"){
            new("ReferenceSequenceTrack", chromosome=chromosome, genome=genome, name=name,
                stream=.import.2bit, reference=path.expand(sequence), ...)
        }else{
            if(missing(importFunction)){
                stop(sprintf("No predefined import function exists for files with extension '%s'. Please manually provide an import function.",
                             ext))
            }else{
                if(!stream){
                    seq <- importFunction(file=sequence)
                    if(!is(seq, "DNAStringSet"))
                        stop("The import function did not provide a valid DNAStringSet object. Unable to build track from file '",
                             sequence, "'")
                    new("SequenceDNAStringSetTrack", sequence=importFunction(file=sequence), chromosome=chromosome,
                        genome=genome, name=name, ...)
                }else{
                    new("ReferenceSequenceTrack", chromosome=chromosome, genome=genome, name=name,
                        stream=importFunction, reference=path.expand(sequence), ...)
                }
            }
        }
    }else{
        stop("Argument sequence must be of class 'BSgenome', 'DNAStringSet' or 'character'")
    }
    return(obj)
}
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## SequenceDNAStringSetTrack:
##
## A track to visualize nucleotide sequences that are stored in a DNSStringSet
## Slots:
##    o sequence: a DNAStringSet object that contains all the sequence data
##----------------------------------------------------------------------------------------------------------------------
setClass("SequenceDNAStringSetTrack",
         representation=representation(sequence="DNAStringSet"),
         contains="SequenceTrack",
         prototype=prototype(sequence=DNAStringSet()))

setMethod("initialize", "SequenceDNAStringSetTrack", function(.Object, sequence, ...) {
    if(missing(sequence) || is.null(sequence))
        sequence <- DNAStringSet()
    .Object@sequence <- sequence
    .Object <- callNextMethod(.Object, ...)
     return(.Object)
})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## SequenceBSgenomeTrack:
##
## A track to visualize nucleotide sequences that are stored in a BSgenome package
## Slots:
##    o sequence: a DNAStringSet object that contains all the sequence data
##    o pointerCache: an environemnt to hold pointers to the BSgenome sequences to prevent garbage collection. This
##       will only be filled once the individual sequences have been accessed for the first time
##----------------------------------------------------------------------------------------------------------------------
setClass("SequenceBSgenomeTrack",
         representation=representation(sequence="BSgenomeOrNULL", pointerCache="environment"),
         contains="SequenceTrack",
         prototype=prototype(sequence=NULL))

setMethod("initialize", "SequenceBSgenomeTrack", function(.Object, sequence=NULL, ...) {
    .Object@sequence <- sequence
    .Object@pointerCache <- new.env()
    .Object <- callNextMethod(.Object, ...)
     return(.Object)
})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## ReferenceSequenceTrack:
##
## The file-based version of the ReferenceTrack class. This will mainly provide a means to dispatch to
## a special 'subseq' method which should stream the necessary data from disk.
##----------------------------------------------------------------------------------------------------------------------
setClass("ReferenceSequenceTrack", contains=c("SequenceDNAStringSetTrack", "ReferenceTrack"))

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritence has some strange features with regards to method selection
setMethod("initialize", "ReferenceSequenceTrack", function(.Object, stream, reference, ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object=.Object, reference=reference, stream=stream)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## AlignmentsTrack
##
## A track that represents aligned NGS reads on a genome. This supports gapped an paired alignments.
##----------------------------------------------------------------------------------------------------------------------
setClassUnion("SequenceTrackOrNULL", c("SequenceTrack", "NULL"))
setClass("AlignmentsTrack",
         representation=representation(stackRanges="GRanges",
                                       sequences="DNAStringSet",
                                       referenceSequence="SequenceTrackOrNULL"),
         contains="StackedTrack",
         prototype=prototype(stacking="squish",
                             name="AlignmentsTrack",
                             coverageOnly=FALSE,
                             stackRanges=GRanges(),
                             sequences=DNAStringSet(),
                             referenceSequence=NULL,
                             dp=DisplayPars(alpha.reads=0.5,
                                            alpha.mismatch=1,
                                            cex=0.7,
                                            cex.mismatch=NULL,
                                            col.coverage=NULL,
                                            col.gap=.DEFAULT_SHADED_COL,
                                            col.mates=.DEFAULT_BRIGHT_SHADED_COL,
                                            col.mismatch=.DEFAULT_SHADED_COL,
                                            col.reads=NULL,
                                            col.sashimi=NULL,
                                            col=.DEFAULT_SHADED_COL,
                                            collapse=FALSE,
                                            coverageHeight=0.1,
                                            fill.coverage=NULL,
                                            fill.reads=NULL,
                                            fill="#BABABA",
                                            fontface.mismatch=2,
                                            lty.coverage=NULL,
                                            lty.gap=NULL,
                                            lty.mates=NULL,
                                            lty.mismatch=NULL,
                                            lty.reads=NULL,
                                            lty=1,
                                            lwd.coverage=NULL,
                                            lwd.gap=NULL,
                                            lwd.mates=NULL,
                                            lwd.mismatch=NULL,
                                            lwd.reads=NULL,
                                            lwd.sashimiMax=10,
                                            lwd=1,
                                            max.height=10,
                                            min.height=5,
                                            minCoverageHeight=50,
                                            minSashimiHeight=50,
                                            noLetters=FALSE,
                                            sashimiFilter=NULL,
                                            sashimiFilterTolerance=0L,
                                            sashimiHeight=0.1,
                                            sashimiScore=1,
                                            sashimiStrand="*",
                                            showMismatches=TRUE,
                                            size=NULL,
                                            transformation=NULL,
                                            type=c("coverage", "pileup"))))

setMethod("initialize", "AlignmentsTrack", function(.Object, stackRanges=GRanges(), stacks=numeric(), sequences=DNAStringSet(),
                                                    referenceSequence=NULL, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "AlignedReadTrack")
    .Object@stackRanges <- stackRanges
    .Object <- callNextMethod(.Object, ...)
    .Object@stacks <- stacks
    .Object@sequences <- sequences
    .Object@referenceSequence <- referenceSequence
    return(.Object)
})



setClass("ReferenceAlignmentsTrack", contains=c("AlignmentsTrack", "ReferenceTrack"))

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritence has some strange features with regards to method selection
setMethod("initialize", "ReferenceAlignmentsTrack", function(.Object, stream, reference, mapping=list(),
                                                             args=list(), defaults=list(), stacks=numeric(),
                                                             stackRanges=GRanges(), sequences=DNAStringSet(),
                                                             referenceSequence=NULL, ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object=.Object, reference=reference, stream=stream,
                                                            mapping=mapping, args=args, defaults=defaults)
    .Object <- callNextMethod(.Object, ...)
    .Object@referenceSequence <- referenceSequence
    return(.Object)
})


## Constructor
AlignmentsTrack <- function(range=NULL, start=NULL, end=NULL, width=NULL, strand, chromosome, genome,
                            stacking="squish", id, cigar, mapq, flag, isize, groupid, status, md, seqs,
                            name="AlignmentsTrack", isPaired=TRUE, importFunction, referenceSequence, ...){
    ## Some defaults
    if(missing(importFunction))
        importFunction <- .import.bam.alignments
    covars <- .getCovars(range)
    isStream <- FALSE
    if(!is.character(range)){
        n <- max(c(length(start), length(end), length(width)), nrow(covars))
        id <- .covDefault(id, covars[["id"]], paste("read", seq_len(n), sep="_"))
        cigar <- .covDefault(cigar, covars[["cigar"]], paste(if(is(range, "GRangesOrIRanges")) width(range) else width, "M", sep=""))
        mapq <- .covDefault(mapq, covars[["mapq"]], rep(as.integer(NA), n))
        flag <- .covDefault(flag, covars[["flag"]], rep(as.integer(NA), n))
        isize <- .covDefault(isize, covars[["isize"]], rep(as.integer(NA), n))
        groupid <- .covDefault(groupid, covars[["groupid"]], seq_len(n))
        md <- .covDefault(md, covars[["md"]], rep(as.character(NA), n))
        status <- .covDefault(status, covars[["status"]], ifelse(groupid %in% groupid[duplicated(groupid)], "mated", "unmated"))
    }
    ## Build a GRanges object from the inputs
    .missingToNull(c("strand", "chromosome", "importFunction", "genome", "id", "cigar", "mapq", "flag", "isize", "groupid", "status",
                     "md", "seqs", "referenceSequence"))
    args <- list(id=id, cigar=cigar, mapq=mapq, flag=flag, isize=isize, groupid=groupid, status=status, strand=strand, md=md,
                 chromosome=chromosome, genome=genome)
    defs <- list(strand="*", chromosome="chrNA", genome=NA, id=as.character(NA), cigar=as.character(NA), mapq=as.integer(NA),
                 flag=as.integer(NA), isize=as.integer(NA), groupid=as.character(NA), status=as.character(NA), md=as.character(NA))
    range <- .buildRange(range=range, start=start, end=end, width=width,
                         args=args, defaults=defs, chromosome=chromosome, trackType="AlignmentsTrack",
                         importFun=importFunction, stream=TRUE, autodetect=TRUE)
    ## This is going to be a list if we have to stream data from a file, otherwise we can compute some additional values
    if(is.list(range)){
        isStream <- TRUE
        slist <- range
        range <- GRanges()
        stackRanges <- GRanges()
        stacks <- NULL
        seqs <- DNAStringSet()
    }else{
        if(is.null(seqs)){
            seqs <- DNAStringSet(sapply(width(range), function(x) paste(rep("N", x), collapse="")))
        }
        tmp <- .computeAlignments(range)
        range <- tmp$range
        stackRanges <- tmp$stackRange
        stacks <- tmp$stacks
    }
    ## If no chromosome was explicitely asked for we just take the first one in the GRanges object
    if(missing(chromosome) || is.null(chromosome))
        chromosome <- if(length(range)>0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    ## And finally the object instantiation
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    if(!isStream){
        return(new("AlignmentsTrack", chromosome=chromosome[1], range=range, stacks=stacks,
                   name=name, genome=genome, stacking=stacking, stackRanges=stackRanges, sequences=seqs,
                   referenceSequence=referenceSequence, ...))
        }else{
            ## A bit hackish but for some functions we may want to know which track type we need but at the
            ## same time we do not want to enforce this as an additional argument
            e <- new.env()
            e[["._trackType"]] <- "AlignmentsTrack"
            e[["._isPaired"]] <- isPaired
            environment(slist[["stream"]]) <- e
            return(new("ReferenceAlignmentsTrack", chromosome=chromosome[1], range=range, stackRanges=stackRanges,
                       name=name, genome=genome, stacking=stacking, stream=slist[["stream"]], reference=slist[["reference"]],
                       mapping=slist[["mapping"]], args=args, defaults=defs, stacks=stacks, referenceSequence=referenceSequence, ...))
        }
}
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## HighlightTrack:
##
## A collection container for several track objects for which a particular area needs to be highlighted.
##----------------------------------------------------------------------------------------------------------------------
setClass("HighlightTrack",
         representation=representation(trackList="list"),
         contains=c("RangeTrack"),
         prototype=prototype(dp=DisplayPars(col="red",
                                            fill="#FFE3E6",
                                            inBackground=TRUE)))

setMethod("initialize", "HighlightTrack", function(.Object, trackList, ...) {
    .Object <- .updatePars(.Object, "HighlightTrack")
    .Object@trackList <- trackList
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

HighlightTrack <- function(trackList=list(), range=NULL, start=NULL, end=NULL, width=NULL, chromosome, genome,
                           name="HighlightTrack",  ...){
    ## Some defaults
    covars <- .getCovars(range)
    n <- max(c(length(start), length(end), length(width)), nrow(covars))
    ## Build a GRanges object from the inputs
    .missingToNull(c("strand", "chromosome", "genome"))
    args <- list(chromosome=chromosome, genome=genome)
    defs <- list(strand="*", density=1, chromosome="chrNA", genome=NA)
    range <- .buildRange(range=range, start=start, end=end, width=width,
                         args=args, defaults=defs, chromosome=chromosome, trackType="HighlightTrack")
    if(is.list(range)){
        range <- GRanges()
    }
    if(!is.list(trackList))
        trackList <- list(trackList)
    if(!all(sapply(trackList, is, "GdObject")))
        stop("All elements in 'trackList' must inherit from 'GdObject'")
    ## If no chromosome was explicitely asked for we just take the first one in the GRanges object
    if(missing(chromosome) || is.null(chromosome))
        chromosome <- if(length(range)>0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    ## And finally the object instantiation
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    return(new("HighlightTrack", trackList=trackList, chromosome=chromosome[1], range=range, name=name, genome=genome, ...))
}
setReplaceMethod("displayPars", signature("HighlightTrack", "list"), function(x, recursive=FALSE, value) {
    x <- setPar(x, value, interactive=FALSE)
    if(recursive){
        x@trackList <- lapply(x@trackList, function(y){
            displayPars(y) <- value
            return(y)
        })
    }
    return(x)
})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## OverlayTrack:
##
## A collection container for several track objects which will all be plotted on top of each other
##----------------------------------------------------------------------------------------------------------------------
setClass("OverlayTrack",
         representation=representation(trackList="list"),
         contains=c("GdObject"),
         prototype=prototype(dp=DisplayPars()))

setMethod("initialize", "OverlayTrack", function(.Object, trackList, ...) {
    .Object <- .updatePars(.Object, "OverlayTrack")
    .Object@trackList <- trackList
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

OverlayTrack <- function(trackList=list(), name="OverlayTrack",  ...){
    if(!is.list(trackList))
        trackList <- list(trackList)
    if(!all(sapply(trackList, is, "GdObject")))
        stop("All elements in 'trackList' must inherit from 'GdObject'")
    return(new("OverlayTrack", trackList=trackList, name=name, ...))
}
setReplaceMethod("displayPars", signature("OverlayTrack", "list"), function(x, recursive=FALSE, value) {
    x <- setPar(x, value, interactive=FALSE)
    if(recursive){
        x@trackList <- lapply(x@trackList, function(y){
            displayPars(y) <- value
            return(y)
        })
    }
    return(x)
})
##----------------------------------------------------------------------------------------------------------------------



##----------------------------------------------------------------------------------------------------------------------
## CustomTrack:
##
## A track class to allow for user-defined plotting functions
##----------------------------------------------------------------------------------------------------------------------
setClass("CustomTrack",
         contains=c("GdObject"),
         representation=representation(plottingFunction="function",
                                       variables="list"),
         prototype=prototype(dp=DisplayPars()))

setMethod("initialize", "CustomTrack", function(.Object, plottingFunction, variables, ...) {
    .Object <- .updatePars(.Object, "CustomTrack")
    .Object@plottingFunction <- plottingFunction
    .Object@variables <- variables
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})


CustomTrack <- function(plottingFunction=function(GdObject, prepare=FALSE, ...){}, variables=list(), name="CustomTrack",  ...){
    return(new("CustomTrack", plottingFunction=plottingFunction, variables=variables, name=name, ...))
}
##----------------------------------------------------------------------------------------------------------------------

## Classes in Gviz

test_that("ImageMap works", {
    mat <- matrix(1, ncol = 4, dimnames = list("a", NULL))
    tags <- list(a = c(a = "tag"))
    expect_s4_class(ImageMap(mat, tags = tags), "ImageMap")
    expect_identical(ImageMap(mat, tags = tags)@coords, mat)
    expect_identical(ImageMap(mat, tags = tags)@tags, tags)
    expect_error(ImageMap(matrix(1, ncol = 3), tags = tags), "must be a numeric matrix with 4 column")
    expect_error(ImageMap(matrix(1, ncol = 4), tags = tags), "Rownames must be set for the matrix in")
    expect_error(ImageMap(mat, tags = list(c(a = "tag1"))), "must be a named list with character vector items.")
    expect_error(ImageMap(mat, tags = list(a = c(a = "tag1", b = "tag2"))), "following values in the")
    expect_error(ImageMap(mat, tags = list(a = c("tag1"))), "items in the 'tags' list must be named character")
})

test_that("DisplayPars works", {
    dp <- DisplayPars(a = 1)

    expect_s4_class(dp, "DisplayPars")
    expect_identical(dp@pars$a, 1)
    expect_error(DisplayPars(1), "All supplied arguments must be named.")
    expect_error(DisplayPars(1, b = 2), "All supplied arguments must be named.")
    # .updateDp
    expect_message(.updateDp(dp), "Note that the behaviour of the")
    expect_identical(.updateDp(dp, interactive = FALSE)@pars, list(a = 1))
    dp@pars <- new.env()
    dp@pars[["a"]] <- 1
    expect_identical(.updateDp(dp, interactive = FALSE)@pars, list(a = 1))
    expect_message(.updateDp(dp, interactive = FALSE), "The DisplayPars object has been updated")
    ## getPar
    expect_identical(getPar(dp), list(a = 1))
    expect_identical(getPar(dp, "a"), 1)
    ## setPar
    expect_identical(setPar(dp, "b", 2, interactive = FALSE)@pars$b, 2)
    expect_error(setPar(dp, "b", c(1, 2), interactive = FALSE), "equal length")
    ## displayPars <-
    displayPars(dp) <- list(b = 2)
    expect_identical(dp@pars, list(a = 1, b = 2))
    ## getPar
    expect_identical(getPar(dp), list(a = 1, b = 2))
    expect_identical(getPar(dp, "a"), 1)
    ## displayPars
    expect_identical(displayPars(dp), list(a = 1, b = 2))
    expect_identical(displayPars(dp, "a"), 1)
    ## as.list
    expect_identical(as.list(dp), list(a = 1, b = 2))
})

test_that("IdeogramTrack works", {
    ideoTrack <- IdeogramTrack(chromosome = "chrI", genome = "sacCer3", bands = cyto.bands)
    expect_s4_class(ideoTrack, "GdObject")
    expect_s4_class(ideoTrack, "RangeTrack")
    expect_s4_class(ideoTrack, "IdeogramTrack")
})

test_that("GenomeAxisTrack works", {
    expect_s4_class(GenomeAxisTrack(), "GdObject")
    expect_s4_class(GenomeAxisTrack(), "GenomeAxisTrack")
})

test_that("DataTrack works", {
    expect_s4_class(DataTrack(), "GdObject")
    expect_s4_class(DataTrack(), "RangeTrack")
    expect_s4_class(DataTrack(), "NumericTrack")
    expect_s4_class(DataTrack(), "DataTrack")
})

test_that("AnnnotationTrack works", {
    expect_s4_class(AnnotationTrack(), "GdObject")
    expect_s4_class(AnnotationTrack(), "RangeTrack")
    expect_s4_class(AnnotationTrack(), "StackedTrack")
    expect_s4_class(AnnotationTrack(), "AnnotationTrack")

    expect_error(AnnotationTrack(stacking = "allover"), "following values for 'stacking'")
})

test_that("GeneRegionTrack works", {
    expect_s4_class(GeneRegionTrack(), "AnnotationTrack")
    expect_s4_class(GeneRegionTrack(), "GeneRegionTrack")
})

test_that("BiomartGeneRegionTrack works", {
    ## expect_s4_class(BiomartGeneRegionTrack(), "BiomartGeneRegionTrack")

    biomartMapping <- list(
        gene_id = "ensembl_gene_id", transcript_id = "ensembl_transcript_id", exon_id = "ensembl_exon_id",
        start = "exon_chrom_start", end = "exon_chrom_end", rank = "rank", strand = "strand",
        symbol = c("external_gene_name", "external_gene_id"), feature = "gene_biotype", chromosome = "chromosome_name",
        u5s = "5_utr_start", u5e = "5_utr_end", u3s = "3_utr_start", u3e = "3_utr_end", cdsl = c("cds_length", "cds_start"),
        phase = "phase"
    )
    expect_identical(.getBMFeatureMap(), biomartMapping)
})

test_that("SequenceTrack works", {
    expect_s4_class(SequenceTrack(), "GdObject")
    expect_s4_class(SequenceTrack(), "SequenceTrack")
    expect_s4_class(SequenceTrack(), "SequenceDNAStringSetTrack")
    expect_s4_class(RNASequenceTrack(), "SequenceRNAStringSetTrack")
})

test_that("AlignmentsTrack works", {
    expect_s4_class(AlignmentsTrack(), "GdObject")
    expect_s4_class(AlignmentsTrack(), "RangeTrack")
    expect_s4_class(AlignmentsTrack(), "StackedTrack")
    expect_s4_class(AlignmentsTrack(), "AlignmentsTrack")
    expect_s4_class(AlignmentsTrack(showIndels = TRUE), "AlignmentsTrack")

    expect_s4_class(AlignmentsTrack(bamfile), "ReferenceTrack")
    expect_s4_class(AlignmentsTrack(bamfile), "AlignmentsTrack")
})

test_that("CustomTrack works", {
    expect_s4_class(CustomTrack(), "GdObject")
    expect_s4_class(CustomTrack(), "CustomTrack")
})

test_that("HighlightTrack works", {
    expect_s4_class(HighlightTrack(), "GdObject")
    expect_s4_class(HighlightTrack(), "RangeTrack")
    expect_s4_class(HighlightTrack(), "HighlightTrack")

    expect_true(is(HighlightTrack()@range, "GRanges"))
    expect_error(
        HighlightTrack(c(annoTrack, geneModels)),
        "All elements in 'trackList' must inherit from 'GdObject'"
    )

    displayPars(highTrack) <- list(col = "black")
    expect_identical(displayPars(highTrack)$col, "black")

    displayPars(highTrack, recursive = TRUE) <- list(col = "red")
    expect_identical(vapply(highTrack@trackList, function(x) displayPars(x)$col,
        FUN.VALUE = character(1)
    ), "red")
})

test_that("OverlayTrack works", {
    expect_s4_class(OverlayTrack(), "GdObject")
    expect_s4_class(OverlayTrack(), "OverlayTrack")

    expect_true(is.list(OverlayTrack(annoTrack)@trackList))
    expect_error(
        OverlayTrack(c(annoTrack, geneModels)),
        "All elements in 'trackList' must inherit from 'GdObject'"
    )

    displayPars(overTrack) <- list(col = "black")
    expect_identical(displayPars(overTrack)$col, "black")

    displayPars(overTrack, recursive = TRUE) <- list(col = "red")
    expect_identical(vapply(overTrack@trackList, function(x) displayPars(x)$col,
        FUN.VALUE = character(1)
    ), c("red", "red"))
})

test_that("DetailsAnnotationTrack works", {
    expect_s4_class(DetailsAnnotationTrack(), "GdObject")
    expect_s4_class(DetailsAnnotationTrack(), "StackedTrack")
    expect_s4_class(DetailsAnnotationTrack(), "RangeTrack")
    expect_s4_class(DetailsAnnotationTrack(), "AnnotationTrack")

    expect_s4_class(detTrack, "AnnotationTrack")
    expect_identical(detTrack@selectFun, selFun)
    expect_identical(detTrack@fun, detFun)
})

test_that("md5 hash for BiomartTrack works", {
    data("biomTrack")
    expect_identical(.bmGuid(biomTrack), "b0cbf1bb0e4e5a1936fe68e36da5f210")
})

test_that("interaction with biomart works", {
    check_biomart()
    biomartTrack <- BiomartGeneRegionTrack(
        genome = "hg38", chromosome = "chr7",
        start = 20e6, end = 21e6, name = "ENSEMBL"
    )
    expect_s4_class(biomartTrack, "GdObject")
    expect_s4_class(biomartTrack, "StackedTrack")
    expect_s4_class(biomartTrack, "RangeTrack")
    expect_s4_class(biomartTrack, "AnnotationTrack")
    expect_s4_class(biomartTrack, "GeneRegionTrack")
    expect_s4_class(biomartTrack, "BiomartGeneRegionTrack")
})

## Gviz methods

test_that("General accessors work", {
    expect_identical(ranges(dataTrack), granges(gr))
    expect_identical(range(dataTrack), ir)

    expect_identical(ranges(axisTrack), gr)
    expect_identical(range(axisTrack), ir)

    expect_identical(seqnames(annoTrack), "chr1")
    expect_identical(seqnames(dataTrack), "chr1")
    expect_identical(seqnames(seqTrack.dna), "chr1")
    expect_identical(seqnames(seqTrack.rna), "chr1")
    expect_identical(seqnames(seqTrack.bs), seqnames(BSgenome.Hsapiens.UCSC.hg19))

    expect_identical(seqlevels(annoTrack), "chr1")
    expect_identical(seqlevels(dataTrack), "chr1")
    expect_identical(seqlevels(seqTrack.dna), "chr1")
    expect_identical(seqlevels(seqTrack.rna), "chr1")
    expect_identical(seqlevels(seqTrack.bs), seqlevels(BSgenome.Hsapiens.UCSC.hg19))

    expect_identical(seqinfo(annoTrack), table("chr1"))

    expect_identical(min(dataTrack), 1L)
    expect_identical(max(dataTrack), 5L)

    expect_identical(start(dataTrack), 1L)
    expect_identical(end(dataTrack), 5L)
    expect_identical(width(dataTrack), 5L)
    expect_identical(length(dataTrack), 1L)

    expect_identical(start(ideoTrack), NULL)
    expect_identical(end(ideoTrack), NULL)
    expect_identical(width(ideoTrack), NULL)
    expect_identical(length(ideoTrack), 4L)

    expect_identical(start(axisTrack), 1L)
    expect_identical(end(axisTrack), 5L)
    expect_identical(width(axisTrack), 5L)

    expect_identical(start(seqTrack.dna), NULL)
    expect_identical(end(seqTrack.dna), NULL)
    expect_identical(width(seqTrack.dna), NULL)
    expect_identical(length(seqTrack.dna), 100L) # !

    expect_identical(length(highTrack), 1L)
    expect_identical(length(overTrack), 2L)
})

test_that("General replacement methods work", {
    ranges(dataTrack) <- gr2
    expect_identical(ranges(dataTrack), gr2)
    expect_identical(range(dataTrack), ir2)

    ranges(axisTrack) <- gr2
    expect_identical(ranges(axisTrack), gr2)
    expect_identical(range(axisTrack), ir2)

    ranges(annoTrack) <- gr2
    expect_identical(ranges(annoTrack), gr2)
    expect_identical(range(annoTrack), ir2)

    start(annoTrack) <- 3L
    expect_identical(start(annoTrack), 3L)
    start(axisTrack) <- 3L
    expect_identical(start(axisTrack), 3L)
    start(ideoTrack) <- 3L
    expect_identical(start(ideoTrack), NULL)

    end(annoTrack) <- 4L
    expect_identical(end(annoTrack), 4L)
    end(axisTrack) <- 4L
    expect_identical(end(axisTrack), 4L)
    end(ideoTrack) <- 4L
    expect_identical(end(ideoTrack), NULL)

    width(annoTrack) <- 10L
    expect_identical(width(annoTrack), 10L)

    width(ideoTrack) <- 10L
    expect_identical(ideoTrack, ideoTrack)
})

test_that("values accessors and replacement methods work", {
    expect_identical(values(annoTrack), data.frame(
        feature = "unknown", group = "1",
        id = "unknown", density = 1, stringsAsFactors = FALSE
    ))
    expect_identical(values(axisTrack), data.frame(score = 1, stringsAsFactors = FALSE))
    expect_identical(values(dataTrack), as.matrix(c(score = 1)))
    expect_identical(values(DataTrack()), matrix(logical(), 0, 0))
    expect_identical(values(alnTrack), NULL)

    values(dataTrack) <- matrix(3, dimnames = list("score", NULL))
    expect_identical(values(dataTrack), as.matrix(c(score = 3)))
    values(dataTrack) <- 3
    expect_identical(values(dataTrack), as.matrix(c(3)))
    expect_error(values(dataTrack) <- matrix("3", dimnames = list("score", NULL)), "Not numeric or dimensions of replacement value do not match.")
    expect_error(values(dataTrack) <- c(3, 3), "Not numeric or invalid length of replacement vector.")
})

test_that("subseq works", {
    expect_error(subseq(SequenceTrack(), start = NA, end = NA, width = 3), "Two out of the three in")
    expect_error(subseq(SequenceTrack(), start = 10, end = 1), "'end' has to be bigger than 'start'")
    expect_error(subseq(seqTrack.dna, start = 10, end = 1), "'end' has to be bigger than 'start'")
    expect_error(subseq(SequenceTrack(), start = 1, end = 10e6 + 2), "Sequence is too big")
    expect_error(subseq(seqTrack.bs, start = 1, end = 10e6 + 2), "Sequence is too big")
    expect_warning(subseq(seqTrack.dna, start = 1, end = 10, width = 10), "are provided, ignoring")
    expect_identical(as.character(subseq(SequenceTrack(), start = 1, end = 10)), as.character(DNAString("----------")))
    expect_identical(as.character(subseq(seqTrack.dna, start = 1, end = 10)), as.character(DNAString("ATTTCCCTGA")))
    expect_identical(as.character(subseq(seqTrack.dna, start = 1, width = 10)), as.character(DNAString("ATTTCCCTGA")))
    expect_identical(as.character(subseq(seqTrack.dna, end = 10, width = 10)), as.character(DNAString("ATTTCCCTGA")))
    expect_identical(as.character(subseq(seqTrack.dna, start = 91, end = 110)), as.character(DNAString("ACGTCTTCCA----------")))
    expect_identical(as.character(subseq(seqTrack.bs, start = 1, width = 10)), as.character(DNAString("NNNNNNNNNN")))
    expect_identical(as.character(subseq(seqTrack.dna, start = 1)), as.character(dna.sq[[1]]))
    expect_identical(as.character(subseq(SequenceTrack(), start = 1)), as.character(DNAString("-")))
    expect_identical(as.character(subseq(seqTrack.dna, end = 1)), as.character(DNAString("A")))
    expect_identical(as.character(subseq(SequenceTrack(), end = 1)), as.character(DNAString("-")))
    displayPars(seqTrack.dna)$complement <- TRUE
    expect_identical(as.character(subseq(seqTrack.dna, start = 1, end = 10)), as.character(DNAString("TAAAGGGACT")))


    expect_error(subseq(SequenceTrack(fastafile, chromosome = "chr1")), "Two out of the three in")
    expect_error(subseq(SequenceTrack(fastafile, chromosome = "chr1"), start = 1), "Two out of the three in")
    expect_warning(as.character(subseq(SequenceTrack(fastafile, chromosome = "chr1"), start = 1, end = 10, width = 10)), "All ")
    expect_error(as.character(subseq(SequenceTrack(fastafile, chromosome = "chr1"), start = NA, end = NA, width = 10)), "Two ")
    expect_identical(as.character(subseq(SequenceTrack(fastafile, chromosome = "chr1"), start = 1, width = 10)), as.character(DNAString("CTANGAGACG")))
    expect_identical(as.character(subseq(SequenceTrack(fastafile, chromosome = "chr1"), end = 10, width = 10)), as.character(DNAString("CTANGAGACG")))
    ## expect_identical(as.character(subseq(SequenceTrack(fastafile, chromosome="chr1"), start=1, end=NA, width=10)), as.character(DNAString("CTANGAGACG")))
    expect_identical(as.character(subseq(SequenceTrack(fastafile, chromosome = "chr1"), start = 1, end = 10)), as.character(DNAString("CTANGAGACG")))
})

test_that("chromosome accessors and replacement methods work", {
    expect_identical(chromosome(annoTrack), "chr1")
    chromosome(annoTrack) <- "chr2"
    expect_identical(chromosome(annoTrack), "chr2")

    expect_identical(chromosome(dataTrack), "chr1")

    expect_identical(chromosome(seqTrack.dna), "chr1")
    chromosome(seqTrack.dna) <- "chr2"
    expect_identical(chromosome(seqTrack.dna), "chr2")

    expect_identical(chromosome(ideoTrack), "chrI")
    chromosome(ideoTrack) <- "chrII"
    expect_identical(chromosome(ideoTrack), "chrII")

    expect_identical(chromosome(highTrack), "chr1")
    chromosome(highTrack) <- "chr2"
    expect_identical(chromosome(highTrack), "chr2")

    expect_identical(chromosome(overTrack), "chr1")
    chromosome(overTrack) <- "chr2"
    expect_identical(chromosome(overTrack), "chr2")

    expect_identical(chromosome(axisTrack), NULL)
    chromosome(axisTrack) <- "chr1"
    expect_identical(chromosome(axisTrack), NULL)

    expect_identical(chromosome(alnTrack), "chrNA")
    chromosome(alnTrack) <- "chr1"
    expect_identical(chromosome(alnTrack), "chr1")
})


test_that("genome accessors and replacement methods work", {
    expect_identical(is.na(genome(annoTrack)), TRUE)
    genome(annoTrack) <- "hg38"
    expect_identical(genome(annoTrack), "hg38")

    expect_identical(is.na(genome(seqTrack.dna)), TRUE)
    ## genome(seqTrack.dna) <- "hg38"
    ## expect_identical(genome(seqTrack.dna), "hg38")

    expect_identical(genome(ideoTrack), "sacCer3")
    ## genome(ideoTrack) <- "sacCer2"
    ## expect_identical(genome(ideoTrack), "sacCer2")

    expect_identical(genome(overTrack), NULL)
    genome(overTrack) <- "hg38"
    expect_identical(genome(overTrack), NULL)
})

test_that("strand accessors and replacement methods work", {
    expect_identical(strand(dataTrack), "*")
    strand(dataTrack) <- "+"
    expect_identical(strand(dataTrack), "+")
    expect_error(strand(dataTrack) <- rep("+", 2), "Invalid")

    expect_identical(strand(annoTrack), "*")
    strand(annoTrack) <- "+"
    expect_identical(strand(annoTrack), "+")

    expect_identical(strand(axisTrack), "*")

    annoTrack <- AnnotationTrack(c(gr, gr2))
    expect_identical(strand(annoTrack), rep("*", 2))
    strand(annoTrack) <- rep("+", 2)
    expect_identical(strand(annoTrack), rep("+", 2))
    expect_error(strand(annoTrack) <- rep("+", 3), "Invalid replacement value or length of replacement")
})

test_that("subsetting with [ works", {
    expect_identical(annoTrack[1]@range, GRanges("chr1", IRanges(1, 5), feature = "unknown", group = "1", id = "unknown", density = 1))
    expect_error(annoTrack[2], "subscript contains out-of-bounds")

    expect_identical(axisTrack[1]@range, GRanges("chr1", IRanges(1, 5), score = 1))
    expect_error(dataTrack[2], "subscript out of bounds")

    expect_identical(ideoTrack[1], ideoTrack)

    expect_identical(dataTrack[1, , ]@range, GRanges("chr1", IRanges(1, 5)))
    expect_error(dataTrack[2], "subscript out of bounds")
    expect_identical(dataTrack[, 1, ]@data, matrix(1, nrow = 1, dimnames = list("score", NULL)))
    expect_error(dataTrack[, 2, ]@data, "subscript contains out-of-bounds indices")
})

test_that("splitting works", {
    aTrack <- AnnotationTrack(c(gr, gr2))
    expect_identical(names(split(aTrack, f = c("a", "b"))), c("a", "b"))
    expect_identical(split(aTrack, f = c("a", "b"))[[1]]@range, aTrack@range[1])
    expect_identical(split(aTrack, f = c("a", "b"))[[2]]@range, aTrack@range[2])

    dTrack <- DataTrack(c(gr, gr2))
    expect_identical(names(split(dTrack, f = c("a", "b"))), c("a", "b"))
    expect_identical(split(dTrack, f = c("a", "b"))[[1]]@range, dTrack@range[1])
    expect_identical(split(dTrack, f = c("a", "b"))[[1]]@data, dTrack@data[, 1, drop = FALSE])
    expect_identical(split(dTrack, f = c("a", "b"))[[2]]@range, dTrack@range[2])
    expect_identical(split(dTrack, f = c("a", "b"))[[2]]@data, dTrack@data[, 2, drop = FALSE])
})

test_that("names accessors and replacement methods work", {
    expect_identical(names(annoTrack), "AnnotationTrack")
    names(annoTrack) <- "AnnoTrack"
    expect_identical(names(annoTrack), "AnnoTrack")
})

test_that("gene, symbol, transcript, feature and exon accessors and replacement methods work", {
    expect_identical(gene(geneTrack), as.character(geneModels$gene))
    gene(geneTrack) <- paste0(as.character(geneModels$gene), ".1")
    expect_identical(gene(geneTrack), paste0(as.character(geneModels$gene), ".1"))

    expect_identical(symbol(geneTrack), as.character(geneModels$symbol))
    symbol(geneTrack) <- paste0(as.character(geneModels$symbol), ".1")
    expect_identical(symbol(geneTrack), paste0(as.character(geneModels$symbol), ".1"))

    expect_identical(transcript(geneTrack), as.character(geneModels$transcript))
    transcript(geneTrack) <- paste0(as.character(geneModels$transcript), ".1")
    expect_identical(transcript(geneTrack), paste0(as.character(geneModels$transcript), ".1"))

    expect_identical(exon(geneTrack), as.character(geneModels$exon))
    exon(geneTrack) <- paste0(as.character(geneModels$exon), ".1")
    expect_identical(exon(geneTrack), paste0(as.character(geneModels$exon), ".1"))

    expect_identical(feature(geneTrack), as.character(geneModels$feature))
    feature(geneTrack) <- paste0(as.character(geneModels$feature), ".1")
    expect_identical(feature(geneTrack), paste0(as.character(geneModels$feature), ".1"))

    expect_identical(feature(dataTrack), NULL)
    feature(dataTrack) <- "1"
    expect_identical(dataTrack, dataTrack)
})

test_that("group and identifier accessors and replacement methods work", {
    expect_identical(group(geneTrack), as.character(geneModels$transcript))
    group(geneTrack) <- paste0(as.character(geneModels$transcript), ".1")
    expect_identical(group(geneTrack), paste0(as.character(geneModels$transcript), ".1"))

    expect_identical(group(dataTrack), NULL)

    expect_identical(identifier(geneTrack), as.character(geneModels$symbol))
    expect_identical(identifier(geneTrack, type = NULL), as.character(geneModels$symbol))
    identifier(geneTrack) <- paste0(as.character(geneModels$symbol), ".1")
    expect_identical(identifier(geneTrack), paste0(as.character(geneModels$symbol), ".1"))

    expect_identical(identifier(annoTrack), "1")
    expect_identical(identifier(annoTrack, type = NULL), "1")
    identifier(annoTrack) <- "1.1"
    expect_identical(identifier(annoTrack), "1.1")
})

test_that("stacking works", {
    expect_identical(stacking(annoTrack), "squish")
    stacking(annoTrack) <- "dense"
    expect_identical(stacking(annoTrack), "dense")

    expect_identical(stacks(annoTrack), NULL)
    expect_identical(stacks(alnTrack), 0)

    expect_identical(Gviz:::setStacks(AnnotationTrack(c(gr)))@stacks, c(unknown = 1L))
})


test_that("position calculation works", {
    expect_identical(position(annoTrack), 3)
    expect_identical(position(annoTrack, from = 1, to = 5), 3)
    expect_identical(position(annoTrack, from = 10, to = 15), numeric())
})


test_that("imageMap and coords work", {
    expect_identical(imageMap(annoTrack), NULL)
    im <- new("ImageMap",
        coords = matrix(1, ncol = 4, nrow = 1, dimnames = list("first")),
        tags = list(first = 1)
    )
    imageMap(annoTrack) <- im
    expect_identical(imageMap(annoTrack), im)

    expect_identical(coords(NULL), NULL)
    expect_identical(coords(im), matrix(1, ncol = 4, nrow = 1, dimnames = list("first")))
    expect_identical(coords(annoTrack), matrix(1, ncol = 4, nrow = 1, dimnames = list("first")))

    expect_identical(tags(NULL), NULL)
    expect_identical(tags(im), list(first = 1))
    expect_identical(tags(annoTrack), list(first = 1))
})

test_that("consolidating of tracks works", {
    expect_identical(chromosome(consolidateTrack(annoTrack, chromosome = "chr2", alpha = TRUE)), "chr2")
    expect_identical(getPar(consolidateTrack(annoTrack, chromosome = "chr2", alpha = TRUE), ".__hasAlphaSupport"), TRUE)
    expect_identical(getPar(consolidateTrack(annoTrack, chromosome = "chr2", alpha = FALSE, size = 2), "size"), 2)
})



test_that("collapsing of tracks works", {
    expect_identical(.myFindOverlaps(gr, gr2), c(queryHits = 1L))
    expect_identical(Gviz:::.myFindOverlaps(gr, GRanges("chr2", IRanges(1, 5))), integer())
})

#' @include RangeTrack-class.R
NULL

## StackedTrack --------------------------------------------------------------

#' StackedTrack class and methods
#'
#'
#' The virtual parent class for all track types in the Gviz package which
#' contain potentially overlapping annotation items that have to be stacked
#' when plotted.
#'
#' @template GdObject-class_slot
#' @template RangeTrack-class_slot
#' @template StackedTrack_slot
#' @template StackedTrack_param
#'
#' @name StackedTrack-class
#'
#' @return A virtual Class: No objects may be created from it.

#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## This is a reference class therefore we show below
#' ## an example from AnnotationTrack
#'
#' ## An empty object
#' AnnotationTrack()
#'
#' ## Construct from individual arguments
#' st <- c(2000000, 2070000, 2100000, 2160000)
#' ed <- c(2050000, 2130000, 2150000, 2170000)
#' str <- c("-", "+", "-", "-")
#' gr <- c("Group1", "Group2", "Group1", "Group3")
#'
#' annTrack <- AnnotationTrack(
#'     start = st, end = ed, strand = str, chromosome = 7,
#'     genome = "hg19", feature = "test", group = gr,
#'     id = paste("annTrack item", 1:4),
#'     name = "generic annotation", stacking = "squish"
#' )
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(annTrack)
#'
#' ## Stacking
#' stacking(annTrack)
#' stacking(annTrack) <- "dense"
#' plotTracks(annTrack)
#' @exportClass StackedTrack
setClass("StackedTrack",
    representation = representation("VIRTUAL",
        stacking = "character",
        stacks = "numeric"
    ),
    prototype = prototype(
        name = "StackedTrack",
        stacking = "squish",
        stackingValues = c("hide", "dense", "squish", "pack", "full"),
        dp = DisplayPars(
            stackHeight = 0.75,
            reverseStacking = FALSE
        )
    ),
    contains = "RangeTrack"
)

## Initialize ----------------------------------------------------------------

## Need to fill the stacks slot here, don't want to recompute all the time
#' @describeIn StackedTrack-class Initialize.
#' @export
setMethod("initialize", "StackedTrack", function(.Object, stacking, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "StackedTrack")
    pt <- getClass("StackedTrack")@prototype
    if (!missing(stacking)) {
        if (!all(stacking %in% pt@stackingValues)) {
            stop(
                sprintf("Problem initializing %s, accepts the following values for 'stacking': ", class(.Object)),
                paste(pt@stackingValues, collapse = ", "), "\n"
            )
        }
        .Object@stacking <- stacking
        r <- list(...)$range
        ## stacks <- if(length(r)>0) disjointBins(ranges(r)) else 0
        ## .Object@stacks <- stacks
        .Object@stacks <- numeric()
    }
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## General accessors ---------------------------------------------------------
## Annotation Accessors ------------------------------------------------------
## Stacking ------------------------------------------------------------------

## Stacking controls what to do with overlapping annotation regions.

#' @describeIn StackedTrack-class  return the current stacking type.
#' @export
setMethod("stacking", "StackedTrack", function(GdObject) GdObject@stacking)


#' @describeIn StackedTrack-class  set the object's stacking type to one in
#' `c(hide, dense, squish, pack,full)`.
#' @export
setReplaceMethod(
    "stacking", c("StackedTrack", "character"),
    function(GdObject, value) {
        pt <- getClass("StackedTrack")@prototype
        if (!all(value %in% pt@stackingValues)) {
            stop(
                "Problem initializing StackedTrack,  need the following values for 'stacking':",
                paste(pt@stackingValues, collapse = ", "), "\n"
            )
        }
        GdObject@stacking <- value
        displayPars(GdObject) <- list(stacking = value)
        return(GdObject)
    }
)

## Recompute or return the stacking information for different types of StackedTrack objects. Stacking in needed when
## annotation regions in the objects overlap and when the stacking type is set to squish, full or pack. Since stacking
## can be dependent on the available space (if feature annotation is added) we need to be able to recompute this before
## we start the actual plotting. For the different sub-classes of StackedTracks we need different behaviour of setStacks:
##   o StackedTrack: there are no groups, so each feature can be treated separately
##   o AnnotationTrack and GeneRegionTrack: features can be grouped, in which case we have to avoid overlapping of the whole group region,
##      i.e, from the start of the first group item to the end of the last. In addition to grouping we have to factor in additional space
##      before each group if group/transcript annotation is enabled (gpar showId==TRUE). To do so we need to figure out the current
##      fontsize on the device, which means that a device already has to be open and the appropriate viewport has been
##      pushed to the stack. Hence we have to call setStacks immediately before the actual plotting.
## 'stacks' should return a vector of stacks, where each factor level of the vector indicates membeship
## to a particular stacking level. 'setStacks' returns the updated GdObject.

#' @describeIn StackedTrack-class return the stack indices for each track item.
#' @export
setMethod("stacks", "StackedTrack",
    function(GdObject) if (length(GdObject@stacks)) GdObject@stacks else NULL
)


#' @describeIn StackedTrack-class  recompute the stacks based on the available
#' space and on the object's track items and stacking settings.
#' @export
setMethod("setStacks", "StackedTrack", function(GdObject, ...) {
    bins <- if (!.needsStacking(GdObject)) rep(1, length(GdObject)) else disjointBins(range(GdObject))
    GdObject@stacks <- bins
    return(GdObject)
})

## Consolidate ---------------------------------------------------------------

#' @describeIn StackedTrack-class Consolidate.
# 'For `StackedTrack`s set the stacking (which could have been passed in as
#' a display parameter)
# #' @keywords internal
#' @export
setMethod("consolidateTrack", signature(GdObject = "StackedTrack"), function(GdObject, ...) {
    GdObject <- callNextMethod()
    st <- .dpOrDefault(GdObject, "stacking")
    if (!is.null(st)) {
        stacking(GdObject) <- st
    }
    return(GdObject)
})

## Collapse  -----------------------------------------------------------------
## There is a natural limit of what can be plotted as individual features caused by the maximum resolution of the device.
## Essentially no object can be smaller than the equivalent of a single pixel on the screen or whatever a pixel corresponds
## to on other devices.
## Thus we need to adapt the all objects in the track to the current resolution. Things that are closer than a certain limit
## distance can be collapsed (if that is desired), things that are smaller than 1 pixel can be blown up to a minimum size.
## All collapseTrack methods should always return a GdObject instance of similar type as the input to allow us to keep the
## collpasing step optional and for all the downstream plotting operations to still work. The internal (mostly the GRanges)
## objects can be modified to whatever is desired. Please note that this method is called after the drawing viewport has been
## pushed, and hence all the coordinate systems are already in place.
## Available arguments are:
##    o GdObject: the input AnnotationTrack or GeneRegionTrack track object
##    o min.width: the minimum width in pixel, everything that's smaller will be expanded to this size.
##    o min.distance: the minimum distance between two features in pixels below which to start collapsing track items.
##    o collapse: logical, collapse overlapping items into a single meta-item.
##    o diff: the equivalent of 1 pixel in the native coordinate system.
##    o xrange: the data range on the x axis. Can be used for some preliminary subsetting to speed things up
## Subset --------------------------------------------------------------------

#' @describeIn StackedTrack-class subset the items in the `StackedTrack` object.
#' This is essentially similar to subsetting of the `GRanges` object in the
#' range slot. For most applications, the subset method may be more appropriate.
#' @export
setMethod("[", signature(x = "StackedTrack"), function(x, i, j, ..., drop = TRUE) {
    x <- callNextMethod(x, i)
    x@stacks <- x@stacks[i]
    return(x)
})

#' @describeIn StackedTrack-class subset a `StackedTrack` by coordinates and sort if necessary.
#' @export
setMethod("subset", signature(x = "StackedTrack"), function(x, from = NULL, to = NULL, sort = FALSE, stacks = FALSE, ...) {
    x <- callNextMethod(x = x, from = from, to = to, sort = sort)
    if (stacks) {
        x <- setStacks(x)
    }
    return(x)
})


## Position ------------------------------------------------------------------
## In some cases we don't need a range but rather a single position. Most of the time this is simply taking the geometric
## mean of the range. If numeric values are associated to positions we have to be able to extract those as well.

## DrawGrid ------------------------------------------------------------------
## Draw a grid in the background of a GdObject. For some subclasses this is meaningless, and the default function will
## return NULL without plotting anything.

## DrawAxis ------------------------------------------------------------------
## The individual bits and pieces of a Gviz plot are all drawn by separate renderers. Currently, those are a y-axis,
## a grid, and the actual track panel.
##
## For certain GdObject subclasses we may want to draw a y-axis. For others an axis is meaningless, and the default function
## will return NULL without plotting anything.

## DrawGD --------------------------------------------------------------------

## All the drawGD methods should support two modes, triggered by the boolean argument 'prepare':
##    In prepare mode: nothing is plotted but the object is prepared for plotting bases on the available space. The return
##       value of the method in this mode should always be the updated object. If nothing needs to be prepared, i.e., if the
##       plotting is independent from the available space, simply return the original object
##    In plotting mode: the object is plotted. Return value is the object with optional HTML image map information
##    added to the imageMap slot
## Since subsetting can be potentially expensive when the data are large we want to minimize this operation. Essentially it
## should be done only once before any other plotting or computation starts, hence we expect the GdObject in the drawGD
## methods to already be trimmed to the correct size

## The default method for all `StackedTrack` types which always should be called
## (this has to be done explicitly using `callNextMethod`)

## Although the stacking type is not stored as a displayParameter we still want to check whether it is
## included there and set the actual stacking of the object accordingly

#' @describeIn StackedTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which
#' the method can be called. Either in 'prepare' mode, in which case no
#' plotting is done but the stacking information is updated based on the
#' available space, or in 'plotting' mode, in which case the actual graphical
#' output is created. Note that the method for this particular subclass is
#' usually called through inheritance and not particularly useful on its own.
#' @export
setMethod("drawGD", signature("StackedTrack"), function(GdObject, ...) {
    debug <- .dpOrDefault(GdObject, "debug", FALSE)
    if ((is.logical(debug) && debug)) {
        browser()
    }
    st <- .dpOrDefault(GdObject, "stacking")
    if (!is.null(st)) {
        stacking(GdObject) <- st
    }
    return(invisible(GdObject))
})

## SetAs ---------------------------------------------------------------------
## Show ----------------------------------------------------------------------

#' @include utils.R
NULL

### ImageMap-class -----------------------------------------------------------

#' ImageMap: HTML image map information
#'
#'
#' HTML image map information for annotation tracks.
#'
#' Objects of the `ImageMap-class` are usually not created by the user,
#' hence the constructor function `ImageMap` is not exported in the name space.
#'
#' @template ImageMap-class_param
#'
#' @template ImageMap-class_slot
#'
#' @name ImageMap-class
#'
#' @examples
#' ## Not provided. This is an internal structure.
#'
#' @export
setClass("ImageMap", representation(coords = "matrix", tags = "list"),
    prototype = prototype(coords = matrix(1, ncol = 4, nrow = 0), tags = list())
)

## Allow for NULL value slots
setClassUnion("ImageMapOrNULL", c("ImageMap", "NULL"))


### ImageMap Constructor -----------------------------------------------------

#' @noRd
#' @keywords internal
ImageMap <- function(coords, tags) {
    if (!(is.matrix(coords) && is.numeric(coords) && ncol(coords) == 4)) {
        stop("'coords' must be a numeric matrix with 4 columns")
    }
    rn <- rownames(coords)
    if (is.null(rn)) {
        stop("Rownames must be set for the matrix in 'coords'")
    }
    if (!is.list(tags) || is.null(names(tags)) || any(names(tags) == "") ||
        !all(vapply(tags, is.character, logical(1)))) {
        stop("'tags' must be a named list with character vector items.")
    }
    n <- unique(unlist(lapply(tags, names)))
    if (is.null(n) || any(n == "")) {
        stop("All items in the 'tags' list must be named character vectors.")
    }
    m <- n %in% rn
    if (!all(m)) {
        stop(
            "The following values in the 'tags' list ",
            "could not be mapped to the 'coords' matrix:\n",
            paste(n[!m], sep = "", collapse = ", ")
        )
    }
    new("ImageMap", coords = coords, tags = tags)
}

####  ImageMap Methods coords  and tags --------------------------------------

#' @describeIn ImageMap-class Generics for `coords`.
#' @exportMethod coords
#' @keywords internal
setGeneric("coords", function(ImageMap, ...) standardGeneric("coords"))

#' @describeIn ImageMap-class Returns the coordinates from the image map.
#' @export
setMethod("coords", "NULL", function(ImageMap) NULL)

#' @describeIn ImageMap-class Returns the coordinates from the image map.
#' @return Returns the coordinates from the image map.
#' @export
setMethod("coords", "ImageMap", function(ImageMap) ImageMap@coords)


#' @describeIn ImageMap-class Generics for `tags`.
#' @exportMethod tags
#' @keywords internal
setGeneric("tags", function(ImageMap, ...) standardGeneric("tags"))

#' @describeIn ImageMap-class Returns the tags from the image map
#' @export
setMethod("tags", "NULL", function(ImageMap) NULL)

#' @describeIn ImageMap-class Returns the tags from the image map
#' @return Returns the tags from the image map.
#' @export
setMethod("tags", "ImageMap", function(ImageMap) ImageMap@tags)

##load("~muellar2/projects/Epigenetics/Methylation/HD2.1-MM_13-Week_Liver/exMA.Rdata")
##load("~muellar2/projects/Epigenetics/Methylation/HD2.1-MM_13-Week_Liver/probeAnno.Rdata")





panel.genomeGraphs <- function (x, y, fullData, subscripts, coord.factor, tracks, sizes, ...) {
	xy <- split(y,x)
	if(length(unique(listLen(xy))) != 1)
		stop("Supplied data is not a rectangular array.")
	data <- t(sapply(xy, function(x) x))
	dataTrack <- makeMatrixRangeTrack(start=sort(unique(fullData[subscripts, "ProbeStart"])), 
									  end=sort(unique(fullData[subscripts, "ProbeEnd"])), data=data)
	args <- list(...)
	setPar(dataTrack, args)
	cp <- current.panel.limits()
	setPar(dataTrack, "ylim", cp$ylim)
	##browser()
	tracks <- c(list(dataTrack), tracks)
	lt <- length(tracks)
	if(length(sizes) != lt)
		sizes <- rep(sizes, lt)[seq_len(lt)]
	plotTracks(tracks, panel.only=TRUE, from=cp$xlim[1]*coord.factor,  to=cp$xlim[2]*coord.factor, 
			   coord.factor=coord.factor, sizes=sizes)	
}





### objects exProbeAnno and exMA are required

### plots a trellis graphics for the 13 week study for a given gene
### using all time points and control and treated
###
### chr: the chromosome name (e.g. "7", "X")
### start: where to start (e.g. nucleotide 26682683)
### exProbeAnno: a probeAnno object from the Ringo package
### maList: the MAList object with the normalized data
### upstream: how much to extend upstream from the start (in nucleotides), must be a negative integer
### downstream: how much to extend downstream of the start (in nucleotides), must be a positive integer
plotAll = function(exProbeAnno, maList, gene=NULL, start=NULL, chr=NULL, upstream=10000, downstream=500, title=NULL,
		ylim, type=c("mountain", "g"), smooth.span=1/12, coord.factor=1000, tracks=list(), track.sizes=1, ...) {
	
	require(IRanges)
	require(lattice)
	require(org.Mm.eg.db) # for chromosomal location
	if ( is.null(start) ) {
		if ( is.null(gene) ) {
			stop("must either specify gene symbol (gene argument) or start (start argument)\n")
		}
		entrezId = suppressWarnings(as.numeric(gene))
		if ( is.na(entrezId) ) {
			entrezId = sym2entrezId(gene)
			if ( is.null(entrezId) ) {
				stop(paste("no entrezId found for gene", gene, "\n"))
			} else if ( length(entrezId) > 1 ) {
				stop("ambiguous mappings for gene ", gene, ": ", paste(entrezId, collapse=", "))
			}
			loc = mget(entrezId, org.Mm.egCHRLOC)
			if ( max(loc[[1]]) < 0 ) {
				loc = mget(entrezId, org.Mm.egCHRLOCEND)
			}
			chr = unique(names(loc[[entrezId]]))
			if ( length(chr) != 1 ) {
				stop(chr, " - cannot handle with chromosome!")
			}
			start = loc[[entrezId]]
			if ( length(start) > 1 ) { # several transcripts
				cat(paste("found several transcripts: ", paste(start, collapse=", "), "\n"))
				if ( min(start) < 0 && max(start) > 0 ) {
					stop(start, " - cannot handle two transcripts on opposite strands")
				}               
				if ( start[1] < 0 ) { # negative strand
					start = max(start)
				} else {
					start = min(start)
				}
			}           
			if ( is.na(start) ) {
				stop("no chromosomal location found for entrezId ", entrezId)
			}                     
			cat(paste("found", gene, "with entrezId", entrezId, "on chr", chr, "at position", start, ": up =", upstream,
							"to", downstream, "\n"))
		}
	} else {
		if ( is.null(chr) ) {
			stop("chromosome cannot be null")
		}
		gene = paste("position", as.character(start))
	}
	
	if ( start < 0 ) {
		start = abs(start)
		tmp = downstream
		downstream = upstream 
		upstream = tmp
	}
	
	cat("finding overlapping probes: ")
	gene.ir = IRanges(start=start-upstream, end=start+downstream)
	chr.ir = IRanges(start=exProbeAnno[paste(chr, "start", sep=".")], end=exProbeAnno[paste(chr, "end", sep=".")])
	gene.probes = findOverlaps(gene.ir, chr.ir)
	hits = subjectHits(gene.probes)
	hit.names = exProbeAnno[paste(chr, "index", sep=".")][hits] # names/indexes are not necessarily unique (why?)!
	probe.idx = (maList$genes$PROBE_ID %in% hit.names)
	m = maList[probe.idx,]
	probe.pos = start(chr.ir[hits]) + (end(chr.ir[hits]) - start(chr.ir[hits]))/2
	probe.start = start(chr.ir[hits])
	probe.end = end(chr.ir[hits])
	
	### create a data frame for lattice
	cat("creating data frame ...\n")
	d <- data.frame(Mvalue=as.numeric(m$M), ProbeStart=probe.start, ProbeEnd=probe.end,
					 Probe=m$genes$PROBE_ID, Slide=factor(rep(m$targets$SlideNumber, each=nrow(m))),
					 Treatment=factor(rep(m$targets[,"Treatment"], each=nrow(m))),
					 Day=ordered(rep(m$targets[,"Day"], each=nrow(m))), stringsAsFactors=FALSE)
	d$ProbeCenter <- rowMeans(cbind(d$ProbeStart, d$ProbeEnd))
	d$Group <- factor(paste(d$Treatment, d$Day, d$Probe, sep=":"))
	
	ir = range(chr.ir[hits])
	from = min(ir)
	to = max(ir)
	len = width(ir)
	if ( is.null(title) ) {
		title = paste("Averaged and smoothed methylation signals for ", gene, " within -", upstream,
				"/+", downstream, " bp", sep="")
	}
	
	cat(start, start+downstream/2, "\n")
	xyplot(Mvalue ~ (ProbeCenter/coord.factor)|Day+Treatment, data=d, type=type,
		   span=smooth.span, xlim=c((start-upstream)/coord.factor, max(c(extendrange(end(chr.ir[hits])), (start+downstream)))/coord.factor), 
		   ylim=ylim, main=title, xlab="probe position", ylab="M-value (log2(IP) - log2(total))", coord.factor=coord.factor,
		   scales=list(x=list(rot=90)), panel=panel.genomeGraphs, fullData=d, tracks=tracks, sizes=track.sizes, ...)
}

## library(biomaRt)
## bm <- useMart("ensembl", dataset="mmusculus_gene_ensembl")
## gene <- makeGeneRegion(biomart=bm, chromosome="chr7", genome="mm9", start=26670000, end=26690000)
## an <- makeAnnotationTrack(start=c(26678000, 26690000), end=c(26680000, 26900000), ID=c("ann A", "ann B"))
## setPar(an, "plotId", TRUE)
## plotAll(exProbeAnno, exMA, "Cyp2b10", layout=c(7,2), cex=2,pch=".")
## plotAll(exProbeAnno, exMA, "Cyp2b10", type=c("mountain", "boxplot"), tracks=list(an, gene), track.sizes=c(5,1,1), downstream=20000)

### convert a gene symbol into it's entrez Id
sym2entrezId <- function(sym) {
	eg <- NULL
	require(org.Mm.eg.db)
	eg <- org.Mm.egSYMBOL2EG[[sym]]
	if(is.null(eg)) ### nothing found - lookup synonyms
		eg <- org.Mm.egALIAS2EG[[sym]]
	return(eg)
}


library(BSgenome.Mmusculus.UCSC.mm9)
xx <- as(subseq(Mmusculus[[7]], 26670000, 26690000), "DNAString")
start <- 26670000
stop <- 26690000
window <- 100
by <- IRanges(start=seq(start, stop-window-1, by=10), width=window)
cg <- DNAString("CG")
tmp =  aggregate(as(Mmusculus[[7]], "DNAString"), by, FUN=function(x) countPattern(cg, x))
xx <- (end(by)-start(by))/2
xxs <- loess.smooth(xx, tmp, span=1/20, evaluation=1000)
cpg <- makeBaseTrack(base=xxs$x, value=xxs$y)
setPar(cpg, "type", "l")
plotAll(exProbeAnno, exMA, "Cyp2b10", type=c("mountain"), tracks=list(cpg, gene), track.sizes=c(5,1,1), downstream=2000, ylim=c(-0.3, 0.3))



plotAll(exProbeAnno, exMA, "Cyp2b10", type=c("p"), tracks=list(cons, cpg, gene), track.sizes=c(5,1,1), downstream=2000)

#' @include GdObject-class.R
NULL

## IdeogramTrack Class -------------------------------------------------------

#' IdeogramTrack class and methods
#'
#'
#' A class to represent the schematic display of a chromosome, also known as an
#' ideogram. The respective information is typically directly fetched from
#' UCSC.
#'
#'
#' Ideograms are schematic depictions of chromosomes, including chromosome band
#' information and centromere location. The relevant data for various species
#' is stored in the UCSC data base. The initializer method of the class will
#' automatically fetch the respective data for a given genome and chromosome
#' from UCSC and fill the appropriate object slots. When plotting
#' \code{IdeogramTracks}, the current genomic location is indicated on the
#' chromosome by a colored box.
#'
#' The \code{Gviz.ucscUrl} option controls which URL is being used to connect
#' to UCSC. For instance, one could switch to the European UCSC mirror by
#' calling \code{options(Gviz.ucscUrl="http://genome-euro.ucsc.edu/cgi-bin/"}.
#'
#' @name IdeogramTrack-class
#' @aliases IdeogramTrack-class IdeogramTrack drawGD,IdeogramTrack-method
#' end,IdeogramTrack-method end<-,IdeogramTrack-method
#' initialize,IdeogramTrack-method show,IdeogramTrack-method
#' start,IdeogramTrack-method start<-,IdeogramTrack-method
#' subset,IdeogramTrack-method width,IdeogramTrack-method
#' width<-,IdeogramTrack-method length,IdeogramTrack-method
#' [,IdeogramTrack-method [,IdeogramTrack,ANY,ANY-method
#' [,IdeogramTrack,ANY,ANY,ANY-method chromosome<-,IdeogramTrack-method
#' genome<-,IdeogramTrack-method position,IdeogramTrack-method
#' @docType class
#' @param chromosome The chromosome for which to create the ideogram. Has to be
#' a valid UCSC chromosome identifier of the form \code{chrx}, or a single
#' integer or numeric character unless
#' \code{option(ucscChromosomeNames=FALSE)}. The user has to make sure that the
#' respective chromosome is indeed defined for the the track's genome.
#' @param genome The genome on which to create the ideogram. This has to be a
#' valid UCSC genome identifier if the ideogram data is to be fetched from the
#' UCSC repository.
#' @param name Character scalar of the track's name used in the title panel
#' when plotting. Defaults to the selected chromosome.
#' @param bands A \code{data.frame} with the cytoband information for all
#' available chromosomes on the genome similar to the data that would be
#' fetched from UCSC. The table needs to contain the mandatory columns
#' \code{chrom}, \code{chromStart}, \code{chromEnd}, \code{name} and
#' \code{gieStain} with the chromosome name, cytoband start and end
#' coordinates, cytoband name and coloring information, respectively. This can
#' be used when no connection to the internet is available or when the cytoband
#' information has been cached locally to avoid the somewhat slow connection to
#' UCSC.
#' @param \dots Additional items which will all be interpreted as further
#' display parameters.
#' @return
#'
#' The return value of the constructor function is a new object of class
#' \code{IdeogramTrack}.
#' @note
#'
#' When fetching ideogram data from UCSC the results are cached for faster
#' acces. See \code{\link{clearSessionCache}} on details to delete these cached
#' items.
#' @section Objects from the Class:
#'
#' Objects can be created using the constructor function \code{IdeogramTrack}.
#' @author Florian Hahne
#' @inherit GdObject-class seealso
#'
#' @examples
#' \dontshow{
#' ## Load some sample data
#' data(idTrack)
#' }
#'
#' ## Construct the object
#' \dontrun{
#' idTrack <- IdeogramTrack(chromosome = 7, genome = "mm9")
#' }
#'
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(idTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(idTrack, from = 5000000, to = 9000000)
#'
#' ## Track names
#' names(idTrack)
#' names(idTrack) <- "foo"
#' plotTracks(idTrack, from = 5000000, to = 9000000)
#'
#'
#' ## Accessors
#' chromosome(idTrack)
#' \dontrun{
#' chromosome(idTrack) <- "chrX"
#' }
#'
#' genome(idTrack)
#' \dontrun{
#' genome(id) <- "hg19"
#' }
#'
#' range(idTrack)
#' ranges(idTrack)
#'
#' ## Annotation
#' values(idTrack)
#'
#' ## coercion
#' as(idTrack, "data.frame")
#' @exportClass IdeogramTrack
setClass("IdeogramTrack",
    contains = "RangeTrack",
    representation = representation(bandTable = "data.frame"),
    prototype = prototype(
        name = "IdeogramTrack",
        bandTable = data.frame(),
        dp = DisplayPars(
            background.title = "transparent",
            bevel = 0.45,
            centromereShape = "triangle",
            cex.bands = 0.7,
            cex = 0.8,
            col = "red",
            col.border.title = "transparent",
            lwd.border.title = 1,
            fill = "#FFE3E6",
            fontface = 1,
            fontfamily = "sans",
            fontcolor = .DEFAULT_SHADED_COL,
            fontsize = 10,
            outline = FALSE,
            showBandId = FALSE,
            lty = 1,
            lwd = 1,
            showId = TRUE,
            showTitle = FALSE,
            size = NULL
        )
    )
)

## Initialize ----------------------------------------------------------------

## Grab the chromosome band and length information from UCSC and fill the ranges slot.

#' @export
setMethod("initialize", "IdeogramTrack", function(.Object, genome, chromosome, bands, name, ...) {
    ## the display parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "IdeogramTrack")
    if (missing(bands)) {
        bands <- NULL
    }
    if (is.null(bands) && (missing(genome) || missing(chromosome))) {
        return(callNextMethod(.Object = .Object, range = GRanges(), genome = NULL, chromosome = NULL, ...))
    }
    if (is.null(bands)) {
        sessionInfo <- .cacheGenomes(genome = genome)
        .Object@bandTable <- sessionInfo$bands
        bands <- sessionInfo$bands
    } else {
        .checkClass(bands, "data.frame")
        cols <- c("chrom", "chromStart", "chromEnd", "name", "gieStain")
        miss <- !cols %in% colnames(bands)
        if (any(miss)) {
            stop(sprintf(
                "The following column%s missing from the bands table: %s",
                ifelse(sum(miss) > 1, "s are", " is"), paste(cols[miss], collapse = ", ")
            ))
        }
        .Object@bandTable <- bands
    }
    chromosome <- if (is.null(chromosome)) as.character(bands[1, "chrom"]) else .chrName(chromosome)[1]
    bands <- bands[bands$chrom == chromosome, ]
    if (nrow(bands) == 0) {
        stop("Chromosome '", chromosome, "' does not exist on UCSC genome '", genome, "'")
    }
    if (is.null(name)) {
        name <- .chrName(chromosome)[1]
    }
    bnames <- as.character(bands$name)
    sel <- is.na(bnames)
    if (any(sel)) {
        bnames[sel] <- paste("band", seq_len(sum(sel)), sep = "_")
    }
    if (any(bnames == "")) {
        bnames[bnames == ""] <- sprintf("band_%i", which(bnames == ""))
    }
    ranges <- GRanges(
        seqnames = bnames, ranges = IRanges(start = bands$chromStart, end = bands$chromEnd),
        name = bnames, type = as.character(bands$gieStain)
    )
    .Object <- callNextMethod(.Object = .Object, range = ranges, genome = genome, chromosome = chromosome, name = name, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters
#' @export
IdeogramTrack <- function(chromosome = NULL, genome, name = NULL, bands = NULL, ...) {
    if (missing(genome)) stop("Need to specify genome for creating an IdeogramTrack")
    new("IdeogramTrack", chromosome = chromosome, genome = genome, name = name, bands = bands, ...)
}

## General accessors ---------------------------------------------------------

#' @export
setMethod("start", "IdeogramTrack", function(x) NULL) # if(length(x)) start(range(x)) else NULL)

#' @export
setReplaceMethod("start", "IdeogramTrack", function(x, value) {
    # start(x@range) <- value
    return(x)
})

#' @export
setMethod("end", "IdeogramTrack", function(x) NULL) # if(length(x)) end(range(x)) else NULL)

#' @export
setReplaceMethod("end", "IdeogramTrack", function(x, value) {
    # end(x@range) <- value
    return(x)
})

#' @export
setMethod("width", "IdeogramTrack", function(x) NULL) # if(length(x)) width(range(x)) else NULL)

#' @export
setReplaceMethod("width", "IdeogramTrack", function(x, value) {
    return(x)
})

#' @export
setMethod("length", "IdeogramTrack", function(x) length(ranges(x)))

#' @export
setReplaceMethod("chromosome", "IdeogramTrack", function(GdObject, value) {
    ## We have changed the class definition to include the bands for all chromosomes, but still want the old objects to work
    chromosome <- .chrName(value[1])
    if (.hasSlot(GdObject, "bandTable") && chromosome %in% as.character(GdObject@bandTable$chrom)) {
        ranges <- GdObject@bandTable[GdObject@bandTable$chrom == chromosome, ]
        bnames <- as.character(ranges$name)
        sel <- is.na(bnames)
        if (any(sel)) {
            bnames[sel] <- paste("band", seq_len(sum(sel)), sep = "_")
        }
        if (any(bnames == "")) {
            bnames[bnames == ""] <- sprintf("band_%i", which(bnames == ""))
        }
        ranges <- GRanges(
            seqnames = bnames, ranges = IRanges(start = ranges$chromStart, end = ranges$chromEnd),
            name = bnames, type = ranges$gieStain
        )
        GdObject@range <- ranges
        GdObject@chromosome <- chromosome
        return(GdObject)
    }
    message("Updating chromosome band information")
    tmp <- IdeogramTrack(genome = genome(GdObject), chromosome = .chrName(value[1]), name = names(GdObject))
    displayPars(tmp) <- displayPars(GdObject, hideInternal = FALSE)
    return(tmp)
})

#' @export
setReplaceMethod("genome", "IdeogramTrack", function(x, value) {
    if (genome(x) != value) {
        message("Updating chromosome band information")
    }
    tmp <- IdeogramTrack(genome = value[1], chromosome = chromosome(x), name = names(x))
    displayPars(tmp) <- displayPars(x, hideInternal = FALSE)
    return(tmp)
})


## Annotation Accessors ------------------------------------------------------
## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------

#' @export
setMethod("[", signature(x = "IdeogramTrack"), function(x, i, j, ..., drop = TRUE) {
    return(x)
})

## Position ------------------------------------------------------------------

#' @export
setMethod("position", signature("IdeogramTrack"), definition = function(GdObject, ...) NULL)

## DrawGD --------------------------------------------------------------------

## Helper function to compute coordinates for a rounded ideogram cap
.roundedCap <- function(bl, tr, st, vals, side = c("left", "right"), bevel = 0.4, n = 100) {
    side <- match.arg(side)
    bevel <- max(1 / n, min(bevel, 0.5))
    coords <- if (bevel <= 0) {
        cbind(c(0, 1, 1, 0), c(1.5, 1.5, -1.5, -1.5))
    } else {
        cbind(
            c(0, sin(seq(0, pi / 2, len = max(2, n * bevel))) + bevel * 4, sin(seq(pi / 2, pi, len = max(2, n * bevel))) + bevel * 4, 0) / (1 + 4 * bevel),
            (c(
                1 + 0.5 - bevel, cos(seq(0, pi / 2, len = max(2, n * bevel))) + (0.5 - bevel),
                cos(seq(pi / 2, pi, len = max(2, n * bevel))) - (0.5 - bevel), cos(pi) - (0.5 - bevel)
            ) + 1 + (0.5 - bevel)) / (2 + 2 * (0.5 - bevel))
        )
    }
    if (side == "right") {
        coords[, 1] <- coords[, 1] * abs(diff(c(bl[1], tr[1]))) + bl[1]
        coords[, 2] <- coords[, 2] * abs(diff(c(bl[2], tr[2]))) + bl[2]
    } else {
        coords[, 1] <- (1 - coords[, 1]) * abs(diff(c(bl[1], tr[1]))) + bl[1]
        coords[, 2] <- (1 - coords[, 2]) * abs(diff(c(bl[2], tr[2]))) + bl[2]
    }
    lcS <- split(as.data.frame(coords), cut(coords[, 1], st, right = TRUE, include.lowest = TRUE, labels = FALSE), drop = TRUE)
    first <- TRUE
    shift <- ifelse(side == "left", 0, 1)
    for (j in names(lcS)) {
        xx <- lcS[[j]][, 1]
        yy <- lcS[[j]][, 2]
        if (!first) {
            prev <- lcS[[as.character(as.numeric(j) - 1)]]
            xx <- c(tail(prev[, 1], 1), xx, tail(prev[, 1], 1))
            yy <- c(1 - tail(prev[, 2], 1), yy, tail(prev[, 2], 1))
        }
        grid.polygon(xx, yy, gp = gpar(col = vals[as.numeric(j) + shift, "col"], fill = vals[as.numeric(j) + shift, "col"]))
        first <- FALSE
    }
    return(coords)
}

## A more generic method to come up with colors for chromosome bands that still relies a bit on biovizBase
#' @importFrom grDevices colorRampPalette
#' @importFrom stats setNames
.getBioColorIdeo <- function(type) {
    type <- as.character(type)
    ocols <- getBioColor("CYTOBAND")
    cols <- c(ocols[c("gneg", "stalk", "acen")], gpos = unname(ocols["gpos100"]), gvar = unname(ocols["gpos100"]))
    gpcols <- unique(grep("gpos", type, value = TRUE))
    crmp <- colorRampPalette(c(cols["gneg"], cols["gpos"]))(100)
    posCols <- setNames(crmp[as.integer(gsub("gpos", "", gpcols))], gpcols)
    return(c(cols, posCols))
}

## The actual drawing method
#' @importFrom grDevices rgb2hsv
#' @export
setMethod("drawGD", signature("IdeogramTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, ...) {
    debug <- .dpOrDefault(GdObject, "debug", FALSE)
    if ((is.logical(debug) && debug) || debug == "prepare") {
        browser()
    }
    imageMap(GdObject) <- NULL
    if (names(GdObject)[1] != chromosome(GdObject)) {
        chrnam <- names(GdObject)[1]
    } else {
        chrnam <- paste("Chromosome", gsub("chr", "", chromosome(GdObject)))
    }
    cex <- .dpOrDefault(GdObject, "cex", 1)
    ## Nothing to do if there are no ranges in the object
    if (!length(GdObject)) {
        return(invisible(GdObject))
    }
    ## In prepare mode we just want to figure out the optimal size
    if (prepare) {
        pres <- .pxResolution()
        nsp <- if (.dpOrDefault(GdObject, "showId", TRUE)) {
            (as.numeric(convertHeight(stringHeight(chrnam), "native")) * cex) +
                as.numeric(convertHeight(unit(20, "points"), "native"))
        } else {
            as.numeric(convertHeight(unit(25, "points"), "native"))
        }
        nsp <- nsp / pres["y"]
        displayPars(GdObject) <- list("neededVerticalSpace" = nsp)
        ## Augment the ranges to fill gaps if there are any
        gaps <- setdiff(IRanges(0, max(end(range(GdObject)))), range(GdObject))
        if (length(gaps)) {
            gaps <- GRanges(seqnames(GdObject)[1], gaps, name = rep(as.character(NA), length(gaps)), type = rep("gneg", length(gaps)))
            rr <- c(ranges(GdObject), gaps)
            ranges(GdObject) <- sort(rr)
        }
        return(invisible(GdObject))
    }
    if ((is.logical(debug) && debug) || debug == "draw") {
        browser()
    }
    ## Do we need some space for the chromosome name?
    if (.dpOrDefault(GdObject, "showId", TRUE)) {
        gp <- .fontGp(GdObject)
        width <- vpLocation()$isize["width"]
        width <- as.numeric(convertWidth(stringWidth(chrnam), "inches")) * cex + 0.2
        wfac <- vpLocation()$isize["width"]
        nspace <- min(width / wfac, 0.75)
        if ((width / wfac) > 0.75) {
            cex <- cex * (0.75 / (width / wfac))
        }
        pushViewport(viewport(x = 0, width = nspace, just = 0, gp = gp))
        grid.text(chrnam, 0, default.units = "native", just = c("left", "center"))
        popViewport(1)
    } else {
        nspace <- 0
    }
    pushViewport(viewport(x = nspace, width = 1 - nspace, just = 0))
    ## A box indicating the current range on the chromosome
    len <- end(range(range(GdObject)))
    fill <- .dpOrDefault(GdObject, "fill", "#FFE3E6")
    if (!missing(minBase) && !missing(maxBase)) {
        grid.rect(minBase / len, 0.1,
            width = min(1, (maxBase - minBase) / len), height = 0.8, just = c("left", "bottom"),
            gp = gpar(col = "transparent", fill = fill)
        )
    }
    ## Color mapping for the bands taken from the biovizBase package
    cols <- .getBioColorIdeo(values(GdObject)$type)
    vals <- data.frame(values(GdObject), col = cols[as.character(values(GdObject)$type)], stringsAsFactors = FALSE)
    ## For the rounded caps we need  to figure out the overlap with existing bands for proper coloring
    bevel <- 0.02
    ol <- queryHits(findOverlaps(range(GdObject), IRanges(start = c(bevel, 1 - bevel) * len, width = 1)))
    st <- start(range(GdObject)) / len
    ed <- end(range(GdObject)) / len
    stExt <- if (length(GdObject) == 1) c(0, bevel, 1 - bevel) else c(st[seq_len(ol[1])], bevel, st[(ol[1] + 1):ol[2]], 1 - bevel)
    valsExt <- if (length(GdObject) == 1) vals[rep(1, 3), ] else rbind(vals[seq_len(ol[1]), ], vals[ol[1], ], vals[(ol[1] + 1):ol[2], ], vals[ol[2], ])
    if (ol[2] < length(st)) {
        stExt <- c(stExt, st[(ol[2] + 1):length(st)])
        valsExt <- rbind(valsExt, vals[(ol[2] + 1):length(st), ])
    }
    wd <- diff(c(stExt, 1))
    ls <- ol[1] + 1
    rs <- ol[2] + 1
    ## The centromere is treated separately
    cent <- grep("acen", valsExt$type)
    if (length(cent)) {
        bef <- ls:(min(cent) - 1)
        aft <- (max(cent) + 1):rs
    } else {
        bef <- ls:rs
        aft <- NULL
    }
    margin <- 0.3
    ## First the normal bands
    lcol <- "black"
    lwd <- 1
    lty <- 1
    lcolBands <- if (.dpOrDefault(GdObject, "outline", FALSE)[1] == TRUE) rep(lcol, nrow(valsExt)) else valsExt[, "col"]
    grid.rect(stExt[bef], margin,
        width = wd[bef], height = 1 - margin * 2, gp = gpar(col = lcolBands[bef], fill = valsExt[bef, "col"]),
        just = c("left", "bottom")
    )
    if (!is.null(aft)) {
        grid.rect(stExt[aft], margin,
            width = wd[aft], height = 1 - margin * 2, gp = gpar(col = lcolBands[aft], fill = valsExt[aft, "col"]),
            just = c("left", "bottom")
        )
    }
    ## Now the centromere, if there is any
    centromereShape <- .dpOrDefault(GdObject, "centromereShape", "triangle")
    if (length(cent) && centromereShape == "triangle") {
        grid.polygon(c(stExt[min(cent)], stExt[min(cent)] + wd[min(cent)], rep(stExt[min(cent)], 2)),
            c(margin, 0.5, (1 - margin), margin),
            gp = gpar(col = cols["acen"], fill = cols["acen"])
        )
        grid.polygon(c(stExt[max(cent)], rep(stExt[max(cent)] + wd[max(cent)], 2), stExt[max(cent)]),
            c(0.5, margin, (1 - margin), 0.5),
            gp = gpar(col = cols["acen"], fill = cols["acen"])
        )
    }
    ## Now the caps
    str <- if (length(st) == 1) c(0, 1) else st
    edr <- if (length(ed) == 1) c(1, 2) else ed
    lc <- .roundedCap(c(stExt[1], margin), c(stExt[ls], 1 - margin), str, vals, side = "left", bevel = .dpOrDefault(GdObject, "bevel", 0.45))
    rc <- .roundedCap(c(tail(stExt, 1), margin), c(1, 1 - margin), edr, vals, side = "right", bevel = .dpOrDefault(GdObject, "bevel", 0.45))
    ## Now some outlines
    grid.lines(lc[, 1], lc[, 2], gp = gpar(col = lcol, lwd = lwd, lty = lty))
    grid.lines(rc[, 1], rc[, 2], gp = gpar(col = lcol, lwd = lwd, lty = lty))
    if (length(cent)) {
        x0 <- c(rep(max(lc[, 1]), 2), rep(stExt[max(cent) + 1], 2), rep(stExt[min(cent)], 2), rep(stExt[max(cent)], 2))
        y0 <- c(rep(c(margin, (1 - margin)), 3), 0.5, 0.5)
        x1 <- c(
            rep(stExt[min(cent)], 2), rep(tail(stExt, 1), 2), rep(stExt[min(cent)] + wd[min(cent)], 2),
            rep(stExt[max(cent)] + wd[max(cent)], 2)
        )
        y1 <- c(rep(c(margin, (1 - margin)), 2), 0.5, 0.5, margin, (1 - margin))
    } else {
        x0 <- rep(max(lc[, 1]), 2)
        y0 <- c(margin, (1 - margin))
        x1 <- rep(max(stExt), 2)
        y1 <- y0
    }
    grid.segments(x0, y0, x1, y1, gp = gpar(col = lcol, lwd = lwd, lty = lty))
    ## centromere (circle)
    if (length(cent) && centromereShape == "circle") {
        sc <- as.numeric(convertHeight(unit(sum(wd[cent]), "points"), "native")) /
            as.numeric(convertWidth(unit(sum(wd[cent]), "points"), "native"))
        grid.circle(
            x = stExt[min(cent)] + sum(wd[cent]) / 2, y = 0.5, r = sc * sum(wd[cent]) / 2,
            gp = gpar(col = lcol, fill = cols["acen"])
        )
    }
    ## The outlines of the box
    if (!missing(minBase) && !missing(maxBase)) {
        col <- .dpOrDefault(GdObject, "col", "red")
        lwd <- .dpOrDefault(GdObject, "lwd", 1)
        lty <- .dpOrDefault(GdObject, "lty", "solid")
        grid.rect(minBase / len, 0.1,
            width = min(1, (maxBase - minBase) / len), height = 0.8, just = c("left", "bottom"),
            gp = gpar(col = col, fill = "transparent", lwd = lwd, lty = lty)
        )
    }
    ## Finally the band annotation if we need it
    if (.dpOrDefault(GdObject, "showBandId", FALSE)) {
        bn <- as.character(values(GdObject)$name)
        cval <- rgb2hsv(col2rgb(cols[as.character(values(GdObject)$type)]))["v", ]
        tcol <- ifelse(cval > 0.9, "black", "white")
        bwidth <- (c(st[-1], 1) - st) / 2
        cex.bands <- .dpOrDefault(GdObject, "cex.bands", 0.7)
        sspace <- as.numeric(convertUnit(unit(0.01, "inches"), "native"))
        swidth <- as.numeric(convertWidth(stringWidth(bn), "native")) * cex.bands + sspace
        sel <- swidth < bwidth
        if (any(sel)) {
            grid.text(x = (st + bwidth)[sel], y = 0.5, label = bn[sel], hjust = 0.5, gp = gpar(col = tcol[sel], cex = cex.bands))
        }
    }
    popViewport(1)
    return(invisible(GdObject))
})

## Show ----------------------------------------------------------------------

#' @export
setMethod(
    "show", signature(object = "IdeogramTrack"),
    function(object) {
        cat(sprintf(
            paste("Ideogram track '%s' for chromosome %s of the %s genome"),
            names(object),
            gsub("^chr", "", chromosome(object)),
            genome(object)
        ), "\n")
    }
)

#' @include RangeTrack-class.R
NULL

## HighlightTrack Class ------------------------------------------------------

#' HighlightTrack class and methods
#'
#'
#' A container for other track objects from the Gviz package that allows for
#' the addition of a common highlighting area across tracks.
#'
#'
#' A track to conceptionally group other Gviz track objects into a meta track
#' for the sole purpose of overlaying all the contained tracks with the same
#' highlighting region as defined by the objects genomic ranges. During
#' rendering the contained tracks will be treated as if they had been provided
#' to the \code{plotTracks} function as individual objects.
#'
#' @template HighlightTrack-class_param
#'
#' @name HighlightTrack-class
#'
#' @return
#'
#' The return value of the constructor function is a new object of class
#' \code{HighlightTrack}.
#' @section Objects from the Class:
#'
#' Objects can be created using the constructor function \code{HighlightTrack}.
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## Object construction:
#' set.seed(123)
#' dat <- runif(100, min = -2, max = 22)
#' gt <- GenomeAxisTrack()
#' dt <- DataTrack(data = dat, start = sort(sample(200, 100)), width = 1, genome = "hg19")
#'
#' ht <- HighlightTrack(trackList = list(gt, dt))
#' @exportClass HighlightTrack
setClass("HighlightTrack",
    representation = representation(trackList = "list"),
    contains = c("RangeTrack"),
    prototype = prototype(dp = DisplayPars(
        col = "red",
        fill = "#FFE3E6",
        inBackground = TRUE
    ))
)

## Initialize ----------------------------------------------------------------

#' @describeIn HighlightTrack-class Initialize.
#' @export
setMethod("initialize", "HighlightTrack", function(.Object, trackList, ...) {
    .Object <- .updatePars(.Object, "HighlightTrack")
    .Object@trackList <- trackList
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

#' @describeIn HighlightTrack-class Constructor function for
#' `HighlightTrack-class`.
#' @export
HighlightTrack <- function(trackList = list(), range = NULL, start = NULL, end = NULL, width = NULL, chromosome, genome,
                           name = "HighlightTrack", ...) {
    ## Some defaults
    covars <- .getCovars(range)
    n <- max(c(length(start), length(end), length(width)), nrow(covars))
    ## Build a GRanges object from the inputs
    .missingToNull(c("strand", "chromosome", "genome"))
    args <- list(chromosome = chromosome, genome = genome)
    defs <- list(strand = "*", density = 1, chromosome = "chrNA", genome = NA)
    range <- .buildRange(
        range = range, start = start, end = end, width = width,
        args = args, defaults = defs, chromosome = chromosome, trackType = "HighlightTrack"
    )
    if (is.list(range)) {
        range <- GRanges()
    }
    if (!is.list(trackList)) {
        trackList <- list(trackList)
    }
    if (!all(vapply(trackList, is, class2 = "GdObject", FUN.VALUE = logical(1)))) {
        stop("All elements in 'trackList' must inherit from 'GdObject'")
    }
    ## If no chromosome was explicitly asked for we just take the first one in the GRanges object
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- if (length(range) > 0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    }
    ## And finally the object instantiation
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    return(new("HighlightTrack", trackList = trackList, chromosome = chromosome[1], range = range, name = name, genome = genome, ...))
}

## General accessors ---------------------------------------------------------

#' @describeIn HighlightTrack-class set display parameters using the values of
#' the named list in value. See \code{\link{settings}} for details on display
#' parameters and customization.
#' @export
setReplaceMethod("displayPars", signature("HighlightTrack", "list"), function(x, recursive = FALSE, value) {
    x <- setPar(x, value, interactive = FALSE)
    if (recursive) {
        x@trackList <- lapply(x@trackList, function(y) {
            displayPars(y) <- value
            return(y)
        })
    }
    return(x)
})

#' @describeIn HighlightTrack-class return the number of subtracks.
#' @export
setMethod("length", "HighlightTrack", function(x) length(x@trackList))

#' @describeIn HighlightTrack-class replace the value of the track's chromosome.
#' This has to be a valid UCSC chromosome identifier or an integer or character
#' scalar that can be reasonably coerced into one.
#' @export
setReplaceMethod("chromosome", "HighlightTrack", function(GdObject, value) {
    GdObject@trackList <- lapply(GdObject@trackList, function(x) {
        chromosome(x) <- value[1]
        x
    })
    GdObject@chromosome <- .chrName(value[1])
    return(GdObject)
})


## Annotation Accessors ------------------------------------------------------
## Stacking ------------------------------------------------------------------

#' @describeIn HighlightTrack-class Rrecompute the stacks based on the available
#' space and on the object's track items and stacking settings.
#' This really just calls the `setStacks` methods for the contained tracks and
#' only exists for dispatching reasons.
#' @export
setMethod("setStacks", "HighlightTrack", function(GdObject, ...) {
    GdObject@trackList <- lapply(GdObject@trackList, setStacks, ...)
    return(GdObject)
})

## Consolidate ---------------------------------------------------------------

#' @describeIn HighlightTrack-class Consolidate
#' For a `HighlightTrack` apply the method on each of the subtracks in
#' the `trackList` slot
# #' @keywords internal
#' @export
setMethod("consolidateTrack", signature(GdObject = "HighlightTrack"), function(GdObject, chromosome, ...) {
    GdObject@trackList <- lapply(GdObject@trackList, consolidateTrack, chromosome = chromosome, ...)
    return(GdObject)
})

## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------

#' @describeIn HighlightTrack-class subset all the contained tracks in an HighlightTrack by coordinates and sort if necessary.
#' @export
setMethod("subset", signature(x = "HighlightTrack"), function(x, ...) {
    x@trackList <- lapply(x@trackList, subset, ...)
    x <- callNextMethod()
    return(x)
})

## Position ------------------------------------------------------------------
## DrawGrid ------------------------------------------------------------------
## DrawAxis ------------------------------------------------------------------
## DrawGD --------------------------------------------------------------------
## SetAs ---------------------------------------------------------------------
## Show ----------------------------------------------------------------------

#' @describeIn HighlightTrack-class Show method.
#' @export
setMethod("show", signature(object = "HighlightTrack"), function(object) {
    cat(sprintf(
        "HighlightTrack '%s' containing %i subtrack%s\n%s\n", names(object), length(object),
        ifelse(length(object) == 1, "", "s"), .annotationTrackInfo(object)
    ))
})

#' @include GdObject-class.R
NULL

#' CustomTrack class and methods
#'
#' A fully customizable track object to be populated via a user-defined
#' plotting function.
#'
#' A track to allow for any sort of plotting, with the currently displayed
#' genomic location set. Essentially this acts as a simple callback into the
#' `Gviz` plotting machinery after all the track panels and coordinates
#' have been set up. It is entirely up to the user what to plot in the track,
#' or even to use the predefined coordinate system. The only prerequisite is
#' that all plotting operations need to utilize Grid graphics.
#'
#' @template CustomTrack-class_param
#'
#' @name CustomTrack-class
#'
#' @return
#' The return value of the constructor function is a new object of class
#' `CustomTrack`.
#'
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#'
#' @examples
#' ## Object construction:
#'
#' ## An empty object
#' CustomTrack()
#' @export
setClass("CustomTrack",
    contains = c("GdObject"),
    representation = representation(
        plottingFunction = "function",
        variables = "list"
    ),
    prototype = prototype(dp = DisplayPars())
)

## Initialize ----------------------------------------------------------------

#' @describeIn CustomTrack-class Initialize.
#' @export
setMethod("initialize", "CustomTrack", function(.Object, plottingFunction, variables, ...) {
    .Object <- .updatePars(.Object, "CustomTrack")
    .Object@plottingFunction <- plottingFunction
    .Object@variables <- variables
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

#' @param plottingFunction A user-defined function to be executed once the
#' track coordinates have been properly set up. The function needs to accept
#' two mandatory arguments: `GdObject`, the `CustomTrack` object to
#' be plotted, and `prepare`, a logical flag indicating whether the
#' function has been called in preparation mode or in drawing mode. It also
#' needs to return the input `GdObject`, potentially with modifications.
#' @param variables A list of additional variables for the user-defined
#' plotting function.
#' @param name Character scalar of the track's name.
#' @param \dots Additional items which will all be interpreted as further
#' display parameters. See `settings` and the "Display Parameters"
#' section below for details.
#' @describeIn CustomTrack-class Objects can be created using the constructor function.
#' @export
CustomTrack <- function(plottingFunction = function(GdObject, prepare = FALSE, ...) {}, variables = list(), name = "CustomTrack", ...) {
    return(new("CustomTrack", plottingFunction = plottingFunction, variables = variables, name = name, ...))
}

## General accessors ---------------------------------------------------------
## Annotation Accessors ------------------------------------------------------
## Stacking ------------------------------------------------------------------
## Consolidate ---------------------------------------------------------------
## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------
## Position ------------------------------------------------------------------
## DrawGrid ------------------------------------------------------------------
## DrawAxis ------------------------------------------------------------------
## DrawGD --------------------------------------------------------------------

#' @describeIn CustomTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @export
setMethod("drawGD", signature("CustomTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, ...) {
    rev <- .dpOrDefault(GdObject, "reverseStrand", FALSE)
    xscale <- if (!rev) c(minBase, maxBase) else c(maxBase, minBase)
    pushViewport(viewport(xscale = xscale, clip = TRUE))
    tmp <- GdObject@plottingFunction(GdObject, prepare = prepare)
    if (!is(tmp, "CustomTrack")) {
        warning("The plotting function of a CustomTrack has to return the input object. Using the original CustomTrack object now.")
    } else {
        GdObject <- tmp
    }
    popViewport(1)
    return(invisible(GdObject))
})

## SetAs ---------------------------------------------------------------------
## Show ----------------------------------------------------------------------

#' @describeIn CustomTrack-class Show method.
#' @export
setMethod("show", signature(object = "CustomTrack"), function(object) {
    cat(sprintf("CustomTrack '%s'\n", names(object)))
})

#' @include GdObject-class.R
NULL

## GenomeAxisTrack Class -----------------------------------------------------

#' GenomeAxisTrack class and methods
#'
#' A class representing a customizable genomic axis.
#'
#'
#' A \code{GenomeAxisTrack} can be customized using the familiar display
#' parameters. By providing a \code{GRanges} or \code{IRanges} object to the
#' constructor, ranges on the axis can be further highlighted.
#'
#' With the \code{scale} display parameter, a small scale indicator can be
#' shown instead of the entire genomic axis. The scale can either be provided
#' as a fraction of the plotting region (it will be rounded to the nearest
#' human readable absolute value) or as an absolute value and is always
#' displayed in bp, kb, mb or gb units. Note that most display parameters for
#' the \code{GenomeAxisTrack} are ignored when a scale is used instead of the
#' full axis. In particular, only the parameters \code{exponent}, \code{alpha},
#' \code{lwd}, \code{col}, \code{cex}, \code{distFromAxis} and \code{labelPos}
#' are used.
#'
#' @template GenomeAxisTrack-class_param
#'
#' @name GenomeAxisTrack-class
#'
#' @return The return value of the constructor function is a new object of class
#' \code{GenomeAxisTrack}.
#'
#' Objects can be created using the constructor function \code{GenomeAxisTrack}.
#'
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## Construct object
#' axTrack <- GenomeAxisTrack(
#'     name = "Axis",
#'     range <- IRanges(start = c(100, 300, 800), end = c(150, 400, 1000))
#' )
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(axTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#'
#' ## Plotting
#' plotTracks(axTrack, from = 0, to = 1100)
#'
#' ## Track names
#' names(axTrack)
#' names(axTrack) <- "foo"
#'
#' ## Subsetting and splitting
#' subTrack <- subset(axTrack, from = 0, to = 500)
#' length(subTrack)
#' subTrack[1]
#' split(axTrack, c(1, 1, 2))
#'
#' ## Accessors
#' start(axTrack)
#' end(axTrack)
#' width(axTrack)
#'
#' strand(axTrack)
#'
#' range(axTrack)
#' ranges(axTrack)
#'
#' ## Annotation
#' values(axTrack)
#'
#' ## Grouping
#' group(axTrack)
#'
#' ## HTML image map
#' coords(axTrack)
#' tags(axTrack)
#' axTrack <- plotTracks(axTrack)$foo
#' coords(axTrack)
#' tags(axTrack)
#'
#' ## adding an axis to another track
#' data(cyp2b10)
#' grTrack <- GeneRegionTrack(
#'     start = 26682683, end = 26711643,
#'     rstart = cyp2b10$start, rends = cyp2b10$end, chromosome = 7, genome = "mm9",
#'     transcript = cyp2b10$transcript, gene = cyp2b10$gene, symbol = cyp2b10$symbol,
#'     name = "Cyp2b10", strand = cyp2b10$strand
#' )
#'
#' plotTracks(list(grTrack, GenomeAxisTrack()))
#' plotTracks(list(grTrack, GenomeAxisTrack(scale = 0.1)))
#' plotTracks(list(grTrack, GenomeAxisTrack(scale = 5000)))
#' plotTracks(list(grTrack, GenomeAxisTrack(scale = 0.5, labelPos = "below")))
#' @exportClass GenomeAxisTrack
setClass("GenomeAxisTrack",
    contains = "GdObject",
    representation = representation(range = "GRanges"),
    prototype(
        range = GRanges(),
        name = "GenomeAxisTrack",
        dp = DisplayPars(
            add35 = FALSE,
            add53 = FALSE,
            background.title = "transparent",
            cex.id = 0.7,
            cex = 0.8,
            col.border.title = "transparent",
            lwd.border.title = 1,
            col.id = "white",
            col.range = "cornsilk4",
            distFromAxis = 1,
            exponent = NULL,
            fill.range = "cornsilk3",
            fontcolor = "#808080",
            fontsize = 10,
            labelPos = "alternating",
            littleTicks = FALSE,
            lwd = 2,
            scale = NULL,
            showId = FALSE,
            showTitle = FALSE,
            ticksAt = NULL,
            size = NULL,
            col = "darkgray"
        )
    )
)
## Initialize ----------------------------------------------------------------

## Only pass on the stuff to the GdObject initializer

#' @describeIn GenomeAxisTrack-class Intialize.
#' @export
setMethod("initialize", "GenomeAxisTrack", function(.Object, range, ids, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "GenomeAxisTrack")
    if (missing(range) || is.null(range)) {
        range <- GRanges()
    }
    if (is(range, "IRanges")) {
        range <- GRanges(ranges = range, seqnames = "dummy", id = ids)
    }
    .Object@range <- range
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

#' @describeIn GenomeAxisTrack-class Constructor
#' @export
GenomeAxisTrack <- function(range = NULL, name = "Axis", id, ...) {
    if (missing(id)) {
        id <- names(range)
    }
    new("GenomeAxisTrack", name = name, range = range, id = id, ...)
}

## General accessors ---------------------------------------------------------

#' @describeIn GenomeAxisTrack-class return the genomic coordinates for the
#' track along with all additional annotation information as an object of
#' class `GRanges`.
setMethod("ranges", "GenomeAxisTrack", function(x) x@range)
setReplaceMethod("ranges", "GenomeAxisTrack", function(x, value) {
    x@range <- value
    return(x)
})

#' @describeIn GenomeAxisTrack-class return the genomic coordinates for the
#'  track as an object of class `IRanges`.
#'  @export
setMethod("range", "GenomeAxisTrack", function(x) ranges(x@range))

#' @describeIn GenomeAxisTrack-class return the start coordinates of the track
#' items.
#' @export
setMethod("start", "GenomeAxisTrack", function(x) if (length(x)) start(range(x)) else NULL)

#' @describeIn GenomeAxisTrack-class replace the start coordinates of the track
#' items.
#' @export
setReplaceMethod("start", "GenomeAxisTrack", function(x, value) {
    start(x@range) <- value
    return(x)
})

#' @describeIn GenomeAxisTrack-class return the end coordinates of the track
#' items.
#' @export
setMethod("end", "GenomeAxisTrack", function(x) if (length(x)) end(range(x)) else NULL)

#' @describeIn GenomeAxisTrack-class replace the end coordinates of the track
#' items.
#' @export
setReplaceMethod("end", "GenomeAxisTrack", function(x, value) {
    end(x@range) <- value
    return(x)
})

#' @describeIn GenomeAxisTrack-class return the with of the track items in
#' genomic coordinates.
#' @export
setMethod("width", "GenomeAxisTrack", function(x) if (length(x)) as.integer(width(range(x))) else NULL)

#' @describeIn GenomeAxisTrack-class return the number of items stored in the ranges slot.
#' @export
setMethod("length", "GenomeAxisTrack", function(x) length(ranges(x)))

#' @describeIn GenomeAxisTrack-class return all additional annotation information
#' except for the genomic coordinates for the track items.
#' @export
setMethod("values", "GenomeAxisTrack", function(x) as.data.frame(values(ranges(x))))

#' @describeIn GenomeAxisTrack-class return a vector of strand specifiers for
#' all track items, in the form '+' for the Watson strand, '-' for the Crick
#' strand or '*' for either of the two.
#' @export
setMethod("strand", "GenomeAxisTrack", function(x) as.character(strand(ranges(x))))

## Annotation Accessors ------------------------------------------------------

## Collapse  -----------------------------------------------------------------

#' @describeIn GenomeAxisTrack-class preprocess the track before plotting.
#' This will collapse overlapping track items based on the available resolution
#' and increase the width and height of all track objects to a minimum value
#' to avoid rendering issues. See collapsing for details.
#' @keywords internal
setMethod(
    "collapseTrack", signature(GdObject = "GenomeAxisTrack"),
    function(GdObject, min.width = 1, min.distance = 0, collapse = TRUE, diff = .pxResolution(coord = "x"), xrange) {

        ## Collapse overlapping ranges (less than minXDist space between them) including the associated attributes using
        ## "|" as separator. For both "strand" and "feature" we take the first available entry, which is not optimal but
        ## seems to be the sanest thing to do here...
        if (collapse) {
            GdObject <- GdObject[order(range(GdObject))]
            r <- ranges(GdObject)
            minXDist <- min.distance * diff
            r <- reduce(r, min.gapwidth = minXDist)
        }
        r <- .resize(r, min.width, diff)
        GdObject@range <- r
        return(GdObject)
    }
)

## Subset --------------------------------------------------------------------

#' @describeIn GenomeAxisTrack-class subset the items in the `GenomeAxisTrack` object.
#' This is essentially similar to subsetting of the `GRanges` object in the
#' `range` slot. For most applications, the subset method may be more appropriate.
#' @export
setMethod("[", signature(x = "GenomeAxisTrack"), function(x, i, j, ..., drop = TRUE) {
    x <- .deepCopyPars(x)
    x@range <- x@range[i, , drop = drop]
    return(x)
})

#' @describeIn GenomeAxisTrack-class plot subset all the contained tracks in an
#' `GenomeAxisTrack` by coordinates and sort if necessary.
#' @export
setMethod("subset", signature(x = "GenomeAxisTrack"), function(x, from = NULL, to = NULL, sort = FALSE, ...) {
    if (!length(x)) {
        return(x)
    }
    ranges <- .defaultRange(x, from = from, to = to)
    lsel <- end(x) < ranges["from"]
    rsel <- start(x) > ranges["to"]
    x <- x[!(lsel | rsel), ]
    if (sort) {
        x <- x[order(range(x)), ]
    }
    return(x)
})

## DrawGD --------------------------------------------------------------------

.expLabel <- function(GdObject, tckText, prune = FALSE) {
    tck <- tckText
    exponent <- if (is.null(.dpOrDefault(GdObject, "exponent"))) {
        exp <- 0
        while (all(abs(tck)[abs(tck) > 0] / 10^exp >= 1)) {
            exp <- exp + 3
        }
        exp - 3
    } else {
        max(0, .dpOrDefault(GdObject, "exponent"))
    }
    if (exponent > 0) {
        tckText <- tckText / (10^exponent)
    }
    if (prune) {
        tmp <- as.character(tckText)
        count <- max(nchar(gsub("*.\\.", "", tmp)))
        while (count > 1 && !any(duplicated(round(tckText, count)))) {
            count <- count - 1
        }
        tckText <- round(tckText, count + 1)
    }
    ## alternative might be to show "bp" also for small integers
    ## "0" = sprintf("%s bp", tckText),
    return(switch(as.character(exponent),
        "0" = sprintf("%i", as.integer(tckText)),
        "3" = sprintf("%s kb", tckText),
        "6" = sprintf("%s mb", tckText),
        "9" = sprintf("%s gb", tckText),
        lapply(tckText, function(x) bquote(paste(.(x), " ", 10^.(exponent))))
    ))
}

#' @describeIn GenomeAxisTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @export
setMethod("drawGD", signature("GenomeAxisTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...) {
    debug <- .dpOrDefault(GdObject, "debug", FALSE)
    if ((is.logical(debug) && debug) || debug == "prepare") {
        browser()
    }
    ## Nothing to do if the coordinate width is 0, so we can quit right away
    imageMap(GdObject) <- NULL
    if ((maxBase - minBase) == 0) {
        return(invisible(GdObject))
    }
    ## We start by setting up the drawing canvas
    if (subset) {
        GdObject <- subset(GdObject, from = minBase, to = maxBase)
    }
    xscale <- if (!.dpOrDefault(GdObject, "reverseStrand", FALSE)) c(minBase, maxBase) else c(maxBase, minBase)
    ## pushViewport(dataViewport(xData=c(minBase, maxBase), yscale=c(-1, 1), extension=0))
    pushViewport(dataViewport(xscale = xscale, yscale = c(-1, 1), extension = 0))
    ## Create a useful data range for the axis
    pres <- .pxResolution()
    curVp <- vpLocation()
    cex <- .dpOrDefault(GdObject, "cex", 0.8)
    lwd <- .dpOrDefault(GdObject, "lwd", 1)
    fontface <- .dpOrDefault(GdObject, "fontface", 1)
    add53 <- .dpOrDefault(GdObject, "add53", FALSE)
    add35 <- .dpOrDefault(GdObject, "add35", FALSE)
    lcex <- cex * 0.75
    textYOff <- pres["y"] * 3
    textXOff <- pres["x"] * 2
    endMargin <- if (add53 || add35) {
        abs(as.numeric(convertWidth(stringWidth("5'"), "native")) * lcex) + (textXOff * 2)
    } else {
        pres["x"] * 5
    }
    axRange <- c(minBase + endMargin, maxBase - endMargin)
    ## We want fixed vertical sizes for axis tracks to avoid akward stretching effects.
    color <- .dpOrDefault(GdObject, "col", "darkgray")[1]
    littleTicks <- .dpOrDefault(GdObject, "littleTicks", FALSE)
    dfact <- max(1, .dpOrDefault(GdObject, "distFromAxis", 1))
    labelPos <- .dpOrDefault(GdObject, "labelPos", "alternating")
    lwdAdd <- (lwd - 1) / 2
    tickHeight <- (ifelse(littleTicks, 2, 1) * 3 * dfact + lwdAdd) * pres["y"]
    ids <- as.character(values(GdObject)$id)
    showIds <- .dpOrDefault(GdObject, "showId", FALSE) && !is.null(ids) && !all(ids == "")
    rcex <- .dpOrDefault(GdObject, "cex.id", 0.7)
    rcol <- .dpOrDefault(GdObject, "col.id", "white")
    sep <- (if (length(GdObject)) {
        if (showIds) {
            max(1.5, ((max(as.numeric(convertHeight(stringHeight(ids), "native")) * rcex) + textYOff) / pres["y"]) / 2)
        } else {
            1.5
        }
    } else {
        1
    }) + lwdAdd
    pyOff <- pres["y"] * sep
    ## In prepare mode we just want to figure out the optimal size
    if (prepare) {
        nsp <- if (is.null(.dpOrDefault(GdObject, "scale"))) {
            (sum(tickHeight, pyOff * 2, textYOff * 2 + (as.numeric(convertHeight(stringHeight("1"), "native")) / 2) * cex) * 2 * 1.3) / pres["y"]
        } else {
            labelPos <- match.arg(labelPos, c("alternating", "revAlternating", "above", "below", "beside"))
            if (labelPos %in% c("above", "below")) {
                (sum(tickHeight, pyOff * 2 + (as.numeric(convertHeight(stringHeight("1"), "native")) / 2) * cex) * 2) / pres["y"]
            } else {
                (sum(tickHeight, pyOff * 2 + (as.numeric(convertHeight(stringHeight("1"), "native")) / 2) * cex)) / pres["y"]
            }
        }
        displayPars(GdObject) <- list("neededVerticalSpace" = nsp)
        popViewport(1)
        return(invisible(GdObject))
    }
    if ((is.logical(debug) && debug) || debug == "draw") {
        browser()
    }
    ## Plot range if there is any
    alpha <- .dpOrDefault(GdObject, "alpha", 1)

    ## in "scale" mode we just plot a simple scale and return ...
    scaleLen <- .dpOrDefault(GdObject, "scale")
    if (!is.null(scaleLen)) {
        len <- (maxBase - minBase + 1)
        if (scaleLen > len) {
            warning(sprintf("scale (%d) cannot be larger than plotted region %d - setting to ~5%\n", scaleLen, len))
            scaleLen <- 0.05
        }
        xoff <- len * 0.03 + minBase
        labelPos <- match.arg(labelPos, c("alternating", "revAlternating", "above", "below", "beside"))
        if (scaleLen <= 1 && scaleLen > 0) { # calculate and round the scale
            scaleLen <- len * scaleLen
            ex <- .dpOrDefault(GdObject, "exponent", floor(log10(scaleLen)))
            v <- round(scaleLen, -ex)
            if (v == 0) v <- scaleLen
        } else { # if the scale is an absolute value don't round
            ex <- .dpOrDefault(GdObject, "exponent", floor(log10(scaleLen)))
            v <- scaleLen
        }

        ## work out exponent/unit
        label <- .expLabel(GdObject, v)
        grid.lines(x = c(xoff, v + xoff), y = c(0, 0), default.units = "native", gp = gpar(col = color, lwd = lwd, alpha = alpha))
        grid.segments(
            x0 = c(xoff, v + xoff), y0 = c(0 - tickHeight, 0 - tickHeight),
            x1 = c(xoff, v + xoff), y1 = c(tickHeight, tickHeight, tickHeight),
            default.units = "native", gp = gpar(col = color, lwd = lwd, alpha = alpha)
        )
        z <- len * 0.01
        if (labelPos == "below") {
            grid.text(
                label = if (is.character(label)) label else label[[1]], x = xoff + v / 2, y = 0 - (tickHeight / 1.5 * dfact),
                just = c("center", "top"), gp = gpar(alpha = alpha, col = color, cex = cex, fontface = fontface), default.units = "native"
            )
        } else if (labelPos == "above") {
            grid.text(
                label = if (is.character(label)) label else label[[1]], x = xoff + v / 2, y = tickHeight / 1.5 * dfact, just = c("center", "bottom"),
                gp = gpar(alpha = alpha, col = color, cex = cex, fontface = fontface), default.units = "native"
            )
        } else {
            grid.text(
                label = if (is.character(label)) label else label[[1]], x = v + xoff + z, y = 0, just = c("left", "center"),
                gp = gpar(alpha = alpha, col = color, cex = cex, fontface = fontface), default.units = "native"
            )
        }
        popViewport(1)
        return(invisible(GdObject))
    }

    GdObject <- GdObject[end(GdObject) > axRange[1] & start(GdObject) < axRange[2]]
    if (length(GdObject)) {
        rfill <- .dpOrDefault(GdObject, "fill.range", "cornsilk3")
        rcolor <- .dpOrDefault(GdObject, "col.range", "cornsilk4")[1]
        diff <- .pxResolution(coord = "x")
        GdObject <- collapseTrack(GdObject, diff = diff, xrange = c(minBase, maxBase))
        start(GdObject) <- pmax(axRange[1], start(GdObject))
        end(GdObject) <- pmin(axRange[2], end(GdObject))
        coords <- cbind(start(GdObject), -0.1, end(GdObject), 0.1)
        grid.rect(
            x = start(GdObject), y = -pyOff, width = width(GdObject), height = pyOff * 2,
            default.units = "native", just = c("left", "bottom"), gp = gpar(col = rcolor, fill = rfill, alpha = alpha)
        )
        vals <- values(GdObject)
        if (showIds) {
            grid.text(ids,
                x = start(GdObject) + width(GdObject) / 2, y = 0,
                gp = gpar(col = rcol, cex = rcex, fontface = fontface),
                default.units = "native", just = c("center", "center")
            )
        }
        ## Calculate the coordinates for the image map
        map <- as.matrix(.getImageMap(coords))
        if (is.null(ids) || length(ids) == 0) {
            ids <- as.character(seq_len(nrow(map)))
        }
        rownames(map) <- make.unique(as.character(ids))
        tags <- lapply(
            list(title = ids, start = as.character(start(GdObject)), end = as.character(end(GdObject))),
            function(x) {
                names(x) <- rownames(map)
                x
            }
        )
        imageMap(GdObject) <- ImageMap(coords = map, tags = tags)
    }
    ## width<1, we can return here, no need for tick marks
    if (abs(diff(axRange)) < 1) {
        popViewport()
        return(invisible(GdObject))
    }
    ## We want two parallel lines with little hooks on the ends
    pyHook <- pres["y"] * (sep + 2 + lwdAdd)
    pxOff <- pres["x"] * 5
    grid.segments(
        x0 = rep(axRange[1], 2), y0 = c(-1, 1) * pyOff, x1 = rep(axRange[2], 2), y1 = c(-1, 1) * pyOff,
        default.units = "native", gp = gpar(col = color, alpha = alpha, lwd = lwd)
    )
    grid.segments(
        x0 = c(axRange[2] - pxOff, axRange[1]), y0 = c(pyHook, -pyOff),
        x1 = c(axRange[2], axRange[1] + pxOff), y1 = c(pyOff, -pyHook),
        default.units = "native", gp = gpar(col = color, alpha = alpha, lwd = lwd)
    )
    ## Here we plot the top level ticks
    tck <- .dpOrDefault(GdObject, "ticksAt", .ticks(axRange))
    tck <- tck[tck < axRange[2] - pxOff * 2 & tck > axRange[1] + pxOff * 2]
    y0t <- rep(c(1, -1) * pyOff, length(tck))[seq_along(tck)]
    y1t <- y0t + rep(c(tickHeight, -tickHeight), length(tck))[seq_along(tck)]
    labelPos <- match.arg(labelPos, c("alternating", "revAlternating", "above", "below", "beside"))
    y0t <- switch(labelPos,
        "alternating" = y0t,
        "revAlternating" = -y0t,
        "above" = abs(y0t),
        "below" = -abs(y0t),
        "beside" = y0t
    )
    y1t <- switch(labelPos,
        "alternating" = y1t,
        "revAlternating" = -y1t,
        "above" = abs(y1t),
        "below" = -abs(y1t),
        "beside" = y1t
    )
    ## ttck <- if(min(diff(tck))==1) tck+0.5 else tck # to align labels with ticks, shift by 0.5
    ttck <- tck + 0.5 # to align labels with ticks, shift by 0.5
    grid.segments(x0 = ttck, x1 = ttck, y0 = y0t, y1 = y1t, default.units = "native", gp = gpar(col = color, alpha = alpha, lwd = lwd, lineend = "square"))
    ## The top level tick labels
    label <- .expLabel(GdObject, tck)
    ylabs <- y1t + (ifelse(y1t > 0, 1, -1) * (textYOff + (as.numeric(convertHeight(stringHeight("1"), "native")) / 2) * cex))
    if (is.character(label)) {
        grid.text(
            label = label, x = ttck, y = ylabs, just = c("centre", "centre"),
            gp = gpar(cex = cex, fontface = fontface), default.units = "native"
        )
    } else {
        for (i in seq_along(label)) {
            grid.text(
                label = label[[i]], x = ttck[i], y = ylabs[i], just = c("centre", "centre"),
                gp = gpar(cex = cex, fontface = fontface), default.units = "native"
            )
        }
    }
    ## The second level ticks and labels if necessary
    if (.dpOrDefault(GdObject, "littleTicks", FALSE) && length(tck) > 1) {
        avSpace <- min(diff(tck))
        spaceFac <- 1.8
        spaceNeeded <- min(as.numeric(convertWidth(stringWidth(if (is.character(label)) label else "000000000"), "native")) / 2) * lcex * spaceFac
        nTcks <- (avSpace %/% spaceNeeded)
        if (nTcks %% 2 == 0) {
            nTcks <- nTcks - 1
        }
        btck <- tck
        if (!(minBase %in% btck)) {
            btck <- c(minBase, btck)
        }
        if (!(maxBase %in% btck)) {
            btck <- c(btck, maxBase)
        }
        y0lt <- y1lt <- ltck <- NULL
        for (i in seq_len(length(btck) - 1))
        {
            toFill <- btck[i:(i + 1)]
            ttck <- if (i == 1) rev(toFill[2] - (avSpace / nTcks) * seq_len(nTcks - 1)) else toFill[1] + (avSpace / nTcks) * seq_len(nTcks - 1)
            ltck <- c(ltck, ttck)
            ord <- if (i == 1) {
                if (y0t[1] > 0) c(1, -1) else c(-1, 1)
            } else if (y0t[i - 1] < 0) c(1, -1) else c(-1, 1)
            y0 <- rep(ord * pyOff, length(ttck))[seq_along(ttck)]
            y1 <- y0 + rep(ord * tickHeight / 2, length(ttck))[seq_along(ttck)]
            y0lt <- c(y0lt, switch(labelPos,
                "alternating" = y0,
                "revAlternating" = y0,
                "above" = abs(y0),
                "below" = -abs(y0)
            ))
            y1lt <- c(y1lt, switch(labelPos,
                "alternating" = y1,
                "revAlternating" = y1,
                "above" = abs(y1),
                "below" = -abs(y1)
            ))
        }
        endPadding <- pres["x"] * 15
        sel <- ltck > min(tck, axRange + endPadding) & ltck < max(tck, axRange - endPadding)
        if (length(ltck[sel]) && min(diff(tck)) > nTcks) {
            grid.segments(
                x0 = ltck[sel], x1 = ltck[sel], y0 = y0lt[sel], y1 = y1lt[sel], default.units = "native",
                gp = gpar(col = color, alpha = alpha, lwd = lwd, lineend = "square")
            )
            llabel <- .expLabel(GdObject, ltck[sel], prune = TRUE)
            ytlabs <- y1lt + (ifelse(y1lt > 0, 1, -1) * (textYOff + (as.numeric(convertHeight(stringHeight("1"), "native")) / 2) * lcex))
            if (is.character(label)) {
                grid.text(
                    label = llabel, x = ltck[sel], y = ytlabs[sel], just = c("centre", "centre"),
                    gp = gpar(cex = lcex, fontface = fontface), default.units = "native"
                )
            } else {
                for (i in seq_along(llabel)) {
                    grid.text(
                        label = llabel[[i]], x = ltck[sel][i], y = ytlabs[sel][i], just = c("centre", "centre"),
                        gp = gpar(cex = lcex, fontface = fontface), default.units = "native"
                    )
                }
            }
        }
    }
    ## The direction indicators
    rev <- .dpOrDefault(GdObject, "reverseStrand", FALSE)
    p5 <- expression("5'")
    p3 <- expression("3'")
    if (add53) {
        grid.text(
            label = p5, x = min(axRange) - textXOff, y = pyOff,
            just = c(ifelse(rev, "left", "right"), "bottom"), gp = gpar(cex = cex * .75, fontface = fontface),
            default.units = "native"
        )
        grid.text(
            label = p3, x = max(axRange) + textXOff, y = pyOff,
            just = c(ifelse(rev, "right", "left"), "bottom"), gp = gpar(cex = cex * .75, fontface = fontface),
            default.units = "native"
        )
    }
    if (add35) {
        grid.text(
            label = p3, x = axRange[1] - textXOff, y = -pyOff,
            just = c(ifelse(rev, "left", "right"), "top"), gp = gpar(cex = cex * .75, fontface = fontface),
            default.units = "native"
        )
        grid.text(
            label = p5, x = axRange[2] + textXOff, y = -pyOff,
            just = c(ifelse(rev, "right", "left"), "top"), gp = gpar(cex = cex * 0.75, fontface = fontface),
            default.units = "native"
        )
    }
    popViewport()
    return(invisible(GdObject))
})


## Show ----------------------------------------------------------------------

#' @describeIn GenomeAxisTrack-class Show method.
#' @export
setMethod(
    "show", signature(object = "GenomeAxisTrack"),
    function(object) {
        cat(sprintf("Genome axis '%s'\n", names(object)))
        if (.dpOrDefault(object, "add53", FALSE)) {
            cat("5->3 label is set\n")
        }
        if (.dpOrDefault(object, "add35", FALSE)) {
            cat("3->5 label is set\n")
        }
        if (.dpOrDefault(object, "littleTicks", FALSE)) {
            cat("littleTicks label is set\n")
        }
        if (length(object)) {
            cat("There are annotated axis regions:\n")
            print(ranges(object))
        }
    }
)

library(Gviz)

## -------------------------------------------------
## GenomeAxisTrack class
## -------------------------------------------------
ax <- GenomeAxisTrack()
dev.new(height=1.5)
par(ask=TRUE)

## basic plotting
plotTracks(ax, from=1, to=100)

## automatic label setting
plotTracks(ax, from=1, to=10000)
plotTracks(ax, from=1, to=10000000)
plotTracks(ax, from=1, to=10000000000)

## manual label setting
plotTracks(ax, from=1, to=10000000000, exponent=8)
plotTracks(ax, from=-3000000, to=10000000, exponent=4)

## direction indicators
plotTracks(ax, from=1, to=10000, add53=TRUE)
plotTracks(ax, from=1, to=10000, add53=TRUE, add35=TRUE)

## text size
plotTracks(ax, from=1, to=10000, add53=TRUE, add35=TRUE, cex=1.5)
plotTracks(ax, from=1, to=10000, add53=TRUE, add35=TRUE, cex=0.6)

## second level labels
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE)

## axis spacing
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, distFromAxis=2)

## ranges
library(IRanges)
ax <- GenomeAxisTrack(range=IRanges(start=c(1,670), end=c(321, 912)), id=c("foo", "bar"))
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE)

## range annotion
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, showId=TRUE)
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, showId=TRUE, cex.id=1.5)

## backgrounds
plotTracks(ax, from=1, to=1000, background.panel="#DAE3E6")
plotTracks(ax, from=1, to=1000, background.title="#DAE3E6")

## frame
plotTracks(ax, from=1, to=1000, frame=TRUE)
plotTracks(ax, from=1, to=1000, background.title="#DAE3E6", col.frame="darkblue", background.panel="#FFDDB5", frame=TRUE)

## colors
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, showId=TRUE, col="darkgreen")
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, showId=TRUE, fontcolor="darkred")
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, showId=TRUE, col.id="darkblue")
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, showId=TRUE, fill.range="salmon2")

## font settings
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, showId=TRUE, fontfamily="serif", fontface=3)

## line width
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, lwd=3)

## tick mark orientation
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, labelPos="revAlternating")
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, labelPos="above")
plotTracks(ax, from=1, to=1000, add53=TRUE, add35=TRUE, littleTicks=TRUE, labelPos="below")


## -------------------------------------------------
## IdeogramTrack class
## -------------------------------------------------

## basic plotting
plotTracks(id, from=10000000, to=20000000)

## colors
plotTracks(id, from=10000000, to=20000000, col="darkblue", fill="lightblue")
plotTracks(id, from=10000000, to=20000000, background.panel="lightgreen")
plotTracks(id, from=10000000, to=20000000, fontcolor="red", background.title="lightgreen")

## line type
plotTracks(id, from=10000000, to=20000000, lty="dotted", lwd=2)

## font settings
plotTracks(id, from=10000000, to=20000000, fontsize=8)
plotTracks(id, from=10000000, to=20000000, cex=2)
plotTracks(id, from=10000000, to=20000000, fontface=3, fontcolor="red", fontfamily="serif")

## shape
plotTracks(id, from=10000000, to=20000000, bevel=0)


## ------------------
## Object creation:
## ------------------


## Annotation track:
## As individual arguments
annTrack <- AnnotationTrack(chromosome=7, feature="test", group=c(1,2,1), 
                            start=c(2000000, 2070000, 2100000), ID=paste("annTrack item", 1:3),  
                            end=c(2050000, 2130000, 2150000), genome="hg19", name="annTrack", 
                            stacking="squish", strand=c("-", "+", "-"), size=1.5)
## Or as data.frame
annTrack2 <- AnnotationTrack(data.frame(start=c(2000000, 2070000, 2100000), 
                                        ID=paste("annTrack item", 1:3), end=c(2050000, 2130000, 2170000), 
                                        feature="test", group=c("Group1","Group2","Group1"), strand=c("-", "+", "-")),
                             genome="hg19", name="annTrack2", 
                             stacking="squish",chromosome=7, size=1.5, showId=TRUE)
names(annTrack) <- "testAnnTrack"
plotTracks(list(annTrack, annTrack2), extend.left=20000)

## The track name


from <- 2000000
to <- 2100000

it <- IdeogramTrack(genome="hg19", name="chromosome 4", chromosome=4)
plotTracks(list(it, annTrack2))


## GeneRegion track
geneTrack <- BiomartGeneRegionTrack(chromosome=4, start=2000000, end=2100000, genome="hg19",
                                    stacking="squish", strand="+-", name="geneTrack")
## Todo: filter testing
plotTracks(geneTrack)

gt1 <- BiomartGeneRegionTrack(chromosome=4, start=2000000, end=2100000, genome="hg19",
                              stacking="squish", strand="+", name="+ Strand")
gt2 <- BiomartGeneRegionTrack(chromosome=4, start=2000000, end=2100000, genome="hg19",
                              stacking="squish", strand="-", name="- Strand")
gt3 <- BiomartGeneRegionTrack(chromosome=4, start=2000000, end=2100000, genome="hg19",
                              stacking="squish", strand="+-", name="RefSeq", filter=Gviz:::.refseqFilter)
plotTracks(list(geneTrack, gt1, gt2, gt3))

displayPars(gt1) <- list(showId=TRUE, fontsize=8)
displayPars(gt2) <- list(showId=TRUE, fontsize=8)
displayPars(gt3) <- list(showId=TRUE, fontsize=8)
plotTracks(list(gt1, gt2, gt3), extend.left=10000)




ga <- GenomeAxisTrack()
ga <- GenomeAxisTrack(range=GenomicRanges::GRanges(seqnames=letters[1:3], range=IRanges(start=c(2045000, 2061000, 2151000), end=c(2050000, 2066000, 2221000))),
                      add53=TRUE, add35=TRUE)
plotTracks(list(gt1, ga, gt2, gt3), extend.left=10000)
displayPars(ga) <- list(littleTicks=TRUE)
plotTracks(list(annTrack2, gt1, ga, gt2, gt3), extend.left=10000)



bases <- seq(min(start(range(gt2))), max(end(range(gt2))), len=50)
start <- bases-(min(diff(bases))/5)*runif(50,1,3)
end <- bases+(min(diff(bases))/5)*runif(50,1,3)
data <- matrix(rnorm(length(bases)*6)+rep(runif(length(bases), -4,4), each=6), nrow=6)

dt <- DataTrack(start=start, end=end, data=data)

plotTracks(list(gt2, ga, dt), extend.left=10000)


dtp <- DataTrack(start=start, end=end, data=data, type="p", name="points")
dtl <- DataTrack(start=start, end=end, data=data, type="l", name="lines")
dtb <- DataTrack(start=start, end=end, data=data, type="b", name="both")
dta <- DataTrack(start=start, end=end, data=data, type="a", name="average")
dts <- DataTrack(start=start, end=end, data=data, type="s", name="steps")
dtg <- DataTrack(start=start, end=end, data=data, type="g", name="grid")
dtr <- DataTrack(start=start, end=end, data=data, type="r", name="regression")
dtsm <- DataTrack(start=start, end=end, data=data, type="smooth", name="loess")
dth <- DataTrack(start=start, end=end, data=data, type="h", name="hist")
dtm <- DataTrack(start=start, end=end, data=data, type="mountain", name="mountain")
dtbp <- DataTrack(start=start, end=end, data=data, type="boxplot", name="boxplot")
dthi <- DataTrack(start=start, end=end, data=data, type="histogram", name="histogram")

plotTracks(list(dtp, dtl, dtb, dta, dts, dtg, dtr, dtsm, dth))

plotTracks(list(dtm, dtbp, dthi))

trunc <- function(vals)
{
    trunc <- c(30,70)
    vals[vals<trunc[1]] <- trunc[1]
    vals[vals>trunc[2]] <- trunc[2]
    return(vals)
}

ut1 <- UcscTrack(genome="hg19", chromosome=4, track="GC", from=1985000, to=2150000,
                 trackType="DataTrack", start="start", end="end", data="score", type="gradient",
                 transformation=trunc, window="auto")

ut2 <- UcscTrack(genome="hg19", chromosome=4, track="Ensembl", from=1900000, to=2300000,
                 table="ensGene", trackType="Gene", rstarts="exonStarts", rends="exonEnds", gene="name2",
                 symbol="name2", transcript="name", strand="strand", ID="name",
                 feature="default", exonList=TRUE, showId=TRUE, fontsize=8)


ut4 <- UcscTrack(genome="canFam2", chromosome=10, track="Other RefSeq", from=13910391-10000, to=13956580+10000,
                 trackType="Gene", rstarts="exonStarts", rends="exonEnds", gene="name2",
                 symbol="name2", transcript="name", strand="strand", ID="name",
                 feature="default", exonList=TRUE, showId=TRUE, fontsize=8)


ut3 <-  UcscTrack(genome="hg19", chromosome=4, track="Repeat", from=1900000, to=2300000,
                  trackType="Annotation", start="genoStart", end="genoEnd", ID="repName",
                  feature="repClass", strand="strand", shape="box", stacking="dense", size=0.5, col="black")

ut4 <- UcscTrack(genome="hg19", chromosome=4, track="knownGene", from=1900000, to=2300000,
                 trackType="Gene", rstarts="exonStarts", rends="exonEnds", gene="name",
                 symbol="name", transcript="name", strand="strand", ID="name",
                 feature="default", exonList=TRUE, showId=TRUE, fontsize=8)

plotTracks(list(ut1, ga, ut2), from=1950000, to=2050000)

plotTracks(list(it, geneTrack, ga, ut2), fontsize=10, showId=TRUE, from=1985000, to=2150000)

plotTracks(list(ut1, ga, ut2, ut3), type="h", from=1985000, to=2150000)


                  


ut4 <-  UcscTrack(genome="hg19", chromosome=4, track="snp131", from=1900000, to=2200000,
                  trackType="Annotation", start="chromStart", end="chromEnd", ID="name",
                  feature="func", strand="strand", shape="box", stacking="squish", size=0.6, fill="black",
                  "coding-synon"="green", nonsense="red", missense="red", frameshift="red",
                  "untranslated-3"="blue", "untranslated-5"="blue", unknown="gray")

plotTracks(list(it, ut1, ga, ut2, ut4, ut3), type="h", from=1985000, to=2150000)

#' @include GdObject-class.R
NULL

## ReferenceTrack-class ------------------------------------------------------

#' ReferenceTrack class and methods
#'
#'
#' A class allow for on-demand streaming of data off the file system.
#'
#' The `availableDefaultMappings` function can be used to find out whether
#' the package defines a mapping scheme between one of the many supported input
#' file types and the metadata columns of the tracks' `GRanges` objects.
#'
#' @name ReferenceTrack-class
#'
#' @slot stream Object of class function. The import function to stream data
#' of the file system. Needs to be able to handle the two mandatory arguments
#' `file` (a `character` containing a valid file path) and `selection`
#' (a `GRanges` object with the genomic region to plot).
#' @slot reference Object of class "character", the path to the file
#' containing the data.
#' @slot mapping Object of class `list`, a default mapping between the
#' metadata columns of the returned `GRanges` object from the import function
#' and the `elemenMetadata` columns that make up the final track object.
#' @slot args Object of class `list`, the passed in constructor arguments
#' during object instantiation. Those will be needed when fetching the data
#' in order to fill all necessary slots.
#' @slot defaults Object of class `list`, the relevant default values to be
#' used when neither mapping nor args provides the necessary information.
#'
#' @return
#' Constructor functions of `AnnotationTrack`, `DataTrack`, `SequenceTrack`
#' and `AlignmentsTrack`` can create a special object of corresponding
#' `Reference*Track` subclass with pointer to the referenced
#' file.
#'
#' @return A virtual class: No objects may be created from it.
#'
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#' # This is a reference class, below example from AlignmentsTrack
#'
#' afrom <- 2960000
#' ato <- 3160000
#' alTrack <- AlignmentsTrack(system.file(
#'     package = "Gviz", "extdata",
#'     "gapped.bam"
#' ), isPaired = TRUE)
#' plotTracks(alTrack, from = afrom, to = ato, chromosome = "chr12")
#' @exportClass ReferenceTrack
setClass("ReferenceTrack",
    representation = representation("VIRTUAL",
        stream = "function",
        reference = "character",
        mapping = "list",
        args = "list",
        defaults = "list"
    ),
    prototype = prototype(
        stream = function(x, selection) {},
        reference = "~",
        mapping = list()
    ),
    validity = function(object) {
        msg <- NULL
        if (!all(c("file", "selection") %in% names(formals(object@stream)))) {
            msg <- "The streaming function in the 'stream' slot needs to define two arguments, 'file' and 'selection'"
        }
        if (!file.exists(object@reference)) {
            msg <- c(msg, sprintf("The referenced file '%s' does not exist", object@reference))
        }
        return(if (is.null(msg)) TRUE else msg)
    }
)

## Initialize ----------------------------------------------------------------

#' @describeIn ReferenceTrack-class Initialize.
#' @export

setMethod("initialize", "ReferenceTrack", function(.Object, stream, reference, mapping = list(),
                                                   args = list(), defaults = list()) {
    .Object@stream <- stream
    .Object@reference <- reference
    .Object@mapping <- mapping
    .Object@args <- args
    .Object@defaults <- defaults
    validObject(.Object)
    return(.Object)
})

## .buildRange ---------------------------------------------------------------
##
## Helper methods to build a GRanges object from the input arguments.
##
## Coordinates for grouped elements may be passed in as comma-separated values
## (e.g. "1,5,9"), in which case we need to split and convert to numeric.
## This also implies that the additional arguments (like feature, group, etc.)
## have to be replicated accordingly. We handle this by passing along the repeat
## vector 'by' to the numeric method below.

## For character scalars the data need to be extracted from a file and we have to deal with parser functions
## and column assignments here.
setMethod(
    ".buildRange", signature("character"),
    function(range, importFun = NULL, trackType, stream = FALSE, args, defaults, autodetect = is.null(importFun), ...) {
        .checkClass(range, "character", 1)
        .checkClass(importFun, c("NULL", "function"), mandatory = FALSE)
        .checkClass(stream, "logical", 1)
        ## We first check for the default column mapping and whether this is a streaming file
        defMap <- .defaultVarMap(tolower(.fileExtension(range)), trackType, stream, !autodetect)
        isStream <- !is.null(defMap[[".stream"]]) && defMap[[".stream"]]
        defMap[[".stream"]] <- NULL
        if (!isStream) {
            data <- if (is.null(importFun)) {
                .registerImportFun(range)
            } else {
                if (!"file" %in% names(formals(importFun))) {
                    stop("The user-defined import function needs to define a 'file' argument")
                }
                importFun(range)
            }
            if (!is(data, "GRanges")) {
                stop(
                    "The import function did not provide a valid GRanges object. Unable to build track from file '",
                    range, "'"
                )
            }
            if (trackType == "DataTrack") {
                ## For data tracks we take all numeric data columns regardless of any mapping
                mc <- .prepareDtData(as.data.frame(mcols(data)), length(data))
                mcols(data) <- t(mc)
            } else {
                ## For the rest we use the mapping as provided by the constructor if available
                cmap <- .resolveColMapping(data, args, defMap)
                args <- cmap$args
                data <- cmap$data
            }
            args[["chromosome"]] <- as.character(seqnames(data))
            args[["strand"]] <- as.character(strand(data))
            return(.buildRange(range = data, args = args, defaults = defaults, trackType = trackType, ...))
        } else {
            if (trackType != "DataTrack") {
                for (i in names(defMap)) {
                    if (is.character(args[[i]]) && length(args[[i]]) == 1) {
                        defMap[[i]] <- args[[i]]
                    }
                }
            }
            return(list(
                reference = path.expand(range), mapping = defMap,
                stream = if (is.null(importFun)) .registerImportFun(range) else importFun
            ))
        }
    }
)

## Show ----------------------------------------------------------------------
## A helper function to plot general information about a ReferenceTrack
.referenceTrackInfo <- function(object, type) {
    message(sprintf(
        "%s '%s'\n| genome: %s\n| active chromosome: %s\n| referenced file: %s",
        type,
        names(object),
        genome(object),
        chromosome(object),
        object@reference
    ))
    if (length(object@mapping) && type != "ReferenceDataTrack") {
        message(sprintf("| mapping: %s\n", paste(names(object@mapping), as.character(object@mapping), sep = "=", collapse = ", ")))
    }
}

# 
# 
# 

.gassign <- function(what) assignInNamespace(ns="Gviz", x=what, 
                                             value=get(what, envir=globalenv()))

assignInNamespace(ns="Gviz", x=".computeGroupRange", value=.computeGroupRange)
.dpOrDefault <- Gviz:::.dpOrDefault
.getStringDims <- Gviz:::.getStringDims
.fontGp <- Gviz:::.fontGp

assignInNamespace(ns="Gviz", x=".barsAndLabels", value=.barsAndLabels)
.pxResolution <- Gviz:::.pxResolution
.getBiotypeColor = Gviz:::.getBiotypeColor

assignInNamespace(value=.dpOrDefault, x=".dpOrDefault", ns="Gviz")
assignInNamespace(value=.dpOrDefaultFont, x=".dpOrDefaultFont", ns="Gviz")
assignInNamespace(value=.fontGp, x=".fontGp", ns="Gviz")

library(Biobase)
assignInNamespace(ns="Gviz", x=".defaultRange", value=.defaultRange)
.dpOrDefault <- Gviz:::.dpOrDefault

assignInNamespace(ns="Gviz", x=".arrowBar", value=.arrowBar)
.pxResolution <- Gviz:::.pxResolution

assignInNamespace(ns="Gviz", x=".boxes", value=.boxes)
.dpOrDefault <- Gviz:::.dpOrDefault
.getBiotypeColor = Gviz:::.getBiotypeColor
.getAnn = Gviz:::.getAnn
.getImageMap <- Gviz:::.getImageMap

assignInNamespace(ns="Gviz", x=".filledBoxes", value=.filledBoxes)
.handleComposite <- Gviz:::.handleComposite

assignInNamespace(ns="Gviz", x=".filledArrow", value=.filledArrow)
.handleComposite <- Gviz:::.handleComposite

assignInNamespace(ns="Gviz", x=".updatePars", value=.updatePars)

assignInNamespace(ns="Gviz", x=".handleComposite", value=.handleComposite)

assignInNamespace(ns="Gviz", x=".setupTextSize", value=.setupTextSize)
.dpOrDefault <- Gviz:::.dpOrDefault
vpLocation <- Gviz:::vpLocation
.verticalSpace <- Gviz:::.verticalSpace
.needsAxis <- Gviz:::.needsAxis
.PLOT_TYPES <- Gviz:::.PLOT_TYPES

assignInNamespace(ns="Gviz", x="plotTracks", value=plotTracks)
.whichStrand <- Gviz:::.whichStrand
.recChromosome <- Gviz:::.recChromosome
.needsTitle <- Gviz:::.needsTitle
.fontGp <- Gviz:::.fontGp
drawAxis <- Gviz:::drawAxis
.getImageMap <- Gviz:::.getImageMap
drawGrid <- Gviz:::drawGrid
ImageMap <- Gviz:::ImageMap

## This is all needed for sourcing plotTracks
.defaultRange <- Gviz:::.defaultRange
setStacks <- Gviz:::setStacks
vpLocation <- Gviz:::vpLocation
.setupTextSize <- Gviz:::.setupTextSize
.needsAxis <- Gviz:::.needsAxis
drawAxis <- Gviz:::drawAxis
.getImageMap <- Gviz:::.getImageMap
drawGrid <- Gviz:::drawGrid
ImageMap <- Gviz:::ImageMap
.needsTitle <- Gviz:::.needsTitle
.whichStrand <- Gviz:::.whichStrand
.recChromosome <- Gviz:::.recChromosome


assignInNamespace(ns="Gviz", x=".needsTitle", value=.needsTitle)
assignInNamespace(ns="Gviz", x=".needsAxis", value=.needsAxis)

library(Gviz)
data(cyp2b10)
dt<-GeneRegionTrack(cyp2b10)
plotTracks(dt)

plotTracks(dt, showId=TRUE, just.group="left", cex.group=0.8, 
           reverseStrand=FALSE, title.width=1)



plotTracks(c(dt), just.group="left", cex.group=1, reverseStrand=FALSE, 
           title.width=1, geneSymbols="symbol", fontface.group="bold.italic", 
           fontcolor.group="red", showExonId=FALSE, fontcolor.item=1, 
           from=25897620-500, to=25897855, fontfamily="serif", 
           fontfamily.item="sans", showId=TRUE, just.group="left")


plotTracks(c(dt), showId=TRUE, alpha=1, lwd=1, col=1, debug="draw", 
           collapseTranscripts=FALSE)


plotTracks(dt, col=1, shape=c("smallArrow", "arrow"))


plotTracks(list(GenomeAxisTrack(), dt), transcriptAnnotation="symbol",
           fill="#FFD58A", just.group="below", extend.left=-0.9)

## This is broken:
plotTracks(list(GenomeAxisTrack(), dt), fill="#FFD58A", showId=TRUE, 
           just.group="below", extend.left=-0.99, extend.right=-0.99)


st <- c(2000000, 2070000, 2100000, 2160000)
ed <- c(2050000, 2130000, 2150000, 2170000)
str <- c("-", "+", "-", "-")
gr <- c("Group1","Group2","Group1", "Group3")

annTrack <- AnnotationTrack(start=st, end=ed, strand=str, chromosome=7, 
                            genome="hg19", feature="test",
                            group=gr, id=paste("annTrack item", 1:4), 
                            name="generic annotation", stacking="squish")
plotTracks(annTrack)

## ============================================================================================================================
## All generic functions defined in the package
## ============================================================================================================================

## Display parameter accessors
setGeneric("setPar", function(x, value, ...) standardGeneric("setPar"))
setGeneric("displayPars<-",
    signature = c("x", "value"),
    function(x, recursive = FALSE, value) standardGeneric("displayPars<-")
)
setGeneric("getPar", def = function(x, name, ...) standardGeneric("getPar"))
setGeneric("displayPars", function(x, name, ...) standardGeneric("displayPars"))

## Annotation accessors
setGeneric("gene", function(GdObject, ...) standardGeneric("gene"))
setGeneric("gene<-", function(GdObject, value) standardGeneric("gene<-"))
setGeneric("symbol", function(GdObject, ...) standardGeneric("symbol"))
setGeneric("symbol<-", function(GdObject, value) standardGeneric("symbol<-"))
setGeneric("transcript", function(GdObject, ...) standardGeneric("transcript"))
setGeneric("transcript<-", function(GdObject, value) standardGeneric("transcript<-"))
setGeneric("exon", function(GdObject, ...) standardGeneric("exon"))
setGeneric("exon<-", function(GdObject, value) standardGeneric("exon<-"))
setGeneric("feature", function(GdObject, ...) standardGeneric("feature"))
setGeneric("feature<-", function(GdObject, value) standardGeneric("feature<-"))
setGeneric("group", function(GdObject, ...) standardGeneric("group"))
setGeneric("group<-", function(GdObject, value) standardGeneric("group<-"))
setGeneric("identifier", function(GdObject, ...) standardGeneric("identifier"))
setGeneric("identifier<-", function(GdObject, value) standardGeneric("identifier<-"))


## General accessors
setGeneric("chromosome", function(GdObject, ...) standardGeneric("chromosome"))
setGeneric("chromosome<-", function(GdObject, value) standardGeneric("chromosome<-"))
## setGeneric("[")
setGeneric("position", function(GdObject, ...) standardGeneric("position"))
setGeneric("imageMap", function(GdObject, ...) standardGeneric("imageMap"))
setGeneric("imageMap<-", function(GdObject, value) standardGeneric("imageMap<-"))
## setGeneric("subset",  function(x, ...) standardGeneric("subset"))
setGeneric("coords", function(ImageMap, ...) standardGeneric("coords"))
setGeneric("tags", function(ImageMap, ...) standardGeneric("tags"))

## Prepare tracks for plotting
setGeneric("consolidateTrack", function(GdObject, ...) standardGeneric("consolidateTrack"))
setGeneric("collapseTrack", function(GdObject, ...) standardGeneric("collapseTrack"))
setGeneric("stacking", function(GdObject, ...) standardGeneric("stacking"))
setGeneric("stacking<-", function(GdObject, value) standardGeneric("stacking<-"))
setGeneric("stacks", function(GdObject, ...) standardGeneric("stacks"))
setGeneric("setStacks", function(GdObject, ...) standardGeneric("setStacks"))
setGeneric("setCoverage", function(GdObject, ...) standardGeneric("setCoverage"))

## Plotting methods
setGeneric("drawAxis", function(GdObject, ...) standardGeneric("drawAxis"))
setGeneric("drawGrid", function(GdObject, ...) standardGeneric("drawGrid"))
setGeneric("drawGD", function(GdObject, ...) standardGeneric("drawGD"))

## We may need those for dispatch in the name space
## if(!isGeneric("lapply"))
##    setGeneric("lapply")
## if(!isGeneric("sapply"))
##    setGeneric("sapply")
## if(!isGeneric("head"))
##    setGeneric("head")
## if(!isGeneric("split"))
##    setGeneric("split")

## Internal methods
setGeneric(".buildRange", function(range, start, end, width, ...) standardGeneric(".buildRange"))

#' @include AnnotationTrack-class.R
NULL

## GeneRegionTrack Class -----------------------------------------------------


#' GeneRegionTrack class and methods
#'
#'
#' A class to hold gene model data for a genomic region.
#'
#'
#' A track containing all gene models in a particular region. The data are
#' usually fetched dynamially from an online data store, but it is also
#' possible to manully construct objects from local data. Connections to
#' particular online data sources should be implemented as sub-classes, and
#' \code{GeneRegionTrack} is just the commone denominator that is being used
#' for plotting later on. There are several levels of data associated to a
#' \code{GeneRegionTrack}:
#'
#' \describe{
#'
#' \item{exon level:}{identifiers are stored in the exon column of the
#' \code{\linkS4class{GRanges}} object in the \code{range} slot. Data may be
#' extracted using the \code{exon} method.}
#'
#' \item{transcript level:}{identifiers are stored in the transcript column of
#' the \code{\linkS4class{GRanges}} object. Data may be extracted using the
#' \code{transcript} method.}
#'
#' \item{gene level:}{identifiers are stored in the gene column of the
#' \code{\linkS4class{GRanges}} object, more human-readable versions in the
#' symbol column. Data may be extracted using the \code{gene} or the
#' \code{symbol} methods.}
#'
#' \item{transcript-type level:}{information is stored in the feature column of
#' the \code{\linkS4class{GRanges}} object. If a display parameter of the same
#' name is specified, the software will use its value for the coloring.}
#'
#' }
#'
#' \code{GeneRegionTrack} objects also know about coding regions and non-coding
#' regions (e.g., UTRs) in a transcript, and will indicate those by using
#' different shapes (wide boxes for all coding regions, thinner boxes for
#' non-coding regions). This is archived by setting the \code{feature} values
#' of the object for non-coding elements to one of the options that are
#' provided in the \code{thinBoxFeature} display parameters. All other elements
#' are considered to be coding elements.
#'
#' @name GeneRegionTrack-class
#'
#' @param range
#'
#' An optional meta argument to handle the different input types. If the
#' \code{range} argument is missing, all the relevant information to create the
#' object has to be provided as individual function arguments (see below).
#'
#' The different input options for \code{range} are:
#'
#' \describe{
#'
#' \item{A \code{TxDb} object:}{ all the necessary gene model information
#' including exon locations, transcript groupings and associated gene ids are
#' contained in \code{TxDb} objects, and the coercion between the two is almost
#' completely automated. If desired, the data to be fetched from the
#' \code{TxDb} object can be restricted using the constructor's
#' \code{chromosome}, \code{start} and \code{end} arguments. See below for
#' details. A direct coercion method \code{as(obj, "GeneRegionTrack")} is also
#' available. A nice added benefit of this input option is that the UTR and
#' coding region information that is part of the original \code{TxDb} object is
#' retained in the \code{GeneRegionTrack}.}
#'
#' \item{A \code{GRanges} object:}{ the genomic ranges for the
#' \code{GeneRegion} track as well as the optional additional metadata columns
#' \code{feature}, \code{transcript}, \code{gene}, \code{exon} and
#' \code{symbol} (see description of the individual function parameters below
#' for details). Calling the constructor on a \code{GRanges} object without
#' further arguments, e.g.  \code{GeneRegionTrack(range=obj)} is equivalent to
#' calling the coerce method \code{as(obj, "GeneRegionTrack")}.}
#'
#' \item{A \code{GRangesList} object:}{ this is very similar to the previous
#' case, except that the grouping information that is part of the list
#' structure is preserved in the \code{GeneRegionTrack}. I.e., all the elements
#' within one list item receive the same group id. For consistancy, there is
#' also a coercion method from \code{GRangesLists} \code{as(obj,
#' "GeneRegionTrack")}. Please note that unless the necessary information about
#' gene ids, symbols, etc. is present in the individual \code{GRanges} meta
#' data slots, the object will not be particularly useful, because all the
#' identifiers will be set to a common default value.}
#'
#' \item{An \code{\linkS4class{IRanges}} object:}{ almost identical to the
#' \code{GRanges} case, except that the chromosome and strand information as
#' well as all additional data has to be provided in the separate
#' \code{chromosome}, \code{strand}, \code{feature}, \code{transcript},
#' \code{symbol}, \code{exon} or \code{gene} arguments, because it can not be
#' directly encoded in an \code{IRanges} object. Note that only the former two
#' are mandatory (if not provided explicitely the more or less reasonable
#' default values \code{chromosome=NA} and \code{strand=*} are used, but not
#' providing information about the gene-to-transcript relationship or the
#' human-readble symbols renders a lot of the class' functionality useles.}
#'
#' \item{A \code{data.frame} object:}{ the \code{data.frame} needs to contain
#' at least the two mandatory columns \code{start} and \code{end} with the
#' range coordinates. It may also contain a \code{chromosome} and a
#' \code{strand} column with the chromosome and strand information for each
#' range. If missing, this information will be drawn from the constructor's
#' \code{chromosome} or \code{strand} arguments. In addition, the
#' \code{feature}, \code{exon}, \code{transcript}, \code{gene} and
#' \code{symbol} data can be provided as columns in the \code{data.frame}. The
#' above comments about potential default values also apply here.}
#'
#' \item{A \code{character} scalar:}{ in this case the value of the
#' \code{range} argument is considered to be a file path to an annotation file
#' on disk. A range of file types are supported by the \code{Gviz} package as
#' identified by the file extension. See the \code{importFunction}
#' documentation below for further details.}
#'
#' }
#' @template GeneRegionTrack-class_param
#' @return
#'
#' The return value of the constructor function is a new object of class
#' \code{GeneRegionTrack}.
#' @section Objects from the class:
#'
#' Objects can be created using the constructor function
#' \code{GeneRegionTrack}.
#'
#' @author Florian Hahne, Steve Lianoglou
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#'
#'
#' ## The empty object
#' GeneRegionTrack()
#'
#' ## Load some sample data
#' data(cyp2b10)
#'
#' ## Construct the object
#' grTrack <- GeneRegionTrack(
#'     start = 26682683, end = 26711643,
#'     rstart = cyp2b10$start, rends = cyp2b10$end, chromosome = 7, genome = "mm9",
#'     transcript = cyp2b10$transcript, gene = cyp2b10$gene, symbol = cyp2b10$symbol,
#'     feature = cyp2b10$feature, exon = cyp2b10$exon,
#'     name = "Cyp2b10", strand = cyp2b10$strand
#' )
#'
#' ## Directly from the data.frame
#' grTrack <- GeneRegionTrack(cyp2b10)
#'
#' ## From a TxDb object
#' if (require(GenomicFeatures)) {
#'     samplefile <- system.file("extdata",
#'                               "hg19_knownGene_sample.sqlite",
#'                               package = "GenomicFeatures")
#'     txdb <- loadDb(samplefile)
#'     GeneRegionTrack(txdb)
#'     GeneRegionTrack(txdb, chromosome = "chr6", start = 35000000, end = 40000000)
#' }
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(grTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(grTrack)
#'
#' ## Track names
#' names(grTrack)
#' names(grTrack) <- "foo"
#' plotTracks(grTrack)
#'
#' ## Subsetting and splitting
#' subTrack <- subset(grTrack, from = 26700000, to = 26705000)
#' length(subTrack)
#' subTrack <- grTrack[transcript(grTrack) == "ENSMUST00000144140"]
#' split(grTrack, transcript(grTrack))
#'
#' ## Accessors
#' start(grTrack)
#' end(grTrack)
#' width(grTrack)
#' position(grTrack)
#' width(subTrack) <- width(subTrack) + 100
#'
#' strand(grTrack)
#' strand(subTrack) <- "-"
#'
#' chromosome(grTrack)
#' chromosome(subTrack) <- "chrX"
#'
#' genome(grTrack)
#' genome(subTrack) <- "hg19"
#'
#' range(grTrack)
#' ranges(grTrack)
#'
#' ## Annotation
#' identifier(grTrack)
#' identifier(grTrack, "lowest")
#' identifier(subTrack) <- "bar"
#'
#' feature(grTrack)
#' feature(subTrack) <- "foo"
#'
#' exon(grTrack)
#' exon(subTrack) <- letters[1:2]
#'
#' gene(grTrack)
#' gene(subTrack) <- "bar"
#'
#' symbol(grTrack)
#' symbol(subTrack) <- "foo"
#'
#' transcript(grTrack)
#' transcript(subTrack) <- c("foo", "bar")
#' chromosome(subTrack) <- "chr7"
#' plotTracks(subTrack)
#'
#' values(grTrack)
#'
#' ## Grouping
#' group(grTrack)
#' group(subTrack) <- "Group 1"
#' transcript(subTrack)
#' plotTracks(subTrack)
#'
#' ## Collapsing transcripts
#' plotTracks(grTrack,
#'     collapseTranscripts = TRUE, showId = TRUE,
#'     extend.left = 10000, shape = "arrow"
#' )
#'
#' ## Stacking
#' stacking(grTrack)
#' stacking(grTrack) <- "dense"
#' plotTracks(grTrack)
#'
#' ## coercion
#' as(grTrack, "data.frame")
#' as(grTrack, "UCSCData")
#'
#' ## HTML image map
#' coords(grTrack)
#' tags(grTrack)
#' grTrack <- plotTracks(grTrack)$foo
#' coords(grTrack)
#' tags(grTrack)
#' @exportClass GeneRegionTrack
setClass("GeneRegionTrack",
    contains = "AnnotationTrack",
    representation = representation(start = "NumericOrNULL", end = "NumericOrNULL"),
    prototype = prototype(
        columns = c("feature", "transcript", "symbol", "gene", "exon"),
        stacking = "squish",
        stacks = 0,
        start = 0,
        end = 0,
        name = "GeneRegionTrack",
        dp = DisplayPars(
            arrowHeadWidth = 10,
            arrowHeadMaxWidth = 20,
            col = NULL,
            collapseTranscripts = FALSE,
            exonAnnotation = NULL,
            fill = "orange",
            min.distance = 0,
            shape = c("smallArrow", "box"),
            showExonId = NULL,
            thinBoxFeature = .THIN_BOX_FEATURES,
            transcriptAnnotation = NULL
        )
    )
)

## Initialize ----------------------------------------------------------------

#' @describeIn GeneRegionTrack-class Initialize.
#' @export
setMethod("initialize", "GeneRegionTrack", function(.Object, start, end, ...) {
    if (is.null(list(...)$range) && is.null(list(...)$genome) && is.null(list(...)$chromosome)) {
        return(.Object)
    }
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "GeneRegionTrack")
    .Object@start <- ifelse(is.null(start), 0, start)
    .Object@end <- ifelse(is.null(end), 0, end)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## ReferenceGeneRegionTrack Class --------------------------------------------

## The file-based version of the GeneRegionTrack class. This will mainly provide a means to dispatch to
## a special 'subset' method which should stream the necessary data from disk.

#' @describeIn GeneRegionTrack-class The file-based version of the `GeneRegionTrack-class`.
#' @exportClass ReferenceGeneRegionTrack
setClass("ReferenceGeneRegionTrack", contains = c("GeneRegionTrack", "ReferenceTrack"))

## Initialize ----------------------------------------------------------------


## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritence has some strange features with regards to method selection

#' @describeIn GeneRegionTrack-class Initialize.
#' @export
setMethod("initialize", "ReferenceGeneRegionTrack", function(.Object, stream, reference, mapping = list(),
                                                             args = list(), defaults = list(), ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object = .Object, reference = reference, stream = stream,
        mapping = mapping, args = args, defaults = defaults)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

## Constructor. The following arguments are supported:
##    o range: one in a whole number of different potential inputs upon which the .buildRanges method will dispatch
##    o start, end: numeric vectors of the track start and end coordinates.
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o rstarts, rends, rwidths: integer vectors of exon start and end locations or widths, or a character vector of
##      comma-delimited exon locations, one vector element for each transcript
##    o strand, feature, exon, transcript, gene, symbol, chromosome: vectors of equal length containing
##       the exon strand, biotype, exon id, transcript id, gene id, human-readable gene symbol and chromosome information
##    o stacking: character controlling the stacking of overlapping items. One in 'hide',
##       'dense', 'squish', 'pack' or 'full'.
##    o name: the name of the track. This will be used for the title panel.
##    o exonList: boolean, causing the values in starts, rends or rwidths to be interpreted as delim-separated
##       lists that have to be exploded. All other annotation arguments will be repeated accordingly.
##    o delim: the delimiter if coordinates are in a list
## All additional items in ... are being treated as DisplayParameters

#' @describeIn GeneRegionTrack-class Constructor function for `GeneRegionTrack-class`.
#' @export
GeneRegionTrack <- function(range = NULL, rstarts = NULL, rends = NULL, rwidths = NULL, strand, feature, exon,
                            transcript, gene, symbol, chromosome, genome, stacking = "squish",
                            name = "GeneRegionTrack", start = NULL, end = NULL, importFunction, stream = FALSE, ...) {
    ## Some defaults
    covars <- if (is.data.frame(range)) range else if (is(range, "GRanges")) as.data.frame(mcols(range)) else data.frame()
    isStream <- FALSE
    if (!is.character(range)) {
        n <- if (is.null(range)) max(c(length(start), length(end), length(width))) else if (is(range, "data.frame")) nrow(range) else length(range)
        if (is.null(covars[["feature"]]) && missing(feature)) {
            feature <- paste("exon", seq_len(n), sep = "_")
        }
        if (is.null(covars[["exon"]]) && missing(exon)) {
            exon <- make.unique(rep(if (!missing(feature) && !is.null(feature)) as.character(feature) else covars[["feature"]], n)[seq_len(n)])
        }
        if (is.null(covars[["transcript"]]) && missing(transcript)) {
            transcript <- paste("transcript", seq_len(n), sep = "_")
        }
        if (is.null(covars[["gene"]]) && missing(gene)) {
            gene <- paste("gene", seq_len(n), sep = "_")
        }
    }
    ## Build a GRanges object from the inputs
    .missingToNull(c("feature", "exon", "transcript", "gene", "symbol", "strand", "chromosome", "importFunction", "genome"))
    args <- list(
        feature = feature, id = exon, exon = exon, transcript = transcript, gene = gene, symbol = symbol, strand = strand,
        chromosome = chromosome, genome = genome
    )
    defs <- list(
        feature = "unknown", id = "unknown", exon = "unknown", transcript = "unknown", genome = NA,
        gene = "unknown", symbol = "unknown", strand = "*", density = 1, chromosome = "chrNA"
    )
    range <- .buildRange(
        range = range, groupId = "transcript", start = rstarts, end = rends, width = rwidths, args = args, defaults = defs,
        chromosome = chromosome, tstart = start, tend = end, trackType = "GeneRegionTrack", importFun = importFunction,
        genome = genome
    )
    if (is.list(range)) {
        isStream <- TRUE
        slist <- range
        range <- GRanges()
    }
    if (is.null(start)) {
        start <- if (!length(range)) NULL else min(start(range))
    }
    if (is.null(end)) {
        end <- if (!length(range)) NULL else max(end(range))
    }
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- if (length(range) > 0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    }
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    if (!isStream) {
        return(new("GeneRegionTrack",
            start = start, end = end, chromosome = chromosome[1], range = range,
            name = name, genome = genome, stacking = stacking, ...
        ))
    } else {
        return(new("ReferenceGeneRegionTrack",
            start = start, end = end, chromosome = chromosome[1], range = range,
            name = name, genome = genome, stacking = stacking, stream = slist[["stream"]],
            reference = slist[["reference"]], mapping = slist[["mapping"]], args = args, defaults = defs, ...
        ))
    }
}

## .buildRange ---------------------------------------------------------------
##
## Helper methods to build a GRanges object from the input arguments.
##
## Coordinates for grouped elements may be passed in as comma-separated values
## (e.g. "1,5,9"), in which case we need to split and convert to numeric.
## This also implies that the additional arguments (like feature, group, etc.)
## have to be replicated accordingly. We handle this by passing along the repeat
## vector 'by' to the numeric method below.


## For TxDb objects we extract the grouping information and use the GRanges method

#' @importClassesFrom GenomicFeatures TxDb
#' @importMethodsFrom GenomicFeatures isActiveSeq "isActiveSeq<-" cdsBy exonsBy fiveUTRsByTranscript threeUTRsByTranscript transcriptsBy transcripts
setMethod(
    ".buildRange", signature("TxDb"),
    function(range, groupId = "transcript", tstart, tend, chromosome, args, ...) {
        ## If chromosome (and optional start and end) information is present we only extract parts of the annotation data
        noSubset <- is.null(tstart) && is.null(tend)
        if (!is.null(chromosome)) {
            chromosome <- .chrName(chromosome)
            ## Seems like TxDb objects use pass by reference for the active chromosomes, so we have to
            ## restore the old values after we are done
            oldAct <- seqlevels(range)
            oldRange <- range
            on.exit({
                restoreSeqlevels(oldRange)
                seqlevels(oldRange, pruning.mode = "coarse") <- oldAct
            })
            restoreSeqlevels(range)
            seqlevels(range, pruning.mode = "coarse") <- chromosome
            sl <- seqlengths(range)
            if (is.null(tstart)) {
                tstart <- rep(1, length(chromosome))
            }
            if (is.null(tend)) {
                tend <- sl[chromosome] + 1
                tend[is.na(tend)] <- tstart[is.na(tend)] + 1
            }
            sRange <- GRanges(seqnames = chromosome, ranges = IRanges(start = tstart, end = tend))
        }
        ## First the mapping of internal transcript ID to transcript name
        txs <- as.data.frame(values(transcripts(range, columns = c("tx_id", "tx_name"))))
        rownames(txs) <- txs[, "tx_id"]
        ## Now the CDS ranges
        t2c <- cdsBy(range, "tx")
        names(t2c) <- txs[names(t2c), 2]
        tids <- rep(names(t2c), elementNROWS(t2c))
        t2c <- unlist(t2c)
        if (length(t2c)) {
            t2c$tx_id <- tids
            t2c$feature_type <- "CDS"
        }
        ## And the 5'UTRS
        t2f <- fiveUTRsByTranscript(range)
        names(t2f) <- txs[names(t2f), 2]
        tids <- rep(names(t2f), elementNROWS(t2f))
        t2f <- unlist(t2f)
        if (length(t2f)) {
            t2f$tx_id <- tids
            t2f$feature_type <- "utr5"
        }
        ## And the 3'UTRS
        t2t <- threeUTRsByTranscript(range)
        names(t2t) <- txs[names(t2t), 2]
        tids <- rep(names(t2t), elementNROWS(t2t))
        t2t <- unlist(t2t)
        if (length(t2t)) {
            t2t$tx_id <- tids
            t2t$feature_type <- "utr3"
        }
        ## And finally all the non-coding transcripts
        nt2e <- exonsBy(range, "tx")
        names(nt2e) <- txs[names(nt2e), 2]
        nt2e <- nt2e[!names(nt2e) %in% c(values(t2c)$tx_id, values(t2f)$tx_id, values(t2t)$tx_id)]
        tids <- rep(names(nt2e), elementNROWS(nt2e))
        nt2e <- unlist(nt2e)
        if (length(nt2e)) {
            nt2e$tx_id <- tids
            nt2e$feature_type <- "ncRNA"
        }
        ## Now we can merge the three back together (we need to change the column names of t2c to make them all the same)
        colnames(values(t2c))[c(1, 2)] <- c("exon_id", "exon_name")
        ## t2e <- c(t2c, t2f, t2t, nt2e) ## This is super-slow, much more efficient if we build the GRanges object from the individual bits and pieces
        vals <- DataFrame(
            exon_id = c(values(t2c)$exon_id, values(t2f)$exon_id, values(t2t)$exon_id, values(nt2e)$exon_id),
            exon_name = c(values(t2c)$exon_name, values(t2f)$exon_name, values(t2t)$exon_name, values(nt2e)$exon_name),
            exon_rank = c(values(t2c)$exon_rank, values(t2f)$exon_rank, values(t2t)$exon_rank, values(nt2e)$exon_rank),
            tx_id = c(values(t2c)$tx_id, values(t2f)$tx_id, values(t2t)$tx_id, values(nt2e)$tx_id),
            feature_type = c(values(t2c)$feature_type, values(t2f)$feature_type, values(t2t)$feature_type, values(nt2e)$feature_type)
        )
        t2e <- GRanges(
            seqnames = c(seqnames(t2c), seqnames(t2f), seqnames(t2t), seqnames(nt2e)),
            ranges = IRanges(
                start = c(start(t2c), start(t2f), start(t2t), start(nt2e)),
                end = c(end(t2c), end(t2f), end(t2t), end(nt2e))
            ),
            strand = c(strand(t2c), strand(t2f), strand(t2t), strand(nt2e))
        )
        if (all(is.na(vals$exon_name))) {
            vals$exon_name <- paste(vals$tx_id, vals$exon_rank, sep = "_")
        }
        values(t2e) <- vals
        if (length(t2e) == 0) {
            return(GRanges())
        }
        ## Add the gene level annotation
        g2t <- transcriptsBy(range, "gene")
        gids <- rep(names(g2t), elementNROWS(g2t))
        g2t <- unlist(g2t)
        values(g2t)[["gene_id"]] <- gids
        values(t2e)$gene_id <- gids[match(values(t2e)$tx_id, as.character(txs[as.character(values(g2t)$tx_id), 2]))]
        vals <- values(t2e)[c("tx_id", "exon_name", "exon_rank", "feature_type", "tx_id", "gene_id")]
        colnames(vals) <- c("transcript", "exon", "rank", "feature", "symbol", "gene")
        ## Add the genome information
        genome(t2e) <- unique(genome(range))
        ## Finally we re-assign, subset if necessary, and sort
        range <- t2e
        values(range) <- vals
        if (!noSubset && !is.null(chromosome)) {
            ## We have to keep all exons for all the overlapping transcripts
            txSel <- unique(subsetByOverlaps(g2t, sRange)$tx_name)
            range <- range[range$transcript %in% txSel]
        }
        args <- list(genome = genome(range)[1])
        return(.buildRange(range = sort(range), chromosome = chromosome, args = args, ...))
    }
)

## For ensDb objects we extract the grouping information and use the GRanges method

#' @importClassesFrom ensembldb EnsDb
#' @importMethodsFrom ensembldb cdsBy exonsBy fiveUTRsByTranscript threeUTRsByTranscript transcriptsBy transcripts
setMethod(
    ".buildRange", signature("EnsDb"),
    function(range, groupId = "transcript", tstart, tend, chromosome, args, ...) {
        ## If chromosome (and optional start and end) information is present we only extract parts of the annotation data
        noSubset <- is.null(tstart) && is.null(tend)
        if (!is.null(chromosome)) {
            chromosome <- .chrName(chromosome)
            sl <- seqlengths(range)
            if (is.null(tstart)) {
                tstart <- rep(1, length(chromosome))
            }
            if (is.null(tend)) {
                tend <- sl[chromosome] + 1
                tend[is.na(tend)] <- tstart[is.na(tend)] + 1
            }
            sRange <- GRanges(seqnames = chromosome, ranges = IRanges(start = tstart, end = tend))
            ## sRange <- GRangesFilter(sRange, type = "any") can we filter directly?
        }
        ## First the mapping of internal transcript ID to transcript name
        txs <- as.data.frame(values(transcripts(range, columns = c("tx_id", "tx_name"))))
        rownames(txs) <- txs[, "tx_id"]
        ## Now the CDS ranges
        t2c <- cdsBy(range, "tx")
        names(t2c) <- txs[names(t2c), 2]
        tids <- rep(names(t2c), elementNROWS(t2c))
        t2c <- unlist(t2c)
        if (length(t2c)) {
            t2c$tx_id <- tids
            t2c$feature_type <- "CDS"
        }
        ## And the 5'UTRS
        t2f <- fiveUTRsByTranscript(range)
        names(t2f) <- txs[names(t2f), 2]
        tids <- rep(names(t2f), elementNROWS(t2f))
        t2f <- unlist(t2f)
        if (length(t2f)) {
            t2f$tx_id <- tids
            t2f$feature_type <- "utr5"
        }
        ## And the 3'UTRS
        t2t <- threeUTRsByTranscript(range)
        names(t2t) <- txs[names(t2t), 2]
        tids <- rep(names(t2t), elementNROWS(t2t))
        t2t <- unlist(t2t)
        if (length(t2t)) {
            t2t$tx_id <- tids
            t2t$feature_type <- "utr3"
        }
        ## And finally all the non-coding transcripts
        nt2e <- exonsBy(range, "tx")
        names(nt2e) <- txs[names(nt2e), 2]
        nt2e <- nt2e[!names(nt2e) %in% c(values(t2c)$tx_id, values(t2f)$tx_id, values(t2t)$tx_id)]
        tids <- rep(names(nt2e), elementNROWS(nt2e))
        nt2e <- unlist(nt2e)
        if (length(nt2e)) {
            nt2e$tx_id <- tids
            nt2e$feature_type <- "ncRNA"
        }
        ## Now we can merge the three back together (we need to change the column names of t2c to make them all the same)
        ## t2e <- c(t2c, t2f, t2t, nt2e) ## This is super-slow, much more efficient if we build the GRanges object from the individual bits and pieces
        vals <- DataFrame(
            exon_id = c(values(t2c)$exon_id, values(t2f)$exon_id, values(t2t)$exon_id, values(nt2e)$exon_id),
            exon_name = c(values(t2c)$exon_id, values(t2f)$exon_id, values(t2t)$exon_id, values(nt2e)$exon_id),
            exon_rank = c(values(t2c)$exon_rank, values(t2f)$exon_rank, values(t2t)$exon_rank, values(nt2e)$exon_rank),
            tx_id = c(values(t2c)$tx_id, values(t2f)$tx_id, values(t2t)$tx_id, values(nt2e)$tx_id),
            feature_type = c(values(t2c)$feature_type, values(t2f)$feature_type, values(t2t)$feature_type, values(nt2e)$feature_type)
        )
        t2e <- GRanges(
            seqnames = c(seqnames(t2c), seqnames(t2f), seqnames(t2t), seqnames(nt2e)),
            ranges = IRanges(
                start = c(start(t2c), start(t2f), start(t2t), start(nt2e)),
                end = c(end(t2c), end(t2f), end(t2t), end(nt2e))
            ),
            strand = c(strand(t2c), strand(t2f), strand(t2t), strand(nt2e))
        )
        if (all(is.na(vals$exon_name))) {
            vals$exon_name <- paste(vals$tx_id, vals$exon_rank, sep = "_")
        }
        values(t2e) <- vals
        if (length(t2e) == 0) {
            return(GRanges())
        }
        ## Add the gene level annotation
        g2t <- transcriptsBy(range, "gene")
        gids <- rep(names(g2t), elementNROWS(g2t))
        g2t <- unlist(g2t)
        values(g2t)[["gene_id"]] <- gids
        values(t2e)$gene_id <- gids[match(values(t2e)$tx_id, as.character(txs[as.character(values(g2t)$tx_id), 2]))]
        vals <- values(t2e)[c("tx_id", "exon_name", "exon_rank", "feature_type", "tx_id", "gene_id")]
        colnames(vals) <- c("transcript", "exon", "rank", "feature", "symbol", "gene")
        ## Add the genome information
        genome(t2e) <- unique(genome(range))
        ## Finally we re-assign, subset if necessary, and sort
        range <- t2e
        values(range) <- vals
        if (!noSubset && !is.null(chromosome)) {
            ## We have to keep all exons for all the overlapping transcripts
            txSel <- unique(subsetByOverlaps(g2t, sRange)$tx_name)
            range <- range[range$transcript %in% txSel]
        }
        args <- list(genome = genome(range)[1])
        return(.buildRange(range = sort(range), chromosome = chromosome, args = args, ...))
    }
)

## General accessors ---------------------------------------------------------
## Annotation Accessors ------------------------------------------------------

#' @describeIn GeneRegionTrack-class Extract the gene identifiers for all
#' gene models.
#' @export
setMethod("gene", signature(GdObject = "GeneRegionTrack"), function(GdObject) .getAnn(GdObject, "gene"))

#' @describeIn GeneRegionTrack-class Replace the gene identifiers for all
#' gene models.
#' The replacement value must be a character of appropriate length or another
#' vector that can be coerced into such.
#' @export
setReplaceMethod("gene", signature("GeneRegionTrack", "character"), function(GdObject, value) .setAnn(GdObject, value, "gene"))

#' @describeIn GeneRegionTrack-class Extract the human-readble gene symbol
#' for all gene models.
#' @export
setMethod("symbol", signature(GdObject = "GeneRegionTrack"), function(GdObject) .getAnn(GdObject, "symbol"))

#' @describeIn GeneRegionTrack-class Replace the human-readable gene symbol
#' for all gene models.
#' The replacement value must be a character of appropriate length or another
#' vector that can be coerced into such.
#' @export
setReplaceMethod("symbol", signature("GeneRegionTrack", "character"), function(GdObject, value) .setAnn(GdObject, value, "symbol"))

#' @describeIn GeneRegionTrack-class Extract the transcript identifiers for all
#' transcripts in the gene models.
#' @export
setMethod("transcript", signature(GdObject = "GeneRegionTrack"), function(GdObject) .getAnn(GdObject, "transcript"))

#' @describeIn GeneRegionTrack-class Replace the transcript identifiers for all
#' transcripts in the gene model. The replacement value must be a character of
#' appropriate length or another vector that can be coerced into such.
#' @export
setReplaceMethod(
    "transcript", signature("GeneRegionTrack", "character"),
    function(GdObject, value) .setAnn(GdObject, value, "transcript")
)

#' @describeIn GeneRegionTrack-class Extract the exon identifiers for all exons
#' in the gene models.
#' @export
setMethod("exon", signature(GdObject = "GeneRegionTrack"), function(GdObject) .getAnn(GdObject, "exon"))

#' @describeIn GeneRegionTrack-class replace the exon identifiers for all exons
#' in the gene model. The replacement value must be a character of appropriate
#' length or another vector that can be coerced into such.
#' @export
setReplaceMethod("exon", signature("GeneRegionTrack", "character"), function(GdObject, value) .setAnn(GdObject, value, "exon"))

#' @describeIn GeneRegionTrack-class extract the group membership for all track items.
#' @export
setMethod("group", "GeneRegionTrack", function(GdObject) transcript(GdObject))

#' @describeIn GeneRegionTrack-class replace the grouping information for track items.
#' The replacement value must be a factor of appropriate length or another
#' vector that can be coerced into such.
#' @export
setReplaceMethod(
    "group", signature("GeneRegionTrack", "character"),
    function(GdObject, value) .setAnn(GdObject, value, "transcript")
)

#' @describeIn GeneRegionTrack-class return track item identifiers.
#' Depending on the setting of the optional argument lowest, these are either
#' the group identifiers or the individual item identifiers.
#' export
setMethod("identifier", "GeneRegionTrack", function(GdObject, type = .dpOrDefault(GdObject, "transcriptAnnotation", "symbol")) {
    if (is.logical(type)) {
        type <- ifelse("symbol", "gene", type[1])
    }
    if (is.null(type)) {
        type <- "symbol"
    }
    id <- switch(as.character(type),
        "symbol" = symbol(GdObject),
        "gene" = gene(GdObject),
        "transcript" = transcript(GdObject),
        "feature" = feature(GdObject),
        "exon" = exon(GdObject),
        "lowest" = exon(GdObject),
        symbol(GdObject)
    )
    id[is.na(id)] <- "NA"
    return(id)
})

#' @describeIn GeneRegionTrack-class Set the track item identifiers.
#' The replacement value has to be a character vector of appropriate length.
#' This always replaces the group-level identifiers, so essentially it is
#' similar to `groups<-`.
#' @export
setReplaceMethod("identifier", c("GeneRegionTrack", "character"), function(GdObject, value) {
    type <- .dpOrDefault(GdObject, "transcriptAnnotation", "symbol")
    switch(as.character(type),
        "symbol" = symbol(GdObject) <- value,
        "gene" = gene(GdObject) <- value,
        "transcript" = transcript(GdObject) <- value,
        "feature" = feature(GdObject) <- value,
        "exon" = exon(GdObject) <- value,
        "lowest" = exon(GdObject) <- value,
        symbol(GdObject) <- value
    )
    return(GdObject)
})

## Stacking ------------------------------------------------------------------
## Consolidate ---------------------------------------------------------------
## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------

## FIXME: Still needs to be implemented

#' @describeIn GeneRegionTrack-class Subset a GeneRegionTrack by coordinates
#' and sort if necessary.
#' @export
setMethod("subset", signature(x = "ReferenceGeneRegionTrack"), function(x, ...) {
    warning("ReferenceGeneRegionTrack objects are not supported yet.")
    return(callNextMethod())
})

## Position ------------------------------------------------------------------
## DrawGrid ------------------------------------------------------------------
## DrawAxis ------------------------------------------------------------------
## DrawGD --------------------------------------------------------------------

#' @describeIn GeneRegionTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @export
setMethod("drawGD", signature("GeneRegionTrack"), function(GdObject, ...) {
    displayPars(GdObject) <- list(showFeatureId = as.vector(.dpOrDefault(GdObject, "showExonId")))
    GdObject <- callNextMethod()
    return(invisible(GdObject))
})

## SetAs ---------------------------------------------------------------------

#' @importFrom rtracklayer GenomicData
setAs(
    "GeneRegionTrack", "UCSCData",
    function(from, to) {
        ranges <- cbind(as(as(ranges(from), "DataFrame"), "data.frame"),
            start = start(from),
            end = end(from), color = .getBiotypeColor(from), strand = strand(from)
        )
        ranges <- ranges[order(start(from)), ]
        ranges <- split(ranges, ranges[, "X.transcript"])
        start <- vapply(ranges, function(x) min(x$start), FUN.VALUE = numeric(1L))
        end <- vapply(ranges, function(x) max(x$end), FUN.VALUE = numeric(1L))
        name <- vapply(ranges, function(x) as.character(unique(x$X.symbol)), FUN.VALUE = character(1L))
        color <- vapply(ranges, function(x) as.character(unique(x$color)), FUN.VALUE = character(1L))
        strand <- vapply(ranges, function(x) as.character(unique(x$strand)), FUN.VALUE = character(1L))
        strand[strand == "*"] <- "+"
        id <- names(ranges)
        blocks <- vapply(ranges, nrow, FUN.VALUE = numeric(1L))
        bsizes <- vapply(ranges, function(x) paste(x$end - x$start + 1, collapse = ","), FUN.VALUE = character(1L))
        bstarts <- vapply(ranges, function(x) paste(x$start - min(x$start), collapse = ","), FUN.VALUE = character(1L))
        dcolor <- as.integer(col2rgb(.dpOrDefault(from, "col")))
        line <- new("BasicTrackLine",
            name = names(from),
            description = names(from),
            visibility = stacking(from), color = dcolor, itemRgb = TRUE
        )
        new("UCSCData", rtracklayer::GenomicData(IRanges(start, end),
            chrom = chromosome(from),
            id = id, name = name, itemRgb = color, blockCount = blocks,
            blockSizes = bsizes, blockStarts = bstarts,
            strand = strand
        ),
        trackLine = line
        )
    }
)

setAs("GRanges", "GeneRegionTrack", function(from, to) GeneRegionTrack(range = from))

setAs("GRangesList", "GeneRegionTrack", function(from, to) GeneRegionTrack(range = from))

setAs("TxDb", "GeneRegionTrack", function(from, to) GeneRegionTrack(range = from))

## Show ----------------------------------------------------------------------

#' @describeIn GeneRegionTrack-class Show method.
#' @export
setMethod("show", signature(object = "GeneRegionTrack"), function(object) {
    cat(sprintf("GeneRegionTrack '%s'\n%s\n", names(object), .annotationTrackInfo(object)))
})

#' @describeIn GeneRegionTrack-class Show method.
#' @export
setMethod("show", signature(object = "ReferenceGeneRegionTrack"), function(object) {
    .referenceTrackInfo(object, "ReferenceGeneRegionTrack")
})

#' Setting display parameters to control the look and feel of the plots
#'
#'
#' The genome track plots in this package are all highly customizable by means
#' of so called 'display parameters'. This page highlights the use of these
#' parameters and list all available settings for the different track classes.
#'
#'
#' All of the package's track objects inherit the \code{dp} slot from the
#' \code{\linkS4class{GdObject}} parent class, which is the main container to
#' store an object's display parameters. Internally, the content of this slot
#' has to be an object of class \code{\linkS4class{DisplayPars}}, but the user
#' is usually not exposed to this low level implementation. Instead, there are
#' two main interaction points, namely the individual object constructor
#' functions and the final \code{\link{plotTracks}} function. In both cases,
#' all additional arguments that are not caught by any of the formally defined
#' function parameters are being interpreted as additional display parameters
#' and are automatically added to the aforementioned slot. The main difference
#' here is that display parameters that are passed on to the constructor
#' function are specific for an individual track object, whereas those supplied
#' to the \code{plotTracks} function will be applied to all the objects in the
#' plotting list. Not all display parameters have an effect on the plotting of
#' all track classes, and those will be silently ignored.
#'
#' One can query the available display parameters for a given class as well as
#' their default values by calling the \code{\link{availableDisplayPars}}
#' function, or by inspecting the man pages of the individual track classes.
#' The structure of the classes defined in this package is hierarchical, and so
#' are the available display parameters, i.e., all objects inherit the
#' parameters defined in the commom \code{GdObject} parent class, and so on.
#'
#' Once a track object has been created, the display parameters are still open
#' for modification. To this end, the \code{\linkS4class{DisplayPars}} replacement
#' method is available for all objects inheriting from class \code{GdObject}.
#' The method takes a named list of parameters as input, e.g.:
#'
#' \code{displayPars(foo) <- list(col="red", lwd=2)}
#'
#' In the same spirit, the currently set display parameters for the object
#' \code{foo} can be inferred using the \code{displayPars} method directly,
#' e.g.:
#'
#' \code{displayPars(foo)}
#'
#' For track objects inheriting from class
#' \code{\linkS4class{AnnotationTrack}}, display parameters that are not
#' formally defined in the class definition or in any of the parent classes are
#' considered to be valid R color identifiers that are used to distinguish
#' between different types of annotation features. For instance, the parameter
#' 'miRNA' will be used to color all annotation features of class miRNA. The
#' annotation types can be set in the constructor function of the track object
#' via the \code{feature} argument. For most of the tracks that have been
#' inferred from one of the online repositories, this classification will
#' usually be downloaded along with the actual annotation data.
#'
#' Users might find themselves changing the same parameters over and over
#' again, and it would make sense to register these modifications in a central
#' location once and for all. To this end the Gviz package supports display
#' parameter schemes. A scheme is essentially just a bunch of nested named
#' lists, where the names on the first level of nesting should correspond to
#' track class names, and the names on the second level to the display
#' parameters to set. The currently active schmeme can be changed by setting
#' the global option \code{Gviz.scheme}, and a new scheme can be registered by
#' using the \code{addScheme} function, providing both the list and the name
#' for the new scheme. The \code{getScheme} function is useful to get the
#' current scheme as a list structure, for instance to use as a skeleton for
#' your own custom scheme.
#'
#' In order to make these settings persitant across R sessions one can create
#' one or several schemes in the global environment in the special object
#' \code{.GvizSchemes}, for instance by putting the necessary code in the
#' \code{.Rprofile} file. This object needs to be a named list of schemes, and
#' it will be collected when the Givz package loads. Its content is then
#' automatically added to the collection of available schemes.
#'
#' Please note that because display parameters are stored with the track
#' objects, a scheme change only has an effect on those objects that are
#' created after the change has taken place.
#'
#' @aliases settings addScheme getScheme
#' @param scheme A named nested list of display parameters, where the first
#' level of nesting represents Gviz track object classes, and the second level
#' of nesting represents parameters.
#' @param name A character scalar with the scheme name.
#' @return
#'
#' The \code{getScheme} function returns current scheme as a list structure.
#' @section Display Parameters:
#'
#' \describe{
#'
#' \item{GenomeAxisTrack:}{
#'
#' \describe{
#'
#' \item{\code{add35=FALSE}:}{ Logical scalar. Add 3' to 5' direction
#' indicators.}
#'
#' \item{\code{add53=FALSE}:}{ Logical scalar. Add 5' to 3' direction
#' indicators.}
#'
#' \item{\code{background.title="transparent"}:}{ Character scalar.  The
#' background color for the title panel. Defaults to omit the background.}
#'
#' \item{\code{cex.id=0.7}:}{ Numeric scalar. The text size for the optional
#' range annotation.}
#'
#' \item{\code{cex=0.8}:}{ Numeric scalar. The overall font expansion factor
#' for the axis annotation text.}
#'
#' \item{\code{col.border.title="transparent"}:}{ Integer or character scalar.
#' The border color for the title panels.}
#'
#' \item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
#' title panels.}
#'
#' \item{\code{col.id="white"}:}{ Character scalar. The text color for the
#' optional range annotation.}
#'
#' \item{\code{col.range="cornsilk4"}:}{ Character scalar. The border color for
#' highlighted regions on the axis.}
#'
#' \item{\code{distFromAxis=1}:}{ Numeric scalar. Control the distance of the
#' axis annotation from the tick marks.}
#'
#' \item{\code{exponent=NULL}:}{ Numeric scalar. The exponent for the axis
#' coordinates, e.g., 3 means mb, 6 means gb, etc. The default is to
#' automatically determine the optimal exponent.}
#'
#' \item{\code{fill.range="cornsilk3"}:}{ Character scalar. The fill color for
#' highlighted regions on the axis.}
#'
#' \item{\code{fontcolor="#808080"}:}{ Character scalar. The font color for the
#' axis annotation text.}
#'
#' \item{\code{fontsize=10}:}{ Numeric scalar. Font size for the axis
#' annotation text in points.}
#'
#' \item{\code{labelPos="alternating"}:}{ Character vector, one in
#' "alternating", "revAlternating", "above" or "below". The vertical
#' positioning of the axis labels. If \code{scale} is not \code{NULL}, the
#' possible values are "above", "below" and "beside".}
#'
#' \item{\code{littleTicks=FALSE}:}{ Logical scalar. Add more fine-grained tick
#' marks.}
#'
#' \item{\code{lwd=2}:}{ Numeric scalar. The line width for the axis
#' elementes.}
#'
#' \item{\code{scale=NULL}:}{ Numeric scalar. If not \code{NULL} a small scale
#' is drawn instead of the full axis, if the value is between 0 and 1 it is
#' interpreted as a fraction of the current plotting region, otherwise as an
#' absolute length value in genomic coordinates.}
#'
#' \item{\code{showId=FALSE}:}{ Logical scalar. Show the optional range
#' highlighting annotation.}
#'
#' \item{\code{showTitle=FALSE}:}{ Logical scalar. Plot a title panel. Defaults
#' to omit the title panel.}
#'
#' \item{\code{ticksAt=NULL}:}{ Numeric scalar. The exact x-position for
#' tickmarks (in base-pairs).}
#'
#' \item{\code{size=NULL}:}{ Numeric scalar. The relative size of the track.
#' Can be overridden in the \code{\link{plotTracks}} function. Defaults to the
#' ideal size based on the other track settings.}
#'
#' \item{\code{col="darkgray"}:}{ Character scalar. The color for the axis
#' lines and tickmarks.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GdObject:}}
#'
#' \describe{
#'
#' \item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
#' items.}
#'
#' \item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
#' title panel.}
#'
#' \item{\code{background.panel="transparent"}:}{ Integer or character scalar.
#' The background color of the content panel.}
#'
#' \item{\code{background.legend="transparent"}:}{ Integer or character scalar.
#' The background color for the legend.}
#'
#' \item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
#' axis annotation. Defaults to \code{NULL}, in which case it is automatically
#' determined based on the available space.}
#'
#' \item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
#' title panel. This effects the fontsize of both the title and the axis, if
#' any. Defaults to \code{NULL}, which means that the text size is
#' automatically adjusted to the available space.}
#'
#' \item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
#' line color for the y axis, if any.}
#'
#' \item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
#' color used for the panel frame, if \code{frame==TRUE}}
#'
#' \item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
#' line color for grid lines, both when \code{type=="g"} in
#' \code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
#' for plot lines. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
#' for plot symbols. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.title="white"}:}{ Integer or character scalar.  The border
#' color for the title panels}
#'
#' \item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
#' content of the track to accomodate the minimum current device resolution.
#' See \code{\link{collapsing}} for details.}
#'
#' \item{\code{fill="lightgray"}:}{ Integer or character scalar.  Default fill
#' color setting for all plotting elements, unless there is a more specific
#' control defined elsewhere.}
#'
#' \item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
#' for the title panels.}
#'
#' \item{\code{fontface=1}:}{ Integer or character scalar. The font face for
#' all text, unless a more specific definition exists.}
#'
#' \item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
#' font family for the title panels.}
#'
#' \item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
#' family for all text, unless a more specific definition exists.}
#'
#' \item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
#' plotting.}
#'
#' \item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
#' grid.}
#'
#' \item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
#' horizontal grid lines, see \code{\link{panel.grid}} for details.}
#'
#' \item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
#' text, unless a more specific definition exists.}
#'
#' \item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
#' type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
#' and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lty="solid"}:}{ Numeric scalar. Default line type setting for
#' all plotting elements, unless there is a more specific control defined
#' elsewhere.}
#'
#' \item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
#' panels}
#'
#' \item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
#' lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
#' display parameter \code{grid==TRUE}.}
#'
#' \item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
#' before collapsing range items, only if \code{collapse==TRUE}. See
#' \code{\link{collapsing}} for details.}
#'
#' \item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
#' coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
#' mirror the plot on the vertical axis.}
#'
#' \item{\code{rotation.title=90}:}{ The rotation angle for the text in the
#' title panel. Even though this can be adjusted, the automatic resizing of the
#' title panel will currently not work, so use at own risk.}
#'
#' \item{\code{rotation=0}:}{ The rotation angle for all text unless a more
#' specific definiton exists.}
#'
#' \item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
#' (only applies to track types where axes are implemented).}
#'
#' \item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
#' vertical grid lines, see \code{\link{panel.grid}} for details.}
#'
#' }
#'
#' }
#'
#' \item{DataTrack:}{
#'
#' \describe{
#'
#' \item{\code{aggregateGroups=FALSE}:}{ Logical scalar. Aggregate the values
#' within a sample group using the aggregation funnction specified in the
#' \code{aggregation} parameter.}
#'
#' \item{\code{aggregation="mean"}:}{ Function or character scalar.  Used to
#' aggregate values in windows or for collapsing overlapping items. The
#' function has to accept a numeric vector as a single input parameter and has
#' to return a numeric scalar with the aggregated value. Alternatively, one of
#' the predefined options \code{mean}, \code{median} \code{sum}, \code{min},
#' \code{max} or \code{extreme} can be supplied as a character scalar. Defaults
#' to \code{mean}.}
#'
#' \item{\code{missingAsZero=TRUE}:}{ Logical scalar. Defines how the missing
#' values are treated in the aggregation procedure with running window. Setting
#' it to \code{TRUE} fills empty positions with zeros, which is default.
#' \code{FALSE} fills empty positions with \code{NA}.}
#'
#' \item{\code{alpha.confint=0.3}:}{ Numeric scalar. The transparency for the
#' confidence intervalls in confint-type plots.}
#'
#' \item{\code{amount=NULL}:}{ Numeric scalar. Amount of jittering in xy-type
#' plots. See \code{\link{panel.xyplot}} for details.}
#'
#' \item{\code{baseline=NULL}:}{ Numeric scalar. Y-axis position of an optional
#' baseline. This parameter has a special meaning for mountain-type and
#' polygon-type plots, see the 'Details' section in
#' \code{\linkS4class{DataTrack}} for more information.}
#'
#' \item{\code{box.legend=FALSE}:}{ Logical scalar. Draw a box around a
#' legend.}
#'
#' \item{\code{box.ratio=1}:}{ Numeric scalar. Parameter controlling the
#' boxplot appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{box.width=NULL}:}{ Numeric scalar. Parameter controlling the
#' boxplot appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{grid=FALSE}:}{ Logical vector. Draw a line grid under the track
#' content.}
#'
#' \item{\code{cex.legend=0.8}:}{ Numeric scalar. The size factor for the
#' legend text.}
#'
#' \item{\code{cex.sampleNames=NULL}:}{ Numeric scalar. The size factor for the
#' sample names text in heatmap or horizon plots.  Defaults to an automatic
#' setting.}
#'
#' \item{\code{cex=0.7}:}{ Numeric scalar. The default pixel size for plotting
#' symbols.}
#'
#' \item{\code{coef=1.5}:}{ Numeric scalar. Parameter controlling the boxplot
#' appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{col.baseline=NULL}:}{ Character scalar. Color for the optional
#' baseline, defaults to the setting of \code{col}.}
#'
#' \item{\code{col.confint=NA}:}{ Character vector. Border colors for the
#' confidence intervals for confint-type plots.}
#'
#' \item{\code{col.boxplotFrame="#808080"}:}{ Character scalar.  Line color of
#' the frame around grouped boxplots.}
#'
#' \item{\code{col.histogram="#808080"}:}{ Character scalar. Line color in
#' histogram-type plots.}
#'
#' \item{\code{col.horizon=NA}:}{ The line color for the segments in the
#' \code{horizon}-type plot. See \code{\link{horizonplot}} for details.}
#'
#' \item{\code{col.mountain=NULL}:}{ Character scalar. Line color in
#' mountain-type and polygon-type plots, defaults to the setting of
#' \code{col}.}
#'
#' \item{\code{col.sampleNames="white"}:}{ Character or integer scalar. The
#' color used for the sample names in heatmap plots.}
#'
#' \item{\code{col=c("#0080ff", "#ff00ff", "darkgreen", "#ff0000", "orange",
#' "#00ff00", "brown")}:}{ Character or integer vector.  The color used for all
#' line and symbol elements, unless there is a more specific control defined
#' elsewhere. Unless \code{groups} are specified, only the first color in the
#' vector is usually regarded.}
#'
#' \item{\code{collapse=FALSE}:}{ Logical scalar. Collapse overlapping ranges
#' and aggregate the underlying data.}
#'
#' \item{\code{degree=1}:}{ Numeric scalar. Parameter controlling the loess
#' calculation for smooth and mountain-type plots.  See
#' \code{\link{panel.loess}} for details.}
#'
#' \item{\code{do.out=TRUE}:}{ Logical scalar. Parameter controlling the
#' boxplot appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{evaluation=50}:}{ Numeric scalar. Parameter controlling the
#' loess calculation for smooth and mountain-type plots. See
#' \code{\link{panel.loess}} for details.}
#'
#' \item{\code{factor=0.5}:}{ Numeric scalar. Factor to control amount of
#' jittering in xy-type plots. See \code{\link{panel.xyplot}} for details.}
#'
#' \item{\code{family="symmetric"}:}{ Character scalar. Parameter controlling
#' the loess calculation for smooth and mountain-type plots. See
#' \code{\link{panel.loess}} for details.}
#'
#' \item{\code{fill.confint=NULL}:}{ Character vector. Fill colors for the
#' confidence intervals for confint-type plots.}
#'
#' \item{\code{fill.histogram=NULL}:}{ Character scalar. Fill color in
#' histogram-type plots, defaults to the setting of \code{fill}.}
#'
#' \item{\code{fill.horizon=c("#B41414", "#E03231", "#F7A99C", "#9FC8DC",
#' "#468CC8", "#0165B3")}:}{ The fill colors for the segments in the
#' \code{horizon}-type plot. This should be a vector of length six, where the
#' first three entries are the colors for positive changes, and the latter
#' three entries are the colors for negative changes. Defaults to a red-blue
#' color scheme. See \code{\link{horizonplot}} for details.}
#'
#' \item{\code{fill.mountain=c("#CCFFFF", "#FFCCFF")}:}{ Character vector of
#' length 2. Fill color in mountain-type and polygon-type plots.}
#'
#' \item{\code{fontface.legend=NULL}:}{ Integer or character scalar. The font
#' face for the legend text.}
#'
#' \item{\code{fontfamily.legend=NULL}:}{ Integer or character scalar. The font
#' family for the legend text.}
#'
#' \item{\code{fontsize.legend=NULL}:}{ Numeric scalar. The pixel size for the
#' legend text.}
#'
#' \item{\code{fontcolor.legend="#808080"}:}{ Integer or character scalar. The
#' font color for the legend text.}
#'
#' \item{\code{gradient=c("#F7FBFF", "#DEEBF7", "#C6DBEF", "#9ECAE1",
#' "#6BAED6", "#4292C6", "#2171B5", "#08519C", "#08306B")}:}{ Character vector.
#' The base colors for the \code{gradient} plotting type or the \code{heatmap}
#' type with a single group. When plotting heatmaps with more than one group,
#' the \code{col} parameter can be used to control the group color scheme,
#' however the gradient will always be from white to 'col' and thus does not
#' offer as much flexibility as this \code{gradient} parameter.}
#'
#' \item{\code{groups=NULL}:}{ Vector coercable to a factor.  Optional sample
#' grouping. See 'Details' section in \code{\linkS4class{DataTrack}} for
#' further information.}
#'
#' \item{\code{horizon.origin=0}:}{ The baseline relative to which changes are
#' indicated on the \code{horizon}-type plot. See \code{\link{horizonplot}} for
#' details.}
#'
#' \item{\code{horizon.scale=NULL}:}{ The scale for each of the segments in the
#' \code{horizon}-type plot. Defaults to 1/3 of the absolute data range. See
#' \code{\link{horizonplot}} for details.}
#'
#' \item{\code{jitter.x=FALSE}:}{ Logical scalar. Toggle on jittering on the x
#' axis in xy-type plots. See \code{\link{panel.xyplot}} for details.}
#'
#' \item{\code{jitter.y=FALSE}:}{ Logical scalar. Toggle off jittering on the y
#' axis in xy-type plots. See \code{\link{panel.xyplot}} for details.}
#'
#' \item{\code{levels.fos=NULL}:}{ Numeric scalar. Parameter controlling the
#' boxplot appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{legend=TRUE}:}{ Boolean triggering the addition of a legend to
#' the track to indicate groups. This only has an effect if at least two groups
#' are present.}
#'
#' \item{\code{lineheight.legend=NULL}:}{ Numeric scalar. The line height for
#' the legend text.}
#'
#' \item{\code{lty.baseline=NULL}:}{ Character or numeric scalar.  Line type of
#' the optional baseline, defaults to the setting of \code{lty}.}
#'
#' \item{\code{lty.mountain=NULL}:}{ Character or numeric scalar.  Line type in
#' mountain-type and polygon-type plots, defaults to the setting of
#' \code{lty}.}
#'
#' \item{\code{lwd.baseline=NULL}:}{ Numeric scalar. Line width of the optional
#' baseline, defaults to the setting of \code{lwd}.}
#'
#' \item{\code{lwd.mountain=NULL}:}{ Numeric scalar. Line width in
#' mountain-type and polygon-type plots, defaults to the setting of
#' \code{lwd}.}
#'
#' \item{\code{min.distance=0}:}{ Numeric scalar. The mimimum distance in pixel
#' below which to collapse ranges.}
#'
#' \item{\code{na.rm=FALSE}:}{ Boolean controlling whether to discard all NA
#' values when plotting or to keep empty spaces for NAs}
#'
#' \item{\code{ncolor=100}:}{ Integer scalar. The number of colors for the
#' 'gradient' plotting type}
#'
#' \item{\code{notch.frac=0.5}:}{ Numeric scalar. Parameter controlling the
#' boxplot appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{notch=FALSE}:}{ Logical scalar. Parameter controlling the
#' boxplot appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{pch=20}:}{ Integer scalar. The type of glyph used for plotting
#' symbols.}
#'
#' \item{\code{separator=0}:}{ Numeric scalar. Number of pixels used to
#' separate individual samples in heatmap- and horizon-type plots.}
#'
#' \item{\code{showColorBar=TRUE}:}{ Boolean. Indicate the data range color
#' mapping in the axis for 'heatmap' or 'gradient' types.}
#'
#' \item{\code{showSampleNames=FALSE}:}{ Boolean. Display the names of the
#' individual samples in a heatmap or a horizon plot.}
#'
#' \item{\code{size=NULL}:}{ Numeric scalar. The relative size of the track.
#' Can be overridden in the \code{\link{plotTracks}} function. By default the
#' size will be set automatically based on the selected plotting type.}
#'
#' \item{\code{span=0.2}:}{ Numeric scalar. Parameter controlling the loess
#' calculation for smooth and mountain-type plots.  See
#' \code{\link{panel.loess}} for details.}
#'
#' \item{\code{stackedBars=TRUE}:}{ Logical scalar. When there are several data
#' groups, draw the histogram-type plots as stacked barplots or grouped side by
#' side.}
#'
#' \item{\code{stats=X[[i]]}:}{ Function. Parameter controlling the boxplot
#' appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{transformation=NULL}:}{ Function. Applied to the data matrix
#' prior to plotting or when calling the \code{score} method. The function
#' should accept exactly one input argument and its return value needs to be a
#' numeric vector which can be coerced back into a data matrix of identical
#' dimensionality as the input data.}
#'
#' \item{\code{type="p"}:}{ Character vector. The plot type, one or several in
#' \code{p},\code{l}, \code{b}, \code{a}, \code{a_confint}, \code{s}, \code{g},
#' \code{r}, \code{S}, \code{confint}, \code{smooth}, \code{histogram},
#' \code{mountain}, \code{polygon}, \code{h}, \code{boxplot}, \code{gradient},
#' \code{heatmap}, \code{horizon}. See 'Details' section in
#' \code{\linkS4class{DataTrack}} for more information on the individual
#' plotting types.}
#'
#' \item{\code{varwidth=FALSE}:}{ Logical scalar. Parameter controlling the
#' boxplot appearance. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{window=NULL}:}{ Numeric or character scalar.  Aggregate the rows
#' values of the data matrix to \code{window} equally sized slices on the data
#' range using the method defined in \code{aggregation}. If negative, apply a
#' running window of size \code{windowSize} using the same aggregation method.
#' Alternatively, the special value \code{auto} causes the function to
#' determine the optimal window size to avoid overplotting, and \code{fixed}
#' uses fixed-size windows of size \code{windowSize}.}
#'
#' \item{\code{windowSize=NULL}:}{ Numeric scalar. The size of the running
#' window when the value of \code{window} is negative.}
#'
#' \item{\code{ylim=NULL}:}{ Numeric vector of length 2. The range of the
#' y-axis scale.}
#'
#' \item{\code{yTicksAt=NULL}:}{ Numeric vector. The points at which y-axis
#' tick-marks are to be drawn. By default, when \code{NULL}, tickmark locations
#' are computed.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GdObject:}}
#'
#' \describe{
#'
#' \item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
#' items.}
#'
#' \item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
#' title panel.}
#'
#' \item{\code{background.panel="transparent"}:}{ Integer or character scalar.
#' The background color of the content panel.}
#'
#' \item{\code{background.title="lightgray"}:}{ Integer or character scalar.
#' The background color for the title panel.}
#'
#' \item{\code{background.legend="transparent"}:}{ Integer or character scalar.
#' The background color for the legend.}
#'
#' \item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
#' axis annotation. Defaults to \code{NULL}, in which case it is automatically
#' determined based on the available space.}
#'
#' \item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
#' title panel. This effects the fontsize of both the title and the axis, if
#' any. Defaults to \code{NULL}, which means that the text size is
#' automatically adjusted to the available space.}
#'
#' \item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
#' line color for the y axis, if any.}
#'
#' \item{\code{col.border.title="white"}:}{ Integer or character scalar. The
#' border color for the title panels.}
#'
#' \item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
#' color used for the panel frame, if \code{frame==TRUE}}
#'
#' \item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
#' line color for grid lines, both when \code{type=="g"} in
#' \code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
#' for plot lines. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
#' for plot symbols. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.title="white"}:}{ Integer or character scalar.  The border
#' color for the title panels}
#'
#' \item{\code{fill="lightgray"}:}{ Integer or character scalar.  Default fill
#' color setting for all plotting elements, unless there is a more specific
#' control defined elsewhere.}
#'
#' \item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
#' color for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
#' for the title panels.}
#'
#' \item{\code{fontface=1}:}{ Integer or character scalar. The font face for
#' all text, unless a more specific definition exists.}
#'
#' \item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
#' font family for the title panels.}
#'
#' \item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
#' family for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
#' unless a more specific definition exists.}
#'
#' \item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
#' plotting.}
#'
#' \item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
#' horizontal grid lines, see \code{\link{panel.grid}} for details.}
#'
#' \item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
#' text, unless a more specific definition exists.}
#'
#' \item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
#' type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
#' and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lty="solid"}:}{ Numeric scalar. Default line type setting for
#' all plotting elements, unless there is a more specific control defined
#' elsewhere.}
#'
#' \item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
#' title panels.}
#'
#' \item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
#' panels}
#'
#' \item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
#' lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
#' display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lwd=1}:}{ Numeric scalar. Default line width setting for all
#' plotting elements, unless there is a more specific control defined
#' elsewhere.}
#'
#' \item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
#' coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
#' mirror the plot on the vertical axis.}
#'
#' \item{\code{rotation.title=90}:}{ The rotation angle for the text in the
#' title panel. Even though this can be adjusted, the automatic resizing of the
#' title panel will currently not work, so use at own risk.}
#'
#' \item{\code{rotation=0}:}{ The rotation angle for all text unless a more
#' specific definiton exists.}
#'
#' \item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
#' (only applies to track types where axes are implemented).}
#'
#' \item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
#' panel. Although this can be set individually for each track, in multi-track
#' plots as created by \code{\link{plotTracks}} there will still be an empty
#' placeholder in case any of the other tracks include a title.  The same holds
#' true for axes. Note that the the title panel background color could be set
#' to transparent in order to completely hide it.}
#'
#' \item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
#' vertical grid lines, see \code{\link{panel.grid}} for details.}
#'
#' }
#'
#' }
#'
#' \item{IdeogramTrack:}{
#'
#' \describe{
#'
#' \item{\code{background.title="transparent"}:}{ Character scalar.  The
#' background color for the title panel. Defaults to omit the background.}
#'
#' \item{\code{bevel=0.45}:}{ Numeric scalar, between 0 and 1.  The level of
#' smoothness for the two ends of the ideogram.}
#'
#' \item{\code{centromereShape="triangle"}:}{ Character scalar.  The shape of
#' the centromere. Only "triangle" or "circle" is accepted. Default to
#' "triangle"}
#'
#' \item{\code{cex.bands=0.7}:}{ Numeric scalar. The font expansion factor for
#' the chromosome band identifier text.}
#'
#' \item{\code{cex=0.8}:}{ Numeric scalar. The overall font expansion factor
#' for the chromosome name text.}
#'
#' \item{\code{col="red"}:}{ Character scalar. The border color used for the
#' highlighting of the currently displayed genomic region.}
#'
#' \item{\code{col.border.title="transparent"}:}{ Integer or character scalar.
#' The border color for the title panels.}
#'
#' \item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
#' title panels.}
#'
#' \item{\code{fill="#FFE3E6"}:}{ Character scalar. The fill color used for the
#' highlighting of the currently displayed genomic region.}
#'
#' \item{\code{fontface=1}:}{ Character scalar. The font face for the
#' chromosome name text.}
#'
#' \item{\code{fontfamily="sans"}:}{ Character scalar. The font family for the
#' chromosome name text.}
#'
#' \item{\code{fontcolor="#808080"}:}{ Character scalar. The font color for the
#' chromosome name text.}
#'
#' \item{\code{fontsize=10}:}{ Numeric scalar. The font size for the chromosome
#' name text.}
#'
#' \item{\code{outline=FALSE}:}{ Logical scalar. Add borders to the individual
#' chromosome staining bands.}
#'
#' \item{\code{showBandId=FALSE}:}{ Logical scalar. Show the identifier for the
#' chromosome bands if there is space for it.}
#'
#' \item{\code{lty=1}:}{ Character or integer scalar. The line type used for
#' the highlighting of the currently displayed genomic region.}
#'
#' \item{\code{lwd=1}:}{ Numeric scalar. The line width used for the
#' highlighting of the currently displayed genomic region.}
#'
#' \item{\code{showId=TRUE}:}{ Logical scalar. Indicate the chromosome name
#' next to the ideogram.}
#'
#' \item{\code{showTitle=FALSE}:}{ Logical scalar. Plot a title panel. Defaults
#' to omit the title panel.}
#'
#' \item{\code{size=NULL}:}{ Numeric scalar. The relative size of the track.
#' Defaults to automatic size setting. Can also be overridden in the
#' \code{\link{plotTracks}} function.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GdObject:}}
#'
#' \describe{
#'
#' \item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
#' items.}
#'
#' \item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
#' title panel.}
#'
#' \item{\code{background.panel="transparent"}:}{ Integer or character scalar.
#' The background color of the content panel.}
#'
#' \item{\code{background.legend="transparent"}:}{ Integer or character scalar.
#' The background color for the legend.}
#'
#' \item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
#' axis annotation. Defaults to \code{NULL}, in which case it is automatically
#' determined based on the available space.}
#'
#' \item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
#' title panel. This effects the fontsize of both the title and the axis, if
#' any. Defaults to \code{NULL}, which means that the text size is
#' automatically adjusted to the available space.}
#'
#' \item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
#' line color for the y axis, if any.}
#'
#' \item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
#' color used for the panel frame, if \code{frame==TRUE}}
#'
#' \item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
#' line color for grid lines, both when \code{type=="g"} in
#' \code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
#' for plot lines. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
#' for plot symbols. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.title="white"}:}{ Integer or character scalar.  The border
#' color for the title panels}
#'
#' \item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
#' content of the track to accomodate the minimum current device resolution.
#' See \code{\link{collapsing}} for details.}
#'
#' \item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
#' for the title panels.}
#'
#' \item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
#' font family for the title panels.}
#'
#' \item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
#' plotting.}
#'
#' \item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
#' grid.}
#'
#' \item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
#' horizontal grid lines, see \code{\link{panel.grid}} for details.}
#'
#' \item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
#' text, unless a more specific definition exists.}
#'
#' \item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
#' type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
#' and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
#' panels}
#'
#' \item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
#' lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
#' display parameter \code{grid==TRUE}.}
#'
#' \item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
#' before collapsing range items, only if \code{collapse==TRUE}. See
#' \code{\link{collapsing}} for details.}
#'
#' \item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
#' coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
#' mirror the plot on the vertical axis.}
#'
#' \item{\code{rotation.title=90}:}{ The rotation angle for the text in the
#' title panel. Even though this can be adjusted, the automatic resizing of the
#' title panel will currently not work, so use at own risk.}
#'
#' \item{\code{rotation=0}:}{ The rotation angle for all text unless a more
#' specific definiton exists.}
#'
#' \item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
#' (only applies to track types where axes are implemented).}
#'
#' \item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
#' vertical grid lines, see \code{\link{panel.grid}} for details.}
#'
#' }
#'
#' }
#'
#' \item{AnnotationTrack:}{
#'
#' \describe{
#'
#' \item{\code{arrowHeadWidth=30}:}{ Numeric scalar. The width of the arrow
#' head in pixels if \code{shape} is \code{fixedArrow}.}
#'
#' \item{\code{arrowHeadMaxWidth=40}:}{ Numeric scalar. The maximum width of
#' the arrow head in pixels if \code{shape} is \code{arrow}.}
#'
#' \item{\code{cex.group=0.6}:}{ Numeric scalar. The font expansion factor for
#' the group-level annotation.}
#'
#' \item{\code{cex=1}:}{ Numeric scalar. The font expansion factor for item
#' identifiers.}
#'
#' \item{\code{col.line="darkgray"}:}{ Character scalar. The color used for
#' connecting lines between grouped items. Defaults to a light gray, but if set
#' to \code{NULL} the same color as for the first item in the group is used.}
#'
#' \item{\code{col="transparent"}:}{ Character or integer scalar.  The border
#' color for all track items.}
#'
#' \item{\code{featureAnnotation=NULL}:}{ Character scalar. Add annotation
#' information to the individual track elements. This can be a value in
#' \code{id}, \code{group} or \code{feature}.  Defaults to \code{id}. Only
#' works if \code{showFeatureId} is not \code{FALSE}.}
#'
#' \item{\code{fill="lightblue"}:}{ Character or integer scalar.  The fill
#' color for untyped items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{fontfamily.group="sans"}:}{ Character scalar. The font family
#' for the group-level annotation.}
#'
#' \item{\code{fontcolor.group="#808080"}:}{ Character or integer scalar. The
#' font color for the group-level annotation.}
#'
#' \item{\code{fontcolor.item="white"}:}{ Character or integer scalar. The font
#' color for item identifiers.}
#'
#' \item{\code{fontface.group=2}:}{ Numeric scalar. The font face for the
#' group-level annotation.}
#'
#' \item{\code{fontsize.group=12}:}{ Numeric scalar. The font size for the
#' group-level annotation.}
#'
#' \item{\code{groupAnnotation=NULL}:}{ Character scalar. Add annotation
#' information as group labels. This can be a value in \code{id}, \code{group}
#' or \code{feature}. Defaults to \code{group}. Only works if \code{showId} is
#' not \code{FALSE}.}
#'
#' \item{\code{just.group="left"}:}{ Character scalar. the justification of
#' group labels. Either \code{left}, \code{right}, \code{above} or
#' \code{below}.}
#'
#' \item{\code{lex=1}:}{ Numeric scalar. The line expansion factor for all
#' track items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{lineheight=1}:}{ Numeric scalar. The font line height for item
#' identifiers.}
#'
#' \item{\code{lty="solid"}:}{ Character or integer scalar. The line type for
#' all track items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{lwd=1}:}{ Integer scalar. The line width for all track items.
#' This is also used to connect grouped items. See \code{\link{grouping}} for
#' details.}
#'
#' \item{\code{mergeGroups=FALSE}:}{ Logical scalar. Merge fully overlapping
#' groups if \code{collapse==TRUE}.}
#'
#' \item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details. For feathered
#' bars indicating the strandedness of grouped items this also controls the
#' height of the arrow feathers.}
#'
#' \item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{rotation=0}:}{ Numeric scalar. The degree of text rotation for
#' item identifiers.}
#'
#' \item{\code{rotation.group=0}:}{ Numeric scalar. The degree of text rotation
#' for group labels.}
#'
#' \item{\code{rotation.item=0}:}{ Numeric scalar. The degree of text rotation
#' for item identifiers.}
#'
#' \item{\code{shape="arrow"}:}{ Character scalar. The shape in which to
#' display the track items. Currently only \code{box}, \code{arrow},
#' \code{fixedArrow}, \code{ellipse}, and \code{smallArrow} are implemented.}
#'
#' \item{\code{showFeatureId=FALSE}:}{ Logical scalar. Control whether to plot
#' the individual track item identifiers.}
#'
#' \item{\code{showId=FALSE}:}{ Logical scalar. Control whether to annotate
#' individual groups.}
#'
#' \item{\code{showOverplotting=FALSE}:}{ Logical scalar. Use a color gradient
#' to show the amount of overplotting for collapsed items. This implies that
#' \code{collapse==TRUE}}
#'
#' \item{\code{size=1}:}{ Numeric scalar. The relative size of the track. Can
#' be overridden in the \code{\link{plotTracks}} function.}
#'
#' }
#'
#' \bold{\emph{Inherited from class StackedTrack:}}
#'
#' \describe{
#'
#' \item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
#' vertical size and spacing between stacked elements. The number defines the
#' proportion of the total available space for the stack that is used to draw
#' the glyphs.  E.g., a value of 0.5 means that half of the available vertical
#' drawing space (for each stacking line) is used for the glyphs, and thus one
#' quarter of the available space each is used for spacing above and below the
#' glyph. Defaults to 0.75.}
#'
#' \item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
#' of stacked items. I.e., features that are plotted on the bottom-most stacks
#' will be moved to the top-most stack and vice versa.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GdObject:}}
#'
#' \describe{
#'
#' \item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
#' items.}
#'
#' \item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
#' title panel.}
#'
#' \item{\code{background.panel="transparent"}:}{ Integer or character scalar.
#' The background color of the content panel.}
#'
#' \item{\code{background.title="lightgray"}:}{ Integer or character scalar.
#' The background color for the title panel.}
#'
#' \item{\code{background.legend="transparent"}:}{ Integer or character scalar.
#' The background color for the legend.}
#'
#' \item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
#' axis annotation. Defaults to \code{NULL}, in which case it is automatically
#' determined based on the available space.}
#'
#' \item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
#' title panel. This effects the fontsize of both the title and the axis, if
#' any. Defaults to \code{NULL}, which means that the text size is
#' automatically adjusted to the available space.}
#'
#' \item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
#' line color for the y axis, if any.}
#'
#' \item{\code{col.border.title="white"}:}{ Integer or character scalar. The
#' border color for the title panels.}
#'
#' \item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
#' color used for the panel frame, if \code{frame==TRUE}}
#'
#' \item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
#' line color for grid lines, both when \code{type=="g"} in
#' \code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
#' for plot symbols. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.title="white"}:}{ Integer or character scalar.  The border
#' color for the title panels}
#'
#' \item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
#' content of the track to accomodate the minimum current device resolution.
#' See \code{\link{collapsing}} for details.}
#'
#' \item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
#' color for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
#' for the title panels.}
#'
#' \item{\code{fontface=1}:}{ Integer or character scalar. The font face for
#' all text, unless a more specific definition exists.}
#'
#' \item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
#' font family for the title panels.}
#'
#' \item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
#' family for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
#' unless a more specific definition exists.}
#'
#' \item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
#' plotting.}
#'
#' \item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
#' grid.}
#'
#' \item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
#' horizontal grid lines, see \code{\link{panel.grid}} for details.}
#'
#' \item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
#' type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
#' and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
#' title panels.}
#'
#' \item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
#' panels}
#'
#' \item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
#' lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
#' display parameter \code{grid==TRUE}.}
#'
#' \item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
#' before collapsing range items, only if \code{collapse==TRUE}. See
#' \code{\link{collapsing}} for details.}
#'
#' \item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
#' coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
#' mirror the plot on the vertical axis.}
#'
#' \item{\code{rotation.title=90}:}{ The rotation angle for the text in the
#' title panel. Even though this can be adjusted, the automatic resizing of the
#' title panel will currently not work, so use at own risk.}
#'
#' \item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
#' (only applies to track types where axes are implemented).}
#'
#' \item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
#' panel. Although this can be set individually for each track, in multi-track
#' plots as created by \code{\link{plotTracks}} there will still be an empty
#' placeholder in case any of the other tracks include a title.  The same holds
#' true for axes. Note that the the title panel background color could be set
#' to transparent in order to completely hide it.}
#'
#' \item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
#' vertical grid lines, see \code{\link{panel.grid}} for details.}
#'
#' }
#'
#' }
#'
#' \item{GeneRegionTrack:}{
#'
#' \describe{
#'
#' \item{\code{arrowHeadWidth=10}:}{ Numeric scalar. The width of the arrow
#' head in pixels if \code{shape} is \code{fixedArrow}.}
#'
#' \item{\code{arrowHeadMaxWidth=20}:}{ Numeric scalar. The maximum width of
#' the arrow head in pixels if \code{shape} is \code{arrow}.}
#'
#' \item{\code{col=NULL}:}{ Character or integer scalar. The border color for
#' all track items. Defaults to using the same color as in \code{fill}, also
#' taking into account different track \code{features}.}
#'
#' \item{\code{collapseTranscripts=FALSE}:}{ Logical or character scalar. Can
#' be one in \code{gene}, \code{longest}, \code{shortest} or \code{meta}. Merge
#' all transcripts of the same gene into one single gene model. In the case of
#' \code{gene} (or \code{TRUE}), this will only keep the start location of the
#' first exon and the end location of the last exon from all transcripts of the
#' gene. For \code{shortest} and \code{longest}, only the longest or shortest
#' transcript model is retained.  For \code{meta}, a meta-transcript containing
#' the union of all exons is formed (essentially identical to the operation
#' \code{reduce(geneModel)}).}
#'
#' \item{\code{exonAnnotation=NULL}:}{ Character scalar. Add annotation
#' information to the individual exon models. This can be a value in
#' \code{symbol}, \code{gene}, \code{transcript}, \code{exon} or
#' \code{feature}. Defaults to \code{exon}. Only works if \code{showExonId} is
#' not \code{FALSE}.}
#'
#' \item{\code{fill="orange"}:}{ Character or integer scalar. The fill color
#' for untyped items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{min.distance=0}:}{ Numeric scalar. The minimum pixel distance
#' before collapsing range items, only if \code{collapse==TRUE}. See
#' \code{\link{collapsing}} for details. Note that a value larger than 0 may
#' lead to UTR regions being merged to CDS regions, which in most cases is not
#' particularly useful.}
#'
#' \item{\code{shape=c("smallArrow", "box")}:}{ Character scalar.  The shape in
#' which to display the track items. Currently only \code{box}, \code{arrow},
#' \code{ellipse}, and \code{smallArrow} are implemented.}
#'
#' \item{\code{showExonId=NULL}:}{ Logical scalar. Control whether to plot the
#' individual exon identifiers.}
#'
#' \item{\code{thinBoxFeature=c("utr", "ncRNA", "utr3", "utr5", "3UTR",
#' "5UTR", "miRNA", "lincRNA", "three_prime_UTR", "five_prime_UTR")}:}{ Character
#' vector. A listing of feature types that should be drawn with thin boxes.
#' Typically those are non-coding elements.}
#'
#' \item{\code{transcriptAnnotation=NULL}:}{ Character scalar.  Add annotation
#' information as transcript labels. This can be a value in \code{symbol},
#' \code{gene}, \code{transcript}, \code{exon} or \code{feature}. Defaults to
#' \code{symbol}.  Only works if \code{showId} is not \code{FALSE}.}
#'
#' }
#'
#' \bold{\emph{Inherited from class AnnotationTrack:}}
#'
#' \describe{
#'
#' \item{\code{cex.group=0.6}:}{ Numeric scalar. The font expansion factor for
#' the group-level annotation.}
#'
#' \item{\code{cex=1}:}{ Numeric scalar. The font expansion factor for item
#' identifiers.}
#'
#' \item{\code{col.line="darkgray"}:}{ Character scalar. The color used for
#' connecting lines between grouped items. Defaults to a light gray, but if set
#' to \code{NULL} the same color as for the first item in the group is used.}
#'
#' \item{\code{featureAnnotation=NULL}:}{ Character scalar. Add annotation
#' information to the individual track elements. This can be a value in
#' \code{id}, \code{group} or \code{feature}.  Defaults to \code{id}. Only
#' works if \code{showFeatureId} is not \code{FALSE}.}
#'
#' \item{\code{fontfamily.group="sans"}:}{ Character scalar. The font family
#' for the group-level annotation.}
#'
#' \item{\code{fontcolor.group="#808080"}:}{ Character or integer scalar. The
#' font color for the group-level annotation.}
#'
#' \item{\code{fontcolor.item="white"}:}{ Character or integer scalar. The font
#' color for item identifiers.}
#'
#' \item{\code{fontface.group=2}:}{ Numeric scalar. The font face for the
#' group-level annotation.}
#'
#' \item{\code{fontsize.group=12}:}{ Numeric scalar. The font size for the
#' group-level annotation.}
#'
#' \item{\code{groupAnnotation=NULL}:}{ Character scalar. Add annotation
#' information as group labels. This can be a value in \code{id}, \code{group}
#' or \code{feature}. Defaults to \code{group}. Only works if \code{showId} is
#' not \code{FALSE}.}
#'
#' \item{\code{just.group="left"}:}{ Character scalar. the justification of
#' group labels. Either \code{left}, \code{right}, \code{above} or
#' \code{below}.}
#'
#' \item{\code{lex=1}:}{ Numeric scalar. The line expansion factor for all
#' track items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{lineheight=1}:}{ Numeric scalar. The font line height for item
#' identifiers.}
#'
#' \item{\code{lty="solid"}:}{ Character or integer scalar. The line type for
#' all track items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{lwd=1}:}{ Integer scalar. The line width for all track items.
#' This is also used to connect grouped items. See \code{\link{grouping}} for
#' details.}
#'
#' \item{\code{mergeGroups=FALSE}:}{ Logical scalar. Merge fully overlapping
#' groups if \code{collapse==TRUE}.}
#'
#' \item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details. For feathered
#' bars indicating the strandedness of grouped items this also controls the
#' height of the arrow feathers.}
#'
#' \item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{rotation=0}:}{ Numeric scalar. The degree of text rotation for
#' item identifiers.}
#'
#' \item{\code{rotation.group=0}:}{ Numeric scalar. The degree of text rotation
#' for group labels.}
#'
#' \item{\code{rotation.item=0}:}{ Numeric scalar. The degree of text rotation
#' for item identifiers.}
#'
#' \item{\code{showFeatureId=FALSE}:}{ Logical scalar. Control whether to plot
#' the individual track item identifiers.}
#'
#' \item{\code{showId=FALSE}:}{ Logical scalar. Control whether to annotate
#' individual groups.}
#'
#' \item{\code{showOverplotting=FALSE}:}{ Logical scalar. Use a color gradient
#' to show the amount of overplotting for collapsed items. This implies that
#' \code{collapse==TRUE}}
#'
#' \item{\code{size=1}:}{ Numeric scalar. The relative size of the track. Can
#' be overridden in the \code{\link{plotTracks}} function.}
#'
#' }
#'
#' \bold{\emph{Inherited from class StackedTrack:}}
#'
#' \describe{
#'
#' \item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
#' vertical size and spacing between stacked elements. The number defines the
#' proportion of the total available space for the stack that is used to draw
#' the glyphs.  E.g., a value of 0.5 means that half of the available vertical
#' drawing space (for each stacking line) is used for the glyphs, and thus one
#' quarter of the available space each is used for spacing above and below the
#' glyph. Defaults to 0.75.}
#'
#' \item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
#' of stacked items. I.e., features that are plotted on the bottom-most stacks
#' will be moved to the top-most stack and vice versa.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GdObject:}}
#'
#' \describe{
#'
#' \item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
#' items.}
#'
#' \item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
#' title panel.}
#'
#' \item{\code{background.panel="transparent"}:}{ Integer or character scalar.
#' The background color of the content panel.}
#'
#' \item{\code{background.title="lightgray"}:}{ Integer or character scalar.
#' The background color for the title panel.}
#'
#' \item{\code{background.legend="transparent"}:}{ Integer or character scalar.
#' The background color for the legend.}
#'
#' \item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
#' axis annotation. Defaults to \code{NULL}, in which case it is automatically
#' determined based on the available space.}
#'
#' \item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
#' title panel. This effects the fontsize of both the title and the axis, if
#' any. Defaults to \code{NULL}, which means that the text size is
#' automatically adjusted to the available space.}
#'
#' \item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
#' line color for the y axis, if any.}
#'
#' \item{\code{col.border.title="white"}:}{ Integer or character scalar. The
#' border color for the title panels.}
#'
#' \item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
#' color used for the panel frame, if \code{frame==TRUE}}
#'
#' \item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
#' line color for grid lines, both when \code{type=="g"} in
#' \code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
#' for plot symbols. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.title="white"}:}{ Integer or character scalar.  The border
#' color for the title panels}
#'
#' \item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
#' content of the track to accomodate the minimum current device resolution.
#' See \code{\link{collapsing}} for details.}
#'
#' \item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
#' color for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
#' for the title panels.}
#'
#' \item{\code{fontface=1}:}{ Integer or character scalar. The font face for
#' all text, unless a more specific definition exists.}
#'
#' \item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
#' font family for the title panels.}
#'
#' \item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
#' family for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
#' unless a more specific definition exists.}
#'
#' \item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
#' plotting.}
#'
#' \item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
#' grid.}
#'
#' \item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
#' horizontal grid lines, see \code{\link{panel.grid}} for details.}
#'
#' \item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
#' type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
#' and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
#' title panels.}
#'
#' \item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
#' panels}
#'
#' \item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
#' lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
#' display parameter \code{grid==TRUE}.}
#'
#' \item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
#' coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
#' mirror the plot on the vertical axis.}
#'
#' \item{\code{rotation.title=90}:}{ The rotation angle for the text in the
#' title panel. Even though this can be adjusted, the automatic resizing of the
#' title panel will currently not work, so use at own risk.}
#'
#' \item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
#' (only applies to track types where axes are implemented).}
#'
#' \item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
#' panel. Although this can be set individually for each track, in multi-track
#' plots as created by \code{\link{plotTracks}} there will still be an empty
#' placeholder in case any of the other tracks include a title.  The same holds
#' true for axes. Note that the the title panel background color could be set
#' to transparent in order to completely hide it.}
#'
#' \item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
#' vertical grid lines, see \code{\link{panel.grid}} for details.}
#'
#' }
#'
#' }
#'
#' \item{BiomartGeneRegionTrack:}{
#'
#' \describe{
#'
#' \item{\code{C_segment="burlywood4"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'C_segment'.}
#'
#' \item{\code{D_segment="lightblue"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'C_segment'.}
#'
#' \item{\code{J_segment="dodgerblue2"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'C_segment'.}
#'
#' \item{\code{Mt_rRNA="yellow"}:}{ Character or integer scalar.  Fill color
#' for annotation objects of type 'Mt_rRNA'.}
#'
#' \item{\code{Mt_tRNA="darkgoldenrod"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'Mt_tRNA'.}
#'
#' \item{\code{Mt_tRNA_pseudogene="darkgoldenrod1"}:}{ Character or integer
#' scalar. Fill color for annotation objects of type 'Mt_tRNA_pseudogene'.}
#'
#' \item{\code{V_segment="aquamarine"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'V_segment'.}
#'
#' \item{\code{miRNA="cornflowerblue"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'L_segment'.}
#'
#' \item{\code{miRNA_pseudogene="cornsilk"}:}{ Character or integer scalar.
#' Fill color for annotation objects of type 'miRNA_pseudogene'.}
#'
#' \item{\code{misc_RNA="cornsilk3"}:}{ Character or integer scalar. Fill color
#' for annotation objects of type 'misc_RNA'.}
#'
#' \item{\code{misc_RNA_pseudogene="cornsilk4"}:}{ Character or integer scalar.
#' Fill color for annotation objects of type 'misc_RNA_pseudogene'.}
#'
#' \item{\code{protein_coding="#FFD58A"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'protein_coding'.}
#'
#' \item{\code{pseudogene="brown1"}:}{ Character or integer scalar.  Fill color
#' for annotation objects of type 'pseudogene'.}
#'
#' \item{\code{rRNA="darkolivegreen1"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'rRNA'.}
#'
#' \item{\code{rRNA_pseudogene="darkolivegreen"}:}{ Character or integer
#' scalar. Fill color for annotation objects of type 'rRNA_pseudogene'.}
#'
#' \item{\code{retrotransposed="blueviolet"}:}{ Character or integer scalar.
#' Fill color for annotation objects of type 'retrotransposed'.}
#'
#' \item{\code{scRNA="gold4"}:}{ Character or integer scalar. Fill color for
#' annotation objects of type 'scRNA'.}
#'
#' \item{\code{scRNA_pseudogene="darkorange2"}:}{ Character or integer scalar.
#' Fill color for annotation objects of type 'scRNA_pseudogene'.}
#'
#' \item{\code{snRNA="coral"}:}{ Character or integer scalar. Fill color for
#' annotation objects of type 'snRNA'.}
#'
#' \item{\code{snRNA_pseudogene="coral3"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'snRNA_pseudogene'.}
#'
#' \item{\code{snoRNA="cyan"}:}{ Character or integer scalar. Fill color for
#' annotation objects of type 'snoRNA'.}
#'
#' \item{\code{snoRNA_pseudogene="cyan2"}:}{ Character or integer scalar. Fill
#' color for annotation objects of type 'snoRNA_pseudogene'.}
#'
#' \item{\code{tRNA_pseudogene="antiquewhite3"}:}{ Character or integer scalar.
#' Fill color for annotation objects of type 'tRNA_pseudogene'.}
#'
#' \item{\code{utr3="#FFD58A"}:}{ Character or integer scalar.  Fill color for
#' annotation objects of type 'utr3'.}
#'
#' \item{\code{utr5="#FFD58A"}:}{ Character or integer scalar.  Fill color for
#' annotation objects of type 'utr5'.}
#'
#' \item{\code{verbose=FALSE}:}{ Logical scalar. Report data loading events
#' from Bioamart or retrieval from cache.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GeneRegionTrack:}}
#'
#' \describe{
#'
#' \item{\code{arrowHeadWidth=10}:}{ Numeric scalar. The width of the arrow
#' head in pixels if \code{shape} is \code{fixedArrow}.}
#'
#' \item{\code{arrowHeadMaxWidth=20}:}{ Numeric scalar. The maximum width of
#' the arrow head in pixels if \code{shape} is \code{arrow}.}
#'
#' \item{\code{col=NULL}:}{ Character or integer scalar. The border color for
#' all track items. Defaults to using the same color as in \code{fill}, also
#' taking into account different track \code{features}.}
#'
#' \item{\code{collapseTranscripts=FALSE}:}{ Logical or character scalar. Can
#' be one in \code{gene}, \code{longest}, \code{shortest} or \code{meta}. Merge
#' all transcripts of the same gene into one single gene model. In the case of
#' \code{gene} (or \code{TRUE}), this will only keep the start location of the
#' first exon and the end location of the last exon from all transcripts of the
#' gene. For \code{shortest} and \code{longest}, only the longest or shortest
#' transcript model is retained.  For \code{meta}, a meta-transcript containing
#' the union of all exons is formed (essentially identical to the operation
#' \code{reduce(geneModel)}).}
#'
#' \item{\code{exonAnnotation=NULL}:}{ Character scalar. Add annotation
#' information to the individual exon models. This can be a value in
#' \code{symbol}, \code{gene}, \code{transcript}, \code{exon} or
#' \code{feature}. Defaults to \code{exon}. Only works if \code{showExonId} is
#' not \code{FALSE}.}
#'
#' \item{\code{fill="orange"}:}{ Character or integer scalar. The fill color
#' for untyped items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{min.distance=0}:}{ Numeric scalar. The minimum pixel distance
#' before collapsing range items, only if \code{collapse==TRUE}. See
#' \code{\link{collapsing}} for details. Note that a value larger than 0 may
#' lead to UTR regions being merged to CDS regions, which in most cases is not
#' particularly useful.}
#'
#' \item{\code{shape=c("smallArrow", "box")}:}{ Character scalar.  The shape in
#' which to display the track items. Currently only \code{box}, \code{arrow},
#' \code{ellipse}, and \code{smallArrow} are implemented.}
#'
#' \item{\code{showExonId=NULL}:}{ Logical scalar. Control whether to plot the
#' individual exon identifiers.}
#'
#' \item{\code{thinBoxFeature=c("utr", "ncRNA", "utr3", "utr5", "3UTR",
#' "5UTR", "miRNA", "lincRNA", "three_prime_UTR", "five_prime_UTR")}:}{ Character
#' vector. A listing of feature types that should be drawn with thin boxes.
#' Typically those are non-coding elements.}
#'
#' \item{\code{transcriptAnnotation=NULL}:}{ Character scalar.  Add annotation
#' information as transcript labels. This can be a value in \code{symbol},
#' \code{gene}, \code{transcript}, \code{exon} or \code{feature}. Defaults to
#' \code{symbol}.  Only works if \code{showId} is not \code{FALSE}.}
#'
#' }
#'
#' \bold{\emph{Inherited from class AnnotationTrack:}}
#'
#' \describe{
#'
#' \item{\code{cex.group=0.6}:}{ Numeric scalar. The font expansion factor for
#' the group-level annotation.}
#'
#' \item{\code{cex=1}:}{ Numeric scalar. The font expansion factor for item
#' identifiers.}
#'
#' \item{\code{col.line="darkgray"}:}{ Character scalar. The color used for
#' connecting lines between grouped items. Defaults to a light gray, but if set
#' to \code{NULL} the same color as for the first item in the group is used.}
#'
#' \item{\code{featureAnnotation=NULL}:}{ Character scalar. Add annotation
#' information to the individual track elements. This can be a value in
#' \code{id}, \code{group} or \code{feature}.  Defaults to \code{id}. Only
#' works if \code{showFeatureId} is not \code{FALSE}.}
#'
#' \item{\code{fontfamily.group="sans"}:}{ Character scalar. The font family
#' for the group-level annotation.}
#'
#' \item{\code{fontcolor.group="#808080"}:}{ Character or integer scalar. The
#' font color for the group-level annotation.}
#'
#' \item{\code{fontcolor.item="white"}:}{ Character or integer scalar. The font
#' color for item identifiers.}
#'
#' \item{\code{fontface.group=2}:}{ Numeric scalar. The font face for the
#' group-level annotation.}
#'
#' \item{\code{fontsize.group=12}:}{ Numeric scalar. The font size for the
#' group-level annotation.}
#'
#' \item{\code{groupAnnotation=NULL}:}{ Character scalar. Add annotation
#' information as group labels. This can be a value in \code{id}, \code{group}
#' or \code{feature}. Defaults to \code{group}. Only works if \code{showId} is
#' not \code{FALSE}.}
#'
#' \item{\code{just.group="left"}:}{ Character scalar. the justification of
#' group labels. Either \code{left}, \code{right}, \code{above} or
#' \code{below}.}
#'
#' \item{\code{lex=1}:}{ Numeric scalar. The line expansion factor for all
#' track items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{lineheight=1}:}{ Numeric scalar. The font line height for item
#' identifiers.}
#'
#' \item{\code{lty="solid"}:}{ Character or integer scalar. The line type for
#' all track items. This is also used to connect grouped items. See
#' \code{\link{grouping}} for details.}
#'
#' \item{\code{lwd=1}:}{ Integer scalar. The line width for all track items.
#' This is also used to connect grouped items. See \code{\link{grouping}} for
#' details.}
#'
#' \item{\code{mergeGroups=FALSE}:}{ Logical scalar. Merge fully overlapping
#' groups if \code{collapse==TRUE}.}
#'
#' \item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details. For feathered
#' bars indicating the strandedness of grouped items this also controls the
#' height of the arrow feathers.}
#'
#' \item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{rotation=0}:}{ Numeric scalar. The degree of text rotation for
#' item identifiers.}
#'
#' \item{\code{rotation.group=0}:}{ Numeric scalar. The degree of text rotation
#' for group labels.}
#'
#' \item{\code{rotation.item=0}:}{ Numeric scalar. The degree of text rotation
#' for item identifiers.}
#'
#' \item{\code{showFeatureId=FALSE}:}{ Logical scalar. Control whether to plot
#' the individual track item identifiers.}
#'
#' \item{\code{showId=FALSE}:}{ Logical scalar. Control whether to annotate
#' individual groups.}
#'
#' \item{\code{showOverplotting=FALSE}:}{ Logical scalar. Use a color gradient
#' to show the amount of overplotting for collapsed items. This implies that
#' \code{collapse==TRUE}}
#'
#' \item{\code{size=1}:}{ Numeric scalar. The relative size of the track. Can
#' be overridden in the \code{\link{plotTracks}} function.}
#'
#' }
#'
#' \bold{\emph{Inherited from class StackedTrack:}}
#'
#' \describe{
#'
#' \item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
#' vertical size and spacing between stacked elements. The number defines the
#' proportion of the total available space for the stack that is used to draw
#' the glyphs.  E.g., a value of 0.5 means that half of the available vertical
#' drawing space (for each stacking line) is used for the glyphs, and thus one
#' quarter of the available space each is used for spacing above and below the
#' glyph. Defaults to 0.75.}
#'
#' \item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
#' of stacked items. I.e., features that are plotted on the bottom-most stacks
#' will be moved to the top-most stack and vice versa.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GdObject:}}
#'
#' \describe{
#'
#' \item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
#' items.}
#'
#' \item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
#' title panel.}
#'
#' \item{\code{background.panel="transparent"}:}{ Integer or character scalar.
#' The background color of the content panel.}
#'
#' \item{\code{background.title="lightgray"}:}{ Integer or character scalar.
#' The background color for the title panel.}
#'
#' \item{\code{background.legend="transparent"}:}{ Integer or character scalar.
#' The background color for the legend.}
#'
#' \item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
#' axis annotation. Defaults to \code{NULL}, in which case it is automatically
#' determined based on the available space.}
#'
#' \item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
#' title panel. This effects the fontsize of both the title and the axis, if
#' any. Defaults to \code{NULL}, which means that the text size is
#' automatically adjusted to the available space.}
#'
#' \item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
#' line color for the y axis, if any.}
#'
#' \item{\code{col.border.title="white"}:}{ Integer or character scalar. The
#' border color for the title panels.}
#'
#' \item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
#' color used for the panel frame, if \code{frame==TRUE}}
#'
#' \item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
#' line color for grid lines, both when \code{type=="g"} in
#' \code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
#' for plot symbols. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.title="white"}:}{ Integer or character scalar.  The border
#' color for the title panels}
#'
#' \item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
#' content of the track to accomodate the minimum current device resolution.
#' See \code{\link{collapsing}} for details.}
#'
#' \item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
#' color for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
#' for the title panels.}
#'
#' \item{\code{fontface=1}:}{ Integer or character scalar. The font face for
#' all text, unless a more specific definition exists.}
#'
#' \item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
#' font family for the title panels.}
#'
#' \item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
#' family for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
#' unless a more specific definition exists.}
#'
#' \item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
#' plotting.}
#'
#' \item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
#' grid.}
#'
#' \item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
#' horizontal grid lines, see \code{\link{panel.grid}} for details.}
#'
#' \item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
#' type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
#' and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
#' title panels.}
#'
#' \item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
#' panels}
#'
#' \item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
#' lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
#' display parameter \code{grid==TRUE}.}
#'
#' \item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
#' coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
#' mirror the plot on the vertical axis.}
#'
#' \item{\code{rotation.title=90}:}{ The rotation angle for the text in the
#' title panel. Even though this can be adjusted, the automatic resizing of the
#' title panel will currently not work, so use at own risk.}
#'
#' \item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
#' (only applies to track types where axes are implemented).}
#'
#' \item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
#' panel. Although this can be set individually for each track, in multi-track
#' plots as created by \code{\link{plotTracks}} there will still be an empty
#' placeholder in case any of the other tracks include a title.  The same holds
#' true for axes. Note that the the title panel background color could be set
#' to transparent in order to completely hide it.}
#'
#' \item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
#' vertical grid lines, see \code{\link{panel.grid}} for details.}
#'
#' }
#'
#' }
#'
#' \item{AlignmentsTrack:}{
#'
#' \describe{
#'
#' \item{\code{alpha.reads=0.5}:}{ Numeric scalar between 0 and 1. The
#' transparency of the individual read icons. Can be used to indicate
#' overlapping regions in read pairs. Only on supported devices.}
#'
#' \item{\code{alpha.mismatch=1}:}{ Numeric scalar between 0 and 1. The
#' transparency of the mismatch base information.}
#'
#' \item{\code{cex=0.7}:}{ Numeric Scalar. The global character expansion
#' factor.}
#'
#' \item{\code{cex.mismatch=NULL}:}{ Numeric Scalar. The character expansion
#' factor for the mismatch base letters.}
#'
#' \item{\code{col.coverage=NULL}:}{ Integer or character scalar.  The line
#' color for the coverage profile.}
#'
#' \item{\code{col.gap="#808080"}:}{ Integer or character scalar.  The color of
#' the line that is bridging the gap regions in gapped alignments.}
#'
#' \item{\code{col.mates="#E0E0E0"}:}{ Integer or character scalar.  The color
#' of the line that is connecting two paired reads.}
#'
#' \item{\code{col.deletion="#000000"}:}{ Integer or character scalar. The
#' color of the line that is bridging the deleted regions in alignments.}
#'
#' \item{\code{col.insertion="#984EA3"}:}{ Integer or character scalar. The
#' color of the line that highlighting insertions in alignments.}
#'
#' \item{\code{col.mismatch="#808080"}:}{ Integer or character scalar. The box
#' color around mismatch bases.}
#'
#' \item{\code{col.reads=NULL}:}{ Integer or character scalar.  The box color
#' around reads.}
#'
#' \item{\code{col.sashimi=NULL}:}{ Integer or character scalar.  The line
#' color for sashimi plots.}
#'
#' \item{\code{col="#808080"}:}{ Integer or character scalar. The default color
#' of all line elements.}
#'
#' \item{\code{collapse=FALSE}:}{ Logical scalar. Do not perform any collapsing
#' of overlapping elements. Currently not supported.}
#'
#' \item{\code{coverageHeight=0.1}:}{ Numeric scalar. The height of the
#' coverage region of the track. Can either be a value between 0 and 1 in which
#' case it is taken as a relative height, or a positive value greater 1 in
#' which case it is interpreted as pixels.}
#'
#' \item{\code{fill.coverage=NULL}:}{ Integer or character scalar.  The fill
#' color for the coverage profile.}
#'
#' \item{\code{fill.reads=NULL}:}{ Integer or character scalar.  The fill color
#' for the read icons.}
#'
#' \item{\code{fill="#BABABA"}:}{ Integer or character scalar.  The default
#' fill color of all plot elements.}
#'
#' \item{\code{fontface.mismatch=2}:}{ Integer scalar. The font face for
#' mismatch bases.}
#'
#' \item{\code{lty.coverage=NULL}:}{ Integer or character scalar.  The line
#' type of the coverage profile.}
#'
#' \item{\code{lty.gap=NULL}:}{ Integer or character scalar. The type of the
#' line that is bridging the gap regions in gapped alignments.}
#'
#' \item{\code{lty.mates=NULL}:}{ Integer or character scalar.  The type of the
#' line that is connecting two paired reads.}
#'
#' \item{\code{lty.deletion=NULL}:}{ Integer or character scalar.  The type of
#' the line that is bridging the deleted regions in alignments.}
#'
#' \item{\code{lty.insertion=NULL}:}{ Integer or character scalar.  The type of
#' the line that highlighting insertions in alignments.}
#'
#' \item{\code{lty.mismatch=NULL}:}{ Integer or character scalar.  The box line
#' type around mismatch bases.}
#'
#' \item{\code{lty.reads=NULL}:}{ Integer or character scalar.  The box line
#' type around mismatch reads.}
#'
#' \item{\code{lty=1}:}{ Integer or character scalar. The default type of all
#' line elements.}
#'
#' \item{\code{lwd.coverage=NULL}:}{ Integer or character scalar.  The line
#' width of the coverage profile.}
#'
#' \item{\code{lwd.gap=NULL}:}{ Integer scalar. The width of the line that is
#' bridging the gap regions in gapped alignments.}
#'
#' \item{\code{lwd.mates=NULL}:}{ Integer scalar. The width of the line that is
#' connecting two paired reads.}
#'
#' \item{\code{lwd.deletion=NULL}:}{ Integer scalar. The width of the line that
#' is bridging the deleted regions in alignments.}
#'
#' \item{\code{lwd.insertion=NULL}:}{ Integer scalar. The width of the line
#' that highlighting insertions in alignments.}
#'
#' \item{\code{lwd.mismatch=NULL}:}{ Integer scalar. The box line width around
#' mismatch bases.}
#'
#' \item{\code{lwd.reads=NULL}:}{ Integer scalar. The box line width around
#' reads.}
#'
#' \item{\code{lwd.sashimiMax=10}:}{ Integer scalar. The maximal width of the
#' line in sashimi plots.}
#'
#' \item{\code{lwd=1}:}{ Integer scalar. The default width of all line
#' elements.}
#'
#' \item{\code{max.height=10}:}{ Integer scalar. The maximum height of an
#' individual read in pixels. Can be used in combination with \code{min.height}
#' to control the read and stacking appearance.}
#'
#' \item{\code{min.height=5}:}{ Integer scalar. The minimum height of an
#' individual read in pixels. Can be used in combination with \code{max.height}
#' to control the read and stacking appearance.}
#'
#' \item{\code{minCoverageHeight=50}:}{ Integer scalar. The minimum height of
#' the coverage section. Uselful in combination with a relative setting of
#' \code{coverageHeight}.}
#'
#' \item{\code{minSashimiHeight=50}:}{ Integer scalar. The minimum height of
#' the sashimi section. Uselful in combination with a relative setting of
#' \code{sashimiHeight}.}
#'
#' \item{\code{noLetters=FALSE}:}{ Logical scalar. Always plot colored boxes
#' for mismatch bases regardles of the available space.}
#'
#' \item{\code{sashimiFilter=NULL}:}{ GRanges object. Only junctions which
#' overlap equally with \code{sashimiFilter} GRanges are shown. Default
#' \code{NULL}, no filtering.}
#'
#' \item{\code{sashimiFilterTolerance=0}:}{ Integer scalar. Only used in
#' combination with \code{sashimiFilter}. It allows to include junctions whose
#' starts/ends are within specified distance from \code{sashimiFilter} GRanges.
#' This is useful for cases where the aligner did not place the junction reads
#' precisely. Default \code{0L} , no tolerance.}
#'
#' \item{\code{sashimiHeight=0.1}:}{ Integer scalar. The height of the sashimi
#' part of the track. Can either be a value between 0 and 1 in which case it is
#' taken as a relative height, or a positive value greater 1 in which case it
#' is interpreted as pixels.}
#'
#' \item{\code{sashimiScore=1}:}{ Integer scalar. The minimum number of reads
#' supporting the junction.}
#'
#' \item{\code{sashimiStrand="*"}:}{ Integer scalar. Only reads which have the
#' specified strand are considered to count the junctions.}
#'
#' \item{\code{sashimiTransformation=NULL}:}{ Function. Applied to the junction
#' score vector prior to plotting. The function should accept exactly one input
#' argument and its return value needs to be a numeric vector of identical
#' length as the input data.}
#'
#' \item{\code{showIndels=FALSE}:}{ Logical scalar. Consider insertions and
#' deletions in coverage and pile-up. Default is \code{FALSE}. If set to
#' \code{TRUE} the deletions defined in CIGAR string are not considered in
#' coverage plot. The deletions are displayed as bridging lines in pile-up
#' track. Insertions are shown as vertical bars.}
#'
#' \item{\code{showMismatches=TRUE}:}{ Logical scalar. Add mismatch
#' information, either as individual base letters or using color coded bars.
#' This implies that the reference sequence has been provided, either to the
#' class constructor or as part of the track list.}
#'
#' \item{\code{size=NULL}:}{ Numeric scalar. The size of the track.  Defaults
#' to automatic sizing.}
#'
#' \item{\code{transformation=NULL}:}{ Function. Applied to the coverage vector
#' prior to plotting. The function should accept exactly one input argument and
#' its return value needs to be a numeric Rle of identical length as the input
#' data.}
#'
#' \item{\code{type=c("coverage", "pileup")}:}{ Character vactor.  The type of
#' information to plot. For \code{coverage} a coverage plot, potentially
#' augmented by base mismatch information, for \code{sashimi} a sashimi plot,
#' showing the juctions, and for \code{pileup} the pileups of the individual
#' reads. These three can be combined.}
#'
#' }
#'
#' \bold{\emph{Inherited from class StackedTrack:}}
#'
#' \describe{
#'
#' \item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
#' vertical size and spacing between stacked elements. The number defines the
#' proportion of the total available space for the stack that is used to draw
#' the glyphs.  E.g., a value of 0.5 means that half of the available vertical
#' drawing space (for each stacking line) is used for the glyphs, and thus one
#' quarter of the available space each is used for spacing above and below the
#' glyph. Defaults to 0.75.}
#'
#' \item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
#' of stacked items. I.e., features that are plotted on the bottom-most stacks
#' will be moved to the top-most stack and vice versa.}
#'
#' }
#'
#' \bold{\emph{Inherited from class GdObject:}}
#'
#' \describe{
#'
#' \item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
#' items.}
#'
#' \item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
#' title panel.}
#'
#' \item{\code{background.panel="transparent"}:}{ Integer or character scalar.
#' The background color of the content panel.}
#'
#' \item{\code{background.title="lightgray"}:}{ Integer or character scalar.
#' The background color for the title panel.}
#'
#' \item{\code{background.legend="transparent"}:}{ Integer or character scalar.
#' The background color for the legend.}
#'
#' \item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
#' axis annotation. Defaults to \code{NULL}, in which case it is automatically
#' determined based on the available space.}
#'
#' \item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
#' title panel. This effects the fontsize of both the title and the axis, if
#' any. Defaults to \code{NULL}, which means that the text size is
#' automatically adjusted to the available space.}
#'
#' \item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
#' line color for the y axis, if any.}
#'
#' \item{\code{col.border.title="white"}:}{ Integer or character scalar. The
#' border color for the title panels.}
#'
#' \item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
#' color used for the panel frame, if \code{frame==TRUE}}
#'
#' \item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
#' line color for grid lines, both when \code{type=="g"} in
#' \code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
#' for plot lines. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
#' for plot symbols. Usually the same as the global \code{col} parameter.}
#'
#' \item{\code{col.title="white"}:}{ Integer or character scalar.  The border
#' color for the title panels}
#'
#' \item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
#' color for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
#' for the title panels.}
#'
#' \item{\code{fontface=1}:}{ Integer or character scalar. The font face for
#' all text, unless a more specific definition exists.}
#'
#' \item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
#' font family for the title panels.}
#'
#' \item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
#' family for all text, unless a more specific definition exists.}
#'
#' \item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
#' unless a more specific definition exists.}
#'
#' \item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
#' plotting.}
#'
#' \item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
#' grid.}
#'
#' \item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
#' horizontal grid lines, see \code{\link{panel.grid}} for details.}
#'
#' \item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
#' text, unless a more specific definition exists.}
#'
#' \item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
#' type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
#' and when display parameter \code{grid==TRUE}.}
#'
#' \item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
#' title panels.}
#'
#' \item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
#' panels}
#'
#' \item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
#' lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
#' display parameter \code{grid==TRUE}.}
#'
#' \item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
#' before collapsing range items, only if \code{collapse==TRUE}. See
#' \code{\link{collapsing}} for details.}
#'
#' \item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
#' pixels to display. All ranges are expanded to this size in order to avoid
#' rendering issues. See \code{\link{collapsing}} for details.}
#'
#' \item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
#' coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
#' mirror the plot on the vertical axis.}
#'
#' \item{\code{rotation.title=90}:}{ The rotation angle for the text in the
#' title panel. Even though this can be adjusted, the automatic resizing of the
#' title panel will currently not work, so use at own risk.}
#'
#' \item{\code{rotation=0}:}{ The rotation angle for all text unless a more
#' specific definiton exists.}
#'
#' \item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
#' (only applies to track types where axes are implemented).}
#'
#' \item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
#' panel. Although this can be set individually for each track, in multi-track
#' plots as created by \code{\link{plotTracks}} there will still be an empty
#' placeholder in case any of the other tracks include a title.  The same holds
#' true for axes. Note that the the title panel background color could be set
#' to transparent in order to completely hide it.}
#'
#' \item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
#' vertical grid lines, see \code{\link{panel.grid}} for details.}
#'
#' }
#'
#' }
#'
#' }
#'
#' @name settings
#'
#' @author Florian Hahne
#'
#' @seealso
#' \code{\linkS4class{AnnotationTrack}}
#'
#' \code{\linkS4class{DataTrack}}
#'
#' \code{\linkS4class{DisplayPars}}
#'
#' \code{\linkS4class{GdObject}}
#'
#' \code{\link{availableDisplayPars}}
#'
#' \code{\link{collapsing}}
#'
#' \code{\link{grouping}}
#'
#' \code{\link{horizonplot}}
#'
#' \code{\link{panel.bwplot}}
#'
#' \code{\link{panel.grid}}
#'
#' \code{\link{panel.loess}}
#'
#' \code{\link{panel.xyplot}}
#'
#' \code{\link{plotTracks}}
#'
#' @examples
#'
#' ## Which scheme is used?
#' getOption("Gviz.scheme")
#'
#' ## Change default settings for GeneRegionTrack
#' scheme <- getScheme()
#' scheme$GeneRegionTrack$fill <- "salmon"
#' scheme$GeneRegionTrack$col <- NULL
#' scheme$GeneRegionTrack$transcriptAnnotation <- "transcript"
#'
#' ## replace default scheme with myScheme
#' addScheme(scheme, "myScheme")
#' options(Gviz.scheme = "myScheme")
#' getOption("Gviz.scheme")
#'
#' data(geneModels)
#' grtrack <- GeneRegionTrack(geneModels, genome = "hg19", chromosome = "chr7", name = "Gene Model")
#' plotTracks(grtrack)
NULL

#' Deprecated functions in package `Gviz`
#'
#' @name Gviz-deprecated
#'
#' @description These functions are provided for compatibility with older
#' versions of `Gviz` only, and will be defunct at the next release.
#' ## The following functions are deprecated and will be made defunct (use the replacement indicated below):
#'
#' (none)
NULL

##----------------------------------------------------------------------------------------------------------------------
## A bunch of package constants
##----------------------------------------------------------------------------------------------------------------------
.DEFAULT_FILL_COL <- "lightgray"
.DEFAULT_OVERPLOT_COL <- "red"
.DEFAULT_LINE_COL <- "black"
.DEFAULT_SHADED_COL <- "#808080"
.DEFAULT_BRIGHT_SHADED_COL <- "#E0E0E0"
.DEFAULT_SYMBOL_COL <- "#0080FF"
.DEFAULT_LINE_COL <- "darkgray"
.PLOT_TYPES <-  c("p", "l", "b", "a", "s", "g", "r", "S",
                  "smooth", "polygon", "horizon", "histogram",
                  "mountain", "h", "boxplot", "gradient", "heatmap", "confint")
.ALIGNMENT_TYPES <- c("coverage", "sashimi", "pileup")
.THIN_BOX_FEATURES <- c("utr", "ncRNA", "utr3", "utr5", "3UTR", "5UTR", "miRNA", "lincRNA",
                        "three_prime_UTR", "five_prime_UTR")
.DEFAULT_HORIZON_COL <- c("#B41414", "#E03231", "#F7A99C", "#9FC8DC", "#468CC8", "#0165B3")
##----------------------------------------------------------------------------------------------------------------------



## Check the class and structure of an object
.checkClass <- function (x, class, length = NULL, verbose = FALSE, mandatory = TRUE){
    if (mandatory && missing(x))
        stop("Argument '", substitute(x), "' is missing with no default",
             call. = verbose)
    msg <- paste("'", substitute(x), "' must be an object of class ",
        paste("'", class, "'", sep = "", collapse = " or "),
        sep = "")
    fail <- !any(sapply(class, function(c, y) is(y, c), x))
    if (!is.null(length) && length(x) != length) {
        if (!is.null(x)) {
            fail <- TRUE
            msg <- paste(msg, "of length", length)
        }
    }
    if (fail)
        stop(msg, call. = verbose)
    else invisible(NULL)
}



## We want to deal with chromosomes in a reasonable way. This coerces likely inputs to a unified
## chromosome name as understood by UCSC. Accepted inputs are:
##    - a single integer or a character coercable to one or integer-character combinations
##    - a character, starting with 'chr' (case insensitive)
## Arguments:
##    o x: a character string to be converted to a valid UCSC chromosome name
##    o force: a logical flag, force prepending of 'chr' if missing
## Value: the UCSC character name
.chrName <- function(x, force=FALSE)
{
    if(!getOption("ucscChromosomeNames") || length(x)==0)
        return(as.character(x))
    xu <- unique(x)
    xum <- sapply(xu, function(y){
        xx <- suppressWarnings(as.integer(y))
        if (!is.na(xx))
            y <- xx
        if(is.numeric(y))
          y <- paste("chr", y, sep = "")
        if (y == "MT") # ensembl  `MT` to `chrM` in UCSC
          y <- "chrM"
        if (y %in% c("M", "X", "Y", "Z", "W")) # mitochondrial genome and sex chromosomes
          y <- paste("chr", y, sep = "")
        head <- tolower(substring(y, 1, 3)) == "chr"
        if(!head && force){
            y <-  paste("chr", y, sep = "")
            head <- TRUE
        }
        if(!head){
            stop(sprintf(paste("Invalid chromosome identifier '%s'\nPlease consider setting options(ucscChromosomeNames=FALSE)",
                "to allow for arbitrary chromosome identifiers."), y))
        }
        substring(y, 1, 3) <- tolower(substring(y, 1, 3))
        y})
    names(xum) <- xu
    return(as.vector(xum[as.character(x)]))
}


## Make a deep copy of the display parameter environment
.deepCopyPars <- function(GdObject)
{
    oldPars <- displayPars(GdObject, hideInternal=FALSE)
    GdObject@dp <- DisplayPars()
    displayPars(GdObject) <- oldPars
    return(GdObject)
}



## One central place to check which display types result in stacking. This may change at some point for some
## unimplemented types...
## Arguments:
##    o GdObject: an object inheriting from class GdObject
## Value: a logical skalar indicating whether stacking is needed or not
.needsStacking <- function(GdObject) stacking(GdObject) %in% c("squish", "pack", "full")



## Get the coordinates for an HTML image map from the annotationTrack plot.
## Arguments:
##     o coordinates: a numeric matrix of annotation region coordinates (the bounding box if not rectangular)
## Value: valid HTML image map coordinates based on the current device dimensions
.getImageMap <- function(coordinates)
{
    devSize <- devRes()*par("din")
    loc <- vpLocation()
    size <- loc$location[3:4] - loc$location[1:2]
    xscale <- current.viewport()$xscale
    yscale <- current.viewport()$yscale
    fw <- diff(xscale)
    fh <- diff(yscale)
    u2px <- function(x) ((x - xscale[1])/fw *size[1]) + loc$location[1]
    u2py <- function(y) (devSize[2] - loc$location[2]) - ((y - yscale[1])/fh *size[2])
    return(data.frame(x1=u2px(coordinates[,1]), y1=u2py(coordinates[,4]),
                      x2=u2px(coordinates[,3]), y2=u2py(coordinates[,2]),
                      stringsAsFactors=FALSE))
}


.whichStrand <- function(trackList){
    if(!is.list(trackList))
        trackList <- list(trackList)
    str <- unlist(lapply(trackList, function(x)
                         if(is(x, "HighlightTrack") || is(x, "OverlayTrack")) sapply(x@trackList, .dpOrDefault, "reverseStrand") else{
                             .dpOrDefault(x, "reverseStrand")}))
    return(ifelse(str, "reverse", "forward"))
}



## A function returning the amount of vertical space needed for a track
## Arguments:
##    o x: an object inheriting from class GdObject
## Value: the relative vertical space needed for the track
.verticalSpace <- function(x, totalSpace)
{
    if(is(x, "AlignedReadTrack")){
        size <- if(is.null(displayPars(x, "size"))){
            type <- match.arg(.dpOrDefault(x, "detail", "coverage"), c("reads", "coverage"))
            if(type == "read")
                if(stacking(x) %in% c("sqish", "full")) 5 else 1 else 7} else displayPars(x, "size")
        return(size)
    }
    if(is(x, "DataTrack") && is.null(displayPars(x, "size"))){
        type <- match.arg(.dpOrDefault(x, "type", "p"), .PLOT_TYPES, several.ok=TRUE)
        size <- if(length(type)==1L){ if(type=="gradient") 1 else if(type=="heatmap") nrow(values(x)) else 5} else 5
        return(size)
    }
    if(is(x, "GenomeAxisTrack") || is(x, "IdeogramTrack") || is(x, "SequenceTrack"))
    {
        nv <- displayPars(x, "neededVerticalSpace")
        size <- displayPars(x, "size")
        if(is.null(size))
            if(!is.null(nv))
            {
                size <- nv
                attr(size, "absolute") <- TRUE
            } else size <- 1
        return(size)
    }
    size <- .dpOrDefault(x, "size", 1)
    if(is(x, "StackedTrack"))
      size <- max(size, min(floor(vpLocation()$size["height"]/10), size*max(stacks(x))))
    return(size)
}



## Return a particular displayPars value or a default
## Arguments:
##    o GdObject: an object inheriting from class GdObject
##    o par: the name of the displayPar, or a list of alternatives to go though before finally taking
##       the supplied default value
##    o default: a default value for the parameter if it can't be found in GdObject
## Value: the value of the displayPar
.dpOrDefault <- function(GdObject, par, default=NULL, fromPrototype=FALSE)
{
    val <- getPar(x=GdObject, name=par, asIs=TRUE)
    val <- val[!sapply(val, is.null)]
    if(length(val)==0) {
       if (fromPrototype) {
          val <- .parMappings[[GdObject@name]][par]
          val <- val[!sapply(val, is.null)]
          if(length(val)==0)
              val <- NULL
       } else {
          val <- default
       }
    }else{
        val <- val[[1]]
    }
    return(val)
}



## A special version of the above for font settings. This will try to extract the respective parent
## defaults before finally taking the provided default value.
## Arguments:
##    o GdObject: an object inheriting from class GdObject
##    o par: the name of the displayPar. Can also be a vector in which case a number of alternatives
##       is tested, then the parent default for the first element until finally moving to the supplied default
##    o type: the sub-type
##    o default: a default value for the parameter if it can't be found in GdObject
## Value: the value of the displayPar
.dpOrDefaultFont <- function(GdObject, par, type=NULL, default){
    name <- if(is.null(type)) par else sprintf("%s.%s", par, type)
    val <- getPar(GdObject, name[1])
    name <- name[-1]
    while(is.null(val) && length(name)){
        val <- getPar(GdObject, name[1])
        name <- name[-1]
    }
    if(is.null(val))
        val <- .dpOrDefault(GdObject, par[1], default)
    return(val)
}


## Return the font settings for a GdObject
.fontGp <- function(GdObject, subtype=NULL, ...){
    if(is(GdObject, "OverlayTrack"))
        GdObject <- GdObject@trackList[[1]]
    gp <- list(fontsize=as.vector(.dpOrDefaultFont(GdObject, "fontsize", subtype, 12))[1],
               fontface=as.vector(.dpOrDefaultFont(GdObject, "fontface", subtype, 1))[1],
               fontfamily=as.character(as.vector(.dpOrDefaultFont(GdObject, "fontfamily", subtype, 1)))[1],
               col=as.vector(.dpOrDefaultFont(GdObject, "fontcolor", subtype, "black"))[1],
               lineheight=as.vector(.dpOrDefaultFont(GdObject, "lineheight", subtype, 1))[1],
               alpha=as.vector(.dpOrDefaultFont(GdObject, "alpha", subtype, 1))[1],
               cex=as.vector(.dpOrDefaultFont(GdObject, "cex", subtype, 1))[1])
    gp[names(list(...))] <- list(...)
    gp <- gp[!sapply(gp, is.null)]
    return(do.call(gpar, gp))
}


## Check a list of GdObjects whether an axis needs to be drawn for each of them.
## Arguments:
##    o object: a list of GdObjects
## Value: a logical vector of the same length as 'objects'
.needsAxis <- function(objects) {
    if(!is.list(objects))
        objects <- list(objects)
    atrack <- sapply(objects, function(x){
        is(x, "NumericTrack") ||
        (is(x, "AlignmentsTrack") && "coverage" %in% match.arg(.dpOrDefault(x, "type", .ALIGNMENT_TYPES), .ALIGNMENT_TYPES, several.ok=TRUE)) ||
        (is(x, "AlignedReadTrack") && .dpOrDefault(x, "detail", "coverage")=="coverage")
    })
    isOnlyHoriz <- sapply(objects, function(x){
        res <- FALSE
        if(is(x, "DataTrack")){
            type <- match.arg(.dpOrDefault(x, "type", "p"), .PLOT_TYPES, several.ok=TRUE)
            res <- length(setdiff(type, "horizon")) == 0 && !.dpOrDefault(x, "showSampleNames", FALSE)
        }
        res
    })
    return(atrack & sapply(objects, .dpOrDefault, "showAxis", TRUE) & !isOnlyHoriz)
}

## Check a list of GdObjects whether a title needs to be drawn for each of them.
## Arguments:
##    o object: a list of GdObjects
## Value: a logical vector of the same length as 'objects'
.needsTitle <- function(objects) {
    if(!is.list(objects))
        objects <- list(objects)
    sapply(objects, function(x){
        if(is(x, "HighlightTrack") || is(x, "OverlayTrack"))
            any(sapply(x@trackList, .dpOrDefault, "showTitle", TRUE)) else .dpOrDefault(x, "showTitle", TRUE)
    })
}


## Helper function to set up the text size based on the available space
## Arguments:
##    o trackList: a list of GdObjects
##    o sizes: a matching vector of relative vertical sizes
##    o title.width: the available width for the title
## Value: a list with items:
##    o spaceNeeded: the necessary vertical space
##    o cex: the character expansion factor
##    o title.width: the updated available title width
##    o spacing: the amount of spacing between tracks
##    o nwrap: the final (wrapped) title text
.setupTextSize <- function(trackList, sizes, title.width, panelOnly=FALSE, spacing=5)
{
    curVp <- vpLocation()
    trackList <- lapply(trackList, function(x) if(is(x, "OverlayTrack")) x@trackList[[1]] else x)
    spaceNeeded <- if(is.null(sizes)) lapply(trackList, .verticalSpace, curVp$size["height"]) else {
        if(length(sizes) != length(trackList))
            stop("The 'sizes' vector has to match the size of the 'trackList'.")
        rev(sizes)
    }
    whichAbs <- sapply(spaceNeeded, function(x) !is.null(attr(x, "absolute")) && attr(x, "absolute"))
    spaceNeeded <- unlist(spaceNeeded)
    leftVetSpace <- curVp$size["height"]-sum(spaceNeeded[whichAbs])
    spaceNeeded[!whichAbs] <- spaceNeeded[!whichAbs]/sum(spaceNeeded[!whichAbs])*leftVetSpace
    spaceNeeded <- spaceNeeded/sum(spaceNeeded)
    if(!panelOnly)
    {
        ## Figure out the fontsize for the titles based on available space. If the space is too small (<atLeast)
        ## we don't plot any text, and we also limit to 'maximum' to avoid overblown labels. If the displayPars
        ## 'cex.title' or 'cex.axis are not NULL, those override everything else.
        nn <- sapply(trackList, names)
        nwrap <- sapply(nn, function(x) paste(strwrap(x, 10), collapse="\n"))
        needAxis <- .needsAxis(trackList)
        isOnlyHoriz <- sapply(trackList, function(x){
            res <- FALSE
            if(is(x, "DataTrack")){
                type <- match.arg(.dpOrDefault(x, "type", "p"), .PLOT_TYPES, several.ok=TRUE)
                res <- length(setdiff(type, "horizon")) == 0
            }
            res
        })
        nwrap[needAxis] <- nn[needAxis]
        lengths <- as.numeric(convertWidth(stringWidth(nwrap),"inches"))+0.2
        heights <- curVp$isize["height"]*spaceNeeded
        atLeast <- 0.5
        maximum <- 1.2
        allCex <- heights/lengths
        parCex <- sapply(trackList, function(x) if(is.null(displayPars(x, "cex.title"))) NA else displayPars(x, "cex.title"))
        toSmall <- allCex<atLeast
        cex <- rep(max(c(atLeast, min(c(allCex[!toSmall], maximum), na.rm=TRUE))), length(allCex))
        cex[!is.na(parCex)] <- parCex[!is.na(parCex)]
        lengthsNoWrap <- as.numeric(convertWidth(stringWidth(nn),"inches"))*cex+0.4
        needsWrapping <- lengthsNoWrap>heights
        nwrap[allCex<atLeast & is.na(parCex)] <- ""
        nwrap[!needsWrapping] <- nn[!needsWrapping]
        ## Figure out the title width based on the available tracks. If there is an axis for at least one of the tracks,
        ## we add 1.5 time the width of a single line of text to accomodate for it.
        wfac <- curVp$isize["width"]
        leaveSpace <- ifelse(any(needAxis), 0.15, 0.2)
        width <- (as.numeric(convertHeight(stringHeight(paste("g_T", nwrap, "g_T", sep="")), "inches"))*cex+leaveSpace)/wfac
        showtitle <- sapply(trackList, .dpOrDefault, "showTitle", TRUE)
        width[!showtitle & !needAxis] <- 0
        width[!showtitle] <- 0
        twfac <- if(missing(title.width) || is.null(title.width)) 1 else title.width
        title.width <- max(width, na.rm=TRUE)
        if(any(needAxis)){
            cex.axis <- structure(sapply(trackList, .dpOrDefault, "cex.axis", 0.6), names=nn)
            axTicks <- unlist(lapply(trackList, function(GdObject){
                if(!is(GdObject, "NumericTrack") && !is(GdObject, "AlignedReadTrack") && !is(GdObject, "AlignmentsTrack"))
                    return(NULL)
                yvals <- if(is(GdObject, "AlignedReadTrack")) runValue(coverage(GdObject, strand="*")) else values(GdObject)
                ylim <- .dpOrDefault(GdObject, "ylim", if(!is.null(yvals) && length(yvals))
                                     range(yvals, na.rm=TRUE, finite=TRUE) else c(-1,1))
                if(diff(ylim)==0)
                    ylim <- ylim+c(-1,1)
                yscale <- extendrange(r=ylim, f=0.05)
                at <- pretty(yscale)
                at[at>=sort(ylim)[1] & at<=sort(ylim)[2]]
                atSpace <- max(as.numeric(convertWidth(stringWidth(at), "inches"))+0.18)*cex.axis[names(GdObject)]
                if(is(GdObject, "DataTrack")){
                    type <- match.arg(.dpOrDefault(GdObject, "type", "p"), .PLOT_TYPES, several.ok=TRUE)
                    if(any(c("heatmap", "gradient") %in% type)){
                        nlevs <- max(1, nlevels(factor(getPar(GdObject, "groups"))))-1
                        atSpace <- atSpace + 0.3 * atSpace + as.numeric(convertWidth(unit(3, "points"), "inches"))*nlevs
                    }
                    if(any(type %in% c("heatmap", "horizon")) && .dpOrDefault(GdObject, "showSampleNames", FALSE)){
                        sn <- rownames(values(GdObject))
                        axSpace <- ifelse(isOnlyHoriz, 0, 10)
                        wd <- max(as.numeric(convertWidth(stringWidth(sn) + unit(axSpace, "points"), "inches")))
                        atSpace <- atSpace + (wd * .dpOrDefault(GdObject, "cex.sampleNames", 0.5))
                    }
                }
                atSpace
            }))
            hAxSpaceNeeded <- (max(axTicks))/wfac
            title.width <- title.width +  hAxSpaceNeeded
        }
    } else {
        title.width <- nwrap <- cex <- NA
    }
    spacing <- as.numeric(convertWidth(unit(spacing, "points"), "npc"))
    title.width <- title.width * twfac
    return(list(spaceNeeded=spaceNeeded, cex=cex, title.width=title.width, spacing=spacing, nwrap=nwrap))
}



## This coerces likely inputs for the genomic strand  to a unified
## strand name. Accepted inputs are:
##    o a single integer, where values <=0 indicate the plus strand, and values >=1 indicate the minus strand
##    o a character, either "+" or "-"
## If extended=TRUE, the additional values 2, "+-", "-+" and "*" are allowed, indicating to use both strands.
## Value: the validated strand name
.strandName <- function(x, extended=FALSE)
{
  fun <- function(x, extended){
    if(!extended) {
      if(is.numeric(x)) {
        x <- min(c(1,max(c(0, as.integer(x)))))
      } else if(is.character(x)) {
        x <- match(x, c("+", "-"))-1
        if(any(is.na(x)))
          stop("The strand has to be specified either as a character ('+' or '-'), or as an integer value (0 or 1)")
      }
    } else {
      if(is.numeric(x)) {
        x <- min(c(2,max(c(0, as.integer(x)))))
      } else if(is.character(x)) {
        x <- min(c(2, match(x, c("+", "-", "+-", "-+", "*"))-1))
        if(any(is.na(x)))
          stop("The strand has to be specified either as a character ('+' or '-'), or as an integer value (0 or 1)")
      }
    }
    x
  }
  return(sapply(x, fun, extended))
}



## Compute native coordinate equivalent to 'min.width' pixel. This assumes that a graphics device is already
## open, otherwise a new window will pop up, which could be a little annoying.
## Arguments:
##    o min.width: the number of pixels
##    o coord: the axis for which to compute the coordinats, one in c("x","y")
## Value: the equivalent of 'min.width' in native coordinates.
.pxResolution <- function(min.width=1, coord=c("x","y"))
{
    coord <- match.arg(coord, several.ok=TRUE)
    curVp <- vpLocation()
    co <- c(x=as.vector(abs(diff(current.viewport()$xscale))/(curVp$size["width"])*min.width),
            y=as.vector(abs(diff(current.viewport()$yscale))/(curVp$size["height"])*min.width))
    return(co[coord])
}


## Deal with composite exons and provide polygon plotting coordinates for them. For all normal exons simply return
## the original bounding box data
## Arguments:
##    o box: the data frame with the bounding box information
##    o type: the type of coordinates for the composite exons, one in 'box', 'arrow' or 'fixedArrow'
## Value: a list with three elements: box, the non-composite exons, pols, the plotting coordinates for the merged composite exons,
##        and polpars, a data.frame of plotting parameters for the polygons
.handleComposite <- function(box, type="box", W=1/4, H=1/3, min.width=10, max.width=Inf){
    box <- box[order(box$start),]
    boxFinal <- data.frame(stringsAsFactors=FALSE)
    polFinal <- data.frame(stringsAsFactors=FALSE)
    polPars <- data.frame(stringsAsFactors=FALSE)
    bss <- split(box, box$transcript)
    for(b in bss){
        ol <- names(which(table(b$exon)>1))
        boxFinal <- rbind(boxFinal, b[!b$exon %in% ol,])
        if(length(ol)){
            b <- b[b$exon %in% ol,]
            r <- IRanges(start=b$cx1, end=b$cx2)
            rr <- reduce(r)
            brs <- split(b, subjectHits(findOverlaps(r, rr)))
            for(j in seq_along(brs)){
                if(nrow(brs[[j]]) == 1){
                    boxFinal <- rbind(boxFinal, brs[[j]])
                }else{
                    xlocs <- as.vector(t(brs[[j]][, c("cx1", "cx2"), drop=FALSE]))
                    ylocs <- unlist(brs[[j]][, c("cy1", "cy2"), drop=FALSE])
                    fh <- 1:(length(ylocs)/2)
                    sh <- (length(ylocs)/2+1):length(ylocs)
                    ylocs[sh] <- rev(ylocs[sh])
                    ylocs <- rep(ylocs, each=2)
                    if(type == "box"){
                        polFinal <- rbind(polFinal, data.frame(x=c(xlocs, rev(xlocs)), y=ylocs,
                                                               id=paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                                                               stringsAsFactors=FALSE))
                        polPars <- rbind(polPars, data.frame(fill=brs[[j]][1, "fill"], col=brs[[j]][1, "col"], stringsAsFactors=FALSE))
                    }else{
                        polPars <- rbind(polPars, data.frame(fill=brs[[j]][1, "fill"], col=brs[[j]][1, "col"], stringsAsFactors=FALSE))
                        str <- brs[[j]]$strand[1]
                        offset <- (abs(brs[[j]]$cy1 - brs[[j]]$cy2)) * H / 2
                        if(str == "+" && abs(diff(xlocs[(length(xlocs)-1):length(xlocs)])) > min.width){
                            offset <- rep(offset, each=2)
                            asel <- (length(xlocs)-1):length(xlocs)
                            bsel <- 1:(min(asel)-1)
                            d <- abs(diff(xlocs[asel]))
                            afp <- if(type == "arrow") rep(xlocs[asel][1] + max(d*W, d-max.width), 2) else {
                                rep(max(xlocs[asel][1], xlocs[asel][2]-W), 2)}
                            xlocs <- c(xlocs[bsel], xlocs[asel][1], afp, xlocs[asel][2], afp, xlocs[asel][1], rev(xlocs[bsel]))
                            mid <- length(ylocs)/2
                            asel <- (mid-1):(mid+2)
                            bsel <- c(1:(mid-2), (mid+3):length(ylocs))
                            fh <- 1:(length(bsel)/2)
                            sh <- (length(bsel)/2+1):length(bsel)
                            ylocs <- c(ylocs[bsel[fh]] + offset[fh], ylocs[asel][1:2] + tail(offset,1), ylocs[asel][1],
                                       ylocs[asel][1] + abs(diff(ylocs[asel][c(1,3)]))/2, ylocs[asel][4],
                                       ylocs[asel][3:4] - tail(offset,1), ylocs[bsel[sh]] - offset[fh])
                            polFinal <- rbind(polFinal, data.frame(x=c(xlocs, rev(xlocs)), y=ylocs,
                                                                   id=paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                                                                   stringsAsFactors=FALSE))
                        }else if(str == "-" && abs(diff(xlocs[1:2])) > min.width){
                            yoffset <- c(rep(offset[-1], each=2), -rev(rep(offset[-1], each=2)))
                            asel <- 1:2
                            bsel <-  3:length(xlocs)
                            d <- abs(diff(xlocs[asel]))
                            afp <- if(type == "arrow") rep(xlocs[asel][2] - max(d*W, d-max.width), 2) else{
                                rep(min(xlocs[asel][2], xlocs[asel][1] + W), 2)}
                            xlocs <- c(xlocs[asel][1], afp, xlocs[asel][2], xlocs[bsel], rev(xlocs[bsel]), xlocs[asel][2], afp, xlocs[asel][1])
                            asel <- c(1:2, (length(ylocs)-1):length(ylocs))
                            bsel <- 3:(length(ylocs)-2)
                            ylocs <- c(ylocs[asel][1] + abs(diff(ylocs[asel][c(1,3)]))/2, ylocs[asel][1], rep(ylocs[asel][1]+offset[1], 2),
                                       ylocs[bsel]+yoffset, rep(ylocs[asel][3]-offset[1], 2), ylocs[asel][3], ylocs[asel][1] + abs(diff(ylocs[asel][c(1,3)]))/2)
                            polFinal <- rbind(polFinal, data.frame(x=xlocs, y=ylocs,
                                                                   id=paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                                                                   stringsAsFactors=FALSE))
                        }else{
                            offset <- c(rep(offset, each=2), -rev(rep(offset, each=2)))
                            polFinal <- rbind(polFinal, data.frame(x=c(xlocs, rev(xlocs)), y=ylocs+offset,
                                                                   id=paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                                                                   stringsAsFactors=FALSE))
                        }
                    }
                }
            }
        }
    }
    return(list(box=boxFinal, pols=polFinal, polpars=polPars))
}


## Take coordinates for the bounding boxes of annotation regions and plot filled arrows inside.
## Arguments:
##    o the data frame with the bounding box information
##    o W: the proportion of the total box width used for the arrow head
##    o H: the proportion of the total box height used for the arrow head
##    o lwd: the boundary line width
##    o lty: the boundary line type
##    o alpha: the transparency
##    o min.width: the minumum width of the arrow head. Below this size a simple box is drawn
## Note that the last arguments 4-9 all have to be of the same length as number of rows in box.
## Value: the function is called for its side-effects of drawing on the graphics device
.filledArrow <- function(box, W=1/4, H=1/3, lwd, lty, alpha, min.width=10, max.width=Inf, absoluteWidth=FALSE) {
    boxC <-  if("transcript" %in% colnames(box)){
        .handleComposite(box, ifelse(absoluteWidth, "fixedArrow", "arrow"), min.width=min.width, max.width=max.width, W=W, H=H) }else {
        list(box=box, pols=data.frame())}
    xx <- yy <- numeric()
    id <- character()
    pars <- data.frame()
    if(nrow(boxC$box)){
        box <- boxC$box
        A <- box[,1:2,drop=FALSE]
        B <- box[,3:4,drop=FALSE]
        ## First everything that is still a box
        osel <- abs(B[,1]-A[,1]) < min.width | !box$strand %in% c("+", "-")
        xx <-  c(A[osel,1], B[osel,1], B[osel,1], A[osel,1])
        offset <- (abs(B[osel,2]-A[osel,2])*H/2)
        yy <- c(rep(A[osel,2]+offset, 2), rep(B[osel,2]-offset, 2))
        id <- rep(seq_len(sum(osel)), 4)
        pars <- data.frame(fill=box$fill, col=box$col, lwd=lwd, lty=lty, alpha=alpha, stringsAsFactors=FALSE)[osel,]
        ## Now the arrows facing right
        sel <- !osel & box$strand=="+"
        id <- c(id, rep(seq(from=if(!length(id)) 1 else max(id)+1, by=1, len=sum(sel)), 7))
        d <- abs(B[sel,1]-A[sel,1])
        alp <- if(!absoluteWidth) rep(A[sel,1]+pmax(d*W, d-rep(max.width, sum(sel))),2) else rep(pmax(B[sel,1]-W, pmin(B[sel,1], A[sel,1])),2)
        xx <- c(xx, A[sel,1], alp, B[sel,1], alp, A[sel,1])
        yy <- c(yy, rep(A[sel,2]+(abs(B[sel,2]-A[sel,2])*H/2),2), A[sel,2], A[sel,2]+(abs(B[sel,2]-A[sel,2])/2), B[sel,2],
                rep(B[sel,2]-(abs(B[sel,2]-A[sel,2])*H/2),2))
        pars <- rbind(pars, data.frame(fill=box$fill, col=box$col, lwd=lwd, lty=lty, alpha=alpha, stringsAsFactors=FALSE)[sel,])
        ## And finally those facing left
        sel <- !osel & box$strand=="-"
        id <- c(id, rep(seq(from=if(!length(id)) 1 else max(id)+1, by=1, len=sum(sel)), 7))
        d <- abs(B[sel,1]-A[sel,1])
        alp <- if(!absoluteWidth) rep(B[sel,1]-pmax(d*W, d-rep(max.width, sum(sel))), 2) else rep(pmin(A[sel,1]+W, pmax(B[sel,1], A[sel,1])),2)
        xx <- c(xx, B[sel,1], alp, A[sel,1], alp, B[sel,1])
        yy <- c(yy, rep(A[sel,2]+(abs(B[sel,2]-A[sel,2])*H/2),2), A[sel,2], A[sel,2]+(abs(B[sel,2]-A[sel,2])/2), B[sel,2],
                rep(B[sel,2]-(abs(B[sel,2]-A[sel,2])*H/2),2))
        pars <- rbind(pars, data.frame(fill=box$fill, col=box$col, lwd=lwd, lty=lty, alpha=alpha, stringsAsFactors=FALSE)[sel,])
    }
    if(nrow(boxC$pols)){
        xx <- c(xx, boxC$pols$x)
        yy <- c(yy, boxC$pols$y)
        id <- c(id, paste("pols", boxC$pols$id))
        pars <- rbind(pars, data.frame(fill=boxC$polpars$fill, col=boxC$polpars$col, lwd=lwd, lty=lty, alpha=alpha, stringsAsFactors=FALSE))
    }
    grid.polygon(x=xx, y=yy, gp=gpar(fill=pars$fill, col=pars$col, alpha=unique(pars$alpha), lwd=pars$lwd, lty=pars$lty), default.units="native",
                 id=factor(id)) # fix for Sys.setenv(`_R_CHECK_LENGTH_1_CONDITION_`="true") Sys.setenv(`_R_CHECK_LENGTH_1_LOGIC2_`="true")
}


## Take coordinates for the bounding boxes of annotation regions and plot boxes, also making sure that
## composite exons in GeneRegionTracks (i.e., overlapping coordinates and same exon id) are merged
## appropriately
## Arguments:
##    o box: the data frame with the bounding box information
##    o lwd: the boundary line width
##    o lty: the boundary line type
##    o alpha: the transparency
## Value: the function is called for its side-effects of drawing on the graphics device
.filledBoxes <- function(box, lwd, lty, alpha){
    if("transcript" %in% colnames(box)){
        box <- .handleComposite(box, "box")
        if(nrow(box$box))
            grid.rect(box$box$cx2, box$box$cy1, width=box$box$cx2 - box$box$cx1, height=box$box$cy2 - box$box$cy1,
                      gp=gpar(col=as.character(box$box$col), fill=as.character(box$box$fill), lwd=lwd, lty=lty, alpha=alpha),
                      default.units="native", just=c("right", "bottom"))
        if(nrow(box$pols))
            grid.polygon(x=box$pols$x, y=box$pols$y, id=factor(box$pols$id),
                         gp=gpar(col=as.character(box$polpars$col), fill=as.character(box$polpars$fill), lwd=lwd, lty=lty, alpha=alpha),
                         default.units="native")
    }else{
        grid.rect(box$cx2, box$cy1, width=box$cx2 - box$cx1, height=box$cy2 - box$cy1,
                  gp=gpar(col=as.character(box$col), fill=as.character(box$fill), lwd=lwd, lty=lty, alpha=alpha),
                  default.units="native", just=c("right", "bottom"))
    }
}


## Take start and end coordinates for genemodel-type annotations and draw a featherd line indicating
## the strand direction
## Arguments:
##    o xx1, xx2: integer vectors of equal length indicating the start and end of the gene models.
##    o strand: the strand information for each gene model. Needs to be of the same length as xx1 and xx2
##    o coords: the coordinates of the exon features, needed to avoid overlaps.
##    o y: the y value for the arrow bar, usually not set since it should always be 20
##    o W: the width of the arrow feathers in pixels
##    o D: the distance between arrow feathers in pixels
##    o H: the height of the arrow feathers in native coordinates (the total bounding box is usually 40)
##    o col: the boundary color
##    o lwd: the boundary line width
##    o lty: the boundary line type
##    o alpha: the transparency
##    o barOnly: only plot the bar, not the feathers
##    o diff: the current pixel resolution
##    o min.height: the minimum total height in pixels for the feathers (i.e., min.height/2 in each direction)
## Value: the function is called for its side-effects of drawing on the graphics device
.arrowBar <- function(xx1, xx2, strand, coords, y=20, W=3, D=10, H, col, lwd, lty, alpha, barOnly=FALSE,
                      diff=.pxResolution(coord="y"), min.height=3){
    exons <- IRanges(start=coords[,1], end=coords[,3])
    levels <- split(exons, coords[,2]%/%1)
    if(!barOnly){
        onePx <- diff
        if(missing(H)){
            onePy <- .pxResolution(coord="y")
            H <- onePy*min.height/2
        }
        fx1 <- fx2 <- scol <- fy1 <- fy2 <- NULL
        for(i in seq_along(xx1)){
            x1 <- xx1[i]
            x2 <- xx2[i]
            len <- diff(c(x1,x2))/onePx
            if(len>D+W*2){
                ax1 <- seq(from=x1+(onePx*W), to=x1+(len*onePx)-(onePx*W), by=onePx*D)
                ax2 <- ax1+(onePx*W)
                feathers <- IRanges(start=ax1-onePx, end=ax2+onePx)
                cur.level <- y[i]%/%1
                sel <- queryHits(findOverlaps(feathers, resize(levels[[cur.level]], width=width(levels[[cur.level]])-1)))
                if(length(sel)){
                    ax1 <- ax1[-sel]
                    ax2 <- ax2[-sel]
                }
                fx1 <- c(fx1, rep(if(strand[i]=="-") ax1 else ax2, each=2))
                fx2 <- c(fx2, rep(if(strand[i]=="-") ax2 else ax1, each=2))
                scol <- c(scol, rep(col[i], length(ax1)*2))
                fy1 <- c(fy1, rep(rep(y[i], length(ax1)*2)))
                fy2 <- c(fy2, rep(c(y[i]-H, y[i]+H), length(ax1)))
            }
        }
        if(!is.null(fx1) && length(fx1))
            grid.segments(fx1, fy1, fx2, fy2, default.units="native", gp=gpar(col=scol, lwd=lwd, lty=lty, alpha=alpha))
    }
    bars <- data.frame(x1=xx1, x2=xx2, y=y, col=col, stringsAsFactors=FALSE)
    cutBars <- data.frame()
    for(i in 1:nrow(bars)){
        b <- bars[i,]
        cur.level <- b$y%/%1
        ct <- if(cur.level != 0) setdiff(IRanges(start=b$x1, end=b$x2),
                                         resize(levels[[cur.level]], width=width(levels[[cur.level]])-1)) else IRanges()
        if(length(ct))
            cutBars <- rbind(cutBars, data.frame(x1=start(ct), x2=end(ct), y=b$y, col=b$col, stringsAsFactors=FALSE))
    }
    ## fix bug when no introns are present
    if (nrow(cutBars))
        grid.segments(cutBars$x1, cutBars$y, cutBars$x2, cutBars$y, default.units="native",
                      gp=gpar(col=cutBars$col, lwd=lwd, lty=lty, alpha=alpha, lineend="square"))
    ##grid.segments(xx1, y, xx2, y, default.units="native", gp=gpar(col=col, lwd=lwd, lty=lty, alpha=alpha, lineend="square"))
}



## Extract track color for different subtypes within the track and use the default
## color value if no other is found, lightblue if no colors are set at all
## Arguments:
##    o GdObject: object inheriting from class GdObject
## Value: a color character
.getBiotypeColor <- function(GdObject) {
    defCol <- .dpOrDefault(GdObject, "fill", .DEFAULT_FILL_COL)
    col <- sapply(as.character(values(GdObject)[, "feature"]),
                  function(x) .dpOrDefault(GdObject, x)[1], simplify=FALSE)
    needsDef <- sapply(col, is.null)
    col[needsDef] <- rep(defCol, sum(needsDef))[1:sum(needsDef)]
    return(unlist(col))
}


## Compute pretty tickmark location (code from tilingArray package)
## Arguments:
##    o x: a vector of data values
## Value: the tick mark coordinates
.ticks <- function(x){
  rx <- range(x)
  lz <- log((rx[2]-rx[1])/3, 10)
  fl <- floor(lz)
  if( lz-fl > log(5, 10))
    fl <- fl +  log(5, 10)
  tw <- round(10^fl)
  i0 <- ceiling(rx[1]/tw)
  i1 <- floor(rx[2]/tw)
  seq(i0, i1)*tw
}



## A lattice-style panel function to draw smoothed 'mountain' plots
## Arguments:
##    o x, y: the x and y coordinates form the plot
##    o span, degree, family, evaluation: parameters that are passed on to loess
##    o lwd, lty, col: color, with and type of the plot lines
##    o fill: fill colors for areas above and under the baseline, a vector of length two
##    o col.line: color of the baseline
##    o baseline: the y value of the horizontal baseline
##    o alpha: the transparancy
## Value: the function is called for its side-effect of drawing on the graphics device
.panel.mountain <- function (x, y, span=2/3, degree=1, family=c("symmetric", "gaussian"), evaluation=50,
                             lwd=plot.line$lwd, lty=plot.line$lty, col, col.line=plot.line$col,
                             baseline, fill, alpha=1, ...)
{
  x <- as.numeric(x)
  y <- as.numeric(y)
  fill <- rep(fill,2)
  ok <- is.finite(x) & is.finite(y)
  if (sum(ok) < 1)
    return()
  if (!missing(col)) {
    if (missing(col.line))
      col.line <- col
  }
  plot.line <- trellis.par.get("plot.line")
  smooth <- loess.smooth(x[ok], y[ok], span = span, family = family,
                         degree = degree, evaluation = evaluation)
  tmp <- as.integer(smooth$y<baseline)
  changePoint = NULL
  for(i in seq_along(tmp))
    if(i>1 && tmp[i]!= tmp[i-1])
      changePoint = c(changePoint, i)
  m <- (smooth$y[changePoint] - smooth$y[changePoint-1]) / (smooth$x[changePoint] - smooth$x[changePoint-1])
  xCross <- ((baseline-smooth$y[changePoint-1])/m) + smooth$x[changePoint-1]
  newX <- newY <- NULL
  j <- 1
  xx <- smooth$x
  yy <- smooth$y
  smooth$x <- c(smooth$x, tail(smooth$x,1))
  smooth$y <- c(smooth$y, baseline)
  xvals <- smooth$x[1]
  yvals <- baseline
  for(i in seq_along(smooth$x)) {
    if(i==length(smooth$x)) {
      xvals <- c(xvals, smooth$x[i])
      yvals <- c(yvals, baseline)
      fcol <- if(mean(yvals)<baseline) fill[1] else fill[2]
      panel.polygon(xvals, yvals, fill=fcol, col=fcol, border=fcol, alpha=alpha)
    } else if(i %in% changePoint) {
      xvals <- c(xvals, xCross[j])
      yvals <- c(yvals, baseline)
      fcol <- if(mean(yvals)<baseline) fill[1] else fill[2]
      panel.polygon(xvals, yvals, fill=fcol, col=fcol, border=fcol, alpha=alpha)
      xvals <- c(xCross[j], smooth$x[i])
      yvals <- c(baseline, smooth$y[i])
      j <- j+1
    } else {
      xvals <- c(xvals, smooth$x[i])
      yvals <- c(yvals, smooth$y[i])
    }
  }
  grid.lines(x=xx, y=yy, gp=gpar(col=col.line, lty=lty, lwd=lwd, alpha=alpha),
               default.units="native", ...)
}

## A lattice-style panel function to draw polygons (like coverage)
## Arguments:
##    o x, y: the x and y coordinates form the plot
##    o lwd, lty, col: color, with and type of the plot lines
##    o fill: fill colors for areas above and under the baseline, a vector of length two
##    o col.line: color of the baseline
##    o baseline: the y value of the horizontal baseline
##    o alpha: the transparancy
## Value: the function is called for its side-effect of drawing on the graphics device
.panel.polygon <- function (x, y, lwd=plot.line$lwd, lty=plot.line$lty, col,
                            col.line=plot.line$col, baseline, fill, alpha=1, ...)
{
  x <- as.numeric(x)
  y <- as.numeric(y)
  fill <- rep(fill,2)
  ok <- is.finite(x) & is.finite(y)
  if (sum(ok) < 1)
    return()
  if (!missing(col)) {
    if (missing(col.line))
      col.line <- col
  }
  x <- x[ok]
  y <- y[ok]
  plot.line <- trellis.par.get("plot.line")
  changePoint = NULL
  tmp <- as.integer(y<baseline)
  for(i in seq_along(tmp))
    if(i>1 && tmp[i]!= tmp[i-1])
      changePoint = c(changePoint, i)
  m <- (y[changePoint] - y[changePoint-1]) / (x[changePoint] - x[changePoint-1])
  xCross <- ((baseline-y[changePoint-1])/m) + x[changePoint-1]
  newX <- newY <- NULL
  j <- 1
  x <- c(x, tail(x,1))
  y <- c(y, baseline)
  xvals <- x[1]
  yvals <- baseline
  for(i in seq_along(x)) {
    if(i==length(x)) {
      xvals <- c(xvals, x[i])
      yvals <- c(yvals, baseline)
      fcol <- if(mean(yvals)<baseline) fill[1] else fill[2]
      panel.polygon(xvals, yvals, fill=fcol, col=fcol, border=fcol, alpha=alpha)
    } else if(i %in% changePoint) {
      xvals <- c(xvals, xCross[j])
      yvals <- c(yvals, baseline)
      fcol <- if(mean(yvals)<baseline) fill[1] else fill[2]
      panel.polygon(xvals, yvals, fill=fcol, col=fcol, border=fcol, alpha=alpha)
      xvals <- c(xCross[j], x[i])
      yvals <- c(baseline, y[i])
      j <- j+1
    } else {
      xvals <- c(xvals, x[i])
      yvals <- c(yvals, y[i])
    }
  }
  grid.lines(x=x, y=y, gp=gpar(col=col.line, lty=lty, lwd=lwd, alpha=alpha),
               default.units="native", ...)
}


## A lattice-style panel function to draw box and whisker plots with groups
## Arguments: see ? panel.bwplot for details
## Value: the function is called for its side-effect of drawing on the graphics device
.panel.bwplot <- function (x, y, box.ratio=1, box.width=box.ratio/(1+box.ratio), lwd, lty, fontsize,
                           pch, col, alpha, cex, font, fontfamily, fontface, fill, varwidth=FALSE, notch=FALSE,
                           notch.frac = 0.5, ..., levels.fos=sort(unique(x)),
                           stats=boxplot.stats, coef=1.5, do.out=TRUE)
{
  if (all(is.na(x) | is.na(y)))
    return()
  x <- as.numeric(x)
  y <- as.numeric(y)
  cur.limits <- current.panel.limits()
  xscale <- cur.limits$xlim
  yscale <- cur.limits$ylim
  if (!notch)
    notch.frac <- 0
  blist <- tapply(y, factor(x, levels = levels.fos), stats,
                  coef = coef, do.out = do.out)
  blist.stats <- t(sapply(blist, "[[", "stats"))
  blist.out <- lapply(blist, "[[", "out")
  blist.height <- box.width
  if (varwidth) {
    maxn <- max(table(x))
    blist.n <- sapply(blist, "[[", "n")
    blist.height <- sqrt(blist.n/maxn) * blist.height
  }
  blist.conf <- if (notch)
    sapply(blist, "[[", "conf")
  else t(blist.stats[, c(2, 4), drop = FALSE])
  ybnd <- cbind(blist.stats[, 3], blist.conf[2, ], blist.stats[, 4], blist.stats[, 4], blist.conf[2, ],
                blist.stats[,3], blist.conf[1, ], blist.stats[, 2], blist.stats[, 2], blist.conf[1, ], blist.stats[, 3])
  xleft <- levels.fos - blist.height/2
  xright <- levels.fos + blist.height/2
  xbnd <- cbind(xleft + notch.frac * blist.height/2, xleft,
                xleft, xright, xright, xright - notch.frac * blist.height/2,
                xright, xright, xleft, xleft, xleft + notch.frac *
                blist.height/2)
  xs <- matrix(NA_real_, nrow = nrow(xbnd) * 2, ncol = ncol(xbnd))
  ys <- matrix(NA_real_, nrow = nrow(xbnd) * 2, ncol = ncol(xbnd))
  xs[seq(along.with = levels.fos, by = 2), ] <- xbnd[seq(along.with = levels.fos),]
  ys[seq(along.with = levels.fos, by = 2), ] <- ybnd[seq(along.with = levels.fos),]
  panel.polygon(t(xs), t(ys), lwd=lwd, lty=lty, col = fill, alpha=alpha, border=col)
  panel.segments(rep(levels.fos, 2), c(blist.stats[, 2], blist.stats[, 4]), rep(levels.fos, 2),
                 c(blist.stats[,1], blist.stats[, 5]), col=col, alpha=alpha,
                 lwd=lwd, lty=lty)
  panel.segments(levels.fos - blist.height/2, c(blist.stats[, 1], blist.stats[, 5]), levels.fos + blist.height/2,
                 c(blist.stats[, 1], blist.stats[, 5]), col=col, alpha=alpha, lwd=lwd, lty=lty)
  if (all(pch == "|")) {
    mult <- if (notch)
      1 - notch.frac
    else 1
    panel.segments(levels.fos - mult * blist.height/2,
                   blist.stats[, 3], levels.fos + mult * blist.height/2,
                   blist.stats[, 3], lwd=lwd, lty=lty,
                   col=col, alpha = alpha)
  }
  else {
    panel.points(x = levels.fos, y = blist.stats[, 3],
                 pch = pch, col = col, alpha = alpha, cex = cex,
                 fontfamily = fontfamily, fontface = .chooseFace(fontface,
                                            font), fontsize = fontsize)
  }
  panel.points(x = rep(levels.fos, sapply(blist.out, length)),
               y = unlist(blist.out), pch=pch, col=col,
               alpha=alpha, cex=cex,
               fontfamily=fontfamily, fontface = .chooseFace(fontface,
                                        font), fontsize = fontsize)
}



## Check which parameters have already been set for a GdObject, and
## update all missing ones from the prototype of the current parent
## class.
## Arguments:
##    o x: an object inheriting from class GdObject
##    o class: the parent class from which to draw the missing parameters
## Value: The updated GdObject
.updatePars <- function(x, class){
    current <- getPar(x, hideInternal=FALSE)
    defaults <- getPar(getClass(class)@prototype@dp, hideInternal=FALSE)
    ## Check whether we need to adjust any of those defaults based on the selected scheme
    sid <- getOption("Gviz.scheme")
    scheme <- if(is.null(sid)) list() else .schemes[[sid]]
    schemePars <- if(is.null(scheme) || is.null(scheme[[class]])) list() else scheme[[class]]
    if(!is.list(schemePars))
        stop(sprintf("Corrupted parameter definition for class '%s' in scheme '%s'", class, sid))
    defaults[names(schemePars)] <- schemePars
    missing <- setdiff(names(defaults), names(current))
    if(is.null(getPar(x, ".__appliedScheme")) && length(schemePars)){
        defaults[[".__appliedScheme"]] <- if(is.null(sid)) NA else sid
        defaults[names(schemePars)] <- schemePars
        missing <- c(union(missing, names(schemePars)), ".__appliedScheme")
    }
    x <- setPar(x, defaults[missing], interactive=FALSE)
    return(x)
}



## The scheme settings registry. This is essentially just a nested list of display parameters, and the values in this list
## will be used to initialize the objects. Please note that a display parameter still needs to be defined in the class
## definition for this to work, and that all parameters that are not set explicitely in the scheme will be taken from
## the defaults in that class definition. Scheme parameters still override everything else, and even parameters that are
## not defined for the class will be added from the scheme settings.
.schemes <- new.env()
.schemes[["default"]] <- list(

                              GdObject=list(
                                            alpha=1,
                                            background.panel="transparent",
                                            background.title="lightgray",
                                            background.legend="transparent",
                                            cex.axis=NULL,
                                            cex.title=NULL,
                                            cex=1,
                                            col.axis="white",
                                            col.frame="lightgray",
                                            col.grid=.DEFAULT_SHADED_COL,
                                            col.line=NULL,
                                            col.symbol=NULL,
                                            col.title="white",
                                            col=.DEFAULT_SYMBOL_COL,
                                            collapse=TRUE,
                                            fill=.DEFAULT_FILL_COL,
                                            fontcolor.title="white",
                                            fontcolor="black",
                                            fontface.title=2,
                                            fontface=1,
                                            fontfamily.title="sans",
                                            fontfamily="sans",
                                            fontsize=12,
                                            frame=FALSE,
                                            grid=FALSE,
                                            h=-1,
                                            lineheight=1,
                                            lty.grid="solid",
                                            lty="solid",
                                            lwd.border=1,
                                            lwd.grid=1,
                                            lwd=1,
                                            min.distance=1,
                                            min.height=3,
                                            min.width=1,
                                            rotation.title=90,
                                            rotation=0,
                                            showAxis=TRUE,
                                            showTitle=TRUE,
                                            size=1,
                                            v=-1
                                            ),

                              StackedTrack=list(
                                                stackHeight=0.75,
                                                reverseStacking=FALSE
                                                ),


                              AnnotationTrack=list(
                                                   arrowHeadWidth=30,
                                                   arrowHeadMaxWidth=40,
                                                   cex.group=0.6,
                                                   cex=1,
                                                   col.line="darkgray",
                                                   col=.DEFAULT_LINE_COL,
                                                   featureAnnotation=NULL,
                                                   fill="lightblue",
                                                   fontcolor.group=.DEFAULT_SHADED_COL,
                                                   fontcolor.item="white",
                                                   fontface.group=2,
                                                   groupAnnotation=NULL,
                                                   lex=1,
                                                   lineheight=1,
                                                   lty="solid",
                                                   lwd=1,
                                                   mergeGroups=FALSE,
                                                   rotation=0,
                                                   shape="arrow",
                                                   showFeatureId=NULL,
                                                   showId=NULL,
                                                   showOverplotting=FALSE,
                                                   size=1
                                                ),

                              DetailsAnnotationTrack=list(
                                                          details.minWidth=100,
                                                          details.ratio=Inf,
                                                          details.size=0.5,
                                                          detailsBorder.col="darkgray",
                                                          detailsBorder.fill="transparent",
                                                          detailsBorder.lty="solid",
                                                          detailsBorder.lwd=1,
                                                          detailsConnector.cex=1,
                                                          detailsConnector.col="darkgray",
                                                          detailsConnector.lty="dashed",
                                                          detailsConnector.lwd=1,
                                                          detailsConnector.pch=20,
                                                          detailsFunArgs=list(),
                                                          groupDetails=FALSE),

                              GeneRegionTrack=list(
                                                   arrowHeadWidth=10,
                                                   arrowHeadMaxWidth=20,
                                                   col=.DEFAULT_LINE_COL,
                                                   collapseTranscripts=FALSE,
                                                   exonAnnotation=NULL,
                                                   fill="#FFD58A",
                                                   min.distance=0,
                                                   shape=c("smallArrow", "box"),
                                                   showExonId=NULL,
                                                   thinBoxFeature=.THIN_BOX_FEATURES,
                                                   transcriptAnnotation=NULL
                                                   ),

                              BiomartGeneRegionTrack=list(
                                                          C_segment="burlywood4",
                                                          D_segment="lightblue",
                                                          J_segment="dodgerblue2",
                                                          Mt_rRNA="yellow",
                                                          Mt_tRNA="darkgoldenrod",
                                                          Mt_tRNA_pseudogene="darkgoldenrod1",
                                                          V_segment="aquamarine",
                                                          miRNA="cornflowerblue",
                                                          miRNA_pseudogene="cornsilk",
                                                          misc_RNA="cornsilk3",
                                                          misc_RNA_pseudogene="cornsilk4",
                                                          protein_coding="orange",
                                                          pseudogene="brown1",
                                                          rRNA="darkolivegreen1",
                                                          rRNA_pseudogene="darkolivegreen" ,
                                                          retrotransposed="blueviolet",
                                                          scRNA="gold4",
                                                          scRNA_pseudogene="darkorange2",
                                                          snRNA="coral",
                                                          snRNA_pseudogene="coral3",
                                                          snoRNA="cyan",
                                                          snoRNA_pseudogene="cyan2",
                                                          tRNA_pseudogene="antiquewhite3",
                                                          utr3="orange",
                                                          utr5="orange"
                                                          ),

                              GenomeAxisTrack=list(
                                                   add35=FALSE,
                                                   add53=FALSE,
                                                   background.title="transparent",
                                                   cex.id=0.7,
                                                   cex=0.8,
                                                   col.id="white",
                                                   col.range="cornsilk4",
                                                   distFromAxis=1,
                                                   exponent=NULL,
                                                   fill.range="cornsilk3",
                                                   fontcolor="#808080",
                                                   fontsize=10,
                                                   labelPos="alternating",
                                                   littleTicks=FALSE,
                                                   lwd=2,
                                                   scale=NULL,
                                                   showId=FALSE,
                                                   showTitle=FALSE,
                                                   size=NULL,
                                                   col="darkgray"
                                                   ),

                              DataTrack=list(
                                             aggregateGroups=FALSE,
                                             aggregation="mean",
                                             amount=NULL,
                                             baseline=NULL,
                                             box.ratio=1,
                                             box.width=NULL,
                                             cex.legend=0.8,
                                             cex.sampleNames=NULL,
                                             cex=0.7,
                                             coef=1.5,
                                             col.baseline=NULL,
                                             col.histogram=.DEFAULT_SHADED_COL,
                                             col.horizon=NA,
                                             col.mountain=NULL,
                                             col.sampleNames="white",
                                             col=trellis.par.get("superpose.line")[["col"]],
                                             collapse=FALSE,
                                             degree=1,
                                             do.out=TRUE,
                                             evaluation=50,
                                             factor=0.5,
                                             family="symmetric",
                                             fill.histogram=NULL,
                                             fill.horizon=c("#B41414", "#E03231", "#F7A99C", "#9FC8DC", "#468CC8", "#0165B3"),
                                             fill.mountain=c("#CCFFFF", "#FFCCFF"),
                                             fontcolor.legend=.DEFAULT_SHADED_COL,
                                             gradient=brewer.pal(9, "Blues"),
                                             groups=NULL,
                                             horizon.origin=0,
                                             horizon.scale=NULL,
                                             jitter.x=FALSE,
                                             jitter.y=FALSE,
                                             levels.fos=NULL,
                                             lty.baseline=NULL,
                                             lty.mountain=NULL,
                                             lwd.baseline=NULL,
                                             lwd.mountain=NULL,
                                             min.distance=0,
                                             na.rm=FALSE,
                                             ncolor=100,
                                             notch.frac=0.5,
                                             notch=FALSE,
                                             pch=20,
                                             separator=0,
                                             showColorBar=TRUE,
                                             showSampleNames=FALSE,
                                             size=NULL,
                                             span=1/5,
                                             stackedBars=TRUE,
                                             stats=boxplot.stats,
                                             transformation=NULL,
                                             type="p",
                                             varwidth=FALSE,
                                             window=NULL,
                                             windowSize=NULL,
                                             ylim=NULL
                                             ),

                              IdeogramTrack=list(
                                                 background.title="transparent",
                                                 bevel=0.45,
                                                 cex.bands=0.7,
                                                 cex=0.8,
                                                 col="red",
                                                 fill="#FFE3E6",
                                                 fontcolor=.DEFAULT_SHADED_COL,
                                                 fontsize=10,
                                                 showBandId=FALSE,
                                                 showId=TRUE,
                                                 showTitle=FALSE,
                                                 size=NULL
                                                 ),

                              SequenceTrack=list(
                                                 add53=FALSE,
                                                 background.title="transparent",
                                                 cex=1,
                                                 col="darkgray",
                                                 complement=FALSE,
                                                 fontcolor=getBioColor("DNA_BASES_N"),
                                                 fontface=2,
                                                 fontsize=10,
                                                 lwd=2,
                                                 min.width=2,
                                                 noLetters=FALSE,
                                                 rotation=0,
                                                 showTitle=FALSE,
                                                 size=NULL
                                                 ),

                              HighlightTrack=list(
                                                  col="red",
                                                  fill="#FFE3E6"
                                                  )

                              )

.schemes[["default.old"]] <- list(
                                  AnnotationTrack=list(col="transparent"))

## A helper function to be called upon package load that tries to find a stored Gviz scheme in the working directory
.collectSchemes <- function(){
    try({
    	if(".GvizSchemes" %in% base::ls(globalenv(), all.names=TRUE)){
            schemes <- get(".GvizSchemes", globalenv())
            if(is.list(schemes) && !is.null(names(schemes)))
                lapply(names(schemes), function(x) addScheme(schemes[[x]], x))
    	}
    },silent=TRUE)
}


## Helper function to select fontfaces
.chooseFace <- function (fontface = NULL, font = 1)
{
    if (is.null(fontface))
        font
    else fontface
}


## Get a scheme
## Arguments:
##    o name: the name of the scheme to get. Defaults to the current one.
## Value: A list containing the scheme
getScheme <- function(name=getOption("Gviz.scheme")){
    s <- NULL
    if(!is.null(name))
        s <- .schemes[[name]]
    if(is.null(s))
        s <- list()
    return(s)
}

## Add a new scheme
## Arguments:
##    o scheme: the scheme to add
##    o name: the name of the scheme to add
addScheme <- function(scheme, name){
    .schemes[[name]] <- scheme
}




## Helper function to compute native coordinate equivalent to 1 pixel and resize all ranges in a GRanges object accordingly
## Arguments:
##    o r: object inheriting from class GdObject
##    o min.width: the minimal pixel width of a region
##    o diff: the pixel resolution
## Value: the updated GdObject
.resize <- function(r, min.width=2, diff=.pxResolution(coord="x"))
{
    if(min.width>0){
        minXDiff <- ceiling(min.width*diff)
        ## Extend all ranges to at least minXDiff
        xdiff <- width(r)
        xsel <- xdiff < minXDiff
        if(any(xsel))
        {
            rr <- if(is(r, "GRanges")) ranges(r) else r
            start(rr)[xsel] <- pmax(1, start(rr)[xsel]-(minXDiff-xdiff[xsel])/2)
            end(rr)[xsel] <- end(rr)[xsel]+(minXDiff-xdiff[xsel])/2
            if(is(r, "GRanges")) r@ranges <- rr else r <- rr
        }
    }
    return(r)
}

## Tables containing the UCSC to ENSEMBL genome mapping
.biomartCurrentVersionTable <- read.delim(system.file(file.path("extdata", "biomartVersionsNow.txt"), package="Gviz"), as.is=TRUE)
.biomartVersionTable <- read.delim(system.file(file.path("extdata", "biomartVersionsLatest.txt"), package="Gviz"), as.is=TRUE)

## Helper function to map between UCSC and ENSEMBl genome information
## Arguments:
##    o id: character scalar, a UCSC genome identifier
## Value: a list with ENSEMBL the genome information
.ucsc2Ensembl <- function(id){
   mt <- match(tolower(id), tolower(.biomartCurrentVersionTable$ucscId))
   val <- .biomartCurrentVersionTable[mt, ]
   if(is.na(mt)){
       mt <- match(tolower(id), tolower(.biomartVersionTable$ucscId))
       val <- .biomartVersionTable[mt, c("species", "value", "dataset", "ucscId", "speciesShort", "speciesLong", "date", "version")]
   }
   return(as.list(val))
}

## Helper function to get the ENSEMBL biomart given a UCSC identifier
## Arguments:
##    o genome: character scalar, a UCSC genome identifier
## Value: a biomaRt object
.getBiomart <- function(genome){
    map <- .ucsc2Ensembl(genome)
    if(map$date == "head"){
        bm <- useMart("ensembl", dataset=map$dataset)
        ds <- listDatasets(bm)
        mt <- ds[match(map$dataset, ds$dataset), "version"]
        if(is.na(mt)){
            stop(sprintf(paste("Gviz thinks that the UCSC genome identifier '%s' should map to the Biomart data set '%s' which is not correct.",
                               "\nPlease manually provide biomaRt object"), genome, map$dataset))
        }
        if(mt != map$value){
            stop(sprintf(paste("Gviz thinks that the UCSC genome identifier '%s' should map to the current Biomart head as '%s',",
                               "but its current version is '%s'.\nPlease manually provide biomaRt object"),
                         genome, map$value, mt))
        }
    }else{
        bm <- useMart(host=sprintf("%s.archive.ensembl.org", tolower(sub(".", "", map$date, fixed=TRUE))), biomart="ENSEMBL_MART_ENSEMBL", dataset=map$dataset)
        ds <- listDatasets(bm)
        mt <- ds[match(map$dataset, ds$dataset), "version"]
        if(is.na(mt)){
            stop(sprintf(paste("Gviz thinks that the UCSC genome identifier '%s' should map to the Biomart data set '%s' which is not correct.",
                               "\nPlease manually provide biomaRt object"), genome, map$dataset))
        }
        if(mt != map$value){
            stop(sprintf(paste("Gviz thinks that the UCSC genome identifier '%s' should map to Biomart archive %s (version %s) as '%s',",
                               "but its version is '%s'.\nPlease manually provide biomaRt object"),
                         genome, sub(".", " ", map$date, fixed=TRUE), map$version, map$value, mt))
        }
    }
    return(bm)
}


## Helper function to translate from a UCSC genome name to a Biomart data set. This also caches the mart
## object in order to speed up subsequent calls
## Arguments:
##    o genome: character giving the UCSC genome
## Value: A BiomaRt connection object
.genome2Dataset <- function(genome)
{
    map <- .ucsc2Ensembl(genome)
    if(is.na(map$date)){
        stop(sprintf("Unable to automatically determine Biomart data set for UCSC genome identifier '%s'.\nPlease manually provide biomaRt object", genome))
    }
    cenv <- environment()
    bm <- .doCache(paste(map$dataset, genome, sep="_"), expression(.getBiomart(genome)), .ensemblCache, cenv)
    return(bm)
}



## Return the plotting range for a GdObject, either from the contained ranges or from overrides.
## This function is vectorized and should also work for lists of GdObjects.
.defaultRange <- function(GdObject, from=NULL, to=NULL, extend.left=0, extend.right=0, factor=0.01, annotation=FALSE)
{
    if(!is.list(GdObject))
        GdObject <- list(GdObject)
    GdObject <- c(GdObject, unlist(lapply(GdObject, function(x) if(is(x, "HighlightTrack") || is(x, "OverlayTrack")) x@trackList else NULL)))
    if(!length(GdObject) || !all(sapply(GdObject, is, "GdObject")))
        stop("All items in the list must inherit from class 'GdObject'")
    GdObject <- GdObject[!sapply(GdObject, is, "OverlayTrack")]
    tfrom <- lapply(GdObject, function(x){tmp <- start(x); if(is(x, "RangeTrack")) tmp <- tmp[seqnames(x)==chromosome(x)]; tmp})
    tfrom <- if(is.null(unlist(tfrom))) Inf else min(sapply(tfrom[listLen(tfrom)>0], min))
    tto <- lapply(GdObject, function(x){tmp <- end(x); if(is(x, "RangeTrack")) tmp <- tmp[seqnames(x)==chromosome(x)]; tmp})
    tto <- if(is.null(unlist(tto))) Inf else max(sapply(tto[listLen(tto)>0], max))
    if((is.null(from) || is.null(to)) && ((is.infinite(tfrom) || is.infinite(tto)) || is(GdObject, "GenomeAxisTrack")))
        stop("Unable to automatically determine plotting ranges from the supplied track(s).\nPlease provide ",
             "range coordinates through the 'from' and 'to' arguments of the plotTracks function.")
    ## FIX the cases with identical "tfrom" and "tto" (one base-pair plotting) by adding +1 to "tto"
    if (tto == tfrom) {
        tto <- tto + 1
    }
    range <- extendrange(r=c(tfrom, tto), f=factor)
    range[1] <- max(1, range[1])
    wasNull <- rep(FALSE, 2)
    if(is.null(from)){
        wasNull[1] <- TRUE
        from <- range[1]
    }
    if(is.null(to)){
        to <- range[2]
        wasNull[2] <- TRUE
    }
    ## We may need some extra space for annotations
    if(annotation){
        rr <- unlist(lapply(GdObject, function(x){
            gr <- .dpOrDefault(x, ".__groupRanges")
            gw <- .dpOrDefault(x, ".__groupLabelWidths", data.frame(before=0, after=0))
            if(is.null(gr) || length(gr) == 0) NULL else c(min(start(gr) + gw$before), max(end(gr) - gw$after))
        }))
        if(!is.null(rr)){
            rr <- matrix(rr, ncol=2, byrow=TRUE)
            if(wasNull[1])
               from <- min(from, rr[,1])
            if(wasNull[2])
                to <- max(to, rr[,2])
        }
    }
    from <- if(extend.left != 0 && extend.left > -1 && extend.left < 1){
        from - (abs(diff(c(from, to))) * extend.left)
    }else{
        from - extend.left
    }
    to <- if(extend.right != 0 && extend.right > -1 && extend.right < 1){
        to + (abs(diff(c(from, to))) * extend.right)
    }else{
        to + extend.right
    }
    if(from > to){
        warning("'from' range can not be larger than 'to', reversing range coordinates")
        tto <- from
        from <- to
        to <- tto
    }
    return(c(from=as.integer(from), to=as.integer(to)))
}


## Figure out the colors to use for a DataTrack object from the supplied display parameters
.getPlottingFeatures <- function(GdObject)
{
    pch <- .dpOrDefault(GdObject, "pch", 20)
    lty <- .dpOrDefault(GdObject, "lty", 1)
    lwd <- .dpOrDefault(GdObject, "lwd", 1)
    cex <- .dpOrDefault(GdObject, "cex", 0.7)
    groups <- .dpOrDefault(GdObject, "groups")
    col <- .dpOrDefault(GdObject, "col", "#0080ff")
    if(is.null(groups)){ ## When there are no groups we force a single color for all lines and points
        col <- col[1]
        col.line <- .dpOrDefault(GdObject, "col.line", col)[1]
        col.symbol <- .dpOrDefault(GdObject, "col.symbol", col)[1]
        pch <- pch[1]
        lwd <- lwd[1]
        lty <- lty[1]
        cex <- cex[1]
    } else { ## Otherwise colors are being mapped to group factors
        if(!is.factor(groups)) {
            groups <- factor(groups)
            nms <- unique(groups)
        } else {
            nms <- levels(groups)          
        }
        col <- .dpOrDefault(GdObject, "col", trellis.par.get("superpose.line")$col)
        col <- rep(col, nlevels(groups))[seq_along(levels(groups))]
        col.line <- rep(.dpOrDefault(GdObject, "col.line", col), nlevels(groups))[seq_along(levels(groups))]
        col.symbol <- rep(.dpOrDefault(GdObject, "col.symbol", col), nlevels(groups))[seq_along(levels(groups))]
        lwd <- rep(lwd, nlevels(groups))[seq_along(levels(groups))]
        lty <- rep(lty, nlevels(groups))[seq_along(levels(groups))]
        pch <- rep(pch, nlevels(groups))[seq_along(levels(groups))]
        cex <- rep(cex, nlevels(groups))[seq_along(levels(groups))]
        names(col) <- names(col.line) <- names(col.symbol) <- names(lwd) <- names(lty) <- names(pch) <- names(cex) <- nms
    }
    col.baseline <- .dpOrDefault(GdObject, "col.baseline", col)[1]
    col.grid <- .dpOrDefault(GdObject, "col.grid", "#e6e6e6")[1]
    fill <- .dpOrDefault(GdObject, "fill", .DEFAULT_FILL_COL)[1]
    fill.histogram <- .dpOrDefault(GdObject, "fill.histogram", fill)[1]
    col.histogram  <- .dpOrDefault(GdObject, "col.histogram", .dpOrDefault(GdObject, "col", .DEFAULT_SHADED_COL))[1]
    lty.grid <- .dpOrDefault(GdObject, "lty.grid", 1)
    lwd.grid <- .dpOrDefault(GdObject, "lwd.grid", 1)
    return(list(col=col, col.line=col.line, col.symbol=col.symbol, col.baseline=col.baseline,
                col.grid=col.grid, col.histogram=col.histogram, fill=fill, fill.histogram=fill.histogram,
                lwd=lwd, lty=lty, pch=pch, cex=cex, lwd.grid=lwd.grid, lty.grid=lty.grid))
}


.legendInfo <- function()
{
    legInfo <- matrix(FALSE, ncol=7, nrow=17, dimnames=list(c("p", "b", "l", "a", "s", "S", "r", "h", "smooth",
                                                              "histogram", "boxplot", "heatmap", "gradient", "mountain", "g", "horizon","confint"),
                                                            c("lty", "lwd", "pch", "col", "cex", "col.lines", "col.symbol")))
    legInfo[2:9, c("lty", "lwd", "col.lines")] <- TRUE
    legInfo[1:2, c("pch", "cex", "col.symbol")] <- TRUE
    legInfo[1:12, "col"] <- TRUE
    return(legInfo)
}

## A helper function to get the currently active chromosomes, also if the track is one of the collection
## track classes
.recChromosome <- function(GdObject){
    chroms <- if(is(GdObject, "HighlightTrack") || is(GdObject, "OverlayTrack"))
        unlist(lapply(GdObject@trackList, .recChromosome)) else chromosome(GdObject)
    return(unique(chroms))
}


## Plot a list of GdObjects as individual tracks similar to the display on the UCSC genome browser
## Arguments:
##    o trackList: a list of GdObjects
##    o from, to: the plotting range, will be figured out automatically from the tracks if missing
##    o sized: a vector of relative vertical sizes, or NULL to auto-detect
##    o panel.only: don't draw track titles, useful to embed in a lattice-like function, this also implies add=TRUE
##    o extend.right, extend.left: extend the coordinates in 'from' and 'too'
##    o title.width: the expansion factor for the width of the title track
## Value: the function is called for its side-effect of drawing on the graphics device
plotTracks <- function(trackList, from=NULL, to=NULL, ..., sizes=NULL, panel.only=FALSE, extend.right=0,
                       extend.left=0, title.width=NULL, add=FALSE, main, cex.main=2, fontface.main=2,
                       col.main="black", margin=6, chromosome=NULL, innerMargin=3){
    ## If we have to open a new device for this but do not run through the whole function because of errors we want to
    ## clean up in the end
    done <- FALSE
    cdev <- dev.cur()
    on.exit(if(cdev==1 && !done) dev.off())
    ## We only need a new plot for regular calls to the function. Both add==TRUE and panel.only=TRUE will add to an existing grid plot
    if(!panel.only && !add)
        grid.newpage()
    if(!is.list(trackList))
        trackList <- list(trackList)
    ## All arguments in ... are considered to be additional display parameters and need to be attached to each item in the track list
    dps <- list(...)
    trackList <- lapply(trackList, function(x) {
        displayPars(x, recursive=TRUE) <- dps
        return(x)
    })

    ## OverlayTracks and HighlightTracks can be discarded if they are empty
    trackList <- trackList[!sapply(trackList, function(x) (is(x, "HighlightTrack") || is(x, "OverlayTrack")) && length(x) < 1)]
    isHt <- which(sapply(trackList, is, "HighlightTrack"))
    isOt <- which(sapply(trackList, is, "OverlayTrack"))
    ## A mix between forward and reverse strand tracks should trigger an alarm
    strds <- unique(.whichStrand(trackList))
    if(!is.null(strds) && length(strds) > 1)
        warning("Plotting a mixture of forward strand and reverse strand tracks.\n Are you sure this is correct?")
    ## We first run very general housekeeping tasks on the tracks for which we don't really need to know anything about device
    ## size, resolution or plotting ranges.
    ## Chromosomes should all be the same for all tracks, if not we will force them to be set to the first one that can be detected.
    ## If plotting ranges are supplied we can speed up a lot of the downstream operations by subsetting first.
    ## We may want to use alpha blending on those devices that support it, but also fall back to non-transparent colors without causing
    ## warnings.
    hasAlpha <- .supportsAlpha()
    chrms <- unique(unlist(lapply(trackList, .recChromosome)))
    if(is.null(chromosome)){
        chrms <- if(!is.null(chrms)) chrms[gsub("^chr", "", chrms)!="NA"] else chrms
        chromosome <- head(chrms, 1)
        if(length(chromosome)==0)
            chromosome <- "chrNA"
        if(!is.null(chrms) && length(unique(chrms))!=1)
            warning("The track chromosomes in 'trackList' differ. Setting all tracks to chromosome '", chromosome, "'", sep="")
    }
    if(!is.null(from) || !(is.null(to))){
        trackList <- lapply(trackList, function(x){
            chromosome(x) <- chromosome
            subset(x, from=from, to=to, chromosome=chromosome, sort=FALSE, stacks=FALSE, use.defaults=FALSE)
        })
    }
    trackList <- lapply(trackList, consolidateTrack, chromosome=chromosome, any(.needsAxis(trackList)), any(.needsTitle(trackList)),
                                title.width, alpha=hasAlpha, ...)

    ## Now we figure out the plotting ranges. If no ranges are given as function arguments we take the absolute min/max of all tracks.
    ranges <- .defaultRange(trackList, from=from, to=to, extend.left=extend.left, extend.right=extend.right, annotation=TRUE)
    ## Now we can subset all the objects in the list to the current boundaries and compute the initial stacking
    trackList <- lapply(trackList, subset, from=ranges["from"], to=ranges["to"], chromosome=chromosome)
    trackList <- lapply(trackList, setStacks, recomputeRanges=FALSE)
    ## Highlight tracks are just a way to add a common highlighting region to several tracks, but other than that we can treat the containing
    ## tracks a normal track objects, and thus unlist them. We only want to record their indexes in the expanded list for later.
    htList <- list()
    expandedTrackList <- if(length(isHt)){
        j <- 1
        tlTemp <- list()
        for(i in seq_along(trackList)){
            if(! i %in% isHt){
                tlTemp <- c(tlTemp, trackList[[i]])
                j <- j+1
            }else{
                tlTemp <- c(tlTemp, trackList[[i]]@trackList)
                htList[[as.character(i)]] <- list(indexes=j:(j+length(trackList[[i]]@trackList)-1),
                                                  track=trackList[[i]])
                j <- j+length(trackList[[i]]@trackList)
            }
        }
        tlTemp
    }else trackList
    ## If there is a AlignmentsTrack and also a SequenceTrack we can tell the former to use the latter, unless already provided
    isAt <- sapply(expandedTrackList, is, "AlignmentsTrack")
    isSt <- sapply(expandedTrackList, is, "SequenceTrack")
    for(ai in which(isAt)){
        if(is.null(expandedTrackList[[ai]]@referenceSequence) && any(isSt))
            expandedTrackList[[ai]]@referenceSequence <- expandedTrackList[[min(which(isSt))]]
    }
    ## We need to reverse the list to get a top to bottom plotting order
    expandedTrackList <- rev(expandedTrackList)
    map <- vector(mode="list", length=length(expandedTrackList))
    titleCoords <- NULL
    names(map) <- rev(sapply(expandedTrackList, names))
    ## Open a fresh page and set up the bounding box, unless add==TRUE
    if(!panel.only) {
        ## We want a margin pixel border
        ## for backward compatibility, if margin has length of 2,
        ## the first one will be used as a horizontal, second as a vertical margin
        if (length(margin)==2) {
          margin <- rev(margin)
        }
        ## we switched to same settings as in par
        ## c(bottom, left, top, right)
        margin <- rep(as.numeric(margin), length.out = 4)
        vpWidth <- vpLocation()$size["width"]
        vpHeight <- vpLocation()$size["height"]
        vpBound <- viewport(x = margin[2L]/vpWidth, y = margin[1L]/vpHeight,
                           width = (vpWidth - sum(margin[c(2, 4)]))/vpWidth,
                           height = (vpHeight - sum(margin[c(1, 3)]))/vpHeight,
                           just = c("left", "bottom"))
        pushViewport(vpBound)
        ## If there is a header we have to make some room for it here
        if(!missing(main) && main != "")
        {
            vpHeader <- viewport(width=1, height=0.1, y=1, just=c("center", "top"))
            pushViewport(vpHeader)
            grid.text(main, gp=gpar(col=col.main, cex=cex.main, fontface=fontface.main))
            popViewport(1)
            vpMain <-  viewport(width=1, height=0.9, y=0.9, just=c("center", "top"))
        }else{
            vpMain <-  viewport(width=1, height=1)
        }
        pushViewport(vpMain)
        ## A first guestimate of the vertical space that's needed
        spaceSetup <- .setupTextSize(expandedTrackList, sizes, title.width, spacing=innerMargin)
    } else {
        vpBound <- viewport()
        pushViewport(vpBound)
        spaceSetup <- .setupTextSize(expandedTrackList, sizes, spacing=innerMargin)
    }
    ## First iteration to set up all the dimensions by calling the drawGD methods in prepare mode, i.e.,
    ## argument prepare=TRUE. Nothing is drawn at this point, and this only exists to circumvent the
    ## chicken and egg problem of not knowing how much space we need until we draw, but also not knowing
    ## where to draw until we know the space needed.
    for(i in rev(seq_along(expandedTrackList)))
    {
        fontSettings <- .fontGp(expandedTrackList[[i]], cex=NULL)
        vpTrack <-  viewport(x=0, y=sum(spaceSetup$spaceNeeded[1:i]), just=c(0,1), width=1, height=spaceSetup$spaceNeeded[i],
                             gp=fontSettings)
        pushViewport(vpTrack)
        vpContent <- if(!panel.only) viewport(x=spaceSetup$title.width + spaceSetup$spacing,
                                              width=1 - spaceSetup$title.width - spaceSetup$spacing * 2, just=0) else viewport(width=1)
        pushViewport(vpContent)
        expandedTrackList[[i]] <- drawGD(expandedTrackList[[i]], minBase=ranges["from"], maxBase=ranges["to"], prepare=TRUE, subset=FALSE)
        popViewport(2)
    }
    ## Now lets recalculate the space and draw for real
    spaceSetup <- .setupTextSize(expandedTrackList, sizes, title.width, spacing=innerMargin)
    ## First the highlight box backgrounds
    htBoxes <- data.frame(stringsAsFactors=FALSE)
    for(hlite in htList){
        if(length(ranges(hlite$track))){
            inds <- setdiff(sort(length(expandedTrackList)-hlite$index+1), which(sapply(expandedTrackList, is, "IdeogramTrack")))
            y <- reduce(IRanges(start=inds, width=1))
            yy <- ifelse(start(y)==1, 0, sum(spaceSetup$spaceNeeded[1:start(y)-1]))
            ht <- sum(spaceSetup$spaceNeeded[start(y):end(y)])
            htBoxes <- rbind(htBoxes, data.frame(y=yy, height=ht, x=start(hlite$track), width=width(hlite$track),
                                                 col=.dpOrDefault(hlite$track, "col", "orange"),
                                                 fill=.dpOrDefault(hlite$track, "fill", "red"),
                                                 lwd=.dpOrDefault(hlite$track, "lwd", 1),
                                                 lty=.dpOrDefault(hlite$track, "lty", 1),
                                                 alpha=.dpOrDefault(hlite$track, "alpha", 1),
                                                 inBackground=.dpOrDefault(hlite$track, "inBackground", TRUE),
                                                 stringsAsFactors=FALSE))
        }
    }
    .drawHtBoxes <- function(htBoxes, background=TRUE){
        htBoxes <- htBoxes[htBoxes$inBackground == background, , drop=FALSE]
        rscales = if(strds[1] == "reverse") c(from=ranges["to"], to=ranges["from"]) else ranges
        if(nrow(htBoxes)){
            vpContent <- if(!panel.only) viewport(x=spaceSetup$title.width + spaceSetup$spacing, xscale=rscales,
                                                  width=1 - spaceSetup$title.width - spaceSetup$spacing*2, just=0) else {
                                                      viewport(width=1, xscale=rscales)}
            pushViewport(vpContent)
            grid.rect(x=htBoxes$x, just=c(0,1), width=htBoxes$width, y=htBoxes$y+htBoxes$height, height=htBoxes$height,
                      gp=gpar(col=htBoxes$col, fill=htBoxes$fill, lwd=htBoxes$lwd, lty=htBoxes$lty, alpha=unique(htBoxes$alpha)), default.units="native") # fix for Sys.setenv(`_R_CHECK_LENGTH_1_CONDITION_`="true") Sys.setenv(`_R_CHECK_LENGTH_1_LOGIC2_`="true")
            popViewport(1)
        }
    }
    if(nrow(htBoxes))
        .drawHtBoxes(htBoxes)
    ## Now the track content
    for(i in rev(seq_along(expandedTrackList)))
    {
        vpTrack <-  viewport(x=0, y=sum(spaceSetup$spaceNeeded[1:i]), just=c(0,1), width=1, height=spaceSetup$spaceNeeded[i])
        pushViewport(vpTrack)
		fill <- .dpOrDefault(expandedTrackList[[i]], "background.title", .DEFAULT_SHADED_COL)
        thisTrack <- if(is(expandedTrackList[[i]], "OverlayTrack")) expandedTrackList[[i]]@trackList[[1]] else expandedTrackList[[i]]
        if(!panel.only) {
            fontSettings <- .fontGp(expandedTrackList[[i]], subtype="title", cex=NULL)
            vpTitle <- viewport(x=0, width=spaceSetup$title.width, just=0, gp=fontSettings)
            pushViewport(vpTitle)
            lwd.border.title <- .dpOrDefault(thisTrack, "lwd.title", 1)
            col.border.title <- .dpOrDefault(thisTrack, "col.border.title", "transparent")
            grid.rect(gp=gpar(fill=fill, col=col.border.title, lwd=lwd.border.title))
            needAxis <- .needsAxis(thisTrack)
            drawAxis(thisTrack, ranges["from"], ranges["to"], subset=FALSE)
            tit <- spaceSetup$nwrap[i]
            ## FIXME: Do we want something smarted for the image map coordinates?
            titleCoords <- rbind(titleCoords, cbind(.getImageMap(cbind(0,0,1,1)),
                                                    title=names(expandedTrackList[[i]])))
            if(.dpOrDefault(thisTrack, "showTitle", TRUE) && !is.null(tit) && tit!="")
            {
                x <- if(needAxis) 0.075 else 0.4
                just <- if(needAxis) c("center", "top") else "center"
                ## FIXME: We need to deal with this when calculating the space for the title bar
                rot <- .dpOrDefault(thisTrack, "rotation.title", 90)
                gp <- .fontGp(thisTrack, "title", cex=spaceSetup$cex[i])
                suppressWarnings(grid.text(tit, unit(x, "npc"), rot=rot, gp=gp, just=just))
            }
            popViewport(1)
        }
        ## Draw the panel background, grid lines if necessary and the panel content
        vpBackground <- if(!panel.only) viewport(x=spaceSetup$title.width,
                                                 width=1-spaceSetup$title.width, just=0) else viewport(width=1)
        pushViewport(vpBackground)
        grid.rect(gp=gpar(col="transparent", fill=.dpOrDefault(thisTrack, "background.panel", "transparent")))
        drawGrid(thisTrack, ranges["from"], ranges["to"])
        popViewport(1)
        fontSettings <- .fontGp(expandedTrackList[[i]], cex=NULL)
        vpContentOuter <- if(!panel.only)
            viewport(x=spaceSetup$title.width, width=1-spaceSetup$title.width,
                     just=0, gp=fontSettings, clip=TRUE) else viewport(width=1, gp=fontSettings, clip=TRUE)
        pushViewport(vpContentOuter)
        vpContent <- if(!panel.only)
            viewport(x=spaceSetup$spacing, width=1-(spaceSetup$spacing*2), just=0, gp=fontSettings) else viewport(width=1, gp=fontSettings)
        pushViewport(vpContent)
        tmp <- drawGD(expandedTrackList[[i]], minBase=ranges["from"], maxBase=ranges["to"], subset=FALSE)
        if(!is.null(tmp))
            map[[(length(map)+1)-i]] <- tmp
        popViewport(2)
        if(.dpOrDefault(thisTrack, "frame", FALSE))
            grid.rect(gp=gpar(col=.dpOrDefault(thisTrack, "col.frame", .DEFAULT_SHADED_COL), fill="transparent"))
        popViewport(1)
    }
    if(nrow(htBoxes))
        .drawHtBoxes(htBoxes, FALSE)
    popViewport(if(panel.only) 1 else 2)
    tc <- as.character(titleCoords[,5])
    tc[which(tc == "" | is.na(tc) | is.null(tc))] = "NA"
    names(tc) <- tc
    if(!is.null(titleCoords))
    {
        tcoord <- as.matrix(titleCoords[,1:4])
        rownames(tcoord) <- names(tc)
        map$titles <- ImageMap(coords=tcoord, tags=list(title=tc))
    }
    done <- TRUE
    return(invisible(map))
}



## Try to extract the (unique) genome information from a GRanges objects with the possibility to fall back to a default value
.getGenomeFromGRange <- function(range, default=NULL){
    gn <- genome(range)
    if(length(unique(gn))>1)
        warning("Only a single genome is supported for this object. Ignoring additional genome information")
    if(length(gn)==0 || all(is.na(gn))){
        if(is.null(default))
            stop("A genome must be supplied when creating this object.")
        return(default[1])
    }
    return(gn[1])
}










## Write all tracks in a list of tracks into
## a single BED file.
exportTracks <- function(tracks, range, chromosome, file)
{
    if(missing(file))
        file <- "customTracks.bed"
    con <- file(file, open="wt")
    writeLines(sprintf("browser position %s:%i-%i", chromosome, range[1], range[2]),
               con)
    writeLines("browser hide all", con)
    for(t in seq_along(tracks))
    {
        track <- tracks[[t]]
        if(length(track)>0 && (is(track, "AnnotationTrack") || is(track, "GeneRegion")))
        {
            track <- as(track, "UCSCData")
            writeLines(as(track@trackLine, "character"), con)
            ## nextMet <- selectMethod("export.bed", c("RangedData", "characterORconnection"))
            ## nextMet(as(track, "RangedData"), con)
            .expBed(as(track, "RangedData"), con)
        }
    }
    close(con)
}

## This funcion is broken in the rtracklayer package
.expBed <- function (object, con, variant = c("base", "bedGraph", "bed15"), color, append)
{
    variant <- match.arg(variant)
    name <- strand <- thickStart <- thickEnd <- color <- NULL
    blockCount <- blockSizes <- blockStarts <- NULL
    df <- data.frame(chrom(object), start(object) - 1, end(object))
    score <- score(object)
    if (!is.null(score)) {
        if (!is.numeric(score) || any(is.na(score)))
            stop("Scores must be non-NA numeric values")
    }
    if (variant == "bedGraph") {
        if (is.null(score))
            score <- 0
        df$score <- score
    }
    else {
        blockSizes <- object$blockSizes
            blockStarts <- object$blockStarts
        if (variant == "bed15" && is.null(blockSizes))
            blockStarts <- blockSizes <- ""
        if (!is.null(blockSizes) || !is.null(blockStarts)) {
            if (is.null(blockSizes))
                stop("'blockStarts' specified without 'blockSizes'")
            if (is.null(blockStarts))
                stop("'blockSizes' specified without 'blockStarts'")
            lastBlock <- function(x) sub(".*,", "", x)
            lastSize <- lastBlock(blockSizes)
            lastStart <- lastBlock(blockStarts)
            if (any(df[[2]] + as.integer(lastSize) + as.integer(lastStart) != df[[3]]) ||
                any(sub(",.*", "", blockStarts) != 0))
                stop("blocks must span entire feature")
            blockCount <- sapply(strsplit(blockSizes, ","), length)
        }
        if (is.null(color))
            color <- object$itemRgb
        if (is.null(color) && !is.null(blockCount))
            color <- "0"
        else if (!is.null(color)) {
            nacol <- is.na(color)
            colmat <- col2rgb(color)
            color <- paste(colmat[1, ], colmat[2, ], colmat[3,
                                                            ], sep = ",")
            color[nacol] <- "0"
        }
        thickStart <- object$thickStart
        thickEnd <- object$thickEnd
        if (is.null(thickStart) && !is.null(color)) {
            thickStart <- start(object)
            thickEnd <- end(object)
        }
        strand <- object$strand
        if (!is.null(thickStart) && is.null(strand)) {
            strand <- rep(NA, nrow(object))
        }
        if (!is.null(strand) && is.null(score))
            score <- 0
        name <- object$name
        if (is.null(name))
            name <- rownames(object)
        if (!is.null(score) && is.null(name))
            name <- rep(NA, nrow(object))
        df$name <- name
        df$score <- score
        df$strand <- strand
        df$thickStart <- thickStart
        df$thickEnd <- thickEnd
        df$itemRgb <- color
        df$blockCount <- blockCount
        df$blockSizes <- blockSizes
        df$blockStarts <- blockStarts
        if (variant == "bed15") {
            df$expCount <- object$expCount
            df$expIds <- object$expIds
            df$expScores <- object$expScores
        }
    }
    scipen <- getOption("scipen")
    options(scipen = 100)
    on.exit(options(scipen = scipen))
    write.table(df, con, sep = "\t", col.names = FALSE, row.names = FALSE,
                quote = FALSE, na = ".", append = append)
}





## Construct a URL to UCSC showing the custom tracks
ucscUrl <- function(chr, range, spec, gen, open=TRUE)
{
    hgid <- system(sprintf("%s %s %s", system.file("lib/testUCSC.pl", package="Gviz"),
                           "customTracks.bed", spec, gen), intern=TRUE, ignore.stderr=TRUE)

    url <- sprintf(paste("http://genome.ucsc.edu/cgi-bin/hgTracks?hgsid=%s&Submit=go+to+genome+browser",
                         "&position=%s%%3A%i-%i", sep=""), hgid, chr, range[1], range[2])
    if(open)
        browseURL(url)
    return(url)
}




.updateObj <- function(object)
{
    availSlots <- getObjectSlots(object)
    availSlotNames <- names(availSlots)
    definedSlotNames <- slotNames(object)
    if(length(availSlotNames)==length(definedSlotNames) && all(sort(availSlotNames) == sort(definedSlotNames)))
        return(object)
    commonSlots <- intersect(definedSlotNames, availSlotNames)
    missingSlots <- setdiff(definedSlotNames, availSlotNames)
    newObject <- new(class(object))
    for (s in commonSlots)
        slot(newObject, s) <- availSlots[[s]]
    return(newObject)
}



vpLocation <- function(){
  xres <- devRes()[1]
  yres <- devRes()[2]
  ## find location and pixel-size of current viewport
  devloc1 <- c(convertX(unit(0, "npc"), "inches"),
              convertY(unit(0, "npc"), "inches"), 1)  %*% current.transform()
  devloc2 <- c(convertX(unit(1, "npc"), "inches"),
              convertY(unit(1, "npc"), "inches"), 1)  %*% current.transform()
  x1 <- (devloc1/devloc1[3])[1]*xres
  y1 <- (devloc1/devloc1[3])[2]*yres
  x2 <- (devloc2/devloc2[3])[1]*xres
  y2 <- (devloc2/devloc2[3])[2]*yres
  loc <- c(x1,y1,x2,y2)
  names(loc) <- c("x1", "y1", "x2", "y2")
  size <- c(x2-x1, y2-y1)
  names(size) <- c("width", "height")
  iloc <- c(x1/xres, y1/yres, x2/yres, y2/yres)
  names(iloc) <- c("x1", "y1", "x2", "y2")
  isize <- size/c(xres,yres)
  names(size) <- c("width", "height")
  return(list(location=loc, size=size, ilocation=iloc,
              isize=isize))
}



devRes <- function(){
  ## find R's resolution for the current device
  if(current.viewport()$name != "ROOT"){
    vpt <- current.vpTree()
    depth <- upViewport(0)
    xres <- abs(as.numeric(convertWidth(unit(1, "inches"), "native")))
    yres <- abs(as.numeric(convertHeight(unit(1, "inches"), "native")))
    downViewport(depth)
  }else{
    xres <- abs(as.numeric(convertWidth(unit(1, "inches"), "native")))
    yres <- abs(as.numeric(convertHeight(unit(1, "inches"), "native")))
  }
  retval <- c(xres, yres)
  names(retval) <- c("xres", "yres")
  return(retval)
}



devDims <- function(width, height, ncol=12, nrow=8, res=72){
 f <- (((ncol+1)*0.1+ncol+1)/((nrow+1)*0.1+nrow+1))
 if((missing(width) & missing(height) || !missing(width) & !missing(height)))
   stop("Need either argument 'width' or argument 'height'")
 if(missing(height))
   return(list(width=width, height=width/f, pwidth=width*res, pheight=width/f*res))
 else
   return(list(width=height*f, height, pwidth=height*f*res, pheight=height*res))
}

## Record the display parameters for each class once
.makeParMapping <- function()
{
    classes <-  c("GdObject", "GenomeAxisTrack", "RangeTrack", "NumericTrack", "DataTrack", "IdeogramTrack", "StackedTrack",
                  "AnnotationTrack", "DetailsAnnotationTrack", "GeneRegionTrack", "BiomartGeneRegionTrack", "AlignedReadTrack")
    defs <- try(sapply(classes, function(x) as(getClassDef(x)@prototype@dp, "list"), simplify=FALSE), silent=TRUE)
    if(!is(defs, "try-error") && is.null(.parMappings))
        assignInNamespace(x=".parMappings", value=defs, ns="Gviz")
}
.parMappings <- NULL

## Show available display parameters for a class and their defaults
availableDisplayPars <- function(class)
{
    if(!is.character(class))
        class <- class(class)
    class <- match.arg(class, c("GdObject", "GenomeAxisTrack", "RangeTrack", "NumericTrack", "DataTrack", "IdeogramTrack", "StackedTrack",
                                "AnnotationTrack", "DetailsAnnotationTrack", "GeneRegionTrack", "BiomartGeneRegionTrack", "AlignedReadTrack",
                                "SequenceTrack", "SequenceBSgenomeTrack", "SequenceDNSStringSetTrack"))
    parents <- names(getClassDef(class)@contains)
    .makeParMapping()
    pars <- .parMappings[c(parents, class)]
    finalPars <- inherited <- list()
    for(p in names(pars))
    {
        finalPars[names(pars[[p]])] <- pars[[p]]
        inherited[names(pars[[p]])] <- p
    }
    finalPars <- finalPars[order(names(finalPars))]
    inherited <- inherited[order(names(inherited))]
    return(new("InferredDisplayPars", name=class, inheritance=unlist(inherited), finalPars))
}

## Compute ellipse outline coordinates for bounding boxes
.box2Ellipse <- function(box, np=50)
{
    t <- seq(0, 2*pi, len=np)
    box$width <- box$cx2-box$cx1
    box$height <- box$cy2-box$cy1
    x <- rep(box$cx2 + -box$width + box$width/2, each=np)
    y <- rep(box$cy1 + box$height/2, each=np)
    a <- rep(box$width/2, each=np)
    b <- rep(box$height/2, each=np)
    tau <- 0
    xt <- x + (a*cos(t)*cos(tau)-b*sin(t)*sin(tau))
    yt <- y + (a*cos(t)*sin(tau)+b*sin(t)*cos(tau))
    return(data.frame(x1=xt, y1=yt, id=rep(seq_len(nrow(box)), each=np)))
}

## We store some preset in the options on package load
.onLoad = function(...){
    .collectSchemes()
    options("ucscChromosomeNames"=TRUE, "Gviz.scheme"="default", "Gviz.ucscUrl"=NULL)
}


## A helper function to replace missing function arguments in a list with NULL values. The function environment needs
## to be passed in as argument 'env' for this to work.
.missingToNull <- function(symbol, env=parent.frame()){
    for(i in symbol){
        mis <- try(do.call(missing, args=list(i), envir=env), silent=TRUE)
        if(!is(mis, "try-error") && mis)
            assign(i, NULL, env)
    }
}


## build a covariates data.frame from a variety of different inputs
.getCovars <- function(x){
    if(is.data.frame(x)){
        x
    }else{
        if(is(x, "GRanges")){
            as.data.frame(mcols(x))
        }else{
            if(is(x, "GRangesList")){
                as.data.frame(mcols(unlist(x)))
            }else{
                data.frame()
                ## stop(sprintf("Don't know how to extract covariates from a %s object", class(x)))
            }
        }
    }
}


## Prepare a data.frame or matrix containing the data for a DataTrack object. This involves trying to coerce
## and dropping non-numeric columns with a warning
.prepareDtData <- function(data, len=0){
    if(ncol(data) && nrow(data)){
        for(i in seq_along(data)){
            if(is.character(data[,i]))
                data[,i] <- type.convert(data[,i], as.is=TRUE)
        }
        isNum <- sapply(data, is.numeric)
        if(any(!isNum))
            warning(sprintf("The following non-numeric data column%s been dropped: %s", ifelse(sum(!isNum)>1, "s have", " has"),
                            paste(colnames(data)[!isNum], collapse=", ")))
        if(sum(dim(data))>0){
            data <- t(data[,isNum, drop=FALSE])
        }
    }else{
        data <- matrix(ncol=len, nrow=0)
    }
    if(all(is.na(data)))
        data <- matrix(ncol=len, nrow=0)
    if(ncol(data) != len)
        stop("The columns in the 'data' matrix must match the genomic regions.")
    return(data)
}



## An import function for gff3 files that tries to resolve the parent-child relationship
## between genes, transcripts and exons
.import.gff3 <- function(file){
    dat <- import.gff3(file)
    res <- try({
        genes <- tolower(dat$type) == "gene"
        ginfo <- mcols(dat[genes, ])
        dat <- dat[!genes]
        transcripts <- tolower(dat$type) == "mrna"
        tinfo <- mcols(dat[transcripts, ])
        dat <- dat[!transcripts]
        mt <- match(as.character(dat$Parent), as.character(tinfo$ID))
        if(!all(is.na(mt))){
            if(!"transcript_id" %in% colnames(mcols(dat))) {
                tid <- rep(NA, length(mt))
                tid[!is.na(mt)] <- tinfo[mt[!is.na(mt)], "ID"]
                mcols(dat)[["transcript_id"]] <- tid
            }
            if(!"transcript_name" %in% colnames(mcols(dat))) {
                tn <- rep(NA, length(mt))
                tn[!is.na(mt)] <- tinfo[mt[!is.na(mt)], "Name"]
                mcols(dat)[["transcript_name"]] <- tn
            }
            mt2 <- rep(NA, dim(mcols(dat))[1])
            mt2[!is.na(mt)] <- match(as.character(tinfo[mt[!is.na(mt)], "Parent"]), as.character(ginfo$ID))

            if(!all(is.na(mt2))){
                if(!"gene_id" %in% colnames(mcols(dat))) {
                    gid <- rep(NA, length(mt2))
                    gid[!is.na(mt2)] <- ginfo[mt[!is.na(mt2)], "ID"]
                    mcols(dat)[["gene_id"]] <- gid
                }
                if(!"gene_name" %in% colnames(mcols(dat))) {
                    gn <- rep(NA, length(mt2))
                    gn[!is.na(mt2)] <- ginfo[mt[!is.na(mt2)], "Name"]
                    mcols(dat)[["gene_name"]] <- gn
                }
            }
        }
        if(all(is.na(mcols(dat)[["ID"]])))
            mcols(dat)[["ID"]] <- paste("item", seq_along(dat), sep="_")
        if(!"exon_id" %in% colnames(mcols(dat)))
            mcols(dat)[["exon_id"]] <- mcols(dat)[["ID"]]
        if(!is.null(mcols(dat)[["gene_name"]]) && all(is.na(mcols(dat)[["gene_name"]])))
            mcols(dat)[["gene_name"]] <- NULL
        if(all(is.na(mcols(dat)[["transcript_name"]])))
            mcols(dat)[["transcript_name"]] <- NULL
        dat
    })
    if(is(res, "try-error")){
        warning(sprintf(paste("File '%s' is not valid according to the GFF3 standard and can not be properly parsed.",
                              "Results may not be what you expected!"), file))
        res <- dat
    }
    return(res)
}

## An import function for bigWig files that knowns how to deal with missing seqnames
.import.bw <- function(file, selection){
    bwf <- BigWigFile(path.expand(file))
    if(missing(selection)){
        rr <- import.bw(con=bwf)
    }else{
        si <- seqinfo(bwf)
        rr <- if(!as.character(seqnames(selection)[1]) %in% seqnames(seqinfo(bwf))){
            GRanges(seqnames(selection)[1], ranges=IRanges(1,2), score=1)[0] }else{
                import.bw(con=bwf, selection=selection)}
    }
    return(rr)
}

## An import function for bam files that distinguishes between DataTracks and AnnotationTracks
## FIXME: We probably want this to be able to deal with Gapped Alignments...
.import.bam <- function(file, selection){
    if(!file.exists(paste(file, "bai", sep=".")) &&
       !file.exists(paste(paste(head(strsplit("xxx.bam", ".", fixed=TRUE)[[1]], -1), collapse="."), "bai", sep=".")))
        stop("Unable to find index for BAM file '", file, "'. You can build an index using the following command:\n\t",
             "library(Rsamtools)\n\tindexBam(\"", file, "\")")
    sinfo <- scanBamHeader(file)[[1]]
    if(parent.env(environment())[["._trackType"]] == "DataTrack"){
        res <- if(!as.character(seqnames(selection)[1]) %in% names(sinfo$targets)){
            mcols(selection) <- DataFrame(score=0)
            selection
        }else{
            param <- ScanBamParam(what=c("pos", "qwidth"), which=selection, flag=scanBamFlag(isUnmappedQuery=FALSE))
            x <- scanBam(file, param=param)[[1]]
            cov <- coverage(IRanges(x[["pos"]], width=x[["qwidth"]]))
            if(length(cov)==0){
                mcols(selection) <- DataFrame(score=0)
                selection
            }else{
                GRanges(seqnames=seqnames(selection), ranges=IRanges(start=start(cov), end=end(cov)), strand="*", score=runValue(cov))
            }
        }
    } else {
        res <- if(!as.character(seqnames(selection)[1]) %in% names(sinfo$targets)){
            mcols(selection) <- DataFrame(id="NA", group="NA")
            selection[0]
        }else{
            param <- ScanBamParam(what=c("pos", "qwidth", "strand", "qname"), which=selection, flag=scanBamFlag(isUnmappedQuery=FALSE))
            x <- scanBam(file, param=param)[[1]]
            GRanges(seqnames=seqnames(selection), ranges=IRanges(start=x[["pos"]], width=x[["qwidth"]]), strand=x[["strand"]],
                    id=make.unique(x[["qname"]]), group=x[["qname"]])
        }
    }
    return(res)
}

.import.bam.alignments <- function(file, selection){
    indNames <- c(sub("\\.bam$", ".bai", file), paste(file, "bai", sep="."))
    index <- NULL
    for(i in indNames){
        if(file.exists(i)){
            index <- i
            break
        }
    }
    if(is.null(index))
        stop("Unable to find index for BAM file '", file, "'. You can build an index using the following command:\n\t",
             "library(Rsamtools)\n\tindexBam(\"", file, "\")")
    pairedEnd <- parent.env(environment())[["._isPaired"]]
    if(is.null(pairedEnd))
        pairedEnd <- TRUE
    bf <- BamFile(file, index=index, asMates=pairedEnd)
    param <- ScanBamParam(which=selection, what=scanBamWhat(), tag="MD", flag=scanBamFlag(isUnmappedQuery=FALSE))
    reads <- if(as.character(seqnames(selection)[1]) %in% names(scanBamHeader(bf)$targets)) scanBam(bf, param=param)[[1]] else list()
    md <- if(is.null(reads$tag$MD)) rep(as.character(NA), length(reads$pos)) else reads$tag$MD
    if(length(reads$pos)){
        layed_seq <- sequenceLayer(reads$seq, reads$cigar)
        region <- unlist(bamWhich(param), use.names=FALSE)
        ans <- stackStrings(layed_seq, start(region), end(region), shift=reads$pos-1L, Lpadding.letter="+", Rpadding.letter="+")
        names(ans) <- seq_along(reads$qname)
    }else{
        ans <- DNAStringSet()
    }
    return(GRanges(seqnames=if(is.null(reads$rname)) character() else reads$rname,
                   strand=if(is.null(reads$strand)) character() else reads$strand,
                   ranges=IRanges(start=reads$pos, width=reads$qwidth),
                   id=reads$qname, cigar=reads$cigar, mapq=reads$mapq, flag=reads$flag, md=md, seq=ans,
                   isize=reads$isize, groupid=if(pairedEnd) reads$groupid else seq_along(reads$pos),
                   status=if(pairedEnd) reads$mate_status else rep(factor("unmated", levels=c("mated", "ambiguous", "unmated")),
                                                                   length(reads$pos))))
}


## An import function for fasta file that supports streaming if an index is present
.import.fasta <- function(file, selection, strict=TRUE){
    ffile <- FastaFile(file)
    if(!file.exists(paste(file, "fai", sep="."))){
        if(strict){
            stop("Unable to find index for fasta file '", file, "'. You can build an index using the following command:\n\t",
                 "library(Rsamtools)\n\tindexFa(\"", file, "\")")
        }else{
            return(readDNAStringSet(file))
        }
    }
    idx <- scanFaIndex(file)
    if(!as.character(seqnames(selection)[1]) %in% as.character(seqnames(idx))){
         return(DNAStringSet())
    }else{
        return(scanFa(file, selection))
    }
}


## An import function for the indexed 2bit format
.import.2bit <- function(file, selection){
    tbf <- TwoBitFile(file)
    if(!as.character(seqnames(selection)[1]) %in% seqnames(seqinfo(tbf))){
        return(DNAStringSet())
    }else{
        tmp <- import(tbf, which=selection)
        names(tmp) <- as.character(seqnames(selection)[1])
        return(tmp)
    }
}



## A mapping of (lower-cased) file extensions to import function calls. Most of those are already implemented in the rtracklayer package.
## If no mapping is found an error will be raised suggesting to provide a user-defined import function.
.registerImportFun <- function(file){
    fileExt <- .fileExtension(file)
    file <- path.expand(file)
    return(switch(fileExt,
                  "gff"=import.gff(file),
                  "gff1"=import.gff1(file),
                  "gff2"=import.gff2(file),
                  "gff3"=.import.gff3(file),
                  "gtf"=import.gff2(file),
                  "bed"=import.bed(file),
                  "bedgraph"=import.bedGraph(file),
                  "wig"=import.wig(file),
                  "bw"=.import.bw,
                  "bigwig"=.import.bw,
                  "bam"=.import.bam,
                  stop(sprintf("No predefined import function exists for files with extension '%s'. Please manually provide an import function.",
                               fileExt))))
}


## Get the file extension for a file, taking into account potential gzipping
.fileExtension <- function(file){
    if(!grepl("\\.", file))
        stop("Unable to identify extension for file '", file, "'")
     ext <- sub(".*\\.", "", sub("\\.gz$|\\.gzip$", "", basename(file)))
    if(ext=="")
        stop("Unable to identify extension for file '", file, "'")
    return(tolower(ext))
}


availableDefaultMapping <- function(file, trackType){
    .checkClass(file, "character", 1)
    .checkClass(trackType, "character", 1)
    ext <- tolower(if(grepl("\\.", file)) .fileExtension(file) else file)
    vm <- .defaultVarMap(ext, trackType, justMap=TRUE)
    vm[[".stream"]] <- NULL
    return(vm)
}


## Helper function to handle defaults function arguments
.covDefault <- function(x, cov, def){
    res <- if(missing(x)){
        if(is.null(cov)){
            def
        }else{
            cov
        }
    }else{x}
    return(res)
}


## A helper function to process alignment information from a GRanges object
.computeAlignments <- function(range){
    res <- list(range=range, stackRanges=GRanges(), stacks=numeric())
    if(length(range)){
        alg <- extractAlignmentRangesOnReference(range$cigar)
        rp <- elementNROWS(alg)
        range <- sort(GRanges(seqnames=rep(seqnames(range), rp), strand=rep("*", sum(rp)), ranges=shift(unlist(alg), rep(start(range), rp)-1),
                              id=rep(range$id, rp), entityId=rep(seq_along(rp), rp), cigar=rep(range$cigar, rp), md=rep(range$md, rp),
                              readStrand=rep(strand(range), rp), mapq=rep(range$mapq, rp), flag=rep(range$flag, rp), isize=rep(range$isize, rp),
                              groupid=rep(range$groupid, rp), status=factor(rep(range$status, rp), levels=c("mated", "ambiguous", "unmated")),
                              uid=seq_len(sum(rp))))
        if(length(range)){
            stTmp <- split(range, range$groupid)
            stackRanges <- unlist(range(stTmp))
            ss <- disjointBins(stackRanges)
            range$stack <- ss[match(range$groupid, names(stackRanges))]
            res <- list(range=range, stackRanges=stackRanges, stacks=range$stack)
        }
    }
    return(res)
}

## Check whether the current device supports alpha channel transparency
.supportsAlpha <- function(){
    d <- dev.cur()
    oldwarn <- getOption("warn")
    on.exit({options(warn=oldwarn)
             if(d==1)
                 dev.off()
         })
    options(warn=2)
    ok <- !is(try({grob <- grid.rect(x=unit(0, "npc"), y=unit(0, "npc"), width=0, height=0, gp=gpar(alpha=0.5))
                   ##grid.remove(grob$name)
               }, silent=TRUE), "try-error")
    return(ok)
}

## Return the alpha display parameter from a GdObject, respecting whether transparency is supported on the device
## or not. This is either drawn from the internal '.__hasAlphaSupport' display parameter, or, if not set, is
## determined dynamically.
.alpha <- function(GdObject, postfix=NULL){
    support <- .dpOrDefault(GdObject, ".__hasAlphaSupport", .supportsAlpha())
    wh <- if(is.null(postfix)) "alpha" else c(paste("alpha", postfix, sep="."), "alpha")
    alpha <- .dpOrDefault(GdObject, wh, 1)
    if(alpha != 1 && !support)
        alpha <- 1
    return(alpha)
}

## Draw horizontal arrows into a viewport indicating cropped read alignments
.moreInd <- function(n=3, direction="up", ...){
    nn <- n * 2 + 1
    x <- rep(seq(1/nn, 1 - (1/nn), len=nn-2)[seq(1, nn-2, by=2)], each=n) + c(-1/nn/2, 0, 1/nn/2)
    y <- rep(if(direction == "up") c(0, 1, 0) else c(1, 0, 1), n)
    grid.polyline(x, y, id=rep(1:n, each=3), gp=gpar(...))
}



## Compute mismatches for AlignmentsTracks based on the read sequences and a reference sequence
.findMismatches <- function(GdObject){
    rgo <- .dpOrDefault(GdObject, ".__plottingRange")
    mmPos <- mmSamp <- mmSeq <- mmStack <- NULL
    if(!is.null(rgo)){
        ref <- as.character(as(subseq(GdObject@referenceSequence, start=rgo["from"], end=rgo["to"]), "Rle"))
        cm <- consensusMatrix(GdObject@sequences, as.prob=FALSE, baseOnly=TRUE)[-5,]
        cmm <- colMaxs(cm)
        css <- colSums(cm)
        cmp <- rbind(t(t(cm)/css), 0)
        rownames(cmp)[5] <- "N"
        sel <- is.na(cmp["A",])
        cmp[,sel] <- 0
        cmp["N", sel] <- 1
        consStr <- strsplit(consensusString(cmp), "")[[1]]
        varRegs <- which(cmm != css | (consStr != "N" & consStr != ref))
        if(length(varRegs)){
            rvg <- ref[varRegs]
            sel <- rvg != "-" & rvg != "N"
            if(any(sel)){
                varRegs <- varRegs[sel]
                rvg <- rvg[sel]
                mmTab <- t(sapply(varRegs, function(x) as.character(subseq(GdObject@sequences, x, width=1))))
                isMm <- t(rvg != "-" & mmTab != "+" & mmTab != "-" & mmTab != rvg)
                mmRelPos <- col(isMm)[which(isMm)]
                mmPos <- varRegs[mmRelPos] + rgo["from"] - 1
                mmSampInd <- row(isMm)[which(isMm)]
                mmSamp <- rownames(isMm)[mmSampInd]
                mmSeq <- mmTab[ncol(isMm) * (mmSampInd - 1) + mmRelPos]
                mmStack <- stacks(GdObject)[match(mmSamp, ranges(GdObject)$entityId)]
            }
        }
    }
    return(data.frame(position=mmPos, stack=mmStack, read=mmSamp, base=as.character(mmSeq), stringsAsFactors=TRUE))
}





## Return the default mappings between the metadata columns of an imported GRanges object and those
## of the track's GRanges object.
.defaultVarMap <- function(inputType, trackType, stream, fromUser=FALSE, justMap=FALSE){
    vm <- list(gtf=list(GeneRegionTrack=list(feature="type",
                                             gene=c("gene_id", "gene_name"),
                                             exon=c("exon_name", "exon_id"),
                                             transcript=c("transcript_name", "transcript_id"),
                                             symbol=c("gene_name", "gene_id"))),
               gff=list(AnnotationTrack=list(feature="type",
                                             group="group"),
                        GeneRegionTrack=list(feature="type",
                                             transcript="group")),
               gff1=list(AnnotationTrack=list(feature="type",
                                              group=group),
                        GeneRegionTrack=list(feature="type",
                                             transcript="group")),
               gff2=list(AnnotationTrack=list(feature="type",
                                              group=c("group", "Parent"),
                                              id=c("ID", "Name", "Alias")),
                        GeneRegionTrack=list(feature="type",
                                             gene=c("gene_id", "gene_name"),
                                             exon=c("exon_name", "exon_id"),
                                             symbol=c("gene_name", "gene_id"))),
               gff3=list(AnnotationTrack=list(feature="type",
                                              id=c("ID", "Name", "Alias"),
                                              group="Parent"),
                         GeneRegionTrack=list(feature="type",
                                              gene=c("gene_id", "gene_name"),
                                              exon=c("exon_name", "exon_id", "ID"),
                                              transcript=c("transcript_name", "transcript_id", "Parent"),
                                              symbol=c("gene_name", "gene_id", "Name", "Alias"))),
               bedgraph=list(DataTrack=list(score="score")),
               wig=list(DataTrack=list(score="score")),
               bed=list(AnnotationTrack=list(feature="itemRgb", id="name")),
               bigwig=list(DataTrack=list(score="score",
                                          .stream=TRUE)),
               bw=list(DataTrack=list(score="score",
                                      .stream=TRUE)),
               bam=list(DataTrack=list(score="score",
                                      .stream=TRUE),
                        AnnotationTrack=list(id="id",
                                             group="group",
                                             .stream=TRUE),
                        AlignmentsTrack=list(id="id",
                                             cigar="cigar",
                                             mapq="mapq",
                                             flag="flag",
                                             isize="isize",
                                             groupid="groupid",
                                             status="status",
                                             md="md",
                                             seq="seq",
                                             .stream=TRUE)))
    if(justMap)
        return(vm[[inputType]][[trackType]])
    if(fromUser){
        vm[[inputType]] <- setNames(list(list(".stream"=stream)), trackType)
    }else{
        if(is.null(vm[[inputType]]) || is.null(vm[[inputType]][[trackType]])){
            warning(sprintf(paste("There are no default mappings from %s files to %s. Please provide a manual mapping",
                                  "in the track constructor if you haven't already done so."),
                            inputType, trackType))
            vm[[inputType]] <- setNames(list(list(".stream"=stream)), trackType)
        }
    }
    return(vm[[inputType]][[trackType]])
}


## Helper function to go through the metadata columns of a DataFrame and match their colnames to a mapping if they
## are available
.resolveColMapping <- function(data, args, defMap){
    colnames(mcols(data)) <- paste(colnames(mcols(data)), "orig", sep="__.__")
    for(i in names(defMap)){
        if(is.character(args[[i]]) && length(args[[i]])==1 && paste(args[[i]], "orig", sep="__.__") %in% colnames(mcols(data))){
            defMap[[i]] <- args[[i]]
            args[[i]] <- NULL
        }
        mt <- match(paste(defMap[[i]], "orig", sep="__.__"), colnames(mcols(data)))
        mt <- mt[!is.na(mt)][1]
        if(!is.na(mt))
            mcols(data)[[i]] <- mcols(data)[,mt]
    }
    mcols(data) <- mcols(data)[, !grepl("__.__", colnames(mcols(data))), drop=FALSE]
    return(list(data=data, args=args, defMap=defMap))
}


## For an AnnotationTrack or a GeneRegionTrack, compute the actual ranges for a complete range element group
## (i.e., a whole transcript or track group) and also add the necessary space for the text label if needed.
## We add this information to the internal display parameters '.__groupRanges', '.__groupLabels' and
## '.__groupLabelWidths'. This function has to be called before stacks are being computed because 'setStacks'
## will use the values in '.__groupRanges'.
## Note that the computed ranges are not quite right because we are only crudely guessing the size of the
## title panels at this stage.
.computeGroupRange <- function(GdObject, hasAxis=FALSE, hasTitle=.dpOrDefault(GdObject, "showTitle", TRUE), title.width=1){
    if(is(GdObject, "AnnotationTrack")){
        finalRanges <- IRanges()
        GdObjectOrig <- GdObject
        GdObject <- GdObject[seqnames(GdObject) == chromosome(GdObject)]
        pr <- .dpOrDefault(GdObject, ".__plottingRange", data.frame(from=min(start(GdObject)), to=max(end(GdObject))))
        if(is.null(title.width))
            title.width <- 1
        if(length(GdObject) > 0){
            gp <- group(GdObject)
            needsGrp <- any(duplicated(gp))
            finalRanges <- if(needsGrp){
                groups <- split(range(GdObject), gp)
                unlist(range(groups))
            }else{
                range(GdObject)
            }
            if(.dpOrDefault(GdObject, ".__hasAnno", FALSE)){
                ## The label justification
                just <- .dpOrDefault(GdObject, "just.group", "left")
                rev <- .dpOrDefault(GdObject, "reverseStrand", FALSE)
                ## A crude guestimate of the space needed for a title
                twidth <- if(hasTitle){
                    fact <- title.width + (hasAxis * 2)
                    .getStringDims(GdObject, "g_T", unit="npc", subtype="title")$height * fact
                }else 0
                tfact <- ifelse(twidth > 1, 1, 1 / (1 - twidth))
                ## The labels and spacers are plotted in a temporary viewport to figure out their size
                labels <- if(needsGrp)
                    sapply(split(identifier(GdObject), gp), function(x)
                           paste(sort(unique(x)), collapse="/")) else setNames(identifier(GdObject), gp)
                xscale <- c(max(pr["from"], min(start(finalRanges))), min(pr["to"], max(end(finalRanges))))
                if(diff(xscale) == 0)
                    xscale[2] <- xscale[2] + 1
                pushViewport(dataViewport(xscale=xscale, extension=0, yscale=c(0,1), gp=.fontGp(GdObject, "group")))
                labelWidths <- setNames(as.numeric(convertWidth(stringWidth(labels),"native")) * tfact * 1.3, names(labels))
                spaceBefore <- as.numeric(convertWidth(unit(3, "points"),"native")) * tfact
                spaceAfter <- as.numeric(convertWidth(unit(7, "points"),"native")) * tfact
                popViewport(1)
                switch(as.character(just),
                       "left"={
                           if(!rev){
                               start(finalRanges) <- start(finalRanges) - (spaceBefore + labelWidths + spaceAfter)
                           }else{
                               end(finalRanges) <- end(finalRanges) + spaceAfter + labelWidths + spaceBefore
                           }
                           sb <- spaceBefore
                           sa <- spaceAfter
                       },
                       "right"={
                           if(!rev){
                               end(finalRanges) <-  end(finalRanges) + spaceAfter + labelWidths + spaceBefore
                           }else{
                               start(finalRanges) <- start(finalRanges) - (spaceBefore + labelWidths + spaceAfter)
                           }
                           sb <- spaceBefore
                           sa <- spaceAfter
                       },
                       "above"={
                           featureWidths <- end(finalRanges) - start(finalRanges)
                           additionalLabelSpace <- ceiling((labelWidths - featureWidths) / 2)
                           additionalLabelSpace[additionalLabelSpace < 0] <- 0
                           end(finalRanges) <- end(finalRanges) + additionalLabelSpace
                           start(finalRanges) <- start(finalRanges) - additionalLabelSpace
                           sa <- sb <- 0
                       },
                       "below"={
                           featureWidths <- end(finalRanges) - start(finalRanges)
                           additionalLabelSpace <- ceiling((labelWidths - featureWidths) / 2)
                           additionalLabelSpace[additionalLabelSpace < 0] <- 0
                           end(finalRanges) <- end(finalRanges) + additionalLabelSpace
                           start(finalRanges) <- start(finalRanges) - additionalLabelSpace
                           sa <- sb <- 0
                       },
                       stop(sprintf("Unknown label justification '%s'", just)))
                displayPars(GdObjectOrig) <- list(".__groupLabelWidths"=data.frame(before=sb, label=labelWidths, after=sa),
                                                  ".__groupLabels"=labels)
            }
        }
        displayPars(GdObjectOrig) <- list(".__groupRanges"=finalRanges)
    }
    return(GdObjectOrig)
}

## Calculate the vectorized string dimensions and return the results in a data.frame with columns 'width' and 'height'
## The unit of the return value can be controlled, and additional parameters like font size and expansion factors can
## be passed in as additional arguments (all in ... is passed on to 'gpar'). If needed, the font defaults for a
## subtype can be extracted by providing the subtype argument.
.getStringDims <- function(GdObject, string, unit="native", subtype=NULL, ...){
    gp <- .fontGp(GdObject, subtype, ...)
    pushViewport(viewport(gp=gp, xscale=current.viewport()$xscale, yscale=current.viewport()$yscale))
    res <- data.frame(width=as.numeric(convertWidth(stringWidth(string), unit)),
               height=as.numeric(convertHeight(stringHeight(string), unit)))
    popViewport(1)
    return(res)
}


## Check whether transcripts are to be collapsed for a GeneRegionTrack
.transcriptsAreCollapsed <- function(GdObject){
    res <- FALSE
    if(is(GdObject, "GeneRegionTrack")){
       ctrans <- .dpOrDefault(GdObject, "collapseTranscripts", FALSE)
       res <- (is.logical(ctrans) && ctrans == TRUE) || ctrans %in% c("gene", "shortest", "longest", "meta")
   }
    return(res)
}

## Create list for drawing sashimi-like plots
## using summarizeJunctions on GAlignments
## plotting is done via grid.xspline (requires x, y, id, score)
.create.summarizedJunctions.for.sashimi.junctions <- function(range) {
    range <- sort(range)
    range <- range[!duplicated(range$entityId)]
    ga <- GAlignments(seqnames=seqnames(range), pos=start(range), cigar=range$cigar,
                      strand=if(is.null(range$readStrand)) strand(range) else range$readStrand,
                      seqlengths=seqlengths(range))
    juns <- summarizeJunctions(ga)
    juns
}

.convert.summarizedJunctions.to.sashimi.junctions <- function(juns, score=1L, lwd.max=10, strand="*", filter=NULL, filterTolerance=0L) {
    ## filter junctions
    if (!is.null(filter)) {
        ## if filterTolerance is > 0 than pass it as maxgap parameter in findOverlaps
        ## make sure it is positive value
        if (filterTolerance < 0) {
            filterTolerance <- abs(filterTolerance)
            warning(sprintf("\"sashimiFilterTolerance\" can't be negative, taking absolute value of it: %d",
                    filterTolerance))
        }
        ovs <- findOverlaps(juns, filter, type="start", maxgap=filterTolerance)
        ove <- findOverlaps(juns, filter, type="end", maxgap=filterTolerance)
        ## combine both Hits objects, select junctions present in both
        ovv <- rbind(as.matrix(ovs), as.matrix(ove))
        ovv <- ovv[duplicated(ovv),,drop=FALSE]
        ## create row/col index for selecting the correct strand
        ws <- strand(filter[ovv[,"subjectHits"]])
        levels(ws) <- c("plus_score", "minus_score", "score")
        ws <- cbind(row=ovv[,"queryHits"], col=as.character(ws))
        ## rol/col subseting will only works on matrix
        M <- as.matrix(values(juns))
        rownames(M) <- 1:nrow(M)
        ##
        filter$score <- 0L
        filter$score[sort(unique(ovv[,"subjectHits"]))] <- tapply(M[ws], ovv[,"subjectHits"], sum)
        juns <- filter
    } else {
        ## if no filter ranges were defined
        ## select strand (default both)
        juns$score <- if (strand=="+") juns$plus_score else if (strand=="-") juns$minus_score else juns$score
    }
    ## filter based on evidence (default no filtering, 1 read)
    juns <- juns[juns$score >= score]
    if (length(juns)) {
        ## count how many overlaps to determine the y
        ov <- findOverlaps(juns, reduce(juns, min.gapwidth=0L))
        ov <- split(queryHits(ov), subjectHits(ov))
        juns$y <- as.integer(unlist(sapply(ov, order)))
        ## scale the score to lwd.max
        juns$scaled <- (lwd.max-1)/pmax((max(juns$score)-min(c(1, juns$score))), 1)*(juns$score-max(juns$score))+lwd.max
        ## create list
        juns <- list(x=as.numeric(rbind(start(juns),
                         mid(ranges(juns)), end(juns))),
                     y=as.numeric(rbind(0, juns$y, 0)),
                     id=rep(seq_len(length(juns)), each=3),
                     score=as.numeric(juns$score),
                     scaled=juns$scaled)
    } else {
        juns <- list(x=numeric(),
                     y=numeric(),
                     id=integer(),
                     score=numeric(),
                     scaled=numeric())
    }
    return(juns)
}

## Gviz functions

test_that("checking the class and structure works", {
    expect_null(.checkClass(data.frame(), "data.frame"))
    expect_error(.checkClass(data.frame(), "matrix"), "of class")
    expect_error(.checkClass(class = "matrix"), "is missing with no default")
    expect_error(.checkClass(data.frame(), "data.frame", length = 2), "of length")
})

test_that("conversion of chromosome names works", {
    expect_identical(.chrName(character()), character())
    options("ucscChromosomeNames" = TRUE)
    expect_identical(.chrName(c("1", "chr1", "M", "MT", "X")), c("chr1", "chr1", "chrM", "chrM", "chrX"))
    expect_identical(.chrName("foo", force = TRUE), "chrfoo")
    expect_error(.chrName("foo"), "Please consider setting options\\(ucscChromosomeNames=FALSE\\)")
    options("ucscChromosomeNames" = FALSE)
    expect_identical(.chrName(c("1", "chr1", "M", "X")), c("1", "chr1", "M", "X"))
})


## not sure how to set up the test for the device resolution
## with opening PDF device?
test_that("finding location and pixel-size of current viewport works", {
    l <- list(
        location = c(x1 = 0, y1 = 0, x2 = 504, y2 = 504),
        size = c(width = 504, height = 504),
        ilocation = c(x1 = 0, y1 = 0, x2 = 7, y2 = 7),
        isize = c(width = 7, height = 7)
    )
    ## if (file.exists("Rplots.pdf")) unlink("Rplots.pdf")
    ## pdf(file="Rplots.pdf", width=7, height=5, pointsize=12)
    ## expect_identical(vpLocation(), l)
    expect_equal(vpLocation(), l)
    ## dev.off()
    ## unlink("Rplots.pdf")
})


test_that("estimating of device resolution works", {
    ## if (file.exists("Rplots.pdf")) unlink("Rplots.pdf")
    ## pdf(file="Rplots.pdf", width=7, height=5, pointsize=12)
    ## expect_identical(devRes(), c(xres=72, yres=72))
    expect_equal(devRes(), c(xres = 72, yres = 72))
    ## dev.off()
    ## unlink("Rplots.pdf")

    ## if (file.exists("Rplots.pdf")) unlink("Rplots.pdf")
    ## pdf(file="Rplots.pdf", width=7, height=5, pointsize=12)
    pushViewport(viewport())
    ## expect_identical(devRes(), c(xres=72, yres=72))
    expect_equal(devRes(), c(xres = 72, yres = 72))
    popViewport()
    ## dev.off()
    ## unlink("Rplots.pdf")
})

test_that("estimating of coordinates for an HTML image map works", {
    m <- matrix(rep(0:1, each = 4), ncol = 4, byrow = TRUE)
    mm <- matrix(rep(as.numeric(c(0, 1, 504, 503)), 2),
        ncol = 4,
        dimnames = list(NULL, c("x1", "y1", "x2", "y2"))
    )
    mm <- as.data.frame(mm)
    ## if (file.exists("Rplots.pdf")) unlink("Rplots.pdf")
    ## pdf(file="Rplots.pdf", width=7, height=5, pointsize=12)
    ## expect_identical(.getImageMap(m), mm)
    expect_equal(.getImageMap(m), mm)
    ## dev.off()
    ## unlink("Rplots.pdf")
})

test_that("deep copying of displaypars works", {
    expect_identical(.needsStacking(annoTrack), TRUE)
    expect_identical(.deepCopyPars(annoTrack), annoTrack)
    expect_identical(displayPars(.deepCopyPars(annoTrack)), displayPars(annoTrack))
})

test_that("check of stacking works", {
    expect_identical(.needsStacking(annoTrack), TRUE)
})

test_that("checking of the strand works", {
    displayPars(dataTrack)$reverseStrand <- TRUE
    displayPars(overTrack@trackList[[1]])$reverseStrand <- TRUE
    expect_identical(.whichStrand(annoTrack), "forward")
    expect_identical(
        .whichStrand(list(dataTrack, annoTrack, highTrack, overTrack)),
        c("reverse", "forward", "forward", "reverse", "forward")
    )
})

test_that("import of sequence from FASTA file works", {
    expect_identical(.import.fasta(fastafile, GRanges("chr3", IRanges(1, 4))), DNAStringSet())
    expect_identical(.import.fasta(fastafile, GRanges("chr1", IRanges(1, 4))), DNAStringSet(c("chr1" = "CTAN")))
})

test_that("import of sequence from 2bit file works", {
    twobitfile <- system.file("extdata/test.2bit", package = "Gviz")
    expect_identical(.import.2bit(twobitfile, GRanges("chr3", IRanges(1, 4))), DNAStringSet())
    expect_identical(.import.2bit(twobitfile, GRanges("chr1", IRanges(1, 4))), DNAStringSet(c("chr1" = "CTAN")))
})

test_that("import of alignments from BAM file works", {
    empty <- GRanges()
    mcols(empty) <- DataFrame(
        id = character(), cigar = character(), mapq = integer(), flag = integer(),
        md = character(), seq = DNAStringSet(), isize = integer(), groupid = integer(),
        status = factor(levels = c("mated", "ambiguous", "unmated"))
    )

    unlink(paste(bamfile, "bai", sep = "."))
    expect_error(.import.bam.alignments(bamfile, GRanges("chr1", IRanges(189891401, 189894000))), "Unable to find index for BAM file")
    bamfile <- Rsamtools::asBam(samfile, indexDestination = FALSE, overwrite = TRUE)
    Rsamtools::indexBam(bamfile)

    expect_identical(.import.bam.alignments(bamfile, GRanges("chr1", IRanges(189891401, 189894000))), bamgr)
    expect_identical(.import.bam.alignments(bamfile, GRanges("chr2", IRanges(1, 2))), empty)
    seqlevels(empty) <- "chr1"
    expect_identical(.import.bam.alignments(bamfile, GRanges("chr1", IRanges(1, 2))), empty)
})

test_that("estimate of required verticalSpace works", {
    expect_identical(.verticalSpace(dataTrack, 10), 5)
    expect_identical(.verticalSpace(axisTrack, 10), 1)
    expect_identical(.verticalSpace(ideoTrack, 10), 1)
    expect_identical(.verticalSpace(seqTrack.dna, 10), 1)
    expect_identical(.verticalSpace(setStacks(annoTrack), 10), 1) ## require stacks
    expect_identical(.verticalSpace(alnTrack, 10), 1)
})



test_that("conversion of ranges to summarizedJunctions works", {
    ## with + strand defined in readStrand column
    range <- GRanges("chr1", IRanges(1, 10), cigar = "1M8N1M", readStrand = Rle(factor("+", levels = c("+", "-", "*"))), entityId = 1)
    juns <- GRanges("chr1", IRanges(start = 2, end = 9), score = 1L, plus_score = 1L, minus_score = 0L)
    expect_identical(.create.summarizedJunctions.for.sashimi.junctions(range), juns)
    ## without strand definition
    range <- GRanges("chr1", IRanges(1, 10), cigar = "1M8N1M", entityId = 1)
    juns <- GRanges("chr1", IRanges(start = 2, end = 9), score = 1L, plus_score = 0L, minus_score = 0L)
    expect_identical(.create.summarizedJunctions.for.sashimi.junctions(range), juns)
})


test_that("conversion of junction to list for plotting works", {
    juns <- GRanges("chr1", IRanges(start = 2, end = 9), score = 1L, plus_score = 0L, minus_score = 0L)
    out <- list(
        x = c(2, 5, 9),
        y = c(0, 1, 0),
        id = c(1L, 1L, 1L),
        score = 1, scaled = 10
    )
    filt <- GRanges("chr1", IRanges(start = 2, end = 9))
    out2 <- list(
        x = c(3, 5, 8),
        y = c(0, 1, 0),
        id = c(1L, 1L, 1L),
        score = 1, scaled = 10
    )
    filt2 <- GRanges("chr1", IRanges(start = 3, end = 8))
    expect_identical(.convert.summarizedJunctions.to.sashimi.junctions(juns), out)
    ## if minimum score filter works
    expect_identical(
        .convert.summarizedJunctions.to.sashimi.junctions(juns, score = 2L),
        list(x = numeric(), y = numeric(), id = integer(), score = numeric(), scaled = numeric())
    )
    ## filter match
    expect_identical(.convert.summarizedJunctions.to.sashimi.junctions(juns, filter = filt), out)
    ## filter match (none)
    expect_identical(
        .convert.summarizedJunctions.to.sashimi.junctions(juns, filter = filt2),
        list(x = numeric(), y = numeric(), id = integer(), score = numeric(), scaled = numeric())
    )
    ## filter match (with filterTolerance)
    expect_identical(.convert.summarizedJunctions.to.sashimi.junctions(juns, filter = filt2, filterTolerance = 1), out2)
    ## negative filterTolerance
    expect_warning(
        .convert.summarizedJunctions.to.sashimi.junctions(juns, filter = filt, filterTolerance = -1),
        "can't be negative, taking absolute value of it"
    )
    ## transformation
    expect_identical(.convert.summarizedJunctions.to.sashimi.junctions(juns, trans = list(function(x) {
        x
    })), out)
    expect_error(.convert.summarizedJunctions.to.sashimi.junctions(juns, trans = 1), "must be a function with a single argument")
    expect_error(.convert.summarizedJunctions.to.sashimi.junctions(juns, trans = function(x) {
        x[-1]
    }), "invalid output")
})


## test_that("only first warning works", {
##   warn_once({
##     for (i in 1:10) { warn_if_first("foo", "oh, no! foo!") }
##     for (i in 1:10) { warn_if_first("bar", "oh, no! bar!") }
##     sapply(1:10, function(x) {
##       warn_if_first("foo", "oh, no! foo again! (not really)")
##       warn_if_first("foobar", "foobar, too!")
##     })
##     "DONE!"
##   })
## })

#' Data sets
#'
#' Some sample data sets used for the illustrative examples and the vignette.
#'
#' @name datasets
#'
#' @aliases bmTrack cyp2b10 idTrack biomTrack biomTrack2 cpgIslands axTrack
#' conservation ensGenes denseAnnTrack geneModels iTrack itrack idxTrack
#' ideoTrack twoGroups from gcContent idTrack knownGenes refGenes snpLocations
#' to ctrack geneDetails dtHoriz bmt
#'
#' @docType data
#' @keywords datasets
NULL

##library(methods)
library(grid)
library(IRanges)
library(rtracklayer)
library(GenomicRanges)
library(lattice)
library(biomaRt)
library(RColorBrewer)
library(Biobase)
library(AnnotationDbi)
library(biovizBase)
library(Biostrings)
library(Rsamtools)
library(GenomicAlignments)
library(BSgenome)

## options(error=recover)
removeAndSource <- function(file, remove=FALSE){
    if(remove){
        exp <- parse(file)
        sapply(exp, function(x){
            x <- as.character(x)
            if(x[[1]]=="setMethod")
                suppressWarnings(try(removeMethod(x[[2]], ifelse(length(grep("\\(", x[3]))>0, eval(parse(text=x[3])), x[3]))))
            if(x[[1]]=="setReplaceMethod")
                suppressWarnings(try(removeMethod(paste(x[[2]], "<-", sep=""), ifelse(length(grep("\\(", x[3]))>0, eval(parse(text=x[3])), x[3]))))
            if(x[[1]]=="setAs")
                suppressWarnings(try(removeMethod("coerce", x[2:3])))
        })
    }
    source(file)
}

path <- "~/Rpacks/Gviz/R"
files <- c("Gviz.R", "AllGenerics.R", "AllClasses.R", "Gviz-methods.R")
sapply(file.path(path, files), removeAndSource)


## dtTrack <- DataTrack(start=seq(1,1000, len=100), width=10, data=matrix(runif(400), nrow=4), chromosome=1, genome="mm9", name="random data")

#' @include GeneRegionTrack-class.R
NULL

## BiomartGeneRegionTrack Class ----------------------------------------------

#' BiomartGeneRegionTrack class and methods
#'
#'
#' A class to hold gene model data for a genomic region fetched dynamically
#' from EBI's Biomart Ensembl data source.
#'
#'
#' A track containing all gene models in a particular region as fetched from
#' EBI's Biomart service. Usually the user does not have to take care of the
#' Biomart connection, which will be established automatically based on the
#' provided genome and chromosome information. However, for full flexibility a
#' valid \code{\linkS4class{Mart}} object may be passed on to the constructor.
#' Please note that this assumes a connection to one of the Ensembl gene data
#' sources, mapping the available query data back to the internal object slots.
#'
#' @template BiomartGeneRegionTrack-class_param
#'
#' @name BiomartGeneRegionTrack-class
#'
#' @return
#'
#' The return value of the constructor function is a new object of class
#' \code{BiomartGeneRegionTrack}.
#' @section Objects from the class:
#'
#' Objects can be created using the constructor function
#' \code{BiomartGeneRegionTrack}.
#'
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @references EBI Biomart webservice at \url{http://www.biomart.org}.
#'
#' @examples
#' \dontshow{
#' ## Load some sample data
#' data(bmTrack)
#' }
#'
#' ## Construct the object
#' \dontrun{
#' bmTrack <- BiomartGeneRegionTrack(
#'     start = 26682683, end = 26711643,
#'     chromosome = 7, genome = "mm9"
#' )
#' }
#'
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(bmTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(bmTrack)
#'
#' ## Track names
#' names(bmTrack)
#' names(bmTrack) <- "foo"
#' plotTracks(bmTrack)
#'
#' ## Subsetting and splitting
#' subTrack <- subset(bmTrack, from = 26700000, to = 26705000)
#' length(subTrack)
#' subTrack <- bmTrack[transcript(bmTrack) == "ENSMUST00000144140"]
#' split(bmTrack, transcript(bmTrack))
#'
#' ## Accessors
#' start(bmTrack)
#' end(bmTrack)
#' width(bmTrack)
#' position(bmTrack)
#' width(subTrack) <- width(subTrack) + 100
#'
#' strand(bmTrack)
#' strand(subTrack) <- "-"
#'
#' chromosome(bmTrack)
#' chromosome(subTrack) <- "chrX"
#'
#' genome(bmTrack)
#' genome(subTrack) <- "hg19"
#'
#' range(bmTrack)
#' ranges(bmTrack)
#'
#' ## Annotation
#' identifier(bmTrack)
#' identifier(bmTrack, "lowest")
#' identifier(subTrack) <- "bar"
#'
#' feature(bmTrack)
#' feature(subTrack) <- "foo"
#'
#' exon(bmTrack)
#' exon(subTrack) <- letters[1:2]
#'
#' gene(bmTrack)
#' gene(subTrack) <- "bar"
#'
#' symbol(bmTrack)
#' symbol(subTrack) <- "foo"
#'
#' transcript(bmTrack)
#' transcript(subTrack) <- c("foo", "bar")
#' chromosome(subTrack) <- "chr7"
#' plotTracks(subTrack)
#'
#' values(bmTrack)
#'
#' ## Grouping
#' group(bmTrack)
#' group(subTrack) <- "Group 1"
#' transcript(subTrack)
#' plotTracks(subTrack)
#'
#' ## Stacking
#' stacking(bmTrack)
#' stacking(bmTrack) <- "dense"
#' plotTracks(bmTrack)
#'
#' ## coercion
#' as(bmTrack, "data.frame")
#' as(bmTrack, "UCSCData")
#'
#' ## HTML image map
#' coords(bmTrack)
#' tags(bmTrack)
#' bmTrack <- plotTracks(bmTrack)$foo
#' coords(bmTrack)
#' tags(bmTrack)
#' @importFrom biomaRt getBM useEnsembl useMart listDatasets listAttributes listFilters
#'
#' @exportClass BiomartGeneRegionTrack
setClass("BiomartGeneRegionTrack",
    contains = "GeneRegionTrack",
    representation = representation(biomart = "MartOrNULL", filter = "list"),
    prototype = prototype(
        biomart = NULL,
        filter = list(),
        columns = c("feature", "transcript", "symbol", "gene", "rank"),
        name = "BiomartGeneRegionTrack",
        dp = DisplayPars(
            C_segment = "burlywood4",
            D_segment = "lightblue",
            J_segment = "dodgerblue2",
            Mt_rRNA = "yellow",
            Mt_tRNA = "darkgoldenrod",
            Mt_tRNA_pseudogene = "darkgoldenrod1",
            V_segment = "aquamarine",
            miRNA = "cornflowerblue",
            miRNA_pseudogene = "cornsilk",
            misc_RNA = "cornsilk3",
            misc_RNA_pseudogene = "cornsilk4",
            protein_coding = "#FFD58A",
            pseudogene = "brown1",
            rRNA = "darkolivegreen1",
            rRNA_pseudogene = "darkolivegreen",
            retrotransposed = "blueviolet",
            scRNA = "gold4",
            scRNA_pseudogene = "darkorange2",
            snRNA = "coral",
            snRNA_pseudogene = "coral3",
            snoRNA = "cyan",
            snoRNA_pseudogene = "cyan2",
            tRNA_pseudogene = "antiquewhite3",
            utr3 = "#FFD58A",
            utr5 = "#FFD58A",
            verbose = FALSE
        )
    )
)

## Helper to return the default biomart to feature mapping
.getBMFeatureMap <- function() {
    return(list(
        gene_id = "ensembl_gene_id", transcript_id = "ensembl_transcript_id", exon_id = "ensembl_exon_id",
        start = "exon_chrom_start", end = "exon_chrom_end", rank = "rank", strand = "strand",
        symbol = c("external_gene_name", "external_gene_id"), feature = "gene_biotype", chromosome = "chromosome_name",
        u5s = "5_utr_start", u5e = "5_utr_end", u3s = "3_utr_start", u3e = "3_utr_end", cdsl = c("cds_length", "cds_start"),
        phase = "phase"
    ))
}

## Helper to do the actual fetching of data from Biomart

#' @importFrom utils modifyList
.fetchBMData <- function(object, chromosome_name = NULL, staged = FALSE) {
    if (!is.null(chromosome_name)) {
        chromosome_name <- gsub("^chr", "", chromosome_name)
    }
    ## The map between Biomart DB fields and annotation features. The individual values can be vectors for cases where there is
    ## ambiguity between different marts. This will be dynamically evaluated against available filters.
    origFeatureMap <- .getBMFeatureMap()
    featureMap <- modifyList(origFeatureMap, as.list(.dpOrDefault(object, ".__featureMap", list())))
    needed <- c(
        "gene_id", "transcript_id", "exon_id", "start", "end", "rank", "strand", "symbol", "feature",
        "chromosome", "u5s", "u5e", "u3s", "u3e", "cdsl", "phase"
    )
    if (!all(needed %in% names(featureMap))) {
        stop("'featureMap' needs to include items '", paste(setdiff(needed, names(featureMap)), collapse = ", "), "'")
    }
    avail <- listAttributes(object@biomart)[, 1]
    ambig <- names(featureMap)[listLen(featureMap) > 1]
    for (i in ambig) {
        mt <- match(featureMap[[i]], avail)
        if (!all(is.na(mt))) {
            featureMap[[i]] <- featureMap[[i]][min(which(!is.na(mt)))]
        } else {
            featureMap[[i]] <- NA
        }
    }
    featureMap <- unlist(featureMap)
    ## Deal with the filters
    filterValues <- as.list(object@filter)
    start <- object@start
    end <- object@end
    for (i in c("start", "end", "chromosome_name")) {
        if (!is.null(get(i)) && length(get(i)) > 0) {
            filterValues[[i]] <- get(i)
        }
    }
    ens <- getBM(as.vector(featureMap),
        filters = names(filterValues),
        values = filterValues, bmHeader = FALSE,
        mart = object@biomart, uniqueRows = TRUE
    )
    colnames(ens) <- names(featureMap)
    if (staged && nrow(ens) > 0) {
        filterValues <- list(start = min(ens$start), end = max(ens$end), chromosome_name = ens[1, "chromosome"])
        ens <- getBM(as.vector(featureMap),
            filters = names(filterValues),
            values = filterValues, bmHeader = FALSE,
            mart = object@biomart, uniqueRows = TRUE
        )
        colnames(ens) <- names(featureMap)
    }
    ## Only those transcripts that have a CDS length will be considered protein_coding
    ens$feature <- ifelse(is.na(ens$cdsl) & ens$feature == "protein_coding", "non_coding", ens$feature)
    ## We may have to split exons if they contain UTRs
    hasUtr <- !is.na(ens$u5s) | !is.na(ens$u3s)
    ensUtr <- ens[hasUtr, , drop = FALSE]
    ensUtr$ffeature <- ifelse(is.na(ensUtr$u5s), "utr3", "utr5")
    ensUtr$us <- ifelse(ensUtr$ffeature == "utr3", ensUtr$u3s, ensUtr$u5s)
    ensUtr$ue <- ifelse(ensUtr$ffeature == "utr3", ensUtr$u3e, ensUtr$u5e)
    ensUtr$u5e <- ensUtr$u5s <- ensUtr$u3e <- ensUtr$u3s <- NULL
    allUtr <- ensUtr$us == ensUtr$start & ensUtr$ue == ensUtr$end
    utrFinal <- ensUtr[allUtr, , drop = FALSE]
    ensUtr <- ensUtr[!allUtr, , drop = FALSE]
    ensUtrS <- split(ensUtr, ifelse(ensUtr$start == ensUtr$us, "left", "right"))
    utrFinal <- rbind(utrFinal, do.call(rbind, lapply(names(ensUtrS), function(i) {
        y <- ensUtrS[[i]]
        if (nrow(y) == 0) {
            return(NULL)
        }
        yy <- y[rep(seq_len(nrow(y)), each = 2), ]
        sel <- seq(1, nrow(yy), by = 2)
        yy[sel, "end"] <- if (i == "left") yy[sel, "ue"] else yy[sel, "us"] - 1
        yy[sel, "ffeature"] <- yy[sel, ifelse(i == "left", "ffeature", "feature")]
        yy[sel, "phase"] <- if (i == "left") -1 else 0
        sel <- seq(2, nrow(yy), by = 2)
        yy[sel, "start"] <- if (i == "left") yy[sel, "ue"] + 1 else yy[sel, "us"]
        yy[sel, "ffeature"] <- yy[sel, ifelse(i == "left", "feature", "ffeature")]
        yy[sel, "phase"] <- if (i == "left") yy[sel, "phase"] else -1
        yy
    })))
    utrFinal$feature <- utrFinal$ffeature
    keep <- c(
        "gene_id", "transcript_id", "exon_id", "start",
        "end", "rank", "strand", "symbol", "feature",
        "chromosome", "phase"
    )
    ens <- rbind(ens[!hasUtr, keep, drop = FALSE], utrFinal[, keep])
    ens$chromosome <- .chrName(ens$chromosome, force = TRUE)
    range <- GRanges(
        seqnames = ens$chromosome,
        ranges = IRanges(start = ens$start, end = ens$end),
        strand = ens$strand, feature = as.character(ens$feature),
        gene = as.character(ens$gene_id), exon = as.character(ens$exon_id),
        transcript = as.character(ens$transcript_id), symbol = as.character(ens$symbol),
        rank = as.numeric(ens$rank), phase = as.integer(ens$phase)
    )
    suppressWarnings(genome(range) <- unname(genome(object)[1]))
    range <- sort(range)
    return(range)
}


## Create an MD5 hash for a BiomartGeneRegion track taking into account the Biomart details for caching

#' @importFrom digest digest
.bmGuid <- function(bmtrack) {
    digest(list(
        genome = genome(bmtrack), host = bmtrack@biomart@host, mart = bmtrack@biomart@biomart, schema = bmtrack@biomart@vschema,
        dataset = bmtrack@biomart@dataset, filters = bmtrack@filter
    ))
}

## Initialize ----------------------------------------------------------------

# #' @noRd
# #' @keywords internal
#' @describeIn BiomartGeneRegionTrack-class Initialize.
#' @export
setMethod("initialize", "BiomartGeneRegionTrack", function(.Object, start = NULL, end = NULL, biomart, filter = list(), range, genome = NULL, chromosome = NULL, strand = NULL,
                                                           featureMap = NULL, symbol = NULL, gene = NULL, transcript = NULL, entrez = NULL, ...) {
    if ((missing(range) || is.null(range)) && is.null(genome) && is.null(chromosome)) {
        return(.Object)
    }
    ## the display parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "BiomartGeneRegionTrack")
    verb <- list(...)$verbose
    displayPars(.Object) <- list(!is.null(verb) && verb == TRUE)
    ## preparing filters
    strand <- .strandName(strand, extended = TRUE)
    if (strand %in% 0:1) {
        filter$strand <- c(1, -1)[strand + 1]
    }
    filterOrig <- filter
    idFilters <- list(
        symbol = c("external_gene_name", "external_gene_id", "hgnc_symbol", "wikigene_name", "dbass3_name"),
        gene = "ensembl_gene_id",
        transcript = "ensembl_transcript_id",
        entrez = "entrezgene"
    )
    staged <- FALSE
    for (i in names(idFilters)) {
        if (!is.null(get(i))) {
            mt <- match(idFilters[[i]], listFilters(biomart)[, 1])
            sfilt <- listFilters(biomart)[mt[!is.na(mt)], 1]
            if (length(sfilt) > 0) {
                staged <- TRUE
                filter[[sfilt[1]]] <- get(i)
                if (!is.null(filter$strand)) {
                    warning(sprintf("Cannot combine %s filter with a strand filter. Strand filtering is ignored.", i))
                    filter$strand <- NULL
                }
                if (!is.null(start) || !is.null(end) || !is.null(chromosome)) {
                    warning(sprintf("Cannot combine %s filter with a range restriction. Ignoring start and end coordinates.", i))
                    start <- end <- chromosome <- NULL
                }
            } else {
                stop(sprintf("Unable to automatically map the %s filter. Manually provide adequate filter list.", i))
            }
        }
    }
    ## We can't have only one in start or end
    if (!is.null(end) && is.null(start)) {
        start <- 1
    } else if (!is.null(start) && is.null(end)) {
        end <- 10e8
    }
    ## filling slots
    .Object@filter <- filter
    .Object@biomart <- biomart
    ## Extending start and end positions to capture genes on the edges. We also need both coordinates set, or none.
    extend <- if (is.null(start) || is.null(end)) 10000 else max(10000, abs(diff(c(end, start))))
    .Object@start <- if (!is.null(start)) max(1, start - extend) else NULL
    .Object@end <- if (!is.null(end)) end + extend else NULL
    displayPars(.Object) <- list(".__featureMap" = featureMap)
    genome(.Object) <- ifelse(is.null(genome), "ANY", genome)
    ## fetching data from Biomart
    range <- if (!is.null(.Object@biomart) && (!is.null(.Object@start) || !is.null(.Object@end) || length(.Object@filter) != 0)) {
        .cacheMartData(.Object, chromosome, staged)
    } else {
        tmp <- GRanges()
        values(tmp) <- DataFrame(feature = "a", gene = "a", exon = "a", transcript = "a", symbol = "a", rank = 0, phase = as.integer(1))[0, ]
        suppressWarnings(genome(tmp) <- unname(genome[1]))
        displayPars(.Object) <- list(".__streamOnly" = TRUE)
        tmp
    }
    .Object@filter <- filterOrig
    if (length(range) == 0) {
        .Object <- setPar(.Object, "size", 0, interactive = FALSE)
    } else {
        chromosome <- if (is.null(chromosome)) seqlevels(range)[1] else chromosome
        rr <- range(range, ignore.strand = TRUE)
        s <- start(rr[seqnames(rr) == chromosome])
        e <- end(rr[seqnames(rr) == chromosome])
        start <- min(s, start)
        end <- max(e, end)
    }
    .Object <- callNextMethod(
        .Object = .Object, range = range, start = start, end = end,
        genome = genome, chromosome = chromosome, strand = strand, ...
    )
    ## We want to warn if searching was performed based on an identifier and now values have been returned
    if ((!is.null(symbol) || !is.null(gene) || !is.null(transcript) || !is.null(entrez)) && length(range) == 0) {
        warning("Search by identifier did not yield any values", call. = FALSE)
    }
    return(.Object)
})

## Constructor ---------------------------------------------------------------

## Constructor. The following arguments are supported:
##    o start, end: numeric vectors of the item start and end coordinates
##    o biomart: a biomaRt object used to to query for gene annotations
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o strand: character, search for gene models on the plus strand ("+" or 0), the
##       minus strand ("-" or 1) or both strands ("+-" or "-+" or 2)
##    o stacking: character controlling the stacking of overlapping items. One in 'hide',
##       'dense', 'squish', 'pack' or 'full'.
##    o filters: list of additional filters for the biomaRt query, where the item names
##          are the filter identifiers and the item values are the filter values.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters

#' @describeIn BiomartGeneRegionTrack-class Constructor function for
#' `BiomartGeneRegionTrack-class`.
#' @export
BiomartGeneRegionTrack <- function(start = NULL, end = NULL, biomart, chromosome = NULL, strand, genome = NULL,
                                   stacking = "squish", filters = list(), featureMap = NULL, name = "BiomartGeneRegionTrack",
                                   symbol = NULL, gene = NULL, entrez = NULL, transcript = NULL, ...) {
    ## Some default checking
    .missingToNull(c("genome"))
    if (missing(strand)) {
        strand <- "*"
    }
    if ((!is.null(start) || !is.null(end)) && is.null(chromosome)) {
        stop("Also need to specify a chromsome when initializing a BiomartGeneRegionTrack with start or end coordinates")
    }
    if (!is.null(chromosome)) {
        chromosome <- .chrName(chromosome)[1]
    }
    if (missing(biomart)) {
        if (is.null(genome)) {
            stop("Need either a valid Mart connection object as 'biomart' argument or a UCSC genome identifier as the 'genome' argument.")
        }
        biomart <- .genome2Dataset(genome)
    } else if (is.null(genome)) {
        genome <- biomart@dataset
    }
    new("BiomartGeneRegionTrack",
        start = start, end = end, chromosome = chromosome, strand = strand,
        biomart = biomart, name = name, genome = genome, stacking = stacking, filter = filters, featureMap = featureMap,
        symbol = symbol, gene = gene, transcript = transcript, entrez = entrez, ...
    )
}


## This filters for genes with refseq IDs only
.refseqFilter <- list("with_refseq_dna" = TRUE)

## General accessors ---------------------------------------------------------
## Annotation Accessors ------------------------------------------------------
## Stacking ------------------------------------------------------------------
## Consolidate ---------------------------------------------------------------
## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------

#' @describeIn BiomartGeneRegionTrack-class subset a `BiomartGeneRegionTrack`
#' by coordinates and sort if necessary.
#' @export
setMethod("subset", signature(x = "BiomartGeneRegionTrack"), function(x, from, to, chromosome, use.defaults = TRUE, ...) {
    granges <- unlist(range(split(ranges(x), group(x))))
    ranges <- if (use.defaults) {
        .defaultRange(x, from = from, to = to)
    } else {
        c(
            from = ifelse(is.null(from), min(start(granges)) - 1, from),
            to = ifelse(is.null(to), max(end(granges)) + 1, to)
        )
    }
    if (ranges["from"] < x@start || ranges["to"] > x@end) {
        x@start <- ranges["from"] - 10000
        x@end <- ranges["to"] + 10000
        ranges(x) <- .cacheMartData(x, .chrName(chromosome))
    }
    return(callNextMethod(x = x, from = ranges["from"], to = ranges["to"], use.defaults = FALSE, ...))
})

## Position ------------------------------------------------------------------
## DrawGrid ------------------------------------------------------------------
## DrawAxis ------------------------------------------------------------------
## DrawGD --------------------------------------------------------------------
## SetAs ---------------------------------------------------------------------
## Show ----------------------------------------------------------------------

## package constants ---------------------------------------------------------
## A bunch of package constants

.DEFAULT_FILL_COL <- "lightgray"
.DEFAULT_OVERPLOT_COL <- "red"
.DEFAULT_LINE_COL <- "black"
.DEFAULT_SHADED_COL <- "#808080"
.DEFAULT_BRIGHT_SHADED_COL <- "#E0E0E0"
.DEFAULT_SYMBOL_COL <- "#0080FF"
.DEFAULT_LINE_COL <- "darkgray"
.PLOT_TYPES <- c(
    "p", "l", "b", "a", "s", "g", "r", "S",
    "smooth", "polygon", "horizon", "histogram",
    "mountain", "h", "boxplot", "gradient",
    "heatmap", "confint"
)
.ALIGNMENT_TYPES <- c("coverage", "sashimi", "pileup")
.THIN_BOX_FEATURES <- c(
    "utr", "ncRNA", "utr3", "utr5", "3UTR", "5UTR",
    "miRNA", "lincRNA", "three_prime_UTR", "five_prime_UTR"
)
.DEFAULT_HORIZON_COL <- c(
    "#B41414", "#E03231", "#F7A99C",
    "#9FC8DC", "#468CC8", "#0165B3"
)
### Imports ------------------------------------------------------------------
#' @import methods
#' @import BiocGenerics
#' @import S4Vectors
#' @import IRanges
#' @import XVector
#' @import GenomeInfoDb
#' @import GenomicRanges
#' @import grid

#' @importFrom lattice current.panel.limits panel.abline panel.grid panel.lines panel.points panel.polygon panel.segments panel.xyplot panel.text trellis.par.get
#' @importFrom RColorBrewer brewer.pal

#' @importMethodsFrom AnnotationDbi colnames get ls mget tail

## #' @importFrom utils browseURL
## #' @importFrom latticeExtra panel.xyarea

### Class unions -------------------------------------------------------------

## Some useful class unions to allow for NULL values in certain slots
setClassUnion("DfOrNULL", c("data.frame", "NULL"))
#' @importClassesFrom biomaRt Mart
setClassUnion("MartOrNULL", c("Mart", "NULL"))
setClassUnion("FactorOrCharacterOrNULL", c("factor", "character", "NULL"))
setClassUnion("NumericOrNULL", c("numeric", "NULL"))
setClassUnion("ListOrEnv", c("list", "environment"))
setClassUnion("GRangesOrIRanges", c("GRanges", "IRanges"))
setClassUnion("NULLOrMissing", c("NULL", "missing"))
#' @importClassesFrom BSgenome BSgenome MaskedBSgenome
setClassUnion("BSgenomeOrNULL", c("BSgenome", "NULL"))

### Functions ----------------------------------------------------------------

## Check the class and structure of an object
.checkClass <- function(x, class, length = NULL, verbose = FALSE, mandatory = TRUE) {
    if (mandatory && missing(x)) {
        stop("Argument '", substitute(x), "' is missing with no default",
            call. = verbose
        )
    }
    msg <- paste("'", substitute(x), "' must be an object of class ",
        paste("'", class, "'", sep = "", collapse = " or "),
        sep = ""
    )
    fail <- !any(vapply(class, function(c, y) is(y, c), FUN.VALUE = logical(1L), y = x))
    if (!is.null(length) && length(x) != length) {
        if (!is.null(x)) {
            fail <- TRUE
            msg <- paste(msg, "of length", length)
        }
    }
    if (fail) {
        stop(msg, call. = verbose)
    } else {
        invisible(NULL)
    }
}

## We want to deal with chromosomes in a reasonable way. This coerces likely inputs to a unified
## chromosome name as understood by UCSC. Accepted inputs are:
##    - a single integer or a character coercible to one or integer-character combinations
##    - a character, starting with 'chr' (case insensitive)
## Arguments:
##    o x: a character string to be converted to a valid UCSC chromosome name
##    o force: a logical flag, force pre-pending of 'chr' if missing
## Value: the UCSC character name

#' @export
.chrName <- function(x, force = FALSE) {
    if (!getOption("ucscChromosomeNames") || length(x) == 0) {
        return(as.character(x))
    }
    xu <- unique(x)
    xum <- vapply(xu, function(y) {
        xx <- suppressWarnings(as.integer(y))
        if (!is.na(xx)) {
            y <- xx
        }
        if (is.numeric(y)) {
            y <- paste("chr", y, sep = "")
        }
        if (y == "MT") { # ensembl  `MT` to `chrM` in UCSC
            y <- "chrM"
        }
        if (y %in% c("M", "X", "Y", "Z", "W")) { # mitochondrial genome and sex chromosomes
            y <- paste("chr", y, sep = "")
        }
        head <- tolower(substring(y, 1, 3)) == "chr"
        if (!head && force) {
            y <- paste("chr", y, sep = "")
            head <- TRUE
        }
        if (!head) {
            stop(
                sprintf("Invalid chromosome identifier '%s'\n", y),
                "Please consider setting options(ucscChromosomeNames=FALSE) ",
                "to allow for arbitrary chromosome identifiers."
            )
        }
        substring(y, 1, 3) <- tolower(substring(y, 1, 3))
        y
    }, FUN.VALUE = character(1L))
    names(xum) <- xu
    return(as.vector(xum[as.character(x)]))
}


## Make a deep copy of the display parameter environment
.deepCopyPars <- function(GdObject) {
    oldPars <- displayPars(GdObject, hideInternal = FALSE)
    GdObject@dp <- DisplayPars()
    displayPars(GdObject) <- oldPars
    return(GdObject)
}



## One central place to check which display types result in stacking. This may change at some point for some
## unimplemented types...
## Arguments:
##    o GdObject: an object inheriting from class GdObject
## Value: a logical scalar indicating whether stacking is needed or not
.needsStacking <- function(GdObject) stacking(GdObject) %in% c("squish", "pack", "full")

## Get the coordinates for an HTML image map from the annotationTrack plot.
## Arguments:
##     o coordinates: a numeric matrix of annotation region coordinates (the bounding box if not rectangular)
## Value: valid HTML image map coordinates based on the current device dimensions

#' @importFrom graphics par
.getImageMap <- function(coordinates) {
    devSize <- devRes() * par("din")
    loc <- vpLocation()
    size <- loc$location[c(3, 4)] - loc$location[c(1, 2)]
    xscale <- current.viewport()$xscale
    yscale <- current.viewport()$yscale
    fw <- diff(xscale)
    fh <- diff(yscale)
    u2px <- function(x) ((x - xscale[1]) / fw * size[1]) + loc$location[1]
    u2py <- function(y) (devSize[2] - loc$location[2]) - ((y - yscale[1]) / fh * size[2])
    return(data.frame(
        x1 = u2px(coordinates[, 1]), y1 = u2py(coordinates[, 4]),
        x2 = u2px(coordinates[, 3]), y2 = u2py(coordinates[, 2]),
        stringsAsFactors = FALSE
    ))
}


.whichStrand <- function(trackList) {
    if (!is.list(trackList)) {
        trackList <- list(trackList)
    }
    str <- unlist(lapply(trackList, function(x) {
        if (is(x, "HighlightTrack") || is(x, "OverlayTrack")) {
            vapply(x@trackList, .dpOrDefault, par = "reverseStrand", FUN.VALUE = logical(1L))
        } else {
            .dpOrDefault(x, "reverseStrand")
        }
    }))
    return(ifelse(str, "reverse", "forward"))
}



## A function returning the amount of vertical space needed for a track
## Arguments:
##    o x: an object inheriting from class GdObject
## Value: the relative vertical space needed for the track
.verticalSpace <- function(x, totalSpace) {
    if (is(x, "DataTrack") && is.null(displayPars(x, "size"))) {
        type <- match.arg(.dpOrDefault(x, "type", "p"), .PLOT_TYPES, several.ok = TRUE)
        size <- if (length(type) == 1L) {
            if (type == "gradient") 1 else if (type == "heatmap") nrow(values(x)) else 5
        } else {
            5
        }
        return(size)
    }
    if (is(x, "GenomeAxisTrack") || is(x, "IdeogramTrack") || is(x, "SequenceTrack")) {
        nv <- displayPars(x, "neededVerticalSpace")
        size <- displayPars(x, "size")
        if (is.null(size)) {
            if (!is.null(nv)) {
                size <- nv
                attr(size, "absolute") <- TRUE
            } else {
                size <- 1
            }
        }
        return(size)
    }
    size <- .dpOrDefault(x, "size", 1)
    if (is(x, "StackedTrack")) {
        size <- max(size, min(floor(vpLocation()$size["height"] / 10), size * max(stacks(x))))
    }
    return(size)
}



## Return a particular displayPars value or a default
## Arguments:
##    o GdObject: an object inheriting from class GdObject
##    o par: the name of the displayPar, or a list of alternatives to go though before finally taking
##       the supplied default value
##    o default: a default value for the parameter if it can't be found in GdObject
## Value: the value of the displayPar
.dpOrDefault <- function(GdObject, par, default = NULL, fromPrototype = FALSE) {
    val <- getPar(x = GdObject, name = par, asIs = TRUE)
    val <- val[!vapply(val, is.null, logical(1))]
    if (length(val) == 0) {
        if (fromPrototype) {
            val <- .parMappings[[GdObject@name]][par]
            val <- val[!vapply(val, is.null, logical(1))]
            if (length(val) == 0) {
                val <- NULL
            }
        } else {
            val <- default
        }
    } else {
        val <- val[[1]]
    }
    return(val)
}



## A special version of the above for font settings. This will try to extract the respective parent
## defaults before finally taking the provided default value.
## Arguments:
##    o GdObject: an object inheriting from class GdObject
##    o par: the name of the displayPar. Can also be a vector in which case a number of alternatives
##       is tested, then the parent default for the first element until finally moving to the supplied default
##    o type: the sub-type
##    o default: a default value for the parameter if it can't be found in GdObject
## Value: the value of the displayPar
.dpOrDefaultFont <- function(GdObject, par, type = NULL, default) {
    name <- if (is.null(type)) par else sprintf("%s.%s", par, type)
    val <- getPar(GdObject, name[1])
    name <- name[-1]
    while (is.null(val) && length(name)) {
        val <- getPar(GdObject, name[1])
        name <- name[-1]
    }
    if (is.null(val)) {
        val <- .dpOrDefault(GdObject, par[1], default)
    }
    return(val)
}


## Return the font settings for a GdObject
.fontGp <- function(GdObject, subtype = NULL, ...) {
    if (is(GdObject, "OverlayTrack")) {
        GdObject <- GdObject@trackList[[1]]
    }
    gp <- list(
        fontsize = as.vector(.dpOrDefaultFont(GdObject, "fontsize", subtype, 12))[1],
        fontface = as.vector(.dpOrDefaultFont(GdObject, "fontface", subtype, 1))[1],
        fontfamily = as.character(as.vector(.dpOrDefaultFont(GdObject, "fontfamily", subtype, 1)))[1],
        col = as.vector(.dpOrDefaultFont(GdObject, "fontcolor", subtype, "black"))[1],
        lineheight = as.vector(.dpOrDefaultFont(GdObject, "lineheight", subtype, 1))[1],
        alpha = as.vector(.dpOrDefaultFont(GdObject, "alpha", subtype, 1))[1],
        cex = as.vector(.dpOrDefaultFont(GdObject, "cex", subtype, 1))[1]
    )
    gp[names(list(...))] <- list(...)
    gp <- gp[!vapply(gp, is.null, logical(1))]
    return(do.call(gpar, gp))
}


## Check a list of GdObjects whether an axis needs to be drawn for each of them.
## Arguments:
##    o object: a list of GdObjects
## Value: a logical vector of the same length as 'objects'
.needsAxis <- function(objects) {
    if (!is.list(objects)) {
        objects <- list(objects)
    }
    atrack <- vapply(objects, function(x) {
        is(x, "NumericTrack") ||
            (is(x, "AlignmentsTrack") && "coverage" %in% match.arg(.dpOrDefault(x, "type", .ALIGNMENT_TYPES), .ALIGNMENT_TYPES, several.ok = TRUE))
    }, FUN.VALUE = logical(1L))
    isOnlyHoriz <- vapply(objects, function(x) {
        res <- FALSE
        if (is(x, "DataTrack")) {
            type <- match.arg(.dpOrDefault(x, "type", "p"), .PLOT_TYPES, several.ok = TRUE)
            res <- length(setdiff(type, "horizon")) == 0 && !.dpOrDefault(x, "showSampleNames", FALSE)
        }
        res
    }, FUN.VALUE = logical(1L))
    return(atrack & vapply(objects, .dpOrDefault, par = "showAxis", default = TRUE, FUN.VALUE = logical(1L)) & !isOnlyHoriz)
}

## Check a list of GdObjects whether a title needs to be drawn for each of them.
## Arguments:
##    o object: a list of GdObjects
## Value: a logical vector of the same length as 'objects'
.needsTitle <- function(objects) {
    if (!is.list(objects)) {
        objects <- list(objects)
    }
    vapply(objects, function(x) {
        if (is(x, "HighlightTrack") || is(x, "OverlayTrack")) {
            any(vapply(x@trackList, .dpOrDefault, par = "showTitle", default = TRUE, FUN.VALUE = logical(1L)))
        } else {
            .dpOrDefault(x, "showTitle", TRUE)
        }
    }, FUN.VALUE = logical(1L))
}


## Helper function to set up the text size based on the available space
## Arguments:
##    o trackList: a list of GdObjects
##    o sizes: a matching vector of relative vertical sizes
##    o title.width: the available width for the title
## Value: a list with items:
##    o spaceNeeded: the necessary vertical space
##    o cex: the character expansion factor
##    o title.width: the updated available title width
##    o spacing: the amount of spacing between tracks
##    o nwrap: the final (wrapped) title text

#' @importFrom grDevices extendrange
.setupTextSize <- function(trackList, sizes, title.width, panelOnly = FALSE, spacing = 5) {
    curVp <- vpLocation()
    trackList <- lapply(trackList, function(x) if (is(x, "OverlayTrack")) x@trackList[[1]] else x)
    spaceNeeded <- if (is.null(sizes)) {
        lapply(trackList, .verticalSpace, curVp$size["height"])
    } else {
        if (length(sizes) != length(trackList)) {
            stop("The 'sizes' vector has to match the size of the 'trackList'.")
        }
        rev(sizes)
    }
    whichAbs <- vapply(spaceNeeded, function(x) !is.null(attr(x, "absolute")) && attr(x, "absolute"), FUN.VALUE = logical(1L))
    spaceNeeded <- unlist(spaceNeeded)
    leftVetSpace <- curVp$size["height"] - sum(spaceNeeded[whichAbs])
    spaceNeeded[!whichAbs] <- spaceNeeded[!whichAbs] / sum(spaceNeeded[!whichAbs]) * leftVetSpace
    spaceNeeded <- spaceNeeded / sum(spaceNeeded)
    if (!panelOnly) {
        ## Figure out the fontsize for the titles based on available space. If the space is too small (<atLeast)
        ## we don't plot any text, and we also limit to 'maximum' to avoid overblown labels. If the displayPars
        ## 'cex.title' or 'cex.axis are not NULL, those override everything else.
        nn <- vapply(trackList, names, FUN.VALUE = character(1L))
        nwrap <- vapply(nn, function(x) paste(strwrap(x, 10), collapse = "\n"), character(1))
        needAxis <- .needsAxis(trackList)
        isOnlyHoriz <- vapply(trackList, function(x) {
            res <- FALSE
            if (is(x, "DataTrack")) {
                type <- match.arg(.dpOrDefault(x, "type", "p"), .PLOT_TYPES, several.ok = TRUE)
                res <- length(setdiff(type, "horizon")) == 0
            }
            res
        }, FUN.VALUE = logical(1L))
        nwrap[needAxis] <- nn[needAxis]
        lengths <- as.numeric(convertWidth(stringWidth(nwrap), "inches")) + 0.2
        heights <- curVp$isize["height"] * spaceNeeded
        atLeast <- 0.5
        maximum <- 1.2
        allCex <- heights / lengths
        parCex <- vapply(trackList, function(x) if (is.null(displayPars(x, "cex.title"))) NA else displayPars(x, "cex.title"), FUN.VALUE = numeric(1L))
        toSmall <- allCex < atLeast
        cex <- rep(max(c(atLeast, min(c(allCex[!toSmall], maximum), na.rm = TRUE))), length(allCex))
        cex[!is.na(parCex)] <- parCex[!is.na(parCex)]
        lengthsNoWrap <- as.numeric(convertWidth(stringWidth(nn), "inches")) * cex + 0.4
        needsWrapping <- lengthsNoWrap > heights
        nwrap[allCex < atLeast & is.na(parCex)] <- ""
        nwrap[!needsWrapping] <- nn[!needsWrapping]
        ## Figure out the title width based on the available tracks. If there is an axis for at least one of the tracks,
        ## we add 1.5 time the width of a single line of text to accomodate for it.
        wfac <- curVp$isize["width"]
        leaveSpace <- ifelse(any(needAxis), 0.15, 0.2)
        width <- (as.numeric(convertHeight(stringHeight(paste("g_T", nwrap, "g_T", sep = "")), "inches")) * cex + leaveSpace) / wfac
        showtitle <- vapply(trackList, .dpOrDefault, "showTitle", TRUE, FUN.VALUE = logical(1L))
        width[!showtitle & !needAxis] <- 0
        width[!showtitle] <- 0
        twfac <- if (missing(title.width) || is.null(title.width)) 1 else title.width
        title.width <- max(width, na.rm = TRUE)
        if (any(needAxis)) {
            cex.axis <- structure(vapply(trackList, .dpOrDefault, "cex.axis", 0.6, FUN.VALUE = numeric(1L)), names = nn)
            axTicks <- unlist(lapply(trackList, function(GdObject) {
                if (!is(GdObject, "NumericTrack") && !is(GdObject, "AlignmentsTrack")) {
                    return(NULL)
                }
                yvals <- values(GdObject)
                ylim <- .dpOrDefault(GdObject, "ylim", if (!is.null(yvals) && length(yvals)) {
                    range(yvals, na.rm = TRUE, finite = TRUE)
                } else {
                    c(-1, 1)
                })
                if (diff(ylim) == 0) {
                    ylim <- ylim + c(-1, 1)
                }
                yscale <- extendrange(r = ylim, f = 0.05)
                at <- pretty(yscale)
                at[at >= sort(ylim)[1] & at <= sort(ylim)[2]]
                atSpace <- max(as.numeric(convertWidth(stringWidth(at), "inches")) + 0.18) * cex.axis[names(GdObject)]
                if (is(GdObject, "DataTrack")) {
                    type <- match.arg(.dpOrDefault(GdObject, "type", "p"), .PLOT_TYPES, several.ok = TRUE)
                    if (any(c("heatmap", "gradient") %in% type)) {
                        nlevs <- max(1, nlevels(factor(getPar(GdObject, "groups")))) - 1
                        atSpace <- atSpace + 0.3 * atSpace + as.numeric(convertWidth(unit(3, "points"), "inches")) * nlevs
                    }
                    if (any(type %in% c("heatmap", "horizon")) && .dpOrDefault(GdObject, "showSampleNames", FALSE)) {
                        sn <- rownames(values(GdObject))
                        axSpace <- ifelse(isOnlyHoriz, 0, 10)
                        wd <- max(as.numeric(convertWidth(stringWidth(sn) + unit(axSpace, "points"), "inches")))
                        atSpace <- atSpace + (wd * .dpOrDefault(GdObject, "cex.sampleNames", 0.5))
                    }
                }
                atSpace
            }))
            hAxSpaceNeeded <- (max(axTicks)) / wfac
            title.width <- title.width + hAxSpaceNeeded
        }
    } else {
        title.width <- nwrap <- cex <- NA
    }
    spacing <- as.numeric(convertWidth(unit(spacing, "points"), "npc"))
    title.width <- title.width * twfac
    return(list(spaceNeeded = spaceNeeded, cex = cex, title.width = title.width, spacing = spacing, nwrap = nwrap))
}



## This coerces likely inputs for the genomic strand  to a unified
## strand name. Accepted inputs are:
##    o a single integer, where values <=0 indicate the plus strand, and values >=1 indicate the minus strand
##    o a character, either "+" or "-"
## If extended=TRUE, the additional values 2, "+-", "-+" and "*" are allowed, indicating to use both strands.
## Value: the validated strand name
.strandName <- function(x, extended = FALSE) {
    fun <- function(x, extended) {
        if (!extended) {
            if (is.numeric(x)) {
                x <- min(c(1, max(c(0, as.integer(x)))))
            } else if (is.character(x)) {
                x <- match(x, c("+", "-")) - 1
                if (any(is.na(x))) {
                    stop("The strand has to be specified either as a character ('+' or '-'), or as an integer value (0 or 1)")
                }
            }
        } else {
            if (is.numeric(x)) {
                x <- min(c(2, max(c(0, as.integer(x)))))
            } else if (is.character(x)) {
                x <- min(c(2, match(x, c("+", "-", "+-", "-+", "*")) - 1))
                if (any(is.na(x))) {
                    stop("The strand has to be specified either as a character ('+' or '-'), or as an integer value (0 or 1)")
                }
            }
        }
        x
    }
    return(vapply(x, fun, extended, FUN.VALUE = numeric(1L)))
}



## Compute native coordinate equivalent to 'min.width' pixel. This assumes that a graphics device is already
## open, otherwise a new window will pop up, which could be a little annoying.
## Arguments:
##    o min.width: the number of pixels
##    o coord: the axis for which to compute the coordinats, one in c("x","y")
## Value: the equivalent of 'min.width' in native coordinates.
.pxResolution <- function(min.width = 1, coord = c("x", "y")) {
    coord <- match.arg(coord, several.ok = TRUE)
    curVp <- vpLocation()
    co <- c(
        x = as.vector(abs(diff(current.viewport()$xscale)) / (curVp$size["width"]) * min.width),
        y = as.vector(abs(diff(current.viewport()$yscale)) / (curVp$size["height"]) * min.width)
    )
    return(co[coord])
}


## Deal with composite exons and provide polygon plotting coordinates for them. For all normal exons simply return
## the original bounding box data
## Arguments:
##    o box: the data frame with the bounding box information
##    o type: the type of coordinates for the composite exons, one in 'box', 'arrow' or 'fixedArrow'
## Value: a list with three elements: box, the non-composite exons, pols, the plotting coordinates for the merged composite exons,
##        and polpars, a data.frame of plotting parameters for the polygons
.handleComposite <- function(box, type = "box", W = 1 / 4, H = 1 / 3, min.width = 10, max.width = Inf) {
    box <- box[order(box$start), ]
    boxFinal <- data.frame(stringsAsFactors = FALSE)
    polFinal <- data.frame(stringsAsFactors = FALSE)
    polPars <- data.frame(stringsAsFactors = FALSE)
    bss <- split(box, box$transcript)
    for (b in bss) {
        ol <- names(which(table(b$exon) > 1))
        boxFinal <- rbind(boxFinal, b[!b$exon %in% ol, ])
        if (length(ol)) {
            b <- b[b$exon %in% ol, ]
            r <- IRanges(start = b$cx1, end = b$cx2)
            rr <- reduce(r)
            brs <- split(b, subjectHits(findOverlaps(r, rr)))
            for (j in seq_along(brs)) {
                if (nrow(brs[[j]]) == 1) {
                    boxFinal <- rbind(boxFinal, brs[[j]])
                } else {
                    xlocs <- as.vector(t(brs[[j]][, c("cx1", "cx2"), drop = FALSE]))
                    ylocs <- unlist(brs[[j]][, c("cy1", "cy2"), drop = FALSE])
                    fh <- seq_len(length(ylocs) / 2)
                    sh <- (length(ylocs) / 2 + 1):length(ylocs)
                    ylocs[sh] <- rev(ylocs[sh])
                    ylocs <- rep(ylocs, each = 2)
                    if (type == "box") {
                        polFinal <- rbind(polFinal, data.frame(
                            x = c(xlocs, rev(xlocs)), y = ylocs,
                            id = paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                            stringsAsFactors = FALSE
                        ))
                        polPars <- rbind(polPars, data.frame(fill = brs[[j]][1, "fill"], col = brs[[j]][1, "col"], stringsAsFactors = FALSE))
                    } else {
                        polPars <- rbind(polPars, data.frame(fill = brs[[j]][1, "fill"], col = brs[[j]][1, "col"], stringsAsFactors = FALSE))
                        str <- brs[[j]]$strand[1]
                        offset <- (abs(brs[[j]]$cy1 - brs[[j]]$cy2)) * H / 2
                        if (str == "+" && abs(diff(xlocs[(length(xlocs) - 1):length(xlocs)])) > min.width) {
                            offset <- rep(offset, each = 2)
                            asel <- (length(xlocs) - 1):length(xlocs)
                            bsel <- seq_len(min(asel) - 1)
                            d <- abs(diff(xlocs[asel]))
                            afp <- if (type == "arrow") {
                                rep(xlocs[asel][1] + max(d * W, d - max.width), 2)
                            } else {
                                rep(max(xlocs[asel][1], xlocs[asel][2] - W), 2)
                            }
                            xlocs <- c(xlocs[bsel], xlocs[asel][1], afp, xlocs[asel][2], afp, xlocs[asel][1], rev(xlocs[bsel]))
                            mid <- length(ylocs) / 2
                            asel <- (mid - 1):(mid + 2)
                            bsel <- c(seq_len(mid - 2), (mid + 3):length(ylocs))
                            fh <- seq_len(length(bsel) / 2)
                            sh <- (length(bsel) / 2 + 1):length(bsel)
                            ylocs <- c(
                                ylocs[bsel[fh]] + offset[fh], ylocs[asel][c(1, 2)] + tail(offset, 1), ylocs[asel][1],
                                ylocs[asel][1] + abs(diff(ylocs[asel][c(1, 3)])) / 2, ylocs[asel][4],
                                ylocs[asel][3:4] - tail(offset, 1), ylocs[bsel[sh]] - offset[fh]
                            )
                            polFinal <- rbind(polFinal, data.frame(
                                x = c(xlocs, rev(xlocs)), y = ylocs,
                                id = paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                                stringsAsFactors = FALSE
                            ))
                        } else if (str == "-" && abs(diff(xlocs[c(1, 2)])) > min.width) {
                            yoffset <- c(rep(offset[-1], each = 2), -rev(rep(offset[-1], each = 2)))
                            asel <- c(1, 2)
                            bsel <- seq(3, length(xlocs))
                            d <- abs(diff(xlocs[asel]))
                            afp <- if (type == "arrow") {
                                rep(xlocs[asel][2] - max(d * W, d - max.width), 2)
                            } else {
                                rep(min(xlocs[asel][2], xlocs[asel][1] + W), 2)
                            }
                            xlocs <- c(xlocs[asel][1], afp, xlocs[asel][2], xlocs[bsel], rev(xlocs[bsel]), xlocs[asel][2], afp, xlocs[asel][1])
                            asel <- c(1, 2, seq((length(ylocs) - 1), length(ylocs)))
                            bsel <- seq(3, (length(ylocs) - 2))
                            ylocs <- c(
                                ylocs[asel][1] + abs(diff(ylocs[asel][c(1, 3)])) / 2, ylocs[asel][1], rep(ylocs[asel][1] + offset[1], 2),
                                ylocs[bsel] + yoffset, rep(ylocs[asel][3] - offset[1], 2), ylocs[asel][3], ylocs[asel][1] + abs(diff(ylocs[asel][c(1, 3)])) / 2
                            )
                            polFinal <- rbind(polFinal, data.frame(
                                x = xlocs, y = ylocs,
                                id = paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                                stringsAsFactors = FALSE
                            ))
                        } else {
                            offset <- c(rep(offset, each = 2), -rev(rep(offset, each = 2)))
                            polFinal <- rbind(polFinal, data.frame(
                                x = c(xlocs, rev(xlocs)), y = ylocs + offset,
                                id = paste(brs[[j]][1, "transcript"], brs[[j]][1, "exon"], j),
                                stringsAsFactors = FALSE
                            ))
                        }
                    }
                }
            }
        }
    }
    return(list(box = boxFinal, pols = polFinal, polpars = polPars))
}


## Take coordinates for the bounding boxes of annotation regions and plot filled arrows inside.
## Arguments:
##    o the data frame with the bounding box information
##    o W: the proportion of the total box width used for the arrow head
##    o H: the proportion of the total box height used for the arrow head
##    o lwd: the boundary line width
##    o lty: the boundary line type
##    o alpha: the transparency
##    o min.width: the minimum width of the arrow head. Below this size a simple box is drawn
## Note that the last arguments 4-9 all have to be of the same length as number of rows in box.
## Value: the function is called for its side-effects of drawing on the graphics device
.filledArrow <- function(box, W = 1 / 4, H = 1 / 3, lwd, lty, alpha, min.width = 10, max.width = Inf, absoluteWidth = FALSE) {
    boxC <- if ("transcript" %in% colnames(box)) {
        .handleComposite(box, ifelse(absoluteWidth, "fixedArrow", "arrow"), min.width = min.width, max.width = max.width, W = W, H = H)
    } else {
        list(box = box, pols = data.frame())
    }
    xx <- yy <- numeric()
    id <- character()
    pars <- data.frame()
    if (nrow(boxC$box)) {
        box <- boxC$box
        A <- box[, c(1, 2), drop = FALSE]
        B <- box[, c(3, 4), drop = FALSE]
        ## First everything that is still a box
        osel <- abs(B[, 1] - A[, 1]) < min.width | !box$strand %in% c("+", "-")
        xx <- c(A[osel, 1], B[osel, 1], B[osel, 1], A[osel, 1])
        offset <- (abs(B[osel, 2] - A[osel, 2]) * H / 2)
        yy <- c(rep(A[osel, 2] + offset, 2), rep(B[osel, 2] - offset, 2))
        id <- rep(seq_len(sum(osel)), 4)
        pars <- data.frame(fill = box$fill, col = box$col, lwd = lwd, lty = lty, alpha = alpha, stringsAsFactors = FALSE)[osel, ]
        ## Now the arrows facing right
        sel <- !osel & box$strand == "+"
        id <- c(id, rep(seq(from = if (!length(id)) 1 else max(id) + 1, by = 1, len = sum(sel)), 7))
        d <- abs(B[sel, 1] - A[sel, 1])
        alp <- if (!absoluteWidth) rep(A[sel, 1] + pmax(d * W, d - rep(max.width, sum(sel))), 2) else rep(pmax(B[sel, 1] - W, pmin(B[sel, 1], A[sel, 1])), 2)
        xx <- c(xx, A[sel, 1], alp, B[sel, 1], alp, A[sel, 1])
        yy <- c(
            yy, rep(A[sel, 2] + (abs(B[sel, 2] - A[sel, 2]) * H / 2), 2), A[sel, 2], A[sel, 2] + (abs(B[sel, 2] - A[sel, 2]) / 2), B[sel, 2],
            rep(B[sel, 2] - (abs(B[sel, 2] - A[sel, 2]) * H / 2), 2)
        )
        pars <- rbind(pars, data.frame(fill = box$fill, col = box$col, lwd = lwd, lty = lty, alpha = alpha, stringsAsFactors = FALSE)[sel, ])
        ## And finally those facing left
        sel <- !osel & box$strand == "-"
        id <- c(id, rep(seq(from = if (!length(id)) 1 else max(id) + 1, by = 1, len = sum(sel)), 7))
        d <- abs(B[sel, 1] - A[sel, 1])
        alp <- if (!absoluteWidth) rep(B[sel, 1] - pmax(d * W, d - rep(max.width, sum(sel))), 2) else rep(pmin(A[sel, 1] + W, pmax(B[sel, 1], A[sel, 1])), 2)
        xx <- c(xx, B[sel, 1], alp, A[sel, 1], alp, B[sel, 1])
        yy <- c(
            yy, rep(A[sel, 2] + (abs(B[sel, 2] - A[sel, 2]) * H / 2), 2), A[sel, 2], A[sel, 2] + (abs(B[sel, 2] - A[sel, 2]) / 2), B[sel, 2],
            rep(B[sel, 2] - (abs(B[sel, 2] - A[sel, 2]) * H / 2), 2)
        )
        pars <- rbind(pars, data.frame(fill = box$fill, col = box$col, lwd = lwd, lty = lty, alpha = alpha, stringsAsFactors = FALSE)[sel, ])
    }
    if (nrow(boxC$pols)) {
        xx <- c(xx, boxC$pols$x)
        yy <- c(yy, boxC$pols$y)
        id <- c(id, paste("pols", boxC$pols$id))
        pars <- rbind(pars, data.frame(fill = boxC$polpars$fill, col = boxC$polpars$col, lwd = lwd, lty = lty, alpha = alpha, stringsAsFactors = FALSE))
    }
    grid.polygon(
        x = xx, y = yy, gp = gpar(fill = pars$fill, col = pars$col, alpha = unique(pars$alpha), lwd = pars$lwd, lty = pars$lty), default.units = "native",
        id = factor(id)
    ) # fix for Sys.setenv(`_R_CHECK_LENGTH_1_CONDITION_`="true") Sys.setenv(`_R_CHECK_LENGTH_1_LOGIC2_`="true")
}


## Take coordinates for the bounding boxes of annotation regions and plot boxes, also making sure that
## composite exons in GeneRegionTracks (i.e., overlapping coordinates and same exon id) are merged
## appropriately
## Arguments:
##    o box: the data frame with the bounding box information
##    o lwd: the boundary line width
##    o lty: the boundary line type
##    o alpha: the transparency
## Value: the function is called for its side-effects of drawing on the graphics device
.filledBoxes <- function(box, lwd, lty, alpha) {
    if ("transcript" %in% colnames(box)) {
        box <- .handleComposite(box, "box")
        if (nrow(box$box)) {
            grid.rect(box$box$cx2, box$box$cy1,
                width = box$box$cx2 - box$box$cx1, height = box$box$cy2 - box$box$cy1,
                gp = gpar(col = as.character(box$box$col), fill = as.character(box$box$fill), lwd = lwd, lty = lty, alpha = alpha),
                default.units = "native", just = c("right", "bottom")
            )
        }
        if (nrow(box$pols)) {
            grid.polygon(
                x = box$pols$x, y = box$pols$y, id = factor(box$pols$id),
                gp = gpar(col = as.character(box$polpars$col), fill = as.character(box$polpars$fill), lwd = lwd, lty = lty, alpha = alpha),
                default.units = "native"
            )
        }
    } else {
        grid.rect(box$cx2, box$cy1,
            width = box$cx2 - box$cx1, height = box$cy2 - box$cy1,
            gp = gpar(col = as.character(box$col), fill = as.character(box$fill), lwd = lwd, lty = lty, alpha = alpha),
            default.units = "native", just = c("right", "bottom")
        )
    }
}


## Take start and end coordinates for genemodel-type annotations and draw a featherd line indicating
## the strand direction
## Arguments:
##    o xx1, xx2: integer vectors of equal length indicating the start and end of the gene models.
##    o strand: the strand information for each gene model. Needs to be of the same length as xx1 and xx2
##    o coords: the coordinates of the exon features, needed to avoid overlaps.
##    o y: the y value for the arrow bar, usually not set since it should always be 20
##    o W: the width of the arrow feathers in pixels
##    o D: the distance between arrow feathers in pixels
##    o H: the height of the arrow feathers in native coordinates (the total bounding box is usually 40)
##    o col: the boundary color
##    o lwd: the boundary line width
##    o lty: the boundary line type
##    o alpha: the transparency
##    o barOnly: only plot the bar, not the feathers
##    o diff: the current pixel resolution
##    o min.height: the minimum total height in pixels for the feathers (i.e., min.height/2 in each direction)
## Value: the function is called for its side-effects of drawing on the graphics device
.arrowBar <- function(xx1, xx2, strand, coords, y = 20, W = 3, D = 10, H, col, lwd, lty, alpha, barOnly = FALSE,
                      diff = .pxResolution(coord = "y"), min.height = 3) {
    exons <- IRanges(start = coords[, 1], end = coords[, 3])
    levels <- split(exons, coords[, 2] %/% 1)
    if (!barOnly) {
        onePx <- diff
        if (missing(H)) {
            onePy <- .pxResolution(coord = "y")
            H <- onePy * min.height / 2
        }
        fx1 <- fx2 <- scol <- fy1 <- fy2 <- NULL
        for (i in seq_along(xx1)) {
            x1 <- xx1[i]
            x2 <- xx2[i]
            len <- diff(c(x1, x2)) / onePx
            if (len > D + W * 2) {
                ax1 <- seq(from = x1 + (onePx * W), to = x1 + (len * onePx) - (onePx * W), by = onePx * D)
                ax2 <- ax1 + (onePx * W)
                feathers <- IRanges(start = ax1 - onePx, end = ax2 + onePx)
                cur.level <- y[i] %/% 1
                sel <- queryHits(findOverlaps(feathers, resize(levels[[cur.level]], width = width(levels[[cur.level]]) - 1)))
                if (length(sel)) {
                    ax1 <- ax1[-sel]
                    ax2 <- ax2[-sel]
                }
                fx1 <- c(fx1, rep(if (strand[i] == "-") ax1 else ax2, each = 2))
                fx2 <- c(fx2, rep(if (strand[i] == "-") ax2 else ax1, each = 2))
                scol <- c(scol, rep(col[i], length(ax1) * 2))
                fy1 <- c(fy1, rep(rep(y[i], length(ax1) * 2)))
                fy2 <- c(fy2, rep(c(y[i] - H, y[i] + H), length(ax1)))
            }
        }
        if (!is.null(fx1) && length(fx1)) {
            grid.segments(fx1, fy1, fx2, fy2, default.units = "native", gp = gpar(col = scol, lwd = lwd, lty = lty, alpha = alpha))
        }
    }
    bars <- data.frame(x1 = xx1, x2 = xx2, y = y, col = col, stringsAsFactors = FALSE)
    cutBars <- data.frame()
    for (i in seq_len(nrow(bars))) {
        b <- bars[i, ]
        cur.level <- b$y %/% 1
        ct <- if (cur.level != 0) {
            setdiff(
                IRanges(start = b$x1, end = b$x2),
                resize(levels[[cur.level]], width = width(levels[[cur.level]]) - 1)
            )
        } else {
            IRanges()
        }
        if (length(ct)) {
            cutBars <- rbind(cutBars, data.frame(x1 = start(ct), x2 = end(ct), y = b$y, col = b$col, stringsAsFactors = FALSE))
        }
    }
    ## fix bug when no introns are present
    if (nrow(cutBars)) {
        grid.segments(cutBars$x1, cutBars$y, cutBars$x2, cutBars$y,
            default.units = "native",
            gp = gpar(col = cutBars$col, lwd = lwd, lty = lty, alpha = alpha, lineend = "square")
        )
    }
    ## grid.segments(xx1, y, xx2, y, default.units="native", gp=gpar(col=col, lwd=lwd, lty=lty, alpha=alpha, lineend="square"))
}



## Extract track color for different subtypes within the track and use the default
## color value if no other is found, lightblue if no colors are set at all
## Arguments:
##    o GdObject: object inheriting from class GdObject
## Value: a color character
.getBiotypeColor <- function(GdObject) {
    defCol <- .dpOrDefault(GdObject, "fill", .DEFAULT_FILL_COL)
    col <- lapply(
        as.character(values(GdObject)[, "feature"]),
        function(x) .dpOrDefault(GdObject, x)[1]
    )
    needsDef <- vapply(col, is.null, FUN.VALUE = logical(1L))
    col[needsDef] <- rep(defCol, sum(needsDef))[seq_len(sum(needsDef))]
    return(unlist(col))
}


## Compute pretty tickmark location (code from tilingArray package)
## Arguments:
##    o x: a vector of data values
## Value: the tick mark coordinates
.ticks <- function(x) {
    rx <- range(x)
    lz <- log((rx[2] - rx[1]) / 3, 10)
    fl <- floor(lz)
    if (lz - fl > log(5, 10)) {
        fl <- fl + log(5, 10)
    }
    tw <- round(10^fl)
    i0 <- ceiling(rx[1] / tw)
    i1 <- floor(rx[2] / tw)
    seq(i0, i1) * tw
}



## A lattice-style panel function to draw smoothed 'mountain' plots
## Arguments:
##    o x, y: the x and y coordinates form the plot
##    o span, degree, family, evaluation: parameters that are passed on to loess
##    o lwd, lty, col: color, with and type of the plot lines
##    o fill: fill colors for areas above and under the baseline, a vector of length two
##    o col.line: color of the baseline
##    o baseline: the y value of the horizontal baseline
##    o alpha: the transparancy
## Value: the function is called for its side-effect of drawing on the graphics device

#' @importFrom stats loess.smooth
.panel.mountain <- function(x, y, span = 2 / 3, degree = 1, family = c("symmetric", "gaussian"), evaluation = 50,
                            lwd = plot.line$lwd, lty = plot.line$lty, col, col.line = plot.line$col,
                            baseline, fill, alpha = 1, ...) {
    x <- as.numeric(x)
    y <- as.numeric(y)
    fill <- rep(fill, 2)
    ok <- is.finite(x) & is.finite(y)
    if (sum(ok) < 1) {
        return()
    }
    if (!missing(col)) {
        if (missing(col.line)) {
            col.line <- col
        }
    }
    plot.line <- lattice::trellis.par.get("plot.line")
    smooth <- loess.smooth(x[ok], y[ok],
        span = span, family = family,
        degree = degree, evaluation = evaluation
    )
    tmp <- as.integer(smooth$y < baseline)
    changePoint <- NULL
    for (i in seq_along(tmp)) {
        if (i > 1 && tmp[i] != tmp[i - 1]) {
            changePoint <- c(changePoint, i)
        }
    }
    m <- (smooth$y[changePoint] - smooth$y[changePoint - 1]) / (smooth$x[changePoint] - smooth$x[changePoint - 1])
    xCross <- ((baseline - smooth$y[changePoint - 1]) / m) + smooth$x[changePoint - 1]
    newX <- newY <- NULL
    j <- 1
    xx <- smooth$x
    yy <- smooth$y
    smooth$x <- c(smooth$x, tail(smooth$x, 1))
    smooth$y <- c(smooth$y, baseline)
    xvals <- smooth$x[1]
    yvals <- baseline
    for (i in seq_along(smooth$x)) {
        if (i == length(smooth$x)) {
            xvals <- c(xvals, smooth$x[i])
            yvals <- c(yvals, baseline)
            fcol <- if (mean(yvals) < baseline) fill[1] else fill[2]
            panel.polygon(xvals, yvals, fill = fcol, col = fcol, border = fcol, alpha = alpha)
        } else if (i %in% changePoint) {
            xvals <- c(xvals, xCross[j])
            yvals <- c(yvals, baseline)
            fcol <- if (mean(yvals) < baseline) fill[1] else fill[2]
            panel.polygon(xvals, yvals, fill = fcol, col = fcol, border = fcol, alpha = alpha)
            xvals <- c(xCross[j], smooth$x[i])
            yvals <- c(baseline, smooth$y[i])
            j <- j + 1
        } else {
            xvals <- c(xvals, smooth$x[i])
            yvals <- c(yvals, smooth$y[i])
        }
    }
    grid.lines(
        x = xx, y = yy, gp = gpar(col = col.line, lty = lty, lwd = lwd, alpha = alpha),
        default.units = "native", ...
    )
}

## A lattice-style panel function to draw polygons (like coverage)
## Arguments:
##    o x, y: the x and y coordinates form the plot
##    o lwd, lty, col: color, with and type of the plot lines
##    o fill: fill colors for areas above and under the baseline, a vector of length two
##    o col.line: color of the baseline
##    o baseline: the y value of the horizontal baseline
##    o alpha: the transparancy
## Value: the function is called for its side-effect of drawing on the graphics device
.panel.polygon <- function(x, y, lwd = plot.line$lwd, lty = plot.line$lty, col,
                           col.line = plot.line$col, baseline, fill, alpha = 1, ...) {
    x <- as.numeric(x)
    y <- as.numeric(y)
    fill <- rep(fill, 2)
    ok <- is.finite(x) & is.finite(y)
    if (sum(ok) < 1) {
        return()
    }
    if (!missing(col)) {
        if (missing(col.line)) {
            col.line <- col
        }
    }
    x <- x[ok]
    y <- y[ok]
    plot.line <- lattice::trellis.par.get("plot.line")
    changePoint <- NULL
    tmp <- as.integer(y < baseline)
    for (i in seq_along(tmp)) {
        if (i > 1 && tmp[i] != tmp[i - 1]) {
            changePoint <- c(changePoint, i)
        }
    }
    m <- (y[changePoint] - y[changePoint - 1]) / (x[changePoint] - x[changePoint - 1])
    xCross <- ((baseline - y[changePoint - 1]) / m) + x[changePoint - 1]
    newX <- newY <- NULL
    j <- 1
    x <- c(x, tail(x, 1))
    y <- c(y, baseline)
    xvals <- x[1]
    yvals <- baseline
    for (i in seq_along(x)) {
        if (i == length(x)) {
            xvals <- c(xvals, x[i])
            yvals <- c(yvals, baseline)
            fcol <- if (mean(yvals) < baseline) fill[1] else fill[2]
            panel.polygon(xvals, yvals, fill = fcol, col = fcol, border = fcol, alpha = alpha)
        } else if (i %in% changePoint) {
            xvals <- c(xvals, xCross[j])
            yvals <- c(yvals, baseline)
            fcol <- if (mean(yvals) < baseline) fill[1] else fill[2]
            panel.polygon(xvals, yvals, fill = fcol, col = fcol, border = fcol, alpha = alpha)
            xvals <- c(xCross[j], x[i])
            yvals <- c(baseline, y[i])
            j <- j + 1
        } else {
            xvals <- c(xvals, x[i])
            yvals <- c(yvals, y[i])
        }
    }
    grid.lines(
        x = x, y = y, gp = gpar(col = col.line, lty = lty, lwd = lwd, alpha = alpha),
        default.units = "native", ...
    )
}


## A lattice-style panel function to draw box and whisker plots with groups
## Arguments: see ? panel.bwplot for details
## Value: the function is called for its side-effect of drawing on the graphics device

#' @importFrom grDevices boxplot.stats
.panel.bwplot <- function(x, y, box.ratio = 1, box.width = box.ratio / (1 + box.ratio), lwd, lty, fontsize,
                          pch, col, alpha, cex, font, fontfamily, fontface, fill, varwidth = FALSE, notch = FALSE,
                          notch.frac = 0.5, ..., levels.fos = sort(unique(x)),
                          stats = boxplot.stats, coef = 1.5, do.out = TRUE) {
    if (all(is.na(x) | is.na(y))) {
        return()
    }
    x <- as.numeric(x)
    y <- as.numeric(y)
    cur.limits <- current.panel.limits()
    xscale <- cur.limits$xlim
    yscale <- cur.limits$ylim
    if (!notch) {
        notch.frac <- 0
    }
    blist <- tapply(y, factor(x, levels = levels.fos), stats,
        coef = coef, do.out = do.out
    )
    blist.stats <- do.call(rbind, lapply(blist, "[[", "stats"))
    blist.out <- lapply(blist, "[[", "out")
    blist.height <- box.width
    if (varwidth) {
        maxn <- max(table(x))
        blist.n <- vapply(blist, "[[", "n", FUN.VALUE = numeric(1L))
        blist.height <- sqrt(blist.n / maxn) * blist.height
    }
    blist.conf <- if (notch) {
        t(vapply(blist, "[[", "conf", FUN.VALUE = numeric(2L)))
    } else {
        blist.stats[, c(2, 4), drop = FALSE]
    }
    ybnd <- cbind(
        blist.stats[, 3], blist.conf[, 2], blist.stats[, 4], blist.stats[, 4], blist.conf[, 2],
        blist.stats[, 3], blist.conf[, 1], blist.stats[, 2], blist.stats[, 2], blist.conf[, 1], blist.stats[, 3]
    )
    xleft <- levels.fos - blist.height / 2
    xright <- levels.fos + blist.height / 2
    xbnd <- cbind(
        xleft + notch.frac * blist.height / 2, xleft,
        xleft, xright, xright, xright - notch.frac * blist.height / 2,
        xright, xright, xleft, xleft, xleft + notch.frac *
            blist.height / 2
    )
    xs <- matrix(NA_real_, nrow = nrow(xbnd) * 2, ncol = ncol(xbnd))
    ys <- matrix(NA_real_, nrow = nrow(xbnd) * 2, ncol = ncol(xbnd))
    xs[seq(along.with = levels.fos, by = 2), ] <- xbnd[seq(along.with = levels.fos), ]
    ys[seq(along.with = levels.fos, by = 2), ] <- ybnd[seq(along.with = levels.fos), ]
    panel.polygon(t(xs), t(ys), lwd = lwd, lty = lty, col = fill, alpha = alpha, border = col)
    panel.segments(rep(levels.fos, 2), c(blist.stats[, 2], blist.stats[, 4]), rep(levels.fos, 2),
        c(blist.stats[, 1], blist.stats[, 5]),
        col = col, alpha = alpha,
        lwd = lwd, lty = lty
    )
    panel.segments(levels.fos - blist.height / 2, c(blist.stats[, 1], blist.stats[, 5]), levels.fos + blist.height / 2,
        c(blist.stats[, 1], blist.stats[, 5]),
        col = col, alpha = alpha, lwd = lwd, lty = lty
    )
    if (all(pch == "|")) {
        mult <- if (notch) {
            1 - notch.frac
        } else {
            1
        }
        panel.segments(levels.fos - mult * blist.height / 2,
            blist.stats[, 3], levels.fos + mult * blist.height / 2,
            blist.stats[, 3],
            lwd = lwd, lty = lty,
            col = col, alpha = alpha
        )
    } else {
        panel.points(
            x = levels.fos, y = blist.stats[, 3],
            pch = pch, col = col, alpha = alpha, cex = cex,
            fontfamily = fontfamily, fontface = .chooseFace(
                fontface,
                font
            ), fontsize = fontsize
        )
    }
    panel.points(
        x = rep(levels.fos, vapply(blist.out, length, FUN.VALUE = numeric(1L))),
        y = unlist(blist.out), pch = pch, col = col,
        alpha = alpha, cex = cex,
        fontfamily = fontfamily, fontface = .chooseFace(
            fontface,
            font
        ), fontsize = fontsize
    )
}



## Check which parameters have already been set for a GdObject, and
## update all missing ones from the prototype of the current parent
## class.
## Arguments:
##    o x: an object inheriting from class GdObject
##    o class: the parent class from which to draw the missing parameters
## Value: The updated GdObject
.updatePars <- function(x, class) {
    current <- getPar(x, hideInternal = FALSE)
    defaults <- getPar(getClass(class)@prototype@dp, hideInternal = FALSE)
    ## Check whether we need to adjust any of those defaults based on the selected scheme
    sid <- getOption("Gviz.scheme")
    scheme <- if (is.null(sid)) list() else .schemes[[sid]]
    schemePars <- if (is.null(scheme) || is.null(scheme[[class]])) list() else scheme[[class]]
    if (!is.list(schemePars)) {
        stop(sprintf("Corrupted parameter definition for class '%s' in scheme '%s'", class, sid))
    }
    defaults[names(schemePars)] <- schemePars
    missing <- setdiff(names(defaults), names(current))
    if (is.null(getPar(x, ".__appliedScheme")) && length(schemePars)) {
        defaults[[".__appliedScheme"]] <- if (is.null(sid)) NA else sid
        defaults[names(schemePars)] <- schemePars
        missing <- c(union(missing, names(schemePars)), ".__appliedScheme")
    }
    x <- setPar(x, defaults[missing], interactive = FALSE)
    return(x)
}

### Schemes ------------------------------------------------------------------

## The scheme settings registry. This is essentially just a nested list of display parameters, and the values in this list
## will be used to initialize the objects. Please note that a display parameter still needs to be defined in the class
## definition for this to work, and that all parameters that are not set explicitly in the scheme will be taken from
## the defaults in that class definition. Scheme parameters still override everything else, and even parameters that are
## not defined for the class will be added from the scheme settings.

#' @importFrom grDevices boxplot.stats
.schemes <- new.env()
.schemes[["default"]] <- list(
    GdObject = list(
        alpha = 1,
        background.panel = "transparent",
        background.title = "lightgray",
        background.legend = "transparent",
        cex.axis = NULL,
        cex.title = NULL,
        cex = 1,
        col.axis = "white",
        col.frame = "lightgray",
        col.grid = .DEFAULT_SHADED_COL,
        col.line = NULL,
        col.symbol = NULL,
        col.title = "white",
        col = .DEFAULT_SYMBOL_COL,
        collapse = TRUE,
        fill = .DEFAULT_FILL_COL,
        fontcolor.title = "white",
        fontcolor = "black",
        fontface.title = 2,
        fontface = 1,
        fontfamily.title = "sans",
        fontfamily = "sans",
        fontsize = 12,
        frame = FALSE,
        grid = FALSE,
        h = -1,
        lineheight = 1,
        lty.grid = "solid",
        lty = "solid",
        lwd.border = 1,
        lwd.grid = 1,
        lwd = 1,
        min.distance = 1,
        min.height = 3,
        min.width = 1,
        rotation.title = 90,
        rotation = 0,
        showAxis = TRUE,
        showTitle = TRUE,
        size = 1,
        v = -1
    ),
    StackedTrack = list(
        stackHeight = 0.75,
        reverseStacking = FALSE
    ),
    AnnotationTrack = list(
        arrowHeadWidth = 30,
        arrowHeadMaxWidth = 40,
        cex.group = 0.6,
        cex = 1,
        col.line = "darkgray",
        col = .DEFAULT_LINE_COL,
        featureAnnotation = NULL,
        fill = "lightblue",
        fontcolor.group = .DEFAULT_SHADED_COL,
        fontcolor.item = "white",
        fontface.group = 2,
        groupAnnotation = NULL,
        lex = 1,
        lineheight = 1,
        lty = "solid",
        lwd = 1,
        mergeGroups = FALSE,
        rotation = 0,
        shape = "arrow",
        showFeatureId = NULL,
        showId = NULL,
        showOverplotting = FALSE,
        size = 1
    ),
    DetailsAnnotationTrack = list(
        details.minWidth = 100,
        details.ratio = Inf,
        details.size = 0.5,
        detailsBorder.col = "darkgray",
        detailsBorder.fill = "transparent",
        detailsBorder.lty = "solid",
        detailsBorder.lwd = 1,
        detailsConnector.cex = 1,
        detailsConnector.col = "darkgray",
        detailsConnector.lty = "dashed",
        detailsConnector.lwd = 1,
        detailsConnector.pch = 20,
        detailsFunArgs = list(),
        groupDetails = FALSE
    ),
    GeneRegionTrack = list(
        arrowHeadWidth = 10,
        arrowHeadMaxWidth = 20,
        col = .DEFAULT_LINE_COL,
        collapseTranscripts = FALSE,
        exonAnnotation = NULL,
        fill = "#FFD58A",
        min.distance = 0,
        shape = c("smallArrow", "box"),
        showExonId = NULL,
        thinBoxFeature = .THIN_BOX_FEATURES,
        transcriptAnnotation = NULL
    ),
    BiomartGeneRegionTrack = list(
        C_segment = "burlywood4",
        D_segment = "lightblue",
        J_segment = "dodgerblue2",
        Mt_rRNA = "yellow",
        Mt_tRNA = "darkgoldenrod",
        Mt_tRNA_pseudogene = "darkgoldenrod1",
        V_segment = "aquamarine",
        miRNA = "cornflowerblue",
        miRNA_pseudogene = "cornsilk",
        misc_RNA = "cornsilk3",
        misc_RNA_pseudogene = "cornsilk4",
        protein_coding = "orange",
        pseudogene = "brown1",
        rRNA = "darkolivegreen1",
        rRNA_pseudogene = "darkolivegreen",
        retrotransposed = "blueviolet",
        scRNA = "gold4",
        scRNA_pseudogene = "darkorange2",
        snRNA = "coral",
        snRNA_pseudogene = "coral3",
        snoRNA = "cyan",
        snoRNA_pseudogene = "cyan2",
        tRNA_pseudogene = "antiquewhite3",
        utr3 = "orange",
        utr5 = "orange"
    ),
    GenomeAxisTrack = list(
        add35 = FALSE,
        add53 = FALSE,
        background.title = "transparent",
        cex.id = 0.7,
        cex = 0.8,
        col.id = "white",
        col.range = "cornsilk4",
        distFromAxis = 1,
        exponent = NULL,
        fill.range = "cornsilk3",
        fontcolor = "#808080",
        fontsize = 10,
        labelPos = "alternating",
        littleTicks = FALSE,
        lwd = 2,
        scale = NULL,
        showId = FALSE,
        showTitle = FALSE,
        size = NULL,
        col = "darkgray"
    ),
    DataTrack = list(
        aggregateGroups = FALSE,
        aggregation = "mean",
        missingAsZero = TRUE,
        amount = NULL,
        baseline = NULL,
        box.ratio = 1,
        box.width = NULL,
        cex.legend = 0.8,
        cex.sampleNames = NULL,
        cex = 0.7,
        coef = 1.5,
        col.baseline = NULL,
        col.boxplotFrame = .DEFAULT_SHADED_COL,
        col.histogram = .DEFAULT_SHADED_COL,
        col.horizon = NA,
        col.mountain = NULL,
        col.sampleNames = "white",
        col = lattice::trellis.par.get("superpose.line")[["col"]],
        collapse = FALSE,
        degree = 1,
        do.out = TRUE,
        evaluation = 50,
        factor = 0.5,
        family = "symmetric",
        fill.histogram = NULL,
        fill.horizon = c("#B41414", "#E03231", "#F7A99C", "#9FC8DC", "#468CC8", "#0165B3"),
        fill.mountain = c("#CCFFFF", "#FFCCFF"),
        fontcolor.legend = .DEFAULT_SHADED_COL,
        gradient = RColorBrewer::brewer.pal(9, "Blues"),
        groups = NULL,
        horizon.origin = 0,
        horizon.scale = NULL,
        jitter.x = FALSE,
        jitter.y = FALSE,
        levels.fos = NULL,
        lty.baseline = NULL,
        lty.mountain = NULL,
        lwd.baseline = NULL,
        lwd.mountain = NULL,
        min.distance = 0,
        na.rm = FALSE,
        ncolor = 100,
        notch.frac = 0.5,
        notch = FALSE,
        pch = 20,
        separator = 0,
        showColorBar = TRUE,
        showSampleNames = FALSE,
        size = NULL,
        span = 1 / 5,
        stackedBars = TRUE,
        stats = boxplot.stats,
        transformation = NULL,
        type = "p",
        varwidth = FALSE,
        window = NULL,
        windowSize = NULL,
        ylim = NULL,
        yTicksAt = NULL
    ),
    IdeogramTrack = list(
        background.title = "transparent",
        bevel = 0.45,
        cex.bands = 0.7,
        cex = 0.8,
        col = "red",
        fill = "#FFE3E6",
        fontcolor = .DEFAULT_SHADED_COL,
        fontsize = 10,
        showBandId = FALSE,
        showId = TRUE,
        showTitle = FALSE,
        size = NULL
    ),
    SequenceTrack = list(
        add53 = FALSE,
        background.title = "transparent",
        cex = 1,
        col = "darkgray",
        complement = FALSE,
        fontface = 2,
        fontsize = 10,
        lwd = 2,
        min.width = 2,
        noLetters = FALSE,
        rotation = 0,
        showTitle = FALSE,
        size = NULL
    ),
    HighlightTrack = list(
        col = "red",
        fill = "#FFE3E6"
    )
)

.schemes[["default.old"]] <- list(
    AnnotationTrack = list(col = "transparent")
)

### Scheme Functions ---------------------------------------------------------

## A helper function to be called upon package load that tries to find a stored Gviz scheme in the working directory
.collectSchemes <- function() {
    try(
        {
            if (".GvizSchemes" %in% base::ls(globalenv(), all.names = TRUE)) {
                schemes <- get(".GvizSchemes", globalenv())
                if (is.list(schemes) && !is.null(names(schemes))) {
                    lapply(names(schemes), function(x) addScheme(schemes[[x]], x))
                }
            }
        },
        silent = TRUE
    )
}


## Helper function to select fontfaces
.chooseFace <- function(fontface = NULL, font = 1) {
    if (is.null(fontface)) {
        font
    } else {
        fontface
    }
}


## Get a scheme
## Arguments:
##    o name: the name of the scheme to get. Defaults to the current one.
## Value: A list containing the scheme
#' @export
getScheme <- function(name = getOption("Gviz.scheme")) {
    s <- NULL
    if (!is.null(name)) {
        s <- .schemes[[name]]
    }
    if (is.null(s)) {
        s <- list()
    }
    return(s)
}

## Add a new scheme
## Arguments:
##    o scheme: the scheme to add
##    o name: the name of the scheme to add
#' @export
addScheme <- function(scheme, name) {
    .schemes[[name]] <- scheme
}

### Helper Functions ---------------------------------------------------------


## Helper function to compute native coordinate equivalent to 1 pixel and resize all ranges in a GRanges object accordingly
## Arguments:
##    o r: object inheriting from class GdObject
##    o min.width: the minimal pixel width of a region
##    o diff: the pixel resolution
## Value: the updated GdObject
.resize <- function(r, min.width = 2, diff = .pxResolution(coord = "x")) {
    if (min.width > 0) {
        minXDiff <- ceiling(min.width * diff)
        ## Extend all ranges to at least minXDiff
        xdiff <- width(r)
        xsel <- xdiff < minXDiff
        if (any(xsel)) {
            rr <- if (is(r, "GRanges")) ranges(r) else r
            start(rr)[xsel] <- pmax(1, start(rr)[xsel] - (minXDiff - xdiff[xsel]) / 2)
            end(rr)[xsel] <- end(rr)[xsel] + (minXDiff - xdiff[xsel]) / 2
            if (is(r, "GRanges")) r@ranges <- rr else r <- rr
        }
    }
    return(r)
}

## Tables containing the UCSC to ENSEMBL genome mapping
.biomartCurrentVersionTable <- read.delim(system.file(file.path("extdata", "biomartVersionsNow.txt"), package = "Gviz"), as.is = TRUE)
.biomartVersionTable <- read.delim(system.file(file.path("extdata", "biomartVersionsLatest.txt"), package = "Gviz"), as.is = TRUE)

## Helper function to map between UCSC and ENSEMBL genome information
## Arguments:
##    o id: character scalar, a UCSC genome identifier
## Value: a list with ENSEMBL the genome information
.ucsc2Ensembl <- function(id) {
    mt <- match(tolower(id), tolower(.biomartCurrentVersionTable$ucscId))
    val <- .biomartCurrentVersionTable[mt, ]
    if (is.na(mt)) {
        mt <- match(tolower(id), tolower(.biomartVersionTable$ucscId))
        val <- .biomartVersionTable[mt, c("species", "value", "dataset", "ucscId", "speciesShort", "speciesLong", "date", "version")]
    }
    return(as.list(val))
}

## Helper function to get the ENSEMBL biomart given a UCSC identifier
## Arguments:
##    o genome: character scalar, a UCSC genome identifier
## Value: a biomaRt object
.getBiomart <- function(genome) {
    map <- .ucsc2Ensembl(genome)
    if (map$date == "head") {
        bm <- useEnsembl(biomart = "ensembl", dataset = map$dataset)
        ds <- listDatasets(bm)
        mt <- ds[match(map$dataset, ds$dataset), "version"]
        if (is.na(mt)) {
            stop(sprintf(
                "Gviz thinks that the UCSC genome identifier '%s' should map to the Biomart data set '%s' which is not correct.",
                genome, map$dataset
            ), "\nPlease manually provide biomaRt object")
        }
        if (mt != map$value) {
            stop(sprintf(
                "Gviz thinks that the UCSC genome identifier '%s' should map to the current Biomart head as '%s', ",
                genome, map$value, mt
            ), "but its current version is '%s'.\nPlease manually provide biomaRt object.")
        }
    } else {
        bm <- useEnsembl(biomart = "ENSEMBL_MART_ENSEMBL", dataset = map$dataset, host = sprintf("%s.archive.ensembl.org", tolower(sub(".", "", map$date, fixed = TRUE))))
        ds <- listDatasets(bm)
        mt <- ds[match(map$dataset, ds$dataset), "version"]
        if (is.na(mt)) {
            stop(sprintf(
                "Gviz thinks that the UCSC genome identifier '%s' should map to the Biomart data set '%s' which is not correct.",
                genome, map$dataset
            ), "\nPlease manually provide biomaRt object")
        }
        if (mt != map$value) {
            stop(
                sprintf(
                    "Gviz thinks that the UCSC genome identifier '%s' should map to Biomart archive %s (version %s) as '%s',",
                    genome, sub(".", " ", map$date, fixed = TRUE), map$version, map$value, mt
                ),
                "but its version is '%s'.\nPlease manually provide biomaRt object"
            )
        }
    }
    return(bm)
}


## Helper function to translate from a UCSC genome name to a Biomart data set. This also caches the mart
## object in order to speed up subsequent calls
## Arguments:
##    o genome: character giving the UCSC genome
## Value: A BiomaRt connection object
.genome2Dataset <- function(genome) {
    map <- .ucsc2Ensembl(genome)
    if (is.na(map$date)) {
        stop(sprintf("Unable to automatically determine Biomart data set for UCSC genome identifier '%s'.\nPlease manually provide biomaRt object", genome))
    }
    cenv <- environment()
    bm <- .doCache(paste(map$dataset, genome, sep = "_"), expression(.getBiomart(genome)), .ensemblCache, cenv)
    return(bm)
}



## Return the plotting range for a GdObject, either from the contained ranges or from overrides.
## This function is vectorized and should also work for lists of GdObjects.

#' @importMethodsFrom BSgenome seqnames
.defaultRange <- function(GdObject, from = NULL, to = NULL, extend.left = 0, extend.right = 0, factor = 0.01, annotation = FALSE) {
    if (!is.list(GdObject)) {
        GdObject <- list(GdObject)
    }
    GdObject <- c(GdObject, unlist(lapply(GdObject, function(x) if (is(x, "HighlightTrack") || is(x, "OverlayTrack")) x@trackList else NULL)))
    if (!length(GdObject) || !all(vapply(GdObject, is, "GdObject", FUN.VALUE = logical(1L)))) {
        stop("All items in the list must inherit from class 'GdObject'")
    }
    GdObject <- GdObject[!vapply(GdObject, is, "OverlayTrack", FUN.VALUE = logical(1L))]
    tfrom <- lapply(GdObject, function(x) {
        tmp <- start(x)
        if (is(x, "RangeTrack")) tmp <- tmp[seqnames(x) == chromosome(x)]
        tmp
    })
    tfrom <- if (is.null(unlist(tfrom))) Inf else min(vapply(tfrom[listLen(tfrom) > 0], min, FUN.VALUE = numeric(1L)))
    tto <- lapply(GdObject, function(x) {
        tmp <- end(x)
        if (is(x, "RangeTrack")) tmp <- tmp[seqnames(x) == chromosome(x)]
        tmp
    })
    tto <- if (is.null(unlist(tto))) Inf else max(vapply(tto[listLen(tto) > 0], max, FUN.VALUE = numeric(1L)))
    if ((is.null(from) || is.null(to)) && ((is.infinite(tfrom) || is.infinite(tto)) || is(GdObject, "GenomeAxisTrack"))) {
        stop(
            "Unable to automatically determine plotting ranges from the supplied track(s).\nPlease provide ",
            "range coordinates through the 'from' and 'to' arguments of the plotTracks function."
        )
    }
    ## FIX the cases with identical "tfrom" and "tto" (one base-pair plotting) by adding +1 to "tto"
    if (tto == tfrom) {
        tto <- tto + 1
    }
    range <- extendrange(r = c(tfrom, tto), f = factor)
    range[1] <- max(1, range[1])
    wasNull <- rep(FALSE, 2)
    if (is.null(from)) {
        wasNull[1] <- TRUE
        from <- range[1]
    }
    if (is.null(to)) {
        to <- range[2]
        wasNull[2] <- TRUE
    }
    ## We may need some extra space for annotations
    if (annotation) {
        rr <- unlist(lapply(GdObject, function(x) {
            gr <- .dpOrDefault(x, ".__groupRanges")
            gw <- .dpOrDefault(x, ".__groupLabelWidths", data.frame(before = 0, after = 0))
            if (is.null(gr) || length(gr) == 0) NULL else c(min(start(gr) + gw$before), max(end(gr) - gw$after))
        }))
        if (!is.null(rr)) {
            rr <- matrix(rr, ncol = 2, byrow = TRUE)
            if (wasNull[1]) {
                from <- min(from, rr[, 1])
            }
            if (wasNull[2]) {
                to <- max(to, rr[, 2])
            }
        }
    }
    from <- if (extend.left != 0 && extend.left > -1 && extend.left < 1) {
        from - (abs(diff(c(from, to))) * extend.left)
    } else {
        from - extend.left
    }
    to <- if (extend.right != 0 && extend.right > -1 && extend.right < 1) {
        to + (abs(diff(c(from, to))) * extend.right)
    } else {
        to + extend.right
    }
    if (from > to) {
        warning("'from' range can not be larger than 'to', reversing range coordinates")
        tto <- from
        from <- to
        to <- tto
    }
    return(c(from = as.integer(from), to = as.integer(to)))
}


## Figure out the colors to use for a DataTrack object from the supplied display parameters
.getPlottingFeatures <- function(GdObject) {
    pch <- .dpOrDefault(GdObject, "pch", 20)
    lty <- .dpOrDefault(GdObject, "lty", 1)
    lwd <- .dpOrDefault(GdObject, "lwd", 1)
    cex <- .dpOrDefault(GdObject, "cex", 0.7)
    groups <- .dpOrDefault(GdObject, "groups")
    col <- .dpOrDefault(GdObject, "col", "#0080ff")
    if (is.null(groups)) { ## When there are no groups we force a single color for all lines and points
        col <- col[1]
        col.line <- .dpOrDefault(GdObject, "col.line", col)[1]
        col.symbol <- .dpOrDefault(GdObject, "col.symbol", col)[1]
        pch <- pch[1]
        lwd <- lwd[1]
        lty <- lty[1]
        cex <- cex[1]
    } else { ## Otherwise colors are being mapped to group factors
        if (!is.factor(groups)) {
            groups <- factor(groups)
        }
        nms <- levels(groups)
        col <- .dpOrDefault(GdObject, "col", lattice::trellis.par.get("superpose.line")$col)
        col <- rep(col, nlevels(groups))[seq_along(levels(groups))]
        col.line <- rep(.dpOrDefault(GdObject, "col.line", col), nlevels(groups))[seq_along(levels(groups))]
        col.symbol <- rep(.dpOrDefault(GdObject, "col.symbol", col), nlevels(groups))[seq_along(levels(groups))]
        lwd <- rep(lwd, nlevels(groups))[seq_along(levels(groups))]
        lty <- rep(lty, nlevels(groups))[seq_along(levels(groups))]
        pch <- rep(pch, nlevels(groups))[seq_along(levels(groups))]
        cex <- rep(cex, nlevels(groups))[seq_along(levels(groups))]
        names(col) <- names(col.line) <- names(col.symbol) <- names(lwd) <- names(lty) <- names(pch) <- names(cex) <- nms
    }
    col.baseline <- .dpOrDefault(GdObject, "col.baseline", col)[1]
    col.grid <- .dpOrDefault(GdObject, "col.grid", "#e6e6e6")[1]
    fill <- .dpOrDefault(GdObject, "fill", .DEFAULT_FILL_COL)[1]
    fill.histogram <- .dpOrDefault(GdObject, "fill.histogram", fill)[1]
    col.histogram <- .dpOrDefault(GdObject, "col.histogram", .dpOrDefault(GdObject, "col", .DEFAULT_SHADED_COL))[1]
    lty.grid <- .dpOrDefault(GdObject, "lty.grid", 1)
    lwd.grid <- .dpOrDefault(GdObject, "lwd.grid", 1)
    return(list(
        col = col, col.line = col.line, col.symbol = col.symbol, col.baseline = col.baseline,
        col.grid = col.grid, col.histogram = col.histogram, fill = fill, fill.histogram = fill.histogram,
        lwd = lwd, lty = lty, pch = pch, cex = cex, lwd.grid = lwd.grid, lty.grid = lty.grid
    ))
}


.legendInfo <- function() {
    legInfo <- matrix(FALSE, ncol = 7, nrow = 17, dimnames = list(
        c(
            "p", "b", "l", "a", "s", "S", "r", "h", "smooth",
            "histogram", "boxplot", "heatmap", "gradient", "mountain", "g", "horizon", "confint"
        ),
        c("lty", "lwd", "pch", "col", "cex", "col.lines", "col.symbol")
    ))
    legInfo[seq(2, 9), c("lty", "lwd", "col.lines")] <- TRUE
    legInfo[seq(1, 2), c("pch", "cex", "col.symbol")] <- TRUE
    legInfo[seq(1, 12), "col"] <- TRUE
    return(legInfo)
}

## A helper function to get the currently active chromosomes, also if the track is one of the collection
## track classes
.recChromosome <- function(GdObject) {
    chroms <- if (is(GdObject, "HighlightTrack") || is(GdObject, "OverlayTrack")) {
        unlist(lapply(GdObject@trackList, .recChromosome))
    } else {
        chromosome(GdObject)
    }
    return(unique(chroms))
}





## Try to extract the (unique) genome information from a GRanges objects with the possibility to fall back to a default value
.getGenomeFromGRange <- function(range, default = NULL) {
    gn <- genome(range)
    if (length(unique(gn)) > 1) {
        warning("Only a single genome is supported for this object. Ignoring additional genome information")
    }
    if (length(gn) == 0 || all(is.na(gn))) {
        if (is.null(default)) {
            stop("A genome must be supplied when creating this object.")
        }
        return(default[1])
    }
    return(gn[1])
}


.updateObj <- function(object) {
    availSlots <- getObjectSlots(object)
    availSlotNames <- names(availSlots)
    definedSlotNames <- slotNames(object)
    if (length(availSlotNames) == length(definedSlotNames) && all(sort(availSlotNames) == sort(definedSlotNames))) {
        return(object)
    }
    commonSlots <- intersect(definedSlotNames, availSlotNames)
    missingSlots <- setdiff(definedSlotNames, availSlotNames)
    newObject <- new(class(object))
    for (s in commonSlots) {
        slot(newObject, s) <- availSlots[[s]]
    }
    return(newObject)
}



vpLocation <- function() {
    xres <- devRes()[1]
    yres <- devRes()[2]
    ## find location and pixel-size of current viewport
    devloc1 <- c(
        convertX(unit(0, "npc"), "inches"),
        convertY(unit(0, "npc"), "inches"), 1
    ) %*% current.transform()
    devloc2 <- c(
        convertX(unit(1, "npc"), "inches"),
        convertY(unit(1, "npc"), "inches"), 1
    ) %*% current.transform()
    x1 <- (devloc1 / devloc1[3])[1] * xres
    y1 <- (devloc1 / devloc1[3])[2] * yres
    x2 <- (devloc2 / devloc2[3])[1] * xres
    y2 <- (devloc2 / devloc2[3])[2] * yres
    loc <- c(x1, y1, x2, y2)
    names(loc) <- c("x1", "y1", "x2", "y2")
    size <- c(x2 - x1, y2 - y1)
    names(size) <- c("width", "height")
    iloc <- c(x1 / xres, y1 / yres, x2 / yres, y2 / yres)
    names(iloc) <- c("x1", "y1", "x2", "y2")
    isize <- size / c(xres, yres)
    names(size) <- c("width", "height")
    return(list(
        location = loc, size = size, ilocation = iloc,
        isize = isize
    ))
}



devRes <- function() {
    ## find R's resolution for the current device
    if (current.viewport()$name != "ROOT") {
        vpt <- current.vpTree()
        depth <- upViewport(0)
        xres <- abs(as.numeric(convertWidth(unit(1, "inches"), "native")))
        yres <- abs(as.numeric(convertHeight(unit(1, "inches"), "native")))
        downViewport(depth)
    } else {
        xres <- abs(as.numeric(convertWidth(unit(1, "inches"), "native")))
        yres <- abs(as.numeric(convertHeight(unit(1, "inches"), "native")))
    }
    retval <- c(xres, yres)
    names(retval) <- c("xres", "yres")
    return(retval)
}



devDims <- function(width, height, ncol = 12, nrow = 8, res = 72) {
    f <- (((ncol + 1) * 0.1 + ncol + 1) / ((nrow + 1) * 0.1 + nrow + 1))
    if ((missing(width) & missing(height) || !missing(width) & !missing(height))) {
        stop("Need either argument 'width' or argument 'height'")
    }
    if (missing(height)) {
        return(list(width = width, height = width / f, pwidth = width * res, pheight = width / f * res))
    } else {
        return(list(width = height * f, height, pwidth = height * f * res, pheight = height * res))
    }
}

## Record the display parameters for each class once

#' @importFrom utils assignInNamespace
.makeParMapping <- function() {
    classes <- c(
        "GdObject", "GenomeAxisTrack", "RangeTrack", "NumericTrack", "DataTrack", "IdeogramTrack", "StackedTrack",
        "AnnotationTrack", "DetailsAnnotationTrack", "GeneRegionTrack", "BiomartGeneRegionTrack", "AlignmentsTrack"
    )
    defs <- try(lapply(classes, function(x) as(getClassDef(x)@prototype@dp, "list")), silent = TRUE)
    if (!is(defs, "try-error") && is.null(.parMappings)) {
        names(defs) <- classes
        assignInNamespace(x = ".parMappings", value = defs, ns = "Gviz")
    }
}
.parMappings <- NULL



## Compute ellipse outline coordinates for bounding boxes
.box2Ellipse <- function(box, np = 50) {
    t <- seq(0, 2 * pi, len = np)
    box$width <- box$cx2 - box$cx1
    box$height <- box$cy2 - box$cy1
    x <- rep(box$cx2 + -box$width + box$width / 2, each = np)
    y <- rep(box$cy1 + box$height / 2, each = np)
    a <- rep(box$width / 2, each = np)
    b <- rep(box$height / 2, each = np)
    tau <- 0
    xt <- x + (a * cos(t) * cos(tau) - b * sin(t) * sin(tau))
    yt <- y + (a * cos(t) * sin(tau) + b * sin(t) * cos(tau))
    return(data.frame(x1 = xt, y1 = yt, id = rep(seq_len(nrow(box)), each = np)))
}

## We store some presets in the options on package load
.onLoad <- function(...) {
    .collectSchemes()
    options("ucscChromosomeNames" = TRUE, "Gviz.scheme" = "default", "Gviz.ucscUrl" = NULL)
}


## A helper function to replace missing function arguments in a list with NULL values. The function environment needs
## to be passed in as argument 'env' for this to work.
.missingToNull <- function(symbol, env = parent.frame()) {
    for (i in symbol) {
        mis <- try(do.call(missing, args = list(i), envir = env), silent = TRUE)
        if (!is(mis, "try-error") && mis) {
            assign(i, NULL, env)
        }
    }
}


## build a covariates data.frame from a variety of different inputs
.getCovars <- function(x) {
    if (is.data.frame(x)) {
        x
    } else {
        if (is(x, "GRanges")) {
            as.data.frame(mcols(x))
        } else {
            if (is(x, "GRangesList")) {
                as.data.frame(mcols(unlist(x)))
            } else {
                data.frame()
                ## stop(sprintf("Don't know how to extract covariates from a %s object", class(x)))
            }
        }
    }
}


## Prepare a data.frame or matrix containing the data for a DataTrack object. This involves trying to coerce
## and dropping non-numeric columns with a warning

#' @importFrom utils type.convert
.prepareDtData <- function(data, len = 0) {
    if (ncol(data) && nrow(data)) {
        for (i in seq_along(data)) {
            if (is.character(data[, i])) {
                data[, i] <- type.convert(data[, i], as.is = TRUE)
            }
        }
        isNum <- vapply(data, is.numeric, FUN.VALUE = logical(1L))
        if (any(!isNum)) {
            warning(sprintf(
                "The following non-numeric data column%s been dropped: %s", ifelse(sum(!isNum) > 1, "s have", " has"),
                paste(colnames(data)[!isNum], collapse = ", ")
            ))
        }
        if (sum(dim(data)) > 0) {
            data <- t(data[, isNum, drop = FALSE])
        }
    } else {
        data <- matrix(ncol = len, nrow = 0)
    }
    if (all(is.na(data))) {
        data <- matrix(ncol = len, nrow = 0)
    }
    if (ncol(data) != len) {
        stop("The columns in the 'data' matrix must match the genomic regions.")
    }
    return(data)
}

### Import functions ---------------------------------------------------------

## An import function for gff3 files that tries to resolve the parent-child relationship
## between genes, transcripts and exons
.import.gff3 <- function(file) {
    dat <- import.gff3(file)
    res <- try({
        genes <- tolower(dat$type) == "gene"
        ginfo <- mcols(dat[genes, ])
        dat <- dat[!genes]
        transcripts <- tolower(dat$type) == "mrna"
        tinfo <- mcols(dat[transcripts, ])
        dat <- dat[!transcripts]
        mt <- match(as.character(dat$Parent), as.character(tinfo$ID))
        if (!all(is.na(mt))) {
            if (!"transcript_id" %in% colnames(mcols(dat))) {
                tid <- rep(NA, length(mt))
                tid[!is.na(mt)] <- tinfo[mt[!is.na(mt)], "ID"]
                mcols(dat)[["transcript_id"]] <- tid
            }
            if (!"transcript_name" %in% colnames(mcols(dat))) {
                tn <- rep(NA, length(mt))
                tn[!is.na(mt)] <- tinfo[mt[!is.na(mt)], "Name"]
                mcols(dat)[["transcript_name"]] <- tn
            }
            mt2 <- rep(NA, dim(mcols(dat))[1])
            mt2[!is.na(mt)] <- match(as.character(tinfo[mt[!is.na(mt)], "Parent"]), as.character(ginfo$ID))

            if (!all(is.na(mt2))) {
                if (!"gene_id" %in% colnames(mcols(dat))) {
                    gid <- rep(NA, length(mt2))
                    gid[!is.na(mt2)] <- ginfo[mt[!is.na(mt2)], "ID"]
                    mcols(dat)[["gene_id"]] <- gid
                }
                if (!"gene_name" %in% colnames(mcols(dat))) {
                    gn <- rep(NA, length(mt2))
                    gn[!is.na(mt2)] <- ginfo[mt[!is.na(mt2)], "Name"]
                    mcols(dat)[["gene_name"]] <- gn
                }
            }
        }
        if (all(is.na(mcols(dat)[["ID"]]))) {
            mcols(dat)[["ID"]] <- paste("item", seq_along(dat), sep = "_")
        }
        if (!"exon_id" %in% colnames(mcols(dat))) {
            mcols(dat)[["exon_id"]] <- mcols(dat)[["ID"]]
        }
        if (!is.null(mcols(dat)[["gene_name"]]) && all(is.na(mcols(dat)[["gene_name"]]))) {
            mcols(dat)[["gene_name"]] <- NULL
        }
        if (all(is.na(mcols(dat)[["transcript_name"]]))) {
            mcols(dat)[["transcript_name"]] <- NULL
        }
        dat
    })
    if (is(res, "try-error")) {
        warning(sprintf(
            "File '%s' is not valid according to the GFF3 standard and can not be properly parsed.",
            file
        ), "\nResults may not be what you expected!")
        res <- dat
    }
    return(res)
}

## An import function for bigWig files that knowns how to deal with missing seqnames

#' @importFrom rtracklayer BigWigFile
.import.bw <- function(file, selection) {
    bwf <- BigWigFile(path.expand(file))
    if (missing(selection)) {
        rr <- import.bw(con = bwf)
    } else {
        si <- seqinfo(bwf)
        rr <- if (!as.character(seqnames(selection)[1]) %in% seqnames(seqinfo(bwf))) {
            GRanges(seqnames(selection)[1], ranges = IRanges(1, 2), score = 1)[0]
        } else {
            import.bw(con = bwf, selection = selection)
        }
    }
    return(rr)
}

## An import function for bam files that distinguishes between DataTracks and AnnotationTracks
## FIXME: We probably want this to be able to deal with Gapped Alignments...
.import.bam <- function(file, selection) {
    if (!file.exists(paste(file, "bai", sep = ".")) &&
        !file.exists(paste(paste(head(strsplit("xxx.bam", ".", fixed = TRUE)[[1]], -1), collapse = "."), "bai", sep = "."))) {
        stop(
            "Unable to find index for BAM file '", file, "'. You can build an index using the following command:\n\t",
            "library(Rsamtools)\n\tindexBam(\"", file, "\")"
        )
    }
    sinfo <- scanBamHeader(file)[[1]]
    if (parent.env(environment())[["._trackType"]] == "DataTrack") {
        res <- if (!as.character(seqnames(selection)[1]) %in% names(sinfo$targets)) {
            mcols(selection) <- DataFrame(score = 0)
            selection
        } else {
            param <- ScanBamParam(what = c("pos", "qwidth"), which = selection, flag = scanBamFlag(isUnmappedQuery = FALSE))
            x <- scanBam(file, param = param)[[1]]
            cov <- coverage(IRanges(x[["pos"]], width = x[["qwidth"]]))
            if (length(cov) == 0) {
                mcols(selection) <- DataFrame(score = 0)
                selection
            } else {
                GRanges(seqnames = seqnames(selection), ranges = IRanges(start = start(cov), end = end(cov)), strand = "*", score = runValue(cov))
            }
        }
    } else {
        res <- if (!as.character(seqnames(selection)[1]) %in% names(sinfo$targets)) {
            mcols(selection) <- DataFrame(id = "NA", group = "NA")
            selection[0]
        } else {
            param <- ScanBamParam(what = c("pos", "qwidth", "strand", "qname"), which = selection, flag = scanBamFlag(isUnmappedQuery = FALSE))
            x <- scanBam(file, param = param)[[1]]
            GRanges(
                seqnames = seqnames(selection), ranges = IRanges(start = x[["pos"]], width = x[["qwidth"]]), strand = x[["strand"]],
                id = make.unique(x[["qname"]]), group = x[["qname"]]
            )
        }
    }
    return(res)
}

#' @importClassesFrom Biostrings DNAStringSet RNAStringSet BStringSet DNAString RNAString BString
#' @importFrom Biostrings DNAStringSet RNAStringSet BStringSet DNAString RNAString BString reverseComplement readDNAStringSet DNA_ALPHABET stackStrings
#' @importFrom Rsamtools scanBamFlag scanBamHeader scanBam ScanBamParam scanFaIndex scanFa BamFile scanBamWhat bamWhich
#' @importFrom GenomicAlignments sequenceLayer
.import.bam.alignments <- function(file, selection) {
    indNames <- c(sub("\\.bam$", ".bai", file), paste(file, "bai", sep = "."))
    index <- NULL
    for (i in indNames) {
        if (file.exists(i)) {
            index <- i
            break
        }
    }
    if (is.null(index)) {
        stop(
            "Unable to find index for BAM file '", file, "'. You can build an index using the following command:\n\t",
            "library(Rsamtools)\n\tindexBam(\"", file, "\")"
        )
    }
    pairedEnd <- parent.env(environment())[["._isPaired"]]
    if (is.null(pairedEnd)) {
        pairedEnd <- TRUE
    }
    flag <- parent.env(environment())[["._flag"]]
    if (is.null(flag)) {
        flag <- scanBamFlag(isUnmappedQuery = FALSE)
    }
    bf <- BamFile(file, index = index, asMates = pairedEnd)
    param <- ScanBamParam(which = selection, what = scanBamWhat(), tag = "MD", flag = flag)
    reads <- if (as.character(seqnames(selection)[1]) %in% names(scanBamHeader(bf)$targets)) scanBam(bf, param = param)[[1]] else list()
    md <- if (is.null(reads$tag$MD)) rep(as.character(NA), length(reads$pos)) else reads$tag$MD
    if (length(reads$pos)) {
        layed_seq <- sequenceLayer(reads$seq, reads$cigar)
        region <- unlist(bamWhich(param), use.names = FALSE)
        ans <- stackStrings(layed_seq, start(region), end(region), shift = reads$pos - 1L, Lpadding.letter = "+", Rpadding.letter = "+")
        names(ans) <- seq_along(reads$qname)
    } else {
        ans <- DNAStringSet()
    }
    return(GRanges(
        seqnames = if (is.null(reads$rname)) character() else reads$rname,
        strand = if (is.null(reads$strand)) character() else reads$strand,
        ranges = IRanges(start = reads$pos, width = reads$qwidth),
        id = if (is.null(reads$qname)) character() else reads$qname,
        cigar = if (is.null(reads$cigar)) character() else reads$cigar,
        mapq = if (is.null(reads$mapq)) integer() else reads$mapq,
        flag = if (is.null(reads$flag)) integer() else reads$flag,
        md = md, seq = ans,
        isize = if (is.null(reads$isize)) integer() else reads$isize,
        groupid = if (pairedEnd) if (is.null(reads$groupid)) integer() else reads$groupid else seq_along(reads$pos),
        status = if (pairedEnd) {
            if (is.null(reads$mate_status)) factor(levels = c("mated", "ambiguous", "unmated")) else reads$mate_status
        } else {
            rep(
                factor("unmated", levels = c("mated", "ambiguous", "unmated")),
                length(reads$pos)
            )
        }
    ))
}


## An import function for fasta file that supports streaming if an index is present
#' @importFrom rtracklayer FastaFile
.import.fasta <- function(file, selection, strict = TRUE) {
    ffile <- FastaFile(file)
    if (!file.exists(paste(file, "fai", sep = "."))) {
        if (strict) {
            stop(
                "Unable to find index for fasta file '", file, "'. You can build an index using the following command:\n\t",
                "library(Rsamtools)\n\tindexFa(\"", file, "\")"
            )
        } else {
            return(readDNAStringSet(file))
        }
    }
    idx <- scanFaIndex(file)
    if (!as.character(seqnames(selection)[1]) %in% as.character(seqnames(idx))) {
        return(DNAStringSet())
    } else {
        return(scanFa(file, selection))
    }
}


## An import function for the indexed 2bit format
#' @importFrom rtracklayer TwoBitFile
.import.2bit <- function(file, selection) {
    tbf <- TwoBitFile(file)
    if (!as.character(seqnames(selection)[1]) %in% seqnames(seqinfo(tbf))) {
        return(DNAStringSet())
    } else {
        tmp <- import(tbf, which = selection)
        names(tmp) <- as.character(seqnames(selection)[1])
        return(tmp)
    }
}



## A mapping of (lower-cased) file extensions to import function calls. Most of those are already implemented in the rtracklayer package.
## If no mapping is found an error will be raised suggesting to provide a user-defined import function.
.registerImportFun <- function(file) {
    fileExt <- .fileExtension(file)
    file <- path.expand(file)
    return(switch(fileExt,
        "gff" = import.gff(file),
        "gff1" = import.gff1(file),
        "gff2" = import.gff2(file),
        "gff3" = .import.gff3(file),
        "gtf" = import.gff2(file),
        "bed" = import.bed(file),
        "bedgraph" = import.bedGraph(file),
        "wig" = import.wig(file),
        "bw" = .import.bw,
        "bigwig" = .import.bw,
        "bam" = .import.bam,
        stop(sprintf(
            "No predefined import function exists for files with extension '%s'. Please manually provide an import function.",
            fileExt
        ))
    ))
}


## Get the file extension for a file, taking into account potential gzipping
.fileExtension <- function(file) {
    if (!grepl("\\.", file)) {
        stop("Unable to identify extension for file '", file, "'")
    }
    ext <- sub(".*\\.", "", sub("\\.gz$|\\.gzip$", "", basename(file)))
    if (ext == "") {
        stop("Unable to identify extension for file '", file, "'")
    }
    return(tolower(ext))
}

#' @describeIn ReferenceTrack-class Function to find out whether the package
#' defines a mapping scheme between one of the many supported input file types
#' and the metadata columns of the tracks's `GRanges` objects.
#'
#' @param file A `character` scalar with a file name or just a file extension.
#' @param trackType A `character` scalar with one of the available track types
#' in the package.
#' @param stream stream
#' @param reference reference
#' @param mapping mapping
#' @param args ars
#' @param defaults defaults
#' @param .Object .Object
#'
#' @export
availableDefaultMapping <- function(file, trackType) {
    .checkClass(file, "character", 1)
    .checkClass(trackType, "character", 1)
    ext <- tolower(if (grepl("\\.", file)) .fileExtension(file) else file)
    vm <- .defaultVarMap(ext, trackType, justMap = TRUE)
    vm[[".stream"]] <- NULL
    return(vm)
}


## Helper function to handle defaults function arguments
.covDefault <- function(x, cov, def) {
    res <- if (missing(x)) {
        if (is.null(cov)) {
            def
        } else {
            cov
        }
    } else {
        x
    }
    return(res)
}


## A helper function to process alignment information from a GRanges object
#' @importFrom GenomicAlignments extractAlignmentRangesOnReference
.computeAlignments <- function(range, drop.D.ranges = FALSE) {
    res <- list(range = range, stackRanges = GRanges(), stacks = numeric())
    if (length(range)) {
        alg <- extractAlignmentRangesOnReference(range$cigar, drop.D.ranges = drop.D.ranges)
        rp <- elementNROWS(alg)
        if (is.null(range$flag)) range$flag <-  as.character(NA)
        range <- sort(GRanges(
            seqnames = rep(seqnames(range), rp), strand = rep("*", sum(rp)), ranges = shift(unlist(alg), rep(start(range), rp) - 1),
            id = rep(range$id, rp), entityId = rep(seq_along(rp), rp), cigar = rep(range$cigar, rp), md = rep(range$md, rp),
            readStrand = rep(strand(range), rp), mapq = rep(range$mapq, rp), flag = rep(range$flag, rp), isize = rep(range$isize, rp),
            groupid = rep(range$groupid, rp), status = factor(rep(range$status, rp), levels = c("mated", "ambiguous", "unmated")),
            uid = seq_len(sum(rp))
        ))
        if (length(range)) {
            stTmp <- split(range, range$groupid)
            stackRanges <- unlist(range(stTmp))
            ss <- disjointBins(stackRanges)
            range$stack <- ss[match(range$groupid, names(stackRanges))]
            res <- list(range = range, stackRanges = stackRanges, stacks = range$stack)
        }
    }
    return(res)
}

## Check whether the current device supports alpha channel transparency
#' @importFrom grDevices dev.cur dev.off
.supportsAlpha <- function() {
    d <- dev.cur()
    oldwarn <- getOption("warn")
    on.exit({
        options(warn = oldwarn)
        if (d == 1) {
            dev.off()
        }
    })
    options(warn = 2)
    ok <- !is(try(
        {
            grob <- grid.rect(x = unit(0, "npc"), y = unit(0, "npc"), width = 0, height = 0, gp = gpar(alpha = 0.5))
            ## grid.remove(grob$name)
        },
        silent = TRUE
    ), "try-error")
    return(ok)
}

## Return the alpha display parameter from a GdObject, respecting whether transparency is supported on the device
## or not. This is either drawn from the internal '.__hasAlphaSupport' display parameter, or, if not set, is
## determined dynamically.
.alpha <- function(GdObject, postfix = NULL) {
    support <- .dpOrDefault(GdObject, ".__hasAlphaSupport", .supportsAlpha())
    wh <- if (is.null(postfix)) "alpha" else c(paste("alpha", postfix, sep = "."), "alpha")
    alpha <- .dpOrDefault(GdObject, wh, 1)
    if (alpha != 1 && !support) {
        alpha <- 1
    }
    return(alpha)
}

## Draw horizontal arrows into a viewport indicating cropped read alignments
.moreInd <- function(n = 3, direction = "up", ...) {
    nn <- n * 2 + 1
    x <- rep(seq(1 / nn, 1 - (1 / nn), len = nn - 2)[seq(1, nn - 2, by = 2)], each = n) + c(-1 / nn / 2, 0, 1 / nn / 2)
    y <- rep(if (direction == "up") c(0, 1, 0) else c(1, 0, 1), n)
    grid.polyline(x, y, id = rep(seq_len(n), each = 3), gp = gpar(...))
}



## Compute mismatches for AlignmentsTracks based on the read sequences and a reference sequence
#' @importFrom matrixStats colMaxs
#' @importMethodsFrom Biostrings consensusMatrix consensusString
.findMismatches <- function(GdObject) {
    rgo <- .dpOrDefault(GdObject, ".__plottingRange")
    mmPos <- mmSamp <- mmSeq <- mmStack <- NULL
    if (!is.null(rgo)) {
        ref <- as.character(as(subseq(GdObject@referenceSequence, start = rgo["from"], end = rgo["to"]), "Rle"))
        cm <- consensusMatrix(GdObject@sequences, as.prob = FALSE, baseOnly = TRUE)[-5, ]
        cmm <- colMaxs(cm)
        css <- colSums(cm)
        cmp <- rbind(t(t(cm) / css), 0)
        rownames(cmp)[5] <- "N"
        sel <- is.na(cmp["A", ])
        cmp[, sel] <- 0
        cmp["N", sel] <- 1
        consStr <- strsplit(consensusString(cmp), "")[[1]]
        varRegs <- which(cmm != css | (consStr != "N" & consStr != ref))
        if (length(varRegs)) {
            rvg <- ref[varRegs]
            sel <- rvg != "-" & rvg != "N"
            if (any(sel)) {
                varRegs <- varRegs[sel]
                rvg <- rvg[sel]
                mmTab <- do.call(rbind, lapply(varRegs, function(x) as.character(subseq(GdObject@sequences, x, width = 1))))
                isMm <- t(rvg != "-" & mmTab != "+" & mmTab != "-" & mmTab != rvg)
                mmRelPos <- col(isMm)[which(isMm)]
                mmPos <- varRegs[mmRelPos] + rgo["from"] - 1
                mmSampInd <- row(isMm)[which(isMm)]
                mmSamp <- rownames(isMm)[mmSampInd]
                mmSeq <- mmTab[ncol(isMm) * (mmSampInd - 1) + mmRelPos]
                mmStack <- stacks(GdObject)[match(mmSamp, ranges(GdObject)$entityId)]
            }
        }
    }
    return(data.frame(position = mmPos, stack = mmStack, read = mmSamp, base = as.character(mmSeq), stringsAsFactors = TRUE))
}





## Return the default mappings between the metadata columns of an imported GRanges object and those
## of the track's GRanges object.

#' @importFrom stats setNames
.defaultVarMap <- function(inputType, trackType, stream, fromUser = FALSE, justMap = FALSE) {
    vm <- list(
        gtf = list(GeneRegionTrack = list(
            feature = "type",
            gene = c("gene_id", "gene_name"),
            exon = c("exon_name", "exon_id"),
            transcript = c("transcript_name", "transcript_id"),
            symbol = c("gene_name", "gene_id")
        )),
        gff = list(
            AnnotationTrack = list(
                feature = "type",
                group = "group"
            ),
            GeneRegionTrack = list(
                feature = "type",
                transcript = "group"
            )
        ),
        gff1 = list(
            AnnotationTrack = list(
                feature = "type",
                group = group
            ),
            GeneRegionTrack = list(
                feature = "type",
                transcript = "group"
            )
        ),
        gff2 = list(
            AnnotationTrack = list(
                feature = "type",
                group = c("group", "Parent"),
                id = c("ID", "Name", "Alias")
            ),
            GeneRegionTrack = list(
                feature = "type",
                gene = c("gene_id", "gene_name"),
                exon = c("exon_name", "exon_id"),
                symbol = c("gene_name", "gene_id")
            )
        ),
        gff3 = list(
            AnnotationTrack = list(
                feature = "type",
                id = c("ID", "Name", "Alias"),
                group = "Parent"
            ),
            GeneRegionTrack = list(
                feature = "type",
                gene = c("gene_id", "gene_name"),
                exon = c("exon_name", "exon_id", "ID"),
                transcript = c("transcript_name", "transcript_id", "Parent"),
                symbol = c("gene_name", "gene_id", "Name", "Alias")
            )
        ),
        bedgraph = list(DataTrack = list(score = "score")),
        wig = list(DataTrack = list(score = "score")),
        bed = list(AnnotationTrack = list(feature = "itemRgb", id = "name")),
        bigwig = list(DataTrack = list(
            score = "score",
            .stream = TRUE
        )),
        bw = list(DataTrack = list(
            score = "score",
            .stream = TRUE
        )),
        bam = list(
            DataTrack = list(
                score = "score",
                .stream = TRUE
            ),
            AnnotationTrack = list(
                id = "id",
                group = "group",
                .stream = TRUE
            ),
            AlignmentsTrack = list(
                id = "id",
                cigar = "cigar",
                mapq = "mapq",
                flag = "flag",
                isize = "isize",
                groupid = "groupid",
                status = "status",
                md = "md",
                seq = "seq",
                .stream = TRUE
            )
        )
    )
    if (justMap) {
        return(vm[[inputType]][[trackType]])
    }
    if (fromUser) {
        vm[[inputType]] <- setNames(list(list(".stream" = stream)), trackType)
    } else {
        if (is.null(vm[[inputType]]) || is.null(vm[[inputType]][[trackType]])) {
            warning(sprintf(
                "There are no default mappings from %s files to %s. Please provide a manual mapping",
                inputType, trackType
            ), " in the track constructor if you haven't already done so.")
            vm[[inputType]] <- setNames(list(list(".stream" = stream)), trackType)
        }
    }
    return(vm[[inputType]][[trackType]])
}


## Helper function to go through the metadata columns of a DataFrame and match their colnames to a mapping if they
## are available
.resolveColMapping <- function(data, args, defMap) {
    if (ncol(mcols(data))) {
        colnames(mcols(data)) <- paste(colnames(mcols(data)), "orig", sep = "__.__")
    }
    for (i in names(defMap)) {
        if (is.character(args[[i]]) && length(args[[i]]) == 1 && paste(args[[i]], "orig", sep = "__.__") %in% colnames(mcols(data))) {
            defMap[[i]] <- args[[i]]
            args[[i]] <- NULL
        }
        mt <- match(paste(defMap[[i]], "orig", sep = "__.__"), colnames(mcols(data)))
        mt <- mt[!is.na(mt)][1]
        if (!is.na(mt)) {
            mcols(data)[[i]] <- mcols(data)[, mt]
        }
    }
    mcols(data) <- mcols(data)[, !grepl("__.__", colnames(mcols(data))), drop = FALSE]
    return(list(data = data, args = args, defMap = defMap))
}


## For an AnnotationTrack or a GeneRegionTrack, compute the actual ranges for a complete range element group
## (i.e., a whole transcript or track group) and also add the necessary space for the text label if needed.
## We add this information to the internal display parameters '.__groupRanges', '.__groupLabels' and
## '.__groupLabelWidths'. This function has to be called before stacks are being computed because 'setStacks'
## will use the values in '.__groupRanges'.
## Note that the computed ranges are not quite right because we are only crudely guessing the size of the
## title panels at this stage.
.computeGroupRange <- function(GdObject, hasAxis = FALSE, hasTitle = .dpOrDefault(GdObject, "showTitle", TRUE), title.width = 1) {
    if (is(GdObject, "AnnotationTrack")) {
        finalRanges <- IRanges()
        GdObjectOrig <- GdObject
        GdObject <- GdObject[seqnames(GdObject) == chromosome(GdObject)]
        pr <- .dpOrDefault(GdObject, ".__plottingRange", data.frame(from = min(start(GdObject)), to = max(end(GdObject))))
        if (is.null(title.width)) {
            title.width <- 1
        }
        if (length(GdObject) > 0) {
            gp <- group(GdObject)
            if (!is.factor(gp)) {
                gp <- factor(gp, levels=unique(gp))
            }
            needsGrp <- any(duplicated(gp))
            finalRanges <- if (needsGrp) {
                groups <- split(range(GdObject), gp)
                unlist(range(groups))
            } else {
                range(GdObject)
            }
            if (.dpOrDefault(GdObject, ".__hasAnno", FALSE)) {
                ## The label justification
                just <- .dpOrDefault(GdObject, "just.group", "left")
                rev <- .dpOrDefault(GdObject, "reverseStrand", FALSE)
                ## A crude guestimate of the space needed for a title
                twidth <- if (hasTitle) {
                    fact <- title.width + (hasAxis * 2)
                    .getStringDims(GdObject, "g_T", unit = "npc", subtype = "title")$height * fact
                } else {
                    0
                }
                tfact <- ifelse(twidth > 1, 1, 1 / (1 - twidth))
                ## The labels and spacers are plotted in a temporary viewport to figure out their size
                labels <- if (needsGrp) {
                    vapply(split(identifier(GdObject), gp), function(x) paste(sort(unique(x)), collapse = "/"), FUN.VALUE = character(1L))
                } else {
                    setNames(identifier(GdObject), gp)
                }
                xscale <- c(max(pr["from"], min(start(finalRanges))), min(pr["to"], max(end(finalRanges))))
                if (diff(xscale) == 0) {
                    xscale[2] <- xscale[2] + 1
                }
                pushViewport(dataViewport(xscale = xscale, extension = 0, yscale = c(0, 1), gp = .fontGp(GdObject, "group")))
                labelWidths <- setNames(as.numeric(convertWidth(stringWidth(labels), "native")) * tfact * 1.3, names(labels))
                spaceBefore <- as.numeric(convertWidth(unit(3, "points"), "native")) * tfact
                spaceAfter <- as.numeric(convertWidth(unit(7, "points"), "native")) * tfact
                popViewport(1)
                switch(as.character(just),
                    "left" = {
                        if (!rev) {
                            start(finalRanges) <- start(finalRanges) - (spaceBefore + labelWidths + spaceAfter)
                        } else {
                            end(finalRanges) <- end(finalRanges) + spaceAfter + labelWidths + spaceBefore
                        }
                        sb <- spaceBefore
                        sa <- spaceAfter
                    },
                    "right" = {
                        if (!rev) {
                            end(finalRanges) <- end(finalRanges) + spaceAfter + labelWidths + spaceBefore
                        } else {
                            start(finalRanges) <- start(finalRanges) - (spaceBefore + labelWidths + spaceAfter)
                        }
                        sb <- spaceBefore
                        sa <- spaceAfter
                    },
                    "above" = {
                        featureWidths <- end(finalRanges) - start(finalRanges)
                        additionalLabelSpace <- ceiling((labelWidths - featureWidths) / 2)
                        additionalLabelSpace[additionalLabelSpace < 0] <- 0
                        end(finalRanges) <- end(finalRanges) + additionalLabelSpace
                        start(finalRanges) <- start(finalRanges) - additionalLabelSpace
                        sa <- sb <- 0
                    },
                    "below" = {
                        featureWidths <- end(finalRanges) - start(finalRanges)
                        additionalLabelSpace <- ceiling((labelWidths - featureWidths) / 2)
                        additionalLabelSpace[additionalLabelSpace < 0] <- 0
                        end(finalRanges) <- end(finalRanges) + additionalLabelSpace
                        start(finalRanges) <- start(finalRanges) - additionalLabelSpace
                        sa <- sb <- 0
                    },
                    stop(sprintf("Unknown label justification '%s'", just))
                )
                displayPars(GdObjectOrig) <- list(
                    ".__groupLabelWidths" = data.frame(before = sb, label = labelWidths, after = sa),
                    ".__groupLabels" = labels
                )
            }
        }
        displayPars(GdObjectOrig) <- list(".__groupRanges" = finalRanges)
    }
    return(GdObjectOrig)
}

## Calculate the vectorized string dimensions and return the results in a data.frame with columns 'width' and 'height'
## The unit of the return value can be controlled, and additional parameters like font size and expansion factors can
## be passed in as additional arguments (all in ... is passed on to 'gpar'). If needed, the font defaults for a
## subtype can be extracted by providing the subtype argument.
.getStringDims <- function(GdObject, string, unit = "native", subtype = NULL, ...) {
    gp <- .fontGp(GdObject, subtype, ...)
    pushViewport(viewport(gp = gp, xscale = current.viewport()$xscale, yscale = current.viewport()$yscale))
    res <- data.frame(
        width = as.numeric(convertWidth(stringWidth(string), unit)),
        height = as.numeric(convertHeight(stringHeight(string), unit))
    )
    popViewport(1)
    return(res)
}


## Check whether transcripts are to be collapsed for a GeneRegionTrack
.transcriptsAreCollapsed <- function(GdObject) {
    res <- FALSE
    if (is(GdObject, "GeneRegionTrack")) {
        ctrans <- .dpOrDefault(GdObject, "collapseTranscripts", FALSE)
        res <- (is.logical(ctrans) && ctrans == TRUE) || ctrans %in% c("gene", "shortest", "longest", "meta")
    }
    return(res)
}

## Create list for drawing sashimi-like plots
## using summarizeJunctions on GAlignments
## plotting is done via grid.xspline (requires x, y, id, score)
##
## id=range$id,
## entityId=range$entityId,
## md=range$md,
## readStrand=range$readStrand,
## mapq=range$mapq,
## flag=range$flag,
## isize=range$isize,
## groupid=range$groupid,
## status=range$status,
## uid=range$uid,
## stack=range$stack,

#' @importFrom GenomicAlignments GAlignments
.ranges2ga <- function(range) {
    range <- sort(range)
    range <- range[!duplicated(range$entityId)]
    ga <- GAlignments(
        seqnames = seqnames(range), pos = start(range), cigar = range$cigar,
        strand = if (is.null(range$readStrand)) strand(range) else range$readStrand,
        seqlengths = seqlengths(range)
    )
    ga
}

#' @importFrom GenomicAlignments summarizeJunctions
.create.summarizedJunctions.for.sashimi.junctions <- function(range) {
    ga <- .ranges2ga(range)
    juns <- summarizeJunctions(ga)
    juns
}

.convert.summarizedJunctions.to.sashimi.junctions <- function(juns, score = 1L, lwd.max = 10, strand = "*", filter = NULL, filterTolerance = 0L, trans = NULL) {
    ## filter junctions
    if (!is.null(filter)) {
        ## if filterTolerance is > 0 than pass it as maxgap parameter in findOverlaps
        ## make sure it is positive value
        if (filterTolerance < 0) {
            filterTolerance <- abs(filterTolerance)
            warning(sprintf(
                "\"sashimiFilterTolerance\" can't be negative, taking absolute value of it: %d",
                filterTolerance
            ))
        }
        ovs <- findOverlaps(juns, filter, type = "start", maxgap = filterTolerance)
        ove <- findOverlaps(juns, filter, type = "end", maxgap = filterTolerance)
        ## combine both Hits objects, select junctions present in both
        ovv <- rbind(as.matrix(ovs), as.matrix(ove))
        ovv <- ovv[duplicated(ovv), , drop = FALSE]
        ## create row/col index for selecting the correct strand
        ws <- strand(filter[ovv[, "subjectHits"]])
        levels(ws) <- c("plus_score", "minus_score", "score")
        ws <- cbind(row = ovv[, "queryHits"], col = as.character(ws))
        ## rol/col subseting will only works on matrix
        M <- as.matrix(values(juns))
        rownames(M) <- seq_len(nrow(M))
        ##
        filter$score <- 0L
        filter$score[sort(unique(ovv[, "subjectHits"]))] <- tapply(M[ws], ovv[, "subjectHits"], sum)
        juns <- filter
    } else {
        ## if no filter ranges were defined
        ## select strand (default both)
        juns$score <- if (strand == "+") juns$plus_score else if (strand == "-") juns$minus_score else juns$score
    }
    ## filter based on evidence (default no filtering, 1 read)
    juns <- juns[juns$score >= score]
    if (length(juns)) {
        ## count how many overlaps to determine the y
        ov <- findOverlaps(juns, reduce(juns, min.gapwidth = 0L))
        ov <- split(queryHits(ov), subjectHits(ov))
        juns$y <- as.integer(unlist(lapply(ov, order)))
        ## apply data transformation if one is set up
        if (is.list(trans)) {
            trans <- trans[[1]]
        }
        if (!is.null(trans)) {
            if (!is.function(trans) || length(formals(trans)) != 1L) {
                stop("Display parameter 'transformation' must be a function with a single argument")
            }
            test <- trans(juns$score)
            if (!is(test, "numeric") || length(test) != length(juns$score)) {
                stop(
                    "The function in display parameter 'transformation' results in invalid output.\n",
                    "It has to return a numeric matrix with the same dimensions as the input data."
                )
            }
            juns$score <- test
        }
        ## scale the score to lwd.max
        juns$scaled <- (lwd.max - 1) / pmax((max(juns$score) - min(c(1, juns$score))), 1) * (juns$score - max(juns$score)) + lwd.max
        ## create list
        juns <- list(
            x = as.numeric(rbind(
                start(juns),
                mid(ranges(juns)), end(juns)
            )),
            y = as.numeric(rbind(0, juns$y, 0)),
            id = rep(seq_len(length(juns)), each = 3),
            score = as.numeric(juns$score),
            scaled = juns$scaled
        )
    } else {
        juns <- list(
            x = numeric(),
            y = numeric(),
            id = integer(),
            score = numeric(),
            scaled = numeric()
        )
    }
    return(juns)
}


# temporary fix for providerVersion
#' @importMethodsFrom BSgenome providerVersion
.providerVersion <- function(sequence) {
    genome <- metadata(sequence)$genome
    if (is.null(genome)) {
        genome <- NA
    }
    return(genome)
}

#' @include ImageMap-class.R
NULL

### DisplayPars-class --------------------------------------------------------

## The initial idea was for the class to uses pass by reference semantic,
## allowing to update the object without reassigning to a symbol. However this
## turned out to be problematic whenever a `GdObject` was copied and to avoid
## confusion with the users about unwanted side-effects we decided to deprecate
## this feature.

#' DisplayPars: A class to control the plotting parameters for GdObjects
#'
#' @name DisplayPars-class
#'
#' @description All tracks within this package are highly customizable. The
#' `DisplayPars` class facilitates this and provides a unified API to the
#' customization parameters.
#'
#' The individual parameters in a `DisplayParameters` class are stored as
#' pointers in an environment. This has the upshot of not having to copy the
#' whole track object when changing parameters, and parameters can be updated
#' without the need to explicitly reassign the track to a symbol (i.e.,
#' updating of parameters happens in place). The downside is that upon copying
#' of track objects, the parameter environment needs to be re-instantiated.
#'
#' Objects can be created using the constructor function `DisplayPars`.
#'
#' The default display parameters for a track object class can be queried using
#' the `availableDisplayPars` function.
#'
#' @slot pars an environment or a list containing parameter key value pairs.
#'
#' @template DisplayParams_param
#'
#' @return The return value of the constructor function is a new object of class
#' `DisplayPars`.
#'
#' @author Florian Hahne
#' @examples
#' ## Construct object
#' dp <- DisplayPars(col = "red", lwd = 2, transformation = log2)
#' dp
#'
#' ## Query parameters
#' displayPars(dp)
#' displayPars(dp, "col")
#' getPar(dp, c("col", "transformation"))
#'
#' ## Modify parameters
#' displayPars(dp) <- list(lty = 1, fontsize = 3)
#' setPar(dp, "pch", 20)
#' dp
#' @export
.DisplayPars <- setClass("DisplayPars", representation(pars = "ListOrEnv"))

### DisplayPars Constructor --------------------------------------------------

## The initializer needs to create the environment, we can't take this
## directly from the class prototype since this would result in using
## the same environment all the time

# #' @describeIn DisplayPars-class Initialize.
# #' @export
# setMethod("initialize", "DisplayPars", function(.Object, ...) {
#     e <- new.env(hash = TRUE)
#     args <- list(...)
#     n <- names(args)
#     if ((is.null(n) & length(args) == 1) || any(n == "")) {
#         stop("All supplied arguments must be named.")
#     }
#     .Object@pars <- args
#     return(.Object)
# })

## Constructor, all supplied arguments are added to the environment.

#' @describeIn DisplayPars-class Constructor function.
#' @export
DisplayPars <- function(...) {
    e <- new.env(hash = TRUE)
    args <- list(...)
    n <- names(args)
    if ((is.null(n) & length(args) == 1) || any(n == "")) {
        stop("All supplied arguments must be named.")
    }
    .DisplayPars(pars = args)
}

### DisplayPars Methods ------------------------------------------------------

## Update function and deprecation message to show that an old environment-based
## DisplayParameter object has been updated to a list-based object.
.updateDp <- function(x, interactive = TRUE) {
    if (interactive) {
        message(
            "Note that the behaviour of the 'setPar' method has changed. ",
            "You need to reassign the result to an object for the side effects ",
            "to happen. Pass-by-reference semantic is no longer supported."
        )
    }
    if (is.environment(x@pars)) {
        x@pars <- as.list(x@pars)
        message("The DisplayPars object has been updated to a list-based representation.")
    }
    return(x)
}

## The accessor methods for the DisplayPars class. (they need to be in
## here because they are being called in the prototypes). For the
## setter method, the input can either be a named list, or a single
## keyword/value pair. The DisplayPars class was first implemented as
## an environment, so we essentially had pass by reference semantic here,
## and the object could be modified without the need to reassign to a
## symbol. However this somewhat broke the the R paradigm, and 'setPar'
## has been deprecated in favour of the more standard 'DisplayPars<-'
## replacement method. The getter methods either return the list
## of all parameters, or a subset of parameters if their names are
## provided as a character vector.  Please note that for convenience
## the result is unlisted if only a single parameter is queried.
## Since version 1.7.3 we have introduced an alias table for display
## parameters in order to harmonize parameter names without loosing
## backwards compatibility.

### DisplayPars Methods Getters ----------------------------------------------

#' @describeIn DisplayPars-class Generics for `getPar`.
setGeneric("getPar", def = function(x, name, ...) standardGeneric("getPar"))

#' @describeIn DisplayPars-class Alias for the `displayPars` method.
#' @export
setMethod(
    "getPar", c("DisplayPars", "character"),
    function(x, name, asIs = FALSE) {
        aliasRes <- .dpAliasReverseTable[name]
        new <- is.na(aliasRes)
        aliasRes[new] <- name[new]
        if (is.environment(x@pars)) {
            name <- intersect(aliasRes, base::ls(x@pars, all.names = TRUE))
            tmp <- mget(name, x@pars)
        } else {
            name <- intersect(aliasRes, names(x@pars))
            tmp <- x@pars[name]
        }
        if (!asIs) {
            tmp <- if (is.list(tmp) && length(tmp) == 1) tmp[[1]] else tmp
        }
        return(if (length(tmp)) tmp else NULL)
    }
)

#' @describeIn DisplayPars-class Alias for the `displayPars` method.
#' @export
setMethod("getPar", c("DisplayPars", "missing"), function(x, hideInternal = TRUE) {
    pars <- as.list(x@pars)
    if (hideInternal) {
        pars <- pars[!grepl("^\\.__", names(pars))]
    }
    return(pars)
})

#' @describeIn DisplayPars-class Generics for `displayPars`.
setGeneric("displayPars", function(x, name, ...) standardGeneric("displayPars"))

#' @describeIn DisplayPars-class Returns all available display parameters.
#' @export
setMethod("displayPars", c("DisplayPars", "missing"), function(x, hideInternal = TRUE) getPar(x, hideInternal = hideInternal))

#' @describeIn DisplayPars-class Returns the value of a subset of display parameters, as identified by `name`.
#' @export
setMethod("displayPars", c("DisplayPars", "character"), function(x, name) getPar(x, name))

#' @describeIn DisplayPars-class Converts `DisplayPars` to `list`.
#' @export
setMethod("as.list", "DisplayPars", function(x) as(x, "list"))

setAs("DisplayPars", "list", function(from, to) if (!is.null(from)) as.list(from@pars) else list())

### DisplayPars Methods Setters ----------------------------------------------

#' @describeIn DisplayPars-class Generics for `SetPar`.
#'
#' SetPar generic function
#'
#' @param x object to set the displayPar value on
#' @param value named value to be set
#'
setGeneric("setPar", function(x, value, ...) standardGeneric("setPar"))

#' @describeIn DisplayPars-class Sets display parameters by the values of the
#' named `list` in value. Note that display parameters in the `DisplayPars-class`
#' are pass-by-reference, so no re-assignment to the symbol `obj` is necessary.
setMethod(
    "setPar", signature("DisplayPars", "list"),
    function(x, value, interactive = TRUE) {
        x <- .updateDp(x, interactive)
        aliasRes <- .dpAliasReverseTable[names(value)]
        new <- is.na(aliasRes)
        aliasRes[new] <- names(value)[new]
        x@pars[aliasRes] <- value
        return(x)
    }
)

#' @describeIn DisplayPars-class set the single display parameter name to value.
#' Note that display parameters in the DisplayPars class are pass-by-reference,
#' so no re-assignment to the symbol `obj` is necessary.
#' @export
setMethod(
    "setPar", signature("DisplayPars", "character"),
    function(x, name, value, interactive = TRUE) {
        if (!(length(name) == 1 && is.null(value)) && length(name) != length(value)) {
            stop("'name' and 'value' must be of equal length")
        }
        x <- .updateDp(x, interactive)
        for (i in seq_along(name)) {
            aliasRes <- .dpAliasReverseTable[name[i]]
            if (is.na(aliasRes)) {
                aliasRes <- name[i]
            }
            x@pars[[aliasRes]] <- value[[i]]
        }
        return(x)
    }
)

#' @describeIn DisplayPars-class Generics for `displayPars<-`.
setGeneric("displayPars<-",
    signature = c("x", "value"),
    function(x, recursive = FALSE, value) standardGeneric("displayPars<-")
)

#' @describeIn DisplayPars-class  Replaces or adds display parameters as provided by the named `list` items.
#' @export
setReplaceMethod("displayPars", signature("DisplayPars", "list"), function(x, recursive = FALSE, value) {
    x <- setPar(x, value, interactive = FALSE)
    return(x)
})

### Display parameters lookup table ------------------------------------------

## An alias table to define display parameter synonyms. Essentially this is a
## simple named list, where the element names represent the preferred parameter
## name under which the actual value is stored, and the element content is a
## character vector of synonyms. Please note that this structure is also turned
## into a reverse lookup table for fast access.

.dpAliasTable <- list(
    "fontcolor.title" = "col.title",
    "rotation.title" = c("rot.title", "rotate.title"),
    "fontcolor.group" = "col.group",
    "fontcolor.item" = "col.item",
    "just.group" = c("labelJust", "labelJustification"),
    "transcriptAnnotation" = "geneSymbols",
    "fontcolor.item" = c("fontcolor.exon", "fontcolor.feature"),
    "fontsize.item" = c("fontsize.exon", "fontsize.feature"),
    "fontfamily.item" = c("fontfamily.exon", "fontfamily.feature"),
    "fontface.item" = c("fontface.exon", "fontface.feature"),
    "lineheight.item" = c("lineheight.exon", "lineheight.feature"),
    "alpha.item" = c("alpha.exon", "alpha.feature"),
    "cex.item" = c("cex.exon", "cex.feature"),
    "col.mate" = c("col.mate", "col.mates", "col.pair", "col.pairs"),
    "lty.mate" = c("lty.mate", "lty.mates", "lty.pair", "lty.pairs"),
    "lwd.mate" = c("lwd.mate", "lwd.mates", "lwd.pair", "lwd.pairs"),
    "alpha.mate" = c("alpha.mate", "alpha.mates", "alpha.pair", "alpha.pairs"),
    "col.gap" = "col.gaps",
    "lwd.gap" = "lwd.gaps",
    "lty.gap" = "lty.gaps",
    "alpha.gap" = "alpha.gaps"
)
.dpAliasReverseTable <- character()
.dpAliasReverseTable[names(.dpAliasTable)] <- names(.dpAliasTable)
#' @importFrom Biobase listLen
.dpAliasReverseTable[unlist(.dpAliasTable, use.names = FALSE)] <- rep(names(.dpAliasTable), Biobase::listLen(.dpAliasTable))
.dpAliasReverseTable <- .dpAliasReverseTable[order(names(.dpAliasReverseTable))]

## Show ----------------------------------------------------------------------

#' @describeIn DisplayPars-class Show method.
#' @export
setMethod("show", "DisplayPars", function(object) {
    cat("Display parameters:\n")
    for (i in base::ls(object@pars))
    {
        cat(i, " = ", sep = "")
        o <- try(as.character(object@pars[[i]]), silent = TRUE)
        if (is(o, "try-error")) print(object@pars[[i]]) else cat(o, "\n")
    }
})

### InferredDisplayPars-class ------------------------------------------------

#' InferredDisplayPars-class
#'
#' A class to allow for querying of available display parameters.
#' Essentially this is a normal list with a bit of a fancier show method.
#'
#' @slot name character. The name of the class.
#' @slot inheritance character. A vector indicating the inheritance structure.
#'
#' @return list
#' @noRd
#' @keywords internal
setClass("InferredDisplayPars", representation(name = "character", inheritance = "character"), contains = "list")

#' @describeIn DisplayPars-class  Return `InferredDisplayPars` as a `list`.
#' @export
setMethod("as.list", "InferredDisplayPars", function(x) as(x, "list"))

setAs(
    "InferredDisplayPars", "list",
    function(from, to) {
        ll <- from@.Data
        names(ll) <- names(from)
        ll
    }
)

## Show ----------------------------------------------------------------------

#' @describeIn DisplayPars-class  Show method.
#' @export
setMethod("show", "InferredDisplayPars", function(object) {
    cat("\nThe following display parameters are available for '", object@name, "' objects:\n",
        "(see ? ", object@name, " for details on their usage)\n\n",
        sep = ""
    )
    for (i in names(object)) {
        cat(i, ifelse(object@inheritance[i] == object@name, "",
            paste(" (inherited from class '", object@inheritance[i], "')", sep = "")
        ),
        ": ",
        sep = ""
        )
        if (is.null(object[[i]])) {
            cat("NULL\n")
        } else {
            o <- try(as.character(object[[i]]), silent = TRUE)
            if (is(o, "try-error")) print(object[[i]]) else cat(o, "\n")
        }
    }
})

### availableDisplayPars function --------------------------------------------

#' @param class Either character scalar or object. Supported classes are:
#' `GdObject`, `GenomeAxisTrack`, `RangeTrack`, `NumericTrack`, `DataTrack`,
#' `IdeogramTrack`, `StackedTrack`, `AnnotationTrack`, `DetailsAnnotationTrack`,
#' `GeneRegionTrack`, `BiomartGeneRegionTrack`, `AlignmentsTrack`,
#' `SequenceTrack`, `SequenceBSgenomeTrack`, `SequenceDNAStringSetTrack`,
#' `SequenceRNAStringSetTrack`
#'
#' @return `availableDisplayPars` returns a list of the default display
#' parameters.
#'
#' @examples
#' ## Default parameters
#' availableDisplayPars("GenomeAxisTrack")
#' @describeIn DisplayPars-class Get default display parameters.
#' @export
availableDisplayPars <- function(class) {
    if (!is.character(class)) {
        class <- class(class)
    }
    class <- match.arg(class, c(
        "GdObject", "GenomeAxisTrack", "RangeTrack", "NumericTrack", "DataTrack",
        "IdeogramTrack", "StackedTrack", "AnnotationTrack", "DetailsAnnotationTrack",
        "GeneRegionTrack", "BiomartGeneRegionTrack", "AlignmentsTrack",
        "SequenceTrack", "SequenceBSgenomeTrack", "SequenceDNAStringSetTrack",
        "SequenceRNAStringSetTrack"
    ))
    parents <- names(getClassDef(class)@contains)
    .makeParMapping()
    pars <- .parMappings[c(parents, class)]
    finalPars <- inherited <- list()
    for (p in names(pars)) {
        finalPars[names(pars[[p]])] <- pars[[p]]
        inherited[names(pars[[p]])] <- p
    }
    finalPars <- finalPars[order(names(finalPars))]
    inherited <- inherited[order(names(inherited))]
    return(new("InferredDisplayPars",
        name = class,
        inheritance = unlist(inherited), finalPars
    ))
}

## smart character coercion
toChar <- function(x) {
    if (length(x) > 1) {
          return(sprintf("c(\"%s\")", paste(x, collapse = "\", \"")))
      }
    if (is.null(x)) {
        return("NULL")
    }
    if (is.function(x)) {
        return(substitute(x))
    }
    if (is.numeric(x)) {
        return(x)
    }
    if (is.logical(x)) {
        return(as.character(x))
    } else {
        return(sprintf("\"%s\"", as.character(x)))
    }
}

## Find the best line break point in a text
.findBestBreak <- function(x, chars = 70) {
    if (nchar(x) <= chars) {
          return(x)
      }
    xs <- strsplit(x, " ")[[1]]
    ind <- which.min(abs(cumsum(nchar(xs) + 1) - chars))
    xres <- paste(xs[1:ind], collapse = " ", sep = "")
    if (length(xs) >= (ind + 1)) {
          xres <- c(xres, paste(xs[(ind + 1):length(xs)], collapse = " ", sep = ""))
      }
    return(xres)
}

## Emacs-style code formatting
indent <- function(x, level = 0, block = TRUE, chars = 70) {
    space <- "  "
    level <- rep(level, length(x))[1:length(x)]
    indent <- sapply(level, function(y) paste(rep(space, y), collapse = ""))
    indent2 <- as.vector(sapply(indent, function(y) paste(y, rep(space, as.integer(!block)), collapse = "", sep = "")))
    xc <- mapply(function(y, z) {
        paste(y, gsub(" {2,}", " ", gsub(" *\\}", "}", gsub("\\{ *", "{", gsub("\n+", "", z)))),
            collapse = "", sep = ""
        )
    }, indent, x)
    res <- mapply(function(y, z) {
        tmp <- .findBestBreak(y)
        xres <- tmp[1]
        while (length(tmp) == 2 & nchar(tmp[2]) > 68) {
            tmp <- .findBestBreak(paste(z, tmp[2], collapse = "", sep = ""), chars = chars)
            xres <- c(xres, tmp[1])
        }
        if (length(tmp) == 2) {
              xres <- c(xres, paste(z, tmp[2], collapse = "", sep = ""))
          }
        return(paste(xres, collapse = "\n", sep = ""))
    }, xc, indent2)
    return(res)
}


.tag <- function(x) {
    tmp <- sub("\\\\", "", attr(x, "Rd_tag"))
    if (!length(tmp)) {
        return(NA)
    } else {
        return(tmp)
    }
}


.tags <- function(x) sapply(x, .tag)


.tagValue <- function(x) {
    if (is.na(.tag(x))) {
        return(x)
    }
    if (.tag(x) == "TEXT") {
        return(as.character(x))
    }
    if (length(x) == 1 && length(.tag(x[[1]]))) {
        return(x[[1]])
    }
    if (length(x) == 1 && !length(.tag(x[[1]]))) {
        return(as.character(x))
    }
    if (length(x) > 1 && length(.tags(x)[!is.na(.tags(x))]) == length(x)) {
        return(x)
    }
    if (length(x) == 2 && all(is.na(.tags(x)))) {
        attr(x[[1]], "Rd_tag") <- "_sectionContent"
        attr(x[[2]], "Rd_tag") <- "_sectionContent"
        return(x)
    }
}


.traverseRd <- function(x, tag, output = NULL) {
    tag <- tolower(tag)
    if (!is.na(.tag(x)) && tag == tolower(.tag(x))) {
        output <- c(output, .tagValue(x))
    }
    for (child in x)
    {
        thisTag <- .tag(child)
        if (!is.na(thisTag)) {
            if (tag == tolower(thisTag)) {
                output <- c(output, .tagValue(child))
            }
            output <- .traverseRd(.tagValue(child), tag, output)
        }
    }
    return(output)
}


renderParsEntry <- function(class, pars, level = 2) {
    det <- details[[class]][names(pars)]
    if (is.null(det)) {
        warning("No details available for class '", class, "'")
        det <- "FIXME: PLEASE ADD PARAMETER DESCRIPTION."
    } else {
        det[is.na(det)] <- "FIXME: PLEASE ADD PARAMETER DESCRIPTION."
    }
    if (any(is.na(det))) {
          warning("There are details missing for class '", class, "'. Please update the documentation")
      }
    pval <- sapply(pars, toChar)
    pvalClean <- pval
    pvalClean[is.na(pval)] <- ""
    pvalClean[sapply(pars, is.function)] <- "function"
    aliases <- sapply(names(pars), function(x) paste(Gviz:::.dpAliasReverseTable[x], collapse = ", "))
    alText <- rep("", length(aliases))
    sel <- aliases != "NA"
    alText[sel] <- sprintf(" \\code{(Aliases: %s)}", aliases[sel])
    return(indent(sprintf(
        "\\item{}{\\code{%s%s%s}%s: %s}\n\n", names(pars),
        ifelse(is.na(pval), "", "="), pvalClean, alText, det
    ),
    level = level, block = FALSE
    ))
}


## create a documentation skeleton for the display parameters
displayParsDoc <- function(class, details) {
    combinations <- list("AnnotationTrack" = c("DetailsAnnotationTrack"))
    parents <- names(getClassDef(class)@contains)
    pars <- sapply(c(class, parents), function(x) try(as.list(getClassDef(x)@prototype@dp), silent = TRUE), simplify = FALSE)
    pars <- pars[!sapply(pars, is, "try-error")]
    text <- c("\\section{Display Parameters}{", if (length(pars[[1]])) {
        pars[[1]] <- pars[[1]][order(names(pars[[1]]))]
        if ("..." %in% names(details[[class]])) {
              pars[[1]][["..."]] <- NA
          }
        det <- details[[class]][names(pars[[1]])]
        if (is.null(det)) {
            warning("No details available for class '", class, "'")
            det <- "FIXME: PLEASE ADD PARAMETER DESCRIPTION."
        } else {
            det[is.na(det)] <- "FIXME: PLEASE ADD PARAMETER DESCRIPTION."
        }
        if (any(is.na(det))) {
              warning("There are details missing for class '", class, "'. Please update the documentation")
          }
        pval <- sapply(pars[[1]], toChar)
        pvalClean <- pval
        pvalClean[is.na(pval)] <- ""
        pvalClean[sapply(pars[[1]], is.function)] <- "function"
        aliases <- sapply(names(pars[[1]]), function(x) paste(Gviz:::.dpAliasReverseTable[x], collapse = ", "))
        alText <- rep("", length(aliases))
        sel <- aliases != "NA"
        alText[sel] <- sprintf(" \\code{(Aliases: %s)}", aliases[sel])
        c(
            indent(paste("The following display parameters are set for objects of class \\code{",
                class, "} upon instantiation, unless one or more of them have already been set ",
                "by one of the optional sub-class initializers, which always get precedence over ",
                "these global defaults. See \\code{\\link{settings}} for details on ",
                "setting graphical parameters for tracks.\n\n  \\describe{\n",
                sep = ""
            ), level = 1),
            renderParsEntry(class, pars[[1]]), indent("}", level = 1)
        )
    } else {
        indent(paste("No formal display parameters are defined for objects of class \\code{",
            class, "}.\n",
            sep = ""
        ), level = 1:2)
    })
    if (!is.null(combinations[[class]])) {
        for (sc in combinations[[class]]) {
              text <- c(
                  text,
                  indent(sprintf(
                      "\\code{%s} adds the following additional display parameters:\n\n  \\describe{\n",
                      sc
                  ), level = 1),
                  renderParsEntry(sc, as.list(getClassDef(sc)@prototype@dp)), indent("}", level = 1)
              )
          }
    }
    pp <- pars[-1]
    pp <- pp[sapply(pp, length) > 0]
    done <- names(pars[[1]])
    if (!is.null(pp) && length(pp) > 0) {
        text <- c(text, indent(c(paste("Additional display parameters are being inherited from the respective parent ",
            "classes. Note that not all of them may have an effect on the plotting of  \\code{",
            class, "} objects.",
            sep = ""
        ), "\\describe{"), level = 1:2))
        for (i in names(pp))
        {
            leftovers <- setdiff(names(pp[[i]]), done)
            done <- union(names(pp[[i]]), done)
            if (length(leftovers)) {
                pp[[i]] <- pp[[i]][leftovers][order(names(pp[[i]][leftovers]))]
                text <- c(
                    text, indent(c(sprintf("\\item{}{\\code{\\linkS4class{%s}}:", i), "\\describe{"),
                        level = 2:3
                    ),
                    renderParsEntry(i, pp[[i]], level = 4),
                    indent(rep("}", 2), level = 3:2)
                )
            }
        }
        text <- c(text, indent("}", 1))
    }
    text <- c(text, indent("}", 0))
    text <- sub("\n*$", "", paste(text, collapse = " \n\n"))
    parsDets <- sapply(union(names(pars[[1]]), names(details[[class]])), function(x) {
        txt <- if (x %in% names(details[[class]])) details[[class]][[x]] else NA
        if (!is.na(txt)) {
              txt <- trimws(gsub("\\s+", " ", gsub(
                  "\n", "",
                  paste(suppressWarnings(unlist(parse_Rd(textConnection(txt)))), collapse = "")
              )))
          }
        list(
            id = x,
            default = pars[[1]][[x]],
            text = txt
        )
    }, simplify = FALSE)
    attr(text, "parameters") <- parsDets
    return(text)
}

extractDpContent <- function(file) {
    require(Gviz)
    section <- "Display Parameters"
    require(tools)
    tmp <- tryCatch(parse_Rd(file), warning = function(x) stop("Error parsing rd file:\n", x))
    tags <- sapply(tmp, attr, "Rd_tag")
    ind1 <- grep(section, tags, ignore.case = TRUE)
    ind <- if (!length(ind1)) {
        sind <- grep("\\\\section", tags, ignore.case = TRUE)
        sind[grep(section, sapply(tmp[sind], function(x) as.character(x[[1]])), ignore.case = TRUE)]
    } else {
        ind1
    }
    if (length(ind) > 1) {
          stop("Section '", section, "'is not unique in file '", file, "'")
      }
    dind <- grep("\\\\describe", sapply(tmp[[ind]][[2]], attr, "Rd_tag"))
    if (!length(dind)) {
          stop("Unable to find Display Parameters description section in file '", file, "'")
      }
    dind <- min(dind)
    desc <- tmp[[ind]][[2]][[dind]]
    itemInds <- grep("\\\\item", sapply(desc, attr, "Rd_tag"))
    items <- lapply(itemInds, function(i) {
        istruct <- unlist(unlist(desc[[i]], recursive = FALSE), recursive = FALSE)
        sel <- sapply(sapply(istruct, function(x) attr(x, "Rd_tag")), function(x) !is.null(x) && x == "RCODE")
        sel <- sort(reduce(IRanges(start = which(sel), width = 1)))
        sel <- seq(from = start(sel[1]), len = width(sel[1]))
        def <- gsub("\n", "", paste(as.character(unlist(istruct[sel])), collapse = ""))
        def2 <- gsub(".*=", "", def)
        item <- unlist(desc[[i]])
        item <- item[!grepl("^\\(Aliases:", item)]
        def <- if (def2 != "function") try(if (grepl("=", def)) eval(parse(text = def2)) else NA, silent = TRUE) else def2
        ## if(gsub("=.*", "", item[1])=="stats") browser()
        if (is(def, "try-error")) {
              def <- gsub(".*=", "", gsub("\n", "", paste(as.character(unlist(istruct[sel])), collapse = "")))
          }
        list(
            id = gsub("=.*", "", item[1]),
            default = def,
            text = trimws(gsub("\\s+", " ", gsub("\n", "", gsub("^: ?| :\\n *|.*): ", "", trimws(paste(item[-sel], collapse = ""))))))
        )
    })
    names(items) <- sapply(items, function(x) x[["id"]])
    return(items)
}


compareDtContent <- function(class, outdir, details) {
    file <- file.path(outdir, paste(class, "class.Rd", sep = "-"))
    fromFile <- extractDpContent(file)
    fromDef <- attr(displayParsDoc(class, details), "parameters")
    allPars <- union(names(fromFile), names(fromDef))
    res <- list()
    tmp <- setdiff(allPars, names(fromFile))
    alias <- Gviz:::.dpAliasReverseTable[tmp]
    alias <- alias[!is.na(alias)]
    tmp <- setdiff(tmp, alias)
    res[["Parameter missing in file"]] <- tmp
    tmp <- setdiff(allPars, names(fromDef))
    if (length(tmp)) {
          tmp <- lapply(tmp, function(x) c(x, fromFile[[x]][["text"]]))
      }
    res[["Parameter missing in definitions"]] <- tmp
    tmp <- setdiff(allPars, names(details[[class]]))
    alias <- Gviz:::.dpAliasReverseTable[tmp]
    alias <- alias[!is.na(alias)]
    tmp <- setdiff(tmp, alias)
    res[["No parameter entry in documentation.R"]] <- tmp
    tmp <- setdiff(allPars, names(as.list(getClassDef(class)@prototype@dp)))
    alias <- Gviz:::.dpAliasReverseTable[tmp]
    alias <- alias[!is.na(alias)]
    tmp <- setdiff(tmp, c(alias, "..."))
    res[["Parameter not initialized in class"]] <- tmp
    commonPars <- intersect(names(fromFile), names(fromDef))
    tmp <- sapply(commonPars, function(x) {
        if (is.character(fromFile[[x]][["default"]]) && fromFile[[x]][["default"]][1] == "function" &&
            is.function(fromDef[[x]][["default"]])) {
              return(NULL)
          }
        ae <- all.equal(fromFile[[x]][["default"]], fromDef[[x]][["default"]])
        return(if (length(ae) == 1 && ae == TRUE) {
            NULL
        } else {
              c(as.list(ae), file = fromFile[[x]][["default"]], def = fromDef[[x]][["default"]])
          })
    }, simplify = FALSE)
    res[["Parameter default difference"]] <- tmp[!sapply(tmp, is.null)]
    tmp <- sapply(commonPars, function(x) {
        ae <- all.equal(fromFile[[x]][["text"]], fromDef[[x]][["text"]])
        return(if (length(ae) == 1 && ae == TRUE) NULL else c(ae, file = fromFile[[x]][["text"]], def = fromDef[[x]][["text"]]))
    }, simplify = FALSE)
    res[["Parameter text difference"]] <- tmp[!sapply(tmp, is.null)]
    return(res)
}


## Parse though an Rd file, find 'section' in there and replace its content by 'content'.
## If there are parse errors in the Rd file either before or after injection the file
## will not be altered.
injectContent <- function(file, content, section) {
    require(Gviz)
    require(tools)
    tmp <- tryCatch(parse_Rd(file), warning = function(x) stop("Error parsing rd file:\n", x))
    tags <- sapply(tmp, attr, "Rd_tag")
    ind1 <- grep(section, tags, ignore.case = TRUE)
    ind <- if (!length(ind1)) {
        sind <- grep("\\\\section", tags, ignore.case = TRUE)
        sind[grep(section, sapply(tmp[sind], function(x) as.character(x[[1]])), ignore.case = TRUE)]
    } else {
        ind1
    }
    if (length(ind) > 1) {
          stop("Section '", section, "'is not unique in file '", file, "'")
      }
    if (!length(ind)) tmp <- c(tmp, list(content)) else tmp[[ind]] <- content
    class(tmp) <- "Rd"
    file.copy(file, file.path(dirname(file), paste("~", basename(file), sep = "")), overwrite = TRUE)
    writeLines(sub("\n*$", "", paste(as.character(as(tmp, "Rd")), collapse = "")), file)
    trash <- tryCatch(parse_Rd(file), warning = function(x) {
        file.copy(file.path(dirname(file), paste("~", basename(file), sep = "")), file, overwrite = TRUE)
        unlink(file.path(dirname(file), paste("~", basename(file), sep = "")))
        warning(
            "Injected code is syntatically incorrect. File '", file, "' has not been changed.\n",
            "Message:\n", x
        )
    })
    return(invisible(content))
}

## Create display parameters section for the settings man page that list all available parameters for all classes
allDisplayParsDoc <- function(details) {
    require(Gviz)
    text <- indent(c("\\section{Display Parameters}{", "\\describe{"), level = 0:1)
    for (cl in c(
        "GenomeAxisTrack", "DataTrack", "IdeogramTrack",
        "AnnotationTrack", "GeneRegionTrack", "BiomartGeneRegionTrack", "AlignmentsTrack"
    ))
    {
        parents <- names(getClassDef(cl)@contains)
        pars <- sapply(c(cl, parents), function(x) as.list(getClassDef(x)@prototype@dp), simplify = FALSE)
        text <- c(text, indent(c(sprintf("\\item{%s}{:", cl), ifelse(length(pars[[1]]), "\\describe{", "{")), level = 2:3))
        done <- NULL
        for (p in names(pars))
        {
            todo <- setdiff(names(pars[[p]]), done)
            done <- union(done, names(pars[[p]]))
            if (length(todo)) {
                det <- details[[p]][todo]
                if (is.null(det)) {
                    warning("No details available for class '", p, "'")
                    det <- "FIXME: PLEASE ADD PARAMETER DESCRIPTION."
                } else {
                    det[is.na(det)] <- "FIXME: PLEASE ADD PARAMETER DESCRIPTION."
                }
                if (any(is.na(det))) {
                      warning("There are details missing for class '", p, "'. Please update the documentation")
                  }
                if (p != cl) {
                    text <- c(text, indent(c("}", sprintf("\\bold{\\emph{Inherited from class %s:}}", p), "\\describe{"), level = 3))
                }
                text <- c(text, indent(sprintf("\\item{}{\\code{%s=%s}: %s}", todo, sapply(pars[[p]][todo], toChar), det), level = 4, block = FALSE))
            }
        }
        text <- c(text, indent(rep("}", 2), level = 3:2))
    }
    text <- c(text, indent(rep("}", 2), level = 1:0))
    return(sub("\n*$", "", paste(text, collapse = " \n\n", sep = "")))
}


updateRdFile <- function(class, outdir) {
    require(Gviz)
    require(tools)
    file <- file.path(outdir, paste(class, "class.Rd", sep = "-"))
    content <- displayParsDoc(class, details = details)
    injectContent(file, content, "Display Parameters")
}

updateSettingsFile <- function(outdir) {
    file <- file.path(outdir, "settings.Rd")
    content <- allDisplayParsDoc(details = details)
    injectContent(file, content, "Display Parameters")
}

updateLinks <- function(outdir, toUpdate) {
    if (missing(toUpdate)) {
          toUpdate <- dir(outdir, pattern = "^[^~].*")
      }
    toUpdate <- file.path(outdir, basename(toUpdate))
    res <- NULL
    for (f in toUpdate)
    {
        tmp <- suppressWarnings(parse_Rd(f))
        classes <- unique(.traverseRd(tmp, "linkS4class"))
        functions <- setdiff(unique(.traverseRd(tmp, "link")), classes)
        if (!length(functions) && !length(classes)) {
              res <- c(res, injectContent(f, "", "seealso"))
          } else {
            text <- c(
                "\\seealso{",
                if (!is.null(classes)) indent(paste("\\code{\\linkS4class{", sort(classes), "}}", sep = ""), 1) else "",
                if (!is.null(functions)) indent(paste("\\code{\\link{", sort(functions), "}}", sep = ""), 1) else "",
                "}"
            )
            res <- c(res, injectContent(f, sub("\n*$", "", paste(text, collapse = "\n\n")), "seealso"))
        }
    }
    return(res)
}

details <- list(
    IdeogramTrack = c(
        background.title = "Character scalar. The background color for the title panel. Defaults to omit the background.",
        bevel = "Numeric scalar, between 0 and 1. The level of smoothness for the two ends of the ideogram.",
        centromereShape = "Character scalar. The shape of the centromere. Only \"triangle\" or \"circle\" is accepted. Default to \"triangle\"",
        cex.bands = "Numeric scalar. The  font expansion factor for the chromosome band identifier text.",
        cex = "Numeric scalar. The overall font expansion factor for the chromosome name text.",
        col = "Character scalar. The border color used for the highlighting of the currently displayed genomic region.",
        col.border.title = "Integer or character scalar. The border color for the title panels.",
        lwd.border.title = "Integer scalar. The border width for the title panels.",
        fill = "Character scalar. The fill color used for the highlighting of the currently displayed genomic region.",
        fontcolor = "Character scalar. The font color for the chromosome name text.",
        fontface = "Character scalar. The font face for the chromosome name text.",
        fontfamily = "Character scalar. The font family for the chromosome name text.",
        fontsize = "Numeric scalar. The font size for the chromosome name text.",
        lty = "Character or integer scalar. The line type used for the highlighting of the currently displayed genomic region.",
        lwd = "Numeric scalar. The line width used for the highlighting of the currently displayed genomic region.",
        outline = "Logical scalar. Add borders to the individual chromosome staining bands.",
        showBandId = "Logical scalar. Show the identifier for the chromosome bands if there is space for it.",
        showId = "Logical scalar. Indicate the chromosome name next to the ideogram.",
        showTitle = "Logical scalar. Plot a title panel. Defaults to omit the title panel.",
        size = "Numeric scalar. The relative size of the track. Defaults to automatic size setting. Can also be overridden in the \\code{\\link{plotTracks}} function."
    ),
    DataTrack = c(
        aggregateGroups = "Logical scalar. Aggregate the values within a sample group using the aggregation funnction specified in the \\code{aggregation} parameter.",
        aggregation = "Function or character scalar. Used to aggregate values in windows or for collapsing overlapping items. The function has to accept a numeric vector as a single input parameter and has to return a numeric scalar with the aggregated value. Alternatively, one of the predefined options \\code{mean}, \\code{median} \\code{sum}, \\code{min}, \\code{max} or \\code{extreme} can be supplied as a character scalar. Defaults to \\code{mean}.",
        missingAsZero = "Logical scalar. Defines how the missing values are treated in the aggregation procedure with running window. Setting it to \\code{TRUE} fills empty positions with zeros, which is default. \\code{FALSE} fills empty positions with \\code{NA}.",
        alpha.confint = "Numeric scalar. The transparency for the confidence intervalls in confint-type plots.",
        amount = "Numeric scalar. Amount of jittering in xy-type plots. See \\code{\\link{panel.xyplot}} for details.",
        baseline = "Numeric scalar. Y-axis position of an optional baseline. This parameter has a special meaning for mountain-type and polygon-type plots, see the 'Details' section in \\code{\\linkS4class{DataTrack}} for more information.",
        box.legend = "Logical scalar. Draw a box around a legend.",
        box.ratio = "Numeric scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        box.width = "Numeric scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        cex.legend = "Numeric scalar. The size factor for the legend text.",
        cex.sampleNames = "Numeric scalar. The size factor for the sample names text in heatmap or horizon plots. Defaults to an automatic setting.",
        cex = "Numeric scalar. The default pixel size for plotting symbols.",
        coef = "Numeric scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        col.baseline = "Character scalar. Color for the optional baseline, defaults to the setting of \\code{col}.",
        col.confint = "Character vector. Border colors for the confidence intervals for confint-type plots.",
        col.boxplotFrame = "Character scalar. Line color of the frame around grouped boxplots.",
        col.histogram = "Character scalar. Line color in histogram-type plots.",
        col.horizon = "The line color for the segments in the \\code{horizon}-type plot. See \\code{\\link{horizonplot}} for details.",
        col.mountain = "Character scalar. Line color in mountain-type and polygon-type plots, defaults to the setting of \\code{col}.",
        col.sampleNames = "Character or integer scalar. The color used for the sample names in heatmap plots.",
        col = "Character or integer vector. The color used for all line and symbol elements, unless there is a more specific control defined elsewhere. Unless \\code{groups} are specified, only the first color in the vector is usually regarded.",
        collapse = "Logical scalar. Collapse overlapping ranges and aggregate the underlying data.",
        degree = "Numeric scalar. Parameter controlling the loess calculation for smooth and mountain-type plots. See \\code{\\link{panel.loess}} for details.",
        do.out = "Logical scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        evaluation = "Numeric scalar. Parameter controlling the loess calculation for smooth and mountain-type plots. See \\code{\\link{panel.loess}} for details.",
        factor = "Numeric scalar. Factor to control amount of jittering in xy-type plots. See \\code{\\link{panel.xyplot}} for details.",
        family = "Character scalar. Parameter controlling the loess calculation for smooth and mountain-type plots. See \\code{\\link{panel.loess}} for details.",
        fill.confint = "Character vector. Fill colors for the confidence intervals for confint-type plots.",
        fill.histogram = "Character scalar. Fill color in histogram-type plots, defaults to the setting of \\code{fill}.",
        fill.horizon = "The fill colors for the segments in the \\code{horizon}-type plot. This should be a vector of length six, where the first three entries are the colors for positive changes, and the latter three entries are the colors for negative changes. Defaults to a red-blue color scheme. See \\code{\\link{horizonplot}} for details.",
        fill.mountain = "Character vector of length 2. Fill color in mountain-type and polygon-type plots.",
        fontcolor.legend = "Integer or character scalar. The font color for the legend text.",
        fontface.legend = "Integer or character scalar. The font face for the legend text.",
        fontfamily.legend = "Integer or character scalar. The font family for the legend text.",
        fontsize.legend = "Numeric scalar. The pixel size for the legend text.",
        gradient = "Character vector. The base colors for the \\code{gradient} plotting type or the \\code{heatmap} type with a single group. When plotting heatmaps with more than one group, the \\code{col} parameter can be used to control the group color scheme, however the gradient will always be from white to 'col' and thus does not offer as much flexibility as this \\code{gradient} parameter.",
        grid = "Logical vector. Draw a line grid under the track content.",
        groups = "Vector coercable to a factor. Optional sample grouping. See 'Details' section in \\code{\\linkS4class{DataTrack}} for further information.",
        horizon.origin = "The baseline relative to which changes are indicated on the \\code{horizon}-type plot. See \\code{\\link{horizonplot}} for details.",
        horizon.scale = "The scale for each of the segments in the \\code{horizon}-type plot. Defaults to 1/3 of the absolute data range. See \\code{\\link{horizonplot}} for details.",
        jitter.x = "Logical scalar. Toggle on jittering on the x axis in xy-type plots. See \\code{\\link{panel.xyplot}} for details.",
        jitter.y = "Logical scalar. Toggle off jittering on the y axis in xy-type plots. See \\code{\\link{panel.xyplot}} for details.",
        legend = "Boolean triggering the addition of a legend to the track to indicate groups. This only has an effect if at least two groups are present.",
        levels.fos = "Numeric scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        lineheight.legend = "Numeric scalar. The line height for the legend text.",
        lty.baseline = "Character or numeric scalar. Line type of the optional baseline, defaults to the setting of \\code{lty}.",
        lty.mountain = "Character or numeric scalar. Line type in mountain-type and polygon-type plots, defaults to the setting of \\code{lty}.",
        lwd.baseline = "Numeric scalar. Line width of the optional baseline, defaults to the setting of \\code{lwd}.",
        lwd.mountain = "Numeric scalar. Line width in mountain-type and polygon-type plots, defaults to the setting of \\code{lwd}.",
        min.distance = "Numeric scalar. The mimimum distance in pixel below which to collapse ranges.",
        na.rm = "Boolean controlling whether to discard all NA values when plotting or to keep empty spaces for NAs",
        ncolor = "Integer scalar. The number of colors for the 'gradient' plotting type",
        notch.frac = "Numeric scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        notch = "Logical scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        pch = "Integer scalar. The type of glyph used for plotting symbols.",
        separator = "Numeric scalar. Number of pixels used to separate individual samples in heatmap- and horizon-type plots.",
        showColorBar = "Boolean. Indicate the data range color mapping in the axis for 'heatmap' or 'gradient' types.",
        showSampleNames = "Boolean. Display the names of the individual samples in a heatmap or a horizon plot.",
        size = "Numeric scalar. The relative size of the track. Can be overridden in the \\code{\\link{plotTracks}} function. By default the size will be set automatically based on the selected plotting type.",
        span = "Numeric scalar. Parameter controlling the loess calculation for smooth and mountain-type plots. See \\code{\\link{panel.loess}} for details.",
        stackedBars = "Logical scalar. When there are several data groups, draw the histogram-type plots as stacked barplots or grouped side by side.",
        stats = "Function. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        transformation = "Function. Applied to the data  matrix prior to plotting or when calling the \\code{score} method. The function should accept exactly one input argument and its return value needs to be a numeric vector which can be coerced back into a data matrix of identical dimensionality as the input data.",
        type = "Character vector. The plot type, one or several in \\code{p},\\code{l}, \\code{b}, \\code{a}, \\code{a_confint}, \\code{s}, \\code{g}, \\code{r}, \\code{S}, \\code{confint}, \\code{smooth}, \\code{histogram}, \\code{mountain}, \\code{polygon}, \\code{h}, \\code{boxplot}, \\code{gradient}, \\code{heatmap}, \\code{horizon}. See 'Details' section in \\code{\\linkS4class{DataTrack}} for more information on the individual plotting types.",
        varwidth = "Logical scalar. Parameter controlling the boxplot appearance. See \\code{\\link{panel.bwplot}} for details.",
        window = "Numeric or character scalar. Aggregate the rows values of the data matrix to \\code{window} equally sized slices on the data range using the method defined in \\code{aggregation}. If negative, apply a running window of size \\code{windowSize} using the same aggregation method. Alternatively, the special value \\code{auto} causes the function to determine the optimal window size to avoid overplotting, and \\code{fixed} uses fixed-size windows of size \\code{windowSize}.",
        windowSize = "Numeric scalar. The size of the running window when the value of \\code{window} is negative.",
        ylim = "Numeric vector of length 2. The range of the y-axis scale.",
        yTicksAt = "Numeric vector. The points at which y-axis tick-marks are to be drawn. By default, when \\code{NULL}, tickmark locations are computed."
    ),
    StackedTrack = c(
        stackHeight = "Numeric between 0 and 1. Controls the vertical size and spacing between stacked elements. The number defines the proportion of the total available space for the stack that is used to draw the glyphs. E.g., a value of 0.5 means that half of the available vertical drawing space (for each stacking line) is used for the glyphs, and thus one quarter of the available space each is used for spacing above and below the glyph. Defaults to 0.75.",
        reverseStacking = "Logical flag. Reverse the y-ordering of stacked items. I.e., features that are plotted on the bottom-most stacks will be moved to the top-most stack and vice versa."
    ),
    GdObject = c(
        alpha = "Numeric scalar. The transparency for all track items.",
        alpha.title = "Numeric scalar. The transparency for the title panel.",
        background.panel = "Integer or character scalar. The background color of the content panel.",
        background.title = "Integer or character scalar. The background color for the title panel.",
        background.legend = "Integer or character scalar. The background color for the legend.",
        cex.axis = "Numeric scalar. The expansion factor for the axis annotation. Defaults to \\code{NULL}, in which case it is automatically determined based on the available space.",
        cex.title = "Numeric scalar. The expansion factor for the title panel. This effects the fontsize of both the title and the axis, if any. Defaults to \\code{NULL}, which means that the text size is automatically adjusted to the available space.",
        cex = "Numeric scalar. The overall font expansion factor for all text and glyphs, unless a more specific definition exists.",
        col.axis = "Integer or character scalar. The font and line color for the y axis, if any.",
        col.border.title = "Integer or character scalar. The border color for the title panels.",
        col.frame = "Integer or character scalar. The line color used for the panel frame, if \\code{frame==TRUE}",
        col.grid = "Integer or character scalar. Default line color for grid lines, both when \\code{type==\"g\"} in \\code{\\link{DataTrack}}s and when display parameter \\code{grid==TRUE}.",
        col.line = "Integer or character scalar. Default colors for plot lines. Usually the same as the global \\code{col} parameter.",
        col.symbol = "Integer or character scalar. Default colors for plot symbols. Usually the same as the global \\code{col} parameter.",
        col.title = "Integer or character scalar. The border color for the title panels",
        col = "Integer or character scalar. Default line color setting for all plotting elements, unless there is a more specific control defined elsewhere.",
        collapse = "Boolean controlling whether to collapse the content of the track to accomodate the minimum current device resolution. See \\code{\\link{collapsing}} for details.",
        fill = "Integer or character scalar. Default fill color setting for all plotting elements, unless there is a more specific control defined elsewhere.",
        fontcolor.title = "Integer or character scalar. The font color for the title panels.",
        fontcolor = "Integer or character scalar. The font color for all text, unless a more specific definition exists.",
        fontface.title = "Integer or character scalar. The font face for the title panels.",
        fontface = "Integer or character scalar. The font face for all text, unless a more specific definition exists.",
        fontfamily.title = "Integer or character scalar. The font family for the title panels.",
        fontfamily = "Integer or character scalar. The font family for all text, unless a more specific definition exists.",
        fontsize = "Numeric scalar. The font size for all text, unless a more specific definition exists.",
        frame = "Boolean. Draw a frame around the track when plotting.",
        grid = "Boolean, switching on/off the plotting of a grid.",
        h = "Integer scalar. Parameter controlling the number of horizontal grid lines, see \\code{\\link{panel.grid}} for details.",
        lineheight = "Numeric scalar. The font line height for all text, unless a more specific definition exists.",
        lty.grid = "Integer or character scalar. Default line type for grid lines, both when \\code{type==\"g\"} in \\code{\\link{DataTrack}}s and when display parameter \\code{grid==TRUE}.",
        lty = "Numeric scalar. Default line type setting for all plotting elements, unless there is a more specific control defined elsewhere.",
        lwd.border.title = "Integer scalar. The border width for the title panels.",
        lwd.grid = "Numeric scalar. Default line width for grid lines, both when \\code{type==\"g\"} in \\code{\\link{DataTrack}}s and when display parameter \\code{grid==TRUE}.",
        lwd.title = "Integer scalar. The border width for the title panels",
        lwd = "Numeric scalar. Default line width setting for all plotting elements, unless there is a more specific control defined elsewhere.",
        min.distance = "Numeric scalar. The minimum pixel distance before collapsing range items, only if \\code{collapse==TRUE}. See \\code{\\link{collapsing}} for details.",
        min.height = "Numeric scalar. The minimum range height in pixels to display. All ranges are expanded to this size in order to avoid rendering issues.  See \\code{\\link{collapsing}} for details.",
        min.width = "Numeric scalar. The minimum range width in pixels to display. All ranges are expanded to this size in order to avoid rendering issues. See \\code{\\link{collapsing}} for details.",
        reverseStrand = "Logical scalar. Set up the plotting coordinates in 3' -> 5' direction if \\code{TRUE}. This will effectively mirror the plot on the vertical axis.",
        rotation.title = "The rotation angle for the text in the title panel. Even though this can be adjusted, the automatic resizing of the title panel will currently not work, so use at own risk.",
        rotation = "The rotation angle for all text unless a more specific definiton exists.",
        showAxis = "Boolean controlling whether to plot a y axis (only applies to track types where axes are implemented).",
        showTitle = "Boolean controlling whether to plot a title panel. Although this can be set individually for each track, in multi-track plots as created by \\code{\\link{plotTracks}} there will still be an empty placeholder in case any of the other tracks include a title. The same holds true for axes. Note that the the title panel background color could be set to transparent in order to completely hide it.",
        size = "Numeric scalar. The relative size of the track. Can be overridden in the \\code{\\link{plotTracks}} function.",
        v = "Integer scalar. Parameter controlling the number of vertical grid lines, see \\code{\\link{panel.grid}} for details.",
        "..." = "additional display parameters are allowed. Those typically take the value of a valid R color descriptors. The parameter names will later be matched to optional track item types as defined in the 'feature' range attribute, and all tracks of the matched types are colored accordingly. See the documentation of the \\code{\\link{GeneRegionTrack}} and \\code{\\link{AnnotationTrack}} classes as well as \\code{\\link{grouping}} for details."
    ),
    GenomeAxisTrack = c(
        add35 = "Logical scalar. Add 3' to 5' direction indicators.",
        add53 = "Logical scalar. Add 5' to 3' direction indicators.",
        background.title = "Character scalar. The background color for the title panel. Defaults to omit the background.",
        cex.id = "Numeric scalar. The text size for the optional range annotation.",
        cex = "Numeric scalar. The overall font expansion factor for the axis annotation text.",
        col.border.title = "Integer or character scalar. The border color for the title panels.",
        col.id = "Character scalar. The text color for the optional range annotation.",
        col.range = "Character scalar. The border color for highlighted regions on the axis.",
        col = "Character scalar. The color for the axis lines and tickmarks.",
        distFromAxis = "Numeric scalar. Control the distance of the axis annotation from the tick marks.",
        exponent = "Numeric scalar. The exponent for the axis coordinates, e.g., 3 means mb, 6 means gb, etc. The default is to automatically determine the optimal exponent.",
        fill.range = "Character scalar. The fill color for highlighted regions on the axis.",
        fontcolor = "Character scalar. The font color for the axis annotation text.",
        fontsize = "Numeric scalar. Font size for the axis annotation text in points.",
        labelPos = "Character vector, one in \"alternating\", \"revAlternating\", \"above\" or \"below\". The vertical positioning of the axis labels. If \\code{scale} is not \\code{NULL}, the possible values are \"above\", \"below\" and \"beside\".",
        littleTicks = "Logical scalar. Add more fine-grained tick marks.",
        lwd = "Numeric scalar. The line width for the axis elementes.",
        lwd.border.title = "Integer scalar. The border width for the title panels.",
        scale = "Numeric scalar. If not \\code{NULL} a small scale is drawn instead of the full axis, if the value is between 0 and 1 it is interpreted as a fraction of the current plotting region, otherwise as an absolute length value in genomic coordinates.",
        showId = "Logical scalar. Show the optional range highlighting annotation.",
        showTitle = "Logical scalar. Plot a title panel. Defaults to omit the title panel.",
        ticksAt = "Numeric scalar. The exact x-position for tickmarks (in base-pairs).",
        size = "Numeric scalar. The relative size of the track. Can be overridden in the \\code{\\link{plotTracks}} function. Defaults to the ideal size based on the other track settings."
    ),
    AnnotationTrack = c(
        arrowHeadMaxWidth = "Numeric scalar. The maximum width of the arrow head in pixels if \\code{shape} is \\code{arrow}.",
        arrowHeadWidth = "Numeric scalar. The width of the arrow head in pixels if \\code{shape} is \\code{fixedArrow}.",
        cex.group = "Numeric scalar. The font expansion factor for the group-level annotation.",
        cex = "Numeric scalar. The font expansion factor for item identifiers.",
        col.line = "Character scalar. The color used for connecting lines between grouped items. Defaults to a light gray, but if set to \\code{NULL} the same color as for the first item in the group is used.",
        col = "Character or integer scalar. The border color for all track items.",
        featureAnnotation = "Character scalar. Add annotation information to the individual track elements. This can be a value in \\code{id}, \\code{group} or \\code{feature}. Defaults to \\code{id}. Only works if \\code{showFeatureId} is not \\code{FALSE}.",
        fontcolor.group = "Character or integer scalar. The font color for the group-level annotation.",
        fontcolor.item = "Character or integer scalar. The font color for item identifiers.",
        fontface.group = "Numeric scalar. The font face for the group-level annotation.",
        fontfamily.group = "Character scalar. The font family for the group-level annotation.",
        fontsize.group = "Numeric scalar. The font size for the group-level annotation.",
        groupAnnotation = "Character scalar. Add annotation information as group labels. This can be a value in \\code{id}, \\code{group} or \\code{feature}. Defaults to \\code{group}. Only works if \\code{showId} is not \\code{FALSE}.",
        just.group = "Character scalar. the justification of group labels. Either \\code{left}, \\code{right}, \\code{above} or \\code{below}.",
        lex = "Numeric scalar. The line expansion factor for all track items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details.",
        lineheight = "Numeric scalar. The font line height for item identifiers.",
        lty = "Character or integer scalar. The line type for all track items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details.",
        lwd = "Integer scalar. The line width for all track items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details.",
        mergeGroups = "Logical scalar. Merge fully overlapping groups if \\code{collapse==TRUE}.",
        min.height = "Numeric scalar. The minimum range height in pixels to display. All ranges are expanded to this size in order to avoid rendering issues.  See \\code{\\link{collapsing}} for details. For feathered bars indicating the strandedness of grouped items this also controls the height of the arrow feathers.",
        min.width = "Numeric scalar. The minimum range width in pixels to display. All ranges are expanded to this size in order to avoid rendering issues. See \\code{\\link{collapsing}} for details.",
        rotation = "Numeric scalar. The degree of text rotation for item identifiers.",
        rotation.group = "Numeric scalar. The degree of text rotation for group labels.",
        rotation.item = "Numeric scalar. The degree of text rotation for item identifiers.",
        shape = "Character scalar. The shape in which to display the track items. Currently only \\code{box}, \\code{arrow}, \\code{fixedArrow}, \\code{ellipse}, and \\code{smallArrow} are implemented.",
        showFeatureId = "Logical scalar. Control whether to plot the individual track item identifiers.",
        showId = "Logical scalar. Control whether to annotate individual groups.",
        showOverplotting = "Logical scalar. Use a color gradient to show the amount of overplotting for collapsed items. This implies that \\code{collapse==TRUE}",
        size = "Numeric scalar. The relative size of the track. Can be overridden in the \\code{\\link{plotTracks}} function.",
        fill = "Character or integer scalar. The fill color for untyped items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details."
    ),
    DetailsAnnotationTrack = c(
        details.minWidth = "Numeric scalar. The minium width in pixels for a details panel, if less space is available no details are plotted.",
        details.ratio = "Numeric scalar. By default, the plotting method tries to fill all available space of the details panel tiles. Depending on the dimensions of your plot and the number of tiles this may lead to fairly stretched plots. Restricting the ration of width over height can help to fine tune for somewhat more sane graphics in these cases. Essentially this adds some white space in between individual tiles to force the desired ratio. Together with the \\code{size} and \\code{details.size} arguments, which control the vertical extension of the whole track and of the details section, this allows for some fairly generic resizing of the tiles.",
        detailsBorder.col = "Character or integer scalar. Line color of the border.",
        detailsBorder.fill = "Character or integer scalar. Background color of the border.",
        detailsBorder.lty = "Character or integer scalar. Line type of the border around each details panel.",
        detailsBorder.lwd = "Integer scalar. Line width of the border.",
        detailsConnector.cex = "Numeric scalar. Relative size of the connector's end points.",
        detailsConnector.col = "Character or integer scalar. Color of the line connecting the \\code{AnnotstionTrack} item with its details panel.",
        detailsConnector.lty = "Character or integer scalar. Type of connecting line.",
        detailsConnector.lwd = "Integer scalar. Line width of the connector.",
        detailsConnector.pch = "Integer scalar. Type of the connector's ends.",
        detailsFunArgs = "List.Additional arguments that get passed on the the details plotting function.",
        details.size = "Numeric scalar. The fraction of vertical space of the track used for the details section.",
        groupDetails = "Logial scalar. Plot details for feature groups rather than for individual features."
    ),
    GeneRegionTrack = c(
        alpha = "Numeric scalar between 0 and 1. The opacity of the plotting elements, if supported by the device.",
        arrowHeadMaxWidth = "Numeric scalar. The maximum width of the arrow head in pixels if \\code{shape} is \\code{arrow}.",
        arrowHeadWidth = "Numeric scalar. The width of the arrow head in pixels if \\code{shape} is \\code{fixedArrow}.",
        cex.group = "Numeric scalar. The font expansion factor for the group-level annotation.",
        cex = "Numeric scalar. The font expansion factor for item identifiers.",
        col = "Character or integer scalar. The border color for all track items. Defaults to using the same color as in \\code{fill}, also taking into account different track \\code{features}.",
        collapseTranscripts = "Logical or character scalar. Can be one in \\code{gene}, \\code{longest}, \\code{shortest} or \\code{meta}. Merge all transcripts of the same gene into one single gene model. In the case of \\code{gene} (or \\code{TRUE}), this will only keep the start location of the first exon and the end location of the last exon from all transcripts of the gene. For \\code{shortest} and \\code{longest}, only the longest or shortest transcript model is retained. For \\code{meta}, a meta-transcript containing the union of all exons is formed (essentially identical to the operation \\code{reduce(geneModel)}).",
        exonAnnotation = "Character scalar. Add annotation information to the individual exon models. This can be a value in \\code{symbol}, \\code{gene}, \\code{transcript}, \\code{exon} or \\code{feature}. Defaults to \\code{exon}. Only works if \\code{showExonId} is not \\code{FALSE}.",
        fill = "Character or integer scalar. The fill color for untyped items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details.",
        fontcolor.group = "Character or integer scalar. The font color for the group-level annotation.",
        fontcolor = "Character or integer scalar. The font color for item identifiers.",
        fontface.group = "Numeric scalar. The font face for the group-level annotation.",
        fontface = "Integer scalar. The font face for item identifiers.",
        fontfamily.group = "Character scalar. The font family for the group-level annotation.",
        fontfamily = "Character scalar. The font family for item identifiers.",
        fontsize.group = "Numeric scalar. The font size for the group-level annotation.",
        fontsize = "Numeric scalar. The font size for item identifiers.",
        geneSymbols = "Logical scalar. Use human-readable gene symbols or gene IDs for the transcript annotation.",
        lex = "Numeric scalar. The line expansion factor for all track items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details.",
        lineheight = "Numeric scalar. The font line height for item identifiers.",
        lty = "Character or integer scalar. The line type for all track items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details.",
        lwd = "Integer scalar. The line width for all track items. This is also used to connect grouped items. See \\code{\\link{grouping}} for details.",
        min.distance = "Numeric scalar. The minimum pixel distance before collapsing range items, only if \\code{collapse==TRUE}. See \\code{\\link{collapsing}} for details. Note that a value larger than 0 may lead to UTR regions being merged to CDS regions, which in most cases is not particularly useful.",
        min.width = "Numeric scalar. The minimum range width in pixels to display. All ranges are expanded to this size in order to avoid rendering issues. See \\code{\\link{collapsing}} for details.",
        rotation = "Numeric scalar. The degree of text rotation for item identifiers.",
        shape = "Character scalar. The shape in which to display the track items. Currently only \\code{box}, \\code{arrow}, \\code{ellipse}, and \\code{smallArrow} are implemented.",
        showExonId = "Logical scalar. Control whether to plot the individual exon identifiers.",
        showId = "Logical scalar. Control whether to annotate individual groups.",
        showOverplotting = "Logical scalar. Use a color gradient to show the amount of overplotting for collapsed items. This implies that \\code{collapse==TRUE}",
        size = "Numeric scalar. The relative size of the track. Can be overridden in the \\code{\\link{plotTracks}} function.",
        thinBoxFeature = "Character vector. A listing of feature types that should be drawn with thin boxes. Typically those are non-coding elements.",
        transcriptAnnotation = "Character scalar. Add annotation information as transcript labels. This can be a value in \\code{symbol}, \\code{gene}, \\code{transcript}, \\code{exon} or \\code{feature}. Defaults to \\code{symbol}. Only works if \\code{showId} is not \\code{FALSE}."
    ),
    BiomartGeneRegionTrack = c(
        "C_segment" = "Character or integer scalar. Fill color for annotation objects of type 'C_segment'.",
        "D_segment" = "Character or integer scalar. Fill color for annotation objects of type 'C_segment'.",
        "J_segment" = "Character or integer scalar. Fill color for annotation objects of type 'C_segment'.",
        "miRNA" = "Character or integer scalar. Fill color for annotation objects of type 'L_segment'.",
        "miRNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'miRNA_pseudogene'.",
        "misc_RNA" = "Character or integer scalar. Fill color for annotation objects of type 'misc_RNA'.",
        "misc_RNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'misc_RNA_pseudogene'.",
        "Mt_rRNA" = "Character or integer scalar. Fill color for annotation objects of type 'Mt_rRNA'.",
        "Mt_tRNA" = "Character or integer scalar. Fill color for annotation objects of type 'Mt_tRNA'.",
        "Mt_tRNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'Mt_tRNA_pseudogene'.",
        "protein_coding" = "Character or integer scalar. Fill color for annotation objects of type 'protein_coding'.",
        "pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'pseudogene'.",
        "retrotransposed" = "Character or integer scalar. Fill color for annotation objects of type 'retrotransposed'.",
        "rRNA" = "Character or integer scalar. Fill color for annotation objects of type 'rRNA'.",
        "rRNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'rRNA_pseudogene'.",
        "scRNA" = "Character or integer scalar. Fill color for annotation objects of type 'scRNA'.",
        "scRNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'scRNA_pseudogene'.",
        "snoRNA" = "Character or integer scalar. Fill color for annotation objects of type 'snoRNA'.",
        "snoRNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'snoRNA_pseudogene'.",
        "snRNA" = "Character or integer scalar. Fill color for annotation objects of type 'snRNA'.",
        "snRNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'snRNA_pseudogene'.",
        "tRNA_pseudogene" = "Character or integer scalar. Fill color for annotation objects of type 'tRNA_pseudogene'.",
        "utr3" = "Character or integer scalar. Fill color for annotation objects of type 'utr3'.",
        "utr5" = "Character or integer scalar. Fill color for annotation objects of type 'utr5'.",
        "V_segment" = "Character or integer scalar. Fill color for annotation objects of type 'V_segment'.",
        "verbose" = "Logical scalar. Report data loading events from Bioamart or retrieval from cache."
    ),
    SequenceTrack = c(
        add53 = "Logical scalar. Add a direction indicator.",
        background.title = "Character scalar. Make the title panel background transparent by default.",
        cex = "The character expansion factor for the size of the sequence letters. Together with \\code{fontsize} this determines the final font size and thus the level of plotable details.",
        col = "Character scalar. The color of the line when no indiviual letters can be plotted due to size limitations.",
        col.border.title = "Integer or character scalar. The border color for the title panels.",
        complement = "Logical scalar. Plot the sequence complement.",
        fontcolor = "Character vector. The colors used for the 5 possible nucleotides (G, A, T, C, N). Defaults to use colors as defined in the \\code{biovizBase} package.",
        fontface = "Numeric scalar. The face of the font.",
        fontsize = "Numeric scalar. Controls the size of the sequence letters and thus also the level of plotable details.",
        lwd = "Numeric scalar. The width of the line when no indiviual letters can be plotted due to size limitations.",
        lwd.border.title = "Integer scalar. The border width for the title panels.",
        min.width = "Numeric scalar. The minimum width in pixels of the colored boxes that are drawn when no indiviual letters can be plotted due to size limitations. If the horizontal space that a single base occupies is smaller than this value, only a horizontal line is drawn to indicate the presence of a sequence.",
        noLetters = "Logical scalar. Always plot colored boxes (or a line) regardles of the available space.",
        rotation = "Numeric scalar. The rotation angle for each individual letter in the sequence.",
        showTitle = "Logical scalar. Do not show a title panel by default.",
        size = "Numeric scalar. The size of the track item. Defaults to auto-detect the size based on the other parameter settings."
    ),
    HighlightTrack = c(
        col = "Integer or character vector. The border color for the highlighting regions.",
        fill = "Integer or character vector. The fill color for the highlighting regions.",
        inBackground = "Logical scalar. Place the box in the background or in the foreground."
    ),
    AlignmentsTrack = c(
        alpha.mismatch = "Numeric scalar between 0 and 1. The transparency of the mismatch base information.",
        alpha.reads = "Numeric scalar between 0 and 1. The transparency of the individual read icons. Can be used to indicate overlapping regions in read pairs. Only on supported devices.",
        cex = "Numeric Scalar. The global character expansion factor.",
        cex.mismatch = "Numeric Scalar. The character expansion factor for the mismatch base letters.",
        col.coverage = "Integer or character scalar. The line color for the coverage profile.",
        col.gap = "Integer or character scalar. The color of the line that is bridging the gap regions in gapped alignments.",
        col.mates = "Integer or character scalar. The color of the line that is connecting two paired reads.",
        col.deletion = "Integer or character scalar. The color of the line that is bridging the deleted regions in alignments.",
        col.insertion = "Integer or character scalar. The color of the line that highlighting insertions in alignments.",
        col.mismatch = "Integer or character scalar. The box color around mismatch bases.",
        col.reads = "Integer or character scalar. The box color around reads.",
        col.sashimi = "Integer or character scalar. The line color for sashimi plots.",
        col = "Integer or character scalar. The default color of all line elements.",
        collapse = "Logical scalar. Do not perform any collapsing of overlapping elements. Currently not supported.",
        coverageHeight = "Numeric scalar. The height of the coverage region of the track. Can either be a value between 0 and 1 in which case it is taken as a relative height, or a positive value greater 1 in which case it is interpreted as pixels.",
        fill.coverage = "Integer or character scalar. The fill color for the coverage profile.",
        fill.reads = "Integer or character scalar. The fill color for the read icons.",
        fill = "Integer or character scalar. The default fill color of all plot elements.",
        fontface.mismatch = "Integer scalar. The font face for mismatch bases.",
        lty.coverage = "Integer or character scalar. The line type of the coverage profile.",
        lty.gap = "Integer or character scalar. The type of the line that is bridging the gap regions in gapped alignments.",
        lty.mates = "Integer or character scalar. The type of the line that is connecting two paired reads.",
        lty.deletion = "Integer or character scalar. The type of the line that is bridging the deleted regions in alignments.",
        lty.insertion = "Integer or character scalar. The type of the line that highlighting insertions in alignments.",
        lty.mismatch = "Integer or character scalar. The box line type around mismatch bases.",
        lty.reads = "Integer or character scalar. The box line type around mismatch reads.",
        lty = "Integer or character scalar. The default type of all line elements.",
        lwd.coverage = "Integer or character scalar. The line width of the coverage profile.",
        lwd.gap = "Integer scalar. The width of the line that is bridging the gap regions in gapped alignments.",
        lwd.mates = "Integer scalar. The width of the line that is connecting two paired reads.",
        lwd.deletion = "Integer scalar. The width of the line that is bridging the deleted regions in alignments.",
        lwd.insertion = "Integer scalar. The width of the line that highlighting insertions in alignments.",
        lwd.mismatch = "Integer scalar. The box line width around mismatch bases.",
        lwd.reads = "Integer scalar. The box line width around reads.",
        lwd.sashimiMax = "Integer scalar. The maximal width of the line in sashimi plots.",
        lwd = "Integer scalar. The default width of all line elements.",
        noLetters = "Logical scalar. Always plot colored boxes for mismatch bases regardles of the available space.",
        max.height = "Integer scalar. The maximum height of an individual read in pixels. Can be used in combination with \\code{min.height} to control the read and stacking appearance. ",
        min.height = "Integer scalar. The minimum height of an individual read in pixels. Can be used in combination with \\code{max.height} to control the read and stacking appearance.",
        minCoverageHeight = "Integer scalar. The minimum height of the coverage section. Uselful in combination with a relative setting of \\code{coverageHeight}.",
        minSashimiHeight = "Integer scalar. The minimum height of the sashimi section. Uselful in combination with a relative setting of \\code{sashimiHeight}.",
        showIndels = "Logical scalar. Consider insertions and deletions in coverage and pile-up. Default is \\code{FALSE}. If set to \\code{TRUE} the deletions defined in CIGAR string are not considered in coverage plot. The deletions are displayed as bridging lines in pile-up track. Insertions are shown as vertical bars.",
        showMismatches = "Logical scalar. Add mismatch information, either as individual base letters or using color coded bars. This implies that the reference sequence has been provided, either to the class constructor or as part of the track list.",
        sashimiFilter = "GRanges object. Only junctions which overlap equally with \\code{sashimiFilter} GRanges are shown. Default \\code{NULL}, no filtering.",
        sashimiFilterTolerance = "Integer scalar. Only used in combination with \\code{sashimiFilter}. It allows to include junctions whose starts/ends are within specified distance from \\code{sashimiFilter} GRanges. This is useful for cases where the aligner did not place the junction reads precisely. Default \\code{0L} , no tolerance.",
        sashimiHeight = "Integer scalar. The height of the sashimi part of the track. Can either be a value between 0 and 1 in which case it is taken as a relative height, or a positive value greater 1 in which case it is interpreted as pixels.",
        sashimiScore = "Integer scalar. The minimum number of reads supporting the junction.",
        sashimiStrand = "Integer scalar. Only reads which have the specified strand are considered to count the junctions.",
        sashimiTransformation = "Function. Applied to the junction score vector prior to plotting. The function should accept exactly one input argument and its return value needs to be a numeric vector of identical length as the input data.",
        size = "Numeric scalar. The size of the track. Defaults to automatic sizing.",
        transformation = "Function. Applied to the coverage vector prior to plotting. The function should accept exactly one input argument and its return value needs to be a numeric Rle of identical length as the input data.",
        type = "Character vactor. The type of information to plot. For \\code{coverage} a coverage plot, potentially augmented by base mismatch information, for \\code{sashimi} a sashimi plot, showing the juctions, and for \\code{pileup} the pileups of the individual reads. These three can be combined."
    )
)


updateDocumentation <- function(outdir = "~/Rpacks/Gviz/man") {
    library(Gviz)
    library(IRanges)
    library(rtracklayer)
    library(GenomicRanges)
    library(lattice)
    library(biomaRt)
    library(RColorBrewer)
    library(Biobase)
    library(grid)
    library(AnnotationDbi)
    ## source(file.path(dirname(outdir), "inst/scripts/sourcePackage.R"))
    dps <- sapply(
        c(
            "GdObject", "GenomeAxisTrack", "RangeTrack", "NumericTrack", "DataTrack", "IdeogramTrack", "StackedTrack",
            "AnnotationTrack", "GeneRegionTrack", "BiomartGeneRegionTrack", "AlignmentsTrack"
        ),
        updateRdFile, outdir
    )
    settings <- updateSettingsFile(outdir)
    links <- updateLinks(outdir)
}



## Instructions to rebuild the docs
## In general, changes to the documentation for a class' display parameters set should be handled in this file, in the details
## list above. Default values for display parameters should be set in the class' initializer method.
##
## To update just a single class:
## run
##       compareDtContent(<className>, <documentationDir>, details)
## and check the results. This should show you what is different between the current man page file and the central definitions
## It will also find things that may have been added/edited manually in the man page file, and might need back-porting into the
## central definition list.
## When you are happy with the results, run
##       updateRdFile(<className>, <documentationDir>)
##       ~/Rpacks/Gviz(<documentationDir>)
##       updateLinks(<documentationDir>)
##
## To update all classes, run
##       updateDocumentation(<documentationDir>)
## Of course you should have checked all changes before using compareDtContent()

#' @include GdObject-class.R
#' @include ReferenceTrack-class.R
NULL

## SequenceTrack Class -------------------------------------------------------

#' SequenceTrack class and methods
#'
#'
#' A track class to represent genomic sequences. The three child classes
#' \code{SequenceDNAStringSetTrack}, \code{SequenceRNAStringSetTrack} and
#' \code{SequenceBSgenomeTrack} do most of the work, however in practise they
#' are of no particular relevance to the user.
#'
#'
#' @name SequenceTrack-class
#' @param sequence
#'
#' A meta argument to handle the different input types, making the construction
#' of a \code{SequenceTrack} as flexible as possible.
#'
#' The different input options for \code{sequence} are:
#'
#' \describe{
#'
#' \item{An object of class \code{\linkS4class{DNAStringSet}}.}{ The individual
#' \code{\linkS4class{DNAString}}s are considered to be the different
#' chromosome sequences.}
#'
#' \item{An object of class \code{\linkS4class{BSgenome}}.}{ The \code{Gviz}
#' package tries to follow the \code{BSgenome} philosophy in that the
#' respective chromosome sequences are only realized once they are first
#' accessed.}
#'
#' \item{A \code{character} scalar:}{ in this case the value of the
#' \code{sequence} argument is considered to be a file path to an annotation
#' file on disk. A range of file types are supported by the \code{Gviz} package
#' as identified by the file extension. See the \code{importFunction}
#' documentation below for further details.}
#' }
#'
#' @template SequenceTrack-class_param
#'
#' @return
#'
#' The return value of the constructor function is a new object of class
#' \code{SequenceDNAStringSetTrack}, \code{SequenceBSgenomeTrack} ore
#' \code{ReferenceSequenceTrack}, depending on the constructor arguments.
#' Typically the user will not have to be troubled with this distinction and
#' can rely on the constructor to make the right choice.
#' @section Objects from the class:
#'
#' Objects can be created using the constructor function \code{SequenceTrack}.
#' @author Florian Hahne
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## An empty object
#' SequenceTrack()
#'
#' ## Construct from DNAStringSet
#' library(Biostrings)
#' letters <- c("A", "C", "T", "G", "N")
#' set.seed(999)
#' seqs <- DNAStringSet(c(chr1 = paste(sample(letters, 100000, TRUE),
#'     collapse = ""
#' ), chr2 = paste(sample(letters, 200000, TRUE), collapse = "")))
#' sTrack <- SequenceTrack(seqs, genome = "hg19")
#' sTrack
#'
#' ## Construct from BSGenome object
#' if (require(BSgenome.Hsapiens.UCSC.hg19)) {
#'     sTrack <- SequenceTrack(Hsapiens)
#'     sTrack
#' }
#'
#'
#' ## Set active chromosome
#' chromosome(sTrack)
#' chromosome(sTrack) <- "chr2"
#' head(seqnames(sTrack))
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(sTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' ## Sequences
#' plotTracks(sTrack, from = 199970, to = 200000)
#' ## Boxes
#' plotTracks(sTrack, from = 199800, to = 200000)
#' ## Line
#' plotTracks(sTrack, from = 1, to = 200000)
#' ## Force boxes
#' plotTracks(sTrack, from = 199970, to = 200000, noLetters = TRUE)
#' ## Direction indicator
#' plotTracks(sTrack, from = 199970, to = 200000, add53 = TRUE)
#' ## Sequence complement
#' plotTracks(sTrack, from = 199970, to = 200000, add53 = TRUE, complement = TRUE)
#' ## Colors
#' plotTracks(sTrack, from = 199970, to = 200000, add53 = TRUE, fontcolor = c(
#'     A = 1,
#'     C = 1, G = 1, T = 1, N = 1
#' ))
#'
#' ## Track names
#' names(sTrack)
#' names(sTrack) <- "foo"
#'
#' ## Accessors
#' genome(sTrack)
#' genome(sTrack) <- "mm9"
#' length(sTrack)
#'
#' ## Sequence extraction
#' subseq(sTrack, start = 100000, width = 20)
#' ## beyond the stored sequence range
#' subseq(sTrack, start = length(sTrack), width = 20)
#' @importClassesFrom Biostrings DNAStringSet RNAStringSet BStringSet DNAString RNAString BString
#' @importClassesFrom BSgenome  BSgenome MaskedBSgenome
#' @importFrom Biostrings DNAStringSet RNAStringSet BStringSet DNAString RNAString BString reverseComplement readDNAStringSet DNA_ALPHABET stackStrings
#' @importFrom BSgenome bsapply
#' @importFrom biovizBase getBioColor
#'
#' @exportClass SequenceTrack
setClass("SequenceTrack",
    representation = representation("VIRTUAL",
        chromosome = "character",
        genome = "character"
    ),
    contains = "GdObject",
    prototype = prototype(
        name = "Sequence",
        dp = DisplayPars(
            add53 = FALSE,
            background.title = "transparent",
            col = "darkgray",
            complement = FALSE,
            fontface = 2,
            fontsize = 10,
            lwd = 2,
            min.width = 2,
            noLetters = FALSE,
            showTitle = FALSE,
            size = NULL
        ),
        genome = as.character(NA),
        chromosome = "chrNA"
    )
)

## Essentially we just update the display parameters here and set the chromosome and the genome

#' @describeIn  SequenceTrack-class Initialize.
#' @export
setMethod("initialize", "SequenceTrack", function(.Object, chromosome, genome, ...) {
    ## the display parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "SequenceTrack")
    if (!missing(chromosome) &&
        !is.na(chromosome) &&
        !is.null(chromosome)) {
        if (!is.null(names(sequence))) {
            .Object@chromosome <- .chrName(names(sequence)[1])[1]
        } else {
            .Object@chromosome <- .chrName(chromosome)[1]
        }
    }
    if (missing(genome) || is.na(genome) || is.null(genome)) {
        genome <- as.character(NA)
    }
    .Object@genome <- genome
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructors --------------------------------------------------------------

#' @importMethodsFrom Biostrings seqtype "seqtype<-"
.SequenceTrack <- function(SeqTrackType, seqtype, sequence, chromosome,
                           genome, name = "SequenceTrack", importFunction,
                           stream = FALSE, ...) {
    .missingToNull(c("chromosome", "genome", "sequence"))
    if (is.null(sequence)) {
        return(new(SeqTrackType, chromosome = chromosome, genome = genome, name = name, ...))
    }
    if (is(sequence, "BSgenome")) {
        if (is.null(genome)) {
            genome <- .providerVersion(sequence)
        }
        if (is.null(chromosome)) {
            chromosome <- seqnames(sequence)[1]
        }
        obj <- new("SequenceBSgenomeTrack", sequence = sequence, chromosome = chromosome, genome = genome, name = name, ...)
    } else if (is(sequence, "XStringSet")) {
        if (seqtype(sequence) != seqtype) {
            seqtype(sequence) <- seqtype
        }
        if (is.null(names(sequence))) {
            stop("The sequences in the ", seqtype, "StringSet must be named")
        }
        if (any(duplicated(names(sequence)))) {
            stop("The sequence names in the ", seqtype, "StringSet must be unique")
        }
        if (is.null(chromosome)) {
            chromosome <- names(sequence)[1]
        }
        obj <- new(SeqTrackType, sequence = sequence, chromosome = chromosome, genome = genome, name = name, ...)
    } else if (is.character(sequence)) {
        sequence <- sequence[1]
        if (!file.exists(sequence)) {
            stop(sprintf("'%s' is not a valid file.", sequence))
        }
        ext <- .fileExtension(sequence)
        obj <- if (missing(importFunction) && ext %in% c("fa", "fasta")) {
            if (!file.exists(paste(sequence, "fai", sep = "."))) {
                new("SequenceDNAStringSetTrack",
                    sequence = readDNAStringSet(sequence), chromosome = chromosome,
                    genome = genome, name = name, ...
                )
            } else {
                new("ReferenceSequenceTrack",
                    chromosome = chromosome, genome = genome, name = name,
                    stream = .import.fasta, reference = path.expand(sequence), ...
                )
            }
        } else if (missing(importFunction) && ext == "2bit") {
            new("ReferenceSequenceTrack",
                chromosome = chromosome, genome = genome, name = name,
                stream = .import.2bit, reference = path.expand(sequence), ...
            )
        } else {
            if (missing(importFunction)) {
                stop(sprintf(
                    "No predefined import function exists for files with extension '%s'. Please manually provide an import function.",
                    ext
                ))
            } else {
                if (!stream) {
                    seq <- importFunction(file = sequence)
                    if (!is(seq, "DNAStringSet")) {
                        stop(
                            "The import function did not provide a valid DNAStringSet object. Unable to build track from file '",
                            sequence, "'"
                        )
                    }
                    new("SequenceDNAStringSetTrack",
                        sequence = importFunction(file = sequence), chromosome = chromosome,
                        genome = genome, name = name, ...
                    )
                } else {
                    new("ReferenceSequenceTrack",
                        chromosome = chromosome, genome = genome, name = name,
                        stream = importFunction, reference = path.expand(sequence), ...
                    )
                }
            }
        }
    } else {
        stop("Argument sequence must be of class 'BSgenome', 'XStringSet' or 'character'")
    }
    return(obj)
}

#' @describeIn SequenceTrack-class Constructor
#' @export
SequenceTrack <- function(sequence, chromosome, genome, name = "SequenceTrack", importFunction, stream = FALSE, ...) {
    .SequenceTrack(
        "SequenceDNAStringSetTrack",
        "DNA",
        sequence,
        chromosome,
        genome,
        ...
    )
}

#' @describeIn SequenceTrack-class Constructor
#' @export
RNASequenceTrack <- function(sequence, chromosome, genome, name = "SequenceTrack", importFunction, stream = FALSE, ...) {
    .SequenceTrack(
        "SequenceRNAStringSetTrack",
        "RNA",
        sequence,
        chromosome,
        genome,
        ...
    )
}

## SequenceDNAStringSetTrack -------------------------------------------------

## We want the following behaviour in the constructor:
##   a) sequence is missing (NULL) => build SequenceDNAStringSetTrack with chromosome NA and genome as supplied or NA if missing
##   b) sequence is DNAStringSet => build SequenceDNAStringSetTrack where chromosome is names(sequence)[1] or the supplied
##      chromosome if available, and genome as supplied or NA if missing
##   c) sequence is BSgenome => build SequenceBSgenomeTrack where chromosome is seqnames(sequence)[1] or the supplied
##      chromosome if available, and genome is the supplied genome or the one extracted from the BSgenome object

##
## SequenceDNAStringSetTrack: A track to visualize nucleotide sequences that are stored in a DNAStringSet
##
## Slots:
##    o sequence: a DNAStringSet object that contains all the sequence data

#' @describeIn SequenceTrack-class The `DNAStringSet`-based version of the `SequenceTrack-class`.
#' @exportClass SequenceDNAStringSetTrack
setClass("SequenceDNAStringSetTrack",
    representation = representation(sequence = "DNAStringSet"),
    contains = "SequenceTrack",
    prototype = prototype(
        sequence = Biostrings::DNAStringSet(),
        dp = DisplayPars(fontcolor = biovizBase::getBioColor("DNA_BASES_N"))
    )
)

#' @describeIn  SequenceTrack-class Initialize.
#' @export
setMethod("initialize", "SequenceDNAStringSetTrack", function(.Object, sequence, ...) {
    if (missing(sequence) || is.null(sequence)) {
        sequence <- Biostrings::DNAStringSet()
    }
    .Object@sequence <- sequence
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})


## SequenceRNAStringSetTrack -------------------------------------------------
##
## SequenceRNAStringSetTrack: A track to visualize nucleotide sequences that are stored in a RNAStringSet
## Slots:
##    o sequence: a RNAStringSet object that contains all the sequence data

#' @describeIn SequenceTrack-class The `RNAStringSet`-based version of the `SequenceTrack-class`.
#' @exportClass SequenceRNAStringSetTrack
setClass("SequenceRNAStringSetTrack",
    representation = representation(sequence = "RNAStringSet"),
    contains = "SequenceTrack",
    prototype = prototype(
        sequence = Biostrings::RNAStringSet(),
        dp = DisplayPars(fontcolor = biovizBase::getBioColor("RNA_BASES_N"))
    )
)

#' @describeIn SequenceTrack-class Initialize `RNAStringSet`-based version of the `SequenceTrack-class`.
#' @export
setMethod("initialize", "SequenceRNAStringSetTrack", function(.Object, sequence, ...) {
    if (missing(sequence) || is.null(sequence)) {
        sequence <- Biostrings::RNAStringSet()
    }
    .Object@sequence <- sequence
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})


## SequenceBSgenomeTrack -----------------------------------------------------
##
## SequenceBSgenomeTrack: A track to visualize nucleotide sequences that are stored in a BSgenome package
##
## Slots:
##    o sequence: a DNAStringSet object that contains all the sequence data
##    o pointerCache: an environment to hold pointers to the BSgenome sequences to prevent garbage collection. This
##       will only be filled once the individual sequences have been accessed for the first time

#' @describeIn SequenceTrack-class The `BSgenome`-based version of the `SequenceTrack-class`.
#' @exportClass SequenceBSgenomeTrack
setClass("SequenceBSgenomeTrack",
    representation = representation(sequence = "BSgenomeOrNULL", pointerCache = "environment"),
    contains = "SequenceTrack",
    prototype = prototype(
        sequence = NULL,
        dp = DisplayPars(fontcolor = biovizBase::getBioColor("DNA_BASES_N"))
    )
)

#' @describeIn  SequenceTrack-class Initialize.
#' @export
setMethod("initialize", "SequenceBSgenomeTrack", function(.Object, sequence = NULL, ...) {
    .Object@sequence <- sequence
    .Object@pointerCache <- new.env()
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})



## ReferenceSequenceTrack ----------------------------------------------------
##
## ReferenceSequenceTrack: The file-based version of the ReferenceTrack class.
##                         This will mainly provide a means to dispatch to a special 'subseq' method
##                         which should stream the necessary data from disk.

#' @describeIn SequenceTrack-class The file-based version of the `SequenceTrack-class`.
#' @exportClass ReferenceSequenceTrack
setClass("ReferenceSequenceTrack", contains = c("SequenceDNAStringSetTrack", "ReferenceTrack"))

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritance has some strange features with regards to method selection

#' @describeIn  SequenceTrack-class Initialize.
#' @export
setMethod("initialize", "ReferenceSequenceTrack", function(.Object, stream, reference, ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object = .Object, reference = reference, stream = stream)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Initialize ----------------------------------------------------------------
## Constructor ---------------------------------------------------------------
## General accessors ---------------------------------------------------------

#' @describeIn SequenceTrack-class return the names (i.e., the chromosome)
#' of the sequences contained in the object.
#' @export
setMethod("seqnames", "SequenceTrack", function(x) as.character(names(x@sequence)))

# setMethod("seqnames", "SequenceDNAStringSetTrack", function(x) as.character(names(x@sequence)))
# setMethod("seqnames", "SequenceRNAStringSetTrack", function(x) as.character(names(x@sequence)))

#' @describeIn SequenceTrack-class return the names (i.e., the chromosome)
#' of the sequences contained in the object.
#' @export
setMethod("seqnames", "SequenceBSgenomeTrack", function(x) as.character(seqnames(x@sequence)))

#' @describeIn SequenceTrack-class return the names (i.e., the chromosome)
#' of the sequences contained in the object. Only those with length > 0.
#' @export
setMethod("seqlevels", "SequenceTrack", function(x) seqnames(x)[width(x@sequence) > 0])

# setMethod("seqlevels", "SequenceDNAStringSetTrack", function(x) seqnames(x)[width(x@sequence) > 0])
# setMethod("seqlevels", "SequenceRNAStringSetTrack", function(x) seqnames(x)[width(x@sequence) > 0])

#' @describeIn SequenceTrack-class return the names (i.e., the chromosome)
#' of the sequences contained in the object. Only those with length > 0.
#' @export
setMethod("seqlevels", "SequenceBSgenomeTrack", function(x) seqnames(x)[BSgenome::bsapply(new("BSParams", X = x@sequence, FUN = length, simplify = TRUE)) > 0]) # maybe seqnames only, to speed-up

#' @describeIn SequenceTrack-class return the start coordinates of the track
#' items.
#' @export
setMethod("start", "SequenceTrack", function(x) NULL)

#' @describeIn SequenceTrack-class return the end coordinates of the track
#' items.
#' @export
setMethod("end", "SequenceTrack", function(x) NULL)

#' @describeIn SequenceTrack-class return the with of the track items in
#' genomic coordinates.
#' @export
setMethod("width", "SequenceTrack", function(x) NULL)

#' @describeIn SequenceTrack-class return the length of the sequence for
#' active chromosome.
#' @export
setMethod("length", "SequenceTrack", function(x) {
    if (chromosome(x) %in% seqnames(x)) length(x@sequence[[chromosome(x)]]) else 0
})

#' @importMethodsFrom Biostrings unmasked complement
setMethod("subseq", "SequenceTrack", function(x, start = NA, end = NA, width = NA) {
    padding <- "-"
    if (!is.na(start[1] + end[1] + width[1])) {
        warning("All 'start', 'stop' and 'width' are provided, ignoring 'width'")
        width <- NA
    }
    ## We want start and end to be set if width is provided
    if (!is.na(width[1])) {
        if (is.na(start) && is.na(end)) {
            stop("Two out of the three in 'start', 'end' and 'width' have to be provided")
        }
        if (is.na(start)) {
            start <- end - width[1] + 1
        }
        if (is.na(end)) {
            end <- start + width[1] - 1
        }
    }
    if (is.na(start)) {
        start <- 1
    }
    w <- length(x)
    if (w > 0) {
        if (is.na(end)) {
            end <- w
        }
        rstart <- max(1, start[1], na.rm = TRUE)
        rend <- max(rstart, min(end[1], w, na.rm = TRUE))
    } else {
        if (is.na(end)) {
            end <- start
        }
        rend <- end
        rstart <- start
    }
    if (rend < rstart || end < start) {
        stop("'end' has to be bigger than 'start'")
    }
    if ((rend - rstart + 1) > 10e6) {
        stop("Sequence is too big! Unable to extract")
    }
    seqtype <- try(seqtype(x@sequence), silent = TRUE)
    if (is(seqtype, "try-error")) {
        seqtype <- "DNA"
    }
    class <- paste0(seqtype, "String")
    finalSeq <- rep(do.call(class, list(padding)), end - start + 1)
    if (chromosome(x) %in% seqnames(x) && rend >= rstart) {
        chrSeq <- x@sequence[[chromosome(x)]]
        seq <- subseq(chrSeq, start = rstart, end = rend)
        if (is(x, "SequenceBSgenomeTrack")) seq <- unmasked(seq)
        subseq(finalSeq, ifelse(start < 1, abs(start) + 2, 1), width = rend - rstart + 1) <- seq
    }
    if (is(x, "SequenceBSgenomeTrack") && chromosome(x) %in% seqnames(x)) {
        x@pointerCache[[chromosome(x)]] <- x@sequence[[chromosome(x)]]
    }
    if (.dpOrDefault(x, "complement", FALSE)) {
        finalSeq <- complement(finalSeq)
    }
    return(finalSeq)
})


setMethod("subseq", "ReferenceSequenceTrack", function(x, start = NA, end = NA, width = NA) {
    if (sum(c(is.na(start[1]), is.na(end[1]), is.na(width[1]))) >= 2) {
        stop("Two out of the three in 'start', 'end' and 'width' have to be provided")
    }
    if (!is.na(start[1] + end[1] + width[1])) {
        warning("All 'start', 'stop' and 'width' are provided, ignoring 'width'")
        width <- NA
    }
    ## We want start and end to be set if width is provided
    if (!is.na(width[1])) {
        if (is.na(start) && is.na(end)) {
            stop("Two out of the three in 'start', 'end' and 'width' have to be provided")
        }
        if (is.na(start)) {
            start <- end - width[1] + 1
        }
        if (is.na(end)) {
            end <- start + width[1] - 1
        }
    }
    x@sequence <- x@stream(file = x@reference, selection = GRanges(chromosome(x), ranges = IRanges(1, end)))
    return(callNextMethod())
})

#' @describeIn SequenceTrack-class return the chromosome for which the track
#' is defined.
#' @export
setMethod("chromosome", "SequenceTrack", function(GdObject) GdObject@chromosome)

#' @describeIn SequenceTrack-class replace the value of the track's chromosome.
#' This has to be a valid UCSC chromosome identifier or an integer or character
#' scalar that can be reasonably coerced into one.
#' @export
setReplaceMethod("chromosome", "SequenceTrack", function(GdObject, value) {
    GdObject@chromosome <- .chrName(value[1])
    return(GdObject)
})

#' @describeIn SequenceTrack-class Set the track's genome.
#' Usually this has to be a valid UCSC identifier, however this is not
#' formally enforced here.
#' @export
setMethod("genome", "SequenceTrack", function(x) x@genome)


## Annotation Accessors ------------------------------------------------------
## Consolidate ---------------------------------------------------------------
## Before starting of the plotting operation there are a bunch of housekeeping task that should be performed on each
## track, and the mileage may vary between track types, hence we add a layer of abstraction here by using a method.
##
## Available arguments are:
##    o GdObject: the input track object
##    o chromosome: the currently active chromosome which may have to be set for a RangeTrack or a SequenceTrack object
##    o ...: additional arguments that are considered to be display parameters

## For all track types we want to update the display parameters

## For RangeTracks and SequenceTracks we want to set the chromosome

#' @describeIn SequenceTrack-class Consolidate/
#' Determine whether there is `chromosome` settings or not, and add this information.
#' @param GdObject the input track object
#' @param chromosome the currently active chromosome which may have to be set
#' for a `RangeTrack` or a `SequenceTrack` object
#' parameters
# #' @keywords internal
#' @export
setMethod("consolidateTrack", signature(GdObject = "SequenceTrack"), function(GdObject, chromosome, ...) {
    if (!is.null(chromosome)) {
        chromosome(GdObject) <- chromosome
    }
    GdObject <- callNextMethod(GdObject, ...)
    return(GdObject)
})
## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------
## DrawGD --------------------------------------------------------------------

#' @describeIn SequenceTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @export
setMethod("drawGD", signature("SequenceTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, ...) {
    debug <- .dpOrDefault(GdObject, "debug", FALSE)
    if ((is.logical(debug) && debug) || debug == "prepare") {
        browser()
    }
    fcol <- .dpOrDefault(GdObject, "fontcolor", getBioColor("DNA_BASES_N"))
    cex <- max(0.3, .dpOrDefault(GdObject, "cex", 1))
    xscale <- if (!.dpOrDefault(GdObject, "reverseStrand", FALSE)) c(minBase, maxBase) else c(maxBase, minBase)
    pushViewport(viewport(xscale = xscale, clip = TRUE, gp = .fontGp(GdObject, cex = cex)))
    if (prepare) {
        pres <- .pxResolution()
        nsp <- max(as.numeric(convertHeight(stringHeight(stringWidth(DNA_ALPHABET)), "native")))
        nsp <- nsp / pres["y"] * 2
        displayPars(GdObject) <- list("neededVerticalSpace" = nsp)
        popViewport(1)
        return(invisible(GdObject))
    }
    if ((is.logical(debug) && debug) || debug == "draw") {
        browser()
    }
    imageMap(GdObject) <- NULL
    delta <- maxBase - minBase
    if (delta == 0) {
        return(invisible(GdObject))
    }
    lwidth <- max(as.numeric(convertUnit(stringWidth(DNA_ALPHABET), "inches")))
    perLetter <- vpLocation()$isize["width"] / (maxBase - minBase + 1)
    diff <- .pxResolution(.dpOrDefault(GdObject, "min.width", 2), coord = "x")
    ## FIXME: Need to deal with sequences that are too long.
    if (diff > 1 || (maxBase - minBase + 1) >= 10e6) {
        grid.lines(
            x = unit(c(minBase, maxBase), "native"), y = 0.5,
            gp = gpar(
                col = .dpOrDefault(GdObject, "col", "darkgray"),
                lwd = .dpOrDefault(GdObject, "lwd", 2)
            )
        )
    } else {
        sequence <- as.character(as(subseq(GdObject, start = minBase, end = maxBase - 1), "Rle"))
        at <- seq((minBase + 0.5), maxBase - 1 + 0.5, by = 1) # to align sequence (letters) with ticks position
        sequence[sequence == "-"] <- ""
        if (perLetter < 0.5 && .dpOrDefault(GdObject, "add53", FALSE)) {
            sequence[c(1, length(sequence))] <- ""
        }
        col <- fcol[toupper(sequence)]
        if (lwidth < perLetter && !.dpOrDefault(GdObject, "noLetters", FALSE)) {
            grid.text(
                x = unit(at, "native"), y = 0.5, label = sequence, rot = .dpOrDefault(GdObject, "rotation", 0),
                gp = gpar(col = col)
            )
        } else {
            grid.rect(
                x = unit(at, "native"), y = 0.05, width = unit(1, "native"), height = 0.9,
                gp = gpar(fill = col, col = "white"), just = c(0.5, 0)
            )
        }
    }
    ## The direction indicators
    if (.dpOrDefault(GdObject, "add53", FALSE)) {
        if (.dpOrDefault(GdObject, "complement", FALSE)) {
            grid.text(label = expression("3'"), x = unit(minBase + 0.1, "native"), just = c(0, 0.5), gp = gpar(col = "#808080", cex = 0.8))
            grid.text(label = expression("5'"), x = unit(maxBase - 0.1, "native"), just = c(1, 0.5), gp = gpar(col = "#808080", cex = 0.8))
        } else {
            grid.text(label = expression("5'"), x = unit(minBase + 0.1, "native"), just = c(0, 0.5), gp = gpar(col = "#808080", cex = 0.8))
            grid.text(label = expression("3'"), x = unit(maxBase - 0.1, "native"), just = c(1, 0.5), gp = gpar(col = "#808080", cex = 0.8))
        }
    }
    popViewport(1)
    return(invisible(GdObject))
})

## SetAs ---------------------------------------------------------------------

setAs("DNAString", "Rle", function(from, to) Rle(strsplit(as.character(from), "")[[1]]))

setAs("RNAString", "Rle", function(from, to) Rle(strsplit(as.character(from), "")[[1]]))

## Show ----------------------------------------------------------------------

.sequenceTrackInfo <- function(object) {
    msg <- sprintf(
        paste("Sequence track '%s':\n",
            "| genome: %s\n",
            "| chromosomes: %s\n",
            "| active chromosome: %s (%s nucleotides)\n",
            sep = ""
        ),
        names(object),
        genome(object),
        length(seqnames(object)),
        chromosome(object),
        length(object)
    )
    if (length(seqnames(object)) > 1) {
        msg <- paste(msg, "Call seqnames() to list all available chromosomes\n",
            "Call chromosome()<- to change the active chromosome\n",
            sep = ""
        )
    }
    return(msg)
}

## We need to show the name, genome, information about the source BSgenome object as well as the currently active chromosome


#' @describeIn  SequenceTrack-class Show method.
#' @export
setMethod(
    "show", signature(object = "SequenceBSgenomeTrack"),
    function(object) {
        cat(.sequenceTrackInfo(object),
            sprintf(
                paste("Parent BSgenome object:\n",
                    "| organism: %s\n",
                    "| provider: %s\n",
                    "| provider version: %s\n",
                    "| release date: %s\n",
                    # "| release name: %s\n",
                    "| package name: %s\n",
                    sep = ""
                ),
                organism(object@sequence),
                provider(object@sequence),
                .providerVersion(object@sequence),
                releaseDate(object@sequence),
                # releaseName(object@sequence),
                object@sequence@pkgname
            ),
            sep = ""
        )
    }
)

## Here we only need the name, genome and currently active chromosome information
#' @describeIn  SequenceTrack-class Show method.
#' @export
setMethod("show", signature(object = "SequenceDNAStringSetTrack"), function(object) cat(.sequenceTrackInfo(object)))

#' @describeIn  SequenceTrack-class Show method.
#' @export
setMethod("show", signature(object = "SequenceRNAStringSetTrack"), function(object) cat(.sequenceTrackInfo(object)))

#' @describeIn  SequenceTrack-class Show method.
#' @export
setMethod("show", signature(object = "ReferenceSequenceTrack"), function(object) {
    .referenceTrackInfo(object, "ReferenceSequenceTrack")
})

#' @include NumericTrack-class.R
#' @include ReferenceTrack-class.R

NULL

## DataTrack Class -----------------------------------------------------------

#' DataTrack class and methods
#'
#' A class to store numeric data values along genomic coordinates. Multiple
#' samples as well as sample groupings are supported, with the restriction of
#' equal genomic coordinates for a single observation across samples.
#'
#' Depending on the setting of the \code{type} display parameter, the data can
#' be plotted in various different forms as well as combinations thereof.
#' Supported plotting types are:
#'
#' \describe{
#'
#' \item{\code{p}:}{ simple xy-plot.}
#'
#' \item{\code{l}:}{ lines plot. In the case of multiple samples this plotting
#' type is not overly usefull since the points in the data matrix are connected
#' in column-wise order. Type \code{a} might be more appropriate in these
#' situations.}
#'
#' \item{\code{b}:}{ combination of xy-plot and lines plot.}
#'
#' \item{\code{a}:}{ lines plot of the column-wise average values.}
#'
#' \item{\code{s}:}{ sort and connect data points along the x-axis}
#'
#' \item{\code{S}:}{ sort and connect data points along the y-axis}
#'
#' \item{\code{g}:}{ add grid lines. To ensure a consitant look and feel across
#' multiple tracks, grid lines should preferentially be added by using the
#' \code{grid} display parameter.}
#'
#' \item{\code{r}:}{ add a regression line to the plot.}
#'
#' \item{\code{h}:}{ histogram-like vertical lines centered in the middle of
#' the coordinate ranges.}
#'
#' \item{\code{smooth}:}{ add a loess fit to the plot. The following display
#' parameters can be used to control the loess calculation: \code{span, degree,
#' family, evaluation}. See \code{\link{panel.loess}} for details.}
#'
#' \item{\code{histogram}:}{ plot data as a histogram, where the width of the
#' histogram bars reflects the width of the genomic ranges in the \code{range}
#' slot.}
#'
#' \item{\code{mountain}:}{ plot a smoothed version of the data relative to a
#' baseline, as defined by the \code{baseline} display parameter. The following
#' display parameters can be used to control the smoothing: \code{span, degree,
#' family, evaluation}. See \code{\link{panel.loess}} for details. The layout
#' of the plot can be further customized via the following display parameters:
#' \code{col.mountain, lwd.mountain, lty.mountain, fill.mountain}.}
#'
#' \item{\code{polygon}:}{ plot data as a polygon (similar to
#' \code{mountain}-type but without smoothing). Data are plotted relative to a
#' baseline, as defined by the \code{baseline} display parameter. The layout of
#' the plot can be further customized via the following display parameters:
#' \code{col.mountain, lwd.mountain, lty.mountain, fill.mountain}.}
#'
#' \item{\code{boxplot}:}{ plot the data as box-and-whisker plots. The layout
#' of the plot can be further customized via the following display parameters:
#' \code{box.ratio, box.width, varwidt, notch, notch.frac, levels.fos, stats,
#' coef, do.out}. See \code{\link{panel.bwplot}} for details.}
#'
#' \item{\code{gradient}:}{ collapse the data across samples and plot this
#' average value as a color-coded gradient. Essenitally this is similar to the
#' heatmap-type plot of a single sample. The layout of the plot can be further
#' customized via the display parameters \code{ncolor} and \code{gradient}
#' which control the number of gradient colors as well as the gradient base
#' colors, respectively.}
#'
#' \item{\code{heatmap}:}{ plot the color-coded values for all samples in the
#' form of a heatmap. The data for individual samples can be visually separated
#' by setting the \code{separator} display parameter. It's value is taken as
#' the amount of spacing in pixels in between two heatmap rows. The layout of
#' the plot can be further customized via the display parameters \code{ncolor}
#' and \code{gradient} which control the number of gradient colors as well as
#' the gradient base colors, respectively.}
#'
#' \item{\code{horizon}:}{ plot continuous data by cutting the y range into
#' segments and overplotting them with color representing the magnitude and
#' direction of deviation. This is particularly useful when comparing multiple
#' samples, in which case the horizon strips are stacked. See
#' \code{\link{horizonplot}} for details. Please note that the \code{origin}
#' and \code{horizonscale} arguments of the Lattice \code{horizonplot} function
#' are available as display parameters \code{horizon.origin} and
#' \code{horizon.scale}.}
#'
#' }
#'
#' For some of the above plotting-types the \code{groups} display parameter can
#' be used to indicate sample sub-groupings. Its value is supposed to be a
#' factor vector of similar length as the number of samples. In most cases, the
#' groups are shown in different plotting colors and data aggregation
#' operations are done in a stratified fashion.
#'
#' The \code{window} display parameter can be used to aggregate the data prior
#' to plotting. Its value is taken as the number of equal-sized windows along
#' the genomic coordinates of the track for which to compute average values.
#' The special value \code{auto} can be used to automatically determine a
#' reasonable number of windows which can be particularly useful when plotting
#' very large genomic regions with many data points.
#'
#' The \code{aggregation} parameter can be set to define the aggregation
#' function to be used when averaging in windows or across collapsed items. It
#' takes the form of either a function which should condense a numeric vector
#' into a single number, or one of the predefined options as character scalars
#' \code{"mean"}, \code{"median"} or \code{"sum"} for mean, median or
#' summation, respectively. Defaults to computing mean values for each sample.
#' Note that the predefined options can be much faster because they are
#' optimized to work on large numeric tables.
#'
#' @template DataTrack-class_param
#'
#' @name DataTrack-class
#'
#'
#' @return
#' The return value of the constructor function is a new object of class
#' \code{DataTrack} or \code{ReferenceDataTrack}.
#' @section Objects from the class:
#'
#' Objects can be created using the constructor function \code{DataTrack}.
#'
#' @author Florian Hahne
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## Object construction:
#'
#' ## An empty object
#' DataTrack()
#'
#' ## from individual arguments
#' dat <- matrix(runif(400), nrow = 4)
#' dtTrack <- DataTrack(
#'     start = seq(1, 1000, len = 100), width = 10, data = dat,
#'     chromosome = 1, genome = "mm9", name = "random data"
#' )
#'
#' ## from GRanges
#' library(GenomicRanges)
#' gr <- GRanges(seqnames = "chr1", ranges = IRanges(seq(1, 1000, len = 100),
#'     width = 10
#' ))
#' values(gr) <- t(dat)
#' dtTrack <- DataTrack(range = gr, genome = "mm9", name = "random data")
#'
#' ## from IRanges
#' dtTrack <- DataTrack(
#'     range = ranges(gr), data = dat, genome = "mm9",
#'     name = "random data", chromosome = 1
#' )
#'
#' ## from a data.frame
#' df <- as.data.frame(gr)
#' colnames(df)[1] <- "chromosome"
#' dtTrack <- DataTrack(range = df, genome = "mm9", name = "random data")
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(dtTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(dtTrack)
#'
#' ## Track names
#' names(dtTrack)
#' names(dtTrack) <- "foo"
#' plotTracks(dtTrack)
#'
#' ## Subsetting and splitting
#' subTrack <- subset(dtTrack, from = 100, to = 300)
#' length(subTrack)
#' subTrack[1:2, ]
#' subTrack[, 1:2]
#' split(dtTrack, rep(1:2, each = 50))
#'
#' ## Accessors
#' start(dtTrack)
#' end(dtTrack)
#' width(dtTrack)
#' position(dtTrack)
#' width(subTrack) <- width(subTrack) - 5
#'
#' strand(dtTrack)
#' strand(subTrack) <- "-"
#'
#' chromosome(dtTrack)
#' chromosome(subTrack) <- "chrX"
#'
#' genome(dtTrack)
#' genome(subTrack) <- "mm9"
#'
#' range(dtTrack)
#' ranges(dtTrack)
#'
#' ## Data
#' values(dtTrack)
#' score(dtTrack)
#'
#' ## coercion
#' as(dtTrack, "data.frame")
#' @importFrom grDevices boxplot.stats
#'
#' @exportClass DataTrack
setClass("DataTrack",
    contains = "NumericTrack",
    representation = representation(data = "matrix", strand = "character"),
    prototype = prototype(
        columns = c("score"),
        name = "DataTrack",
        dp = DisplayPars(
            aggregateGroups = FALSE,
            aggregation = "mean",
            missingAsZero = TRUE,
            alpha.confint = 0.3,
            amount = NULL,
            baseline = NULL,
            box.legend = FALSE,
            box.ratio = 1,
            box.width = NULL,
            grid = FALSE,
            cex.legend = 0.8,
            cex.sampleNames = NULL,
            cex = 0.7,
            coef = 1.5,
            col.baseline = NULL,
            col.confint = NA,
            col.boxplotFrame = .DEFAULT_SHADED_COL,
            col.histogram = .DEFAULT_SHADED_COL,
            col.horizon = NA,
            col.mountain = NULL,
            col.sampleNames = "white",
            col = trellis.par.get("superpose.line")[["col"]],
            collapse = FALSE,
            degree = 1,
            do.out = TRUE,
            evaluation = 50,
            factor = 0.5,
            family = "symmetric",
            fill.confint = NULL,
            fill.histogram = NULL,
            fill.horizon = c("#B41414", "#E03231", "#F7A99C", "#9FC8DC", "#468CC8", "#0165B3"),
            fill.mountain = c("#CCFFFF", "#FFCCFF"),
            fontface.legend = NULL,
            fontfamily.legend = NULL,
            fontsize.legend = NULL,
            fontcolor.legend = .DEFAULT_SHADED_COL,
            gradient = RColorBrewer::brewer.pal(9, "Blues"),
            groups = NULL,
            horizon.origin = 0,
            horizon.scale = NULL,
            jitter.x = FALSE,
            jitter.y = FALSE,
            levels.fos = NULL,
            legend = TRUE,
            lineheight.legend = NULL,
            lty.baseline = NULL,
            lty.mountain = NULL,
            lwd.baseline = NULL,
            lwd.mountain = NULL,
            min.distance = 0,
            na.rm = FALSE,
            ncolor = 100,
            notch.frac = 0.5,
            notch = FALSE,
            pch = 20,
            separator = 0,
            showColorBar = TRUE,
            showSampleNames = FALSE,
            size = NULL,
            span = 1 / 5,
            stackedBars = TRUE,
            stats = boxplot.stats,
            transformation = NULL,
            type = "p",
            varwidth = FALSE,
            window = NULL,
            windowSize = NULL,
            ylim = NULL,
            yTicksAt = NULL
        )
    )
)

## Initialize ----------------------------------------------------------------

## Only pass on the stuff to the GdObject initializer

#' @describeIn DataTrack-class  Initialize.
#' @export
setMethod("initialize", "DataTrack", function(.Object, data = matrix(), strand, ...) {
    ## the display parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "DataTrack")
    .Object@data <- data
    if (!missing(strand)) {
        .Object@strand <- unique(strand)
    }
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## The file-based version of the DataTrack class. This will mainly provide a means to dispatch to
## a special 'subset' method which should stream the necessary data from disk.

#' @describeIn DataTrack-class The file-based version of the `DataTrack-class`.
#' @exportClass ReferenceDataTrack
setClass("ReferenceDataTrack", contains = c("DataTrack", "ReferenceTrack"))

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritance has some strange features with regards to method selection

#' @describeIn DataTrack-class  Initialize.
#' @export
setMethod("initialize", "ReferenceDataTrack", function(.Object, stream, reference, mapping = list(),
                                                       args = list(), defaults = list(), ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object = .Object, reference = reference, stream = stream,
        mapping = mapping, args = args, defaults = defaults)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

## Constructor. The following arguments are supported:
##    o range: an object of class 'GRanges' containing the data coordinates, or an object of class data.frame
##       with the two mandatroy columns 'start' and 'end'. The coordinates (i.e., the length of the GRanges object)
##       have to match the columns of the data matrix (see below).
##    o data: a numeric matrix of data points with number of columns equal to the number of coordinates in 'range',
##      or a numeric vector of appropriate length that will be coerced in such a one-row matrix.
##    o start, end, width: numeric vectors of the item start and end coordinates, have to match the columns in 'data'
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o strand: character, display data on the plus strand ("+" or 0), the minus strand ("-" or 1) or both
##      strands ("+-" or "-+" or 2). Currently has to be unique for the whole track.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters

#' @describeIn DataTrack-class Constructor function for `DataTrack-class`
#' @export
DataTrack <- function(range = NULL, start = NULL, end = NULL, width = NULL, data, chromosome, strand, genome,
                      name = "DataTrack", importFunction, stream = FALSE, ...) {
    ## Build a GRanges object from the inputs
    wasGR <- is(range, "GRanges") || is.character(range)
    fromFile <- is.character(range)
    isStream <- FALSE
    .missingToNull(c("strand", "chromosome", "importFunction", "genome"))
    args <- list(strand = strand, chromosome = chromosome, genome = genome)
    defs <- list(strand = "*", chromosome = "chrNA")
    range <- .buildRange(
        range = range, start = start, end = end, width = width, args = args, defaults = defs,
        asIRanges = FALSE, chromosome = chromosome, genome = NA, trackType = "DataTrack",
        importFun = importFunction, stream = stream
    )
    if (is.list(range)) {
        isStream <- TRUE
        slist <- range
        range <- GRanges()
    }
    ## Some default checking
    if (length(unique(strand(range))) > 1) {
        stop("The strand has to be unique for all ranges in a DataTrack object.")
    }
    if (!missing(data) && length(range) > 0) {
        if (is.character(data)) {
            if (!wasGR) {
                stop("Columns indices for the data section are only allowed when 'range' is of class 'GRanges'")
            }
            mt <- is.na(match(data, colnames(values(range))))
            if (any(mt)) {
                warning("Unable to match data columns: ", paste(data[mt], collapse = ","))
            }
            data <- as.data.frame(values(range)[, data[!mt], drop = FALSE])
        }
        if (is.null(dim(data))) {
            dim(data) <- c(1, length(data))
        }
        if (is.matrix(data)) {
            data <- as.data.frame(t(data))
        }
    } else {
        data <- if (ncol(values(range))) as.data.frame(values(range)) else matrix(nrow = 0, ncol = 0)
    }
    data <- .prepareDtData(data, len = length(range))
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- if (length(range) > 0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    }
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    values(range) <- NULL
    if (!isStream) {
        return(new("DataTrack",
            chromosome = chromosome, strand = as.character(strand(range)), range = range,
            name = name, genome = genome, data = data, ...
        ))
    } else {
        ## A bit hackish but for some functions we may want to know which track type we need but at the
        ## same time we do not want to enforce this as an additional argument
        e <- new.env()
        e[["._trackType"]] <- "DataTrack"
        environment(slist[["stream"]]) <- e
        return(new("ReferenceDataTrack",
            chromosome = chromosome, strand = as.character(strand(range)), range = range,
            name = name, genome = genome, data = data, stream = slist[["stream"]], reference = slist[["reference"]],
            mapping = slist[["mapping"]], args = args, defaults = defs, ...
        ))
    }
}

## General accessors ---------------------------------------------------------

#' @describeIn DataTrack-class return the raw data values of the object, i.e.,
#' the data matrix in the `data` slot.
#' @export
setMethod("values", "DataTrack", function(x, all = FALSE) {
    if (sum(dim(x@data)) == 0) {
        x@data
    } else {
        sel <- if (all) rep(TRUE, ncol(x@data)) else seqnames(x) == chromosome(x)
        x@data[, sel, drop = FALSE]
    }
})

#' @describeIn DataTrack-class Replace the data matrix in the `data` slot.
#' @export
setReplaceMethod("values", "DataTrack", function(x, value) {
    if (!is.matrix(value)) {
        if (!is.numeric(value) || length(value) != length(x)) {
            stop("Not numeric or invalid length of replacement vector.")
        } else {
            value <- t(as.matrix(value))
        }
    } else {
        if (!is.numeric(value) || ncol(value) != length(x)) {
            stop("Not numeric or dimensions of replacement value do not match.")
        }
    }
    x@data <- value
    return(x)
})

#' @describeIn DataTrack-class return a vector of strand specifiers for all
#' track items, in the form '+' for the Watson strand, '-' for the Crick
#' strand or '*' for either of the two.
#' @export
setMethod("strand", "DataTrack", function(x) x@strand)

#' @describeIn DataTrack-class replace the strand information for the track items.
#' The replacement value needs to be an appropriate scalar or vector of strand values.
#' @export
setReplaceMethod("strand", "DataTrack", function(x, value) {
    if (!is.character(value) || length(value) != 1 || !value %in% c("+", "-", "*")) {
        stop("Invalid replacement value")
    }
    x@strand <- value
    return(x)
})

#' @describeIn DataTrack-class Split a `DataTrack` object by an appropriate
#' factor vector (or another vector that can be coerced into one).
#' The output of this operation is a list of `DataTrack` objects.
#' @export
setMethod("split", signature("DataTrack"),
    definition = function(x, f, ...) {
        f <- factor(f)
        rs <- as.list(split(ranges(x), f))
        ds <- split(t(values(x)), f)
        nr <- nrow(values(x))
        rnms <- rownames(values(x))
        mapply(function(y, z) {
            x@range <- y
            x@data <- matrix(z, nrow = nr, byrow = TRUE, dimnames = list(rnms, NULL))
            return(x)
        }, rs, ds)
    }
)

## Annotation Accessors ------------------------------------------------------

#' @describeIn DataTrack-class returns NULL since there is no grouping
#' information for the ranges in a `DataTrack`.
#' @export
setMethod("feature", signature(GdObject = "DataTrack"), function(GdObject) NULL)

#' @describeIn DataTrack-class this return the unaltered input object since
#' there is no grouping information for the ranges in a `DataTrack`.
#' @export
setReplaceMethod("feature", signature("DataTrack", "character"), function(GdObject, value) GdObject)

## Consolidate ---------------------------------------------------------------
## Collapse  -----------------------------------------------------------------

#' @importFrom Biobase rowMax rowMin
#' @importFrom matrixStats rowMedians rowMins
.aggregator <- function(GdObject) {
    agFun <- .dpOrDefault(GdObject, "aggregation", "mean")
    if (is.list(agFun)) {
        agFun <- agFun[[1]]
    }
    fun <- if (is.character(agFun)) {
        switch(agFun,
            "mean" = rowMeans,
            "sum" = rowSums,
            "median" = rowMedians,
            "extreme" = function(x) apply(x, 1, .extreme),
            "min" = Biobase::rowMin,
            "max" = Biobase::rowMax,
            rowMeans
        )
    } else {
        if (is.function(agFun)) {
            function(x) apply(x, 1, agFun)
        } else {
            stop(
                "display parameter 'aggregation' has to be a function or a character ",
                "scalar in c('mean', 'median', 'sum', 'min', 'max')"
            )
        }
    }
    return(fun)
}

.runaggregator <- function(GdObject) {
    agFun <- .dpOrDefault(GdObject, "aggregation", "mean")
    if (is.list(agFun)) {
        agFun <- agFun[[1]]
    }
    fun <- if (is.character(agFun)) {
        switch(agFun,
            "mean" = runmean,
            "sum" = runsum,
            "median" = runmed2 <- function(x, k, na.rm = FALSE, ...) {
                na.action <- if (na.rm) {
                    "na.omit"
                } else {
                    "+Big_alternate"
                }
                runmed(x = as.numeric(x), k = k, na.action = na.action)
            },
            "min" = runqmin <- function(x, k, i = 1, ...) {
                runq(x = x, k = k, i = i, ...)
            },
            "max" = runqmax <- function(x, k, i = k, ...) {
                runq(x = x, k = k, i = i, ...)
            },
            runmean
        )
    } else {
        if (is.function(agFun)) {
            # na.rm currently not implemented...
            function(x, k, na.rm = FALSE, endrule = "constant") {
                ans <- vapply(0:(length(x) - k), function(offset) agFun(x[seq_len(k) + offset]), FUN.VALUE = numeric(1))
                ans <- Rle(ans)
                if (endrule == "constant") {
                    j <- (k + 1L) %/% 2L
                    runLength(ans)[1L] <- runLength(ans)[1L] + (j - 1L)
                    runLength(ans)[nrun(ans)] <- runLength(ans)[nrun(ans)] + (j - 1L)
                }
                ans
            }
        } else {
            stop(
                "display parameter 'aggregation' has to be a function or a character ",
                "scalar in c('mean', 'median', 'sum', 'min', 'max')"
            )
        }
    }
    return(fun)
}

## For DataTracks we want to collapse data values using the aggregation function provided by calling .aggregator().
## In addition values can be aggregated over fixed window slices when gpar 'window' is not NULL, and using a sliding
## window approach when 'window' == -1

#' @describeIn DataTrack-class preprocess the track before plotting.
#' This will collapse overlapping track items based on the available resolution
#' and increase the width and height of all track objects to a minimum value
#' to avoid rendering issues. See collapsing for details.
#' @keywords internal
setMethod("collapseTrack", signature(GdObject = "DataTrack"), function(GdObject, diff = .pxResolution(coord = "x"), xrange) {
    if (!length(GdObject)) {
        return(GdObject)
    }
    ## first the data transformation if needed
    values(GdObject) <- score(GdObject)
    collapse <- .dpOrDefault(GdObject, "collapse", FALSE)
    min.width <- .dpOrDefault(GdObject, "min.width", 2)
    min.distance <- max(0, .dpOrDefault(GdObject, "min.distance", 0))
    ## When an averaging window has been set, split the data up into these average chunks
    window <- .dpOrDefault(GdObject, "window")
    windowSize <- .dpOrDefault(GdObject, "windowSize")
    missingAsZero <- .dpOrDefault(GdObject, "missingAsZero", TRUE)
    if (!is.null(window) || collapse) {
        GdObject <- GdObject[, order(range(GdObject))]
    }
    r <- ranges(GdObject)
    drange <- c(floor(xrange[1]), ceiling(xrange[2]))
    if (!is.null(window)) {
        rr <- if (is(r, "GRanges")) ranges(r) else r
        fw <- FALSE
        if (window == "auto") {
            window <- min(
                ncol(values(GdObject)), 1000,
                ceiling(width(range(rr)) / (min.width * diff))
            )
        }
        if (window == "fixed") {
            fw <- TRUE
            window <- 100
        }
        if (!is.numeric(window) || length(window) != 1L) {
            stop("gpar 'window' must be a numeric scalar")
        }
        window <- as.integer(window)
        sc <- values(GdObject)
        agFun <- .aggregator(GdObject)
        if (window == 1) {
            sc <- matrix(agFun(sc), ncol = 1)
            rtmp <- IRanges(start = max(1, drange[1]), end = max(1, drange[2] - 1))
            r <- if (is(r, "GRanges")) GRanges(seqnames = seqnames(r)[1], ranges = rtmp) else rtmp
        } else if (window < 1) {
            if (is.null(windowSize)) {
                windowSize <- (max(GdObject) - min(GdObject)) / 100
            }
            if (windowSize %% 2 != 1) {
                windowSize <- windowSize + 1
            }
            if (missingAsZero) {
                rm <- vector("integer", width(range(range(GdObject))))
            } else {
                rm <- as.integer(rep(NA, width(range(range(GdObject)))))
            }
            ind <- unlist(mapply(function(x, y) x:y, start(GdObject), end(GdObject))) - min(GdObject) + 1
            rm[ind] <- rep(sc[1, ], width(GdObject))
            runAgFun <- .runaggregator(GdObject)
            runwin <- suppressWarnings(runAgFun(Rle(as.numeric(rm)), k = windowSize, endrule = "constant", na.rm = TRUE))
            seqSel <- findRun(as.integer(position(GdObject)) - min(GdObject) + 1, runwin)
            newDat <- matrix(runValue(runwin)[seqSel], nrow = 1)
            if (nrow(sc) > 1) {
                newDat <- rbind(
                    newDat,
                    do.call(rbind, lapply(2:nrow(sc), function(x) {
                        rm[ind] <- rep(sc[x, ], width(GdObject))
                        runwin <- suppressWarnings(runAgFun(Rle(as.numeric(rm)), k = windowSize, endrule = "constant", na.rm = TRUE))
                        seqSel <- findRun(as.integer(position(GdObject)) - min(GdObject) + 1, runwin)
                        runValue(runwin)[seqSel]
                        # suppressWarnings(runValue(runmean(Rle(as.numeric(rm)), k = windowSize, endrule = "constant", na.rm = TRUE)))[seqSel]
                    }))
                )
            }
            sc <- newDat
        } else {
            if (!is.null(window) && window > diff(drange)) {
                window <- diff(drange)
            }
            if (!fw || is.null(windowSize)) {
                windowSize <- diff(drange) %/% window
            } else {
                window <- max(1, diff(drange) %/% windowSize)
            }
            remain <- (diff(drange) - (window * windowSize)) / 2
            ir <- IRanges(
                start = seq(from = drange[1] + remain, to = drange[2] - remain - windowSize, length.out = window),
                width = windowSize
            )
            if (remain > 0) {
                ir <- c(
                    IRanges(start = drange[1], width = ceiling(remain)), ir,
                    IRanges(start = drange[2] - ceiling(remain), width = ceiling(remain))
                )
            }
            ol <- as.matrix(findOverlaps(ir, rr))
            scn <- lapply(split(ol[, 2], ol[, 1]), function(i) agFun(sc[, i, drop = FALSE]))
            scn <- do.call(cbind, scn)
            colnames(scn) <- as.character(unique(ol[, 1]))
            sc <- matrix(NA, ncol = length(ir), nrow = nrow(scn))
            sc[, as.integer(colnames(scn))] <- scn
            r <- if (is(r, "GRanges")) {
                GRanges(
                    seqnames = chromosome(GdObject), ranges = ir,
                    strand = unique(as.character(strand(GdObject)))
                )
            } else {
                ir
            }
        }
        GdObject@range <- r
        GdObject@data <- sc
    }
    ## If groups need to be averaged we have to do it here
    groups <- .dpOrDefault(GdObject, "groups")
    if (!is.null(groups) && .dpOrDefault(GdObject, "aggregateGroups", FALSE)) {
        if (!is.factor(groups)) {
            groups <- factor(groups)
        }
        agFun <- .aggregator(GdObject)
        dat <- values(GdObject)
        rownames(dat) <- groups
        datNew <- do.call(rbind, lapply(levels(groups), function(x) agFun(t(dat[groups == x, , drop = FALSE]))))
        GdObject@data <- datNew
        displayPars(GdObject) <- list(groups = levels(groups))
    }
    ## Compute native coordinate equivalent to 1 pixel and resize
    r <- .resize(r, min.width, diff)
    ## Collapse overlapping ranges (less than minXDist space between them) including the associated attributes using
    ## "|" as separator. For both "strand" and "feature" we take the first available entry, which is not optimal but
    ## seems to be the sanest thing to do here...
    if (collapse) {
        minXDist <- min.distance * diff
        rr <- if (is(r, "GRanges")) ranges(r) else r
        if (minXDist < 1) {
            ## We have to fake smaller ranges because reduce will merge also neighboring ranges
            width(rr) <- width(rr) - 1
            rr <- reduce(rr, min.gapwidth = minXDist)
            width(rr) <- width(rr) + 1
        } else {
            rr <- reduce(r, min.gapwidth = minXDist)
        }
        sc <- values(GdObject)
        if (length(rr) == 1) {
            r <- GRanges(seqnames = 1, strand = strand(GdObject)[1], ranges = rr)
            GdObject@range <- r
            GdObject@data <- matrix(rowMeans(sc, na.rm = TRUE), ncol = 1)
        } else if (length(rr) < length(r)) {
            startInd <- sort(unique(vapply(start(rr), function(x) which(start(r) == x), FUN.VALUE = numeric(1L))))
            st <- strand(GdObject)
            startInd <- if (tail(startInd, 1) == length(r)) c(startInd, length(r) + 1) else c(startInd, length(r))
            vsplit <- split(t(as.data.frame(sc, stringsAsFactors = FALSE)), cut(seq_len(length(r)), startInd, iclude.lowest = TRUE, right = FALSE))
            agFun <- .dpOrDefault(GdObject, "aggregation", "mean")
            if (is.list(agFun)) {
                agFun <- agFun[[1]]
            }
            newScore <- if (is.character(agFun)) {
                switch(agFun,
                    "mean" = lapply(vsplit, function(x) rowMeans(matrix(x, nrow = nrow(sc), byrow = TRUE), na.rm = TRUE)),
                    "sum" = lapply(vsplit, function(x) rowSums(matrix(x, nrow = nrow(sc), byrow = TRUE), na.rm = TRUE)),
                    "median" = lapply(vsplit, function(x) rowMedians(matrix(x, nrow = nrow(sc), byrow = TRUE), na.rm = TRUE)),
                    lapply(vsplit, function(x) rowMeans(matrix(x, nrow = nrow(sc), byrow = TRUE), na.rm = TRUE))
                )
            } else {
                if (is.function(agFun)) {
                    lapply(vsplit, function(x) apply(matrix(x, nrow = nrow(sc), byrow = TRUE), 1, function(y) agFun(y)[1]))
                } else {
                    stop("display parameter 'aggregation' has to be a function or a character ", "scalar in c('mean', 'median', 'sum')")
                }
            }
            newScore <- unlist(newScore)
            r <- GRanges(seqnames = seq_len(length(rr)), strand = st, ranges = rr)
            GdObject@data <- newScore
            GdObject@range <- r
        }
    }
    ## Reconstruct the RangedData object and return
    GdObject@range <- r
    return(GdObject)
})

## Subset --------------------------------------------------------------------

#' @describeIn DataTrack-class subset the items in the `DataTrack` object.
#' This is essentially similar to subsetting of the `GRanges` object in the
#' `range` slot. For most applications, the subset method may be more appropriate.
#' @export
setMethod("[", signature(x = "DataTrack"), function(x, i, j, ..., drop = FALSE) {
    x <- .deepCopyPars(x)
    if (!missing(i)) {
        x@data <- x@data[i, , drop = drop]
        displayPars(x) <- list(groups = as.vector(.dpOrDefault(x, "groups")[i]))
    }
    if (!missing(j)) {
        x@range <- x@range[j, ]
        if (ncol(x@data) > 0) {
            x@data <- x@data[, j, drop = drop]
        }
    }
    return(x)
})

## For DataTracks we cut exactly, and also reduce to the current chromosome unless told explicitly not to

#' @describeIn DataTrack-class Subset a `DataTrack` by coordinates
#' and sort if necessary.
#' @export
setMethod("subset", signature(x = "DataTrack"), function(x, from = NULL, to = NULL, sort = FALSE, drop = TRUE, use.defaults = TRUE, ...) {
    ## Subset to a single chromosome first
    if (drop) {
        csel <- seqnames(x) != chromosome(x)
        if (any(csel)) {
            x <- x[, !csel]
        }
    }
    if (!length(x)) {
        return(x)
    }
    ranges <- if (use.defaults) .defaultRange(x, from = from, to = to) else c(from = ifelse(is.null(from), -Inf, from), to = ifelse(is.null(to), Inf, to))
    x <- x[, start(x) >= ranges["from"] & end(x) <= ranges["to"]]
    if (sort) {
        x <- x[, order(range(x))]
    }
    return(x)
})

## ReferenceDataTracks need to stream the data from file and then pass the results on to the next method

#' @describeIn DataTrack-class Subset a `ReferenceDataTrack` by coordinates
#' and sort if necessary.
#' @export
setMethod("subset", signature(x = "ReferenceDataTrack"), function(x, from, to, chromosome, ...) {
    ## We only need to reach out into the referenced file once if the range is already contained in the object
    if (missing(from) || is.null(from) || missing(to) || is.null(to)) {
        stop("Need both start and end location to subset a ReferenceDataTrack")
    }
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- Gviz::chromosome(x)
    }
    subRegion <- GRanges(seqnames = chromosome[1], ranges = IRanges(start = from, end = to))
    if (length(ranges(x)) == 0 || !all(overlapsAny(ranges(x), subRegion))) {
        vals <- x@stream(x@reference, subRegion)
        x@range <- vals
        mcols(x@range) <- NULL
        x@data <- .prepareDtData(if (ncol(values(vals))) as.data.frame(values(vals)) else matrix(nrow = 0, ncol = 0), length(vals))
        chromosome(x) <- chromosome[1]
    }
    return(callNextMethod(x = x, from = from, to = to, drop = FALSE, ...))
})

## Position ------------------------------------------------------------------

setMethod("score", signature("DataTrack"), function(x, from = NULL, to = NULL, sort = FALSE, transformation = TRUE, ...) {
    if (!is.null(from) && !is.null(to)) {
        x <- subset(x, from = from, to = to, sort = sort, ...)
    }
    vals <- values(x)
    ## apply data transformation if one is set up
    trans <- .dpOrDefault(x, "transformation")
    if (is.list(trans)) {
        trans <- trans[[1]]
    }
    if (transformation && !is.null(trans)) {
        if (!is.function(trans) || length(formals(trans)) != 1L) {
            stop("gpar 'transformation' must be a function with a single argument")
        }
        test <- trans(vals)
        if (!is.numeric(test) || !is.matrix(test) || !all(dim(test) == dim(vals))) {
            stop(
                "The function in gpar 'transformation' results in invalid output.\n",
                "It has to return a numeric matrix with the same dimensions as the input data."
            )
        }
        vals <- test
    }
    return(vals)
})

## DrawAxis ------------------------------------------------------------------

#' @describeIn DataTrack-class add a y-axis to the title panel of a track.
#' @export
setMethod("drawAxis", signature(GdObject = "DataTrack"), function(GdObject, ...) {
    if (as.logical(.dpOrDefault(GdObject, "legend", FALSE)) && !is.null(.dpOrDefault(GdObject, ".__groupLevels"))) {
        pushViewport(viewport(
            y = 1, height = unit(1, "npc") - unit(.dpOrDefault(GdObject, ".__verticalSpace"), "inches"),
            just = c(0.5, 1)
        ))
        on.exit(popViewport(1))
    }
    type <- match.arg(.dpOrDefault(GdObject, "type", "p"), .PLOT_TYPES, several.ok = TRUE)
    isOnlyHoriz <- length(setdiff(type, "horizon")) == 0
    if (!isOnlyHoriz && .dpOrDefault(GdObject, "showAxis", TRUE)) {
        callNextMethod()
    } else {
        if (.dpOrDefault(GdObject, "showSampleNames", FALSE)) {
            groups <- .dpOrDefault(GdObject, "groups")
            sn <- if (is.null(groups)) rownames(values(GdObject)) else rev(unlist(split(rownames(values(GdObject)), factor(groups))))
            cex.sn <- .dpOrDefault(GdObject, "cex.sampleNames", .dpOrDefault(GdObject, "cex.axis", 1))
            col.cn <- .dpOrDefault(GdObject, "col.sampleNames", "white")
            wd <- max(as.numeric(convertWidth(stringWidth(sn) + unit(10, "points"), "npc"))) * cex.sn
            samNames <- viewport(x = 1, width = wd, just = 1, yscale = c(-0.05, 1.05))
            pushViewport(samNames)
            nr <- nrow(values(GdObject))
            if (nr > 1) {
                yy <- head(seq(0.05, 0.95, len = nr + 1), -1)
                yy <- yy + diff(yy)[[1]] / 2
            } else {
                yy <- 0.5
            }
            grid.text(x = rep(0.5, nr), y = yy, label = rev(sn), just = 0.5, gp = gpar(cex = cex.sn, col = col.cn))
            popViewport(1)
        }
    }
})

## DrawGD --------------------------------------------------------------------

## Helper function to return the absolute extreme value in a vector
.extreme <- function(x) if (all(is.na(x))) NA else x[which.max(abs(x))]

## Map numeric range to values from 1 to n
.z2icol <- function(z, n, xrange = range(z, na.rm = TRUE)) {
    res <- round((z - xrange[1]) / diff(xrange) * (n - 1)) + 1
    res[res > n] <- n
    res[res < 1] <- 1
    return(res)
}

#' @describeIn DataTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @importFrom grDevices boxplot.stats
#' @importFrom latticeExtra panel.horizonplot
#' @export
setMethod("drawGD", signature("DataTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...) {
    debug <- .dpOrDefault(GdObject, "debug", FALSE)
    if ((is.logical(debug) && debug) || debug == "prepare") {
        browser()
    }
    imageMap(GdObject) <- NULL
    type <- .dpOrDefault(GdObject, "type", "p")
    type <- match.arg(type, .PLOT_TYPES, several.ok = TRUE)
    ## Grouping may be useful for some of the plot types, may be ignored for others
    vals <- values(GdObject)
    groups <- .dpOrDefault(GdObject, "groups")
    if (!is.null(groups) && length(groups) != nrow(vals)) {
        stop("'groups' must be a vector of the same length as the number of rows in the data matrix (", nrow(vals), ")")
    }
    if (!is.null(groups) && !is.factor(groups)) {
        groups <- factor(groups)
    }
    stacked <- .dpOrDefault(GdObject, "stackedBars", FALSE)
    ## The general "col" parameter should be the default for all relevant colors except when there are groups.
    pcols <- .getPlottingFeatures(GdObject)
    ## In prepare mode we collapse the track to allow for aggregation and so on since we need the final data
    ## values to draw the axis.
    if (prepare) {
        if (subset) {
            GdObject <- subset(GdObject, from = minBase, to = maxBase)
        }
        xscale <- if (!.dpOrDefault(GdObject, "reverseStrand", FALSE)) c(minBase, maxBase) else c(maxBase, minBase)
        pushViewport(viewport(xscale = xscale, yscale = c(0, 1), clip = TRUE))
        diff <- .pxResolution(coord = "x")
        GdObject <- collapseTrack(GdObject, diff = diff, xrange = c(minBase, maxBase))
        popViewport(1)
        ## If we have groups and stacked histograms we have to adjust the ylim values, also for regular histograms
        if ("histogram" %in% type) {
            vals <- values(GdObject)
            groups <- rep(groups, ncol(vals))
            ylim <- .dpOrDefault(GdObject, "ylim")
            if (!is.null(groups) && nlevels(groups) > 1) {
                if (ncol(vals)) {
                valsS <- split(vals, groups)
                valsS <- do.call(cbind, lapply(seq_along(valsS), function(i) {
                    tmp <- t(matrix(valsS[[i]], ncol = ncol(vals)))
                    if (ncol(tmp)) {
                        colnames(tmp) <- rep(names(valsS)[i], ncol(tmp))
                    }
                    tmp
                }))
                } else {
                    valsS <- matrix(nrow = nlevels(groups), ncol = 0, dimnames = list(levels(groups)))
                }
                displayPars(GdObject) <- list(".__valsS" = valsS)
                if (stacked == TRUE && is.null(ylim)) {
                    ylim <- suppressWarnings(range(unlist(apply(valsS, 1, function(x) {
                        x <- x[!is.na(x)]
                        sel <- x >= 0
                        tmp <- NULL
                        if (!all(is.na(sel))) {
                            if (any(sel)) {
                                tmp <- c(min(x[sel]), sum(x[sel]))
                            }
                            if (any(!sel)) {
                                tmp <- c(max(x[!sel]), tmp, sum(x[!sel]))
                            }
                        }
                        tmp
                    }))))
                    if (length(type) > 1) {
                        ylim <- range(c(ylim, vals))
                    }
                    displayPars(GdObject) <- list(ylim = ylim)
                }
            } else {
                if (is.null(ylim)) {
                     valsA <- t(vals)
                    ylim <- if (!length(valsA)) c(-1, 1) else c(min(c(0, valsA), na.rm = TRUE), max(valsA, na.rm = TRUE))
                    if (length(type) > 1) {
                        ylim <- range(c(ylim, vals), na.rm = TRUE)
                    }
                    displayPars(GdObject) <- list(ylim = ylim)
                }
            }
        }
        ## If we want a legend we have to figure out how much vertical space is needed
        grps <- .dpOrDefault(GdObject, "groups")
        if (!is.factor(grps)) {
            grps <- factor(grps)
        }
        if (is.null(grps) || nlevels(grps) == 1 || length(setdiff(type, c("gradient", "mountain", "grid", "horizon"))) == 0) {
            displayPars(GdObject) <- list(legend = FALSE)
        }
        if (as.logical(as.logical(.dpOrDefault(GdObject, "legend", FALSE))) && nlevels(grps) > 1) {
            pushViewport(viewport(width = unit(1, "npc") - unit(0.2, "inches"), gp = .fontGp(GdObject, "legend")))
            grps <- levels(grps)
            legInfo <- .legendInfo()[type, , drop = FALSE]
            for (i in colnames(legInfo)) {
                legInfo[, i] <- any(legInfo[, i]) && !any(duplicated(pcols[[i]][seq_along(grps)]))
            }
            legFactors <- sort(names(which(apply(legInfo, 2, any))))
            boxSize <- if (length(setdiff(legFactors, c("col", "cex"))) == 0) 0.1 else 0.3
            spacing <- 0.1
            hspacing <- 0.02
            lengths <- as.numeric(convertUnit(stringWidth(grps), "inches"))
            heights <- as.numeric(convertWidth(stringHeight(grps), "inches"))
            colWidth <- max(lengths + boxSize + spacing * 2)
            availSpace <- vpLocation()$isize
            colNum <- max(1, availSpace["width"] %/% colWidth)
            rowNum <- ceiling(length(grps) / colNum)
            rowHeight <- max(c(heights, 0.1))
            vertSpace <- (rowHeight * rowNum) + (hspacing * (rowNum - 1)) + 0.2
            displayPars(GdObject) <- list(
                ".__verticalSpace" = vertSpace, ".__layoutDims" = c(rowNum, colNum),
                ".__boxSize" = boxSize, ".__spacing" = spacing, ".__groupLevels" = grps,
                ".__legFactors" = legFactors
            )
            popViewport(1)
        }
        return(invisible(GdObject))
    }
    if ((is.logical(debug) && debug) || debug == "draw") {
        browser()
    }
    ## We only proceed if there is something to draw within the ranges, but still may have to add the grid and the legend.
    ## Legend drawing causes another viewport for all the other graphics to be opened and will be called after all other
    ## drawing has finished, hence we call it in on.exit
    if (subset) {
        GdObject <- subset(GdObject, from = minBase, to = maxBase)
    }
    alpha <- .dpOrDefault(GdObject, "alpha", 1)
    ## The optional legend is plotted below the data
    grpLevels <- .dpOrDefault(GdObject, ".__groupLevels")
    if (as.logical(.dpOrDefault(GdObject, "legend", FALSE)) && !is.null(grpLevels)) {
        lSpace <- .dpOrDefault(GdObject, ".__verticalSpace")
        pushViewport(viewport(
            y = 1, height = unit(1, "npc") - unit(lSpace, "inches"),
            just = c(0.5, 1)
        ))
        on.exit({
            popViewport(1)
            cex <- .dpOrDefault(GdObject, "cex.legend", 0.8)
            legFactors <- .dpOrDefault(GdObject, ".__legFactors", character())
            pushViewport(viewport(y = 0, height = unit(lSpace, "inches"), just = c(0.5, 0), gp = .fontGp(GdObject, "legend")))
            pushViewport(viewport(width = unit(1, "npc") - unit(0.1, "inches"), height = unit(1, "npc") - unit(0.1, "inches")))
            boxSize <- .dpOrDefault(GdObject, ".__boxSize")
            spacing <- .dpOrDefault(GdObject, ".__spacing")
            dims <- .dpOrDefault(GdObject, ".__layoutDims")
            for (i in seq_along(grpLevels)) {
                grpLev <- grpLevels[i]
                row <- (((i) - 1) %/% dims[2]) + 1
                col <- (((i) - 1) %% dims[2]) + 1
                pushViewport(viewport(width = 1 / dims[2], height = 1 / dims[1], x = (1 / dims[2]) * (col - 1), y = 1 - ((1 / dims[1]) * (row - 1)), just = c(0, 1)))
                grid.rect(gp = gpar(col = "transparent", fill = .dpOrDefault(GdObject, "background.legend", "transparent")))
                if (length(setdiff(legFactors, c("col"))) == 0) {
                    grid.rect(
                        width = unit(boxSize, "inches"), height = unit(boxSize, "inches"), x = 0, just = c(0, 0.5),
                        gp = gpar(fill = pcols$col[grpLev], col = .DEFAULT_SHADED_COL)
                    )
                } else {
                    if (any(c("pch", "col.symbol") %in% legFactors)) {
                        panel.points(unit(boxSize / 2, "inches"), 0.5, pch = pcols$pch[grpLev], cex = pcols$cex[grpLev], col = pcols$col.symbol[grpLev])
                    }
                    if (any(c("lwd", "lty", "col.lines") %in% legFactors)) {
                        ## panel.lines(unit(c(0,boxSize), "inches"), c(0.5, 0.5), col=pcols$col.line[grpLev], lwd=pcols$lwd[grpLev], lty=pcols$lty[grpLev])
                        grid.lines(unit(c(0, boxSize), "inches"), c(0.5, 0.5), gp = gpar(col = pcols$col.line[grpLev], lwd = pcols$lwd[grpLev], lty = pcols$lty[grpLev]))
                    }
                }
                grid.text(x = unit(boxSize + spacing, "inches"), y = 0.5, just = c(0, 0.5), label = grpLevels[i])
                popViewport(1)
            }
            if (.dpOrDefault(GdObject, "box.legend", FALSE)) {
                grid.rect(width = (1 / dims[2]) * length(grpLevels), x = 0, just = "left", gp = gpar(fill = NA))
            }
            popViewport(2)
        })
    }
    if (!length(GdObject)) {
        if ("g" %in% type) {
            panel.grid(
                h = .dpOrDefault(GdObject, "h", -1), v = .dpOrDefault(GdObject, "v", -1),
                col = .dpOrDefault(GdObject, "col.grid", "#e6e6e6"), lty = .dpOrDefault(GdObject, "lty.grid", 1),
                lwd = .dpOrDefault(GdObject, "lwd.grid", 1), alpha = alpha
            )
        }
        return(invisible(GdObject))
    }
    vals <- values(GdObject)
    ylim <- suppressWarnings(.dpOrDefault(GdObject, "ylim", range(vals, na.rm = TRUE, finite = TRUE)))
    if (diff(ylim) == 0) {
        ylim <- ylim + c(-1, 1)
    }
    if (all(is.infinite(ylim))) {
        ylim <- c(0, 1)
    }
    ylimExt <- extendrange(r = ylim, f = 0.05)
    xscale <- if (!.dpOrDefault(GdObject, "reverseStrand", FALSE)) c(minBase, maxBase) else c(maxBase, minBase)
    pushViewport(viewport(xscale = xscale, yscale = ylimExt, clip = TRUE))
    ## The plotting parameters, some defaults from the lattice package first
    plot.symbol <- trellis.par.get("plot.symbol")
    superpose.symbol <- trellis.par.get("superpose.symbol")
    superpose.line <- trellis.par.get("superpose.line")
    groups <- rep(groups, ncol(vals))
    ## For loess calculation we need some settings
    span <- .dpOrDefault(GdObject, "span", 1 / 5)
    degree <- .dpOrDefault(GdObject, "degree", 1)
    family <- .dpOrDefault(GdObject, "family", c("symmetric", "gaussian"))
    evaluation <- .dpOrDefault(GdObject, "evaluation", 50)
    font <- .dpOrDefault(GdObject, "font", if (is.null(groups)) plot.symbol$font else superpose.symbol$font)
    fontface <- .dpOrDefault(GdObject, "fontface", if (is.null(groups)) plot.symbol$fontface else superpose.symbol$fontface)
    fontsize <- .dpOrDefault(GdObject, "fontsize", if (is.null(groups)) plot.symbol$fontsize else superpose.symbol$fontsize)
    ## An optional baseline to be added
    baseline <- .dpOrDefault(GdObject, "baseline")
    lwd.baseline <- .dpOrDefault(GdObject, "lwd.baseline", pcols$lwd[1])
    lty.baseline <- .dpOrDefault(GdObject, "lty.baseline", pcols$lty[1])
    ## The actual plotting values
    pos <- position(GdObject)
    x <- rep(pos + 0.5, each = nrow(vals)) # to align it with ticks position
    y <- as.numeric(vals)
    ## A grid should always be plotted first, so we need to catch this here
    wg <- match("g", type, nomatch = NA_character_)
    if (!is.na(wg)) {
        panel.grid(
            h = .dpOrDefault(GdObject, "h", -1), v = .dpOrDefault(GdObject, "v", -1),
            col = pcols$col.grid, lty = pcols$lty.grid, lwd = pcols$lwd.grid
        )
        type <- type[-wg]
    }
    ## The special type 'mountain' has to be handled separately
    if ("mountain" %in% type) {
        mbaseline <- if (is.null(baseline)) 0 else baseline[1]
        fill.mountain <- .dpOrDefault(GdObject, "fill.mountain", superpose.symbol$fill)[c(1, 2)]
        col.mountain <- .dpOrDefault(GdObject, "col.mountain", pcols$col)[1]
        col.baseline <- .dpOrDefault(GdObject, "col.baseline", col.mountain)[1]
        lwd.mountain <- .dpOrDefault(GdObject, "lwd.mountain", pcols$lwd)[1]
        lty.mountain <- .dpOrDefault(GdObject, "lty.mountain", pcols$lty)[1]
        .panel.mountain(x, y,
            col = col.mountain, fill = fill.mountain, span = span, degree = degree, family = family,
            evaluation = evaluation, lwd = lwd.mountain, lty = lty.mountain, col.line = col.mountain, alpha = alpha,
            baseline = mbaseline
        )
        if (!is.na(mbaseline)) {
            panel.abline(h = mbaseline, col = col.baseline, lwd = lwd.baseline, lty = lty.baseline, alpha = alpha)
        }
    }
    ## The special type 'polygon' has to be handled separately
    if ("polygon" %in% type) {
        mbaseline <- if (is.null(baseline)) 0 else baseline[1]
        fill.mountain <- .dpOrDefault(GdObject, "fill.mountain", superpose.symbol$fill)[c(1, 2)]
        col.mountain <- .dpOrDefault(GdObject, "col.mountain", pcols$col)[1]
        col.baseline <- .dpOrDefault(GdObject, "col.baseline", col.mountain)[1]
        lwd.mountain <- .dpOrDefault(GdObject, "lwd.mountain", pcols$lwd)[1]
        lty.mountain <- .dpOrDefault(GdObject, "lty.mountain", pcols$lty)[1]
        .panel.polygon(x, y,
            col = col.mountain, fill = fill.mountain, lwd = lwd.mountain,
            lty = lty.mountain, col.line = col.mountain, alpha = alpha,
            baseline = mbaseline
        )
        if (!is.na(mbaseline)) {
            panel.abline(h = mbaseline, col = col.baseline, lwd = lwd.baseline, lty = lty.baseline, alpha = alpha)
        }
    }
    ## Also the type 'boxplot' is handled up front
    if ("boxplot" %in% type) {
        diff <- .pxResolution(coord = "x")
        box.ratio <- .dpOrDefault(GdObject, "box.ratio", 1)
        sx <- sort(unique(x))
        sxd <- if (length(sx) == 1) 1 else diff(sx)
        box.width <- .dpOrDefault(GdObject, "box.width", (((min(sxd) * 0.5) / box.ratio) / diff)) * diff
        if (!is.null(groups)) {
            tw <- min(width(GdObject))
            spacer <- diff
            nb <- nlevels(groups)
            bw <- .dpOrDefault(GdObject, "box.width", ((tw - (nb + 2) * spacer) / nb) / diff) * diff
            bcex <- min(pcols$cex[1], (bw / diff) / 20)
            by <- lapply(split(vals, groups), matrix, ncol = ncol(vals))
            for (j in seq_along(by)) {
                nn <- nrow(by[[j]])
                off <- (width(GdObject) - (bw * nb) - ((nb + 2) * spacer)) / 2
                xx <- rep(start(GdObject) + (j * spacer) + (j * bw) + off, each = nn) - (bw / 2)
                .panel.bwplot(xx, as.numeric(by[[j]]),
                    box.ratio = box.ratio, box.width = (bw / 2) / box.ratio, pch = pcols$pch[1],
                    lwd = pcols$lwd[1], lty = pcols$lty[1], fontsize = fontsize,
                    col = pcols$col.histogram, cex = bcex, font = font, fontfamily = font, fontface = fontface,
                    fill = pcols$col[j], varwidth = .dpOrDefault(GdObject, "varwidth", FALSE),
                    notch = .dpOrDefault(GdObject, "notch", FALSE), notch.frac = .dpOrDefault(GdObject, "notch.frac", 0.5),
                    levels.fos = .dpOrDefault(GdObject, "level.fos", sort(unique(xx))),
                    stats = .dpOrDefault(GdObject, "stats", boxplot.stats), coef = .dpOrDefault(GdObject, "coef", 1.5),
                    do.out = .dpOrDefault(GdObject, "do.out", TRUE), alpha = alpha
                )
            }
            diffY <- .pxResolution(coord = "y", 2)
            outline <- apply(vals, 2, range)
            grid.rect(start(GdObject), outline[1, ] - diffY,
                width = width(GdObject), height = abs(outline[2, ] - outline[1, ]) + (2 * diffY),
                gp = gpar(col = .dpOrDefault(GdObject, "col.boxplotFrame", .DEFAULT_SHADED_COL), fill = "transparent", alpha = alpha, lty = "dotted"),
                default.units = "native", just = c("left", "bottom")
            )
        } else {
            bcex <- min(pcols$cex[1], ((box.width * 2) / diff) / 20)
            .panel.bwplot(x, y,
                box.ratio = box.ratio, box.width = box.width, pch = pcols$pch[1],
                lwd = pcols$lwd[1], lty = pcols$lty[1], fontsize = fontsize,
                col = pcols$col.histogram, cex = bcex, font = font, fontfamily = font, fontface = fontface,
                fill = pcols$fill[1], varwidth = .dpOrDefault(GdObject, "varwidth", FALSE),
                notch = .dpOrDefault(GdObject, "notch", FALSE), notch.frac = .dpOrDefault(GdObject, "notch.frac", 0.5),
                levels.fos = .dpOrDefault(GdObject, "level.fos", sort(unique(x))),
                stats = .dpOrDefault(GdObject, "stats", boxplot.stats), coef = .dpOrDefault(GdObject, "coef", 1.5),
                do.out = .dpOrDefault(GdObject, "do.out", TRUE), alpha = alpha
            )
        }
    }
    ## 'histogram' fills up the full range area if its width is > 1
    if ("histogram" %in% type) {
        ylimSort <- sort(ylimExt)
        yy <- if (ylimSort[1] <= 0 && ylimSort[2] >= 0) 0 else ylimSort[1]
        if (!is.null(groups) && nlevels(groups) > 1) {
            valsS <- .dpOrDefault(GdObject, ".__valsS")
            if (stacked) {
                curMinPos <- curMaxPos <- rep(yy, nrow(valsS))
                for (s in seq_len(ncol(valsS))) {
                    if (!all(is.na(valsS[, s]))) {
                        sel <- !is.na(valsS[, s]) & valsS[, s] >= 0
                        yyy <- curMinPos
                        yyy[sel] <- curMaxPos[sel]
                        offset <- yyy
                        offset[offset != yy] <- 0
                        grid.rect(start(GdObject), yyy,
                            width = width(GdObject), height = valsS[, s] - offset,
                            gp = gpar(col = "transparent", fill = pcols$col[colnames(valsS)[s]], lwd = pcols$lwd[1], lty = pcols$lty[1], alpha = alpha), default.units = "native",
                            just = c("left", "bottom")
                        )
                        curMaxPos[sel] <- curMaxPos[sel] + (valsS[sel, s] - offset[sel])
                        curMinPos[!sel] <- curMinPos[!sel] + (valsS[!sel, s] - offset[!sel])
                    }
                }
                diff <- .pxResolution(coord = "x", pcols$lwd[1] + 1)
                tooNarrow <- width(GdObject) < diff
                if (!all(tooNarrow)) {
                    grid.rect(start(GdObject)[!tooNarrow], curMinPos[!tooNarrow],
                        width = width(GdObject)[!tooNarrow],
                        height = (curMaxPos - curMinPos)[!tooNarrow],
                        gp = gpar(fill = "transparent", col = pcols$col.histogram, lwd = pcols$lwd[1], lty = pcols$lty[1], alpha = alpha),
                        default.units = "native", just = c("left", "bottom")
                    )
                }
            } else {
                spacer <- .pxResolution(min.width = 1, coord = "x")
                yOff <- .pxResolution(min.width = 1, coord = "y")
                outline <- apply(valsS, 1, function(x) range(c(yy, x), na.rm = TRUE))
                grid.rect(start(GdObject), outline[1, ] - yOff,
                    width = width(GdObject), height = apply(outline, 2, diff) + (yOff * 2),
                    gp = gpar(col = pcols$col.histogram, fill = pcols$fill.histogram, lwd = pcols$lwd[1], lty = pcols$lty[1], alpha = alpha), default.units = "native",
                    just = c("left", "bottom")
                )
                len <- ncol(valsS)
                subW <- (width(GdObject) - (spacer * (len + 1))) / len
                sel <- subW > spacer
                ## FIXME: how do we treat this if there is not enough space to plot?
                sel <- !logical(length(subW))
                if (any(sel)) {
                    subW <- subW[sel]
                    valsS <- valsS[sel, ]
                    subX <- rep(start(GdObject)[sel], len) + (subW * rep(seq_len(len) - 1, each = sum(sel))) +
                        (spacer * rep(seq_len(len), each = sum(sel)))
                    grid.rect(subX, yy,
                        width = rep(subW, len), height = valsS - yy,
                        gp = gpar(
                            col = "transparent", fill = rep(pcols$col[seq_len(len)], each = sum(sel)),
                            lwd = pcols$lwd[1], lty = pcols$lty[1], alpha = alpha
                        ), default.units = "native",
                        just = c("left", "bottom")
                    )
                }
            }
        } else {
            valsS <- t(vals)
            grid.rect(start(GdObject), yy,
                width = width(GdObject), height = valsS - yy,
                gp = gpar(col = pcols$col.histogram, fill = pcols$fill.histogram, lwd = pcols$lwd[1], lty = pcols$lty[1], alpha = alpha), default.units = "native",
                just = c("left", "bottom")
            )
        }
    }
    ## gradient summarizes the data as a color gradient
    if ("gradient" %in% type) {
        ncolor <- .dpOrDefault(GdObject, "ncolor", 100)
        gradient <- colorRampPalette(.dpOrDefault(GdObject, "gradient", brewer.pal(9, "Blues")))(ncolor)
        valsScaled <- .z2icol(colMeans(vals, na.rm = TRUE), ncolor, sort(ylim))
        grid.rect(start(GdObject), sort(ylim)[1],
            width = width(GdObject), height = abs(diff(ylim)),
            gp = gpar(col = gradient[valsScaled], fill = gradient[valsScaled], alpha = alpha),
            default.units = "native", just = c("left", "bottom")
        )
    }
    ## heatmap does the same, but for each sample individually
    if ("heatmap" %in% type) {
        ncolor <- .dpOrDefault(GdObject, "ncolor", 100)
        valsScaled <- .z2icol(vals, ncolor, sort(ylim))
        nr <- nrow(vals)
        yy <- seq(min(ylim), max(ylim), len = nr + 1)[-1]
        ydiff <- .pxResolution(coord = "y")
        separator <- .dpOrDefault(GdObject, "separator", 0) * ydiff
        if (!is.null(groups)) {
            valsS <- split(vals, groups)
            freq <- table(factor(.dpOrDefault(GdObject, "groups")))
            cmf <- c(0, cumsum(freq))
            for (s in seq_along(valsS))
            {
                gradient <- colorRampPalette(c("white", pcols$col[s]))(ncolor + 5)[-seq_len(5)]
                valsScaled <- .z2icol(valsS[[s]], ncolor, sort(ylim))
                grid.rect(rep(start(GdObject), each = freq[s]), yy[(cmf[s] + 1):cmf[s + 1]],
                    width = rep(width(GdObject), each = freq[s]),
                    height = max(ydiff, abs(diff(ylim)) * (1 / nr) - separator),
                    gp = gpar(col = gradient[valsScaled], fill = gradient[valsScaled], alpha = alpha),
                    default.units = "native", just = c("left", "top")
                )
            }
        } else {
            gradient <- colorRampPalette(.dpOrDefault(GdObject, "gradient", brewer.pal(9, "Blues")))(ncolor)
            grid.rect(rep(start(GdObject), each = nr), rev(yy),
                width = rep(width(GdObject), each = nr),
                height = max(ydiff, abs(diff(ylim)) * (1 / nr) - separator),
                gp = gpar(col = gradient[valsScaled], fill = gradient[valsScaled], alpha = alpha),
                default.units = "native", just = c("left", "top")
            )
        }
    }
    ## For the horizon plot we can use the latticeExtra panel function, but need to reset the y-range
    if ("horizon" %in% type) {
        nband <- 3
        origin <- .dpOrDefault(GdObject, "horizon.origin", 0)
        gr <- if (is.null(groups)) rep(1, nrow(vals)) else factor(.dpOrDefault(GdObject, "groups"))
        yy <- lapply(split(as.data.frame(vals), gr), colMeans, na.rm = TRUE)
        hfill <- .dpOrDefault(GdObject, "fill.horizon", .DEFAULT_HORIZON_COL)
        hcol <- .dpOrDefault(GdObject, "col.horizon", NA)
        separator <- ceiling(.dpOrDefault(GdObject, "separator", 0) / 2)
        pushViewport(viewport(height = 0.95, clip = TRUE))
        for (i in seq_along(yy)) {
            yi <- yy[[i]]
            horizonscale <- .dpOrDefault(GdObject, "horizon.scale", max(abs(yi - origin), na.rm = TRUE) / nband)
            yr <- origin + c(0, horizonscale)
            pushViewport(viewport(y = (i - 1) / length(yy), height = 1 / length(yy), just = c(0.5, 0), clip = TRUE))
            pushViewport(viewport(height = unit(1, "npc") - unit(separator, "points"), clip = TRUE))
            xscale <- if (!.dpOrDefault(GdObject, "reverseStrand", FALSE)) c(minBase, maxBase) else c(maxBase, minBase)
            pushViewport(viewport(xscale = xscale, yscale = yr, clip = TRUE))
            panel.horizonplot(pos, yi, border = hcol, col.regions = hfill)
            popViewport(3)
        }
        popViewport(1)
    }

    ## plot key-value pairs defined here.
    plot_args <- list(
        type = type, groups = groups, pch = pcols$pch,
        col = pcols$col, col.line = pcols$col.line, col.symbol = pcols$col.symbol, fill = pcols$fill,
        font = font, fontfamily = font, fontface = fontface, lty = pcols$lty, cex = pcols$cex, lwd = pcols$lwd, horizontal = FALSE,
        span = span, degree = degree, family = family, evaluation = evaluation,
        jitter.x = .dpOrDefault(GdObject, "jitter.x", FALSE), jitter.y = .dpOrDefault(GdObject, "jitter.y", FALSE),
        factor = .dpOrDefault(GdObject, "factor", 0.5), amount = .dpOrDefault(GdObject, "amount"),
        alpha = alpha
    )

    ## The rest uses the lattice panel function
    na.rm <- .dpOrDefault(GdObject, "na.rm", FALSE)
    sel <- is.na(y)
    if (na.rm && any(sel)) {
        x <- x[!sel]
        y <- y[!sel]
        groups <- groups[!sel]
    }
    plot_args[["x"]] <- x
    plot_args[["y"]] <- y
    plot_args[["groups"]] <- groups
    plot_args[["subscripts"]] <- seq_along(x)

    ## confidence interval bands
    if ("confint" %in% type) {
        ## column-wise SD calculation
        vectorizedSD <- function(mat, na.rm) {
            ssq <- colSums(mat^2, na.rm = na.rm)
            sumel <- colSums(mat, na.rm = na.rm)
            N <- nrow(mat)
            var <- (1 / (N - 1)) * (ssq - (sumel^2) / N)
            return(sqrt(var))
        }
        debugMode <- FALSE

        my.panel.bands <- function(df, col, fill, font, fontface, ...) {
            upper <- df$high
            lower <- df$low
            x <- df$x
            y <- df$y
            na_idx <- which(is.na(upper))
            ## case 1. there are no error bars to plot at all
            if (length(na_idx) == length(upper)) {
                if (debugMode) message("\t Case 1: all empty. returning")
                return(TRUE)
                ## case 2. no missing points
            } else if (length(na_idx) < 1) {
                if (debugMode) message("\t Case 2: one continuous polygon")
                panel.polygon(c(x, rev(x)), c(upper, rev(lower)),
                    border = col, col = fill, alpha = alpha, ...
                )
                ## case 3. have complete data with some or no missing points
            } else {
                curr_start <- min(which(!is.na(upper)))
                if (debugMode) message(sprintf("\t Case 3: %i of %i NA", length(na_idx), length(upper)))
                curr_na_pos <- 1
                while (curr_na_pos <= length(na_idx)) {
                    if (debugMode) message(sprintf("\t\tcurr_na_pos = %i, na_idx length= %i", curr_na_pos, length(na_idx)))
                    ## complete the current poly
                    idx <- curr_start:(na_idx[curr_na_pos] - 1)
                    panel.polygon(c(x[idx], rev(x[idx])), c(upper[idx], rev(lower[idx])),
                        col = fill, border = col, alpha = alpha, ...
                    )
                    ## contiguous empty spots - skip
                    while ((na_idx[curr_na_pos + 1] == na_idx[curr_na_pos] + 1) && (curr_na_pos < length(na_idx))) {
                        if (debugMode) message(sprintf("\t\ttight-loop:curr_na_pos = %i", curr_na_pos))
                        curr_na_pos <- curr_na_pos + 1
                    }
                    ## at this point, either we've finished NA spots or the next one is far away.
                    ## In any case start a poly and move to the next NA spot
                    curr_start <- na_idx[curr_na_pos] + 1
                    curr_na_pos <- curr_na_pos + 1
                }
                ## there is one last polygon at the end of the view range
                if (na_idx[length(na_idx)] < length(upper)) {
                    if (debugMode) message("\tWrapping last polygon")
                    idx <- curr_start:length(upper)
                    panel.polygon(c(x[idx], rev(x[idx])), c(upper[idx], rev(lower[idx])),
                        col = fill, border = col, alpha = alpha, ...
                    )
                }
            }
        }

        fill <- .dpOrDefault(GdObject, "fill.confint", pcols$col)
        col <- .dpOrDefault(GdObject, "col.confint", pcols$col)
        alpha <- .dpOrDefault(GdObject, "alpha.confint")
        outg <- NULL

        if (!is.null(groups)) {
            groups <- .dpOrDefault(GdObject, "groups")
            by <- lapply(split(vals, groups), matrix, ncol = ncol(vals))
            mu <- list()
            confint <- list()
            minnie <- Inf
            maxie <- -Inf

            df <- NULL
            outPlot <- NULL
            mu <- list()
            confint <- list()
            xvals <- position(GdObject)

            ## buffer variation to set final limits
            for (j in seq_along(by)) {
                mu[[j]] <- colMeans(by[[j]], na.rm = TRUE)
                locusSD <- vectorizedSD(by[[j]], na.rm)
                confint[[j]] <- 1.96 * (locusSD / sqrt(nrow(by[[j]])))

                curr_low <- mu[[j]] - confint[[j]]
                curr_high <- mu[[j]] + confint[[j]]
                minnie <- min(c(minnie, curr_low))
                maxie <- max(c(maxie, curr_high))
            }

            names(fill) <- NULL
            for (j in seq_along(by)) {
                g <- names(by)[j]
                if (debugMode) message(g)
                df <- data.frame(
                    x = position(GdObject), y = mu[[j]],
                    low = mu[[j]] - confint[[j]], high = mu[[j]] + confint[[j]],
                    groups = factor(g)
                )
                my.panel.bands(df, col[j], fill[j], alpha, ...)
            }
        } else {
            mu <- colMeans(vals, na.rm = TRUE)
            locusSD <- vectorizedSD(vals, na.rm)
            confint <- 1.96 * (locusSD / sqrt(nrow(vals)))

            df <- data.frame(
                x = position(GdObject), y = mu,
                low = mu - confint, high = mu + confint,
                groups = factor(1)
            )

            my.panel.bands(df, col[1], fill[1], alpha, ...)
        }
    }
    do.call("panel.xyplot", plot_args)
    if (!any(c("mountain", "polygon") %in% type) && !is.null(baseline) && !is.na(baseline)) {
        panel.abline(h = baseline, col = pcols$col.baseline, lwd = lwd.baseline, lty = lty.baseline, alpha = alpha)
    }
    popViewport(1)

    return(invisible(GdObject))
})

## SetAs ---------------------------------------------------------------------

setAs("DataTrack", "data.frame", function(from, to) {
    tmp <- as.data.frame(ranges(from))
    colnames(tmp)[1] <- "chromosome"
    tmp <- cbind(tmp, as.data.frame(t(values(from, all = TRUE))))
    return(tmp)
})

setAs("GRanges", "DataTrack", function(from, to) DataTrack(range = from))

## Show ----------------------------------------------------------------------

## A helper function to plot information regarding additional features on other chromosomes
.addFeatInfo <- function(object, addfeat) {
    freqs <- table(seqnames(object))
    freqs <- freqs[setdiff(names(freqs), chromosome(object))]
    nrChr <- length(freqs)
    msg <- sprintf(
        "There %s %s additional annotation feature%s on %s further chromosome%s%s",
        ifelse(addfeat > 1, "are", "is"),
        addfeat,
        ifelse(addfeat > 1, "s", ""),
        nrChr,
        ifelse(nrChr > 1, "s", ""),
        ifelse(nrChr == 1, sprintf(" (%s)", names(freqs)), "")
    )
    if (nrChr > 1) {
        msg <- if (nrChr > 10) {
            c(
                msg, paste("  ", head(names(freqs), 5), ": ", head(freqs, 5), sep = "", collapse = "\n"),
                "  ...", paste("  ", tail(names(freqs), 5), ": ", tail(freqs, 5), sep = "", collapse = "\n")
            )
        } else {
            c(msg, paste("  ", names(freqs), ": ", freqs, " features", sep = "", collapse = "\n"))
        }
        msg <- c(msg, paste(
            "Call seqlevels(obj) to list all available chromosomes",
            "or seqinfo(obj) for more detailed output"
        ))
    }
    return(msg)
}

#' @describeIn DataTrack-class  Show method.
#' @export
setMethod(
    "show", signature(object = "DataTrack"),
    function(object) {
        msg <- sprintf(
            paste("DataTrack '%s'\n| genome: %s\n| active chromosome: %s\n",
                "| positions: %s\n| samples:%s\n| strand: %s",
                sep = ""
            ),
            names(object),
            genome(object),
            chromosome(object),
            length(object),
            nrow(values(object)),
            strand(object)[1]
        )
        addfeat <- ncol(object@data) - length(object)
        if (addfeat > 0) {
            msg <- c(msg, .addFeatInfo(object, addfeat), "Call chromosome(obj) <- 'chrId' to change the active chromosome")
        }
        cat(paste(msg, collapse = "\n"), "\n")
    }
)

#' @describeIn DataTrack-class  Show method.
#' @export
setMethod("show", signature(object = "ReferenceDataTrack"), function(object) {
    .referenceTrackInfo(object, "ReferenceDataTrack")
})

#' @include GdObject-class.R
NULL

## RangeTrack Class ----------------------------------------------------------

#' RangeTrack class and methods
#'
#'
#' The virtual parent class for all track items in the Gviz package that
#' contain some form of genomic ranges (start, end, strand, chromosome
#' and the associated genome.)
#'
#'
#' @name RangeTrack-class
#'
#' @template GdObject-class_slot
#' @template RangeTrack-class_slot
#'
#' @template RangeTrack-class_param
#'
#' @return A virtual class: No objects may be created from it.
#'
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## This is a reference class therefore we show below
#' ## an example from AnnotationTrack
#'
#' ## An empty object
#' AnnotationTrack()
#'
#' ## Construct from individual arguments
#' st <- c(2000000, 2070000, 2100000, 2160000)
#' ed <- c(2050000, 2130000, 2150000, 2170000)
#' str <- c("-", "+", "-", "-")
#' gr <- c("Group1", "Group2", "Group1", "Group3")
#'
#' annTrack <- AnnotationTrack(
#'     start = st, end = ed, strand = str, chromosome = 7,
#'     genome = "hg19", feature = "test", group = gr,
#'     id = paste("annTrack item", 1:4),
#'     name = "generic annotation", stacking = "squish"
#' )
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(annTrack)
#' @exportClass RangeTrack
setClass("RangeTrack",
    representation = representation("VIRTUAL",
        range = "GRangesOrIRanges",
        chromosome = "character",
        genome = "character"
    ),
    contains = "GdObject",
    prototype = prototype(
        chromosome = "chr1",
        dp = DisplayPars(),
        genome = "ANY",
        name = "RangeTrack",
        range = GRanges()
    )
)

## Coercing all input to the appropriate form
#' @describeIn RangeTrack-class Initialize.
#' @export
setMethod("initialize", "RangeTrack", function(.Object, range, chromosome, genome, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "RangeTrack")
    if (!missing(chromosome) && !is.null(chromosome)) {
        .Object@chromosome <- .chrName(chromosome)[1]
    }
    if (!missing(genome) && !is.null(genome)) {
        .Object@genome <- genome
    }
    if (!missing(range) && is(range, "GRanges")) {
        .Object@range <- range
    }
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})


## .buildRange ---------------------------------------------------------------
##
## Helper methods to build a GRanges object from the input arguments.
##
## Coordinates for grouped elements may be passed in as comma-separated values
## (e.g. "1,5,9"), in which case we need to split and convert to numeric.
## This also implies that the additional arguments (like feature, group, etc.)
## have to be replicated accordingly. We handle this by passing along the repeat
## vector 'by' to the numeric method below.

#' @importFrom Biobase rowMax
setMethod(
    ".buildRange", signature(
        "NULLOrMissing", "FactorOrCharacterOrNULL", "FactorOrCharacterOrNULL",
        "FactorOrCharacterOrNULL"
    ),
    function(range, start, end, width, asIRanges = FALSE, ...) {
        ## The inputs coordinates are all empty
        if (!length(start) && !length(end)) {
            return(if (asIRanges) IRanges() else GRanges())
        }
        delim <- ","
        coords <- c("start", "end", "width")
        lengths <- vapply(coords, function(x) length(get(x)), FUN.VALUE = numeric(1L))
        items <- structure(as.list(rep(0, 3)), names = coords)
        by <- NULL
        for (i in coords) {
            val <- get(i)
            if (!is.null(val)) {
                val <- strsplit(as.character(val), delim)
                lengths[i] <- length(val)
                items[[i]] <- listLen(val)
                val <- unlist(val)
                assign(i, as.numeric(val))
            }
        }
        len <- max(lengths)
        if (!all(unlist(lapply(items, function(x) length(x) == 1 && x == 0)))) {
            by <- Biobase::rowMax(do.call(cbind, lapply(items, function(x) {
                if (length(x) == 1) rep(x, len) else if (length(x)) x else rep(0, len)
            })))
        }
        return(.buildRange(start = start, end = end, width = width, asIRanges = asIRanges, by = by, len = len, ...))
    }
)

## Helper function to handle settings and defaults
.fillWithDefaults <- function(range = as.data.frame(matrix(ncol = 0, nrow = len)),
                              defaults, args, len, by = NULL, ignore = NULL) {
    for (a in setdiff(names(defaults), ignore)) {
        range[[a]] <- if (is.null(args[[a]])) {
            if (is.null(defaults[[a]])) {
                stop("The mandatory argument '", a, "' is missing with no default")
            }
            val <- defaults[[a]]
            if (length(val) == 1) {
                val <- rep(val, len)
            }
            if (!length(val) %in% c(len, sum(by))) {
                stop("Number of elements in '", a, "' is invalid")
            }
            if (!is.null(by) && length(val) != sum(by)) rep(val, by) else val
        } else {
            val <- args[[a]]
            if (length(val) == 1) {
                val <- rep(val, len)
            }
            if (!length(val) %in% c(len, sum(by))) {
                stop("Number of elements in argument '", a, "' is invalid")
            }
            if (!is.null(by) && length(val) != sum(by)) rep(val, by) else val
        }
    }
    return(range)
}

## For numeric vectors we can immediately create a data frame after some sanity checking and pass that on to the next method.
setMethod(
    ".buildRange", signature("NULLOrMissing", "NumericOrNULL", "NumericOrNULL", "NumericOrNULL"),
    function(range, start, end, width, asIRanges = FALSE, by = NULL, len, args, defaults, ...) {
        ## Some of the arguments are mutually exclusive and we want to catch this here.
        if (is.null(width)) {
            ## The inputs coordinates are all empty
            ## CHECK if this might solve the issue on windows
            if (is.null(start) && is.null(end)) {
                return(if (asIRanges) IRanges() else GRanges())
            }
            if (is.null(start) || is.null(end)) {
                stop("Must specify either start and end or width")
            }
        } else {
            if (is.null(end) && !is.null(start)) {
                end <- as.integer(start) + as.integer(width)
            } else if (is.null(start) && !is.null(end)) {
                start <- as.integer(end) - as.integer(width)
            } else {
                stop("Can't pass all three of 'start', 'end' and 'width'")
            }
        }
        if (length(start) != 1 && length(end) != 1 && length(start) != length(end)) {
            stop("Start and end must be vectors of the same length")
        }
        if (missing(len)) {
            len <- length(start)
        }
        range <- data.frame()
        if (length(start) > 0) {
            if (asIRanges) {
                return(IRanges(start = as.integer(start), end = as.integer(end)))
            }
            range <- .fillWithDefaults(data.frame(start = as.integer(start), end = as.integer(end)), defaults, args, len, by)
        }
        return(.buildRange(range = range, asIRanges = asIRanges, args = args["genome"], defaults = defaults, ...))
    }
)

## For data.frames we need to check for additional arguments
## (like feature, group, etc.), the chromosome information
## and create the final GRanges object
setMethod(
    ".buildRange", signature("data.frame"),
    function(range, asIRanges = FALSE, args = list(), defaults = list(), chromosome = NULL, trackType, ...) {
        if (asIRanges) {
            range <- .fillWithDefaults(range, defaults, args, len = nrow(range), ignore = setdiff(names(defaults), c("start", "end", "genome")))
            return(IRanges(start = as.integer(range$start), end = as.integer(range$end)))
        }
        mandArgs <- c("start", "end", "genome", names(defaults))
        ## Not quite sure how whether existing chromosome information in a GRanges object should generally have precedence over the
        ## chromosome constructor, but probably that should be the case
        if ("chromosome" %in% colnames(range)) {
            args$chromosome <- NULL
        }
        missing <- setdiff(union(setdiff(mandArgs, c(colnames(range))), names(which(!vapply(args, is.null, FUN.VALUE = logical(1L))))), "genome")
        range <- .fillWithDefaults(range, defaults[missing], args[missing], len = nrow(range))
        range$chromosome <- .chrName(as.character(range$chromosome))
        grange <- GRanges(ranges = IRanges(start = range$start, end = range$end), strand = range$strand, seqnames = range$chromosome)
        mcols(grange) <- range[, setdiff(colnames(range), c(
            "start", "end", "strand", "width", "chromosome", "genome", "seqnames",
            "ranges", "seqlevels", "seqlengths", "isCircular", "element"
        ))]
        if (trackType != "DataTrack") {
            mcols(grange) <- mcols(grange)[, intersect(names(defaults), colnames(mcols(grange)))]
        }
        suppressWarnings(genome(grange) <- unname(if (is.null(args[["genome"]])) defaults[["genome"]] else as.character(args[["genome"]])[[1]]))
        return(grange)
    }
)


## For GRanges we just need to check for the existence of additional
## arguments (like feature, group, etc.)
setMethod(
    ".buildRange", signature("GRanges"),
    function(range, asIRanges = FALSE, args = list(), defaults = list(), trackType = NULL, ...) {
        if (asIRanges) {
            return(ranges(range))
        }
        if (length(range)) {
            mandArgs <- names(defaults)
            ## Not quite sure how whether existing chromosome information in a GRanges object should generally have precedence over the
            ## chromosome constructor, but probably that should be the case
            args$chromosome <- NULL
            range <- renameSeqlevels(range, setNames(.chrName(seqlevels(range)), seqlevels(range)))
            missing <- setdiff(union(setdiff(mandArgs, c("chromosome", "strand", colnames(mcols(range)))), names(which(!vapply(args, is.null, FUN.VALUE = logical(1L))))), "genome")
            newVars <- .fillWithDefaults(DataFrame(chromosome = as.character(seqnames(range)), strand = as.character(strand(range)), mcols(range), check.names = FALSE),
                defaults[missing], args[missing],
                len = length(range), ignore = c("flag")
            )
            if (any(c("start", "end", "strand", "chromosome") %in% colnames(newVars))) {
                gen <- genome(range)
                range <- GRanges(
                    seqnames = if (is.null(newVars[["chromosome"]])) seqnames(range) else (newVars[["chromosome"]]),
                    strand = if (is.null(newVars[["strand"]])) strand(range) else (newVars[["strand"]]),
                    ranges = IRanges(
                        start = if (is.null(newVars[["start"]])) start(range) else (newVars[["start"]]),
                        end = if (is.null(newVars[["end"]])) end(range) else (newVars[["end"]])
                    )
                )
                if (length(unique(gen)) != 1) {
                    warning("Tracks can only be defined for a single genome. Forcing all reads to belong to genome '", gen[1], "'")
                }
                defaults[["genome"]] <- as.character(gen)[1]
            }
            mcols(range) <- newVars[, setdiff(colnames(newVars), c(
                "start", "end", "strand", "width", "chromosome", "genome", "seqnames",
                "ranges", "seqlevels", "seqlengths", "isCircular", "element"
            )), drop = FALSE]
        }
        if (trackType != "DataTrack") {
            mcols(range) <- mcols(range)[, intersect(names(defaults), colnames(mcols(range)))]
        }
        ## The genome information may or may not be encoded in the GRanges object at this time but we want it in there for sure
        genome <- if (!is.null(args[["genome"]])) args[["genome"]] else .getGenomeFromGRange(range, defaults[["genome"]])
        suppressWarnings(genome(range) <- unname(genome))[1]
        return(range)
    }
)

## For IRanges we need to deal with additional arguments
## (like feature, group, etc.) and create the final GRanges object
setMethod(
    ".buildRange", signature("IRanges"),
    function(range, asIRanges = FALSE, args = list(), defaults = list(), chromosome = NULL, strand, ...) {
        if (asIRanges) {
            return(range)
        }
        if (missing(chromosome) || is.null(chromosome)) {
            stop("Unable to find chromosome information in any of the arguments")
        }
        range <- GRanges(seqnames = .chrName(chromosome), ranges = range, strand = if (!is.null(args$strand)) args$strand else "*")
        if (length(range)) {
            vals <- .fillWithDefaults(defaults = defaults, args = args, len = (length(range)), by = NULL, ignore = "strand")
            mcols(range) <- vals
        }
        return(range)
    }
)

## For GRangesLists we capture the grouping information from the list
## structure, `unlist` and use the `GRanges` method
setMethod(
    ".buildRange", signature("GRangesList"),
    function(range, groupId = "group", ...) {
        grps <- rep(names(range), elementNROWS(range))
        range <- unlist(range)
        names(range) <- NULL
        mcols(range)[[groupId]] <- grps
        return(.buildRange(range = range, ...))
    }
)



## RangeTrack Methods ranges, range ------------------------------------------

## Extract the full GRanges object from the range slot
## of an object inheriting from RangeTrack

#' @describeIn RangeTrack-class return the genomic coordinates for the track
#' along with all additional annotation information as an object of
#' class `GRanges.`
setMethod("ranges", "RangeTrack", function(x) x@range)
setReplaceMethod("ranges", "RangeTrack", function(x, value) {
    x@range <- value
    return(x)
})

## Extract the IRanges part of the GRanges object from the range slot
## of an object inheriting from RangeTrack

#' @describeIn RangeTrack-class return the genomic coordinates for the
#' track as an object of class IRanges.
#' @export
setMethod("range", "RangeTrack", function(x) ranges(x@range))

## RangeTrack Methods seqnames, seqlevels and seqinfo, genome ----------------

#' @describeIn RangeTrack-class return the track's seqnames.
#' @export
setMethod("seqnames", "RangeTrack", function(x) as.character(seqnames(ranges(x))))

#' @describeIn RangeTrack-class return the track's seqlevels.
#' @export
setMethod("seqlevels", "RangeTrack", function(x) unique(seqnames(x)))

#' @describeIn RangeTrack-class return the track's seqinfo.
#' @export
setMethod("seqinfo", "RangeTrack", function(x) table(seqnames(x)))

#' @describeIn RangeTrack-class return the track's genome.
#' @export
setMethod("genome", "RangeTrack", function(x) x@genome)

#' @describeIn RangeTrack-class set the track's genome. Usually this has to
#' be a valid UCSC identifier, however this is not formally enforced here.
#' @export
setReplaceMethod("genome", "RangeTrack", function(x, value) {
    x@genome <- value[1]
    genome(ranges(x)) <- as.vector(value[1])
    return(x)
})

#' @describeIn RangeTrack-class return the chromosome for which the track is defined.
#' @export
setMethod("chromosome", "RangeTrack", function(GdObject) GdObject@chromosome)

#' @describeIn RangeTrack-class replace the value of the track's chromosome.
#' This has to be a valid UCSC chromosome identifier or an integer or character
#' scalar that can be reasonably coerced into one.
#' @export
setReplaceMethod("chromosome", "RangeTrack", function(GdObject, value) {
    GdObject@chromosome <- .chrName(value[1])
    return(GdObject)
})
## RangeTrack Methods start, end, min and max ranges -------------------------

#' @describeIn RangeTrack-class the start of the track items in genomic
#' coordinates.
#' @export
setMethod("start", "RangeTrack", function(x) if (length(x)) as.integer(start(range(x))) else NULL)

#' @describeIn RangeTrack-class replace the start of the track items in
#' genomic coordinates.
#' @export
setReplaceMethod("start", "RangeTrack", function(x, value) {
    start(x@range) <- value
    return(x)
})

#' @describeIn RangeTrack-class the end of the track items in genomic
#' coordinates.
#' @export
setMethod("end", "RangeTrack", function(x) if (length(x)) as.integer(end(range(x))) else NULL)

#' @describeIn RangeTrack-class replace the end of the track items in
#' genomic coordinates.
#' @export
setReplaceMethod("end", "RangeTrack", function(x, value) {
    end(x@range) <- value
    return(x)
})

#' @describeIn RangeTrack-class the width of the track items in genomic
#' coordinates.
#' @export
setMethod("width", "RangeTrack", function(x) if (length(x)) as.integer(width(range(x))) else NULL)

#' @describeIn RangeTrack-class replace the width of the track items in genomic
#' coordinates.
#' @export
setReplaceMethod("width", "RangeTrack", function(x, value) {
    width(x@range) <- value
    return(x)
})

#' @describeIn RangeTrack-class return the start position for the leftmost range item.
#' @export
setMethod("min", "RangeTrack", function(x) min(start(x)))

#' @describeIn RangeTrack-class return the end position for the rightmost range item.
#' @export
setMethod("max", "RangeTrack", function(x) max(end(x)))

#' @describeIn RangeTrack-class return the number of items in the track.
#' @export
setMethod("length", "RangeTrack", function(x) sum(seqnames(x) == chromosome(x)))

#' @describeIn RangeTrack-class  return a vector of strand specifiers for all
#' track items, in the form '+' for the Watson strand, '-' for the Crick
#' strand or '*' for either of the two.
#' @export
setMethod("strand", "RangeTrack", function(x) as.character(strand(ranges(x))))

#' @describeIn RangeTrack-class replace the strand information for the track
#' items. The replacement value needs to be an appropriate scalar or vector
#' of strand values.
#' @export
setReplaceMethod("strand", "RangeTrack", function(x, value) {
    if ((length(value) != 1 && length(value) != length(x)) || !all(value %in% c("+", "-", "*"))) {
        stop(
            "Invalid replacement value or length of replacement value ",
            "for the strand information does not match the ",
            "number of items in the track"
        )
    }
    r <- ranges(x)
    strand(r) <- value
    x@range <- r
    return(x)
})

#' @describeIn RangeTrack-class the arithmetic mean of the track item's
#' coordionates, i.e., `(end(obj)-start(obj))/2`.
#' @export
setMethod("position", signature("RangeTrack"), definition = function(GdObject, from = NULL, to = NULL, sort = FALSE, ...) {
    if (!is.null(from) && !is.null(to)) {
        GdObject <- subset(GdObject, from = from, to = to, sort = sort, ...)
    }
    pos <- if (length(GdObject)) rowMeans(cbind(start(GdObject), end(GdObject))) else numeric()
    return(pos)
})

## RangeTrack Methods subsetting, split---------------------------------------

#' @describeIn RangeTrack-class subset the items in the `RangeTrack` object.
#' This is essentially similar to subsetting of the `GRanges` object in the
#' `range` slot. For most applications, the subset method may be more appropriate.
#' @export
setMethod("[", signature(x = "RangeTrack"), function(x, i, j, ..., drop = TRUE) {
    x <- .deepCopyPars(x)
    x@range <- x@range[i, , drop = drop]
    return(x)
})

#' @describeIn RangeTrack-class subset a `RangeTrack` by coordinates and
#' sort if necessary.
#' @export
setMethod("subset", signature(x = "RangeTrack"), function(x, from = NULL, to = NULL, sort = FALSE, drop = TRUE, use.defaults = TRUE, ...) {
    ## Not needed anymore...
    ## Subset to a single chromosome first
    if (drop) {
        csel <- seqnames(x) != chromosome(x)
        if (any(csel)) {
            x <- x[, !csel]
        }
    }
    if (!length(x)) {
        return(x)
    }
    ranges <- if (use.defaults) .defaultRange(x, from = from, to = to) else c(from = ifelse(is.null(from), -Inf, from), to = ifelse(is.null(to), Inf, to))
    lsel <- end(x) < ranges["from"]
    if (any(lsel)) {
        lsel[max(0, max(which(lsel)) - 1)] <- FALSE
    }
    rsel <- start(x) > ranges["to"]
    if (any(rsel)) {
        rsel[min(length(x), min(which(rsel)) + 1)] <- FALSE
    }
    if (any(lsel) || any(rsel)) {
        x <- x[!(lsel | rsel), ]
    }
    if (sort) {
        x <- x[order(range(x)), ]
    }
    return(x)
})



#' @describeIn RangeTrack-class split a `RangeTrack` object by an appropriate
#' factor vector (or another vector that can be coerced into one). The output
#' of this operation is a list of objects of the same class as the input
#' object, all inheriting from class `RangeTrack.`
#' @export
setMethod("split", signature("RangeTrack"),
    definition = function(x, f, ...) {
        rs <- split(ranges(x), factor(f))
        lapply(rs, function(y) {
            x@range <- y
            return(x)
        })
    }
)

## RangeTrack Methods value, feature -----------------------------------------

#' @describeIn RangeTrack-class return all additional annotation information
#' except for the genomic coordinates for the track items as a `data.frame`.
#' @export
setMethod("values", "RangeTrack", function(x) as.data.frame(values(ranges(x))))

#' @describeIn RangeTrack-class return the grouping information for track
#' items. For certain sub-classes, groups may be indicated by different colour
#'  schemes when plotting. See grouping or `AnnotationTrack` and
#'  `GeneRegionTrack` for details.
#'  @export
setMethod("feature", signature(GdObject = "RangeTrack"), function(GdObject) .getAnn(GdObject, "feature"))

#' @describeIn RangeTrack-class set the grouping information for track items.
#' This has to be a factor vector (or another type of vector that can be
#' coerced into one) of the same length as the number of items in the
#' `RangeTrack.` See grouping or `AnnotationTrack` and `GeneRegionTrack` for
#'  details.
#'  @export
setReplaceMethod("feature", signature("RangeTrack", "character"), function(GdObject, value) .setAnn(GdObject, value, "feature"))


## RangeTrack Methods consolidate --------------------------------------------

#' @describeIn RangeTrack-class Consolidate.
#' @param GdObject the input track object
#' @param chromosome the currently active chromosome which may have to be set
#' for a `RangeTrack` or a `SequenceTrack` object
#' parameters
# #' @keywords internal
#' @export
setMethod("consolidateTrack", signature(GdObject = "RangeTrack"), function(GdObject, chromosome, ...) {
    if (!is.null(chromosome)) {
        chromosome(GdObject) <- chromosome
    }
    GdObject <- callNextMethod(GdObject, ...)
    return(GdObject)
})

## RangeTrack SetAs ----------------------------------------------------------


#' @noRd
#' @keywords internal
setAs(
    "RangeTrack", "data.frame",
    function(from, to) as(as(ranges(from), "DataFrame"), "data.frame")
)

#' @include DataTrack-class.R
#' @include ReferenceTrack-class.R

NULL

## AnnotationTrack Class -----------------------------------------------------

#' AnnotationTrack class and methods
#'
#' A fairly generic track object for arbitrary genomic range annotations, with
#' the option of grouped track items. The extended `DetailsAnnotationTrack`
#' provides a more flexible interface to add user-defined custom
#' information for each range.
#'
#' @name AnnotationTrack-class
#'
#' @template GdObject-class_slot
#' @template RangeTrack-class_slot
#' @template StackedTrack_slot
#' @template AnnotationTrack_slot
#'
#' @template AnnotationTrack_param
#'
#' @author Florian Hahne, Arne Mueller
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## An empty object
#' AnnotationTrack()
#'
#' ## Construct from individual arguments
#' st <- c(2000000, 2070000, 2100000, 2160000)
#' ed <- c(2050000, 2130000, 2150000, 2170000)
#' str <- c("-", "+", "-", "-")
#' gr <- c("Group1", "Group2", "Group1", "Group3")
#'
#' annTrack <- AnnotationTrack(
#'     start = st, end = ed, strand = str, chromosome = 7,
#'     genome = "hg19", feature = "test", group = gr,
#'     id = paste("annTrack item", 1:4),
#'     name = "generic annotation", stacking = "squish"
#' )
#'
#' ## Or from a data.frame
#' df <- data.frame(
#'     start = st, end = ed, strand = str, id = paste("annTrack item", 1:4),
#'     feature = "test", group = gr
#' )
#' annTrack <- AnnotationTrack(
#'     range = df, genome = "hg19", chromosome = 7,
#'     name = "generic annotation", stacking = "squish"
#' )
#'
#' ## Or from a GRanges object
#' gr <- GRanges(
#'     seqnames = "chr7", range = IRanges(start = df$start, end = df$end),
#'     strand = str
#' )
#' genome(gr) <- "hg19"
#' mcols(gr) <- df[, -(1:3)]
#' annTrack <- AnnotationTrack(
#'     range = gr, name = "generic annotation",
#'     stacking = "squish"
#' )
#'
#' ## Finally from a GRangesList
#' grl <- split(gr, values(gr)$group)
#' AnnotationTrack(grl)
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#' ## Plotting
#' plotTracks(annTrack)
#'
#' ## Track names
#' names(annTrack)
#' names(annTrack) <- "foo"
#' plotTracks(annTrack)
#'
#' ## Subsetting and splitting
#' subTrack <- subset(annTrack, to = 2155000)
#' length(subTrack)
#' subTrack[1:2]
#' split(annTrack, c(1, 2, 1, 2))
#'
#' ## Accessors
#' start(annTrack)
#' end(annTrack)
#' width(annTrack)
#' position(annTrack)
#' width(subTrack) <- width(subTrack) + 1000
#'
#' strand(annTrack)
#' strand(subTrack) <- "-"
#'
#' chromosome(annTrack)
#' chromosome(subTrack) <- "chrX"
#'
#' genome(annTrack)
#' genome(subTrack) <- "mm9"
#'
#' range(annTrack)
#' ranges(annTrack)
#'
#' ## Annotation
#' identifier(annTrack)
#' identifier(annTrack, "lowest")
#' identifier(subTrack) <- "bar"
#'
#' feature(annTrack)
#' feature(subTrack) <- "foo"
#'
#' values(annTrack)
#'
#' ## Grouping
#' group(annTrack)
#' group(subTrack) <- "Group 1"
#' chromosome(subTrack) <- "chr7"
#' plotTracks(subTrack)
#'
#' ## Stacking
#' stacking(annTrack)
#' stacking(annTrack) <- "dense"
#' plotTracks(annTrack)
#'
#' ## coercion
#' as(annTrack, "data.frame")
#' as(annTrack, "UCSCData")
#'
#' ## HTML image map
#' coords(annTrack)
#' tags(annTrack)
#' annTrack <- plotTracks(annTrack)$foo
#' coords(annTrack)
#' tags(annTrack)
#'
#' ## DetailsAnnotationTrack
#' library(lattice) # need to use grid grapics
#'
#' ## generate two random distributions per row (probe/feature)
#' ## the difference between the distributions increases from probe 1 to 4
#' m <- matrix(c(rgamma(400, 1)), ncol = 100)
#' m[, 51:100] <- m[, 51:100] + 0:3
#' ## rownames must be accessible by AnnotationTrack element identifier
#' rownames(m) <- identifier(annTrack, "lowest")
#'
#' ## create a lattice density plot for the values (signals) of the two groups
#' ## as the chart must be placed into a pre-set grid view port we have to use
#' ## print without calling plot.new! Note, use a common prefix for all lattice.
#' ## Avoid wasting space by removing y-axis decorations.
#'
#' ## Note, in this example 'm' will be found in the environment the 'details'
#' ## function is defined in. To avoid overwriting 'm' you should use a closure
#' ## or environment to access 'm'.
#' details <- function(identifier, ...) {
#'     d <- data.frame(signal = m[identifier, ], group = rep(c("grp1", "grp2"), each = 50))
#'     print(densityplot(~signal,
#'         group = group, data = d, main = identifier,
#'         scales = list(draw = FALSE, x = list(draw = TRUE)), ylab = "", xlab = "",
#'     ), newpage = FALSE, prefix = "plot")
#' }
#'
#' deTrack <- AnnotationTrack(
#'     range = gr, genome = "hg19", chromosome = 7,
#'     name = "generic annotation with details per entry", stacking = "squish",
#'     fun = details, details.ratio = 1
#' )
#'
#' plotTracks(deTrack)
#'
#' set.seed(1234)
#' deTrack <- AnnotationTrack(
#'     range = gr, genome = "hg19", chromosome = 7,
#'     name = "generic annotation with details per entry",
#'     stacking = "squish", fun = details,
#'     details.ratio = 1, selectFun = function(...) {
#'         sample(c(FALSE, TRUE), 1)
#'     }
#' )
#'
#' plotTracks(deTrack)
#' @importClassesFrom rtracklayer UCSCData
#'
#' @exportClass AnnotationTrack
setClass("AnnotationTrack",
    contains = "StackedTrack",
    prototype = prototype(
        columns = c("feature", "group", "id"),
        stacking = "squish",
        name = "AnnotationTrack",
        dp = DisplayPars(
            arrowHeadWidth = 30,
            arrowHeadMaxWidth = 40,
            cex.group = 0.6,
            cex = 1,
            col.line = "darkgray",
            col = "transparent",
            featureAnnotation = NULL,
            fill = "lightblue",
            fontfamily.group = "sans",
            fontcolor.group = .DEFAULT_SHADED_COL,
            fontcolor.item = "white",
            fontface.group = 2,
            fontsize.group = 12,
            groupAnnotation = NULL,
            just.group = "left",
            lex = 1,
            lineheight = 1,
            lty = "solid",
            lwd = 1,
            mergeGroups = FALSE,
            min.height = 3,
            min.width = 1,
            rotation = 0,
            rotation.group = 0,
            rotation.item = 0,
            shape = "arrow",
            showFeatureId = FALSE,
            showId = FALSE,
            showOverplotting = FALSE,
            size = 1
        )
    )
)

## Initialize ----------------------------------------------------------------

## Essentially we just check for the correct GRanges columns here
#' @describeIn AnnotationTrack-class Show method.
#' @export
setMethod("initialize", "AnnotationTrack", function(.Object, ...) {
    if (is.null(list(...)$range) && is.null(list(...)$genome) && is.null(list(...)$chromosome)) {
        return(.Object)
    }
    ## the display parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "AnnotationTrack")
    range <- list(...)$range
    if (!is.null(range) && length(.Object)) {
        if (!all(.Object@columns %in% colnames(values(range)))) {
            stop(
                "Problem initializing AnnotationTrack, need the following columns:",
                toString(.Object@columns, collapse = ", ")
            )
        }
        grp <- if (is(.Object, "GeneRegionTrack")) values(range)$transcript else values(range)$group
        if (any(vapply(split(as.character(strand(range)), grp), function(x) length(unique(x)), numeric(1)) != 1)) {
            stop("Grouped elments of a RangeTrack can not be on opposing strands")
        }
    }
    .Object <- callNextMethod()
    return(.Object)
})

## The file-based version of the AnnotationTrack class. This will mainly provide a means to dispatch to
## a special 'subset' method which should stream the necessary data from disk.

#' @describeIn AnnotationTrack-class The file-based version of the `AnnotationTrack-class`.
#' @exportClass ReferenceAnnotationTrack
setClass("ReferenceAnnotationTrack", contains = c("AnnotationTrack", "ReferenceTrack"))

## Initialize ----------------------------------------------------------------

## This just needs to set the appropriate slots that are being inherited from ReferenceTrack because the
## multiple inheritance has some strange features with regards to method selection
#' @describeIn AnnotationTrack-class Initialize.
#' @export
setMethod("initialize", "ReferenceAnnotationTrack", function(.Object, stream, reference, mapping = list(),
                                                             args = list(), defaults = list(), ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object = .Object, reference = reference, stream = stream,
        mapping = mapping, args = args, defaults = defaults)
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## Constructor ---------------------------------------------------------------

## Constructor. The following arguments are supported:
##    o range: a data.frame or a GRanges object containing the information
##       about the track items. If a data.frame, it needs to be coerceable
##     to a GRanges object, i.e., it needs at least the mandatory 'start', 'stop' and
##     'strand' columns. Additional optional columns are:
##        - feature: the type of the item. Can be mapped to colors via the DisplayPars.
##        - group: a grouping factor to connect track items.
##        - id: a unique identifier for a feature. This will be plotted if showId==TRUE
##             Note that internally we use the value of ID as the seqnames slot in the
##             internal GRanges object. Defaults for all missing columns are generated.
##    Instead of using the 'range' parameter, all these values can also be passed as
##    individual vectors, in which case they need to be of similar length.
##    o start, end, width: numeric vectors of the item start and end coordinates
##    o strand: the strand information may be provided in the form '+' for
##       the Watson strand, '-' for the Crick strand or '*' for any of the two.
##    o feature, group, id: individual vectors of equal length as described above.
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o stacking: character controlling the stacking of overlapping items. One in 'hide',
##       'dense', 'squish', 'pack' or 'full'.
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as further DisplayParameters

#' @describeIn AnnotationTrack-class Constructor function for
#' `AnnotationTrack-class`

#' @return
#' The return value of the constructor function is a new object of class
#' \code{AnnotationTrack} or of class \code{DetailsAnnotationTrack}, depending
#' on the constructor arguments. Typically the user will not have to be
#' troubled with this distinction and can rely on the constructor to make the
#' right choice.
#'
#' @section Objects from the class:
#'
#' Objects can be created using the constructor function
#' \code{AnnotationTrack}.
#' @export
AnnotationTrack <- function(range = NULL, start = NULL, end = NULL, width = NULL, feature, group, id, strand, chromosome,
                            genome, stacking = "squish", name = "AnnotationTrack", fun, selectFun, importFunction,
                            stream = FALSE, ...) {
    ## Some defaults
    covars <- .getCovars(range)
    isStream <- FALSE
    if (!is.character(range)) {
        n <- max(c(length(start), length(end), length(width)), nrow(covars))
        if (is.null(covars[["feature"]]) && missing(feature)) {
            feature <- rep("unknown", n)
        }
        if (is.null(covars[["id"]]) && missing(id)) {
            id <- make.unique(rep(if (!is.null(feature)) as.character(feature) else covars[["feature"]], n)[seq_len(n)])
        }
        if (is.null(covars[["group"]]) && missing(group)) {
            group <- seq_len(n)
        }
    }
    ## Build a GRanges object from the inputs
    .missingToNull(c("feature", "group", "id", "strand", "chromosome", "importFunction", "genome"))
    args <- list(feature = feature, group = group, id = id, strand = strand, chromosome = chromosome, genome = genome)
    defs <- list(feature = "unknown", group = "unknown", id = "unknown", strand = "*", density = 1, chromosome = "chrNA", genome = NA)
    range <- .buildRange(
        range = range, groupId = "group", start = start, end = end, width = width,
        args = args, defaults = defs, chromosome = chromosome, trackType = "AnnotationTrack",
        importFun = importFunction, stream = stream
    )
    if (is.list(range)) {
        isStream <- TRUE
        slist <- range
        range <- GRanges()
    }
    ## Pipes have a special meaning for merged groups, so we can't have them in the initial group vector
    mcols(range)[["group"]] <- gsub("|", "", mcols(range)[["group"]], fixed = TRUE)
    ## If no chromosome was explicitely asked for we just take the first one in the GRanges object
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- if (length(range) > 0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    }
    ## And finally the object instantiation, we have to distinguish between DetailsAnnotationTracks and normal ones
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    if (missing(fun)) {
        if (!isStream) {
            return(new("AnnotationTrack",
                chromosome = as.character(chromosome[1]), range = range,
                name = name, genome = genome, stacking = stacking, ...
            ))
        } else {
            ## A bit hackish but for some functions we may want to know which track type we need but at the
            ## same time we do not want to enforce this as an additional argument
            e <- new.env()
            e[["._trackType"]] <- "AnnotationTrack"
            environment(slist[["stream"]]) <- e
            return(new("ReferenceAnnotationTrack",
                chromosome = as.character(chromosome[1]), range = range,
                name = name, genome = genome, stacking = stacking, stream = slist[["stream"]], reference = slist[["reference"]],
                mapping = slist[["mapping"]], args = args, defaults = defs, ...
            ))
        }
    } else {
        if (!is.function(fun)) {
            stop("'fun' must be a function")
        }
        if (missing(selectFun)) {
            selectFun <- function(...) {
                return(TRUE)
            }
        }
        if (!is.function(selectFun)) {
            stop("'selectFun' must be a function")
        }
        return(new("DetailsAnnotationTrack",
            chromosome = as.character(chromosome[1]), range = range,
            name = name, genome = genome, stacking = stacking, fun = fun, selectFun = selectFun, ...
        ))
    }
}

#' @describeIn AnnotationTrack-class directly extends `AnnotationTrack.`
#' @exportClass DetailsAnnotationTrack
setClass("DetailsAnnotationTrack",
    contains = "AnnotationTrack",
    representation = representation(fun = "function", selectFun = "function"),
    prototype = prototype(
        fun = function(...) {},
        selectFun = function(...) {
            return(TRUE)
        },
        dp = DisplayPars(
            details.minWidth = 100,
            details.ratio = Inf,
            details.size = 0.5,
            detailsBorder.col = "darkgray",
            detailsBorder.fill = "transparent",
            detailsBorder.lty = "solid",
            detailsBorder.lwd = 1,
            detailsConnector.cex = 1,
            detailsConnector.col = "darkgray",
            detailsConnector.lty = "dashed",
            detailsConnector.lwd = 1,
            detailsConnector.pch = 20,
            detailsFunArgs = list(),
            groupDetails = FALSE
        )
    )
)

#' @describeIn AnnotationTrack-class Constructor function for
#' `DetailsAnnotationTrack-class`
#'
#' The `DetailsAnnotationTrack` class directly extends `AnnotationTrack.`
#' The purpose of this track type is to add an arbitrarily detailed plot
#' section (typically consisting of additional quantitative data) for each
#' range element of an `AnnotationTrack.` This allows a locus wide view of
#' annotation elements together with any kind of details per feature or element
#'  that may for instance provide insight on how some complex quantitative
#'  measurements change according to their position in a locus. If the
#'  quantitative data is too complex for a `DataTrack` e.g. because it requires
#'  extra space or a trellis-like representation, a `DetailsAnnotationTrack` can
#'  be used instead. Example: An `AnnotationTrack` shows the positions of a
#'  number of probes from a microarray, and you want a histogram of the signal
#'  intensity distribution derived from all samples at each of these probe
#'  location. Another example usage would be to show for each element of an
#'  `AnnotationTrack` an xy-plot of the signal against some clinical measurement
#'  such as blood pressure. The limitation for applications of this type of
#'  track is basically only the available space of the device you are
#'  plotting to.
#'
#' This flexibility is possible by utilizing a simple function model
#' to perform all the detailed plotting. The functionality of this plotting
#' function fun is totally up to the user, and the function environment is
#' prepared in a way that all necessary information about the plotted
#' annotation feature is available. To restrict the details section to only
#' selected number of annotation features one can supply another function
#' `selectFun`, which decides for each feature separately whether details are
#' available or not. Finally, an arbitrary number of additional arguments can
#' be passed on to these two function by means of the `detailsFunArgs` display
#' parameter. This is expected to be a named list, and all list elements are
#' passed along to the plotting function fun and to the selector function
#' `selectFun` as additional named arguments. Please note that some argument
#' names like `start`, `end` or `identifier` are reserved and can not be used
#' in the `detailsFunArgs` list. For examples of plotting functions,
#' see the 'Examples' section.
#' @export
DetailsAnnotationTrack <- function(...) AnnotationTrack(...)

## Initialize ----------------------------------------------------------------

#' @describeIn AnnotationTrack-class Initialize.
#' @export
setMethod("initialize", "DetailsAnnotationTrack", function(.Object, fun, selectFun, ...) {
    ## the diplay parameter defaults
    .Object <- .updatePars(.Object, "DetailsAnnotationTrack")
    .makeParMapping()
    .Object@fun <- fun
    .Object@selectFun <- selectFun
    .Object <- callNextMethod(.Object, ...)
    return(.Object)
})

## General accessors ---------------------------------------------------------
## Annotation Accessors ------------------------------------------------------

#' @describeIn AnnotationTrack-class extract the group membership for all track items.
#' @export
setMethod("group", "AnnotationTrack", function(GdObject) .getAnn(GdObject, "group"))

#' @describeIn AnnotationTrack-class replace the grouping information for track items.
#' The replacement value must be a factor of appropriate length or another
#' vector that can be coerced into such.
#' @export
setReplaceMethod("group", signature("AnnotationTrack", "character"), function(GdObject, value) .setAnn(GdObject, value, "group"))

## Context-dependent meta-accessors to the identifier data of an AnnotationTrack and a GeneRegionTrack. For the former, those will
## be the content of the 'id', the 'group' or the 'feature' metadata column, for the latter, one in
## the selection of 'symbol', 'gene', 'transcript', 'feature' or 'exon'. For historical reasons, logical values are also allowed, where TRUE
## is equivalent to 'symbol' and FALSE is equivalent to 'gene'. If not provided explicitely in the 'type' argument, the identifier
## type is inferred from the 'transcriptAnnotation' display parameter (a.k.a. 'geneSymbols') for a GeneRegionTrack, and from the
## 'groupAnnotation' parameter for an AnnotationTrack. The special value 'lowest' should work across all classes and provided the most
## detailed annotation ('id' for AnnotationTrack and 'exon' for GeneRegionTrack).

#' @describeIn AnnotationTrack-class return track item identifiers.
#' Depending on the setting of the optional argument lowest, these are either
#' the group identifiers or the individual item identifiers.
#' @export
setMethod("identifier", "AnnotationTrack", function(GdObject, type = .dpOrDefault(GdObject, "groupAnnotation", "group")) {
    if (is.null(type)) {
        type <- "group"
    }
    id <- switch(as.character(type),
        "group" = group(GdObject),
        "id" = .getAnn(GdObject, "id"),
        "feature" = feature(GdObject),
        "lowest" = .getAnn(GdObject, "id"),
        group(GdObject)
    )
    id[is.na(id)] <- "NA"
    return(id)
})

#' @describeIn AnnotationTrack-class Set the track item identifiers.
#' The replacement value has to be a character vector of appropriate length.
#' This always replaces the group-level identifiers, so essentially it is
#' similar to `groups<-`.
#' @export
setReplaceMethod("identifier", c("AnnotationTrack", "character"), function(GdObject, value) {
    type <- .dpOrDefault(GdObject, "groupAnnotation", "group")
    switch(as.character(type),
        "group" = group(GdObject) <- value,
        "id" = ranges(GdObject)$id <- value,
        "feature" = feature(GdObject) <- value,
        "lowest" = ranges(GdObject)$id <- value,
        group(GdObject) <- value
    )
    return(GdObject)
})
## Stacking ------------------------------------------------------------------

#' @describeIn AnnotationTrack-class Recompute the stacks based on the available
#' space and on the object's track items and stacking settings.
#' @export
setMethod("setStacks", "AnnotationTrack", function(GdObject, recomputeRanges = TRUE) {
    if (!.needsStacking(GdObject) || length(GdObject) == 0) {
        ## No stacks needed, so there is just a single bin
        bins <- rep(1, length(GdObject))
    } else {
        gp <- group(GdObject)
        if (!is.factor(gp)) {
            gp <- factor(gp, levels=unique(gp))
        }
        needsGrp <- any(duplicated(gp))
        lranges <- .dpOrDefault(GdObject, ".__groupRanges")
        gpt <- if (needsGrp) table(gp) else rep(1, length(GdObject))
        if (recomputeRanges || is.null(lranges) || length(lranges) != length(gpt)) {
            GdObject <- .computeGroupRange(GdObject)
            lranges <- .dpOrDefault(GdObject, ".__groupRanges")
        }
        uid <- if (.transcriptsAreCollapsed(GdObject)) {
            sprintf("uid%i", seq_along(identifier(GdObject)))
        } else {
            make.unique(identifier(GdObject, type = "lowest"))
        }

        bins <- rep(disjointBins(lranges), gpt)
        names(bins) <- if (needsGrp) unlist(split(uid, gp)) else uid
        bins <- bins[uid]
    }
    bins <- if (length(bins)) bins else 0
    GdObject@stacks <- bins
    return(GdObject)
})
## Consolidate ---------------------------------------------------------------

#' @describeIn AnnotationTrack-class Consolidate.
#' Determine whether there is group label annotation or not, and add this
#' information as the internal display parameter `.__hasAnno`. Precompute
#' the grouped ranges together with optional labels in order to determine
#' the correct plotting range later.
#' @keywords internal
setMethod("consolidateTrack", signature(GdObject = "AnnotationTrack"), function(GdObject, hasAxis = FALSE,
                                                                                hasTitle = .dpOrDefault(GdObject, "showTitle", TRUE),
                                                                                title.width = NULL, ...) {
    GdObject <- callNextMethod(GdObject, ...)
    if (length(GdObject)) {
        ## ids are shown if either set by the showId parameter, or through the use of the transcriptAnnotation or
        ## groupAnnotation parameters
        ids <- identifier(GdObject)
        sid <- .dpOrDefault(GdObject, "showId")
        ta <- if (is(GdObject, "GeneRegionTrack")) .dpOrDefault(GdObject, "transcriptAnnotation") else .dpOrDefault(GdObject, "groupAnnotation")
        sid <- if (is.null(sid)) !is.null(ta) && ta != "none" else sid
        hasAnno <- sid && !all(ids == "")
        ## element ids are shown if either the showFeatureId or showExonId parameters are TRUE, or if featureAnnotation
        ## or exonAnnotation is set
        sfid <- if (is(GdObject, "GeneRegionTrack")) .dpOrDefault(GdObject, "showExonId") else .dpOrDefault(GdObject, "showFeatureId")
        fa <- if (is(GdObject, "GeneRegionTrack")) .dpOrDefault(GdObject, "exonAnnotation") else .dpOrDefault(GdObject, "featureAnnotation")
        sfid <- if (is.null(sfid)) !is.null(fa) && fa != "none" else sfid
        displayPars(GdObject) <- list(".__hasAnno" = hasAnno, showId = sid, showFeatureId = sfid, showExonId = sfid)
        GdObject <- .computeGroupRange(GdObject, hasAxis = hasAxis, hasTitle = hasTitle, title.width = title.width)
    } else {
        displayPars(GdObject) <- list(".__hasAnno" = FALSE, showId = FALSE, showFeatureId = FALSE, showExonId = FALSE)
    }
    return(GdObject)
})

## Collapse  -----------------------------------------------------------------

## A slightly quicker function to compute overlaps between two GRanges objects
.myFindOverlaps <- function(gr1, gr2) {
    gr1 <- sort(gr1)
    gr2 <- sort(gr2)
    gr1 <- split(gr1, seqnames(gr1))
    gr2 <- split(gr2, seqnames(gr2))
    queryHits(findOverlaps(ranges(gr1), ranges(gr2)))
}
## Find all elements in a GRanges object 'grange' with distance smaller than 'minXDist' and merge them along with their additional
## metadata columns. 'elements' is a frequency table of items per group, and it is needed to figure out whether all items of a given
## group have been merged. 'GdObject' is the input track object from which certain information has to be extracted. The output of this
## function is a list with elements
##   o range: the updated GRanges object
##   o needsRestacking: logical flag indicating whether stacks have to be recomputed
##   o split: the original merged annotation (need to work on this, currently not used)
##   o merged: logical vector indicating which of the elements in 'range' constitute fully merged groups
.collapseAnnotation <- function(grange, minXDist, elements, GdObject, offset = 0) {
    needsRestacking <- TRUE
    annoSplit <- merged <- NULL
    anno <- as.data.frame(grange)
    for (i in colnames(anno)) {
        if (is.factor(anno[, i])) {
            anno[, i] <- as.character(anno[, i])
        }
    }
    cols <- c("strand", "density", "gdensity", "feature", "id", "start", "end", if (is(GdObject, "GeneRegionTrack")) {
        c("gene", "exon", "transcript", "symbol", "rank")
    } else {
        "group"
    })
    missing <- which(!cols %in% colnames(anno))
    for (i in missing) {
        anno[, cols[missing]] <- if (cols[i] == "density") 1 else NA
    }
    rRed <- if (length(grange) > 1) reduce(grange, min.gapwidth = minXDist, with.revmap = TRUE) else grange
    if (length(rRed) < length(grange)) {
        ## Some of the items have to be merged and we need to make sure that the additional annotation data that comes with it
        ## is processed in a sane way.
        needsRestacking <- TRUE
        mapping <- rep(seq_along(rRed$revmap), elementNROWS(rRed$revmap))
        ## We start by finding the items that have not been reduced
        identical <- mapping %in% which(table(mapping) == 1)
        newVals <- anno[identical, cols]
        ## Here we hijack the seqnames column to indicate whether the whole group has been merged
        if (nrow(newVals)) {
            newVals$seqnames <- elements[as.character(anno[identical, "seqnames"])] == 1
            newVals$gdensity <- ifelse(elements[as.character(anno[identical, "seqnames"])] == 1, 1, NA)
        }
        ## Now find out which original items have been merged
        grange <- grange[!identical]
        rRed <- rRed[-(mapping[identical])]
        index <- mapping[!identical]
        annoSplit <- split(anno[!identical, ], index)
        cid <- function(j) sprintf("[Cluster_%i]  ", j + offset)
        ## FIXME: We could speed this up by running it in C
        tmpVals <- lapply(seq_along(annoSplit), function(i) {
            x <- annoSplit[[i]]
            if (is(GdObject, "GeneRegionTrack")) {
                c(
                    strand = ifelse(length(unique(x[, "strand"])) == 1, as.character(x[1, "strand"]), "*"),
                    density = sum(as.integer(x[, "density"])),
                    gdensity = ifelse(is.na(head(x[, "gdensity"], 1)), 1, sum(as.integer(x[, "gdensity"]))),
                    feature = ifelse(length(unique(x[, "feature"])) == 1, as.character(x[1, "feature"]), "composite"),
                    id = ifelse(length(unique(x[, "id"])) == 1, as.character(x[1, "id"]), cid(i)),
                    start = min(x[, "start"]),
                    end = max(x[, "end"]),
                    gene = ifelse(length(unique(x[, "gene"])) == 1, as.character(x[1, "gene"]), cid(i)),
                    exon = ifelse(length(unique(x[, "exon"])) == 1, as.character(x[1, "exon"]), cid(i)),
                    transcript = ifelse(length(unique(x[, "transcript"])) == 1, as.character(x[1, "transcript"]), cid(i)),
                    symbol = ifelse(length(unique(x[, "symbol"])) == 1, as.character(x[1, "symbol"]), cid(i)),
                    rank = min(as.integer(x[, "rank"])), seqnames = as.vector(nrow(x) == elements[x[1, "seqnames"]])
                )
            } else {
                c(
                    strand = ifelse(length(unique(x[, "strand"])) == 1, as.character(x[1, "strand"]), "*"),
                    density = sum(as.integer(x[, "density"])),
                    gdensity = ifelse(is.na(head(x[, "gdensity"], 1)), 1, sum(as.integer(x[, "gdensity"]))),
                    feature = ifelse(length(unique(x[, "feature"])) == 1, as.character(x[1, "feature"]), "composite"),
                    id = ifelse(length(unique(x[, "id"])) == 1, as.character(x[1, "id"]), cid(i)),
                    start = min(x[, "start"]),
                    end = max(x[, "end"]),
                    group = ifelse(length(unique(x[, "group"])) == 1, as.character(x[1, "group"]), cid(i)),
                    seqnames = as.vector(nrow(x) == elements[x[1, "seqnames"]])
                )
            }
        })
        newVals <- rbind(newVals, as.data.frame(do.call(rbind, tmpVals), stringsAsFactors = FALSE))
        merged <- as.logical(newVals$seqnames)
        grange <- GRanges(
            seqnames = chromosome(GdObject), strand = newVals[, "strand"],
            ranges = IRanges(start = as.integer(newVals[, "start"]), end = as.integer(newVals[, "end"]))
        )
        cnMatch <- match(c(colnames(values(GdObject)), "gdensity"), colnames(newVals))
        mcols(grange) <-
            if (any(is.na(cnMatch))) newVals[, setdiff(colnames(newVals), c("strand", "start", "end", "seqnames"))] else newVals[, cnMatch]
    } else {
        grange2 <- GRanges(seqnames = chromosome(GdObject), strand = strand(grange), ranges = ranges(grange))
        mcols(grange2) <- mcols(grange)
        grange <- grange2
    }
    return(list(range = grange, needsRestacking = needsRestacking, split = annoSplit, merged = merged, offset = length(annoSplit)))
}

## For AnnotationTracks we need to collapse the all regions along with the additional annotation.
## For GeneRegionTracks we essentially need to do the same thing as for AnnotationTracks, however the additional annotation columns
## are quite different. We do this in multiple turn with increasing levels of complexity:
##    1.) merge all individual items within a group that can no longer be separated
##    2.) merge overlapping groups with just a single remaining item (optional, if mergeGroups==TRUE)

#' @describeIn AnnotationTrack-class preprocess the track before plotting.
#' This will collapse overlapping track items based on the available resolution
#' and increase the width and height of all track objects to a minimum value
#' to avoid rendering issues. See collapsing for details.
#' @keywords internal
setMethod(
    "collapseTrack", signature(GdObject = "AnnotationTrack"),
    function(GdObject, diff = .pxResolution(coord = "x"), xrange) {
        ## We first add the original unmodified GdObject as a display parameter to be able to reference back if we ever need to
        displayPars(GdObject) <- list(".__OriginalGdObject" = .deepCopyPars(GdObject))
        collapse <- .dpOrDefault(GdObject, "collapse", TRUE)
        min.width <- .dpOrDefault(GdObject, "min.width", 2)
        min.distance <- .dpOrDefault(GdObject, "min.distance", 2)
        minXDist <- max(0, ceiling(min.distance * diff))
        r <- ranges(GdObject)
        ## Compute native coordinate equivalent to 1 pixel and resize
        rNew <- .resize(r, min.width, diff)
        needsRestacking <- any(r != rNew)
        r <- rNew
        ## Collapse all items within a group to a single meta-item (if collapseTranscripts is set)
        if (is(GdObject, "GeneRegionTrack")) {
            ctrans <- .dpOrDefault(GdObject, "collapseTranscripts", FALSE)
            if (is.logical(ctrans) && ctrans) {
                ctrans <- "gene"
            }
            switch(ctrans,
                "gene" = {
                    newVals <- unlist(endoapply(split(values(r), paste(gene(GdObject), strand(GdObject))), head, 1))
                    newVals$exon <- NA
                    newVals$feature <- "merged"
                    newVals$transcript <- newVals$gene
                    r <- unlist(range(split(r, gene(GdObject))))
                    mcols(r) <- newVals
                    GdObject@range <- r
                },
                "longest" = {
                    r <- unlist(endoapply(split(r, gene(GdObject)), function(x) {
                        xs <- split(x, x$transcript)
                        xs[[which.max(vapply(xs, function(y) abs(diff(as.numeric(as.data.frame(ranges(range(y)))[, c("start", "end")]))), FUN.VALUE = numeric(1L)))]]
                    }))
                    GdObject@range <- r
                },
                "shortest" = {
                    r <- unlist(endoapply(split(r, gene(GdObject)), function(x) {
                        xs <- split(x, x$transcript)
                        xs[[which.min(vapply(xs, function(y) abs(diff(as.numeric(as.data.frame(ranges(range(y)))[, c("start", "end")]))), FUN.VALUE = numeric(1L)))]]
                    }))
                    GdObject@range <- r
                },
                "meta" = {
                    newVals <- unlist(endoapply(split(values(r), paste(gene(GdObject), strand(GdObject))), head, 1))
                    newVals$feature <- "merged"
                    newVals$transcript <- newVals$gene
                    rtmp <- reduce(split(r, paste(gene(GdObject), strand(GdObject))))
                    newVals <- newVals[rep(seq_along(rtmp), elementNROWS(rtmp)), ]
                    newVals$exon <- paste("merged_exon_", unlist(lapply(elementNROWS(rtmp), function(x) seq(1, x)), use.names = FALSE), sep = "")
                    r <- unlist(rtmp)
                    mcols(r) <- newVals
                    GdObject@range <- r
                }
            )
        }
        ## Collapse overlapping ranges (less than minXDist space between them) and process the annotation data
        if (collapse) {
            ## Merge all items in those groups for which no individual items can be separated
            elements <- table(group(GdObject))
            rr <- GRanges(seqnames = as.character(group(GdObject)), ranges = IRanges(start = start(r), end = end(r)), strand = strand(r))
            mcols(rr) <- mcols(r)
            rr <- sort(unique(rr))
            mergedAnn <- .collapseAnnotation(rr, minXDist, elements, GdObject)
            needsRestacking <- needsRestacking || mergedAnn$needsRestacking
            ## Now we take a look whether there are any groups that could be merged (if mergeGroups is TRUE)
            if (.dpOrDefault(GdObject, "mergeGroups", FALSE) && any(mergedAnn$merged)) {
                rr <- sort(mergedAnn$range[mergedAnn$merged])
                strand(rr) <- "*"
                mergedAnn2 <- .collapseAnnotation(rr, minXDist, elements, GdObject, mergedAnn$offset)
                needsRestacking <- needsRestacking || mergedAnn2$needsRestacking
                mergedAnn$range <- c(mergedAnn$range[!mergedAnn$merged], mergedAnn2$range)
            }
            r <- mergedAnn$range
        }
        ## Reconstuct the track object and return
        GdObject@range <- r
        ## if(needsRestacking)
        GdObject <- setStacks(GdObject)
        return(GdObject)
    }
)

## Subset --------------------------------------------------------------------

## In order to keep the grouping information for track regions in the clipped areas we have to
## keep all group elements that overlap with the range. We still want to record the requested
## ranges in the internal '.__plottingRange' display parameter.

#' @describeIn AnnotationTrack-class subset a `AnnotationTrack` by coordinates
#' and sort if necessary.
#' @export
setMethod("subset", signature(x = "AnnotationTrack"), function(x, from = NULL, to = NULL, sort = FALSE, stacks = FALSE, use.defaults = TRUE, ...) {
    ## Subset to a single chromosome first
    lx <- length(x)
    csel <- seqnames(x) != chromosome(x)
    if (any(csel)) {
        x <- x[!csel]
    }
    if (length(x)) {
        ## Nothing to do if everything is within the range
        granges <- unlist(range(split(ranges(x), group(x))))
        ranges <- if (use.defaults) {
            .defaultRange(x, from = from, to = to)
        } else {
            c(
                from = ifelse(is.null(from), min(start(granges)) - 1, from),
                to = ifelse(is.null(to), max(end(granges)) + 1, to)
            )
        }
        if (!(any(end(x) < ranges["from"] | start(x) > ranges["to"]))) {
            if (stacks) {
                x <- setStacks(x)
            }
            return(x)
        }
        ## Now remove everything except for the overlapping groups by first subselecting all groups in the range...
        gsel <- names(granges)[subjectHits(findOverlaps(GRanges(seqnames = chromosome(x), ranges = IRanges(min(ranges), max(ranges))), granges))]
        x <- x[group(x) %in% gsel]
        if (sort) {
            x <- x[order(range(x)), ]
        }
        if (stacks) {
            x <- setStacks(x)
        }
        displayPars(x) <- list(".__plottingRange" = ranges)
    }
    if (length(x) != lx) {
        x <- .computeGroupRange(x)
    }
    return(x)
})

## ReferenceDataTracks need to stream the data from file and then pass the results on to the next method

#' @describeIn AnnotationTrack-class subset a `ReferenceAnnotationTrack` by
#' coordinates and sort if necessary.
#' @export
setMethod("subset", signature(x = "ReferenceAnnotationTrack"), function(x, from, to, chromosome, ...) {
    ## We only need to reach out into the referenced file once if the range is already contained in the object
    if (missing(from) || is.null(from) || missing(to) || is.null(to)) {
        stop("Need both start and end location to subset a ReferenceAnnotationTrack")
    }
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- Gviz::chromosome(x)
    }
    subRegion <- GRanges(seqnames = chromosome[1], ranges = IRanges(start = from, end = to))
    if (length(ranges(x)) == 0 || all(overlapsAny(ranges(x), subRegion))) {
        cMap <- .resolveColMapping(x@stream(x@reference, subRegion), x@args, x@mapping)
        x@range <- .buildRange(cMap$data, args = cMap$args, defaults = x@defaults, trackType = "AnnotationTrack")
        chromosome(x) <- chromosome[1]
    }
    return(callNextMethod(x = x, from = from, to = to, drop = FALSE, ...))
})

## Position ------------------------------------------------------------------
## DrawGrid ------------------------------------------------------------------
setMethod("drawGrid", signature(GdObject = "AnnotationTrack"), function(GdObject, from, to) {
    if (.dpOrDefault(GdObject, "grid", FALSE)) {
        pushViewport(dataViewport(xData = c(from, to), extension = c(0, 0), yData = 0:1, clip = TRUE))
        panel.grid(
            h = 0, v = .dpOrDefault(GdObject, "v", -1),
            col = .dpOrDefault(GdObject, "col.grid", "#e6e6e6"), lty = .dpOrDefault(GdObject, "lty.grid", 1),
            lwd = .dpOrDefault(GdObject, "lwd.grid", 1)
        )
        popViewport(1)
    }
})
## DrawAxis ------------------------------------------------------------------
## DrawGD --------------------------------------------------------------------

## Draw gene models as found in AnnotationTracks or GeneRegionTracks
##
## Calculate all coordinates and values for the individual stacks first, and append to the
## input list 'toDraw'. This allows us to plot the whole track at once, making use of grid's
## vectorization. Without this tweak, every stack would have to be drawn individually, which
## can be painfully slow...
## Because all of the y coordinates are calculated in the coordinate system of the current
## viewport for the respective stack we have to add offset values.

## Compute the coordinates and colors for all track items (e.g. exons in a GeneRegionTrack)
.boxes <- function(GdObject, offsets) {
    ylim <- c(0, 1)
    h <- diff(ylim)
    middle <- mean(ylim)
    sh <- max(0, min(h, .dpOrDefault(GdObject, "stackHeight", 0.75)))
    space <- (h - (h * sh)) / 2
    shift <- switch(.dpOrDefault(GdObject, "stackJust", "middle"),
        "top" = space,
        "bottom" = -space,
        0
    )
    if (inherits(GdObject, "GeneRegionTrack")) {
        thinBox <- .dpOrDefault(GdObject, "thinBoxFeature", .THIN_BOX_FEATURES)
        space <- ifelse(feature(GdObject) %in% thinBox, space + ((middle -
            space) / 2), space)
    }
    shape <- .dpOrDefault(GdObject, "shape", "arrow")
    color <- .getBiotypeColor(GdObject)
    id <- identifier(GdObject, type = .dpOrDefault(
        GdObject, ifelse(is(GdObject, "GeneRegionTrack"), "exonAnnotation", "featureAnnotation"),
        "lowest"
    ))
    sel <- grepl("\\[Cluster_[0-9]*\\]", id)
    id[sel] <- sprintf(
        "%i merged\n%s", as.integer(.getAnn(GdObject, "density")[sel]),
        ifelse(class(GdObject) %in% c("AnnotationTrack", "DetailsAnnotationTrack"), "features", "exons")
    )
    yy1 <- ylim[1] + space + offsets + shift
    yy2 <- ylim[2] - space + offsets + shift
    boxes <- data.frame(
        cx1 = start(GdObject), cy1 = yy1, cx2 = start(GdObject) + width(GdObject), cy2 = yy2,
        fill = color, strand = strand(GdObject), text = id, textX = start(GdObject) + (width(GdObject) / 2), textY = middle + offsets,
        .getImageMap(cbind(start(GdObject), yy1, end(GdObject), yy2)),
        start = start(GdObject), end = end(GdObject), values(GdObject), exonId = id, origExonId = identifier(GdObject, type = "lowest"),
        stringsAsFactors = FALSE
    )
    rownames(boxes) <- if (.transcriptsAreCollapsed(GdObject)) {
        sprintf("uid%i", seq_along(identifier(GdObject)))
    } else {
        make.unique(identifier(GdObject, type = "lowest"))
    }
    return(boxes)
}

## Compute the coordinates for the bars connecting grouped items and the group labels

#' @importFrom grDevices col2rgb hsv rgb2hsv
.barsAndLabels <- function(GdObject) {
    bins <- stacks(GdObject)
    stacks <- max(bins)
    res <- .pxResolution(coord = "x")
    gp <- group(GdObject)
    if (!is.factor(gp)) {
        gp <- factor(gp, levels=unique(gp))
    }
    grpSplit <- split(range(GdObject), gp)
    grpRanges <- unlist(range(grpSplit))
    needBar <- vapply(grpSplit, length, FUN.VALUE = numeric(1L)) > 1 & width(grpRanges) > res
    ## If we draw the bar from start to end of the range we sometimes see little overlaps that extend beyond the first or last item.
    ## In order to fix this, we just subtract the equivalent of min.width pixels from both ends of each group range
    min.swidth <- res * .dpOrDefault(GdObject, "min.width", 2)
    nstart <- start(grpRanges[needBar]) + min.swidth
    nend <- end(grpRanges[needBar]) - min.swidth
    sel <- (nend - nstart) > 0
    start(grpRanges[needBar][sel]) <- nstart[sel]
    end(grpRanges[needBar][sel]) <- nend[sel]
    strand <- vapply(split(strand(GdObject), gp), function(x) {
        tmp <- unique(x)
        if (length(tmp) > 1) "*" else tmp
    }, FUN.VALUE = character(1L))
    yloc <- vapply(split((stacks - bins) + 1, gp), function(x) unique(x), FUN.VALUE = numeric(1L)) + 0.5
    color <- if (length(grep("__desatCol", values(GdObject)$feature[1]))) {
        .dpOrDefault(GdObject, "fill", .DEFAULT_FILL_COL)
    } else {
        vapply(split(.getBiotypeColor(GdObject), gp), head, 1, FUN.VALUE = character(1L))
    }
    bars <- data.frame(
        sx1 = start(grpRanges)[needBar], sx2 = end(grpRanges)[needBar], y = yloc[needBar], strand = strand[needBar],
        col = color[needBar], stringsAsFactors = FALSE
    )
    labs <- .dpOrDefault(GdObject, ".__groupLabels")[names(grpRanges)]
    if (!is.null(labs)) {
        lsel <- grepl("\\[Cluster_[0-9]*\\]", labs)
        if (any(lsel)) {
            gdens <- as.integer(vapply(split(.getAnn(GdObject, "gdensity"), gp), head, 1, FUN.VALUE = character(1L)))
            labs[lsel] <- sprintf(
                "%i merged %s  ", gdens[lsel],
                ifelse(class(GdObject) %in% c("AnnotationTrack", "DetailsAnnotationTrack"), "groups", "transcript models")
            )
        }
        just <- .dpOrDefault(GdObject, "just.group", "left")
        rev <- .dpOrDefault(GdObject, "reverseStrand", FALSE)
        sizes <- .dpOrDefault(GdObject, ".__groupLabelWidths")[names(grpRanges), , drop = FALSE]
        pr <- .dpOrDefault(GdObject, ".__plottingRange", data.frame(from = min(start(GdObject)), to = max(end(GdObject))))
        ## grpRangesCut <- restrict(grpRanges, start=as.integer(pr["from"]), end=as.integer(pr["to"]))
        switch(just,
            "left" = {
                cx <- if (!rev) start(grpRanges) - sizes$after else end(grpRanges) + sizes$after
                cy <- yloc
                algn <- c("right", "center")
            },
            "right" = {
                cx <- if (!rev) end(grpRanges) + sizes$after else start(grpRanges) - sizes$after
                cy <- yloc
                algn <- c("left", "center")
            },
            "above" = {
                ## cx <- start(grpRangesCut) + width(grpRangesCut)/2
                ## indx <- which(seq_len(length(grpRanges)) %in% queryHits(findOverlaps(grpRanges, grpRangesCut)))
                ## cx <- cx[indx] # needs revision
                ## cy <- yloc[indx] + 0.5
                ## labs <- labs[indx]
                cx <- start(grpRanges) + width(grpRanges) / 2
                cy <- yloc + 0.5
                algn <- c("center", "top")
            },
            "below" = {
                ## cx <- start(grpRangesCut) + width(grpRangesCut)/2
                ## indx <- which(seq_len(length(grpRanges)) %in% queryHits(findOverlaps(grpRanges, grpRangesCut)))
                ## cx <- cx[indx] # needs revision
                ## cy <- yloc[indx] - 0.5
                ## labs <- labs[indx]
                cx <- start(grpRanges) + width(grpRanges) / 2
                cy <- yloc - 0.5
                algn <- c("center", "bottom")
            },
            stop(sprintf("Unknown label justification '%s'", just))
        )

        labels <- data.frame(txt = labs, x = cx, y = cy, stringsAsFactors = FALSE)
    } else {
        labels <- algn <- NA
    }
    return(list(bars = bars, labels = labels, align = algn))
}

#' @describeIn AnnotationTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @export
setMethod("drawGD", signature("AnnotationTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...) {
    debug <- .dpOrDefault(GdObject, "debug", FALSE)
    if ((is.logical(debug) && debug) || debug == "prepare") {
        browser()
    }
    imageMap(GdObject) <- NULL
    if (!length(GdObject)) {
        return(invisible(GdObject))
    }
    ## In prepare mode we need to make sure that the stacking information is updated from the optional display parameter (by calling
    ## the StackedTrack drawGD method) and also perform the collapsing of track items which could potentially lead to re-stacking.
    if (prepare) {
        GdObject <- callNextMethod(GdObject, ...)
        bins <- stacks(GdObject)
        stacks <- max(bins)
        ## We need to collapse the track object based on the current screen resolution (note that this may trigger re-stacking)
        pushViewport(dataViewport(xData = c(minBase, maxBase), extension = 0, yscale = c(1, stacks + 1), clip = TRUE))
        GdObject <- collapseTrack(GdObject, diff = .pxResolution(coord = "x"), xrange = c(minBase, maxBase))
        popViewport(1)
        return(invisible(GdObject))
    }
    if ((is.logical(debug) && debug) || debug == "draw") {
        browser()
    }
    ## If there are too many stacks for the available device resolution we cast an error, otherwise we set the viewport
    bins <- stacks(GdObject)
    stacks <- max(bins)
    rev <- .dpOrDefault(GdObject, "reverseStrand", FALSE)
    xscale <- if (!rev) c(minBase, maxBase) else c(maxBase, minBase)
    yscale <- if (!.dpOrDefault(GdObject, "reverseStacking", FALSE)) c(1, stacks + 1) else c(stacks + 1, 1)
    pushViewport(dataViewport(xscale = xscale, extension = 0, yscale = yscale, clip = TRUE))
    res <- .pxResolution(coord = "x")
    curVp <- vpLocation()
    if (curVp$size["height"] / stacks < .dpOrDefault(GdObject, "min.height", 3)) {
        stop("Too many stacks to draw. Either increase the device size or limit the drawing to a smaller region.")
    }
    ## We adjust the color saturation to indicate overplotting if necessary
    if (.dpOrDefault(GdObject, "showOverplotting", FALSE)) {
        dens <- as.numeric(values(GdObject)$density)
        if (length(unique(dens)) != 1) {
            minSat <- max(0.25, 1 / max(dens))
            minDens <- min(dens)
            rDens <- diff(range(dens))
            saturation <- minSat + ((dens - minDens) / rDens / (1 / (1 - minSat)))
            bc <- unique(.getBiotypeColor(GdObject))
            baseCol <- rgb2hsv(col2rgb(bc))
            desatCols <- unlist(lapply(saturation, function(x) hsv(baseCol[1, ], x, baseCol[3, ])))
            names(desatCols) <- paste(unique(feature(GdObject)), rep(dens, each = length(bc)), sep = "_")
            feature(GdObject) <- paste(feature(GdObject), dens, sep = "_")
            desatCols <- desatCols[unique(names(desatCols))]
            displayPars(GdObject) <- as.list(desatCols)
        }
    }
    ## Now we can pre-compute all the coordinates and settings for the elements to be drawn, ...
    box <- .boxes(GdObject, (stacks - bins) + 1)
    barsAndLab <- .barsAndLabels(GdObject)
    bar <- barsAndLab$bars
    bartext <- barsAndLab$labels
    ## ... get all the necessary display parameters
    shape <- .dpOrDefault(GdObject, "shape", "arrow")
    col.line <- .dpOrDefault(GdObject, "col.line")[1]
    border <- .dpOrDefault(GdObject, "col")[1]
    if (is.null(border)) {
        border <- ifelse(is(GdObject, "GeneRegionTrack"), NA, "transparent")
    }
    lwd <- .dpOrDefault(GdObject, "lwd", 2)
    lty <- .dpOrDefault(GdObject, "lty", 1)
    alpha <- .dpOrDefault(GdObject, "alpha", 1)
    rotation <- .dpOrDefaultFont(GdObject, "rotation", "item", 0)
    rotation.group <- .dpOrDefaultFont(GdObject, "rotation", "group", 0)
    just <- .dpOrDefault(GdObject, "just.group", "left")
    ## ... and finally draw whatever is needed
    if (nrow(box) > 0) {
        ## If we want to place a label on top or below the ranges we need to know how much size that will take up
        ## and adjust the plotting shapes accordingly
        drawLabel <- .dpOrDefault(GdObject, ".__hasAnno", FALSE) && !is.null(bartext) && !anyNA(bartext) &&
            nrow(bartext) > 0 && stacking(GdObject) != "dense"
        bs <- as.vector((stacks - bins) + 1)
        if (drawLabel && just %in% c("above", "below")) {
            labelHeights <- max(.getStringDims(GdObject, bartext$txt, subtype = "group")$height)
            labelSpace <- as.numeric(convertHeight(unit(2, "points"), "native"))
            avSpace <- (1 - max(box$cy2 - box$cy1)) / 2
            vadjust <- max(0, (labelHeights + (2 * labelSpace)) - avSpace)
            bh <- 1 - (avSpace * 2)
            bhNew <- bh - vadjust
            sfac <- bhNew / bh
            if (just == "above") {
                bar$y <- bar$y - (vadjust / 2)
                box$textY <- box$textY - (vadjust / 2)
                bartext$y <- bartext$y - labelSpace
                box$cy1 <- bs + ((box$cy1 %% 1 - avSpace) * sfac) + avSpace
                box$cy2 <- bs + ((box$cy2 %% 1 - avSpace) * sfac) + avSpace
            } else {
                bar$y <- bar$y + (vadjust / 2)
                box$textY <- box$textY + (vadjust / 2)
                bartext$y <- bartext$y + labelSpace
                box$cy1 <- bs + ((box$cy1 %% 1 - avSpace) * sfac) + avSpace + vadjust
                box$cy2 <- bs + ((box$cy2 %% 1 - avSpace) * sfac) + avSpace + vadjust
            }
        }
        ## Plotting of the (arrow)bar
        if (nrow(bar) > 0) {
            .arrowBar(bar$sx1, bar$sx2,
                y = bar$y, bar$strand, box[, seq_len(4), drop = FALSE],
                W = .dpOrDefault(GdObject, "arrowFeatherWidth", 3), D = .dpOrDefault(GdObject, "arrowFeatherDistance", 20),
                col = if (is.null(col.line)) bar$col else rep(col.line, length(bar$col)), lwd = lwd, lty = lty,
                alpha = alpha, barOnly = (!"smallArrow" %in% .dpOrDefault(GdObject, "shape", "box") || stacking(GdObject) == "dense"),
                diff = res, min.height = .dpOrDefault(GdObject, "min.height", 3)
            )
        }
        ## Plotting of the boxes
        box$col <- if (is.na(border)) box$fill else border
        if ("box" %in% shape || ("smallArrow" %in% shape && !("arrow" %in% shape || "ellipse" %in% shape))) {
            .filledBoxes(box, lwd = lwd, lty = lty, alpha = alpha)
        }
        ## Plotting of the ellipses
        if ("ellipse" %in% shape) {
            ellCoords <- .box2Ellipse(box)
            grid.polygon(
                x = ellCoords$x1, y = ellCoords$y1, id = ellCoords$id,
                gp = gpar(col = if (is.na(border)) box$fill else border, fill = box$fill, lwd = lwd, lty = lty, alpha = alpha),
                default.units = "native"
            )
        }
        ## Plotting of the filled arrows
        if ("arrow" %in% shape && !"box" %in% shape) {
            .filledArrow(box, lwd = lwd, lty = lty, alpha = alpha, min.width = 4 * res, max.width = .dpOrDefault(GdObject, "arrowHeadMaxWidth", 40) * res)
        }
        ## Plotting of the filled arrows with fixed head size
        if ("fixedArrow" %in% shape && !"box" %in% shape) {
            .filledArrow(box,
                lwd = lwd, lty = lty, alpha = alpha, min.width = 4 * res, absoluteWidth = TRUE,
                W = .dpOrDefault(GdObject, "arrowHeadWidth", 30) * res
            )
        }
        ## Plotting of the item labels
        if (.dpOrDefault(GdObject, "showFeatureId", FALSE)) {
            # grid.text(str2expression(box$text), box$textX, box$textY,
            grid.text(box$text, box$textX, box$textY,
                rot = rotation, gp = .fontGp(GdObject, subtype = "item"),
                default.units = "native", just = c("center", "center")
            )
        }
        ## Plotting of the group labels
        if (drawLabel) {
            grid.text(bartext$txt, bartext$x, bartext$y,
                rot = rotation.group, gp = .fontGp(GdObject, subtype = "group"),
                default.units = "native", just = barsAndLab$align
            )
        }
    }
    popViewport(1)
    ## Finally we set up the image map
    ## FIXME: we may want to record the merging information here
    im <- if (!is.null(box)) {
        coords <- as.matrix(box[, c("x1", "y1", "x2", "y2"), drop = FALSE])
        restCols <- setdiff(colnames(box), c("x1", "x2", "y1", "y2", "cx1", "cx2", "cy1", "cy2", "textX", "textY"))
        tags <- lapply(restCols, function(x) {
            tmp <- as.character(box[, x])
            names(tmp) <- rownames(coords)
            tmp
        })
        names(tags) <- restCols
        tags$title <- identifier(GdObject)
        ImageMap(coords = coords, tags = tags)
    } else {
        NULL
    }
    imageMap(GdObject) <- im
    return(invisible(GdObject))
})

## DrawGD DetailsAnnotationTrack ---------------------------------------------

## Create a data.frame with the distinct details function arguments (like start, end, ...)
.buildArgsDf <- function(GdObject) {
    groupDetails <- .dpOrDefault(GdObject, "groupDetails", FALSE)
    rr <- if (groupDetails) unlist(range(split(ranges(GdObject), group(GdObject)))) else ranges(GdObject)
    args <- data.frame(
        start = as.integer(start(rr)), end = as.integer(end(rr)), strand = as.character(strand(rr)),
        chromosome = as.character(seqnames(rr)),
        identifier = as.character(if (groupDetails) names(rr) else identifier(GdObject, type = "lowest")),
        stringsAsFactors = FALSE
    )
    return(args)
}

#' @describeIn AnnotationTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @export
setMethod(
    "drawGD", signature("DetailsAnnotationTrack"),
    function(GdObject, minBase, maxBase, prepare = FALSE, ...) {
        debug <- .dpOrDefault(GdObject, "debug", FALSE)
        if ((is.logical(debug) && debug) || debug == "prepare") {
            browser()
        }
        adf <- .buildArgsDf(GdObject)
        args <- .dpOrDefault(GdObject, "detailsFunArgs", fromPrototype = TRUE)
        groupDetails <- .dpOrDefault(GdObject, "groupDetails", FALSE)
        if (prepare) {
            GdObject <- callNextMethod()
            GdObject <- GdObject[order(start(GdObject))]
            indices <- if (groupDetails) seq_len(length(unique(group(GdObject)))) else seq_len(length(GdObject))
            xscale <- if (!.dpOrDefault(GdObject, "reverseStrand", FALSE)) c(minBase, maxBase) else c(maxBase, minBase)
            pushViewport(viewport(xscale = xscale))
            select <- lapply(indices, function(i) {
                warn <- FALSE
                iargs <- as.list(adf[i, ])
                iargs$index <- i
                iargs$GdObject <- GdObject
                iargs$GdObject.original <- .dpOrDefault(GdObject, ".__OriginalGdObject", GdObject)
                args <- c(args[setdiff(names(args), names(iargs))], iargs)
                res <- do.call(GdObject@selectFun, args)
                if (length(res) != 1 || !is.logical(res) || is.na(res)) {
                    res <- TRUE
                    warn <- TRUE
                }
                c(res = res, warn = warn)
            })
            select <- do.call(rbind, select)
            if (any(select[, "warn"])) {
                warning("The result of function 'selectFun' has to be a single logical value. Forcing the value to 'TRUE'")
            }
            select <- select[, "res"]
            popViewport(1)
            displayPars(GdObject) <- list(".__select" = select)
            return(invisible(GdObject))
        }
        if ((is.logical(debug) && debug) || debug == "draw") {
            browser()
        }
        n <- length(GdObject)
        col <- rep(.dpOrDefault(GdObject, "detailsConnector.col", fromPrototype = TRUE), n)[seq_len(n)]
        lty <- rep(.dpOrDefault(GdObject, "detailsConnector.lty", fromPrototype = TRUE), n)[seq_len(n)]
        lwd <- rep(.dpOrDefault(GdObject, "detailsConnector.lwd", fromPrototype = TRUE), n)[seq_len(n)]
        pch <- rep(.dpOrDefault(GdObject, "detailsConnector.pch", fromPrototype = TRUE), n)[seq_len(n)]
        cex <- rep(.dpOrDefault(GdObject, "detailsConnector.cex", fromPrototype = TRUE), n)[seq_len(n)]
        border.lty <- rep(.dpOrDefault(GdObject, "detailsBorder.lty", fromPrototype = TRUE), n)[seq_len(n)]
        border.lwd <- rep(.dpOrDefault(GdObject, "detailsBorder.lwd", fromPrototype = TRUE), n)[seq_len(n)]
        border.col <- rep(.dpOrDefault(GdObject, "detailsBorder.col", fromPrototype = TRUE), n)[seq_len(n)]
        border.fill <- rep(.dpOrDefault(GdObject, "detailsBorder.fill", fromPrototype = TRUE), n)[seq_len(n)]
        minwidth <- .dpOrDefault(GdObject, "details.minWidth", fromPrototype = TRUE)
        size <- .dpOrDefault(GdObject, "details.size", fromPrototype = TRUE)
        xyratio <- .dpOrDefault(GdObject, "details.ratio", fromPrototype = TRUE)
        if (0 >= size || size > 1) {
            warning("details.size must be >0 and <1 - reset to 0.5")
            size <- 0.5
        }
        selection <- .dpOrDefault(GdObject, ".__select", rep(TRUE, length(GdObject)))
        len <- sum(selection)
        bins <- if (!groupDetails) stacks(GdObject) else unlist(lapply(split(stacks(GdObject), group(GdObject)), unique))
        stacks <- max(bins)
        if (len > 0) {
            if (((maxBase - minBase) / len) / .pxResolution(coord = "x") < minwidth) {
                warning("too much detail for available space (plot fewer annotation or increase details.minWidth)!")
                popViewport(1)
                GdObject <- callNextMethod()
                return(GdObject)
            }
            rr <- if (groupDetails) unlist(range(split(ranges(GdObject), group(GdObject)))) else ranges(GdObject)
            xloc1 <- (end(rr) - start(rr)) / 2 + start(rr)
            yloc1 <- (stacks - (bins - 0.5) + 1)
            xloc2 <- ((1 / len * seq_len(len)) - 1 / len + (1 / len * 0.5))
            yloc2 <- rep(1, len)
            ## draw details plots (via user supplied function 'fun')
            pushViewport(viewport(height = size, y = 1 - size, just = c(0.5, 0)))
            w <- 1
            v <- 0
            vpl <- vpLocation()
            r <- vpl$size["width"] / len / vpl$size["height"]
            if (r > xyratio) {
                w <- xyratio / r
                v <- ((1 / len) - (1 / len * w)) / 2
            }
            indices <- if (groupDetails) seq_len(length(unique(group(GdObject)))) else seq_len(length(GdObject))
            j <- 1
            pres <- list()
            hasError <- FALSE
            for (i in indices[selection]) {
                pushViewport(viewport(width = 1 / len * w, x = ((1 / len * j) - 1 / len) + (v), just = c(0, 0.5)))
                grid.rect(gp = gpar(col = border.col[i], lwd = border.lwd[i], lty = border.lty[i], fill = border.fill[i]))
                iargs <- as.list(adf[i, ])
                iargs$index <- i
                iargs$GdObject <- GdObject
                iargs$GdObject.original <- .dpOrDefault(GdObject, ".__OriginalGdObject", GdObject)
                args <- c(args[setdiff(names(args), names(iargs))], iargs)
                pres[[as.character(j)]] <- try(do.call(GdObject@fun, args), silent = TRUE)
                if (!is.null(pres) && is(pres[[as.character(j)]], "try-error")) {
                    hasError <- TRUE
                    grid.segments(x0 = c(0.1, 0.1), x1 = c(0.9, 0.9), y0 = c(0.9, 0.1), y1 = c(0.1, 0.9), gp = gpar(col = "red", lwd = 3))
                }
                popViewport(1)
                j <- j + 1
            }
            if (hasError) {
                warning("There have been errors in the detail plotting function:\n", paste(pres, collapse = "\n"))
            }
            popViewport(1)
            ## plot AnnotationTrack and connectors to details
            xscale <- if (!.dpOrDefault(GdObject, "reverseStrand", FALSE)) c(minBase, maxBase) else c(maxBase, minBase)
            pushViewport(viewport(
                xscale = xscale,
                yscale = c(1, stacks + 1), clip = FALSE,
                height = 1 - size, y = 0, just = c(.5, 0)
            ))
            GdObject <- callNextMethod()
            grid.segments(
                x0 = unit(xloc1[selection], "native"), x1 = xloc2, y0 = unit(yloc1[selection], "native"),
                y1 = yloc2, gp = gpar(col = col, lwd = lwd, lty = lty, cex = cex)
            )
            grid.points(x = unit(xloc2, "npc"), y = unit(yloc2, "npc"), gp = gpar(col = col, cex = cex), pch = pch)
            grid.points(x = unit(xloc1[selection], "native"), y = unit(yloc1[selection], "native"), gp = gpar(col = col, cex = cex), pch = pch)
            popViewport(1)
        } else {
            GdObject <- callNextMethod()
        }
        return(invisible(GdObject))
    }
)

## SetAs ---------------------------------------------------------------------

#' @importFrom rtracklayer GenomicData
setAs(
    "AnnotationTrack", "UCSCData",
    function(from, to) {
        ranges <- range(from)
        dcolor <- as.integer(col2rgb(.dpOrDefault(from, "col")))
        line <- new("BasicTrackLine",
            name = names(from),
            description = names(from),
            visibility = stacking(from), color = dcolor, itemRgb = TRUE
        )
        vals <- values(from)
        color <- .getBiotypeColor(from)
        strand <- as.character(strand(from))
        strand[strand == "*"] <- "+"
        new("UCSCData", rtracklayer::GenomicData(ranges,
            chrom = chromosome(from),
            id = gsub(" ", "_", vals$id),
            name = gsub(" ", "_", as.character(vals$id)), itemRgb = color,
            strand = strand
        ),
        trackLine = line
        )
    }
)

setAs("GRanges", "AnnotationTrack", function(from, to) AnnotationTrack(range = from))

setAs("GRangesList", "AnnotationTrack", function(from, to) AnnotationTrack(range = from))

## Show ----------------------------------------------------------------------

## A helper function to plot general information about an AnnotationTrack
.annotationTrackInfo <- function(object) {
    msg <- sprintf(
        paste("| genome: %s\n| active chromosome: %s\n",
            "| annotation features: %s",
            sep = ""
        ),
        genome(object),
        chromosome(object),
        length(object)
    )
    addfeat <- length(object@range) - length(object)
    if (addfeat > 0) {
        msg <- c(msg, .addFeatInfo(object, addfeat), "Call chromosome(obj) <- 'chrId' to change the active chromosome")
    }
    return(paste(msg, collapse = "\n"))
}

## We have to show the name, genome and currently active chromosome, and, if more ranges are available on additional
## chromosomes some information about that

#' @describeIn AnnotationTrack-class Show method.
#' @export
setMethod("show", signature(object = "AnnotationTrack"), function(object) {
    cat(sprintf("AnnotationTrack '%s'\n%s\n", names(object), .annotationTrackInfo(object)))
})

#' @describeIn AnnotationTrack-class Show method.
#' @export
setMethod("show", signature(object = "ReferenceAnnotationTrack"), function(object) {
    .referenceTrackInfo(object, "ReferenceAnnotationTrack")
})

#' @include DataTrack-class.R
#' @include ReferenceTrack-class.R
#' @include StackedTrack-class.R
#' @include SequenceTrack-class.R
NULL


setClassUnion("SequenceTrackOrNULL", c("SequenceTrack", "NULL"))

## AlignmentsTrack Class -----------------------------------------------------

#' AlignmentsTrack class and methods
#'
#'
#' A class to represent short sequences that have been aligned to a reference
#' genome as they are typically generated in next generation sequencing
#' experiments.
#'
#' @name AlignmentsTrack-class
#'
#' @param range An optional meta argument to handle the different input types.
#' If the \code{range} argument is missing, all the relevant information to
#' create the object has to be provided as individual function arguments
#' (see below).
#'
#' The different input options for \code{range} are:
#'
#' \describe{
#'
#' \item{A \code{character} string:}{ the path to a \code{BAM} file containing
#' the read alignments. To be precise, this will result in the instantiation of
#' a \code{ReferenceAlignmentsTrack} object, but for the user this
#' implementation detail should be of no concern.}
#'
#' \item{A \code{GRanges} object:}{ the genomic ranges of the individual reads
#' as well as the optional additional metadata columns \code{id}, \code{cigar},
#' \code{mapq}, \code{flag}, \code{isize}, \code{groupid}, \code{status},
#' \code{md} and \code{seqs} (see description of the individual function
#' parameters below for details). Calling the constructor on a \code{GRanges}
#' object without further arguments, e.g. \code{AlignmentsTrack(range=obj)} is
#' equivalent to calling the coerce method \code{as(obj, "AlignmentsTrack")}.}
#'
#' \item{An \code{\linkS4class{IRanges}} object:}{ almost identical to the
#' \code{GRanges} case, except that the chromosome and strand information as
#' well as all additional metadata has to be provided in the separate
#' \code{chromosome}, \code{strand}, \code{feature}, \code{group} or \code{id}
#' arguments, because it can not be directly encoded in an \code{IRanges}
#' object. Note that none of those inputs are mandatory, and if not provided
#' explicitely the more or less reasonable default values \code{chromosome=NA}
#' and \code{strand="*"} are used. }
#'
#' \item{A \code{data.frame} object:}{ the \code{data.frame} needs to contain
#' at least the two mandatory columns \code{start} and \code{end} with the
#' range coordinates. It may also contain a \code{chromosome} and a
#' \code{strand} column with the chromosome and strand information for each
#' range. If missing it will be drawn from the separate \code{chromosome} or
#' \code{strand} arguments. In addition, the \code{id}, \code{cigar},
#' \code{mapq}, \code{flag}, \code{isize}, \code{groupid}, \code{status},
#' \code{md} and \code{seqs} data can be provided as additional columns. The
#' above comments about potential default values also apply here.}
#'
#' }
#'
#' @template AlignmentsTrack-class_param
#'
#' @return
#'
#' The return value of the constructor function is a new object of class
#' \code{AlignmentsTrack} or \code{ReferenceAlignmentsTrack}.
#' @section Objects from the Class:
#'
#' Objects can be created using the constructor function
#' \code{AlignmentsTrack}.
#'
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @examples
#' ## Creating objects
#' afrom <- 2960000
#' ato <- 3160000
#' alTrack <- AlignmentsTrack(system.file(
#'     package = "Gviz", "extdata",
#'     "gapped.bam"
#' ), isPaired = TRUE)
#' plotTracks(alTrack, from = afrom, to = ato, chromosome = "chr12")
#'
#' ## Omit the coverage or the pile-ups part
#' plotTracks(alTrack,
#'     from = afrom, to = ato, chromosome = "chr12",
#'     type = "coverage"
#' )
#' plotTracks(alTrack,
#'     from = afrom, to = ato, chromosome = "chr12",
#'     type = "pileup"
#' )
#'
#' ## Including sequence information with the constructor
#' if (require(BSgenome.Hsapiens.UCSC.hg19)) {
#'     strack <- SequenceTrack(Hsapiens, chromosome = "chr21")
#'     afrom <- 44945200
#'     ato <- 44947200
#'     alTrack <- AlignmentsTrack(system.file(
#'         package = "Gviz", "extdata",
#'         "snps.bam"
#'     ), isPaired = TRUE, referenceSequence = strack)
#'     plotTracks(alTrack, chromosome = "chr21", from = afrom, to = ato)
#'
#'     ## Including sequence information in the track list
#'     alTrack <- AlignmentsTrack(system.file(
#'         package = "Gviz", "extdata",
#'         "snps.bam"
#'     ), isPaired = TRUE)
#'     plotTracks(c(alTrack, strack),
#'         chromosome = "chr21", from = 44946590,
#'         to = 44946660
#'     )
#' }
#' @importFrom Rsamtools scanBamFlag scanBamHeader scanBam ScanBamParam scanFaIndex scanFa BamFile scanBamWhat bamWhich
#'
#' @exportClass AlignmentsTrack
setClass("AlignmentsTrack",
    representation = representation(
        stackRanges = "GRanges",
        sequences = "DNAStringSet",
        referenceSequence = "SequenceTrackOrNULL"
    ),
    contains = "StackedTrack",
    prototype = prototype(
        stacking = "squish",
        name = "AlignmentsTrack",
        coverageOnly = FALSE,
        stackRanges = GRanges(),
        sequences = Biostrings::DNAStringSet(),
        referenceSequence = NULL,
        dp = DisplayPars(
            alpha.reads = 0.5,
            alpha.mismatch = 1,
            cex = 0.7,
            cex.mismatch = NULL,
            col.coverage = NULL,
            col.gap = .DEFAULT_SHADED_COL,
            col.mates = .DEFAULT_BRIGHT_SHADED_COL,
            col.deletion = "#000000",
            col.insertion = "#984EA3",
            col.mismatch = .DEFAULT_SHADED_COL,
            col.reads = NULL,
            col.sashimi = NULL,
            col = .DEFAULT_SHADED_COL,
            collapse = FALSE,
            coverageHeight = 0.1,
            fill.coverage = NULL,
            fill.reads = NULL,
            fill = "#BABABA",
            fontface.mismatch = 2,
            lty.coverage = NULL,
            lty.gap = NULL,
            lty.mates = NULL,
            lty.deletion = NULL,
            lty.insertion = NULL,
            lty.mismatch = NULL,
            lty.reads = NULL,
            lty = 1,
            lwd.coverage = NULL,
            lwd.gap = NULL,
            lwd.mates = NULL,
            lwd.deletion = NULL,
            lwd.insertion = NULL,
            lwd.mismatch = NULL,
            lwd.reads = NULL,
            lwd.sashimiMax = 10,
            lwd = 1,
            max.height = 10,
            min.height = 5,
            minCoverageHeight = 50,
            minSashimiHeight = 50,
            noLetters = FALSE,
            sashimiFilter = NULL,
            sashimiFilterTolerance = 0L,
            sashimiHeight = 0.1,
            sashimiScore = 1,
            sashimiStrand = "*",
            sashimiTransformation = NULL,
            showIndels = FALSE,
            showMismatches = TRUE,
            size = NULL,
            transformation = NULL,
            type = c("coverage", "pileup")
        )
    )
)

## Initialize ----------------------------------------------------------------

#' @describeIn AlignmentsTrack-class Initialize.
#' @export
setMethod("initialize", "AlignmentsTrack", function(.Object, stackRanges = GRanges(), stacks = numeric(), sequences = DNAStringSet(),
                                                    referenceSequence = NULL, ...) {
    ## the diplay parameter defaults
    .makeParMapping()
    .Object <- .updatePars(.Object, "AlignmentsTrack")
    .Object@stackRanges <- stackRanges
    .Object <- callNextMethod(.Object, ...)
    .Object@stacks <- stacks
    .Object@sequences <- sequences
    .Object@referenceSequence <- referenceSequence
    return(.Object)
})

## ReferenceAlignmentsTrack Class --------------------------------------------

#' @describeIn AlignmentsTrack-class The file-based version of the `AlignmentsTrack-class`.
#' @exportClass ReferenceAlignmentsTrack
setClass("ReferenceAlignmentsTrack", contains = c("AlignmentsTrack", "ReferenceTrack"))

## Initialize ----------------------------------------------------------------

## This just needs to set the appropriate slots that are being inherited
## from ReferenceTrack because the multiple inheritance has some strange
## features with regards to method selection

#' @importClassesFrom Biostrings DNAStringSet
#' @importFrom Biostrings DNAStringSet
#' @describeIn AlignmentsTrack-class Initialize.
#' @export
setMethod("initialize", "ReferenceAlignmentsTrack", function(.Object, stream, reference, mapping = list(),
                                                             args = list(), defaults = list(), stacks = numeric(),
                                                             stackRanges = GRanges(), sequences = Biostrings::DNAStringSet(),
                                                             referenceSequence = NULL, ...) {
    .Object <- selectMethod("initialize", "ReferenceTrack")(.Object = .Object, reference = reference, stream = stream,
        mapping = mapping, args = args, defaults = defaults)
    .Object <- callNextMethod(.Object, ...)
    .Object@referenceSequence <- referenceSequence
    return(.Object)
})

## Constructor ---------------------------------------------------------------

## Constructor
#' @describeIn AlignmentsTrack-class Constructor for `AlignmentsTrack-class`.
#' @export
AlignmentsTrack <- function(range = NULL, start = NULL, end = NULL, width = NULL, strand, chromosome, genome,
                            stacking = "squish", id, cigar, mapq, flag = scanBamFlag(isUnmappedQuery = FALSE), isize, groupid, status, md, seqs,
                            name = "AlignmentsTrack", isPaired = TRUE, importFunction, referenceSequence, ...) {
    ## Some defaults
    if (missing(importFunction)) {
        importFunction <- .import.bam.alignments
    }
    covars <- .getCovars(range)
    isStream <- FALSE
    if (!is.character(range)) {
        n <- max(c(length(start), length(end), length(width)), nrow(covars))
        id <- .covDefault(id, covars[["id"]], paste("read", seq_len(n), sep = "_"))
        cigar <- .covDefault(cigar, covars[["cigar"]], paste(if (is(range, "GRangesOrIRanges")) width(range) else width, "M", sep = ""))
        mapq <- .covDefault(mapq, covars[["mapq"]], rep(as.integer(NA), n))
        flag <- .covDefault(flag, covars[["flag"]], rep(as.integer(NA), n))
        isize <- .covDefault(isize, covars[["isize"]], rep(as.integer(NA), n))
        groupid <- .covDefault(groupid, covars[["groupid"]], seq_len(n))
        md <- .covDefault(md, covars[["md"]], rep(as.character(NA), n))
        status <- .covDefault(status, covars[["status"]], ifelse(groupid %in% groupid[duplicated(groupid)], "mated", "unmated"))
    }
    ## Build a GRanges object from the inputs
    .missingToNull(c(
        "strand", "chromosome", "importFunction", "genome", "id", "cigar", "mapq", "flag", "isize", "groupid", "status",
        "md", "seqs", "referenceSequence"
    ))
    args <- list(
        id = id, cigar = cigar, mapq = mapq, flag = flag, isize = isize, groupid = groupid, status = status, strand = strand, md = md,
        chromosome = chromosome, genome = genome
    )
    defs <- list(
        strand = "*", chromosome = "chrNA", genome = NA, id = as.character(NA), cigar = as.character(NA), mapq = as.integer(NA),
        flag = as.integer(NA), isize = as.integer(NA), groupid = as.character(NA), status = as.character(NA), md = as.character(NA)
    )
    range <- .buildRange(
        range = range, start = start, end = end, width = width,
        args = args, defaults = defs, chromosome = chromosome, trackType = "AlignmentsTrack",
        importFun = importFunction, stream = TRUE, autodetect = TRUE
    )
    ## This is going to be a list if we have to stream data from a file, otherwise we can compute some additional values
    if (is.list(range)) {
        isStream <- TRUE
        slist <- range
        range <- GRanges()
        stackRanges <- GRanges()
        stacks <- NULL
        seqs <- DNAStringSet()
    } else {
        if (is.null(seqs)) {
            seqs <- DNAStringSet(vapply(width(range), function(x) paste(rep("N", x), collapse = ""), character(1)))
        }
        addArgs <- list(...)
        if ("showIndels" %in% names(addArgs)) {
            showIndels <- addArgs$showIndels
        } else {
            showIndels <- FALSE
        }
        tmp <- .computeAlignments(range, drop.D.ranges = showIndels)
        range <- tmp$range
        stackRanges <- tmp$stackRange
        stacks <- tmp$stacks
    }
    ## If no chromosome was explicitly asked for we just take the first one in the GRanges object
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- if (length(range) > 0) .chrName(as.character(seqnames(range)[1])) else "chrNA"
    }
    ## And finally the object instantiation
    genome <- .getGenomeFromGRange(range, ifelse(is.null(genome), character(), genome[1]))
    if (!isStream) {
        return(new("AlignmentsTrack",
            chromosome = chromosome[1], range = range, stacks = stacks,
            name = name, genome = genome, stacking = stacking, stackRanges = stackRanges, sequences = seqs,
            referenceSequence = referenceSequence, ...
        ))
    } else {
        ## A bit hackish but for some functions we may want to know which track type we need but at the
        ## same time we do not want to enforce this as an additional argument
        e <- new.env()
        e[["._trackType"]] <- "AlignmentsTrack"
        e[["._isPaired"]] <- isPaired
        e[["._flag"]] <- flag
        environment(slist[["stream"]]) <- e
        return(new("ReferenceAlignmentsTrack",
            chromosome = chromosome[1], range = range, stackRanges = stackRanges,
            name = name, genome = genome, stacking = stacking, stream = slist[["stream"]], reference = slist[["reference"]],
            mapping = slist[["mapping"]], args = args, defaults = defs, stacks = stacks, referenceSequence = referenceSequence, ...
        ))
    }
}

## General accessors ---------------------------------------------------------

#' @describeIn AlignmentsTrack-class Return all additional annotation
#' information except for the genomic coordinates for the track items as
#' a `data.frame`.
#' @export
setMethod("values", "AlignmentsTrack", function(x) .dpOrDefault(x, ".__coverage"))

#' @describeIn AlignmentsTrack-class replace the value of the track's chromosome.
#' This has to be a valid UCSC chromosome identifier or an integer or character
#' scalar that can be reasonably coerced into one.
#' @export
setReplaceMethod("chromosome", "AlignmentsTrack", function(GdObject, value) {
    GdObject <- callNextMethod()
    if (!is.null(GdObject@referenceSequence)) {
        chromosome(GdObject@referenceSequence) <- value[1]
    }
    return(GdObject)
})



## Annotation Accessors ------------------------------------------------------
## Stacking ------------------------------------------------------------------

#' @describeIn AlignmentsTrack-class return the stack indices for each track item.
#' @export
setMethod(
    "stacks", "AlignmentsTrack",
    function(GdObject) if (length(GdObject)) ranges(GdObject)$stack else 0
)

#' @describeIn AlignmentsTrack-class recompute the stacks based on the available
#' space and on the object's track items and stacking settings.
#' @export
setMethod("setStacks", "AlignmentsTrack", function(GdObject, ...) {
    if (length(GdObject)) {
        bins <- if (!.needsStacking(GdObject)) rep(1, length(GdObject)) else disjointBins(GdObject@stackRanges)
        GdObject@stacks <- bins
        ranges(GdObject)$stack <- bins[match(ranges(GdObject)$groupid, names(GdObject@stackRanges))]
    }
    return(GdObject)
})


## Consolidate ---------------------------------------------------------------
## Collapse  -----------------------------------------------------------------
## Subset --------------------------------------------------------------------
## AlignmentTracks can be subset by using the information in the stackRanges slot, but for the actual reads we need to make sure that
## we keep all the bits that belong to a given group. We still want to record the requested ranges in the internal '.__plottingRange'
## display parameter.

#' @describeIn AlignmentsTrack-class Subset a `AlignmentsTrack` by coordinates
#' and sort if necessary.
#' @export
setMethod("subset", signature(x = "AlignmentsTrack"), function(x, from = NULL, to = NULL, stacks = FALSE, use.defaults = TRUE, ...) {
    ## Subset to a single chromosome first
    lx <- length(x)
    csel <- seqnames(x) != chromosome(x)
    if (any(csel)) {
        x <- x[!csel]
    }
    if (length(x)) {
        ## Nothing to do if everything is within the range, otherwise we subset the stackRanges and keep all group items
        ranges <- if (use.defaults) {
            .defaultRange(x, from = from, to = to)
        } else {
            c(
                from = ifelse(is.null(from), min(start(x@stackRanges)) - 1, from),
                to = ifelse(is.null(to), max(end(x@stackRanges)) + 1, to)
            )
        }
        displayPars(x) <- list(".__plottingRange" = ranges)
        sr <- subsetByOverlaps(x@stackRanges, GRanges(seqnames = chromosome(x)[1], ranges = IRanges(start = ranges["from"], end = ranges["to"])))
        if (length(sr) < length(x@stackRanges)) {
            x@stackRanges <- sr
            x@range <- x@range[x@range$groupid %in% names(sr)]
            if (stacks) {
                x <- setStacks(x)
            }
        }
    }
    return(x)
})

## ReferenceAlignmentsTracks need to stream the data from file and then pass the results on to the next method

#' @describeIn AlignmentsTrack-class Subset a `ReferenceAlignmentsTrack` by coordinates
#' and sort if necessary.
#' @export
setMethod("subset", signature(x = "ReferenceAlignmentsTrack"), function(x, from, to, chromosome, ...) {
    ## We only need to reach out into the referenced file once if the range is already contained in the object
    if (missing(from) || is.null(from) || missing(to) || is.null(to)) {
        stop("Need both start and end location to subset a ReferenceAnnotationTrack")
    }
    if (missing(chromosome) || is.null(chromosome)) {
        chromosome <- Gviz::chromosome(x)
    } else {
        chromosome <- .chrName(chromosome)
    }
    subRegion <- GRanges(seqnames = chromosome[1], ranges = IRanges(start = from, end = to))
    oldRange <- .dpOrDefault(x, ".__plottingRange")
    isIn <- length(ranges(x)) != 0 && !is.null(oldRange) && ranges(subRegion) %within% IRanges(oldRange["from"], oldRange["to"])
    if (!isIn) {
        cMap <- .resolveColMapping(x@stream(x@reference, subRegion), x@args, x@mapping)
        seqs <- cMap$data$seq
        cMap$data$seq <- NULL
        range <- .computeAlignments(.buildRange(cMap$data, args = cMap$args, defaults = x@defaults, trackType = "AnnotationTrack"), drop.D.ranges = .dpOrDefault(x, "showIndels", FALSE))
        ranges(x) <- range$range
        x@stackRanges <- range$stackRanges
        x@stacks <- range$stacks
        x@sequences <- seqs
    } else {
        x@sequences <- subseq(x@sequences, start = from - (oldRange["from"] - 1), width = to - from + 1)
    }
    chromosome(x) <- chromosome[1]
    return(callNextMethod(x = x, from = from, to = to, drop = FALSE, ...))
})


## Position ------------------------------------------------------------------
## DrawGrid ------------------------------------------------------------------

setMethod("drawGrid", signature(GdObject = "AlignmentsTrack"), function(GdObject, from, to) {
    if (.dpOrDefault(GdObject, "grid", FALSE)) {
        yvals <- values(GdObject)
        ylim <- .dpOrDefault(GdObject, "ylim", if (!is.null(yvals) && length(yvals)) {
            range(yvals, na.rm = TRUE, finite = TRUE)
        } else {
            c(-1, 1)
        })
        if (diff(ylim) == 0) {
            ylim <- ylim + c(-1, 1)
        }
        yscale <- c(if (is.null(.dpOrDefault(GdObject, "transformation"))) 0 else min(ylim), max(ylim) + diff(range(ylim)) * 0.05)
        covHeight <- .dpOrDefault(GdObject, ".__coverageHeight", 0)
        pushViewport(viewport(y = 1 - covHeight["npc"], height = covHeight["npc"], just = c(0.5, 0), yscale = yscale, clip = TRUE))
        panel.grid(
            v = 0, h = .dpOrDefault(GdObject, "h", -1),
            col = .dpOrDefault(GdObject, "col.grid", "#e6e6e6"), lty = .dpOrDefault(GdObject, "lty.grid", 1),
            lwd = .dpOrDefault(GdObject, "lwd.grid", 1)
        )
        popViewport(1)
    }
})

## DrawAxis ------------------------------------------------------------------

#' @describeIn AlignmentsTrack-class add a y-axis to the title panel of a track.
#' @export
setMethod("drawAxis", signature(GdObject = "AlignmentsTrack"), function(GdObject, ...) {
    type <- match.arg(.dpOrDefault(GdObject, "type", .ALIGNMENT_TYPES), .ALIGNMENT_TYPES, several.ok = TRUE)
    if ("coverage" %in% type) {
        yvals <- values(GdObject)
        ylim <- .dpOrDefault(GdObject, "ylim", if (!is.null(yvals) && length(yvals)) {
            range(yvals, na.rm = TRUE, finite = TRUE)
        } else {
            c(-1, 1)
        })
        if (diff(ylim) == 0) {
            ylim <- ylim + c(-1, 1)
        }
        hSpaceAvail <- vpLocation()$isize["width"] / 6
        yscale <- c(if (is.null(.dpOrDefault(GdObject, "transformation"))) 0 else min(ylim), max(ylim) + diff(range(ylim)) * 0.05)
        col <- .dpOrDefault(GdObject, "col.axis", "white")
        acex <- .dpOrDefault(GdObject, "cex.axis")
        acol <- .dpOrDefault(GdObject, "col.axis", "white")
        at <- pretty(yscale)
        at <- at[at >= sort(ylim)[1] & at <= sort(ylim)[2]]
        covHeight <- .dpOrDefault(GdObject, ".__coverageHeight", c(npc = 0, points = 0))
        covSpace <- .dpOrDefault(GdObject, ".__coverageSpace", 0)
        pushViewport(viewport(y = 1 - (covHeight["npc"] + covSpace), height = covHeight["npc"], just = c(0.5, 0)))
        if (is.null(acex)) {
            vSpaceNeeded <- max(as.numeric(convertWidth(stringHeight(at), "inches"))) * length(at) * 1.5
            hSpaceNeeded <- max(as.numeric(convertWidth(stringWidth(at), "inches")))
            vSpaceAvail <- abs(diff(range(at))) / abs(diff(yscale)) * vpLocation()$isize["height"]
            acex <- max(0.6, min(vSpaceAvail / vSpaceNeeded, hSpaceAvail / hSpaceNeeded))
        }
        vpTitleAxis <- viewport(x = 0.95, width = 0.2, yscale = yscale, just = 0)
        pushViewport(vpTitleAxis)
        suppressWarnings(grid.yaxis(gp = gpar(col = acol, cex = acex), at = at))
        grid.lines(x = c(0, 0), y = ylim, gp = gpar(col = acol), default.units = "native")
        popViewport(2)
    } else {
        covHeight <- c(npc = 0, points = 0)
        covSpace <- 0
    }
    if ("sashimi" %in% type) {
        sash <- .dpOrDefault(GdObject, ".__sashimi", list(x = numeric(), y = numeric(), id = integer(), score = numeric()))
        yscale <- if (length(sash$y)) c(-(max(sash$y) + diff(range(sash$y)) * 0.05), 0) else c(-1, 0)
        ylim <- if (length(sash$y)) c(-max(sash$y), yscale[1] + max(sash$y)) else c(-1, 0)
        hSpaceAvail <- vpLocation()$isize["width"] / 6
        col <- .dpOrDefault(GdObject, "col.axis", "white")
        acex <- .dpOrDefault(GdObject, "cex.axis")
        acol <- .dpOrDefault(GdObject, "col.axis", "white")
        labs <- if (length(sash$score)) pretty(c(1, sash$score)) else pretty(c(1, .dpOrDefault(GdObject, ".__sashimiScore", 10)))
        at <- seq(ylim[1], ylim[2], length.out = length(labs))
        sashHeight <- .dpOrDefault(GdObject, ".__sashimiHeight", c(npc = 0, points = 0))
        sashSpace <- .dpOrDefault(GdObject, ".__sashimiSpace", 0)
        pushViewport(viewport(
            y = 1 - (sashHeight["npc"] + sashSpace + covHeight["npc"] + covSpace),
            height = sashHeight["npc"], just = c(0.5, 0)
        ))
        if (is.null(acex)) {
            vSpaceNeeded <- max(as.numeric(convertWidth(stringHeight(labs), "inches"))) * length(at) * 1.5
            hSpaceNeeded <- max(as.numeric(convertWidth(stringWidth(labs), "inches")))
            vSpaceAvail <- abs(diff(range(at))) / abs(diff(yscale)) * vpLocation()$isize["height"]
            acex <- max(0.6, min(vSpaceAvail / vSpaceNeeded, hSpaceAvail / hSpaceNeeded))
        }
        vpTitleAxis <- viewport(x = 0.75, width = 0.2, yscale = yscale, just = 0)
        pushViewport(vpTitleAxis)
        suppressWarnings(grid.yaxis(gp = gpar(col = acol, cex = acex), at = at, label = labs))
        grid.polygon(x = c(0, 0, 1), y = c(ylim[1], ylim[2], ylim[2]), default.units = "native", gp = gpar(col = acol, fill = acol))
        popViewport(2)
    }
    if (.dpOrDefault(GdObject, ".__isCropped", FALSE)) {
        vspacing <- as.numeric(convertHeight(unit(2, "points"), "npc"))
        vsize <- as.numeric(convertHeight(unit(4, "points"), "npc"))
        pushViewport(viewport(height = vsize, y = vspacing, just = c(0.5, 0)))
        .moreInd(direction = "down", lwd = 2)
        popViewport(1)
    }
})

## DrawGD --------------------------------------------------------------------

#' @describeIn AlignmentsTrack-class plot the object to a graphics device.
#' The return value of this method is the input object, potentially updated
#' during the plotting operation. Internally, there are two modes in which the
#' method can be called. Either in 'prepare' mode, in which case no plotting is
#' done but the object is preprocessed based on the available space, or in
#' 'plotting' mode, in which case the actual graphical output is created.
#' Since subsetting of the object can be potentially costly, this can be
#' switched off in case subsetting has already been performed before or
#' is not necessary.
#'
#' @importFrom GenomicAlignments cigarRangesAlongReferenceSpace
#' @export
setMethod("drawGD", signature("AlignmentsTrack"), function(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...) {
    debug <- .dpOrDefault(GdObject, "debug", FALSE)
    imageMap(GdObject) <- NULL
    if (!length(GdObject)) {
        return(invisible(GdObject))
    }
    rev <- .dpOrDefault(GdObject, "reverseStrand", FALSE)
    revs <- !.dpOrDefault(GdObject, "reverseStacking", FALSE)
    vSpacing <- as.numeric(convertHeight(unit(3, "points"), "npc"))
    type <- match.arg(.dpOrDefault(GdObject, "type", .ALIGNMENT_TYPES), .ALIGNMENT_TYPES, several.ok = TRUE)
    ylim <- .dpOrDefault(GdObject, "ylim")
    ## In prepare mode we need to make sure that the stacking information is updated from the optional display parameter (by calling
    ## the StackedTrack drawGD method), and compute the coverage vector which will be needed for the axis
    if (prepare) {
        if ((is.logical(debug) && debug) || "prepare" %in% debug) {
            browser()
        }
        GdObject <- callNextMethod(GdObject, ...)
        ## The mismatched bases need to be extracted from the read sequences and the reference sequence
        if (.dpOrDefault(GdObject, "showMismatches", TRUE) && !is.null(GdObject@referenceSequence)) {
            mm <- .findMismatches(GdObject)
            if (nrow(mm)) {
                displayPars(GdObject) <- list(".__mismatches" = mm)
            }
        }
        ## The coverage calculation and the height of the coverage section
        if ("coverage" %in% type) {
            covSpace <- as.numeric(convertHeight(unit(5, "points"), "npc"))
            if ("pileup" %in% type) {
                covHeight <- .dpOrDefault(GdObject, "coverageHeight", 0.1)
                if (covHeight > 0 && covHeight < 1) {
                    covHeight <- as.numeric(convertHeight(unit(covHeight, "npc"), "points"))
                }
                covHeight <- max(.dpOrDefault(GdObject, "minCoverageHeight", 50), covHeight)
                covHeight <- c(points = covHeight, npc = as.numeric(convertHeight(unit(covHeight, "points"), "npc")))
            } else if ("sashimi" %in% type) {
                covHeight <- c(npc = 0.5 - (covSpace * 2), points = 1)
            } else {
                covHeight <- c(npc = 1 - (covSpace * 2), points = 1)
            }
            coverage <- coverage(range(GdObject), width = maxBase)
            ## apply data transformation if one is set up
            trans <- .dpOrDefault(GdObject, "transformation")
            if (is.list(trans)) {
                trans <- trans[[1]]
            }
            if (!is.null(trans)) {
                if (!is.function(trans) || length(formals(trans)) != 1L) {
                    stop("Display parameter 'transformation' must be a function with a single argument")
                }
                test <- trans(coverage)
                if (!is(test, "Rle") || length(test) != length(coverage)) {
                    stop(
                        "The function in display parameter 'transformation' results in invalid output.\n",
                        "It has to return a numeric matrix with the same dimensions as the input data."
                    )
                }
                coverage <- test
            }
            displayPars(GdObject) <- list(
                ".__coverage" = coverage,
                ".__coverageHeight" = covHeight,
                ".__coverageSpace" = covSpace
            )
        } else {
            covHeight <- c(npc = 0, points = 0)
            covSpace <- 0
        }
        ## The sashimi calculation and the height of the sashimi section
        if ("sashimi" %in% type) {
            sashSpace <- as.numeric(convertHeight(unit(5, "points"), "npc"))
            if ("pileup" %in% type) {
                sashHeight <- .dpOrDefault(GdObject, "sashimiHeight", 0.1)
                if (sashHeight > 0 && sashHeight < 1) {
                    sashHeight <- as.numeric(convertHeight(unit(sashHeight, "npc"), "points"))
                }
                sashHeight <- max(.dpOrDefault(GdObject, "minSashimiHeight", 50), sashHeight)
                sashHeight <- c(points = sashHeight, npc = as.numeric(convertHeight(unit(sashHeight, "points"), "npc")))
            } else if ("coverage" %in% type) {
                sashHeight <- c(npc = 0.5 - (sashSpace * 2), points = 1)
            } else {
                sashHeight <- c(npc = 1 - (sashSpace * 2), points = 1)
            }
            sashScore <- .dpOrDefault(GdObject, "sashimiScore", 1L)
            sashLwdMax <- .dpOrDefault(GdObject, "lwd.sashimiMax", 10)
            sashStrand <- .dpOrDefault(GdObject, "sashimiStrand", "*")
            sashFilter <- .dpOrDefault(GdObject, "sashimiFilter", NULL)
            sashFilterTolerance <- .dpOrDefault(GdObject, "sashimiFilterTolerance", 0L)
            sashNumbers <- .dpOrDefault(GdObject, "sashimiNumbers", FALSE)
            sash <- .dpOrDefault(GdObject, "sashimiJunctions", NULL)
            if (is.null(sash)) {
                sash <- .create.summarizedJunctions.for.sashimi.junctions(ranges(GdObject))
            } else {
                if (!is(sash, "GRanges")) {
                    stop("\"sashimiJunctions\" object must be of \"GRanges\" class!")
                }
                sashMcolName <- if (sashStrand == "+") "plus_score" else if (sashStrand == "-") "minus_score" else "score"
                if (sum(colnames(mcols(sash)) == sashMcolName) != 1) {
                    stop(sprintf("\"mcols\" of \"sashimiJunctions\" object must contain column named \"%s\",\n which matches the specified (%s) \"sashimiStrand\"!", sashMcolName, sashStrand))
                }
            }
            sashTransform <- .dpOrDefault(GdObject, c("sashimiTransformation", "transformation"))
            sash <- .convert.summarizedJunctions.to.sashimi.junctions(
                juns = sash,
                score = sashScore,
                lwd.max = sashLwdMax,
                strand = sashStrand,
                filter = sashFilter,
                filterTolerance = sashFilterTolerance,
                trans = sashTransform
            )
            displayPars(GdObject) <- list(
                ".__sashimi" = sash,
                ".__sashimiHeight" = sashHeight,
                ".__sashimiSpace" = sashSpace,
                ".__sashimiNumbers" = sashNumbers
            )
        } else {
            sashHeight <- c(npc = 0, points = 0)
            sashSpace <- 0
        }
        if ("pileup" %in% type) {
            ## If there are more bins than we can plot we reduce the number until they fit
            pushViewport(viewport(height = 1 - (covHeight["npc"] + covSpace + sashHeight["npc"] + sashSpace) - vSpacing * 2, y = vSpacing, just = c(0.5, 0)))
            bins <- ranges(GdObject)$stack
            curVp <- vpLocation()
            mih <- min(curVp$size["height"], .dpOrDefault(GdObject, c("min.height", "max.height"), 3))
            mah <- min(curVp$size["height"], max(mih, .dpOrDefault(GdObject, c("max.height", "min.height"), 8)))
            bins <- stacks(GdObject)
            if (curVp$size["height"] / max(bins) < mih) {
                maxStack <- curVp$size["height"] %/% mih
                sel <- if (revs) bins <= maxStack else bins > max(bins) - maxStack
                ranges(GdObject) <- ranges(GdObject)[sel]
                displayPars(GdObject) <- list(".__isCropped" = TRUE)
                bins <- stacks(GdObject)
            }
            yrange <- range(bins) + c(-0.5, 0.5)
            add <- max(0, (curVp$size["height"] %/% mah) - max(bins))
            yrange <- if (revs) c(yrange[1], yrange[2] + add) else c(yrange[1] - add, yrange[2])
            displayPars(GdObject) <- list(".__yrange" = yrange)
            popViewport(1)
        }
        return(invisible(GdObject))
    }
    if ((is.logical(debug) && debug) || "draw" %in% debug) {
        browser()
    }
    mm <- .dpOrDefault(GdObject, ".__mismatches")
    ## The coverage plot first
    xscale <- if (!rev) c(minBase, maxBase) else c(maxBase, minBase)
    readInfo <- ranges(GdObject)
    covHeight <- .dpOrDefault(GdObject, ".__coverageHeight", c(npc = 0, points = 0))
    covSpace <- .dpOrDefault(GdObject, ".__coverageSpace", 0)
    if ("coverage" %in% type) {
        cov <- .dpOrDefault(GdObject, ".__coverage", Rle(lengths = maxBase, values = as.integer(0)))
        yscale <- c(if (is.null(.dpOrDefault(GdObject, "transformation"))) 0 else min(cov), max(cov) + diff(range(cov)) * 0.05)
        if (!is.null(ylim)) {
            yscale <- ylim
        }
        vp <- viewport(
            height = covHeight["npc"], y = 1 - (covHeight["npc"] + covSpace), just = c(0.5, 0), xscale = xscale,
            yscale = yscale, clip = TRUE
        )
        pushViewport(vp)
        res <- .pxResolution(coord = "x")
        gp <- gpar(
            col = .dpOrDefault(GdObject, c("col.coverage", "col"), .DEFAULT_SHADED_COL),
            fill = .dpOrDefault(GdObject, c("fill.coverage", "fill"), "#BABABA"),
            lwd = .dpOrDefault(GdObject, c("lwd.coverage", "lwd"), 1),
            lty = .dpOrDefault(GdObject, c("lty.coverage", "lty"), 1),
            alpha = .alpha(GdObject)
        )
        ## We can compact this if the resolution is not sufficient to speed up the drawing.
        mminBase <- max(1, minBase)
        if (res > 2) {
            brks <- ceiling((maxBase - mminBase) / res)
            x <- seq(mminBase, maxBase, len = brks)
            y <- tapply(as.integer(cov[mminBase:maxBase]), cut(mminBase:maxBase, breaks = brks), mean)
        } else {
            x <- mminBase:maxBase
            y <- as.integer(cov[x])
        }
        grid.polygon(c(minBase - max(1, res), 0, x, maxBase + max(1, res)), c(0, 0, y, 0), default.units = "native", gp = gp)
        grid.lines(y = c(0, 0), gp = gpar(col = gp$col, alpha = gp$alpha))
        if (!is.null(mm)) {
            fcol <- .dpOrDefault(GdObject@referenceSequence, "fontcolor", getBioColor("DNA_BASES_N"))
            vpos <- tapply(as.character(mm$base[mm$base != "."]), mm$position[mm$base != "."], table, simplify = FALSE)
            x <- rep(as.integer(names(vpos)), listLen(vpos))
            y <- unlist(lapply(vpos, cumsum), use.names = FALSE)
            col <- fcol[unlist(lapply(vpos, names), use.names = FALSE)]
            grid.rect(
                x = x, y = y, height = unlist(vpos, use.names = FALSE), width = 1, default.units = "native", just = c(0, 1),
                gp = gpar(col = "transparent", fill = col)
            )
        }
        popViewport(1)
        twoPx <- 2 * as.numeric(convertHeight(unit(1, "points"), "npc"))
        vp <- viewport(height = twoPx, y = 1 - (covHeight["npc"] + covSpace + twoPx), just = c(0.5, 0))
        pushViewport(vp)
        grid.rect(gp = gpar(
            fill = .dpOrDefault(GdObject, "background.title"), col = "transparent",
            alpha = .dpOrDefault(GdObject, "alpha.title")
        ), width = 2)
        popViewport(1)
    }
    ## The sashimi plot as second
    xscale <- if (!rev) c(minBase, maxBase) else c(maxBase, minBase)
    sashHeight <- .dpOrDefault(GdObject, ".__sashimiHeight", c(npc = 0, points = 0))
    sashSpace <- .dpOrDefault(GdObject, ".__sashimiSpace", 0)
    if ("sashimi" %in% type) {
        sash <- .dpOrDefault(GdObject, ".__sashimi", list(x = numeric(), y = numeric(), id = integer(), score = numeric(), scaled = numeric()))
        sashNumbers <- .dpOrDefault(GdObject, ".__sashimiNumbers", FALSE)
        yscale <- if (length(sash$y)) c(-(max(sash$y) + diff(range(sash$y)) * 0.15), 0) else c(-1, 0) # changed from 0.05 to 0.15 to make sure that numbers fit in the viewport
        vp <- viewport(
            height = sashHeight["npc"], y = 1 - (covHeight["npc"] + covSpace + sashHeight["npc"] + sashSpace), just = c(0.5, 0),
            xscale = xscale, yscale = yscale, clip = TRUE
        )
        pushViewport(vp)
        gp <- gpar(
            col = .dpOrDefault(GdObject, c("col.sashimi", "col"), .DEFAULT_SHADED_COL),
            fill = .dpOrDefault(GdObject, c("fill.sashimi", "fill"), "#FFFFFF"),
            lwd = .dpOrDefault(GdObject, c("lwd.sashimi", "lwd"), 1),
            lty = .dpOrDefault(GdObject, c("lty.sashimi", "lty"), 1),
            alpha = .alpha(GdObject)
        )
        if (length(sash$x)) {
            grid.xspline(sash$x, -sash$y,
                id = sash$id, shape = -1, open = TRUE,
                default.units = "native", gp = gpar(col = gp$col, lwd = sash$scaled)
            )
            ## print the number of reads together with the connecting lines (currently no scaling/resolution)
            if (sashNumbers) {
                grid.rect(sash$x[c(FALSE, TRUE, FALSE)], -sash$y[c(FALSE, TRUE, FALSE)],
                    width = convertUnit(stringWidth(sash$score) * 1.5, "inches"),
                    height = convertUnit(stringHeight(sash$score) * 1.5, "inches"),
                    default.units = "native", gp = gpar(col = gp$col, fill = gp$fill)
                )
                grid.text(
                    label = sash$score, sash$x[c(FALSE, TRUE, FALSE)], -sash$y[c(FALSE, TRUE, FALSE)],
                    default.units = "native", gp = gpar(col = gp$col)
                )
            }
        }
        popViewport(1)
    }
    ## Now the pileups
    if ("pileup" %in% type) {
        cex <- max(0.3, .dpOrDefault(GdObject, c("cex.mismatch", "cex"), 0.7))
        pushViewport(viewport(
            height = 1 - (covHeight["npc"] + covSpace + sashHeight["npc"] + sashSpace) - vSpacing * 4, y = vSpacing, just = c(0.5, 0),
            gp = .fontGp(GdObject, "mismatch", cex = cex)
        ))
        bins <- stacks(GdObject)
        stacks <- max(bins)
        yscale <- .dpOrDefault(GdObject, ".__yrange")
        if (revs) {
            yscale <- rev(yscale)
        }
        pushViewport(dataViewport(xscale = xscale, extension = 0, yscale = yscale, clip = TRUE))
        ## Figuring out resolution and the necessary plotting details
        res <- .pxResolution(coord = "x")
        ylim <- c(0, 1)
        h <- diff(ylim)
        middle <- mean(ylim)
        sh <- max(0, min(h, .dpOrDefault(GdObject, "stackHeight", 0.75))) / 2
        boxOnly <- res > 10
        if (boxOnly) {
            x <- c(start(readInfo), rep(end(readInfo) + 1, 2), start(readInfo))
            y <- c(rep(readInfo$stack + sh, 2), rep(readInfo$stack - sh, 2))
            id <- rep(readInfo$uid, 4)
        } else {
            ## We first precompute the coordinates for all the arrow polygons
            uid2strand <- setNames(as.character(readInfo$readStrand), as.character(readInfo$uid))
            arrowMap <- unlist(setNames(lapply(split(as.character(readInfo$uid), readInfo$entityId), function(x) {
                str <- uid2strand[x][1]
                setNames(str, if (str == "+") tail(x, 1) else head(x, 1))
            }), NULL))
            readInfo$arrow <- as.character(NA)
            readInfo$arrow[match(names(arrowMap), readInfo$uid)] <- arrowMap
            ## The parts that don't need arrow heads
            sel <- is.na(readInfo$arrow) | readInfo$arrow == "*"
            x <- c(start(readInfo)[sel], rep(end(readInfo)[sel] + 1, 2), start(readInfo)[sel])
            y <- c(rep(readInfo$stack[sel] + sh, 2), rep(readInfo$stack[sel] - sh, 2))
            id <- rep(readInfo$uid[sel], 4)
            ## The arrow heads facing right
            w <- Gviz:::.pxResolution(coord = "x", 5)
            sel <- readInfo$arrow == "+"
            ah <- pmax(start(readInfo)[sel], end(readInfo)[sel] + 1 - w)
            x <- c(x, start(readInfo)[sel], ah, end(readInfo)[sel] + 1, ah, start(readInfo)[sel])
            y <- c(y, rep(readInfo$stack[sel] + sh, 2), readInfo$stack[sel], rep(readInfo$stack[sel] - sh, 2))
            id <- c(id, rep(readInfo$uid[sel], 5))
            ## The arrow heads facing left
            sel <- readInfo$arrow == "-"
            ah <- pmin(end(readInfo)[sel], start(readInfo)[sel] + w)
            x <- c(x, start(readInfo)[sel], ah, rep(end(readInfo)[sel] + 1, 2), ah)
            y <- c(y, readInfo$stack[sel], rep(readInfo$stack[sel] + sh, 2), rep(readInfo$stack[sel] - sh, 2))
            id <- c(id, rep(readInfo$uid[sel], 5))
        }
        nn <- length(unique(readInfo$uid))
        gps <- data.frame(
            col = rep(.dpOrDefault(GdObject, c("col.reads", "col"), .DEFAULT_SHADED_COL), nn),
            fill = rep(.dpOrDefault(GdObject, c("fill.reads", "fill"), .DEFAULT_BRIGHT_SHADED_COL), nn),
            lwd = rep(.dpOrDefault(GdObject, c("lwd.reads", "lwd"), 1), nn),
            lty = rep(.dpOrDefault(GdObject, c("lty.reads", "lty"), 1), nn),
            alpha = rep(.alpha(GdObject, "reads"), nn), stringsAsFactors = FALSE
        )
        ## Finally we draw reads (we need to draw in two steps because of the different alpha levels, reads vs. mismatches)
        grid.polygon(x = x, y = y, id = id, default.units = "native", gp = gpar(col = gps$col, fill = gps$fill, lwd = gps$lwd, lty = gps$lty, alpha = unique(gps$alpha))) # fix for Sys.setenv(`_R_CHECK_LENGTH_1_CONDITION_`="true"); Sys.setenv(`_R_CHECK_LENGTH_1_LOGIC2_`="true")
        ## Now the coordinates for the connecting lines
        lineCoords <- NULL
        if (anyDuplicated(readInfo$entityId) != 0) {
            stTmp <- split(readInfo, readInfo$entityId)
            mateRanges <- unlist(range(stTmp))
            mateGaps <- gaps(GRanges(ranges = ranges(readInfo), seqnames = readInfo$entityId))
            rmap <- mateRanges[as.character(seqnames(mateGaps))]
            mateGaps <- mateGaps[start(rmap) <= start(mateGaps) & end(rmap) >= end(mateGaps)]
            gy <- readInfo$stack[match(as.character(seqnames(mateGaps)), readInfo$entityId)]
            lineCoords <- data.frame(
                x1 = start(mateGaps), y1 = gy, x2 = end(mateGaps) + 1, y2 = gy,
                col = .dpOrDefault(GdObject, c("col.gap", "col"), .DEFAULT_SHADED_COL),
                lwd = .dpOrDefault(GdObject, c("lwd.gap", "lwd"), 1),
                lty = .dpOrDefault(GdObject, c("lty.gap", "lty"), 1),
                alpha = .alpha(GdObject, "gap"), stringsAsFactors = FALSE
            )
            lineCoords <- lineCoords[!duplicated(lineCoords), ]
        } else {
            mateRanges <- setNames(readInfo, readInfo$entityId)
        }
        if (any(readInfo$status != "unmated") && anyDuplicated(readInfo$groupid) != 0) {
            pairGaps <- gaps(GRanges(ranges = ranges(mateRanges), seqnames = readInfo$groupid[match(names(mateRanges), readInfo$entityId)]))
            rmap <- GdObject@stackRanges[as.character(seqnames(pairGaps))]
            pairGaps <- pairGaps[start(rmap) <= start(pairGaps) & end(rmap) >= end(pairGaps)]
            gy <- readInfo$stack[match(as.character(seqnames(pairGaps)), readInfo$groupid)]
            if (length(pairGaps)) {
                pairsCoords <- data.frame(
                    x1 = start(pairGaps) - 1, y1 = gy, x2 = end(pairGaps) + 1, y2 = gy,
                    col = .dpOrDefault(GdObject, c("col.mates", "col"), .DEFAULT_BRIGHT_SHADED_COL),
                    lwd = .dpOrDefault(GdObject, c("lwd.mates", "lwd"), 1),
                    lty = .dpOrDefault(GdObject, c("lty.mates", "lty"), 1),
                    alpha = .alpha(GdObject, "mates"), stringsAsFactors = FALSE
                )
            } else {
                pairsCoords <- NULL
            }
            lineCoords <- rbind(lineCoords, pairsCoords[!duplicated(pairsCoords), ])
        }
        ## Consider the indels if needed
        ## - deletion as lines
        ## - insertions as vertical bars
        showIndels <- .dpOrDefault(GdObject, "showIndels", FALSE)
        delCoords <- NULL
        insCoords <- NULL
        if (showIndels) {
            cigarTmp <- DataFrame(cigar = readInfo$cigar, start = start(readInfo), entityId = readInfo$entityId, groupId = readInfo$groupid)
            cigarTmp <- cigarTmp[order(cigarTmp$entityId, cigarTmp$start), ]
            cigarTmp <- cigarTmp[!duplicated(cigarTmp$entityId), ]
            delGaps <- unlist(cigarRangesAlongReferenceSpace(cigarTmp$cigar, pos = cigarTmp$start, ops = "D", f = as.factor(cigarTmp$entityId)))
            gy <- readInfo$stack[match(names(delGaps), readInfo$entityId)]
            if (length(delGaps)) {
                delCoords <- data.frame(
                    x1 = start(delGaps) + 1, y1 = gy, x2 = end(delGaps) + 1, y2 = gy,
                    col = .dpOrDefault(GdObject, c("col.deletion", "col"), .DEFAULT_BRIGHT_SHADED_COL),
                    lwd = .dpOrDefault(GdObject, c("lwd.deletion", "lwd"), 1),
                    lty = .dpOrDefault(GdObject, c("lty.deletion", "lty"), 1),
                    alpha = .alpha(GdObject, "deletions"), stringsAsFactors = FALSE
                )
                lineCoords <- rbind(delCoords, lineCoords)
                lineCoords <- lineCoords[!duplicated(lineCoords[, c("x1", "y1", "x2", "y2")]), ]
            }
            insGaps <- unlist(cigarRangesAlongReferenceSpace(cigarTmp$cigar, pos = cigarTmp$start, ops = "I", f = as.factor(cigarTmp$entityId)))
            gy <- readInfo$stack[match(names(insGaps), readInfo$entityId)]
            if (length(insGaps)) {
                ## should both x coordinates be equal to start
                insCoords <- data.frame(
                    x1 = start(insGaps), y1 = gy - sh, x2 = start(insGaps), y2 = gy + sh,
                    col = .dpOrDefault(GdObject, c("col.insertion", "col"), .DEFAULT_BRIGHT_SHADED_COL),
                    lwd = .dpOrDefault(GdObject, c("lwd.insertion", "lwd"), 1),
                    lty = .dpOrDefault(GdObject, c("lty.insertion", "lty"), 1),
                    alpha = .alpha(GdObject, "insertions"), stringsAsFactors = FALSE
                )
            }
        }
        ## The mismatch information on the reads if needed
        mmLetters <- NULL
        if (!is.null(mm)) {
            fcol <- .dpOrDefault(GdObject@referenceSequence, "fontcolor", getBioColor("DNA_BASES_N"))
            ccol <- ifelse(rgb2hsv(col2rgb(fcol))["s", ] < 0.5, "black", "white")
            vpl <- vpLocation()
            lwidth <- max(as.numeric(convertUnit(stringWidth(DNA_ALPHABET), "inches"))) * 0.9337632
            lheight <- max(as.numeric(convertUnit(stringHeight(DNA_ALPHABET), "inches")))
            perLetterW <- vpl$isize["width"] / (maxBase - minBase + 1)
            perLetterH <- vpl$isize["height"] / abs(diff(current.viewport()$yscale))
            res <- .pxResolution(coord = "x")
            mw <- res * .dpOrDefault(GdObject, "min.width", 1)
            mwy <- max(1, mw)
            if (nrow(mm)) {
                x <- c(mm$position + rep(c(0, mwy, mwy, 0), each = nrow(mm)))
                y <- c(rep(mm$stack - sh, 2), rep(mm$stack + sh, 2))
                id <- c(rep(seq(max(id, na.rm = TRUE) + 1, len = nrow(mm)), 4))
                gps <- data.frame(
                    col = rep(if (!(lwidth < perLetterW && lheight < perLetterH)) {
                        "transparent"
                    } else {
                        .dpOrDefault(GdObject, "col.mismatch", .DEFAULT_SHADED_COL)
                    }, nrow(mm)),
                    fill = rep(fcol[as.character(mm$base)]),
                    lwd = rep(.dpOrDefault(GdObject, "lwd.mismatch", 1), nrow(mm)),
                    lty = rep(.dpOrDefault(GdObject, "lty.mismatch", 1), nrow(mm)),
                    alpha = rep(.dpOrDefault(GdObject, "alpha.mismatch", 1), nrow(mm)),
                    stringsAsFactors = FALSE
                )
                ## Finally we draw mm (we need to draw in two steps because of the different alpha levels, reads vs. mismatches)
                grid.polygon(x = x, y = y, id = id, default.units = "native", gp = gpar(col = gps$col, fill = gps$fill, lwd = gps$lwd, lty = gps$lty, alpha = unique(gps$alpha))) # fix for Sys.setenv(`_R_CHECK_LENGTH_1_CONDITION_`="true"); Sys.setenv(`_R_CHECK_LENGTH_1_LOGIC2_`="true")
                if (!.dpOrDefault(GdObject, "noLetters", FALSE) && lwidth < perLetterW && lheight < perLetterH) {
                    mmLetters <- data.frame(x = mm$position + 0.5, y = mm$stack, label = mm$base, col = ccol[mm$base], stringsAsFactors = FALSE)
                }
            }
        }
        if (!is.null(lineCoords)) {
            grid.segments(lineCoords$x1, lineCoords$y1, lineCoords$x2, lineCoords$y2,
                gp = gpar(
                    col = lineCoords$col, alpha = unique(lineCoords$alpha),
                    lwd = lineCoords$lwd, lty = lineCoords$lty
                ),
                default.units = "native"
            )
        }
        if (!is.null(insCoords)) {
            grid.segments(insCoords$x1, insCoords$y1, insCoords$x2, insCoords$y2,
                gp = gpar(
                    col = insCoords$col, alpha = unique(insCoords$alpha),
                    lwd = insCoords$lwd, lty = insCoords$lty
                ),
                default.units = "native"
            )
        }
        if (!is.null(mmLetters)) {
            grid.text(
                x = mmLetters$x, y = mmLetters$y, label = mmLetters$label,
                gp = gpar(col = mmLetters$col), default.units = "native"
            )
        }
        popViewport(2)
    }
    ## Eventually we set up the image map
    ## imageMap(GdObject) <- im
    return(invisible(GdObject))
})

## SetAs ---------------------------------------------------------------------
## Show ----------------------------------------------------------------------

#' @describeIn AlignmentsTrack-class Show method.
#' @export
setMethod(
    "show", signature(object = "AlignmentsTrack"),
    function(object) {
        cat(sprintf(
            paste("AlignmentsTrack track '%s' \n",
                "| genome: %s\n",
                "| active chromosome: %s\n",
                "| containing %i read%s\n",
                sep = ""
            ),
            names(object), genome(object),
            gsub("^chr", "", chromosome(object)),
            length(object),
            ifelse(length(object) == 1, "", "s")
        ))
    }
)

#' @describeIn AlignmentsTrack-class Show method.
#' @export
setMethod("show", signature(object = "ReferenceAlignmentsTrack"), function(object) {
    .referenceTrackInfo(object, "ReferenceAlignmentsTrack")
})

## Plot a list of GdObjects as individual tracks similar to the display on the UCSC genome browser
## Arguments:
##    o trackList: a list of GdObjects
##    o from, to: the plotting range, will be figured out automatically from the tracks if missing
##    o sized: a vector of relative vertical sizes, or NULL to auto-detect
##    o panel.only: don't draw track titles, useful to embed in a lattice-like function, this also implies add=TRUE
##    o extend.right, extend.left: extend the coordinates in 'from' and 'too'
##    o title.width: the expansion factor for the width of the title track
## Value: the function is called for its side-effect of drawing on the graphics device


#' The main plotting function for one or several Gviz tracks.
#'
#' `plotTracks` is the main interface when plotting single track objects,
#' or lists of tracks linked together across the same genomic coordinates.
#' Essentially, the resulting plots are very similar to the graphical output of
#' the UCSC Genome Browser, except for all of the interactivity.
#'
#' Gviz tracks are plotted in a vertically stacked layout. Each track
#' panel is split up into a title section containing the track name, as well as
#' an optional axis for tracks containing numeric data, and a data section
#' showing the actual data along genomic coordinates. In that sense, the output
#' is very similar to the UCSC Genome Browser.
#'
#' The layout of the individual tracks is highly customizable though so called
#' "display parameters". See \code{\link{settings}} for details.
#'
#' While plotting a track, the software automatically computes HTML image map
#' coordinates based on the current graphics device. These coordinates as well
#' as the associated annotation information can later be used to embed images
#' of the plots in semi-interactive HTML pages. See
#' \code{\linkS4class{ImageMap}} for details.
#'
#' @name plotTracks
#'
#' @param trackList A list of Gviz track objects, all inheriting from
#' class \code{\linkS4class{GdObject}}. The tracks will all be drawn to the
#' same genomic coordinates, either as defined by the `from` and `to`
#' arguments if supplied, or by the maximum range across all individual items
#' in the list.
#' @param from,to Character scalar, giving the range of genomic coordinates to
#' draw the tracks in. Note that `to` cannot be larger than `from`.
#' If `NULL`, the plotting ranges are derived from the individual tracks.
#' See `extend.left` and `extend.right` below for the definition of
#' the final plotting ranges.
#' @param \dots Additional arguments which are all interpreted as display
#' parameters to tweak the appearance of the plot. These parameters are global,
#' meaning that they will be used for all tracks in the list where they
#' actually make sense, and they override the track-internal settings. See
#' [settings] for details on display parameters.
#' @param sizes A numeric vector of relative vertical sizes for the individual
#' tracks of length equal to the number of tracks in `trackList`, or
#' `NULL` to auto-detect the most appropriate vertical size proportions.
#' @param panel.only Logical flag, causing the tracks to be plotted as
#' lattice-like panel functions without resetting the plotting canvas and
#' omitting the title pane. This allows to embed tracks into a trellis layout.
#' Usually the function is called for a single track only when
#' `panel.only==TRUE`.
#' @param extend.right,extend.left Numeric scalar, extend the plotting range to
#' the right or to the left by a fixed number of bases. The final plotting
#' range is defined as `from-extend.left` to `to+extend.right`.
#' @param title.width A expansion factor for the width of the title panels.
#' This can be used to make more space, e.g. to accommodate for more detailed
#' data axes. The default is to use as much space as needed to fit all the
#' annotation text.
#' @param add Logical flag, add the plot to an existing plotting canvas without
#' re-initialising.
#' @param main Character scalar, the plots main header.
#' @param cex.main,fontface.main,col.main The fontface, color and expansion
#' factor settings for the main header.
#' @param margin The margin width to add to the plot in pixels.
#' @param innerMargin The inner margin width to add to the plot in pixels.
#' @param chromosome Set the chromosome for all the tracks in the track list.
#'
#' @return A list of Gviz tracks, each one augmented by the computed image map
#' coordinates in the `imageMap` slot, along with the additional `ImageMap`
#' object `titles` containing information about the title panels.
#'
#' @author Florian Hahne
#'
#' @seealso
#' \code{\linkS4class{GdObject}}
#'
#' \code{\linkS4class{ImageMap}}
#'
#' \code{\linkS4class{ImageMap}}
#'
#' \code{\linkS4class{StackedTrack}}
#'
#' \code{\link{settings}}
#'
#' @examples
#' ## Create some tracks to plot
#' st <- c(2000000, 2070000, 2100000, 2160000)
#' ed <- c(2050000, 2130000, 2150000, 2170000)
#' str <- c("-", "+", "-", "-")
#' gr <- c("Group1", "Group2", "Group1", "Group3")
#' annTrack <- AnnotationTrack(
#'     start = st, end = ed, strand = str, chromosome = 7,
#'     genome = "hg19", feature = "test", group = gr,
#'     id = paste("annTrack item", 1:4),
#'     name = "annotation track foo",
#'     stacking = "squish"
#' )
#'
#' ax <- GenomeAxisTrack()
#'
#' dt <- DataTrack(
#'     start = seq(min(st), max(ed), len = 10), width = 18000,
#'     data = matrix(runif(40), nrow = 4), genome = "hg19", chromosome = 7,
#'     type = "histogram", name = "data track bar"
#' )
#' \dontshow{
#' ## For some annoying reason the postscript device does not know about
#' ## the sans font
#' if (!interactive()) {
#'     font <- ps.options()$family
#'     displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
#'     displayPars(ax) <- list(fontfamily = font, fontfamily.title = font)
#'     displayPars(dt) <- list(fontfamily = font, fontfamily.title = font)
#' }
#' }
#'
#'
#' ## Now plot the tracks
#' res <- plotTracks(list(ax, annTrack, dt))
#'
#' ## Plot only a subrange
#' res <- plotTracks(list(ax, annTrack, dt), from = 2080000, to = 2156000)
#'
#' ## Extend plotting ranges
#' res <- plotTracks(list(ax, annTrack, dt), extend.left = 200000, extend.right = 200000)
#'
#' ## Add a header
#' res <- plotTracks(list(ax, annTrack, dt),
#'     main = "A GenomGraphs plot",
#'     col.main = "darkgray"
#' )
#'
#' ## Change vertical size and title width
#' res <- plotTracks(list(ax, annTrack, dt), sizes = c(1, 1, 5))
#'
#' names(annTrack) <- "foo"
#' res <- plotTracks(list(ax, annTrack), title.width = 0.6)
#'
#' ## Adding and lattice like plots
#' library(grid)
#' grid.newpage()
#' pushViewport(viewport(height = 0.5, y = 1, just = "top"))
#' grid.rect()
#' plotTracks(annTrack, add = TRUE)
#' popViewport(1)
#' pushViewport(viewport(height = 0.5, y = 0, just = "bottom"))
#' grid.rect()
#' plotTracks(dt, add = TRUE)
#' popViewport(1)
#' \dontrun{
#' library(lattice)
#' myPanel <- function(x, ...) {
#'     plotTracks(annTrack,
#'         panel.only = TRUE,
#'         from = min(x), to = max(x), shape = "box"
#'     )
#' }
#' a <- seq(1900000, 2250000, len = 40)
#' xyplot(b ~ a | c, data.frame(a = a, b = 1, c = cut(a, 4)),
#'     panel = myPanel,
#'     scales = list(x = "free")
#' )
#' }
#'
#' @export
#' @importFrom lattice current.panel.limits panel.abline panel.grid panel.lines panel.points panel.polygon panel.segments panel.xyplot panel.text trellis.par.get
#'
plotTracks <- function(trackList, from = NULL, to = NULL, ..., sizes = NULL, panel.only = FALSE, extend.right = 0,
                       extend.left = 0, title.width = NULL, add = FALSE, main, cex.main = 2, fontface.main = 2,
                       col.main = "black", margin = 6, chromosome = NULL, innerMargin = 3) {
    ## If we have to open a new device for this but do not run through the whole function because of errors we want to
    ## clean up in the end
    done <- FALSE
    cdev <- dev.cur()
    on.exit(if (cdev == 1 && !done) dev.off())
    ## We only need a new plot for regular calls to the function. Both add==TRUE and panel.only=TRUE will add to an existing grid plot
    if (!panel.only && !add) {
        grid.newpage()
    }
    if (!is.list(trackList)) {
        trackList <- list(trackList)
    }
    ## All arguments in ... are considered to be additional display parameters and need to be attached to each item in the track list
    dps <- list(...)
    trackList <- lapply(trackList, function(x) {
        displayPars(x, recursive = TRUE) <- dps
        return(x)
    })

    ## OverlayTracks and HighlightTracks can be discarded if they are empty
    trackList <- trackList[!vapply(trackList, function(x) (is(x, "HighlightTrack") || is(x, "OverlayTrack")) && length(x) < 1, FUN.VALUE = logical(1L))]
    isHt <- which(vapply(trackList, is, "HighlightTrack", FUN.VALUE = logical(1L)))
    isOt <- which(vapply(trackList, is, "OverlayTrack", FUN.VALUE = logical(1L)))
    ## A mix between forward and reverse strand tracks should trigger an alarm
    strds <- unique(.whichStrand(trackList))
    if (!is.null(strds) && length(strds) > 1) {
        warning("Plotting a mixture of forward strand and reverse strand tracks.\n Are you sure this is correct?")
    }
    ## We first run very general housekeeping tasks on the tracks for which we don't really need to know anything about device
    ## size, resolution or plotting ranges.
    ## Chromosomes should all be the same for all tracks, if not we will force them to be set to the first one that can be detected.
    ## If plotting ranges are supplied we can speed up a lot of the downstream operations by subsetting first.
    ## We may want to use alpha blending on those devices that support it, but also fall back to non-transparent colors without causing
    ## warnings.
    hasAlpha <- .supportsAlpha()
    chrms <- unique(unlist(lapply(trackList, .recChromosome)))
    if (is.null(chromosome)) {
        chrms <- if (!is.null(chrms)) chrms[gsub("^chr", "", chrms) != "NA"] else chrms
        chromosome <- head(chrms, 1)
        if (length(chromosome) == 0) {
            chromosome <- "chrNA"
        }
        if (!is.null(chrms) && length(unique(chrms)) != 1) {
            warning("The track chromosomes in 'trackList' differ. Setting all tracks to chromosome '", chromosome, "'", sep = "")
        }
    }
    if (!is.null(from) || !(is.null(to))) {
        trackList <- lapply(trackList, function(x) {
            chromosome(x) <- chromosome
            subset(x, from = from, to = to, chromosome = chromosome, sort = FALSE, stacks = FALSE, use.defaults = FALSE)
        })
    }
    trackList <- lapply(trackList, consolidateTrack,
        chromosome = chromosome, any(.needsAxis(trackList)), any(.needsTitle(trackList)),
        title.width, alpha = hasAlpha, ...
    )

    ## Now we figure out the plotting ranges. If no ranges are given as function arguments we take the absolute min/max of all tracks.
    ranges <- .defaultRange(trackList, from = from, to = to, extend.left = extend.left, extend.right = extend.right, annotation = TRUE)
    ## Now we can subset all the objects in the list to the current boundaries and compute the initial stacking
    trackList <- lapply(trackList, subset, from = ranges["from"], to = ranges["to"], chromosome = chromosome)
    trackList <- lapply(trackList, setStacks, recomputeRanges = FALSE)
    ## Highlight tracks are just a way to add a common highlighting region to several tracks, but other than that we can treat the containing
    ## tracks a normal track objects, and thus unlist them. We only want to record their indexes in the expanded list for later.
    htList <- list()
    expandedTrackList <- if (length(isHt)) {
        j <- 1
        tlTemp <- list()
        for (i in seq_along(trackList)) {
            if (!i %in% isHt) {
                tlTemp <- c(tlTemp, trackList[[i]])
                j <- j + 1
            } else {
                tlTemp <- c(tlTemp, trackList[[i]]@trackList)
                htList[[as.character(i)]] <- list(
                    indexes = j:(j + length(trackList[[i]]@trackList) - 1),
                    track = trackList[[i]]
                )
                j <- j + length(trackList[[i]]@trackList)
            }
        }
        tlTemp
    } else {
        trackList
    }
    ## If there is a AlignmentsTrack and also a SequenceTrack we can tell the former to use the latter, unless already provided
    isAt <- vapply(expandedTrackList, is, "AlignmentsTrack", FUN.VALUE = logical(1L))
    isSt <- vapply(expandedTrackList, is, "SequenceTrack", FUN.VALUE = logical(1L))
    for (ai in which(isAt)) {
        if (is.null(expandedTrackList[[ai]]@referenceSequence) && any(isSt)) {
            expandedTrackList[[ai]]@referenceSequence <- expandedTrackList[[min(which(isSt))]]
        }
    }
    ## We need to reverse the list to get a top to bottom plotting order
    expandedTrackList <- rev(expandedTrackList)
    map <- vector(mode = "list", length = length(expandedTrackList))
    titleCoords <- NULL
    names(map) <- rev(vapply(expandedTrackList, names, FUN.VALUE = character(1L)))
    ## Open a fresh page and set up the bounding box, unless add==TRUE
    if (!panel.only) {
        ## We want a margin pixel border
        ## for backward compatibility, if margin has length of 2,
        ## the first one will be used as a horizontal, second as a vertical margin
        if (length(margin) == 2) {
            margin <- rev(margin)
        }
        ## we switched to same settings as in par
        ## c(bottom, left, top, right)
        margin <- rep(as.numeric(margin), length.out = 4)
        vpWidth <- vpLocation()$size["width"]
        vpHeight <- vpLocation()$size["height"]
        vpBound <- viewport(
            x = margin[2L] / vpWidth, y = margin[1L] / vpHeight,
            width = (vpWidth - sum(margin[c(2, 4)])) / vpWidth,
            height = (vpHeight - sum(margin[c(1, 3)])) / vpHeight,
            just = c("left", "bottom")
        )
        pushViewport(vpBound)
        ## If there is a header we have to make some room for it here
        if (!missing(main) && main != "") {
            vpHeader <- viewport(width = 1, height = 0.1, y = 1, just = c("center", "top"))
            pushViewport(vpHeader)
            grid.text(main, gp = gpar(col = col.main, cex = cex.main, fontface = fontface.main))
            popViewport(1)
            vpMain <- viewport(width = 1, height = 0.9, y = 0.9, just = c("center", "top"))
        } else {
            vpMain <- viewport(width = 1, height = 1)
        }
        pushViewport(vpMain)
        ## A first guestimate of the vertical space that's needed
        spaceSetup <- .setupTextSize(expandedTrackList, sizes, title.width, spacing = innerMargin)
    } else {
        vpBound <- viewport()
        pushViewport(vpBound)
        spaceSetup <- .setupTextSize(expandedTrackList, sizes, spacing = innerMargin)
    }
    ## First iteration to set up all the dimensions by calling the drawGD methods in prepare mode, i.e.,
    ## argument prepare=TRUE. Nothing is drawn at this point, and this only exists to circumvent the
    ## chicken and egg problem of not knowing how much space we need until we draw, but also not knowing
    ## where to draw until we know the space needed.
    for (i in rev(seq_along(expandedTrackList)))
    {
        fontSettings <- .fontGp(expandedTrackList[[i]], cex = NULL)
        vpTrack <- viewport(
            x = 0, y = sum(spaceSetup$spaceNeeded[seq_len(i)]), just = c(0, 1), width = 1, height = spaceSetup$spaceNeeded[i],
            gp = fontSettings
        )
        pushViewport(vpTrack)
        vpContent <- if (!panel.only) {
            viewport(
                x = spaceSetup$title.width + spaceSetup$spacing,
                width = 1 - spaceSetup$title.width - spaceSetup$spacing * 2, just = 0
            )
        } else {
            viewport(width = 1)
        }
        pushViewport(vpContent)
        expandedTrackList[[i]] <- drawGD(expandedTrackList[[i]], minBase = ranges["from"], maxBase = ranges["to"], prepare = TRUE, subset = FALSE)
        popViewport(2)
    }
    ## Now lets recalculate the space and draw for real
    spaceSetup <- .setupTextSize(expandedTrackList, sizes, title.width, spacing = innerMargin)
    ## First the highlight box backgrounds
    htBoxes <- data.frame(stringsAsFactors = FALSE)
    for (hlite in htList) {
        if (length(ranges(hlite$track))) {
            inds <- setdiff(sort(length(expandedTrackList) - hlite$index + 1), which(vapply(expandedTrackList, is, "IdeogramTrack", FUN.VALUE = logical(1L))))
            y <- reduce(IRanges(start = inds, width = 1))
            yy <- ifelse(start(y) == 1, 0, sum(spaceSetup$spaceNeeded[seq_len(start(y)) - 1])) # check
            ht <- sum(spaceSetup$spaceNeeded[start(y):end(y)])
            htBoxes <- rbind(htBoxes, data.frame(
                y = yy, height = ht, x = start(hlite$track), width = width(hlite$track),
                col = .dpOrDefault(hlite$track, "col", "orange"),
                fill = .dpOrDefault(hlite$track, "fill", "red"),
                lwd = .dpOrDefault(hlite$track, "lwd", 1),
                lty = .dpOrDefault(hlite$track, "lty", 1),
                alpha = .dpOrDefault(hlite$track, "alpha", 1),
                inBackground = .dpOrDefault(hlite$track, "inBackground", TRUE),
                stringsAsFactors = FALSE
            ))
        }
    }
    .drawHtBoxes <- function(htBoxes, background = TRUE) {
        htBoxes <- htBoxes[htBoxes$inBackground == background, , drop = FALSE]
        rscales <- if (strds[1] == "reverse") c(from = ranges["to"], to = ranges["from"]) else ranges
        if (nrow(htBoxes)) {
            vpContent <- if (!panel.only) {
                viewport(
                    x = spaceSetup$title.width + spaceSetup$spacing, xscale = rscales,
                    width = 1 - spaceSetup$title.width - spaceSetup$spacing * 2, just = 0
                )
            } else {
                viewport(width = 1, xscale = rscales)
            }
            pushViewport(vpContent)
            grid.rect(
                x = htBoxes$x, just = c(0, 1), width = htBoxes$width, y = htBoxes$y + htBoxes$height, height = htBoxes$height,
                gp = gpar(col = htBoxes$col, fill = htBoxes$fill, lwd = htBoxes$lwd, lty = htBoxes$lty, alpha = unique(htBoxes$alpha)), default.units = "native"
            )
            popViewport(1)
        }
    }
    if (nrow(htBoxes)) {
        .drawHtBoxes(htBoxes)
    }
    ## Now the track content
    for (i in rev(seq_along(expandedTrackList))) {
        vpTrack <- viewport(x = 0, y = sum(spaceSetup$spaceNeeded[seq_len(i)]), just = c(0, 1), width = 1, height = spaceSetup$spaceNeeded[i])
        pushViewport(vpTrack)
        fill <- .dpOrDefault(expandedTrackList[[i]], "background.title", .DEFAULT_SHADED_COL)
        thisTrack <- if (is(expandedTrackList[[i]], "OverlayTrack")) {
            tmpThisTrack <- expandedTrackList[[i]]@trackList
             while (any(vapply(tmpThisTrack, is, "OverlayTrack", FUN.VALUE = logical(1L)))) {
                 tmpThisTrack <- rapply(tmpThisTrack, function(x) if (is(x, "OverlayTrack")) x@trackList else x)
             }
            tmpSpaceSetup <- .setupTextSize(list(tmpThisTrack[[1]]), sizes[1], title.width, spacing = innerMargin)
            spaceSetup$nwrap[i] <- tmpSpaceSetup$nwrap[1]
            if (spaceSetup$title.width < tmpSpaceSetup$title.width) {
                spaceSetup$title.width <- tmpSpaceSetup$title.width
            }
            tmpThisTrack[[1]]
        } else {
            expandedTrackList[[i]]
        }
        if (!panel.only) {
            fontSettings <- .fontGp(thisTrack, subtype = "title", cex = NULL)
            vpTitle <- viewport(x = 0, width = spaceSetup$title.width, just = 0, gp = fontSettings)
            pushViewport(vpTitle)
            lwd.border.title <- .dpOrDefault(thisTrack, "lwd.title", 1)
            col.border.title <- .dpOrDefault(thisTrack, "col.border.title", "transparent")
            grid.rect(gp = gpar(fill = fill, col = col.border.title, lwd = lwd.border.title))
            needAxis <- .needsAxis(thisTrack)
            drawAxis(thisTrack, ranges["from"], ranges["to"], subset = FALSE)
            tit <- spaceSetup$nwrap[i]
            ## FIXME: Do we want something smarted for the image map coordinates?
            titleCoords <- rbind(titleCoords, cbind(.getImageMap(cbind(0, 0, 1, 1)),
                title = names(thisTrack)
            ))
            if (.dpOrDefault(thisTrack, "showTitle", TRUE) && !is.null(tit) && tit != "") {
                x <- if (needAxis) 0.075 else 0.4
                just <- if (needAxis) c("center", "top") else "center"
                ## FIXME: We need to deal with this when calculating the space for the title bar
                rot <- .dpOrDefault(thisTrack, "rotation.title", 90)
                gp <- .fontGp(thisTrack, "title", cex = spaceSetup$cex[i])
                suppressWarnings(grid.text(tit, unit(x, "npc"), rot = rot, gp = gp, just = just))
            }
            popViewport(1)
        }
        ## Draw the panel background, grid lines if necessary and the panel content
        vpBackground <- if (!panel.only) {
            viewport(
                x = spaceSetup$title.width,
                width = 1 - spaceSetup$title.width, just = 0
            )
        } else {
            viewport(width = 1)
        }
        pushViewport(vpBackground)
        grid.rect(gp = gpar(col = "transparent", fill = .dpOrDefault(thisTrack, "background.panel", "transparent")))
        drawGrid(thisTrack, ranges["from"], ranges["to"])
        popViewport(1)
        fontSettings <- .fontGp(expandedTrackList[[i]], cex = NULL)
        vpContentOuter <- if (!panel.only) {
            viewport(
                x = spaceSetup$title.width, width = 1 - spaceSetup$title.width,
                just = 0, gp = fontSettings, clip = TRUE
            )
        } else {
            viewport(width = 1, gp = fontSettings, clip = TRUE)
        }
        pushViewport(vpContentOuter)
        vpContent <- if (!panel.only) {
            viewport(x = spaceSetup$spacing, width = 1 - (spaceSetup$spacing * 2), just = 0, gp = fontSettings)
        } else {
            viewport(width = 1, gp = fontSettings)
        }
        pushViewport(vpContent)
        tmp <- drawGD(expandedTrackList[[i]], minBase = ranges["from"], maxBase = ranges["to"], subset = FALSE)
        if (!is.null(tmp)) {
            map[[(length(map) + 1) - i]] <- tmp
        }
        popViewport(2)
        if (.dpOrDefault(thisTrack, "frame", FALSE)) {
            grid.rect(gp = gpar(col = .dpOrDefault(thisTrack, "col.frame", .DEFAULT_SHADED_COL), fill = "transparent"))
        }
        popViewport(1)
    }
    if (nrow(htBoxes)) {
        .drawHtBoxes(htBoxes, FALSE)
    }
    popViewport(if (panel.only) 1 else 2)
    tc <- as.character(titleCoords[, 5])
    tc[which(tc == "" | is.na(tc) | is.null(tc))] <- "NA"
    names(tc) <- tc
    if (!is.null(titleCoords)) {
        tcoord <- as.matrix(titleCoords[, seq(1, 4)])
        rownames(tcoord) <- names(tc)
        map$titles <- ImageMap(coords = tcoord, tags = list(title = tc))
    }
    done <- TRUE
    return(invisible(map))
}

## This script will try to identify genome versions in all available
## ENSEBML archives as well as the current release, and match these
## versions back to UCSC genome identifiers.

library(biomaRt)

parseArchiveDate <- function(x) {
    datePart <- sub("\\.archive.*", "", sub("^https://", "", x))
    return(as.Date(paste(datePart, "1", sep = ""), format = "%b%Y%d"))
}


getBmFromDate <- function(date) {
    archive <- sprintf("https://%s%s.archive.ensembl.org", format(date, "%b"), format(date, "%Y"))
    res <- try(capture.output(suppressMessages(bm <- useMart(
        host = archive,
        biomart = "ENSEMBL_MART_ENSEMBL"
    ))),
    silent = TRUE
    )
    if (is(res, "try-error")) {
        bm <- NA
    }
    return(bm)
}

year <- function(x) {
    if (missing(x)) {
        x <- Sys.Date()
    }
    return(as.integer(format(x, "%Y")))
}

month <- function(x) {
    if (missing(x)) {
        x <- Sys.Date()
    }
    return(as.integer(format(x, "%m")))
}

day <- function(x) {
    if (missing(x)) {
        x <- Sys.Date()
    }
    return(as.integer(format(x, "%d")))
}

knownEnsemblArchives <- c(
    `Ensembl 105: Dec 2021` = "https://Dec2021.archive.ensembl.org",
    `Ensembl 104: May 2021` = "https://May2021.archive.ensembl.org",
    `Ensembl 103: Feb 2021` = "https://Feb2021.archive.ensembl.org",
    `Ensembl 102: Nov 2020` = "https://Nov2020.archive.ensembl.org",
    `Ensembl 101: Aug 2020` = "https://Aug2020.archive.ensembl.org",
    `Ensembl 100: Apr 2020` = "https://Apr2020.archive.ensembl.org",
    `Ensembl 99: Jan 2020` = "https://Jan2020.archive.ensembl.org",
    `Ensembl 98: Sep 2019` = "https://Sep2019.archive.ensembl.org",
    `Ensembl 97: Jul 2019` = "https://Jul2019.archive.ensembl.org",
    `Ensembl 96: Apr 2019` = "https://Apr2019.archive.ensembl.org",
    `Ensembl 95: Jan 2019` = "https://Jan2019.archive.ensembl.org",
    `Ensembl 94: Oct 2018` = "https://Oct2018.archive.ensembl.org",
    `Ensembl 93: Jul 2018` = "https://Jul2018.archive.ensembl.org",
    `Ensembl 92: Apr 2018` = "https://Apr2018.archive.ensembl.org",
    `Ensembl 91: Dec 2017` = "https://Dec2017.archive.ensembl.org",
    `Ensembl 90: Aug 2017` = "https://Aug2017.archive.ensembl.org",
    `Ensembl 89: May 2017` = "https://May2017.archive.ensembl.org",
    `Ensembl 88: Mar 2017` = "https://Mar2017.archive.ensembl.org",
    `Ensembl 87: Dec 2016` = "https://Dec2016.archive.ensembl.org",
    `Ensembl 86: Oct 2016` = "https://Oct2016.archive.ensembl.org",
    `Ensembl 80: May 2015` = "https://May2015.archive.ensembl.org",
    `Ensembl 77: Oct 2014` = "https://Oct2014.archive.ensembl.org",
    `Ensembl 75: Feb 2014` = "https://Feb2014.archive.ensembl.org",
    `Ensembl 54: May 2009` = "https://May2009.archive.ensembl.org"
)

parseOneArchive <- function(date) {
    bm <- getBmFromDate(date)
    datasets <- listDatasets(bm)
    return(datasets)
}

parseEnsemblArchives <- function(known) {
    archiveDates <- sort(parseArchiveDate(known))
    ## We may have to extend our search date range in case that the known archives vector is
    ## out of date and later archives exist
    latest <- max(archiveDates)
    daysBack <- as.integer(Sys.Date() - latest)
    if (daysBack > day()) {
        daysSince <- Sys.Date() - (1:daysBack)
        archiveDates <- sort(unique(c(
            archiveDates,
            as.Date(paste(unique(format(daysSince, "%b%Y")), "1", sep = ""),
                format = "%b%Y%d"
            )
        )))
    }
}

## libs ---------------------------------------------------------------------
library(BSgenome.Hsapiens.UCSC.hg19)

## general ------------------------------------------------------------------
set.seed(789)
ir <- IRanges(1L, 5L)
gr <- GRanges("chr1", ir, score = 1)
ir2 <- IRanges(2L, 6L)
gr2 <- GRanges("chr1", ir2, score = 2)
dna.sq <- DNAStringSet(c(chr1 = paste(sample(DNA_BASES, 100, replace = TRUE), collapse = "")))
rna.sq <- RNAStringSet(c(chr1 = paste(sample(RNA_BASES, 100, replace = TRUE), collapse = "")))
cyto.bands <- data.frame(
    chrom = rep(c("chrI", "chrII"), each = 4),
    chromStart = rep(c(1L, 148071L, 151524L, 154977L), 2),
    chromEnd = rep(c(148071L, 151524L, 154977L, 230218L), 2),
    name = rep(c(NA, "CEN1", "CEN1", NA), 2),
    gieStain = rep(c("gneg", "acen", "acen", "gneg"), 2),
    stringsAsFactors = FALSE
)
## internet access ----------------------------------------------------------
# hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent = TRUE), "try-error")
hasUcscConnection <- FALSE
check_ucsc <- function() {
    if (!hasUcscConnection) {
        skip("UCSC not available")
    }
}

#
# oto <- options(timeout = 5)
# hasBiomartConnection <- (!is(try(download.file("http://www.biomart.org", tempfile(), quiet = TRUE)), "try-error") &&
#     !is(try(biomaRt::listMarts(), silent = TRUE), "try-error"))
#options(timeout = oto)
hasBiomartConnection <- FALSE
check_biomart <- function() {
    if (!hasBiomartConnection) {
        skip("Biomart not available")
    }
}

## Uncommenting this helps when the UCSC server has a hickup but still lets you connect:
## hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent=TRUE), "try-error") &&
##   !is(try(IdeogramTrack(genome="hg19", chromosome=7), silent=TRUE), "try-error")

## tmp files ----------------------------------------------------------------
## BAM file
sam <- c(
    "@HD	VN:1.0	GO:none	SO:coordinate",
    "@SQ	SN:chr1	LN:267910886",
    "@PG	ID:SpliceMap	VN:3.3.5.2 (55)",
    "@CO	file merged using Picard tools",
    "NRCHBS-WDL30299:125:D1415ACXX:8:2215:20868:43279	65	chr1	189891483	255	15S51M6207N10M	=	189897797	47	CGGGACCGTGGTGAGTCAGCGAGTAGGAACTACTCAGGAACTACTCACTTCACTGACAGCCGTAAGTCACTCTGAC	;99;;;5((2@:-)()2:?<88;196?<??1))<>=9?9>>?<>??8>1>>=?;?>?;3=?<9===9=>==>>77=	XS:A:+	NM:i:0	XC:i:15",
    "NRCHBS-WDL30299:126:D14UTACXX:8:2113:17433:81932	177	chr1	189892202	255	76M	=	189892390	113	GCCAAATCAATTCTTTCTTCTCCTAAGCTGCTTTGGTTAGGGTCGTTTATCACAGCAACAGAAAAGTAAGTAAAAC	CC@FFFFFHHHHHJJJIJJIJJJJIJJJJIJJJJJIJJJIIJDGHHIFJJJJIJJJIJJJJJFIJJHHHHHHHHFC	NM:i:0	XC:i:0",
    "NRCHBS-WDL30299:126:D14UTACXX:8:2113:17433:81932	113	chr1	189892390	255	76M	=	189892202	113	GGGCTTTTGCAGGGTTGGGCTGTCCTCCCCTGCCTCTTCTCCAGCCCTTTAATCTTTCAGCACCCAGATCAAGCCG	CCCFFFFFHHHHHJCGHGIIJJIIJJIJJJIJJIJJGHHJIJIIJJJJJIIIGEHHFFFFDDDDDDDDDDDDCBD9	MD:Z:14C61	NM:i:1	XC:i:0",
    "NRCHBS-WDL30299:125:D1415ACXX:8:1313:16113:75579	65	chr1	189893347	255	76M	=	189893470	48	CTGCAGCGGCTCATACAAGACCTTTGCTTAATAGGGTTTTCTTTACCCTAGAAAAATTCCCCTGTGATTTAAGGGT	CCCFFFFFHHHHHJJJJJJIJJJJJJJJJJJJJJJJFHIJJJJJJJJJJJJJJJJIJJJJJJGHHHHFFFFFFDEA	NM:i:0	XC:i:0",
    "NRCHBS-WDL30299:126:D14UTACXX:8:2215:4526:88225	65	chr1	189893352	255	76M	=	189893457	30	GCGGCTCATACAAGACCTTTGCTTAATAGGGTTTTCTTTACCCTAGAAAAATTCCCCTGTGATTTAAGGGTACAGA	@@@DDADDHAF3CGBFH@HIIIEGIGEEHII?CBGDAFHIGFFGIIEGCH@EHGBFHEGCGGCCHICEGG6ACEHE	NM:i:0	XC:i:0"
)
samfile <- tempfile(fileext = ".sam")
cat(paste(sam, collapse = "\n"), file = samfile)
bamfile <- Rsamtools::asBam(samfile, indexDestination = FALSE)
Rsamtools::indexBam(bamfile)

## FASTA
fastafile <- system.file("extdata/test.fa", package = "Gviz")

bamgr <- GRanges("chr1", IRanges(
    c(189892390L, 189892202L, 189893347L, 189891483L, 189893352L),
    c(189892465L, 189892277L, 189893422L, 189891558L, 189893427L)
),
strand = rep(c("-", "+"), c(2, 3))
)
mcols(bamgr) <- DataFrame(
    id = c(
        "NRCHBS-WDL30299:126:D14UTACXX:8:2113:17433:81932",
        "NRCHBS-WDL30299:126:D14UTACXX:8:2113:17433:81932",
        "NRCHBS-WDL30299:125:D1415ACXX:8:1313:16113:75579",
        "NRCHBS-WDL30299:125:D1415ACXX:8:2215:20868:43279",
        "NRCHBS-WDL30299:126:D14UTACXX:8:2215:4526:88225"
    ),
    cigar = c("76M", "76M", "76M", "15S51M6207N10M", "76M"),
    mapq = rep(255L, 5),
    flag = c(113L, 177L, 65L, 65L, 65L),
    md = c("14C61", NA, NA, NA, NA),
    seq = DNAStringSet(c(
        `1` = paste(rep("+", 2600), collapse = ""),
        `2` = paste(rep("+", 2600), collapse = ""),
        `3` = paste(rep("+", 2600), collapse = ""),
        `4` = paste(rep("+", 2600), collapse = ""),
        `5` = paste(rep("+", 2600), collapse = "")
    )),
    isize = c(113L, 113L, 48L, 47L, 30L),
    groupid = c(1L, 1L, 2L, 3L, 4L),
    status = factor(rep(c("mated", "unmated"), c(2, 3)),
        levels = c("mated", "ambiguous", "unmated")
    )
)



selFun <- function(identifier, start, end, track, GdObject, ...) {
    gcount <- table(group(GdObject))
    ## This computes the width of 2 pixels in genomic coordinates
    pxRange <- Gviz:::.pxResolution(min.width = 20, coord = "x")
    return((end - start) < pxRange && gcount[identifier] == 1)
}
detFun <- function(identifier, GdObject.original, ...) {
    plotTracks(list(
        GenomeAxisTrack(scale = 0.3, size = 0.2, cex = 0.7),
        GdObject.original[group(GdObject.original) == identifier]
    ),
    add = TRUE, showTitle = FALSE
    )
}

data(geneDetails)
data(geneModels)

## classes ------------------------------------------------------------------

## IdeogramTrack
ideoTrack <- IdeogramTrack(chromosome = "chrI", genome = "sacCer3", bands = cyto.bands)

## GenomeAxisTrack
axisTrack <- GenomeAxisTrack(gr)

## DataTrack
dataTrack <- DataTrack(gr)

## AnnotationTrack
annoTrack <- AnnotationTrack(gr)

## GeneRegionTrack
geneTrack <- GeneRegionTrack(geneModels, genome = "hg19", chromosome = "chr7", name = "foo")
## BiomartGeneRegionTrack

## DetailsAnnotationTrack
detTrack <- DetailsAnnotationTrack(geneDetails,
    fun = detFun, selectFun = selFun,
    groupDetails = TRUE, details.size = 0.5,
    detailsConnector.cex = 0.5, detailsConnector.lty = "dotted",
    shape = c("smallArrow", "arrow"), groupAnnotation = "group"
)

## SequenceTrack
seqTrack.dna <- SequenceTrack(dna.sq)
seqTrack.rna <- RNASequenceTrack(rna.sq)
seqTrack.bs <- SequenceTrack(BSgenome.Hsapiens.UCSC.hg19)

## AlignmentsTrack
alnTrack <- AlignmentsTrack(bamfile)

## HighlightTrack
highTrack <- HighlightTrack(dataTrack, ranges = gr, chromosome = "chr1")

## OverlayTrack
overTrack <- OverlayTrack(c(dataTrack, dataTrack))

library(Gviz)
library(parallel)
library(biomaRt)
library(Biobase)
library(rtracklayer)
library(RMySQL)

fetchAllGenomesFromUcsc <- function() {
    mydb <- dbConnect(MySQL(),
        user = "genome", dbname = "hgcentral",
        host = "genome-mysql.cse.ucsc.edu"
    )
    rs <- dbSendQuery(mydb, "select * from dbDb")
    out <- fetch(rs, n = -1)
    out <- out[out$active != 0, ] # only active
    cols <- c(
        "name", "description", "organism", "scientificName",
        "sourceName", "taxId", "orderKey"
    ) # smallest orderKey == rescent
    out <- out[, cols]
    out <- out[order(out$organism, out$orderKey), ]
    return(out)
}


testBiomartVersion <- function() {
    allGenomes <- union(
        read.delim(system.file(package = "Gviz", "extdata/biomartVersionsLatest.txt"))$ucscId,
        read.delim(system.file(package = "Gviz", "extdata/biomartVersionsNow.txt"))$ucscId
    )
    res <- mclapply(allGenomes, function(genome) {
        return(try(
            {
                map <- Gviz:::.ucsc2Ensembl(genome)
                res <- list()
                if (map$date == "head") {
                    bm <- useMart("ensembl", dataset = map$dataset)
                    ds <- listDatasets(bm)
                    mt <- ds[match(map$dataset, ds$dataset), "version"]
                    if (is.na(mt)) {
                        res <- list(current = genome, set = map$dataset, type = "head", cause = "error")
                    }
                    if (mt != map$value) {
                        res <- list(current = genome, set = map$value, setCurrent = mt, type = "head", cause = "mismatch")
                    }
                } else {
                    bm <- useMart(
                        host = sprintf("https://%s.archive.ensembl.org", tolower(sub(".", "", map$date, fixed = TRUE))),
                        biomart = "ENSEMBL_MART_ENSEMBL", dataset = map$dataset
                    )
                    ds <- listDatasets(bm)
                    mt <- ds[match(map$dataset, ds$dataset), "version"]
                    if (is.na(mt)) {
                        res <- list(current = genome, set = map$dataset, type = "archive", cause = "error")
                    }
                    if (mt != map$value) {
                        res <- list(
                            current = genome, set = map$value, setArchive = sub(".", " ", map$date, fixed = TRUE),
                            setVersion = map$version, setCurrent = mt, type = "archive", cause = "mismatch"
                        )
                    }
                }
                res
            },
            silent = TRUE
        ))
    })
    res <- res[listLen(res) > 0]
    res[sapply(res, is, "try-error")] <- list(type = "error")
    allFields <- unique(unlist(lapply(res, names)))
    dt <- as.data.frame(do.call(rbind, lapply(res, function(x) t(as.data.frame(unlist(x)[allFields])))),
        stringsAsFactors = FALSE
    )
    rownames(dt) <- NULL
    gens <- ucscGenomes()
    gens$version <- as.numeric(gsub("[a-zA-Z]*", "", gens$db))
    gensS <- split(gens, gsub("[0-9]*$", "", gens$db))
    highestVersion <- as.data.frame(t(sapply(gensS, function(x) unlist(x[which.max(x$version), ]))),
        stringsAsFactors = FALSE
    )
    rownames(gens) <- gens$db
    mt <- match(gsub("[0-9]*$", "", dt$current), rownames(highestVersion))
    tmp <- highestVersion[mt, c("db", "date", "name", "version")]
    colnames(tmp) <- paste("UCSCLatest", colnames(tmp), sep = "_")
    dt2 <- cbind(dt, tmp)
    tmp2 <- gens[dt2$current, c("db", "date", "name", "version")]
    colnames(tmp2) <- paste("UCSC", colnames(tmp2), sep = "_")
    dt3 <- cbind(dt2, tmp2)
    dt3 <- dt3[rowSums(is.na(dt3)) < ncol(dt3), ]
    # if(require(writexl)) {
    #     write_xlsx(list("Sheet1"=dt3), path="ensMappings.xlsx")
    # }
    write.table(dt3, file = "ensMappings.txt", sep = "\t", quote = FALSE)
    return(dt3)
}

# ucsc <- fetchAllGenomesFromUcsc()

# library(tidyverse)
# library(rvest)
# url <- "http://Dec2021.archive.ensembl.org/info/website/archives/assembly.html"
# tab <- url %>%
#     read_html() %>%
#     html_table(na.strings = "")
# tab <- tab[[1]] %>%
#     left_join(tab[[2]], by="")
# colnames(tab)[1] <- "scientificName"
# tab %>%
#     pivot_longer(-scientificName, names_to = "dateversion") %>%
#     dplyr::filter(!is.na(value))
#
#
# url <- "http://dec2021.archive.ensembl.org/info/about/species.html"
# species <- url %>%
#          read_html() %>%
#          html_table(na.strings = c("", "-"))

#' @include RangeTrack-class.R
NULL

## NumericTrack-class --------------------------------------------------------

#' NumericTrack class and methods
#'
#' The virtual parent class for all track items in the Gviz package designed to
#' contain numeric data. This class merely exists for dispatching purpose.
#'
#' @template GdObject-class_slot
#' @template RangeTrack-class_slot
#'
#' @template NumericTrack_param
#'
#' @name NumericTrack-class
#'
#' @return A virtual class: No objects may be created from it.
#'
#' @author Florian Hahne
#'
#' @inherit GdObject-class seealso
#'
#' @exportClass NumericTrack
setClass("NumericTrack",
    representation = representation("VIRTUAL"),
    prototype = prototype(
        name = "NumericTrack",
        dp = DisplayPars()
    ),
    contains = "RangeTrack"
)


## NumericTrack Methods drawAxis ---------------------------------------------

#' @describeIn NumericTrack-class add a y-axis to the title panel of a track.
#' @importFrom grDevices extendrange colorRampPalette
#' @export
setMethod("drawAxis", signature(GdObject = "NumericTrack"), function(GdObject, from, to, ...) {
    type <- match.arg(.dpOrDefault(GdObject, "type", "p"), .PLOT_TYPES, several.ok = TRUE)
    yvals <- values(GdObject)
    ylim <- .dpOrDefault(GdObject, "ylim", if (!is.null(yvals) && length(yvals)) {
        range(yvals, na.rm = TRUE, finite = TRUE)
    } else {
        c(-1, 1)
    })
    if (diff(ylim) == 0) {
        ylim <- ylim + c(-1, 1)
    }
    hSpaceAvail <- vpLocation()$isize["width"] / 6
    yscale <- extendrange(r = ylim, f = 0.05)
    col <- .dpOrDefault(GdObject, "col.axis", "white")
    acex <- .dpOrDefault(GdObject, "cex.axis")
    acol <- .dpOrDefault(GdObject, "col.axis", "white")
    at <- .dpOrDefault(GdObject, "yTicksAt", pretty(yscale))
    at <- at[which(at >= sort(ylim)[1] & at <= sort(ylim)[2])]
    if (!length(at)) {
        at <- sort(ylim)[c(1, 2)]
    }
    if (is.null(acex)) {
        vSpaceNeeded <- max(as.numeric(convertWidth(stringHeight(at), "inches"))) * length(at) * 1.5
        hSpaceNeeded <- max(as.numeric(convertWidth(stringWidth(at), "inches")))
        vSpaceAvail <- abs(diff(range(at))) / abs(diff(yscale)) * vpLocation()$isize["height"]
        acex <- max(0.6, min(vSpaceAvail / vSpaceNeeded, hSpaceAvail / hSpaceNeeded))
    }
    nlevs <- max(1, nlevels(factor(.dpOrDefault(GdObject, "groups"))))
    if (any(type %in% c("heatmap", "horizon")) && .dpOrDefault(GdObject, "showSampleNames", FALSE)) {
        groups <- .dpOrDefault(GdObject, "groups")
        sn <- if (is.null(groups)) rownames(values(GdObject)) else rev(unlist(split(rownames(values(GdObject)), factor(groups))))
        cex.sn <- .dpOrDefault(GdObject, "cex.sampleNames", acex)
        col.cn <- .dpOrDefault(GdObject, "col.sampleNames", "white")
        wd <- max(as.numeric(convertWidth(stringWidth(sn) + unit(10, "points"), "npc"))) * cex.sn
        samNames <- viewport(x = 1, width = wd, just = 1, yscale = c(-0.05, 1.05))
        pushViewport(samNames)
        nr <- nrow(values(GdObject))
        if (nr > 1) {
            yy <- head(seq(0.05, 0.95, len = nr + 1), -1)
            yy <- yy + diff(yy)[[1]] / 2
        } else {
            yy <- 0.5
        }
        grid.text(x = rep(0.5, nr), y = yy, label = rev(sn), just = 0.5, gp = gpar(cex = cex.sn, col = col.cn))
        popViewport(1)
        samAxis <- viewport(x = 1 - wd, width = 1 - wd, just = 1)
        pushViewport(samAxis)
        on.exit(popViewport(1))
    }
    ## if any of the types are gradient or heatmap we want the gradient scale
    if (any(type %in% c("gradient", "heatmap")) && .dpOrDefault(GdObject, "showColorBar", TRUE)) {
        ## viewport to hold the color strip
        shift <- ifelse(all(type %in% c("gradient", "heatmap")), 1, 0)
        pcols <- .getPlottingFeatures(GdObject)
        ncolor <- .dpOrDefault(GdObject, "ncolor", 100)
        vpAxisCont <- viewport(x = unit(1, "npc") - unit(2 - shift, "points"), width = unit(1, "npc") - unit(2 - shift, "points"), just = 1)
        pushViewport(vpAxisCont)
        for (i in seq_len(nlevs)) {
            ## create color palette
            cr <- c("white", pcols$col[i])
            if (nlevs < 2) {
                cr <- .dpOrDefault(GdObject, "gradient", cr)
            }
            palette <- colorRampPalette(cr)(ncolor + 5)[-seq_len(5)]
            pshift <- ifelse(i == nlevs, 1 - shift, 0)
            vpTitleAxis <- viewport(
                x = unit(1, "npc") - unit(4 * (i - 1), "points"), width = unit(4 + pshift, "points"),
                yscale = yscale, just = 1
            )
            pushViewport(vpTitleAxis)
            ## draw a rectangle for each color
            if (all(type %in% c("gradient", "heatmap"))) {
                if (i == nlevs) {
                    suppressWarnings(grid.yaxis(gp = gpar(col = acol, cex = acex), at = at))
                }
                grid.rect(
                    y = unit(seq(ylim[1], ylim[2], length.out = ncolor + 1), "native")[-(ncolor + 1)], x = unit(0, "npc") - unit(1, "points"),
                    width = 1, height = 1 / ncolor, gp = gpar(fill = palette, lty = 0), just = c("left", "bottom")
                )
            } else {
                grid.rect(
                    y = unit(seq(ylim[1], ylim[2], length.out = ncolor + 1), "native")[-(ncolor + 1)], x = 0,
                    width = 1, height = 1 / ncolor, gp = gpar(fill = palette, lty = 0), just = c("left", "bottom")
                )
                if (i == nlevs) {
                    suppressWarnings(grid.yaxis(gp = gpar(col = acol, cex = acex), at = at))
                    grid.lines(x = c(0, 0), y = ylim, gp = gpar(col = acol), default.units = "native")
                }
            }
            popViewport(1)
        }
        popViewport(1)
    } else {
        vpTitleAxis <- viewport(x = 0.95, width = 0.2, yscale = yscale, just = 0)
        pushViewport(vpTitleAxis)
        suppressWarnings(grid.yaxis(gp = gpar(col = acol, cex = acex), at = at))
        grid.lines(x = c(0, 0), y = ylim, gp = gpar(col = acol), default.units = "native")
        popViewport(1)
    }
})

## NumericTrack Methods drawGrid ---------------------------------------------

#' @describeIn NumericTrack-class superpose a grid on top of a track.
setMethod("drawGrid", signature(GdObject = "NumericTrack"), function(GdObject, from, to) {
    if (.dpOrDefault(GdObject, "grid", FALSE)) {
        vals <- score(GdObject)
        ylim <- .dpOrDefault(GdObject, "ylim", range(vals, na.rm = TRUE, finite = TRUE))
        if (diff(ylim)) {
            pushViewport(dataViewport(xData = c(from, to), yData = ylim, extension = c(0, 0.1), clip = TRUE))
            panel.grid(
                h = .dpOrDefault(GdObject, "h", -1), v = .dpOrDefault(GdObject, "v", -1),
                col = .dpOrDefault(GdObject, "col.grid", "#e6e6e6"), lty = .dpOrDefault(GdObject, "lty.grid", 1),
                lwd = .dpOrDefault(GdObject, "lwd.grid", 1)
            )
            popViewport(1)
        }
    }
})

## ----init, echo=FALSE, results='hide'-----------------------------------------
library(knitr)

## 1. issue with too large vignette, maybe optimizing png?
## unfortunately it requires additional system libs -> not feasible
##           `pngquant=""`, `optipng = "-o7"` both in `opts_chunk`
## SOLUTION: might be to use `fig.retina = 1` and only set it to 2 (default)
##           for figures which requires higher resolution
##           or even decrease res `dpi = 72`, also both in `opts_chunk`
## 
## knit_hooks$set(optipng = hook_optipng,
##                pngquant = hook_pngquant)
##
## 2. crop=NULL or FALSE =>  fix vignette rendering based on yihui/knitr#1796
## added also error=FALSE to include_graphics

opts_chunk$set(
  collapse = TRUE,
  tidy = FALSE,
  fig.retina = 1,
  ## comment = "#>",
  error = FALSE,
  warning = FALSE,
  message = FALSE,
  crop = NULL                           
)

## check the output type
out_type <- opts_knit$get("rmarkdown.pandoc.to")
if (is.null(out_type))
    out_type <- "html"

## add styling
if (out_type == "html") {
    BiocStyle::markdown()
} else if (out_type == "latex") {
    BiocStyle::latex()
}

## ----vignetteGvizSetup, echo=FALSE, results='hide'----------------------------
source(system.file("scripts/documentation.R", package="Gviz"))
xtabDetails <- details

addParTable <- function(xtabDetails, class, 
                        skip=c("showTitle", "size", "background.title"), 
                        add=NULL, out_type="html") {
  Parameters <- data.frame("Display Parameter"=names(xtabDetails[[class]]),
                           "Description"=xtabDetails[[class]], 
                           check.names=FALSE)
  if(!is.null(add)) {
    Parameters <- cbind(Parameters, add)
  }
  Parameters <- Parameters[order(Parameters[,1]),]  
  sel <- !Parameters[,1] %in% skip
  Parameters <- Parameters[sel,]
  Parameters[,2] <- gsub("\\\\\\code\\{(.*?)\\}", "`\\1`", Parameters[,2])
  Parameters[,2] <- gsub("\\\\\\link[sS]4[Cc]lass\\{(.*?)\\}", "\\1", 
                         Parameters[,2])
  Parameters[,2] <- gsub("\\\\\\link\\{(.*?)\\}", "\\1", Parameters[,2])
  rownames(Parameters) <- NULL
  if (out_type == "html") {
     out <- kable(Parameters)
  } else if (out_type == "latex") {
     out <- kable(Parameters, format="latex", booktabs=TRUE, longtable=TRUE)
  }
  print(out)
  return(invisible())
}

## hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent=TRUE), "try-error")
hasUcscConnection <- FALSE
## oto <- options(timeout=5)
hasBiomartConnection <- FALSE
## hasBiomartConnection <- (!is(try(download.file("http://www.biomart.org", tempfile(), quiet=TRUE)), "try-error") &&
##                          !is(try(biomaRt::listMarts(), silent=TRUE), "try-error"))
## options(timeout=oto)

## ## clear BiomaRt cache
## biomaRt::biomartCacheClear()

## ## Uncommenting this helps when the UCSC server has a hickup but still lets you connect:
## hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent=TRUE), "try-error") && 
##   !is(try(IdeogramTrack(genome="hg19", chromosome=7), silent=TRUE), "try-error")

## ----loadPackage, cache=FALSE-------------------------------------------------
library(Gviz)

## ----AnnotationTrack----------------------------------------------------------
library(GenomicRanges)
data(cpgIslands)
class(cpgIslands)
chr <- as.character(unique(seqnames(cpgIslands)))
gen <- genome(cpgIslands)
atrack <- AnnotationTrack(cpgIslands, name = "CpG")

## ----plotAnnotationTrack, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(atrack)

## ----GenomeAxisTrack----------------------------------------------------------
gtrack <- GenomeAxisTrack()

## ----plotGenomeAxisTrack, fig.width=7.5, fig.height=1.1-----------------------
plotTracks(list(gtrack, atrack))

## ----showIdeogramTrack, eval=FALSE--------------------------------------------
#  itrack <- IdeogramTrack(genome = gen, chromosome = chr)

## ----doIdeogramTrack, echo=FALSE, results='hide'------------------------------
if(hasUcscConnection) {
  itrack <- IdeogramTrack(genome = gen, chromosome = chr)
} else{
  data(itrack)
}

## ----plotIdeogramTrack, fig.width=7.5, fig.height=1.5-------------------------
plotTracks(list(itrack, gtrack, atrack))

## ----GeneRegionTrack, fig.width=7.5, fig.height=3-----------------------------
data(geneModels)
grtrack <- GeneRegionTrack(geneModels, genome = gen,
                           chromosome = chr, name = "Gene Model")
plotTracks(list(itrack, gtrack, atrack, grtrack))

## ----zooming, fig.width=7.5, fig.height=2.2-----------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack),
           from = 26700000, to = 26750000)

## ----zooming2, fig.width=7.5, fig.height=3------------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack),
           extend.left = 0.5, extend.right = 1000000)

## ----zooming3, fig.width=7.5, fig.height=3------------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack), 
           extend.left = 0.5, extend.right = 1000000, col = NULL)

## ----zooming4, fig.width=7.5, fig.height=3.1----------------------------------
library(BSgenome.Hsapiens.UCSC.hg19)
strack <- SequenceTrack(Hsapiens, chromosome = chr)
plotTracks(list(itrack, gtrack, atrack, grtrack, strack), 
           from = 26591822, to = 26591852, cex = 0.8)

## ----DataTrack, fig.width=7.5, fig.height=4-----------------------------------
set.seed(255)
lim <- c(26700000, 26750000)
coords <- sort(c(lim[1], 
                 sample(seq(from = lim[1], to = lim[2]), 99), 
                 lim[2]))
dat <- runif(100, min = -10, max = 10)
dtrack <- DataTrack(data = dat, start = coords[-length(coords)],
                    end = coords[-1], chromosome = chr, genome = gen, 
                    name = "Uniform")
plotTracks(list(itrack, gtrack, atrack, grtrack, dtrack), 
           from = lim[1], to = lim[2])

## ----DataTrackHist, fig.width=7.5, fig.height=4-------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack, dtrack), 
           from = lim[1], to = lim[2], type = "histogram")

## ----displayPars1f, fig.width=7.5, fig.height=3-------------------------------
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr, 
                           name = "Gene Model",
                           transcriptAnnotation = "symbol",
                           background.title = "brown")
head(displayPars(grtrack))
displayPars(grtrack) <- list(background.panel = "#FFFEDB", col = NULL)
head(displayPars(grtrack))

plotTracks(list(itrack, gtrack, atrack, grtrack))

## ----displayPars2f, fig.width=7.5, fig.height=3-------------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack), 
           background.panel = "#FFFEDB", background.title = "darkblue")

## ----displayPars3-------------------------------------------------------------
dp <- availableDisplayPars(grtrack)
tail(dp)

## ----displayPars4, fig.width=7.5, fig.height=1.5------------------------------
getOption("Gviz.scheme")
scheme <- getScheme()
scheme$GeneRegionTrack$fill <- "salmon"
scheme$GeneRegionTrack$col <- NULL
scheme$GeneRegionTrack$transcriptAnnotation <- "transcript"
addScheme(scheme, "myScheme")
options(Gviz.scheme = "myScheme")
grtrack <- GeneRegionTrack(geneModels, genome = gen,
                           chromosome = chr, name = "Gene Model")
plotTracks(grtrack)
options(Gviz.scheme="default")
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr,
                           name = "Gene Model",
                           transcriptAnnotation = "symbol")

## ----schemes, eval=FALSE------------------------------------------------------
#  .GvizSchemes <- list(myScheme = list(
#    GeneRegionTrack=list(fill = "salmon", col = NULL,
#                         transcriptAnnotation = "transcript")))

## ----plottingdirections, fig.width=7.5, fig.height=3--------------------------
plotTracks(list(itrack, gtrack, atrack, grtrack), reverseStrand = TRUE)

## ----GenomeAxisTrackClass1, fig.width=7.5, fig.height=0.75--------------------
axisTrack <- GenomeAxisTrack()
plotTracks(axisTrack, from = 1e6, to = 9e6)

## ----GenomeAxisTrackClass2, fig.width=7.5, fig.height=0.75--------------------
axisTrack <- GenomeAxisTrack(range=IRanges(start = c(2e6, 4e6),
                                           end = c(3e6, 7e6),
                                           names = rep("N-stretch", 2)))
plotTracks(axisTrack, from = 1e6, to = 9e6)

## ----GenomeAxisTrackClass2a, fig.width=7.5, fig.height=0.75-------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, showId = TRUE)

## ----GenomeAxisTrackClass3, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, add53 = TRUE, add35 = TRUE)

## ----GenomeAxisTrackClass4, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, add53 = TRUE,
           add35 = TRUE, littleTicks = TRUE)

## ----GenomeAxisTrackClass5, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, exponent = 4)

## ----GenomeAxisTrackClass6, fig.width=7.5, fig.height=0.75--------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, labelPos = "below")

## ----GenomeAxisTrackClass7, fig.width=7.5, fig.height=0.5---------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, scale = 0.5)

## ----GenomeAxisTrackClass8, fig.width=7.5, fig.height=0.5---------------------
plotTracks(axisTrack, from = 1e6, to = 9e6, scale = 0.5, 
           labelPos = "below")

## ----GenomeAxisTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"GenomeAxisTrack", out_type = out_type)

## ----IdeogramTrackClass1Show, eval=FALSE--------------------------------------
#  ideoTrack <- IdeogramTrack(genome = "hg19", chromosome = "chrX")
#  plotTracks(ideoTrack, from = 85e6, to = 129e6)

## ----IdeogramTrackClass1Do, fig.width=7.5, fig.height=0.5, echo=FALSE, results='hide'----
if(hasUcscConnection) {
  ideoTrack <- IdeogramTrack(genome = "hg19", chromosome = "chrX")
} else{
  data(ideoTrack)
}
plotTracks(ideoTrack, from = 85e6, to = 129e6)

## ----IdeogramTrackClass2, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE)

## ----IdeogramTrackClass3, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE, 
           showBandId = TRUE, cex.bands = 0.5)

## ----IdeogramTrackClass4, fig.width=7.5, fig.height=0.5-----------------------
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE,
           centromereShape = "circle")

## ----IdeogramTrackClassTable, echo=FALSE, results='asis'----------------------
addParTable(xtabDetails, "IdeogramTrack", out_type = out_type)

## ----DataClass1, fig.width=7.5, fig.height=1.5--------------------------------
data(twoGroups)
dTrack <- DataTrack(twoGroups, name = "uniform")
plotTracks(dTrack)

## ----types, echo=FALSE, results='asis'----------------------------------------
types <- data.frame(Value=c("p", "l", "b", "a", "s", "S", "g", "r", "h", "confint", "smooth", "histogram", "mountain", "polygon", "boxplot", "gradient", "heatmap", "horizon"),
                    Type=c("dot plot", "lines plot", "dot and lines plot", "lines plot of average (i.e., mean) values", "stair steps (horizontal first)",
                           "stair steps (vertical first)", "add grid lines", "add linear regression line", "histogram lines", "confidence intervals for average values", "add loess curve",
                           "histogram (bar width equal to range with)", "'mountain-type' plot relative to a baseline",
                           "'polygon-type' plot relative to a baseline", "box and whisker plot",
                           "false color image of the summarized values", "false color image of the individual values",
                           "Horizon plot indicating magnitude and direction of a change relative to a baseline"))

if (out_type == "html") {
  kable(types)
} else if (out_type == "latex") {
  kable(types, "latex", booktabs = TRUE, longtable = TRUE)
}

## ----typePlots, fig.width=7.5, fig.height=8.5, echo=FALSE, results='hide'-----
pushViewport(viewport(layout = grid.layout(nrow = 9, ncol = 2)))
i <- 1
for(t in types$Value) {
  pushViewport(viewport(layout.pos.col = ((i - 1) %% 2) + 1,
                        layout.pos.row = ((i - 1) %/% 2)+1))
    if(t != "horizon"){
        names(dTrack) <- t
        plotTracks(dTrack, type = t, add = TRUE, 
                   cex.title = 0.8, margin = 0.5)
    } else {
        data(dtHoriz)
        names(dtHoriz) <- "horizon *"
        plotTracks(dtHoriz[8, ], type = "horizon", add = TRUE, 
                   cex.title = 0.8, margin = 0.5, 
                   showAxis = FALSE, horizon.origin = 0.7)
    }
    i <- i + 1
    popViewport(1)
}
popViewport(1)
names(dTrack) <- "uniform"

## ----mutitype, results='hide', fig.width=7.5, fig.height=1.5------------------
plotTracks(dTrack, type = c("boxplot", "a", "g"))

## ----sampNames, fig.width=7.5, fig.height=1.5---------------------------------
colnames(mcols(twoGroups))
plotTracks(dTrack, type = c("heatmap"), showSampleNames = TRUE, 
           cex.sampleNames = 0.6)

## ----grouping, results='hide', fig.width=7.5, fig.height=1.5------------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3),
           type = c("a", "p", "confint"))

## ----typeGroupedPlots, fig.width=7.5, fig.height=6, echo=FALSE, results='hide'----
pushViewport(viewport(layout=grid.layout(nrow=9, ncol=1)))
i <- 1
for(t in c("a", "s", "confint", "smooth", "histogram", "boxplot", "heatmap", "horizon", "hor. type")) {
  pushViewport(viewport(layout.pos.col=((i-1)%%1)+1, layout.pos.row=((i-1)%/%1)+1))
  names(dTrack) <- t
  if(!t %in% c("horizon", "hor. type")) {
    plotTracks(dTrack, type=t, add=TRUE, cex.title=0.8, groups=rep(1:2, each=3), margin=0.5)
  } else if (t == "horizon") {
    plotTracks(dtHoriz[c(1,8),], type="horizon", add=TRUE, cex.title=0.8, margin=0.5, showAxis=FALSE, horizon.origin=0.3,
               groups=1:2)
  } else {
    plotTracks(dTrack, type="histogram", stackedBars=FALSE, add=TRUE, cex.title=0.8, groups=rep(1:2, each=3), margin=0.5)
  }
  i <- i+1
  popViewport(1)
}
popViewport(1)
names(dTrack) <- "uniform"

## ----groupingLegend, results='hide', fig.width=7.5, fig.height=1.5------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3),
           type = c("a", "p"), legend = TRUE)

## ----horizLegend, results='hide', fig.width=7.5, fig.height=1.5---------------
data(dtHoriz)
dtHoriz <- dtHoriz[1:6, ]
plotTracks(dtHoriz, type = "horiz", groups = rownames(values(dtHoriz)),
           showSampleNames = TRUE, cex.sampleNames = 0.6, separator = 1)

## ----filedt1, fig.width=7.5, fig.height=1-------------------------------------
bgFile <- system.file("extdata/test.bedGraph", package = "Gviz")
dTrack2 <- DataTrack(range = bgFile, genome = "hg19", type = "l", 
                     chromosome = "chr19", name = "bedGraph")
class(dTrack2)
plotTracks(dTrack2)

## ----filedt2------------------------------------------------------------------
library(rtracklayer)
dTrack3 <-  DataTrack(range = bgFile, genome = "hg19", type = "l", 
                      chromosome = "chr19", name = "bedGraph",
                      importFunction = function(file) import(con=file))
identical(dTrack2, dTrack3)

## ----filedt3, fig.width=7.5, fig.height=1-------------------------------------
bamFile <- system.file("extdata/test.bam", package = "Gviz")
dTrack4 <- DataTrack(range = bamFile, genome = "hg19", type = "l", 
                     name = "Coverage", window = -1, 
                     chromosome = "chr1")
class(dTrack4)
dTrack4
plotTracks(dTrack4, from = 189990000, to = 190000000)

## ----filedt4, fig.width=7.5, fig.height=1-------------------------------------
plotTracks(dTrack4, chromosome = "chr1", from = 189891483, to = 190087517)

## ----filedt5------------------------------------------------------------------
myImportFun <- function(file, selection){
    ## do something here
}
DataTrack(range = bamFile, genome = "hg19", type = "l",
          name = "Coverage", window = -1, chromosome = "chr1",
          importFunction = myImportFun, stream = TRUE)

## ----biggerdata, results='hide', fig.width=7.5, fig.height=1.5----------------
dat <- sin(seq(pi, 10*pi, len=500))
dTrack.big <- DataTrack(start = seq(1, 100000, len = 500), width = 15, 
                        chromosome = "chrX", genome = "hg19", 
                        name = "sinus",
                        data = sin(seq(pi, 5 * pi, len = 500)) * 
                          runif(500, 0.5, 1.5))
plotTracks(dTrack.big, type="hist")

## ----aggregation, results='hide', fig.width=7.5, fig.height=1.5---------------
plotTracks(dTrack.big, type = "hist", window = 50)

## ----aggregation2, results='hide', fig.width=7.5, fig.height=1.5--------------
plotTracks(dTrack.big, type = "hist", window = -1, windowSize = 2500)

## ----transformation, results='hide', fig.width=7.5, fig.height=1.5------------
plotTracks(dTrack.big, type = "l", 
           transformation = function(x) { x[x < 0] <- 0; x })

## ----groupingAv1, results='hide', fig.width=7.5, fig.height=1.5---------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3), 
           type = c("b"), aggregateGroups = TRUE)

## ----groupingAv2, results='hide', fig.width=7.5, fig.height=1.5---------------
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3), 
           type = c("b"), aggregateGroups = TRUE, aggregation = "max")

## ----DataTrackClassTable, echo=FALSE, results='asis'--------------------------
addParTable(xtabDetails,"DataTrack", out_type = out_type)

## ----anntrack1, results='hide', fig.width=7.5, fig.height=0.5-----------------
aTrack <- AnnotationTrack(start = c(10, 40, 120), width = 15, 
                          chromosome = "chrX", 
                          strand = c("+", "*", "-"),
                          id = c("Huey", "Dewey", "Louie"), 
                          genome = "hg19", name = "foo")
plotTracks(aTrack)

## ----anntrack2, results='hide', fig.width=7.5, fig.height=0.5-----------------
plotTracks(aTrack, shape = "box", featureAnnotation = "id")

## ----anntrack3, results='hide', fig.width=7.5, fig.height=0.5-----------------
plotTracks(aTrack, shape = "ellipse", featureAnnotation = "id", 
           fontcolor.feature = "darkblue")

## ----anntrack4f, results='hide', fig.width=7.5, fig.height=0.5----------------
aTrack.groups <- AnnotationTrack(start = c(50, 180, 260, 460, 860, 1240), 
                                 width = c(15, 20, 40, 100, 200, 20),
                                 chromosome = "chrX",
                                 strand = rep(c("+", "*", "-"), 
                                              c(1, 3, 2)),
                                 group = rep(c("Huey", "Dewey", "Louie"), 
                                             c(1, 3, 2)),
                                 genome = "hg19", name = "foo")
plotTracks(aTrack.groups, groupAnnotation = "group")

## ----anntrack4af, results='hide', fig.width=7.5, fig.height=0.5---------------
plotTracks(aTrack.groups, groupAnnotation = "group", 
           just.group = "right")

## ----anntrack4bf, results='hide', fig.width=7.5, fig.height=0.5---------------
plotTracks(aTrack.groups, groupAnnotation = "group", 
           just.group = "above")

## ----stacking1, results='hide', fig.width=7.5, fig.height=0.5-----------------
aTrack.stacked <- AnnotationTrack(start = c(50, 180, 260, 800, 600, 1240),
                                  width = c(15, 20, 40, 100, 500, 20),
                                  chromosome = "chrX",
                                  strand = "*",
                                  group = rep(c("Huey", "Dewey", "Louie"), 
                                              c(1, 3, 2)),
                                  genome = "hg19", name = "foo")
plotTracks(aTrack.stacked, groupAnnotation="group")

## ----stacking2, results='hide', fig.width=7.5, fig.height=0.5-----------------
plotTracks(aTrack.stacked, stacking = "dense")

## ----features-----------------------------------------------------------------
feature(aTrack.stacked)
feature(aTrack.stacked) <- c("foo", "bar", "bar", "bar", "no", "no")

## ----featuresIdPlot, results='hide', fig.width=7.5, fig.height=0.5------------
plotTracks(aTrack.stacked, featureAnnotation = "feature", 
           groupAnnotation = "feature", fontcolor.feature = 1, 
           cex.feature = 0.7)

## ----featuresPlotf, results='hide', fig.width=7.5, fig.height=0.5-------------
plotTracks(aTrack.stacked, groupAnnotation = "group", 
           foo = "darkred", bar = "darkgreen")

## ----overplotting, results='hide', fig.width=7.5, fig.height=0.75-------------
data("denseAnnTrack")
plotTracks(denseAnnTrack, showOverplotting = TRUE)

## ----collapse1f, results='hide', fig.width=7.5, fig.height=0.85---------------
data(collapseTrack)
plotTracks(ctrack)

## ----collapse2f, results='hide', fig.width=7.5, fig.height=0.85---------------
plotTracks(ctrack, min.width = 1)

## ----collapse3f, results='hide', fig.width=7.5, fig.height=0.85---------------
plotTracks(ctrack, min.width = 1, collapse = TRUE)

## ----collapse4f, results='hide', fig.width=7.5, fig.height=0.85---------------
plotTracks(ctrack, min.width = 3, min.distance = 5, collapse = TRUE)

## ----collapse5f, results='hide', fig.width=7.5, fig.height=0.65---------------
plotTracks(ctrack, min.width = 3, min.distance = 5, collapse = TRUE,
           mergeGroups = TRUE, extend.left = 0.1)

## ----fileat1, fig.width=7.5, fig.height=1-------------------------------------
aTrack2 <- AnnotationTrack(range = bamFile, genome = "hg19", 
                           name = "Reads", chromosome = "chr1")
class(aTrack2)
aTrack2
plotTracks(aTrack2, from = 189995000, to = 190000000)

## ----fileat2, fig.width=7.5, fig.height=1-------------------------------------
aTrack3 <- AnnotationTrack(range = bamFile, genome = "hg19", 
                           name = "Reads", chromosome = "chr1", 
                           group = "id")
aTrack3
plotTracks(aTrack3, from = 189995000, to = 190000000)

## ----fileat3------------------------------------------------------------------
availableDefaultMapping(bamFile, "AnnotationTrack")

## ----fileat4, fig.width=7.5, fig.height=2-------------------------------------
plotTracks(list(dTrack4, aTrack2), from = 189990000, to = 190000000)

## ----AnnotationTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"AnnotationTrack", out_type = out_type)

## ----generegtrackf, results='hide', fig.width=7.5, fig.height=1.5-------------
data(geneModels)
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr, 
                           name = "foo")
head(gene(grtrack))
head(transcript(grtrack))
head(exon(grtrack))
head(symbol(grtrack))
plotTracks(grtrack)

## ----generegtrack2af, results='hide', fig.width=7.5, fig.height=2.5-----------
plotTracks(grtrack, transcriptAnnotation = "symbol")

## ----generegtrack2bf, results='hide', fig.width=7.5, fig.height=2.5-----------
plotTracks(grtrack, transcriptAnnotation = "transcript")

## ----generegtrack2cf, results='hide', fig.width=7.5, fig.height=1-------------
plotTracks(grtrack, exonAnnotation = "exon", extend.left = -0.8, 
           fontcolor.exon = 1)

## ----generegtrack3f, results='hide', fig.width=7.5, fig.height=1--------------
plotTracks(grtrack, collapseTranscripts = TRUE, shape = "arrow", 
           transcriptAnnotation = "symbol")

## ----generegtrack3g, results='hide', fig.width=7.5, fig.height=1--------------
plotTracks(grtrack, collapseTranscripts = "longest", shape = "arrow", 
           transcriptAnnotation = "symbol")

## ----generegtrack3h, results='hide', fig.width=7.5, fig.height=1--------------
plotTracks(grtrack, collapseTranscripts = "meta", shape = "arrow", 
           transcriptAnnotation = "symbol")

## ----tdb2grt1-----------------------------------------------------------------
library(GenomicFeatures)
samplefile <- system.file("extdata", "hg19_knownGene_sample.sqlite", 
                          package = "GenomicFeatures")
txdb <- loadDb(samplefile)
GeneRegionTrack(txdb)

## ----tdb2grt2-----------------------------------------------------------------
txTr <- GeneRegionTrack(txdb, chromosome = "chr6", 
                        start = 35000000,  end = 40000000)

## ----generegtrack4f, results='hide', fig.width=7.5, fig.height=1--------------
feature(txTr)
plotTracks(txTr)

## ----generegtrack4g, eval=FALSE, echo=FALSE, results='hide', fig.width=7.5, fig.height=1----
#  library(org.Hs.eg.db)
#  symbol(txTr) <- mapIds(org.Hs.eg.db,
#                         keys=sub("\\.\\d+$", "", gene(txTr)),
#                         keytype="ENSEMBL", column="SYMBOL")
#  symbol(txTr) <- ifelse(is.na(symbol(txTr)), gene(txTr), symbol(txTr))
#  plotTracks(txTr, transcriptAnnotation = "symbol")

## ----ensDb, eval=FALSE--------------------------------------------------------
#  library(ensembldb)
#  library(EnsDb.Hsapiens.v75)
#  
#  edb <- EnsDb.Hsapiens.v75
#  seqlevelsStyle(edb) <- "UCSC"
#  
#  eTrack <- GeneRegionTrack(edb, chromosome = "chr6",
#                            start = 35000000, end = 40000000)
#  
#  plotTracks(eTrack)

## ----GeneRegionTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"GeneRegionTrack", out_type = out_type)

## ----BiomartGeneRegionTrackShow, eval=FALSE-----------------------------------
#  library(biomaRt)
#  bm <- useEnsembl(host = "https://grch37.ensembl.org",
#                biomart = "ENSEMBL_MART_ENSEMBL",
#                dataset = "hsapiens_gene_ensembl")
#  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr,
#                                      start = 20e6, end = 21e6,
#                                      name = "ENSEMBL", biomart = bm)
#  plotTracks(biomTrack)

## ----BiomartGeneRegionTrackDo, echo=FALSE, results='hide', fig.width=7.5, fig.height=1.25----
library(biomaRt)
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org", 
                biomart = "ENSEMBL_MART_ENSEMBL", 
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                      start = 20e6, end = 21e6,
                                      name = "ENSEMBL", biomart = bm)
} else {
    data(biomTrack)
    biomTrack <- as(biomTrack, "GeneRegionTrack")
}
plotTracks(biomTrack)

## ----BiomartGeneRegionTrackCol, fig.width=7.5, fig.height=1.25----------------
plotTracks(biomTrack, col.line = NULL, col = NULL)

## ----BiomartGeneRegionTrackHeight, fig.width=7.5, fig.height=1.25-------------
plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)

## ----BiomartGeneRegionTrackFilterShow, eval=FALSE-----------------------------
#  bm <- useEnsembl(host = "https://grch37.ensembl.org",
#                biomart = "ENSEMBL_MART_ENSEMBL",
#                dataset = "hsapiens_gene_ensembl")
#  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr,
#                                      start = 20e6, end = 21e6,
#                                      name = "ENSEMBL",
#                                      filter = list(with_refseq_mrna=TRUE),
#                                      biomart = bm)
#  plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)

## ----BiomartGeneRegionTrackFilterDo, fig.width=7.5, fig.height=1.25, echo=FALSE, results='hide'----
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org",
                biomart = "ENSEMBL_MART_ENSEMBL",
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                      start = 20e6, end = 21e6,
                                      name = "ENSEMBL",
                                      filter = list(with_refseq_mrna=TRUE), 
                                      biomart = bm)
  plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)
} else {
  data(biomTrack2)
  biomTrack2 <- as(biomTrack2, "GeneRegionTrack")
  plotTracks(biomTrack2, col.line = NULL, col = NULL, stackHeight = 0.3)
}

## ----BiomartGeneRegionTrackSymbolShow, eval=FALSE-----------------------------
#  bm <- useEnsembl(host = "https://grch37.ensembl.org",
#                biomart = "ENSEMBL_MART_ENSEMBL",
#                dataset = "hsapiens_gene_ensembl")
#  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", name = "ENSEMBL",
#                                      symbol = "ABCB5", biomart = bm)
#  plotTracks(biomTrack, transcriptAnnotation = "symbol")

## ----BiomartGeneRegionTrackSymbolDo, fig.width=7.5, fig.height=1.25, echo=FALSE, results='hide'----
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org", 
                biomart = "ENSEMBL_MART_ENSEMBL", 
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", name = "ENSEMBL", 
                                      symbol = "ABCB5", biomart = bm)
} else {
    ranges(biomTrack) <- ranges(biomTrack)[symbol(biomTrack) == "ABCB5"]
}
plotTracks(biomTrack, transcriptAnnotation="symbol")

## ----BiomartGeneRegionTrackCustom, fig.width=7.5, fig.height=1.25, eval=FALSE, echo=FALSE----
#  library(biomaRt)
#  bm <- useEnsembl(host="dec2012.archive.ensembl.org", biomart="ENSEMBL_MART_ENSEMBL",
#                dataset="hsapiens_gene_ensembl")
#  fm <- Gviz:::.getBMFeatureMap()
#  fm[["symbol"]] <- "external_gene_id"
#  biomTrack <- BiomartGeneRegionTrack(genome="hg19", chromosome="chr7", start=20e6, end=21e6,name="ENSEMBL",
#                                      featureMap=fm, biomart=bm)
#  plotTracks(biomTrack, col.line=NULL, col=NULL, stackHeight=0.3)

## ----BiomartGeneRegionTrackClassTable, echo=FALSE, results='asis'-------------
addInfo <- t(data.frame(displayPars(biomTrack, names(details[["BiomartGeneRegionTrack"]]))))
colnames(addInfo) <- "Color"
addParTable(xtabDetails,"BiomartGeneRegionTrack", add=addInfo, out_type = out_type)

## ----DetailsAnnotationTrack1--------------------------------------------------
library(GenomicRanges)
probes <- GRanges(seqnames = "chr7", ranges = IRanges(
      start = c(2000000, 2070000, 2100000, 2160000),
      end = c(2050000, 2130000, 2150000, 2170000)), 
  strand = c("-", "+", "-", "-"))

## ----DetailsAnnotationTrack2--------------------------------------------------
methylation <- matrix(c(rgamma(400, 1)),
                  ncol = 100,
                  dimnames = list(paste("probe", 1:4, sep = ""), NULL))
methylation[, 51:100] <- methylation[, 51:100] + 0:3
sgroups <- rep(c("grp1", "grp2"), each = 50)

## ----DetailsAnnotationTrack3--------------------------------------------------
library(lattice)
details <- function(identifier, ...) {
  d <- data.frame(signal = methylation[identifier, ], group = sgroups)
  print(densityplot(~signal, group = group, data = d, 
                    main = list(label = identifier, cex = 0.7),
                    scales = list(draw = FALSE, x = list(draw = TRUE)), 
                    ylab = "", xlab = ""), 
        newpage = FALSE, prefix = "plot")
}

## ----DetailsAnnotationTrack4, results='hide', fig.width=7.5, fig.height=5-----
deTrack <- AnnotationTrack(range = probes, genome = "hg19", 
                           chromosome = 7, id = rownames(methylation),
                           name = "probe details", stacking = "squish",
                           fun = details)
plotTracks(deTrack)

## ----DetailsAnnotationTrack5--------------------------------------------------
selFun <- function(identifier, start, end, track, GdObject, ...){
    gcount <- table(group(GdObject))
    ## This computes the width of 2 pixels in genomic coordinates
    pxRange <- Gviz:::.pxResolution(min.width = 20, coord = "x")
    return((end - start) < pxRange && gcount[identifier] == 1)
}

## ----DetailsAnnotationTrack6--------------------------------------------------
detFun <- function(identifier, GdObject.original, ...){
  plotTracks(list(GenomeAxisTrack(scale = 0.3, size = 0.2, cex = 0.7), 
        GdObject.original[group(GdObject.original) == identifier]),
        add = TRUE, showTitle = FALSE)
}

## ----DetailsAnnotationTrack7f, results='hide', fig.width=7.5, fig.height=2----
data(geneDetails)
deTrack2 <- AnnotationTrack(geneDetails, fun = detFun, 
                            selectFun = selFun,
                            groupDetails = TRUE, details.size = 0.5, 
                            detailsConnector.cex = 0.5, 
                            detailsConnector.lty = "dotted",
                            shape = c("smallArrow", "arrow"), 
                            groupAnnotation = "group")
plotTracks(deTrack2, extend.left = 90000)

## ----DetailsAnnotationTrack8, results='hide', fig.width=7.5, fig.height=3-----
plotTracks(deTrack, details.size = 0.75, detailsConnector.pch = NA, 
           detailsConnector.col = "darkred", 
           detailsBorder.fill = "#FFE3BF", 
           detailsBorder.col = "darkred", shape ="box", 
           detailsConnector.lty = "dotted")

## ----DetailsAnnotationTrackClassTableSec, echo=FALSE, results='asis'----------
addParTable(xtabDetails,"DetailsAnnotationTrack", out_type = out_type)

## ----SequenceTrack1-----------------------------------------------------------
library(BSgenome.Hsapiens.UCSC.hg19)
sTrack <- SequenceTrack(Hsapiens)
sTrack

## ----SequenceTrack2, results='hide', fig.width=7.5, fig.height=0.5------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050)

## ----SequenceTrack3, results='hide', fig.width=7.5, fig.height=0.5------------
fcol <- c(A="darkgray", C="darkgray", T="darkgray", G="darkgray")
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050, 
           fontcolor = fcol)

## ----SequenceTrack4, results='hide', fig.width=7.5, fig.height=0.5------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050,
           add53 = TRUE)

## ----SequenceTrack5, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050, 
           add53 = TRUE, complement = TRUE)

## ----SequenceTrack6, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20100)

## ----SequenceTrack7, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 201000)

## ----SequenceTrack8, results='hide', fig.width=7.5, fig.height=0.3------------
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20100, cex = 0.5)

## ----SequenceTrackClassTableSec, echo=FALSE, results='asis'-------------------
addParTable(xtabDetails,"SequenceTrack", out_type = out_type)

## ----alignmentstrack_1_do, echo=FALSE, results='hide'-------------------------
afrom <- 2960000
ato <- 3160000
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "gapped.bam"), 
  isPaired = TRUE)
data(alTrackGenes)

## ----alignmentstrack_1_show, eval=FALSE---------------------------------------
#  afrom <- 2960000
#  ato <- 3160000
#  alTrack <- AlignmentsTrack(
#    system.file(package = "Gviz", "extdata", "gapped.bam"),
#    isPaired = TRUE)
#  bmt <- BiomartGeneRegionTrack(genome = "hg19", chromosome = "chr12",
#                                start = afrom, end = ato,
#                                filter = list(with_ox_refseq_mrna = TRUE),
#                                stacking = "dense")

## ----alignmentstrack_2, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom, to = ato, 
           chromosome = "chr12")

## ----alignmentstrack_3, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom, to = ato,
           chromosome = "chr12", min.height = 0, 
           coverageHeight = 0.08, minCoverageHeight = 0)

## ----alignmentstrack_4, results='hide', fig.width=7.5, fig.height=2-----------
plotTracks(c(alTrack, bmt), from = afrom, to = ato, 
           chromosome = "chr12", type = "coverage")

## ----alignmentstrack_5, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom + 12700, 
           to = afrom + 15200, chromosome = "chr12")

## ----alignmentstrack_5_1, results='hide', fig.width=7.5, fig.height=2---------
plotTracks(c(bmt, alTrack), from = afrom + 12700, 
           to = afrom + 15200, chromosome = "chr12",
           type = c("coverage", "sashimi"))

## ----alignmentstrack_5_2, results='hide', fig.width=7.5, fig.height=2---------
introns <- GRanges("chr12", IRanges(start = c(2973662, 2973919),
                                    end = c(2973848, 2974520)))
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12", type = c("coverage", "sashimi"), 
           sashimiFilter = introns)

## ----alignmentstrack_5_3, results='hide', fig.width=7.5, fig.height=2---------
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12", type = c("coverage", "sashimi"), 
           sashimiFilter = introns, sashimiFilterTolerance = 5L)

## ----alignmentstrack_6, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200,
           chromosome = "chr12", reverseStacking = TRUE,
           col.mates = "purple", col.gap = "orange", type = "pileup")

## ----alignmentstrack_6_1, results='hide', fig.width=7.5, fig.height=5---------
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "gapped.bam"), 
  isPaired = FALSE)
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12")

## ----alignmentstrack_7, results='hide', fig.width=7.5, fig.height=5-----------
afrom <- 44945200
ato <- 44947200
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "snps.bam"), isPaired = TRUE)
plotTracks(alTrack, chromosome = "chr21", from = afrom, to = ato)

## ----alignmentstrack_8, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(alTrack, sTrack), chromosome = "chr21", 
           from = afrom, to = ato)

## ----alignmentstrack_9, results='hide', fig.width=7.5, fig.height=5-----------
plotTracks(c(alTrack, sTrack), chromosome = "chr21",
           from = 44946590, to = 44946660)

## ----alignmentstrack_10, results='hide', fig.width=7.5, fig.height=5----------
plotTracks(c(alTrack, sTrack), chromosome = "chr21", from = 44946590, 
           to = 44946660, cex = 0.5, min.height = 8)

## ----alignmentstrack_11, results='hide', fig.width=7.5, fig.height=5----------
indelTrack1 <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "indels.bam"), 
  name = "Standard")
indelTrack2 <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "indels.bam"), 
  showIndels=TRUE, name="Indels")
plotTracks(c(indelTrack1, indelTrack2), 
           chromosome = "chr2", from = 126442000, to = 126453000)

## ----AlignmentsTrackClassTable, echo=FALSE, results='asis'--------------------
addParTable(xtabDetails,"AlignmentsTrack", out_type = out_type)

## ----UCSC1, echo=FALSE,fig.cap="A screen shot of a UCSC genome browser view around the FMR1 locus on the mouse chromosome."----
include_graphics("ucsc1.png")

## ----UCSC2, echo=FALSE, fig.cap="A screen shot of a UCSC table browser view on the UCSC Known Genes track."----
include_graphics("ucsc2.png")

## ----ucscTrack1, eval=FALSE---------------------------------------------------
#  from <- 65921878
#  to <- 65980988
#  knownGenes <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                          track = "UCSC Genes", table="knownGene",
#                          from = from, to = to,
#                          trackType = "GeneRegionTrack",
#                          rstarts = "exonStarts", rends = "exonEnds",
#                          gene = "name", symbol = "name",
#                          transcript = "name", strand = "strand",
#                          fill = "#8282d2", name = "UCSC Genes")

## ----ucscTrack2, eval=FALSE---------------------------------------------------
#  refGenes <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                        track="Other RefSeq", table = "xenoRefGene",
#                        from = from, to = to,
#                        trackType = "GeneRegionTrack",
#                        rstarts = "exonStarts", rends = "exonEnds",
#                        gene = "name",  symbol = "name2",
#                        transcript = "name", strand = "strand",
#                        fill = "#8282d2", stacking = "dense",
#                        name = "Other RefSeq")
#  
#  ensGenes <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                        track="Ensembl Genes", table = "ensGene",
#                        from = from, to = to,
#                        trackType = "GeneRegionTrack",
#                        rstarts = "exonStarts", rends = "exonEnds",
#                        gene = "name", symbol = "name2",
#                        transcript = "name", strand = "strand",
#                        fill = "#960000", name = "Ensembl Genes")

## ----ucscTrack3, eval=FALSE---------------------------------------------------
#  cpgIslands <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                          track="CpG Islands", table = "cpgIslandExt",
#                          from = from, to = to,
#                          trackType = "AnnotationTrack",
#                          start = "chromStart", end = "chromEnd",
#                          id = "name", shape = "box", fill = "#006400",
#                          name = "CpG Islands")
#  
#  snpLocations <-  UcscTrack(genome = "mm9", chromosome = "chrX",
#                             track="SNPs (128)", table = "snp128",
#                             from = from, to = to,
#                             trackType = "AnnotationTrack",
#                             start = "chromStart", end = "chromEnd",
#                             id = "name", feature = "func",
#                             strand = "strand", shape = "box",
#                             stacking = "dense", fill = "black",
#                             name = "SNPs")

## ----ucscTrack4, eval=FALSE---------------------------------------------------
#  conservation <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                            track = "Conservation",
#                            table = "phyloP30wayPlacental",
#                            from = from, to = to, trackType = "DataTrack",
#                            start = "start", end = "end", data = "value",
#                            type = "hist", window = "auto",
#                            col.histogram = "darkblue",
#                            fill.histogram = "darkblue",
#                            ylim = c(-3.7, 4), name = "Conservation")
#  
#  gcContent <- UcscTrack(genome = "mm9", chromosome = "chrX",
#                         track = "GC Percent", table = "gc5Base",
#                         from = from, to = to, trackType = "DataTrack",
#                         start = "start", end = "end", data = "value",
#                         type = "hist", window = -1, windowSize = 1500,
#                         fill.histogram = "black", col.histogram = "black",
#                         ylim = c(30, 70), name = "GC Percent")

## ----ucscTrack5, eval=FALSE---------------------------------------------------
#  axTrack <- GenomeAxisTrack()
#  idxTrack <- IdeogramTrack(genome="mm9", chromosome="chrX")

## ----ucscTrackLoad, echo=FALSE, results='hide'--------------------------------
data(ucscItems)

## ----ucscTrack6, results='hide', fig.width=7.5, fig.height=5.5----------------
plotTracks(list(idxTrack, axTrack, knownGenes, refGenes, ensGenes, 
                cpgIslands, gcContent, conservation, snpLocations), 
           from = from, to = to, showTitle = FALSE)

## ----HighlightTrack, fig.width=7.5, fig.height=4------------------------------
ht <- HighlightTrack(trackList = list(atrack, grtrack, dtrack),
                     start = c(26705000, 26720000), width = 7000,
                     chromosome = 7)
plotTracks(list(itrack, gtrack, ht), from = lim[1], to = lim[2])

## ----HighlightTrack2, fig.width=7.5, fig.height=4-----------------------------
ht1 <- HighlightTrack(trackList=list(itrack, gtrack, atrack), 
                      start = c(26705000, 26720000), width = 7000, 
                      chromosome = 7)
ht2 <- HighlightTrack(trackList = dtrack, start = c(26705000, 26720000),
                      width = 7000, chromosome = 7)
plotTracks(list(ht1, grtrack, ht2), from = lim[1], to = lim[2])

## ----HighlightTrackClassTable, echo=FALSE, results='asis'---------------------
addParTable(xtabDetails,"HighlightTrack", out_type = out_type)

## ----OverlayTrack, fig.width=7.5, fig.height=3--------------------------------
dat <- runif(100, min = -2, max = 22)
dtrack2 <- DataTrack(data = dat, start = coords[-length(coords)], 
                end = coords[-1], chromosome = chr, genome = gen, 
                name = "Uniform2",  groups = factor("sample 2", 
                  levels = c("sample 1", "sample 2")), legend = TRUE)
displayPars(dtrack) <- list(groups = factor("sample 1", 
              levels = c("sample 1", "sample 2")), legend = TRUE)
ot <- OverlayTrack(trackList=list(dtrack2, dtrack))
ylims <- extendrange(range(c(values(dtrack), values(dtrack2))))
plotTracks(list(itrack, gtrack, ot), from = lim[1], to = lim[2], 
           ylim = ylims, type = c("smooth", "p"))

## ----OverlayTrack2, fig.width=7.5, fig.height=3-------------------------------
displayPars(dtrack) <- list(alpha.title = 1, alpha = 0.5)
displayPars(dtrack2) <- list(alpha.title = 1, alpha = 0.5)
ot <- OverlayTrack(trackList = list(dtrack, dtrack2))
plotTracks(list(itrack, gtrack, ot), from = lim[1], to = lim[2], 
           ylim = ylims, type = c("hist"), window = 30)

## ----multPlot1----------------------------------------------------------------
chroms <- c("chr1", "chr2", "chr3", "chr4")
maTrack <- AnnotationTrack(range=GRanges(seqnames = chroms, 
    ranges = IRanges(start = 1,  width = c(100, 400, 200,1000)),
    strand = c("+", "+", "-", "+")), genome = "mm9", 
    chromosome = "chr1", name = "foo")

mdTrack <- DataTrack(
  range = GRanges(seqnames = rep(chroms, c(10, 40, 20, 100)),
                  ranges = IRanges(start = c(seq(1, 100, len = 10),
                                             seq(1, 400, len = 40), 
                                             seq(1, 200, len = 20),
                                             seq(1, 1000, len = 100)), 
                                   width = 9), values = runif(170)),
  data = "values", chromosome = "chr1", genome = "mm9", name = "bar")

## ----multPlot2----------------------------------------------------------------
mgTrack <- GenomeAxisTrack(scale = 50, labelPos = "below", exponent = 3)
chromosome(itrack) <- "chr1"

## ----multPlot3, results='hide', fig.width=7.5, fig.height=4-------------------
ncols <- 2
nrows <- length(chroms) %/% ncols
grid.newpage()
pushViewport(viewport(layout = grid.layout(nrows, ncols)))
for(i in seq_along(chroms)) {
  pushViewport(viewport(layout.pos.col = ((i - 1) %% ncols) + 1,
                        layout.pos.row = (((i) - 1) %/% ncols) + 1))
    plotTracks(list(itrack, maTrack, mdTrack, mgTrack), 
               chromosome = chroms[i], add = TRUE)
    popViewport(1)
}

## ----multPlot4, results='hide', fig.width=7.5, fig.height=4-------------------
library(lattice)
chroms <- data.frame(chromosome = chroms)
xyplot(1 ~ chromosome | chromosome, data = chroms, panel = function(x) {
  plotTracks(list(itrack , maTrack, mdTrack, mgTrack), 
             chromosome = x, add = TRUE, showId = FALSE) },
  scales = list(draw = FALSE), xlab = NULL, ylab = NULL)

## ----biocStruct1, echo=FALSE, results='asis'----------------------------------
dt <- data.frame(`Gviz class` = rep(c("AnnotationTrack","GeneRegionTrack","DataTrack","SequenceTrack"), c(4,5,3,2)),
                 `Bioconductor class`=c("data.frame","IRanges","GRanges","GRangesList",
                                        "data.frame","IRanges","GRanges","GRangesList","TxDb",
                                        "data.frame","IRanges","GRanges",
                                        "DNAStringSet", "BSgenome"),
                 `Method`=c("Constructor","Constructor + additional arguments","Constructor or setAs method, additional data in metadata columns","Constructor or setAs method",
                            "Constructor","Constructor + additional arguments","Constructor or setAs method, additional data in metadata columns","Constructor or setAs method, additional data in metadata columns", "Constructor or setAs method",
                             "Constructor","Constructor + additional data matrix","Constructor or setAs method, numeric data in metadata columns",
                            "DNAStringSet & Constructor", "Constructor"), 
                 stringsAsFactors = FALSE, check.names=FALSE)
dt[duplicated(dt[,1]),1] <- "" 
if (out_type == "html") {
  kable(dt)
} else if (out_type == "latex") {
  kable(dt, "latex", booktabs=TRUE, longtable=TRUE)
}

## ----biocStruct2, echo=FALSE, results='asis'----------------------------------
dt <- data.frame(`Gviz class`=rep(c("AnnotationTrack","GeneRegionTrack","DataTrack","SequenceTrack","AlignmentsTrack"), c(5, 4, 4, 2, 1)),
                 `File type`=c("BED","GFF","GFF2","GFF3","BAM",
                               "GTF","GFF","GFF2","GFF3",
                               "BedGraph","WIG","BigWig","BAM",
                               "FASTA","2Bit",
                               "BAM"),
                 `Extension`=c(".bed",".gff, .gff1, ",".gff2",".gff3",".bam",
                               ".gtf",".gff, .gff1",".gff2",".gff3",
                               ".bedGraph",".wig",".bigWig",".bam",
                               ".fa, .fasta",".2bit",
                               ".bam"),
                 `Streaming`=c("no","no","no","no","YES",
                               "no","no","no","no",
                               "no","no","YES","YES",
                               "YES","YES",
                               "YES"),
                 `Details`=c("Genomic locations from the mandatory `chrom`, `chromStart` and `chromEnd` fields, and optionally the strand from the strand field. If present, the information in the field is mapped to track item ids, and is mapped to track item feature type. All other fields are currently ignored.", "Only the following basic GFF fields are recognized: `seqname`, `start`, `end`, `strand`, (mapped to track item feature type) and `group` (to allow for track item grouping).", "Same as above, but feature grouping information may be provided either as `Group` or `Parent` attribute. Feature ids are mapped to one of the ID, Name or Alias attributes.", "Same as above, but feature grouping information has to be provided as the `Parent` attribute.", "Only start and end locations as well as the strand information for the reads are used. Read identifiers are used for track item grouping.", 
                             "A somewhat looser format definition for `gtf` files is applied here where gene, transcript and exon identifiers and names can be parsed from the `gene_id`, `gene_name`, `transcript_id`, `transcript_name`, `exon_id`, or `exon_id attributes`", "This only supports very limited item grouping and thus complete gene models can not be properly encoded.", "In most instances this is identical to the `GTF` standard and it could make sense to rename the file accordingly.", "The gene-to-transcript and transcript-to- exon relationships are encoded in the parent and `type` attributes and the parser tries to accommodate most of the existing `GFF3` variants.",
                             "", "", "", "Read coverage only is extracted from the bam file.",
                             "Streaming only possible if an index file is found in the same directory as the original fasta file.", "",
                             "Always needs an index file is found in the same directory as the original `BAM` file."),
                 stringsAsFactors = FALSE, check.names=FALSE)
dt[duplicated(dt[,1]),1] <- "" 
if (out_type == "html") {
  kable(dt)
} else if (out_type == "latex") {
  kable(dt, "latex", booktabs=TRUE, longtable=TRUE)
}

## ----session-info-------------------------------------------------------------
sessionInfo()

## UcscTrack -----------------------------------------------------------------
##
## Strictly speaking this is not a class, but rather some sort of meta-constructor for several of the previously
## defined track types directly from UCSC data. It will fetch online data from a particular track (or a sub-table of
## a track) and feed it to one of the original constructors with user-provided argument mappings.
## ---------------------------------------------------------------------------
## Constructor. The following arguments are supported:
##    o track: a character of one of the available UCSC tracks
##    o table: a character of one one of the sub-tables of the track, or NULL to fetch all
##    o trackType: a character giving the name of the constructor to pass the data to, one in
##       c("AnnotationTrack", "GeneRegionTrack", "DataTrack", "GenomeAxisTrack")
##    o genome, chromosome: the reference genome and active chromosome for the track.
##    o from: the starting and end coordinates of the track data
##    o name: the name of the track. This will be used for the title panel.
## All additional items in ... are being treated as DisplayParameters

## A simple caching mechanism for UCSC session information. The overhead for establishing a connection to UCSC is
## quite significant and we can shave off 5 to 10 seconds here by caching sessions and associated information
## for a particular genome and chromosome.
.ucscCache <- new.env()
.ensemblCache <- new.env()
.martCache <- new.env()

#' @importFrom rtracklayer ucscGenomes browserSession
#' @importMethodsFrom rtracklayer chrom close getTable "tableName<-" track
#' ucscTableQuery trackNames tableNames import import.gff import.gff1
#' import.gff2 import.gff3 import.2bit import.bed15 import.bw import.ucsc
#' import.bed import.bedGraph import.chain import.wig seqinfo
#'
#' @export
.doCache <- function(token, expression, env, callEnv = environment()) {
    if (!token %in% base::ls(env)) {
        res <- eval(expression, envir = callEnv)
        assign(x = token, value = res, envir = env)
        res
    } else {
        env[[token]]
    }
}
.cacheTracks <- function(genome, chromosome, track, env = .ucscCache) {
    genomes <- .doCache("availableGenomes", expression(rtracklayer::ucscGenomes()), env)
    if (!genome %in% as.character(genomes[, "db"])) {
        stop("'", genome, "' is not a valid UCSC genome.")
    }
    sessionToken <- paste("session", genome, sep = "_")
    tracksToken <- paste("tracks", genome, sep = "_")
    tablesToken <- paste("tables", track, genome, sep = "_")
    cenv <- environment()
    session <- .doCache(
        sessionToken,
        expression({
            myUcscUrl <- getOption("Gviz.ucscUrl")
            tmp <- if (is.null(myUcscUrl)) browserSession() else browserSession(url = myUcscUrl)
            genome(tmp) <- genome
            tmp
        }), env, cenv
    )
    availTracks <- .doCache(tracksToken, expression(trackNames(ucscTableQuery(session))), env, cenv)
    track <- match.arg(track, sort(c(availTracks, names(availTracks))))
    if (!is.na(availTracks[track])) {
        track <- names(availTracks[track])
    }
    availTables <- .doCache(tablesToken, expression({
        query <- ucscTableQuery(session, track)
        sort(tableNames(query))
    }), env, cenv)
    chrInfo <- seqlengths(session)
    return(list(
        session = session, availTracks = availTracks, availTables = availTables, track = track,
        chrInfo = chrInfo
    ))
}
.cacheGenomes <- function(genome = NULL, env = .ucscCache) {
    availToken <- "availableGenomes"
    genomesToken <- paste("genomeBands", genome, sep = "_")
    genomes <- .doCache(availToken, expression(rtracklayer::ucscGenomes()), env)
    bands <- NULL
    if (!is.null(genome)) {
        cenv <- environment()
        bands <- .doCache(genomesToken, expression({
            if (!genome %in% as.character(genomes[, "db"])) {
                stop("'", genome, "' is not a valid UCSC genome.")
            }
            sessionToken <- paste("session", genome, sep = "_")
            session <- .doCache(
                sessionToken,
                expression({
                    myUcscUrl <- getOption("Gviz.ucscUrl")
                    tmp <- if (is.null(myUcscUrl)) browserSession() else browserSession(url = myUcscUrl)
                    genome(tmp) <- genome
                    tmp
                }), env, cenv
            )
            query <- tryCatch(ucscTableQuery(session, table = "cytoBandIdeo"), error = function(e) {
                warning(
                    "There doesn't seem to be any cytoband data available for genome '", genome,
                    "' at UCSC or the service is temporarily down. Trying to fetch the chromosome length data."
                )
                tryCatch(ucscTableQuery(session, table = "chromInfo"), error = function(e) {
                    stop(
                        "There doesn't seem to be any chromosome length data available for genome '", genome,
                        "' at UCSC or the service is temporarily down."
                    )
                })
            })
            out <- getTable(query)
            if (all(c("chrom", "size") %in% colnames(out))) {
                out <- data.frame(chrom = out$chrom, chromStart = 0, chromEnd = out$size, name = "", gieStain = "gneg", stringsAsFactors = FALSE)
            }
            out
        }), env, cenv)
    }

    return(list(availableGenomes = genomes, bands = bands))
}
.cacheMartData <- function(bmtrack, chromosome = NULL, staged = FALSE) {
    uid <- .bmGuid(bmtrack)
    req <- if (!is.null(bmtrack@start) && !is.null(bmtrack@end)) GRanges(seqnames = chromosome[1], IRanges(start = bmtrack@start, bmtrack@end)) else NULL
    if (is.null(chromosome) || is.null(.martCache[[uid]])) {
        data <- .fetchBMData(bmtrack, chromosome, staged)
        if (!is.null(req)) {
            .martCache[[uid]] <- list(data = data, ranges = req)
        } else {
            req <- range(data)
        }
        if (length(data) && .dpOrDefault(bmtrack, "verbose", FALSE)) {
            message("Loaded data from Biomart for region ", paste(sprintf("%s:%i-%i(%s)", seqnames(req), start(req), end(req), strand(req)), collapse = " and "))
        }
    } else {
        rr <- .martCache[[uid]][["ranges"]]
        dd <- .martCache[[uid]][["data"]]
        if (!is.null(req) && suppressWarnings(req %within% rr)) {
            genes <- unique(subsetByOverlaps(dd, req)$gene)
            data <- dd[seqnames(dd) == chromosome[1] & dd$gene %in% genes]
            if (.dpOrDefault(bmtrack, "verbose", FALSE)) {
                message(sprintf("Retrieved data from cache for region %s:%i-%i(%s)", chromosome, start(req), end(req), strand(req)))
            }
        } else {
            data <- .fetchBMData(bmtrack, chromosome, staged)
            if (is.null(req)) {
                req <- range(data)
            }
            .martCache[[uid]][["data"]] <- suppressWarnings(c(.martCache[[uid]][["data"]], data[!(seqnames(data) == chromosome[1] & data$gene %in% dd$gene)]))
            .martCache[[uid]][["ranges"]] <- suppressWarnings(union(rr, req))
            if (length(req) && .dpOrDefault(bmtrack, "verbose", FALSE)) {
                message("Loaded data from Biomart for region ", paste(sprintf("%s:%i-%i(%s)", seqnames(req), start(req), end(req), strand(req)), collapse = " and "))
            }
        }
    }
    return(data)
}

## empty the session cache
#' @export
clearSessionCache <- function() {
    assignInNamespace(".ucscCache", new.env(), ns = "Gviz")
    assignInNamespace(".ensemblCache", new.env(), ns = "Gviz")
    assignInNamespace(".martCache", new.env(), ns = "Gviz")
}


## Constructor


#' Meta-constructor for Gviz tracks fetched directly from the various
#' UCSC data sources.
#'
#'
#' The UCSC data base provides a wealth of annotation information. This
#' function can be used to access UCSC, to retrieve the data available there
#' and to return it as an annotation track object amenable to plotting with
#' \code{\link{plotTracks}}.
#'
#' \code{clearSessionCache} is can be called to remove all cached items from
#' the session which are generated when connecting with the UCSC data base.
#'
#'
#' The data stored at the UCSC data bases can be of different formats: gene or
#' transcript model data, simple annotation features like CpG Island locations
#' or SNPs, or numeric data like conservation or mapability. This function
#' presents a unified API to download all kinds of data and to map them back to
#' one of the annotation track objects defined in this package. The type of
#' object to hold the data has to be given in the \code{trackType} argument,
#' and subsequently the function passes all data on to the respective object
#' constructor. All additional named arguments are considered to be relevant
#' for the constructor of choice, and single character scalars are replaced by
#' the respective data columns in the downloaded UCSC tables if available. For
#' instance, assuming the table for track 'foo' contains the columns 'id',
#' 'type', 'fromLoc' and 'toLoc', giving the feature identifier, type, start
#' end end location. In order to create an \code{\linkS4class{AnnotationTrack}}
#' object from that data, we have to pass the additional named arguments
#' \code{id="id"}, \code{feature="type"}, \code{start="fromLoc"} and
#' codeend="toLoc" to the \code{UcscTrack} function. The complete function call
#' could look like this:
#'
#' \code{UcscTrack(track="foo", genome="mm9", chromosome=3, from=1000,
#' to=10000, trackType="AnnotationTrack", id="id", feature="type",
#' start="from", end="to")}
#'
#' To reduce the bandwidth, some caching of the UCSC connection takes place. In
#' order to remove these cached session items, call \code{clearSessionCache}.
#'
#' The \code{Gviz.ucscUrl} option controls which URL is being used to connect
#' to UCSC. For instance, one could switch to the European UCSC mirror by
#' calling \code{options(Gviz.ucscUrl="http://genome-euro.ucsc.edu/cgi-bin/"}.
#'
#' @aliases UcscTrack clearSessionCache
#' @param track Character, the name of the track to fetch from UCSC. To find
#' out about available tracks please consult the online table browser at
#' \url{http://genome.ucsc.edu/cgi-bin/hgTables?command=start}.
#' @param table Character, the name of the table to fetch from UCSC, or
#' \code{NULL}, in which case the default selection of tables is used. To find
#' out about available tables for a given track please consult the online table
#' browser at \url{http://genome.ucsc.edu/cgi-bin/hgTables?command=start}.
#' @param trackType Character, one in \code{c("AnnotationTrack",
#' "GeneRegionTrack", "DataTrack", "GenomeAxisTrack")}. The function will try
#' to coerce the downloaded data in an object of this class. See below for
#' details.
#' @param genome Character, a valid USCS genome identifier for which to fetch
#' the data.
#' @param chromosome Character, a valid USCS character identifier for which to
#' fetch the data.
#' @param name Character, the name to use for the resulting track object.
#' @param from,to A range of genomic locations for which to fetch data.
#' @param \dots All additional named arguments are expected to be either
#' display parameters for the resulting objects, or character scalars of column
#' names in the downloaded UCSC data tables that are matched by name to
#' available arguments in the respective constructor functions as defined by
#' the \code{trackType} argument. See Details section for more information.
#' @return
#'
#' An annotation track object as determined by \code{trackType}.
#' @author Florian Hahne
#' @seealso
#'
#' \code{\linkS4class{AnnotationTrack}}
#'
#' \code{\linkS4class{DataTrack}}
#'
#' \code{\linkS4class{GeneRegionTrack}}
#'
#' \code{\linkS4class{GenomeAxisTrack}}
#'
#' \code{\link{plotTracks}}
#' @examples
#' \dontrun{
#'
#' ## Create UcscTrack for Known Genes from mm9 genome
#' from <- 65921878
#' to <- 65980988
#' knownGenes <- UcscTrack(
#'     genome = "mm9", chromosome = "chrX", track = "knownGene",
#'     from = from, to = to, trackType = "GeneRegionTrack",
#'     rstarts = "exonStarts", rends = "exonEnds", gene = "name",
#'     symbol = "name", transcript = "name", strand = "strand",
#'     fill = "#8282d2", name = "UCSC Genes"
#' )
#' }
#'
#' ## if the UCSC is not accessible load prepared object
#' data(ucscItems)
#'
#' ## knownGenes is essentially GeneRegionTrack
#' knownGenes
#'
#' ## plotting
#' plotTracks(knownGenes, chromosome = "chrX", from = 65920688, to = 65960068)
#' @export
UcscTrack <- function(track, table = NULL,
                      trackType = c(
                          "AnnotationTrack", "GeneRegionTrack",
                          "DataTrack", "GenomeAxisTrack"
                      ),
                      genome, chromosome, name = NULL, from, to, ...) {
    trackType <- match.arg(trackType)
    if (missing(genome) || !isSingleString(genome)) stop("Need to specify genome for creating a UcscTrack")
    if (missing(chromosome)) stop("Need to specify chromosome for creating a UcscTrack")
    chromosome <- .chrName(chromosome)[1]
    sessionInfo <- .cacheTracks(genome = genome, chromosome = chromosome, track = track, env = .ucscCache)
    if (missing(from)) {
        from <- 1
    }
    if (missing(to)) {
        to <- sessionInfo$chrInfo[chromosome]
    }
    gr <- GRanges(ranges = IRanges(start = from, end = to), seqnames = chromosome)
    suppressWarnings(genome(gr) <- unname(genome))[1]
    query <- ucscTableQuery(sessionInfo$session, sessionInfo$track, gr)
    if (!is.null(table)) {
        table <- match.arg(table, sessionInfo$availTables)
        tableName(query) <- table
    }
    if (is.null(name)) {
        name <- if (is.null(table)) track else paste(sessionInfo$track, table)
    }
    tableDat <- if (trackType == "DataTrack") {
        tmp <- try(track(query), silent = TRUE)
        if (is(tmp, "try-error")) {
            warning(tmp)
            data.frame()
        } else {
            as.data.frame(tmp)
        }
    } else {
        tmp <- try(getTable(query), silent = TRUE)
        if (is(tmp, "try-error")) {
            warning(tmp)
            data.frame()
        } else {
            tmp
        }
    }
    if (is(tmp, "try-error") && nrow(tableDat) == 0) {
        stop("Error fetching data from UCSC")
    }
    args <- lapply(list(...), function(x) {
        if (is.character(x) && length(x) == 1) {
            if (!x %in% colnames(tableDat)) x else unlist(tableDat[, x])
        } else {
            x
        }
    })
    if (trackType == "GeneRegionTrack") {
        args$start <- from
        args$end <- to
    }
    args <- lapply(args, function(x) if (!length(x)) NULL else x)
    trackObject <- do.call(trackType, args = c(list(chromosome = chromosome, genome = genome, name = name), args))
    return(trackObject)
}

#' Dynamic content based on the available resolution
#'
#' When plotting features linearly along genomic coordinates one frequently
#' runs into the problem of too little resolution to adequately display all
#' details. Most genome browsers try to reasonably reduce the amount of detail
#' that is shown based on the current zoom level.
#'
#' Most track classes in this package define an internal `collapseTrack`
#' method which tries to adjust the plotted content to the available
#' resolution, aims at reducing over-plotting and prevents rendering issues,
#' e.g. when lines are too thin to be plotted. This feature can be toggled on
#' or off using the `collapse` display parameter (see
#' `settings` for details on setting these parameters).
#'
#' In the simplest case (for `AnnotationTrack` objects) this
#' involves expanding all shown features to a minimum pixel width and height
#' (using display parameters `min.width` and `min.heigh`) and
#' collapsing overlapping annotation items (as defined by the parameter
#' `min.distance` into one single item to prevent over-plotting.
#'
#' For objects of class `DataTrack`, the data values
#' underlying collapsed regions will be summarized based on the `summary`
#' display parameter. See the class' documentation for more details.
#'
#' @name collapsing
#'
#' @seealso
#' \code{\linkS4class{AnnotationTrack}}
#'
#' \code{\linkS4class{DataTrack}}
#'
#' \code{\link{settings}}
NULL

---
title: "The Gviz User Guide"
bibliography: Gviz-refs.bib
author: 
  - name: "Florian Hahne"
    affiliation: 
    - Novartis Institute for Biomedical Research, Basel, Switzerland
    email: florian.hahne@novartis.com
  - name: "Robert Ivánek"
    affiliation: 
    - Department of Biomedicine, University of Basel, Basel, Switzerland
    - Swiss Institute of Bioinformatics, Basel, Switzerland
    email: robert.ivanek@unibas.ch
package: "`r BiocStyle::pkg_ver('Gviz')`"
output:
  BiocStyle::html_document:
    toc_float: true
  BiocStyle::pdf_document: 
    default
vignette: >
    %\VignetteIndexEntry{The Gviz User Guide}
    %\VignetteKeywords{visualization, genomics, grid}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
editor_options:
  chunk_output_type: 
    console
---

__Last edited__: 19 May 2020

```{r init, echo=FALSE, results='hide'}
library(knitr)

## 1. issue with too large vignette, maybe optimizing png?
## unfortunately it requires additional system libs -> not feasible
##           `pngquant=""`, `optipng = "-o7"` both in `opts_chunk`
## SOLUTION: might be to use `fig.retina = 1` and only set it to 2 (default)
##           for figures which requires higher resolution
##           or even decrease res `dpi = 72`, also both in `opts_chunk`
## 
## knit_hooks$set(optipng = hook_optipng,
##                pngquant = hook_pngquant)
##
## 2. crop=NULL or FALSE =>  fix vignette rendering based on yihui/knitr#1796
## added also error=FALSE to include_graphics

opts_chunk$set(
  collapse = TRUE,
  tidy = FALSE,
  fig.retina = 1,
  ## comment = "#>",
  error = FALSE,
  warning = FALSE,
  message = FALSE,
  crop = NULL                           
)

## check the output type
out_type <- opts_knit$get("rmarkdown.pandoc.to")
if (is.null(out_type))
    out_type <- "html"

## add styling
if (out_type == "html") {
    BiocStyle::markdown()
} else if (out_type == "latex") {
    BiocStyle::latex()
}
```

```{r vignetteGvizSetup, echo=FALSE, results='hide'}
source(system.file("scripts/documentation.R", package="Gviz"))
xtabDetails <- details

addParTable <- function(xtabDetails, class, 
                        skip=c("showTitle", "size", "background.title"), 
                        add=NULL, out_type="html") {
  Parameters <- data.frame("Display Parameter"=names(xtabDetails[[class]]),
                           "Description"=xtabDetails[[class]], 
                           check.names=FALSE)
  if(!is.null(add)) {
    Parameters <- cbind(Parameters, add)
  }
  Parameters <- Parameters[order(Parameters[,1]),]  
  sel <- !Parameters[,1] %in% skip
  Parameters <- Parameters[sel,]
  Parameters[,2] <- gsub("\\\\\\code\\{(.*?)\\}", "`\\1`", Parameters[,2])
  Parameters[,2] <- gsub("\\\\\\link[sS]4[Cc]lass\\{(.*?)\\}", "\\1", 
                         Parameters[,2])
  Parameters[,2] <- gsub("\\\\\\link\\{(.*?)\\}", "\\1", Parameters[,2])
  rownames(Parameters) <- NULL
  if (out_type == "html") {
     out <- kable(Parameters)
  } else if (out_type == "latex") {
     out <- kable(Parameters, format="latex", booktabs=TRUE, longtable=TRUE)
  }
  print(out)
  return(invisible())
}

## hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent=TRUE), "try-error")
hasUcscConnection <- FALSE
## oto <- options(timeout=5)
hasBiomartConnection <- FALSE
## hasBiomartConnection <- (!is(try(download.file("http://www.biomart.org", tempfile(), quiet=TRUE)), "try-error") &&
##                          !is(try(biomaRt::listMarts(), silent=TRUE), "try-error"))
## options(timeout=oto)

## ## clear BiomaRt cache
## biomaRt::biomartCacheClear()

## ## Uncommenting this helps when the UCSC server has a hickup but still lets you connect:
## hasUcscConnection <- !is(try(rtracklayer::browserSession(), silent=TRUE), "try-error") && 
##   !is(try(IdeogramTrack(genome="hg19", chromosome=7), silent=TRUE), "try-error")
```

# Introduction

In order to make sense of genomic data one often aims to plot such
data in a genome browser, along with a variety of genomic annotation
features, such as gene or transcript models, CpG island, repeat
regions, and so on. These features may either be extracted from public
data bases like ENSEMBL or UCSC, or they may be generated or curated
in-house. Many of the currently available genome browsers do a
reasonable job in displaying genome annotation data, and there are
options to connect to some of them from within `R` (e.g., using the
`r Biocpkg("rtracklayer")` package). However, none of these solutions
offer the flexibility of the full `R` graphics system to display large
numeric data in a multitude of different ways. The `r Biocpkg("Gviz")`
package [@Hahne2016] aims to close this gap by providing a structured
visualization framework to plot any type of data along genomic
coordinates. It is loosely based on the `r Biocpkg("GenomeGraphs")`
package by Steffen Durinck and James Bullard, however the complete
class hierarchy as well as all the plotting methods have been
restructured in order to increase performance and flexibility. All
plotting is done using the grid graphics system, and several
specialized annotation classes allow to integrate publicly available
genomic annotation data from sources like
[UCSC](https://genome.ucsc.edu/) or
[ENSEMBL](https://www.ensembl.org/). 


# Basic Features

The fundamental concept behind the `r Biocpkg("Gviz")` package is similar to the
approach taken by most genome browsers, in that individual types of
genomic features or data are represented by separate tracks. Within
the package, each track constitutes a single object inheriting from
class `GdObject`, and there are constructor functions as well
as a broad range of methods to interact with and to plot these
tracks. When combining multiple objects, the individual tracks will
always share the same genomic coordinate system, thus taking the
burden of aligning the plot elements from the user.

It is worth mentioning that, at a given time, tracks in the sense of the
`r Biocpkg("Gviz")` package are only defined for a single chromosome on a specific 
genome, at least for the duration of a given plotting operation. You will later see
that a track may still contain information for multiple chromosomes,
however most of this is hidden except for the currently active
chromosome, and the user will have to explicitly switch the chromosome
to access the inactive parts. While the package in principle imposes
no fixed structure on the chromosome or on the genome names, it makes
sense to stick to a standardized naming paradigm, in particular when
fetching additional annotation information from online resources. By
default this is enforced by a global option
`ucscChromosomeNames`, which is set during package loading and
which causes the package to check all supplied chromosome names for
validity in the sense of the UCSC definition (chromosomes have to
start with the `chr` string). You may decide to turn this feature
off by calling `options(ucscChromosomeNames=FALSE)`. For the
remainder of this vignette however, we will make use of the UCSC
genome and chromosome identifiers, e.g., the `chr7` chromosome
on the mouse `mm9` genome.

The different track classes will be described in more detail in the
\@ref(track-classes) section further below. For now, let's just
take a look at a typical `r Biocpkg("Gviz")` session to get an idea of what this is
all about. We begin our presentation of the available functionality by
loading the package:

```{r loadPackage, cache=FALSE}
library(Gviz)
```

The most simple genomic features consist of start and stop
coordinates, possibly overlapping each other. CpG islands or
microarray probes are real life examples for this class of
features. In the Bioconductor world those are most often represented
as run-length encoded vectors, for instance in the `IRanges`
and `GRanges` classes. To seamlessly integrate with other
Bioconductor packages, we can use the same data structures to generate
our track objects. A sample set of CpG island coordinates has been
saved in the `cpgIslands` object and we can use that for our
first annotation track object. The constructor function
`AnnotationTrack` is a convenient helper to create the
object.

```{r AnnotationTrack}
library(GenomicRanges)
data(cpgIslands)
class(cpgIslands)
chr <- as.character(unique(seqnames(cpgIslands)))
gen <- genome(cpgIslands)
atrack <- AnnotationTrack(cpgIslands, name = "CpG")
```

Please note that the `AnnotationTrack` constructor (as most
constructors in this package) is fairly flexible and can accommodate
many different types of inputs. For instance, the start and end
coordinates of the annotation features could be passed in as
individual arguments `start` and `end`, as a
`data.frame` or even as an `IRanges` or
`GRangesList` object. Furthermore, a whole bunch of coercion
methods are available for those package users that prefer the more
traditional R coding paradigm, and they should allow operations along
the lines of `as(obj, 'AnnotationTrack')`. You may want to
consult the class' manual page for more information, or take a look at
the \@ref(bioconductor-integration) section for a listing of the
most common data structures and their respective counterparts in the
`r Biocpkg("Gviz")` package.

With our first track object being created we may now proceed to the
plotting. There is a single function `plotTracks` that
handles all of this. As we will learn in the remainder of this
vignette, `plotTracks` is quite powerful and has a number of
very useful additional arguments. For now we will keep things very
simple and just plot the single CpG islands annotation track.

```{r plotAnnotationTrack, fig.width=7.5, fig.height=0.5}
plotTracks(atrack)
```

As you can see, the resulting graph is not particularly
spectacular. There is a title region showing the track's name on a
gray background on the left side of the plot and a data region showing
the seven individual CpG islands on the right. This structure is
similar for all the available track objects classes and it somewhat
mimics the layout of the popular UCSC Genome Browser. If you are not
happy with the default settings, the `r Biocpkg("Gviz")`
package offers a multitude of options to fine-tune the track
appearance, which will be shown in the \@ref(plotting-parameters)
section.

Apart from the relative distance of the CpG islands, this
visualization does not tell us much. One obvious next step would be to
indicate the genomic coordinates we are currently looking at in order
to provide some reference. For this purpose, the `r Biocpkg("Gviz")` package offers
the `GenomeAxisTrack` class. Objects from the class can be
created using the constructor function of the same name.

```{r GenomeAxisTrack}
gtrack <- GenomeAxisTrack()
```

Since a `GenomeAxisTrack` object is always relative to the
other tracks that are plotted, there is little need for additional
arguments. Essentially, the object just tells the
`plotTracks` function to add a genomic axis to the
plot. Nonetheless, it represent a separate annotation track just as
the CpG island track does. We can pass this additional track on to
`plotTracks` in the form of a list.

```{r plotGenomeAxisTrack, fig.width=7.5, fig.height=1.1}
plotTracks(list(gtrack, atrack))
```

You may have realized that the genomic axis does not take up half of
the available vertical plotting space, but only uses the space
necessary to fit the axis and labels. Also the title region for this
track is empty. In general, the `r Biocpkg("Gviz")` package tries to find reasonable
defaults for all the parameters controlling the look and feel of a
plots so that appealing visualizations can be created without too much
tinkering. However, all features on the plot including the relative
track sizes can also be adjusted manually.

As mentioned before in the beginning of this vignette, a plotted track
is always defined for exactly one chromosome on a particular
genome. We can include this information in our plot by means of a
chromosome ideogram. An ideogram is a simplified visual representation
of a chromosome, with the different chromosomal staining bands
indicated by color, and the centromere (if present) indicated by the
shape. The necessary information to produce this visualization is
stored in online data repositories, for instance at UCSC. The `r Biocpkg("Gviz")`
package offers very convenient connections to some of these
repositories, and the `IdeogramTrack` constructor function is
one example for such a connection. With just the information about a
valid UCSC genome and chromosome, we can directly fetch the chromosome
ideogram information and construct a dedicated track object that can
be visualized by `plotTracks`. Please not that you will need
an established internet connection for this to work, and that fetching
data from UCSC can take quite a long time, depending on the server
load. The `r Biocpkg("Gviz")` package tries to cache as much data as possible to
reduce the bandwidth in future queries.

```{r showIdeogramTrack, eval=FALSE}
itrack <- IdeogramTrack(genome = gen, chromosome = chr)
```

```{r doIdeogramTrack, echo=FALSE, results='hide'}
if(hasUcscConnection) {
  itrack <- IdeogramTrack(genome = gen, chromosome = chr)
} else{
  data(itrack)
}
```

Similar to the previous examples, we stick the additional track object
into a list in order to plot it.

```{r plotIdeogramTrack, fig.width=7.5, fig.height=1.5}
plotTracks(list(itrack, gtrack, atrack))
```

Ideogram tracks are the one exception in all of `r Biocpkg("Gviz")`'s track objects
in the sense that they are not really displayed on the same coordinate
system like all the other tracks. Instead, the current genomic
location is indicated on the chromosome by a red box (or, as in this
case, a red line if the width is too small to fit a box).

So far we have only looked at very basic annotation features and how
to give a point of reference to our plots. Naturally, we also want to
be able to handle more complex genomic features, such as gene
models. One potential use case would be to utilize gene model
information from an existing local source. Alternatively, we could
download such data from one of the available online resources like UCSC
or ENSEBML, and there are constructor functions to handle these
tasks. For this example we are going to load gene model data from a
stored `data.frame`. The track class of choice here is a
`GeneRegionTrack` object, which can be created via the
constructor function of the same name. Similar to the
`AnnotationTrack` constructor there are multiple possible ways
to pass in the data.

```{r GeneRegionTrack, fig.width=7.5, fig.height=3}
data(geneModels)
grtrack <- GeneRegionTrack(geneModels, genome = gen,
                           chromosome = chr, name = "Gene Model")
plotTracks(list(itrack, gtrack, atrack, grtrack))
```

In all those previous examples the plotted genomic range has been
determined automatically from the input tracks. Unless told otherwise,
the package will always display the region from the leftmost item to
the rightmost item in any of the tracks.  Of course such a static view
on a chromosomal region is of rather limited use. We often want to
zoom in or out on a particular plotting region to see more details or
to get a broader overview. To that end, `plotTracks`
supports the `from` and `to` arguments that let us
choose an arbitrary genomic range to plot.

```{r zooming, fig.width=7.5, fig.height=2.2}
plotTracks(list(itrack, gtrack, atrack, grtrack),
           from = 26700000, to = 26750000)
```

Another pair of arguments that controls the zoom state are
`extend.left` and `extend.right`. Rather than `from` and `to`, 
those arguments are relative to the
currently displayed ranges, and can be used to quickly extend the view
on one or both ends of the plot. In addition to positive or negative
absolute integer values one can also provide a float value between -1
and 1 which will be interpreted as a zoom factor, i.e., a value of 0.5
will cause zooming in to half the currently displayed range.

```{r zooming2, fig.width=7.5, fig.height=3}
plotTracks(list(itrack, gtrack, atrack, grtrack),
           extend.left = 0.5, extend.right = 1000000)
```

You may have noticed that the layout of the gene model track has
changed depending on the zoom level. This is a feature of the `r Biocpkg("Gviz")`
package, which automatically tries to find the optimal visualization
settings to make best use of the available space. At the same time,
when features on a track are too close together to be plotted as
separate items with the current device resolution, the package will
try to reasonably merge them in order to avoid overplotting.

Often individual ranges on a plot tend to grow quite narrow, in
particular when zooming far out, and a couple of tweaks become helpful
in order to get nice plots, for instance to drop the bounding borders
of the exons.

```{r zooming3, fig.width=7.5, fig.height=3}
plotTracks(list(itrack, gtrack, atrack, grtrack), 
           extend.left = 0.5, extend.right = 1000000, col = NULL)
```

When zooming further in it may become interesting to take a look at
the actual genomic sequence at a given position, and the `r Biocpkg("Gviz")` package
provides the track class `SequenceTrack` that let's you do just
that. Among several other options it can draw the necessary sequence
information from one of the `BSgenome` packages.

```{r zooming4, fig.width=7.5, fig.height=3.1}
library(BSgenome.Hsapiens.UCSC.hg19)
strack <- SequenceTrack(Hsapiens, chromosome = chr)
plotTracks(list(itrack, gtrack, atrack, grtrack, strack), 
           from = 26591822, to = 26591852, cex = 0.8)
```

So far we have replicated the features of a whole bunch of other
genome browser tools out there. The real power of the package comes
with a rather general track type, the
`DataTrack`. `DataTrack` object are essentially
run-length encoded numeric vectors or matrices, and we can use them to
add all sorts of numeric data to our genomic coordinate plots. There
are a whole bunch of different visualization options for these tracks,
from dot plots to histograms to box-and-whisker plots. The individual
rows in a numeric matrix are considered to be different data groups or
samples, and the columns are the raster intervals in the genomic
coordinates. Of course, the data points (or rather the data ranges) do
not have to be evenly spaced; each column is associated with a
particular genomic location. For demonstration purposes we can create
a simple `DataTrack` object from randomly sampled data.

```{r DataTrack, fig.width=7.5, fig.height=4}
set.seed(255)
lim <- c(26700000, 26750000)
coords <- sort(c(lim[1], 
                 sample(seq(from = lim[1], to = lim[2]), 99), 
                 lim[2]))
dat <- runif(100, min = -10, max = 10)
dtrack <- DataTrack(data = dat, start = coords[-length(coords)],
                    end = coords[-1], chromosome = chr, genome = gen, 
                    name = "Uniform")
plotTracks(list(itrack, gtrack, atrack, grtrack, dtrack), 
           from = lim[1], to = lim[2])
```

The first thing to notice is that the title panel to the right of the
plot now contains a y-axis indicating the range of the displayed
data. The default plotting type for numeric vectors is a simple dot
plot. This is by far not the only visualization option, and in a sense
it is wasting quite a lot of information because the run-length
encoded ranges are not immediately apparent. We can change the plot
type by supplying the `type` argument to
`plotTracks`. A complete description of the available
plotting options is given in section \@ref(track-classes), and a
more detailed treatment of the so-called 'display parameters' that
control the look and feel of a track is given in the
\@ref(plotting-parameters) section.

```{r DataTrackHist, fig.width=7.5, fig.height=4}
plotTracks(list(itrack, gtrack, atrack, grtrack, dtrack), 
           from = lim[1], to = lim[2], type = "histogram")
```

As we can see, the data values in the numeric vector are indeed
matched to the genomic coordinates of the `DataTrack`
object. Such a visualization can be particularly helpful when
displaying for instance the coverage of NGS reads along a chromosome,
or to show the measurement values of mapped probes from a micro array
experiment.

This concludes our first introduction into the `r Biocpkg("Gviz")`
package. The remainder of this vignette will deal in much more depth
with the different concepts and the various track classes and plotting
options.

# Plotting parameters {#plotting-parameters}

## Setting parameters

Although not implicitly mentioned before, we have already made use of
the plotting parameter facilities in the `r Biocpkg("Gviz")` package, or, as we will
call them from now on, the 'display parameters'. Display parameters
are properties of individual track objects (i.e., of any object
inheriting from the base `GdObject` class). They can either be
set during object instantiation as additional arguments to the
constructor functions or, for existing track objects, using the
`displayPars` replacement method. In the former case, all
named arguments that can not be matched to any of the constructor's
formal arguments are considered to be display parameters, regardless of
their type or whether they are defined for a particular track class or
not. The following code example rebuilds our `GeneRegionTrack`
object with a bunch of display parameters and demonstrates the use of
the `displayPars` accessor and replacement methods.

```{r displayPars1f, fig.width=7.5, fig.height=3}
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr, 
                           name = "Gene Model",
                           transcriptAnnotation = "symbol",
                           background.title = "brown")
head(displayPars(grtrack))
displayPars(grtrack) <- list(background.panel = "#FFFEDB", col = NULL)
head(displayPars(grtrack))

plotTracks(list(itrack, gtrack, atrack, grtrack))
```

For our gene model track we have now added the gene symbols of the
different transcripts to the plot, removed the gray border around the
individual exons models, and changed the background color of both the
title and the data panel to a yellowish hue. There is a third option
to set display parameters for a single plotting operation (rather than
the permanent setting in the track object) by passing in additional
arguments to the `plotTracks` function. We have already made
use of this feature, for instance in the previous data plotting type
example. It is worth mentioning that all display parameters which are
passed along with the `plotTracks` function apply to
_all_ track objects in the plot. For some objects classes a
particular display parameter may not make any sense, and in that case
it is simply ignored. Also, the settings only apply for one single
plotting operation and will not be retained in the plotted track
objects. They do however get precedence over the object-internal
parameters. The following line of code exemplifies this behavior.

```{r displayPars2f, fig.width=7.5, fig.height=3}
plotTracks(list(itrack, gtrack, atrack, grtrack), 
           background.panel = "#FFFEDB", background.title = "darkblue")
```

In order to make full use of the flexible parameter system we need to
know which display parameters control which aspect of which track
class. The obvious source for this information are the man pages of
the respective track classes, which list all available parameters
along with a short description of their effect and default values in
the `Display Parameters` section. Alternatively, we can use
the `availableDisplayPars` function, which prints out the
available parameters for a class as well as their default values in a
list-like structure. The single argument to the function is either a
class name of a track object class, or the object itself, in which
case its class is automatically detected.

```{r displayPars3}
dp <- availableDisplayPars(grtrack)
tail(dp)
```

As we can see, display parameters can be inherited from parent
classes. For the regular user this is not important at all, however it
nicely exemplifies the structure of the class hierarchy in the `r Biocpkg("Gviz")`
package.

## Schemes

Users might find themselves changing the same parameters over and over
again, and it would make sense to register these modifications in a
central location once and for all. To this end the `r Biocpkg("Gviz")` package
supports display parameter schemes. A scheme is essentially just a
bunch of nested named lists, where the names on the first level of
nesting should correspond to track class names, and the names on the
second level to the display parameters to set. The currently active
scheme can be changed by setting the global option
`Gviz.scheme`, and a new scheme can be registered by using the
`addScheme` function, providing both the list and the name
for the new scheme. The `getScheme` function is useful to
get the current scheme as a list structure, for instance to use as a
skeleton for your own custom scheme.

```{r displayPars4, fig.width=7.5, fig.height=1.5}
getOption("Gviz.scheme")
scheme <- getScheme()
scheme$GeneRegionTrack$fill <- "salmon"
scheme$GeneRegionTrack$col <- NULL
scheme$GeneRegionTrack$transcriptAnnotation <- "transcript"
addScheme(scheme, "myScheme")
options(Gviz.scheme = "myScheme")
grtrack <- GeneRegionTrack(geneModels, genome = gen,
                           chromosome = chr, name = "Gene Model")
plotTracks(grtrack)
options(Gviz.scheme="default")
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr,
                           name = "Gene Model",
                           transcriptAnnotation = "symbol")
```

In order to make these settings persistent across R sessions one can
create one or several schemes in the global environment in the special
object `.GvizSchemes`, for instance by putting the necessary
code in the `.Rprofile` file. This object needs to be a named
list of schemes, and it will be collected when the `r Biocpkg("Gviz")` package
loads. Its content is then automatically added to the collection of
available schemes.  The following pseudo-code exemplifies this and
could go into an `.Rprofile` file.

```{r schemes, eval=FALSE}
.GvizSchemes <- list(myScheme = list(
  GeneRegionTrack=list(fill = "salmon", col = NULL, 
                       transcriptAnnotation = "transcript")))
```

Please note that because display parameters are stored
with the track objects, a scheme change only has an effect on those
objects that are created after the change has taken place.

## Plotting direction

By default all tracks will be plotted in a 5' -> 3' direction. It
sometimes can be useful to actually show the data relative to the
opposite strand. To this end one can use the `reverseStrand`
display parameter, which does just what its name suggests. Since the
combination of forward and reverse stranded tracks on a single plot
does not make too much sense, one should usually set this as a global
display parameter in the `plotTracks` function. The function
will however cast a warning if a mixture of forward and reverse strand
tracks has been passed in for plotting.

```{r plottingdirections, fig.width=7.5, fig.height=3}
plotTracks(list(itrack, gtrack, atrack, grtrack), reverseStrand = TRUE)
```

As you can see, the fact that the data has been plotted on the reverse
strand is also reflected in the `GenomeAxis` track.

# Track classes {#track-classes}

In this section we will highlight all of the available annotation
track classes in the `r Biocpkg("Gviz")` package. For the complete reference of all
the nuts and bolts, including all the available methods, please see
the respective class man pages. We will try to keep this vignette up
to date, but in cases of discrepancies between here and the man pages
you should assume the latter to be correct.

## GenomeAxisTrack

`GenomeAxisTrack` objects can be used to add some reference to
the currently displayed genomic location to a `r Biocpkg("Gviz")` plot. In their most
basic form they are really just a horizontal axis with genomic
coordinate tick-marks. Using the `GenomeAxisTrack`
constructor function is the recommended way to instantiate objects
from the class. There is no need to know in advance about a particular
genomic location when constructing the object. Instead, the displayed
coordinates will be determined from the context, e.g., from the
`from` and `to` arguments of the
`plotTracks` function, or, when plotted together with other
track objects, from their genomic locations.

```{r GenomeAxisTrackClass1, fig.width=7.5, fig.height=0.75}
axisTrack <- GenomeAxisTrack()
plotTracks(axisTrack, from = 1e6, to = 9e6)
```

As an optional feature one can highlight particular regions
on the axis, for instance to indicated stretches of N nucleotides or
gaps in genomic alignments. Such regions have to be supplied to the
optional `range` argument of the constructor function as
either an `GRanges` or an `IRanges` object.

```{r GenomeAxisTrackClass2, fig.width=7.5, fig.height=0.75}
axisTrack <- GenomeAxisTrack(range=IRanges(start = c(2e6, 4e6),
                                           end = c(3e6, 7e6),
                                           names = rep("N-stretch", 2)))
plotTracks(axisTrack, from = 1e6, to = 9e6)
```

If names have been supplied with the `range` argument, those
can also be added to the plot.

```{r GenomeAxisTrackClass2a, fig.width=7.5, fig.height=0.75}
plotTracks(axisTrack, from = 1e6, to = 9e6, showId = TRUE)
```

### Display parameters for GenomeAxisTrack objects
There are a whole bunch of display parameters to alter the appearance
of `GenomeAxisTrack` plots. For instance, one could add
directional indicators to the axis using the `add53` and
`add35` parameters.

```{r GenomeAxisTrackClass3, fig.width=7.5, fig.height=0.75}
plotTracks(axisTrack, from = 1e6, to = 9e6, add53 = TRUE, add35 = TRUE)
```

Sometimes the resolution of the tick marks is not sufficient, in which
case the `littleTicks` argument can be used to have a more
fine-grained axis annotation.

```{r GenomeAxisTrackClass4, fig.width=7.5, fig.height=0.75}
plotTracks(axisTrack, from = 1e6, to = 9e6, add53 = TRUE,
           add35 = TRUE, littleTicks = TRUE)
```

The `r Biocpkg("Gviz")` package tries to come up with reasonable defaults for the
axis annotation. In our previous example, the genomic coordinates are
indicated in megabases. We can control this via the `exponent`
parameter, which takes an integer value greater then zero. The
location of the tick marks are displayed as a fraction of
$10^{exponent}$.

```{r GenomeAxisTrackClass5, fig.width=7.5, fig.height=0.75}
plotTracks(axisTrack, from = 1e6, to = 9e6, exponent = 4)
```

Another useful parameter, `labelPos` controls the arrangement
of the tick marks. It takes one of the values `alternating`,
`revAlternating`, `above` or `below`. For instance we
could align all tick marks underneath the axis.

```{r GenomeAxisTrackClass6, fig.width=7.5, fig.height=0.75}
plotTracks(axisTrack, from = 1e6, to = 9e6, labelPos = "below")
```

Sometimes a full-blown axis is just too much, and all we really need
in the plot is a small scale indicator. We can change the appearance
of the `GenomeAxisTrack` object to such a limited
representation by setting the `scale` display
parameter. Typically, this will be a numeric value between 0 and 1,
which is interpreted as the fraction of the plotting region used for
the scale. The plotting method will apply some rounding to come up
with reasonable and human-readable values. For even more control we
can pass in a value larger than 1, which is considered to be an
absolute range length. In this case, the user is responsible for the
scale to actually fit in the current plotting range.

```{r GenomeAxisTrackClass7, fig.width=7.5, fig.height=0.5}
plotTracks(axisTrack, from = 1e6, to = 9e6, scale = 0.5)
```

We still have control over the placement of the scale label via the
`labelPos`, parameter, which now takes the values
`above`, `below` and `beside`.

```{r GenomeAxisTrackClass8, fig.width=7.5, fig.height=0.5}
plotTracks(axisTrack, from = 1e6, to = 9e6, scale = 0.5, 
           labelPos = "below")
```

For a complete listing of all the available display parameters please
see the table below or the man page of the `GenomeAxisTrack`
class by typing in `?GenomeAxisTrack` on the `R` command line.

```{r GenomeAxisTrackClassTable, echo=FALSE, results='asis'}
addParTable(xtabDetails,"GenomeAxisTrack", out_type = out_type)
```

## IdeogramTrack
While a genomic axis provides helpful points of reference to a plot,
it is sometimes important to show the currently displayed region in
the broader context of the whole chromosome. Are we looking at distal
regions, or somewhere close to the centromere? And how much of the
complete chromosome is covered in our plot. To that end the `r Biocpkg("Gviz")`
package defines the `IdeogramTrack` class, which is an
idealized representation of a single chromosome. When plotted, these
track objects will always show the whole chromosome, regardless of the
selected genomic region. However, the displayed coordinates are
indicated by a box that sits on the ideogram image. The chromosomal
data necessary to draw the ideogram is not part of the `r Biocpkg("Gviz")` package
itself, instead it is downloaded from an online source (UCSC). Thus it
is important to use both chromosome and genome names that are
recognizable in the UCSC data base when dealing with
`IdeogramTrack` objects. You might want to consult the [UCSC
webpage](http://genome.ucsc.edu/) or use the
`ucscGenomes` function in the `rtracklayer` package
for a listing of available genomes.

Assuming the chromosome data are available online, a simple call to
the `IdeogramTrack` constructor function including the
desired genome and chromosome name are enough to instantiate the
object. Since the connection to UCSC can be slow, the package tries to
cache data that has already been downloaded for the duration of the `R`
session. If needed, the user can manually clear the cache by calling
the `clearSessionCache` function. Of course it is also
possible to construct `IdeogramTrack` objects from local
data. Please see the class' man page for details.

```{r IdeogramTrackClass1Show, eval=FALSE}
ideoTrack <- IdeogramTrack(genome = "hg19", chromosome = "chrX")
plotTracks(ideoTrack, from = 85e6, to = 129e6)
```

```{r IdeogramTrackClass1Do, fig.width=7.5, fig.height=0.5, echo=FALSE, results='hide'}
if(hasUcscConnection) {
  ideoTrack <- IdeogramTrack(genome = "hg19", chromosome = "chrX")
} else{
  data(ideoTrack)
}
plotTracks(ideoTrack, from = 85e6, to = 129e6)
```

We can turn off the explicit plotting of the chromosome name by
setting the `showId` display parameter to `FALSE`.

```{r IdeogramTrackClass2, fig.width=7.5, fig.height=0.5}
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE)
```

The chromosome bands in the ideogram come with a unique identifier,
and we can add this information to the plot, at least for those bands
that are wide enough to accommodate the text.

```{r IdeogramTrackClass3, fig.width=7.5, fig.height=0.5}
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE, 
           showBandId = TRUE, cex.bands = 0.5)
```

We can also change the shape of the centromere to "circle" instead of default "triangle".

```{r IdeogramTrackClass4, fig.width=7.5, fig.height=0.5}
plotTracks(ideoTrack, from = 85e6, to = 129e6, showId = FALSE,
           centromereShape = "circle")
```


### Display parameters for IdeogramTrack objects

For a complete listing of all the available display parameters please
see the table below or the man page of the `IdeogramTrack`
class by typing in `?IdeogramTrack` on the `R` command line.

```{r IdeogramTrackClassTable, echo=FALSE, results='asis'}
addParTable(xtabDetails, "IdeogramTrack", out_type = out_type)
```

## DataTrack

Probably the most powerful of all the track classes in the `r Biocpkg("Gviz")`
package are `DataTracks`. Essentially they constitute
run-length encoded numeric vectors or matrices, meaning that one or
several numeric values are associated to a particular genomic
coordinate range. These ranges may even be overlapping, for instance
when looking at results from a running window operation. There can be
multiple samples in a single data set, in which case the ranges are
associated to the columns of a numeric matrix rather than a numeric
vector, and the plotting method provides tools to incorporate sample
group information. Thus the starting point for creating
`DataTrack` objects will always be a set of ranges, either in
the form of an `IRanges` or `GRanges` object, or
individually as start and end coordinates or widths. The second
ingredient is a numeric vector of the same length as the number of
ranges, or a numeric matrix with the same number of columns. Those may
even already be part of the input `GRanges` object as
`elemenMetadata` values. For a complete description of all the
possible inputs please see the class' online documentation. We can
pass all this information to the `DataTrack` constructor function to
instantiate an object. We will load our sample data from an
`GRanges` object that comes as part of the `r Biocpkg("Gviz")` package.

```{r DataClass1, fig.width=7.5, fig.height=1.5}
data(twoGroups)
dTrack <- DataTrack(twoGroups, name = "uniform")
plotTracks(dTrack)
```

The default visualization for our very simplistic sample
`DataTrack` is a rather uninspiring dot plot. The track comes
with a scale to indicate the range of the numeric values on the
y-axis, apart from that it looks very much like the previous
examples. A whole battery of display parameters is to our disposal to
control the track's look and feel. The most important one is the
`type` parameter. It determines the type of plot to use and
takes one or several of the following values:

```{r types, echo=FALSE, results='asis'}
types <- data.frame(Value=c("p", "l", "b", "a", "s", "S", "g", "r", "h", "confint", "smooth", "histogram", "mountain", "polygon", "boxplot", "gradient", "heatmap", "horizon"),
                    Type=c("dot plot", "lines plot", "dot and lines plot", "lines plot of average (i.e., mean) values", "stair steps (horizontal first)",
                           "stair steps (vertical first)", "add grid lines", "add linear regression line", "histogram lines", "confidence intervals for average values", "add loess curve",
                           "histogram (bar width equal to range with)", "'mountain-type' plot relative to a baseline",
                           "'polygon-type' plot relative to a baseline", "box and whisker plot",
                           "false color image of the summarized values", "false color image of the individual values",
                           "Horizon plot indicating magnitude and direction of a change relative to a baseline"))

if (out_type == "html") {
  kable(types)
} else if (out_type == "latex") {
  kable(types, "latex", booktabs = TRUE, longtable = TRUE)
}
```


Displayed below are the same sample data as before but plotted with the different type settings ^[A different data set is plotted for the horizon type for the sake of clarity.]:

```{r typePlots, fig.width=7.5, fig.height=8.5, echo=FALSE, results='hide'}
pushViewport(viewport(layout = grid.layout(nrow = 9, ncol = 2)))
i <- 1
for(t in types$Value) {
  pushViewport(viewport(layout.pos.col = ((i - 1) %% 2) + 1,
                        layout.pos.row = ((i - 1) %/% 2)+1))
    if(t != "horizon"){
        names(dTrack) <- t
        plotTracks(dTrack, type = t, add = TRUE, 
                   cex.title = 0.8, margin = 0.5)
    } else {
        data(dtHoriz)
        names(dtHoriz) <- "horizon *"
        plotTracks(dtHoriz[8, ], type = "horizon", add = TRUE, 
                   cex.title = 0.8, margin = 0.5, 
                   showAxis = FALSE, horizon.origin = 0.7)
    }
    i <- i + 1
    popViewport(1)
}
popViewport(1)
names(dTrack) <- "uniform"
```

data while others are clearly designed for multivariate inputs. The
`a` type for instance averages the values at each genomic
location before plotting the derived values as a line. The decision
for a particular plot type is totally up to the user, and one could
even overlay multiple types by supplying a character vector rather
than a character scalar as the `type` argument. For example,
this will combine a boxplot with an average line and a data grid.



```{r mutitype, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack, type = c("boxplot", "a", "g"))
```

For the heatmap plotting type we arrange all the data in a
well-structured two-dimensional matrix which gives us the opportunity
to add a little extra information about the individual
samples. Depending on how the `DataTrack` was created in the
first place we can choose to display the sample names (which in our
case correspond to the column names of the input `GRanges`
object). The plot also highlights another feature of the heatmap type:
the y-axis now shows a mapping of the numeric values into the color
range.

```{r sampNames, fig.width=7.5, fig.height=1.5}
colnames(mcols(twoGroups))
plotTracks(dTrack, type = c("heatmap"), showSampleNames = TRUE, 
           cex.sampleNames = 0.6)
```

### Data Grouping

An additional layer of flexibility is added by making use of `r Biocpkg("Gviz")`'s
grouping functionality. The individual samples (i.e., rows in the data
matrix) can be grouped together using a factor variable, and, if
reasonable, this grouping is reflected in the layout of the respective
track types. For instance our example data could be derived from two
different sample groups with three replicates each, and we could easily
integrate this information into our plot.

```{r grouping, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3),
           type = c("a", "p", "confint"))
```

For the dot plot representation the individual group levels are
indicated by color coding. For the `a` type, the averages are
now computed for each group separately and also indicated by two lines
with similar color coding. Grouping is not supported for all plotting
types, for example the `mountain` and `polygon` type
already use color coding to convey a different message and for the
`gradient` type the data are already collapsed to a single
variable. The following gives an overview over some of the other
groupable `DataTrack` types. Please note that there are many
more display parameters that control the layout of both grouped and of
ungrouped `DataTracks`. You may want to check the class' help
page for details. ^[Please note, that in the example, a different data 
set is plotted for the horizon type for the sake of clarity.]

```{r typeGroupedPlots, fig.width=7.5, fig.height=6, echo=FALSE, results='hide'}
pushViewport(viewport(layout=grid.layout(nrow=9, ncol=1)))
i <- 1
for(t in c("a", "s", "confint", "smooth", "histogram", "boxplot", "heatmap", "horizon", "hor. type")) {
  pushViewport(viewport(layout.pos.col=((i-1)%%1)+1, layout.pos.row=((i-1)%/%1)+1))
  names(dTrack) <- t
  if(!t %in% c("horizon", "hor. type")) {
    plotTracks(dTrack, type=t, add=TRUE, cex.title=0.8, groups=rep(1:2, each=3), margin=0.5)
  } else if (t == "horizon") {
    plotTracks(dtHoriz[c(1,8),], type="horizon", add=TRUE, cex.title=0.8, margin=0.5, showAxis=FALSE, horizon.origin=0.3,
               groups=1:2)
  } else {
    plotTracks(dTrack, type="histogram", stackedBars=FALSE, add=TRUE, cex.title=0.8, groups=rep(1:2, each=3), margin=0.5)
  }
  i <- i+1
  popViewport(1)
}
popViewport(1)
names(dTrack) <- "uniform"
```

If we need to display some additional information about the individual
group levels we can make use of the `legend` display parameter
to add a simple legend to the plot. Depending on the plot type and on
some of the other display parameters, the look of this legend may vary
slightly.

```{r groupingLegend, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3),
           type = c("a", "p"), legend = TRUE)
```

For a grouped horizon plot the group labels have to be shown in a
similar fashion as for heatmaps, i.e., by setting the
`showSampleNames` argument to `TRUE`.

```{r horizLegend, results='hide', fig.width=7.5, fig.height=1.5}
data(dtHoriz)
dtHoriz <- dtHoriz[1:6, ]
plotTracks(dtHoriz, type = "horiz", groups = rownames(values(dtHoriz)),
           showSampleNames = TRUE, cex.sampleNames = 0.6, separator = 1)
```

### Building DataTrack objects from files
A number of standard file types exist that all store numeric data
along genomic coordinates. We have tried to make such files accessible
in the `r Biocpkg("Gviz")` package by providing additional options to the
`DataTrack` constructor function. In the previous examples
the `range` argument was a `GRanges` object. Instead,
we can also pass in the path to a file on disk by means of a character
scalar. The `DataTrack` class supports the most common file
types like `wig`, `bigWig` or `bedGraph`, but also
knows how to deal with `bam` files. You may have realized that
some of these files are indexed, and we have taken the approach to
stream the data from indexed files on the fly when it is needed for
plotting.

However let's first start with the simple example of a `bedGraph`
file. These files support a single data sample, and thus are
equivalent to a `GRanges` object with a single numeric
metadata column. `bedGraph` files are not indexed,
so we have to load the whole file content when instantiating the
object.

```{r filedt1, fig.width=7.5, fig.height=1}
bgFile <- system.file("extdata/test.bedGraph", package = "Gviz")
dTrack2 <- DataTrack(range = bgFile, genome = "hg19", type = "l", 
                     chromosome = "chr19", name = "bedGraph")
class(dTrack2)
plotTracks(dTrack2)
```

As we can see the constructor has returned a regular
`DataTrack` object. The function to be used in order to read
the data off the file has been automatically chosen by the package
based on the file extension of the input file. Of course the number of
these supported standard file types is limited, and a user may want to
import a non-standard file through the same mechanism. To this end,
the `DataTrack` constructor defines an additional argument
called `importFunction`. As the name suggests, the value of
this argument is a function which needs to handle the mandatory
`file` argument. Upon evaluation this argument will be filled
in with the path to the data file, and the user-defined function needs
to provide all logic necessary to parse that file into a valid
`GRanges` object. From this point on everything will happen just
as if the `range` argument had been this `GRanges`
object. In other words, numeric metadata columns will be
shown as individual samples and non-numeric columns will be silently
ignored. We can exemplify this in the next code chunk. Note that the
`r Biocpkg("Gviz")` package is using functionality from the `r Biocpkg("rtracklayer")`
package for most of the file import operations, just as we do here in
a more explicit way.

```{r filedt2}
library(rtracklayer)
dTrack3 <-  DataTrack(range = bgFile, genome = "hg19", type = "l", 
                      chromosome = "chr19", name = "bedGraph",
                      importFunction = function(file) import(con=file))
identical(dTrack2, dTrack3)
```

So far one could have easily done the whole process in two separate
steps: first import the data from the file into a `GRanges`
object and then provided this object to the constructor. The real
power of the file support in the `r Biocpkg("Gviz")` package comes with streaming
from indexed files. As mentioned before, only the relevant part of the
data has to be loaded during the plotting operation, so the underlying
data files may be quite large without decreasing the performance or
causing too big of a memory footprint. We will exemplify this feature
here using a small `bam` file that is provided with the
package. `bam` files contain alignments of sequences (typically
from a next generation sequencing experiment) to a common
reference. The most natural representation of such data in a
`DataTrack` is to look at the alignment coverage at a given
position only and to encode this in a single metadata column.

```{r filedt3, fig.width=7.5, fig.height=1}
bamFile <- system.file("extdata/test.bam", package = "Gviz")
dTrack4 <- DataTrack(range = bamFile, genome = "hg19", type = "l", 
                     name = "Coverage", window = -1, 
                     chromosome = "chr1")
class(dTrack4)
dTrack4
plotTracks(dTrack4, from = 189990000, to = 190000000)
```

As seen in the previous code chunk, the `dTrack4` object is now
of class `ReferenceDataTrack`. For the user this distinction is
not particularly relevant with the exception that the
`length` method for this class almost always returns
`0` because the content of the object is only realized during the
plotting operation. Obviously, streaming from the disk comes with a
price in that file access is much slower than accessing RAM, however
the file indexing allows for fairly rapid data retrieval, and other
processes during the plotting operation tend to be much more costly,
anyways. It is worth mentioning however that each plotting operation
will cause reading off the file, and there are currently no caching
mechanisms in place to avoid that. Nevertheless, plotting a larger
chunk of the `bam` file still finishes in a reasonable time.

```{r filedt4, fig.width=7.5, fig.height=1}
plotTracks(dTrack4, chromosome = "chr1", from = 189891483, to = 190087517)
```

Of course users can provided their own file parsing function just like
we showed in the previous example. The import function now needs to be
able to deal with a second mandatory argument `selection`,
which is a `GRanges` object giving the genomic interval that
has to be imported from the file. In addition one needs to tell the
`DataTrack` constructor that data should be streamed off a file
by setting the `stream` argument to `TRUE`.

```{r filedt5}
myImportFun <- function(file, selection){
    ## do something here
}
DataTrack(range = bamFile, genome = "hg19", type = "l",
          name = "Coverage", window = -1, chromosome = "chr1",
          importFunction = myImportFun, stream = TRUE)
```

### Data transformations

The `r Biocpkg("Gviz")` package offers quite some flexibility to transform data on
the fly. This involves both rescaling operations (each data point is
transformed on the track's y-axis by a transformation function) as
well as summarization and smoothing operations (the values for several
genomic locations are summarized into one derived value on the track's
x-axis). To illustrate this let's create a significantly bigger
`DataTrack` than the one we used before, containing purely
synthetic data for only a single sample.

```{r biggerdata, results='hide', fig.width=7.5, fig.height=1.5}
dat <- sin(seq(pi, 10*pi, len=500))
dTrack.big <- DataTrack(start = seq(1, 100000, len = 500), width = 15, 
                        chromosome = "chrX", genome = "hg19", 
                        name = "sinus",
                        data = sin(seq(pi, 5 * pi, len = 500)) * 
                          runif(500, 0.5, 1.5))
plotTracks(dTrack.big, type="hist")
```

Since the available resolution on our screen is limited we can no
longer distinguish between individual coordinate ranges. The `r Biocpkg("Gviz")`
package tries to avoid overplotting by collapsing overlapping ranges
(assuming the `collapseTracks` parameter is set to
`TRUE`). However, it is often desirable to summarize the data,
for instance by binning values into a fixed number of windows followed
by the calculation of a meaningful summary statistic. This can be
archived by a combination of the `window` and
`aggregation` display parameters. The former can be an integer
value greater than zero giving the number of evenly-sized bins to
aggregate the data in. The latter is supposed to be a user-supplied
function that accepts a numeric vector as a single input parameter and
returns a single aggregated numerical value. For simplicity, the most
obvious aggregation functions can be selected by passing in a
character scalar rather than a function. Possible values are
`mean`, `median, `extreme`, `sum, `min` and
`max`. These presets are also much faster because they have been
optimized to operate on large numeric matrices. The default is to
compute the mean value of all the binned data points.

```{r aggregation, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack.big, type = "hist", window = 50)
```

Instead of binning the data in fixed width bins one can also use the
`window` parameter to perform more elaborate running window
operations. For this to happen the parameter value has to be smaller
than zero, and the additional display parameter `windowSize`
can be used to control the size of the running window. This operation
does not change the number of coordinate ranges on the plot, but
instead the original value at a particular position is replaced by the
respective sliding window value at the same position. A common use
case for sliding windows on genomic ranges is to introduce a certain
degree of smoothing to the data.

```{r aggregation2, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack.big, type = "hist", window = -1, windowSize = 2500)
```

In addition to transforming the data on the x-axis we can also apply
arbitrary transformation functions on the y-axis. One obvious use-case
would be to log-transform the data prior to plotting. The framework is
flexible enough however to allow for arbitrary transformation
operations. The mechanism works by providing a function as the
`transformation` display parameter, which takes as input a
numeric vector and returns a transformed numeric vector of the same
length. The following code for instance truncates the plotted data to
values greater than zero.

```{r transformation, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack.big, type = "l", 
           transformation = function(x) { x[x < 0] <- 0; x })
```

As seen before, the `a` type allows to plot average values for
each of the separate groups. There is however an additional parameter
`aggregateGroups` that generalizes group value
aggregations. In the following example we display, for each group and
at each position, the average values in the form of a dot-and-lines
plot.

```{r groupingAv1, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3), 
           type = c("b"), aggregateGroups = TRUE)
```

This functionality again also relies on the setting of the
`aggregation` parameter, and we can easily change it to
display the maximum group values instead.

```{r groupingAv2, results='hide', fig.width=7.5, fig.height=1.5}
plotTracks(dTrack, groups = rep(c("control", "treated"), each = 3), 
           type = c("b"), aggregateGroups = TRUE, aggregation = "max")
```

### Display parameters for DataTrack objects

For a complete listing of all the available display parameters please
see the table below or the man page of the `DataTrack`
class by typing in `?DataTrack` on the `R` command line.

```{r DataTrackClassTable, echo=FALSE, results='asis'}
addParTable(xtabDetails,"DataTrack", out_type = out_type)
```

## AnnotationTrack

`AnnotationTrack` objects are the multi-purpose tracks in the
`r Biocpkg("Gviz")` package. Essentially they consist of one or several genomic
ranges that can be grouped into composite annotation elements if
needed. In principle this would be enough to represent everything from
CpG islands to complex gene models, however for the latter the package
defines the specialized `GeneRegionTrack` class, which will be
highlighted in a separate section. Most of the features discussed here
will also apply to `GeneRegionTrack` objects, though. As a
matter of fact, the `GeneRegionTrack` class inherits directly
from class `AnnotationTrack`.

`AnnotationTrack` objects are easily instantiated using the
constructor function of the same name. The necessary building blocks
are the range coordinates, a chromosome and a genome identifier. Again
we try to be flexible in the way this information can be passed to the
function, either in the form of separate function arguments, as
`IRanges`, `GRanges` or `data.frame`
objects. Optionally, we can pass in the strand information for the
annotation features and some useful identifiers. A somewhat special
case is to build the object from a `GRangesList` object, which
will automatically preserve the element grouping information contained
in the list structure. For the full details on the constructor
function and the accepted arguments see `?AnnotationTrack`. Let's
take a look at a very simple track:

```{r anntrack1, results='hide', fig.width=7.5, fig.height=0.5}
aTrack <- AnnotationTrack(start = c(10, 40, 120), width = 15, 
                          chromosome = "chrX", 
                          strand = c("+", "*", "-"),
                          id = c("Huey", "Dewey", "Louie"), 
                          genome = "hg19", name = "foo")
plotTracks(aTrack)
```

The ranges are plotted as simple boxes if no strand information is
available, or as arrows to indicate their direction. We can change the
range item shapes by setting the `shape` display parameter. It can
also be helpful to add the names for the individual features to the
plot. This can be archived by setting the `featureAnnotation`
parameter to `'id'`.

```{r anntrack2, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(aTrack, shape = "box", featureAnnotation = "id")
```

```{r anntrack3, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(aTrack, shape = "ellipse", featureAnnotation = "id", 
           fontcolor.feature = "darkblue")
```

In this very simplistic example each annotation feature consisted of a
single range. In real life the genomic annotation features that we
encounter often consists of several sub-units. We can create such
composite `AnnotationTrack` objects by providing a grouping
factor to the constructor. It needs to be of similar length as the
total number of atomic features in the track, i.e, the number of
genomic ranges that are passed to the constructor. The levels of the
this factor will be used as internal identifiers for the individual
composite feature groups, and we can toggle on their printing by
setting `groupAnnotation` to 'group'.

```{r anntrack4f, results='hide', fig.width=7.5, fig.height=0.5}
aTrack.groups <- AnnotationTrack(start = c(50, 180, 260, 460, 860, 1240), 
                                 width = c(15, 20, 40, 100, 200, 20),
                                 chromosome = "chrX",
                                 strand = rep(c("+", "*", "-"), 
                                              c(1, 3, 2)),
                                 group = rep(c("Huey", "Dewey", "Louie"), 
                                             c(1, 3, 2)),
                                 genome = "hg19", name = "foo")
plotTracks(aTrack.groups, groupAnnotation = "group")
```

We can control the placement of the group labels through the
`just.group` parameter.

```{r anntrack4af, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(aTrack.groups, groupAnnotation = "group", 
           just.group = "right")
```

```{r anntrack4bf, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(aTrack.groups, groupAnnotation = "group", 
           just.group = "above")
```

Arranging items on the plotting canvas is relatively straight forward
as long as there are no overlaps between individual regions or groups
of regions. Those inevitably cause overplotting which could seriously
obfuscate the information on the plot. A logical solution to this
problem is to stack overlapping items in separate horizontal lines to
accommodate all of them. This involves some optimization, and the `r Biocpkg("Gviz")`
package automatically tries to come up with the most compact
arrangement. Let's exemplify this feature with a slightly modified
`AnnotationTrack` object.

```{r stacking1, results='hide', fig.width=7.5, fig.height=0.5}
aTrack.stacked <- AnnotationTrack(start = c(50, 180, 260, 800, 600, 1240),
                                  width = c(15, 20, 40, 100, 500, 20),
                                  chromosome = "chrX",
                                  strand = "*",
                                  group = rep(c("Huey", "Dewey", "Louie"), 
                                              c(1, 3, 2)),
                                  genome = "hg19", name = "foo")
plotTracks(aTrack.stacked, groupAnnotation="group")
```

We now have our three annotation feature groups distributed over two
horizontal lines. One can control the stacking of overlapping items
using the `stacking` display parameter. Currently the three
values `squish`, `dense` and `hide` are
supported. Horizontal stacking is enabled via the `squish`
option, which also is the default. The `dense` setting forces
overlapping items to be joined in one meta-item and `hide` all
together disables the plotting of `AnnotationTrack`
items. Please note that adding identifiers to the plot only works for
the `squish` option.

```{r stacking2, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(aTrack.stacked, stacking = "dense")
```

In addition to annotation groups there is also the notion of a feature
type in the `r Biocpkg("Gviz")` package. Feature types are simply different types of
annotation regions (e.g., mRNA transcripts, miRNAs, rRNAs, etc.) that
are indicated by different colors. There is no limit on the number of
different features, however each element in a grouped annotation item
needs to be of the same feature type. We can query and set features
using the `feature` and `feature<-` methods.

```{r features}
feature(aTrack.stacked)
feature(aTrack.stacked) <- c("foo", "bar", "bar", "bar", "no", "no")
```

Features types can also be shown as either the group or the item
annotation on the plot.

```{r featuresIdPlot, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(aTrack.stacked, featureAnnotation = "feature", 
           groupAnnotation = "feature", fontcolor.feature = 1, 
           cex.feature = 0.7)
```

Unless we tell the `r Biocpkg("Gviz")` package how to deal with the respective
feature types they will all be treated in a similar fashion, i.e.,
they will be plotted using the default color as defined by the
`fill` display parameter. To define colors for individual
feature types we simply have to add them as additional display
parameters, where the parameter name matches the feature type and its
value is supposed to be a valid R color qualifier. Of course this
implies that we can only use feature names that are not already
occupied by other display parameters defined in the package.


```{r featuresPlotf, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(aTrack.stacked, groupAnnotation = "group", 
           foo = "darkred", bar = "darkgreen")
```

Stacking of annotation items to avoid overplotting only works as long
as there is enough real estate on the plotting canvas to separate all
items, i.e., we need all items to be at least a single pixel wide to
correctly display them. This limitation is automatically enforced by
the `r Biocpkg("Gviz")` package, however it implies that unless neighboring items
are more than one pixel apart we can not distinguish between them and
will inevitably introduce a certain amount of overplotting. This means
that on a common screen device we can only look at a very limited
genomic region of a few kb in full resolution. Given that an average
chromosome is in the order of a few gb, we still need a reasonable way
to deal with the overplotting problem despite the item stacking
functionality. As default, the `r Biocpkg("Gviz")` package will merge all overlapping
items into one unified meta-item and only plot that (see 'Collapse'
section below for details). In order to indicate the amount of
overplotting that was introduced by this process we can use the
`showOverplotting` display parameter. It uses a color scale
(based on the original colors defined for the track), with lighter
colors indicating areas of low or no overplotting, and more saturated
colors indicating areas of high overplotting density. We exemplify
this feature on an `AnnotationTrack` object that represents a
good portion of a real human chromosome.

```{r overplotting, results='hide', fig.width=7.5, fig.height=0.75}
data("denseAnnTrack")
plotTracks(denseAnnTrack, showOverplotting = TRUE)
```



### Collapsing
All track types that inherit from class `AnnotationTrack` support
the collapsing of overlapping track items, either because they have
initially been defined as overlapping coordinates, or because the
current device resolution does not allow to sufficiently separate
them. For instance, two elements of a feature group may be separated
by 100 base pairs on the genomic scale, however when plotted to the
screen, those 100 base pairs translate to a distance of less than one
pixel. In this case we can no longer show the items as two separate
entities. One solution to this problem would be to allow for
arbitrary overplotting, in which case the last one of the overlapping
items that is drawn on the device wins. This is not optimal in many
ways, and it also poses a significant burden on the graphical engine
because a lot of stuff has to be drawn which no one will ever see.

To this end the `r Biocpkg("Gviz")` package provides an infrastructure to reasonably
collapse overlapping items, thereby adjusting the information content
that can be shown to the available device resolution. By default this
feature is turned on, and the user does not have to worry too much
about it. However, one should be aware of the consequences this may
have on a given visualization. If you absolutely do not want
collapsing to take place, you may completely turn it off by setting
the display parameter `collapse` to `FALSE`. Please note
that by doing this the `showOverplotting` parameter will also
stop working. If you opt in, there is some considerable amount of
detailed control to fine tune the collapsing to your needs.

Let's start with a small example track for which element collapsing has
been turned off and no adjustments to the ranges have been made. We
plot both the item identifiers and the group identifiers to exemplify
what is going on.

```{r collapse1f, results='hide', fig.width=7.5, fig.height=0.85}
data(collapseTrack)
plotTracks(ctrack)
```

The first thing to notice is that the for item `d` we do see the
item identifier but not the range itself. This is due to the fact that
the width of the item is smaller than a single pixel, and hence the
graphics system can not display it (Note that this is only true for
certain devices. The quartz device on the Mac seems to be a little
smarter about this). There are also the two items `e` and
`f` which seem to overlay each other completely, and another two
items which appear to be just a single fused  item (`k` and
`l`). Again, this is a resolution issue as their relative
distance is smaller than a single pixel, so all we see is a single
range and some ugly overplotted identifiers. We can control the first
issue by setting the minimum pixel width of a plotted item to be one
pixel using the `min.width` display parameter.

```{r collapse2f, results='hide', fig.width=7.5, fig.height=0.85}
plotTracks(ctrack, min.width = 1)
```

Now the item `d` has a plotable size and can be drawn to the
device. The overplotted items are still rather annoying, but the only way to
get rid of those is to turn item collapsing back on.

```{r collapse3f, results='hide', fig.width=7.5, fig.height=0.85}
plotTracks(ctrack, min.width = 1, collapse = TRUE)
```

Now all items that could not be separated by at least one pixel have
been merged into a single meta-item, and the software has taken care
of the identifiers for you, too. The merging operation is aware of the
grouping information, so no two groups where joint together. Sometimes
a single pixel width or a single pixel distance is not enough to get a
good visualization. In these cases one could decide to enforce even
larger values. We can do this not only for the minimum width, but
also for the minimum distance by setting the `min.distance`
parameter.

```{r collapse4f, results='hide', fig.width=7.5, fig.height=0.85}
plotTracks(ctrack, min.width = 3, min.distance = 5, collapse = TRUE)
```

This time also the two items `b` and `c` have been merged,
and all ranges are now at least 3 pixels wide. Depending on the
density of items on the plot even this reduction can be
insufficient. Because we did not merge complete groups we might still
end up with quite a lot of stacks to accommodate all the
information. To this end the display parameter `mergeGroups` can
be used to disable absolute group separation. Rather than blindly
merging all groups (as it is done when `stacking='dense'`)
however, the software will only join those overlapping group ranges
for which all items are already merged into a single meta item.

```{r collapse5f, results='hide', fig.width=7.5, fig.height=0.65}
plotTracks(ctrack, min.width = 3, min.distance = 5, collapse = TRUE,
           mergeGroups = TRUE, extend.left = 0.1)
```

### Building AnnotationTrack objects from files
Just like we showed before for `DataTrack` objects it is
possible to stream the data for an `AnnotationTrack` object off
the file system. We will again use the `bam` file as an
example. This time the default import function reads the coordinates
of all the sequence alignments from the file and even knows how to
deal with grouped elements.

```{r fileat1, fig.width=7.5, fig.height=1}
aTrack2 <- AnnotationTrack(range = bamFile, genome = "hg19", 
                           name = "Reads", chromosome = "chr1")
class(aTrack2)
aTrack2
plotTracks(aTrack2, from = 189995000, to = 190000000)
```

Since `AnnotationTrack` objects are somewhat more complicated
than `DataTrack` objects, the constructor provides another
level of flexibility when reading the data from a file. More
specifically, the user has the ability to map the provided
metadata columns of the `GRanges` object that is
returned by the import function to the respective columns in the final
`AnnotationTrack` object. If no explicit mapping is provided,
the package will try to find a default mapping based on the input file
type and the desired track type. The current mapping of a track is
displayed by its `show` method as can be seen above. The
mechanism for changing this default mapping is quite straight forward:
if the values in the relevant constructor arguments (in this case
`group`, `id` and `feature`) are character
scalars, the metadata columns of the `GRanges`
object that is returned by the import function are mapped against
their values. For instance we can map the `id` column to the
`group` attribute and thus suppress the grouping of sequence
alignments that originate from the same sequencing read.

```{r fileat2, fig.width=7.5, fig.height=1}
aTrack3 <- AnnotationTrack(range = bamFile, genome = "hg19", 
                           name = "Reads", chromosome = "chr1", 
                           group = "id")
aTrack3
plotTracks(aTrack3, from = 189995000, to = 190000000)
```

Please note that if no metadata column can be mapped to the
constructor arguments their values are assumed to be normal character
scalars and they will be recycled accordingly to fill the track's
slots. One can ask for the package's default mappings by using the
`availableDefaultMapping` function.

```{r fileat3}
availableDefaultMapping(bamFile, "AnnotationTrack")
```

We can now plot both the `DataTrack` representation as well as
the `AnnotationTrack` representation of the `bam` file
together to prove that the underlying data are indeed identical.

```{r fileat4, fig.width=7.5, fig.height=2}
plotTracks(list(dTrack4, aTrack2), from = 189990000, to = 190000000)
```


### Display parameters for AnnotationTrack objects

For a complete listing of all the available display parameters please
see the table below or the man page of the `AnnotationTrack`
class by typing in `?AnnotationTrack` on the `R` command line.

```{r AnnotationTrackClassTable, echo=FALSE, results='asis'}
addParTable(xtabDetails,"AnnotationTrack", out_type = out_type)
```

## GeneRegionTrack

`GeneRegionTrack` objects are in principle very similar to
`AnnotationTrack` objects. The only difference is that they are
a little more gene/transcript centric, both in terms of plotting
layout and user interaction, and that they may define a global start
and end position (a feature which is not particularly relevant for the
normal user). The constructor function of the same name is a
convenient tool to instantiate the object from a variety of different
sources. In a nutshell, we need to pass start and end positions (or
the width) of each annotation feature in the track and also supply the
exon, transcript and gene identifiers for each item which will be used
to create the transcript groupings. A somewhat special case is to
build a `GeneRegionTrack` object directly from one of the
popular `TxDb` objects, an option that is treated in
more detail below. For more information about all the available
options see the class's manual page (`?GeneRegionTrack`).

There are a number of accessor methods that make it easy to query and
replace for instance exon, transcript or gene assignments. There is
also some support for gene aliases or gene symbols which are often
times more useful than cryptic data base gene identifiers. The
following code that re-uses the `GeneRegionTrack` object from
the first section exemplifies some of these features.

```{r generegtrackf, results='hide', fig.width=7.5, fig.height=1.5}
data(geneModels)
grtrack <- GeneRegionTrack(geneModels, genome = gen, chromosome = chr, 
                           name = "foo")
head(gene(grtrack))
head(transcript(grtrack))
head(exon(grtrack))
head(symbol(grtrack))
plotTracks(grtrack)
```

Both exon- and transcript-centric annotation can be added to the plot
using the `transcriptAnnotation` and `exonAnnotation`
display parameters.

```{r generegtrack2af, results='hide', fig.width=7.5, fig.height=2.5}
plotTracks(grtrack, transcriptAnnotation = "symbol")
```

```{r generegtrack2bf, results='hide', fig.width=7.5, fig.height=2.5}
plotTracks(grtrack, transcriptAnnotation = "transcript")
```

```{r generegtrack2cf, results='hide', fig.width=7.5, fig.height=1}
plotTracks(grtrack, exonAnnotation = "exon", extend.left = -0.8, 
           fontcolor.exon = 1)
```

Since we have the gene and transcript level information as part of our
`GeneRegionTrack` objects we can ask the package to collapse
all of our gene models from individual exons and transcripts down to
gene body locations by setting the `collapseTranscripts`
display parameter to `TRUE`.

```{r generegtrack3f, results='hide', fig.width=7.5, fig.height=1}
plotTracks(grtrack, collapseTranscripts = TRUE, shape = "arrow", 
           transcriptAnnotation = "symbol")
```

Collapsing down all the way to a gene is not always the desired
operation, and the parameters offers a bit more control. For instance
we could decide to plot just the longest transcript by setting it to
`longest`.

```{r generegtrack3g, results='hide', fig.width=7.5, fig.height=1}
plotTracks(grtrack, collapseTranscripts = "longest", shape = "arrow", 
           transcriptAnnotation = "symbol")
```

Or we can plot the union of all the exons in some sort of meta-transcript.

```{r generegtrack3h, results='hide', fig.width=7.5, fig.height=1}
plotTracks(grtrack, collapseTranscripts = "meta", shape = "arrow", 
           transcriptAnnotation = "symbol")
```

### Building GeneRegionTrack objects from TxDbs

The `r Biocpkg("GenomicFeatures")` packages provides an elegant framework
to download gene model information from online sources and to store it
locally in a SQLite data base. Because these so called
`TxDb` objects have become the de-facto standard for
genome annotation information in Bioconductor we tried to make it as
simple as possible to convert them into
`GeneRegionTracks`. Essentially one only has to call the
constructor function with the `TxDb` object as a single
argument. We exemplify this on a small sample data set that comes with
the `r Biocpkg("GenomicFeatures")` package.

```{r tdb2grt1}
library(GenomicFeatures)
samplefile <- system.file("extdata", "hg19_knownGene_sample.sqlite", 
                          package = "GenomicFeatures")
txdb <- loadDb(samplefile)
GeneRegionTrack(txdb)
```

In this context, the constructor's `chromosome`,
`start` and `end` argument take on a slightly different
meaning in that they can be used to subset the data that is fetched
from the `TxDb` object. Please note that while the
`chromosome` alone can be supplied, providing `start`
or `end` without the chromosome information will not work.

```{r tdb2grt2}
txTr <- GeneRegionTrack(txdb, chromosome = "chr6", 
                        start = 35000000,  end = 40000000)
```

A nice bonus when building `GeneRegionTracks` from
`TxDb` objects is that we get additional information
about coding and non-coding regions of the transcripts, i.e.,
coordinates of the 5' and 3' UTRs and of the CDS regions. The class'
plotting method can use this information to distinguish between coding
and non-coding regions based on the shape of the feature. All coding
regions are plotted just as we have seen in the previous examples,
whereas the non-coding regions are drawn as slightly thinner boxes. The
distinction between coding and non-coding is made on the basis of the
object's `feature` values in combination with a special display
parameter `thinBoxFeature` that enumerates all feature types that
are to be treated as non-coding. Obviously this feature is available
to all `GeneRegionTracks`, not only the ones that were build
from `TxDb` objects. However, the coding information
has to be added manually and the default value of the
`thinBoxFeature` parameter may not be sufficient to cover all
possible cases. It is up to the user to come up with a complete
list of non-coding feature types depending on the source of the data.

```{r generegtrack4f, results='hide', fig.width=7.5, fig.height=1}
feature(txTr)
plotTracks(txTr)
```

<!--
The `TxDb` do not provide gene symbols, however there is a possibility to set 
them using the `r Biocpkg("AnnotationDbi")` framework and corresponding 
`org.eg...db` package. For example here we can use `r Biocpkg("org.eg.Hs.db")`.

```{r generegtrack4g, eval=FALSE, echo=FALSE, results='hide', fig.width=7.5, fig.height=1}
library(org.Hs.eg.db)
symbol(txTr) <- mapIds(org.Hs.eg.db, 
                       keys=sub("\\.\\d+$", "", gene(txTr)), 
                       keytype="ENSEMBL", column="SYMBOL")
symbol(txTr) <- ifelse(is.na(symbol(txTr)), gene(txTr), symbol(txTr))
plotTracks(txTr, transcriptAnnotation = "symbol")
```
-->

### Building GeneRegionTrack objects from EnsDbs

The `r Biocpkg("Gviz")` is able to build `GeneRegionTrack` from `EnsDb` 
objects. They are similar to `TxDb` objects however their content is based on 
Ensembl Genome Browser. The `r Biocpkg("ensembldb")` package provides a complete 
framework to download gene model information from Ensembl Genome Browser and 
to store it locally in a SQLite database as well. For commonly used organisms and 
recent Ensembl versions, those databases are available directly in the 
`r Biocpkg("AnnotationHub")`. The `r Biocpkg("ensembldb")` package itself 
provides `getGeneRegionTrackForGviz` function to build the 
`GeneRegionTrack` object from the `EnsDb` objects.

In recent versions of `r Biocpkg("Gviz")` package, the `GeneRegionTrack`can be 
also build by calling the constructor function with the `EnsDb` object as a single 
argument. We exemplify this using the database provided by 
`r Biocpkg("EnsDb.Hsapiens.v75")` package.

It is important to mention that naming convention for chromosomes is different
between UCSC and Ensembl genome browsers. The `r Biocpkg("Gviz")` uses by 
default the convention from UCSC and therefore it is important to either:
 - set the chromosome names of `EnsDb` to UCSC style by calling 
   `seqlevelsStyle(edb) <- "UCSC"`
 - or set `Gviz` to accept non-UCSC style chromosome names by calling 
   `options(ucscChromosomeNames = FALSE)`


```{r ensDb, eval=FALSE}
library(ensembldb)
library(EnsDb.Hsapiens.v75)

edb <- EnsDb.Hsapiens.v75
seqlevelsStyle(edb) <- "UCSC"

eTrack <- GeneRegionTrack(edb, chromosome = "chr6", 
                          start = 35000000, end = 40000000)

plotTracks(eTrack)
```

### Display parameters for GeneRegionTrack objects

For a complete listing of all the available display parameters please
see the table below or the man page of the `GeneRegionTrack`
class by typing in `?GeneRegionTrack` on the `R` command line.

```{r GeneRegionTrackClassTable, echo=FALSE, results='asis'}
addParTable(xtabDetails,"GeneRegionTrack", out_type = out_type)
```

## BiomartGeneRegionTrack

As seen before it can be very useful to quickly download gene
annotation information from an online repository rather than having to
construct it each time from scratch. To this end, the `r Biocpkg("Gviz")` package
also defines the `BiomartGeneRegionTrack` class, which directly
extends `GeneRegionTrack` but provides a direct interface to
the ENSEMBL Biomart service (yet another interface to the UCSC data
base content is highlighted in one of the next sections). Rather than
providing all the bits and pieces for the full gene model, we just
enter a genome, chromosome and a start and end position on this
chromosome, and the constructor function
`BiomartGeneRegionTrack` will automatically contact ENSEMBL,
fetch the necessary information and build the gene model on the
fly. Please note that you will need an internet connection for this to
work, and that contacting Biomart can take a significant amount of
time depending on usage and network traffic. Hence the results are
almost never going to be returned instantaneously.

NOTE: `r Biocpkg("Gviz")` tries hard to automatically set up the necessary connection to
the Biomart service to retrieve the relevant data. Sometimes however
it can be advantageous to fully control the Biomart connector, for
instance when trying to retrieve data from an archived Ensembl version
(by default, `r Biocpkg("biomaRt")` will always link to the most recent
version). To that end, the `biomart` parameter can be used to
supply a `biomaRt` object. Please note that a data set has to
be selected in that object. In the example code below, the older version of human genome assembly (hg19) and matching Biomart version are used.

```{r BiomartGeneRegionTrackShow, eval=FALSE}
library(biomaRt)
bm <- useEnsembl(host = "https://grch37.ensembl.org", 
              biomart = "ENSEMBL_MART_ENSEMBL", 
              dataset = "hsapiens_gene_ensembl")
biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                    start = 20e6, end = 21e6,
                                    name = "ENSEMBL", biomart = bm)
plotTracks(biomTrack)
```
```{r BiomartGeneRegionTrackDo, echo=FALSE, results='hide', fig.width=7.5, fig.height=1.25}
library(biomaRt)
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org", 
                biomart = "ENSEMBL_MART_ENSEMBL", 
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                      start = 20e6, end = 21e6,
                                      name = "ENSEMBL", biomart = bm)
} else {
    data(biomTrack)
    biomTrack <- as(biomTrack, "GeneRegionTrack")
}
plotTracks(biomTrack)
```

You may have noticed in the above plot that the track includes
`feature` information which is displayed by the different feature
colors and box sizes. This information has been automatically
extracted from Biomart and the respective color coding is part of the
class' definition. We can highlight the feature classes even more by
using similarly colored bars to connect the grouped elements and by
dropping the bounding boxes around the individual exon models.

```{r BiomartGeneRegionTrackCol, fig.width=7.5, fig.height=1.25}
plotTracks(biomTrack, col.line = NULL, col = NULL)
```

For dense stacked plots like this it can sometimes be helpful to
control the vertical spacing between the stacking rows. A simple way
to do this is available for each object inheriting form the
`StackedTrack` class by means of the `stackHeight`
display parameter. It takes a value between 0 and 1 that controls
which fraction of the available vertical space on a stacking line
should be used to draw the feature glyphs.

```{r BiomartGeneRegionTrackHeight, fig.width=7.5, fig.height=1.25}
plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)
```

The `filter` argument to the
`BiomartGeneRegionTrack` constructor can be used to pass on
arbitrary additional filters to the Biomart query. We could for
instance limit the returned gene models to RefSeq models only. Please
see the documentation in the `r Biocpkg("biomaRt")` package for details.


```{r BiomartGeneRegionTrackFilterShow, eval=FALSE}
bm <- useEnsembl(host = "https://grch37.ensembl.org", 
              biomart = "ENSEMBL_MART_ENSEMBL", 
              dataset = "hsapiens_gene_ensembl")
biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                    start = 20e6, end = 21e6,
                                    name = "ENSEMBL", 
                                    filter = list(with_refseq_mrna=TRUE), 
                                    biomart = bm)
plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)
```
```{r BiomartGeneRegionTrackFilterDo, fig.width=7.5, fig.height=1.25, echo=FALSE, results='hide'}
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org",
                biomart = "ENSEMBL_MART_ENSEMBL",
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", chromosome = chr, 
                                      start = 20e6, end = 21e6,
                                      name = "ENSEMBL",
                                      filter = list(with_refseq_mrna=TRUE), 
                                      biomart = bm)
  plotTracks(biomTrack, col.line = NULL, col = NULL, stackHeight = 0.3)
} else {
  data(biomTrack2)
  biomTrack2 <- as(biomTrack2, "GeneRegionTrack")
  plotTracks(biomTrack2, col.line = NULL, col = NULL, stackHeight = 0.3)
}
```

Sometimes it can be more convenient to get a model for a particular
gene rather than defining a genomic range. One could use the filter
mechanisms as described before for this purpose, however the
`BiomartGeneRegionTrack` implements a more generic way. By
providing the gene symbol as the `symbol` argument, the track
will be initialized around the locus for this gene, if it can be
found. Alternatively, one can provide the Ensembl transcript or gene id
via the  `transcript` or `gene` arguments, or an Entrez id via
the `entrez` argument. In all of these cases the constructor
will also fetch models for other genes that overlap the same
region. If that is not desired one needs to provide an explicit
Biomart filter.

```{r BiomartGeneRegionTrackSymbolShow, eval=FALSE}
bm <- useEnsembl(host = "https://grch37.ensembl.org", 
              biomart = "ENSEMBL_MART_ENSEMBL",
              dataset = "hsapiens_gene_ensembl")
biomTrack <- BiomartGeneRegionTrack(genome = "hg19", name = "ENSEMBL", 
                                    symbol = "ABCB5", biomart = bm)
plotTracks(biomTrack, transcriptAnnotation = "symbol")
```
```{r BiomartGeneRegionTrackSymbolDo, fig.width=7.5, fig.height=1.25, echo=FALSE, results='hide'}
if(hasBiomartConnection) {
  bm <- useEnsembl(host = "https://grch37.ensembl.org", 
                biomart = "ENSEMBL_MART_ENSEMBL", 
                dataset = "hsapiens_gene_ensembl")
  biomTrack <- BiomartGeneRegionTrack(genome = "hg19", name = "ENSEMBL", 
                                      symbol = "ABCB5", biomart = bm)
} else {
    ranges(biomTrack) <- ranges(biomTrack)[symbol(biomTrack) == "ABCB5"]
}
plotTracks(biomTrack, transcriptAnnotation="symbol")
```

<!--
### Using custom biomaRt objects for BiomartGeneRegionTracks
`r Biocpkg("Gviz")` tries hard to automatically set up the necessary connection to
the Biomart service to retrieve the relevant data. Sometimes however
it can be advantageous to fully control the Biomart connector, for
instance when trying to retrieve data from an archived Ensembl version
(by default, `r Biocpkg("biomaRt")` will always link to the most recent
version). To that end, the `biomart` parameter can be used to
supply a `biomaRt` object. Please note that a data set has to
be selected in that object. A second prerequisite for this to work is
that `r Biocpkg("Gviz")` is able to guess which of the values that are queried
from the Ensembl data base represent which track feature. If there
were no changes in the schema of Ensembl, or if the schema for all
marts was similar this would not be an issue, but that is not the
case. In the following example we show how to fetch data for the same
region from the older version Ensembl version 47. The internal
`.getBMFeatureMap` is helpful to set up the feature mapping
which is necessary because Ensembl used to call the gene symbol field
`external_gene_id` in that release.

```{r BiomartGeneRegionTrackCustom, fig.width=7.5, fig.height=1.25, eval=FALSE, echo=FALSE}
library(biomaRt)
bm <- useEnsembl(host="dec2012.archive.ensembl.org", biomart="ENSEMBL_MART_ENSEMBL",
              dataset="hsapiens_gene_ensembl")
fm <- Gviz:::.getBMFeatureMap()
fm[["symbol"]] <- "external_gene_id"
biomTrack <- BiomartGeneRegionTrack(genome="hg19", chromosome="chr7", start=20e6, end=21e6,name="ENSEMBL",
                                    featureMap=fm, biomart=bm)
plotTracks(biomTrack, col.line=NULL, col=NULL, stackHeight=0.3)
```
-->

### Display parameters for BiomartGeneRegionTrack objects

For a complete listing of all the available display parameters please
see the table above in the previous `GeneRegionTrack` section
or the man page of the `BiomartGeneRegionTrack` class by typing
in `?BiomartGeneRegionTrack` on the `R` command line.

As mentioned above, one additional benefit when fetching the data
through Biomart is that we also receive some information about the
annotation feature types, which is automatically used for the color
coding of the track. The following table shows the available feature
types.

```{r BiomartGeneRegionTrackClassTable, echo=FALSE, results='asis'}
addInfo <- t(data.frame(displayPars(biomTrack, names(details[["BiomartGeneRegionTrack"]]))))
colnames(addInfo) <- "Color"
addParTable(xtabDetails,"BiomartGeneRegionTrack", add=addInfo, out_type = out_type)
```

## DetailsAnnotationTrack

It is sometimes desirable to add more detailed information to
particular ranges in an `r Biocpkg("Gviz")` plot for which the notion of genomic
coordinates no longer makes sense. For instance, the ranges in an
`AnnotationTrack` may represent probe locations on a genome,
and for each of these probes a number of measurements from multiple
samples and from different sample groups are available. To this end,
the `DetailsAnnotationTrack` provides a flexible interface to
further annotate genomic regions with arbitrary additional
information. This is archived by splitting the
`AnnotationTrack` plotting region into two horizontal sections:
the lower section containing the range data in genomic coordinates, and
the upper one containing the additional data for each of the displayed
ranges in vertically tiled panels of equal size. The connection between
a range item and its details panel is indicated by connecting
lines.

The content of the individual details panels has to be filled in by a
user-defined plotting function that uses grid (or lattice) plotting
commands. This function has to accept a number of mandatory
parameters, notably the start, end, strand, chromosome and identifier
information for the genomic range, as well as an integer counter
indicating the index of the currently plotted details tile. This
information can be used to fetch arbitrary details, e.g. from a list,
and environment or even from a `GRanges` object which will
then be processed and visualized within the plotting function. This
may sound rather abstract, and for more details please refer to the
class' help page. For now we just want to demonstrate the
functionality in a simple little example. We begin by defining a
`GRanges` object containing 4 genomic locations. In our example
those are considered to be probe locations from a methylation array.

```{r DetailsAnnotationTrack1}
library(GenomicRanges)
probes <- GRanges(seqnames = "chr7", ranges = IRanges(
      start = c(2000000, 2070000, 2100000, 2160000),
      end = c(2050000, 2130000, 2150000, 2170000)), 
  strand = c("-", "+", "-", "-"))
```

For each of these probes we have methylation measurements from a large
number of different samples in a numeric matrix, and within the
samples there are two treatment groups. The aim is to compare the
distribution of measurement values between these two groups at each
probe locus.

```{r DetailsAnnotationTrack2}
methylation <- matrix(c(rgamma(400, 1)),
                  ncol = 100,
                  dimnames = list(paste("probe", 1:4, sep = ""), NULL))
methylation[, 51:100] <- methylation[, 51:100] + 0:3
sgroups <- rep(c("grp1", "grp2"), each = 50)
```

Of course we could use a `DataTrack` with the box-plot
representation for this task, however we do have strand-specific data
here and some of the probes can be overlapping, so all this
information would be lost. We are also interested in the particular
shape of the data distribution, so a density plot representation is
what we really need. Luckily, the `r Rpackage("lattice")` package gives us
a nice `densityplot` function that supports grouping, so all
that's left to do now is to write a little wrapper that handles the
extraction of the relevant data from the matrix. This is easily
achieved by using the range identifiers, which conveniently map to
the row names of the data matrix.

```{r DetailsAnnotationTrack3}
library(lattice)
details <- function(identifier, ...) {
  d <- data.frame(signal = methylation[identifier, ], group = sgroups)
  print(densityplot(~signal, group = group, data = d, 
                    main = list(label = identifier, cex = 0.7),
                    scales = list(draw = FALSE, x = list(draw = TRUE)), 
                    ylab = "", xlab = ""), 
        newpage = FALSE, prefix = "plot")
}
```

Finally, it is as simple as calling the `AnnotationTrack`
constructor, passing along the wrapper function and calling
`plotTracks`.

```{r DetailsAnnotationTrack4, results='hide', fig.width=7.5, fig.height=5}
deTrack <- AnnotationTrack(range = probes, genome = "hg19", 
                           chromosome = 7, id = rownames(methylation),
                           name = "probe details", stacking = "squish",
                           fun = details)
plotTracks(deTrack)
```

It should be noted here that in our little example we rely on the
methylation data matrix and the grouping vector to be present in the
working environment. This is not necessarily the cleanest solution and
one should consider storing additional data in an environment, passing
it along using the `detailFunArgs` parameter, or making it
part of the details function in form of a closure. The class' help
page provides further instructions.

Another use case for the `DetailsAnnotationTrack` class is to
deal with the problem of very different feature sizes within a single
track. For instance, we may be looking at a rather large genomic
region containing one big transcript with many widely spaced exons and
a bunch of smaller, more compact transcripts. In this case it would be
helpful to provide a zoomed in version of those smaller
transcripts. In order to achieve this we can make use of the class'
`groupDetails` display parameter, which applies the detail
plotting function over each range group rather than over individual
range items. First we define a function that selects those groups with
a plotted size smaller than 10 pixels. We make use of the unexported
function `.pxResolution` here to come up with the mapping between
pixel coordinates and genomic coordinates.

```{r DetailsAnnotationTrack5}
selFun <- function(identifier, start, end, track, GdObject, ...){
    gcount <- table(group(GdObject))
    ## This computes the width of 2 pixels in genomic coordinates
    pxRange <- Gviz:::.pxResolution(min.width = 20, coord = "x")
    return((end - start) < pxRange && gcount[identifier] == 1)
}
```

The actual detail plotting function is fairly trivial. In the details
viewport we simply call `plotTracks` function on the subset of
items from each group that has been selected before without plotting
the track titles, and also add a scale indicator for some
reference. It is worth mentioning however that we need to extract the
ranges of the zoomed in track items from the unmodified track object
(the `Gdobject.original` argument in the details function)
because the object that is plotted in the lower panel contains only
the collapsed ranges.

```{r DetailsAnnotationTrack6}
detFun <- function(identifier, GdObject.original, ...){
  plotTracks(list(GenomeAxisTrack(scale = 0.3, size = 0.2, cex = 0.7), 
        GdObject.original[group(GdObject.original) == identifier]),
        add = TRUE, showTitle = FALSE)
}
```

Finally, we load some sample data, turn it into a `DetailsAnnotationTrack` object and plot it.

```{r DetailsAnnotationTrack7f, results='hide', fig.width=7.5, fig.height=2}
data(geneDetails)
deTrack2 <- AnnotationTrack(geneDetails, fun = detFun, 
                            selectFun = selFun,
                            groupDetails = TRUE, details.size = 0.5, 
                            detailsConnector.cex = 0.5, 
                            detailsConnector.lty = "dotted",
                            shape = c("smallArrow", "arrow"), 
                            groupAnnotation = "group")
plotTracks(deTrack2, extend.left = 90000)
```


### Display parameters for DetailsAnnotationTrack objects

In addition to the display parameters for the `AnnotationTrack`
class, some additional parameters can be used to control the look and
feel of the details sections. For a complete listing of all the available display parameters please
see the tables below and the one above in the `AnnotationTrack` section
or the man page of the `DetailsAnnotationTrack` class by typing
in `?DetailsAnnotationTrack` on the `R` command line.

```{r DetailsAnnotationTrack8, results='hide', fig.width=7.5, fig.height=3}
plotTracks(deTrack, details.size = 0.75, detailsConnector.pch = NA, 
           detailsConnector.col = "darkred", 
           detailsBorder.fill = "#FFE3BF", 
           detailsBorder.col = "darkred", shape ="box", 
           detailsConnector.lty = "dotted")
```

```{r DetailsAnnotationTrackClassTableSec, echo=FALSE, results='asis'}
addParTable(xtabDetails,"DetailsAnnotationTrack", out_type = out_type)
```

## SequenceTrack

So far we have displayed all kinds of ranges on a genome, but never
really looked at the underlying genomic sequence. To that end, the
`r Biocpkg("Gviz")` package defines the `SequenceTrack` class which can hold
genomic sequence information derived from either a
`DNAStringSet` object of, even more convenient, from one of the
`BSgenome` packages. There is not formal checking of the
track's genome, so it is up to the user to provide reasonable inputs
to the class' constructor. As with all the other track types, the
constructor has the same name as the class. In this example we build a
`SequenceTrack` from the human hg19 UCSC genome.

```{r SequenceTrack1}
library(BSgenome.Hsapiens.UCSC.hg19)
sTrack <- SequenceTrack(Hsapiens)
sTrack
```

Here we retain the same benefits as for regular `BSgenome`
objects in that the actual sequence is only loaded into memory when a
particular chromosome is first accessed. Plotting the track yields the
expected result: a character representation of the genomic sequence in
the current plot window. It may be worth noting that the color scheme
that is used to encode the individual nucleotides is defined in the
`r Biocpkg("biovizBase")` package and the user is referred to its
documentation for more details.

```{r SequenceTrack2, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050)
```

Sometimes it may be too busy to plot both the sequence letters and the
color coding, and we can assign arbitrary colors by setting the
`fontcolor` display parameter. The convention here is that we
need to supply a named vector, with one entry for each of the five
possible nucleotides (A, C, T, G, and N). If any of the entries is
missing, the respective letter will not be drawn at all.

```{r SequenceTrack3, results='hide', fig.width=7.5, fig.height=0.5}
fcol <- c(A="darkgray", C="darkgray", T="darkgray", G="darkgray")
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050, 
           fontcolor = fcol)
```

In this case we are plotting the sequence on the forward strand and we
may want to make this clear by adding direction indicators.

```{r SequenceTrack4, results='hide', fig.width=7.5, fig.height=0.5}
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050,
           add53 = TRUE)
```

If instead we plot the complement sequence on the reverse strand, the
indicators are automatically adjusted.

```{r SequenceTrack5, results='hide', fig.width=7.5, fig.height=0.3}
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20050, 
           add53 = TRUE, complement = TRUE)
```

So far we have been able to fit the sequence onto our plotting device
without overplotting. I.e., all the letters nicely fit in to their
respective position in the genomic coordinate system. However this
restricts us to very small windows which we can reasonably display. A
much more compact version of the same information is to use colored
boxes rather than individual letters. Those boxes can be stacked much
closer together, which increases the possible window size quite a
bit. The user does not really need to worry about this as the plotting
method will automatically make a reasonable decision based on the
available space.

```{r SequenceTrack6, results='hide', fig.width=7.5, fig.height=0.3}
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20100)
```

For added flexibility one can set the `noLetters` display
parameter to `TRUE` to always force the drawing of boxes.  Of
course also the colored box representation of a sequence has its
limits, and if we cross the threshold when individual boxes can not be
separated anymore, the plotting method falls back to drawing a single
line indicating the presence of a sequence at the given position.

```{r SequenceTrack7, results='hide', fig.width=7.5, fig.height=0.3}
plotTracks(sTrack, chromosome = 1, from = 20000, to = 201000)
```

Finally, the selected font size is also contributing to the available
space, and we can cram a little more sequence into a given window by
decreasing it.

```{r SequenceTrack8, results='hide', fig.width=7.5, fig.height=0.3}
plotTracks(sTrack, chromosome = 1, from = 20000, to = 20100, cex = 0.5)
```

### Display parameters for SequenceTrack objects

For a complete listing of all the available display parameters please
see the table below or the man page of the `SequenceTrack`
class by typing in `?SequenceTrack` on the `R` command line.

```{r SequenceTrackClassTableSec, echo=FALSE, results='asis'}
addParTable(xtabDetails,"SequenceTrack", out_type = out_type)
```

## AlignmentsTrack

Plots of aligned sequences, typically from next generation sequencing
experiments can be quite helpful, for instance when visually
inspecting the validity of a called SNP. Those alignments are usually
stored in `BAM` files.  As shown in some of the previous section
we can deal with these files in a fairly crude way by coupling them to
either an `AnnotationTrack` or a `DataTrack`. The utility
of the generated plots however is fairly limited, and we want in
particular support for gapped and paired alignments, and show
differences of the individual read sequences to a reference.

The `AlignmentsTrack` class is designed to help in exactly
these tasks. In its typical manifestation it is also a member of the
`ReferenceTrack` class, enabling memory efficient streaming
directly off a file. Even though this file does not necessarily have
to be a `BAM` file, the automatic import will only work for
those. For all other file representations of alignments a user will
have to provide their own import function, and this topic is further
discussed in the class' documentation. For this demonstration let's use
a small `BAM` file for which paired NGS reads have been mapped to
an extract of the human hg19 genome. The data originate from an RNASeq
experiment, and the alignments have been performed using the
`STAR` aligner allowing for gaps. We also download some gene
annotation data for that region from Biomart.

Again, creating the `AlignmentsTrack` object is as simple as
calling the constructor function of the same name. We set the
`isPaired` argument to `TRUE` in order to tell the
constructor that it should expect paired reads. Please note that
`TRUE` is the default value for this parameter and usually things
will still work even if the reads do not originate from a paired end
library. In other words, the default setting could be used as a
poor-man's auto-detection, however it make more sense to supply the
argument explicitly if you know what type of data to expect.

```{r alignmentstrack_1_do, echo=FALSE, results='hide'}
afrom <- 2960000
ato <- 3160000
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "gapped.bam"), 
  isPaired = TRUE)
data(alTrackGenes)
```

```{r alignmentstrack_1_show, eval=FALSE}
afrom <- 2960000
ato <- 3160000
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "gapped.bam"),
  isPaired = TRUE)
bmt <- BiomartGeneRegionTrack(genome = "hg19", chromosome = "chr12",
                              start = afrom, end = ato,
                              filter = list(with_ox_refseq_mrna = TRUE),
                              stacking = "dense")
```

We can create a first plot of our read alignments data from a birds
eye perspective, say a range of 20,000 nucleotides.

```{r alignmentstrack_2, results='hide', fig.width=7.5, fig.height=5}
plotTracks(c(bmt, alTrack), from = afrom, to = ato, 
           chromosome = "chr12")
```

Now this already shows us the general layout of the track: on top we
have a panel with the read coverage information in the form of a
histogram, and below that a pile-up view of the individual
reads. There is only a certain amount of vertical space available for
the plotting, and not the whole depth of the pile-up can be displayed
here. This fact is indicated by the white downward-pointing arrows in
the title panel. We could address this issue by playing around with
the `max.height`, `min.height` or
`stackHeight` display parameters which all control the height
or the vertical spacing of the stacked reads. Or we could reduce the
size of the coverage section by setting the `coverageHeight`
or the `minCoverageHeight` parameters. Please see the class
documentation for more details.

```{r alignmentstrack_3, results='hide', fig.width=7.5, fig.height=5}
plotTracks(c(bmt, alTrack), from = afrom, to = ato,
           chromosome = "chr12", min.height = 0, 
           coverageHeight = 0.08, minCoverageHeight = 0)
```

From that far out the pile-ups are not particularly useful, and we
can turn those off by setting the `type` display parameter
accordingly.

```{r alignmentstrack_4, results='hide', fig.width=7.5, fig.height=2}
plotTracks(c(alTrack, bmt), from = afrom, to = ato, 
           chromosome = "chr12", type = "coverage")
```

Let's zoom in a bit further to check out the details of the pile-ups
section.

```{r alignmentstrack_5, results='hide', fig.width=7.5, fig.height=5}
plotTracks(c(bmt, alTrack), from = afrom + 12700, 
           to = afrom + 15200, chromosome = "chr12")
```

The direction of the individual  reads is indicated by the arrow head,
and  read  pairs are  connected by  a bright  gray  line.  Gaps in
the alignments are show by the connecting dark gray lines. Numbers of
reads supporting each gap can be visualized as "sashimi plot"
[@Katz22012015] by adding `'sashimi'` as an additional
argument to the `type` display parameter.

The axis in the title panel shows the relationship between the line
width the the number of reads spanning a given junction. The height of
each arc is not informative, it is optimized to fit all overlapping
junctions in a small window.

```{r alignmentstrack_5_1, results='hide', fig.width=7.5, fig.height=2}
plotTracks(c(bmt, alTrack), from = afrom + 12700, 
           to = afrom + 15200, chromosome = "chr12",
           type = c("coverage", "sashimi"))
```

It is also possible to filter the junctions by including the
coordinates as an additional argument `sashimiFilter`. In
this case only junction which equally overlaps with specified
`GRanges` object are shown.

```{r alignmentstrack_5_2, results='hide', fig.width=7.5, fig.height=2}
introns <- GRanges("chr12", IRanges(start = c(2973662, 2973919),
                                    end = c(2973848, 2974520)))
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12", type = c("coverage", "sashimi"), 
           sashimiFilter = introns)
```

However sometimes the aligners do not place the junction reads
precisely on the exon-intron boundaries and it is therefore useful to
include also junctions which are shifted by few bases. This can be done
by setting the additional argument
`sashimiFilterTolerance` to small positive integer value.

```{r alignmentstrack_5_3, results='hide', fig.width=7.5, fig.height=2}
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12", type = c("coverage", "sashimi"), 
           sashimiFilter = introns, sashimiFilterTolerance = 5L)
```

On devices that support transparency we  can also see that some of
the read pairs are actually overlapping. Like  before, we can not
display  the full depth of the pile-up,  and the clipping that takes
place hides  a lot of the reads from  the view. Another way of
getting to see at  least some of them is by reversing the stacking
order. We can also play around with some of the other display
parameters that control the look and feel of the reads.

```{r alignmentstrack_6, results='hide', fig.width=7.5, fig.height=5}
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200,
           chromosome = "chr12", reverseStacking = TRUE,
           col.mates = "purple", col.gap = "orange", type = "pileup")
```

As mentioned before we can control whether the data should be treated
as paired end or single end data by setting the `isPaired`
argument in the constructor. Here is how we could take a look at the
data in the same file, but in single end mode.

```{r alignmentstrack_6_1, results='hide', fig.width=7.5, fig.height=5}
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "gapped.bam"), 
  isPaired = FALSE)
plotTracks(c(bmt, alTrack), from = afrom + 12700, to = afrom + 15200, 
           chromosome = "chr12")
```

So far we did not look at the individual read sequences at all. Even
though they are contained in the `BAM` file, their value is
fairly limited without the reference sequence against which the reads
have initially been aligned. Typically the mismatch positions and the
variant bases are not included. (Please note that this is not entirely
true for all `BAM` files. One can store base mismatch positions
in the CIGAR string, or even the full variant description in the
non-standard `MD` field. However very few aligners currently make
use of these features. We may implement support for those later if
found necessary.)

To better show the features of the `AlignmentsTrack` for
sequence variants we will load a different data set, this time from a
whole genome DNASeq SNP calling experiment. Again the reference genome
is hg19 and the alignments have been performed using `Bowtie2`.

```{r alignmentstrack_7, results='hide', fig.width=7.5, fig.height=5}
afrom <- 44945200
ato <- 44947200
alTrack <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "snps.bam"), isPaired = TRUE)
plotTracks(alTrack, chromosome = "chr21", from = afrom, to = ato)
```

Somehow we need to tell the `AlignmentsTrack` about the
reference genome, and there are two ways to archive this. Either we
provide a `SequenceTrack` object with the reference to the
constructor function via the `referenceSequence` argument, or
we stick it into the track list that is provided to the
`plotTrack` which is smart enough to detect it there. For
this example we chose the second option and re-use the
`SequenceTrack` object from the previous section.

```{r alignmentstrack_8, results='hide', fig.width=7.5, fig.height=5}
plotTracks(c(alTrack, sTrack), chromosome = "chr21", 
           from = afrom, to = ato)
```

The mismatched bases are now indicated on both the individual reads in
the pileup section and also in the coverage plot in the form of a
stacked histogram. When zooming in to one of the obvious heterozygous
SNP positions we can reveal even more details.

```{r alignmentstrack_9, results='hide', fig.width=7.5, fig.height=5}
plotTracks(c(alTrack, sTrack), chromosome = "chr21",
           from = 44946590, to = 44946660)
```

Just like shown before in the `SequenceTrack` we can even
show individual letters here as long as there is enough space to fit
them. This can be controlled using the display parameters
`min.height`, `max.height` and `cex`.

```{r alignmentstrack_10, results='hide', fig.width=7.5, fig.height=5}
plotTracks(c(alTrack, sTrack), chromosome = "chr21", from = 44946590, 
           to = 44946660, cex = 0.5, min.height = 8)
```

It is also possible to visualize the "indels" present in the alignments. This 
can be achieved by setting the `showIndels` parameter to `TRUE`. The coverage 
plot will no longer count the part of the reads corresponding to deletions. The
appearance of insertions and deletions in pile-up track can be defined by 
setting the color, line width and line type (`col.deletion`, `col.insertion`, 
`lty.deletion`, `lty.insertion`, `lwd.deletion`, `lwd.insertion`). By default 
the insertions are indicated with violet vertical bars and the deletions 
are shown as black connecting lines.

```{r alignmentstrack_11, results='hide', fig.width=7.5, fig.height=5}
indelTrack1 <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "indels.bam"), 
  name = "Standard")
indelTrack2 <- AlignmentsTrack(
  system.file(package = "Gviz", "extdata", "indels.bam"), 
  showIndels=TRUE, name="Indels")
plotTracks(c(indelTrack1, indelTrack2), 
           chromosome = "chr2", from = 126442000, to = 126453000)
```

### Display parameters for AlignmentsTrack objects

For a complete listing of all the available display parameters please
see the table below or the man page of the `AlignmentsTrack`
class by typing in `?AlignmentsTrack` on the `R` command line.

```{r AlignmentsTrackClassTable, echo=FALSE, results='asis'}
addParTable(xtabDetails,"AlignmentsTrack", out_type = out_type)
```

# Creating tracks from UCSC data

The UCSC data bases contain a multitude of genome annotation data for
dozens of different organisms. Some of those data are very simple
annotations like CpG island locations or SNP locations. Others are
more complicated gene models, or even numeric annotations like
conservation information. In order to provide a unified interface to
all this information, the `r Biocpkg("Gviz")` package defines a meta-constructor
function `UcscTrack`. The idea here is that we can express
all of the available UCSC data in one of the package's track types. We
use the functionality provided in the `r Biocpkg("rtracklayer")` package
to connect to UCSC and to download the relevant information. As a
little illustrative example, let's reproduce a view from the famous
UCSC genome browser using the `r Biocpkg("Gviz")` package. As a final result we want
to show something similar to Figure \@ref(fig:UCSC1).

```{r UCSC1, echo=FALSE,fig.cap="A screen shot of a UCSC genome browser view around the FMR1 locus on the mouse chromosome."}
include_graphics("ucsc1.png")
```

To start we first need to know about the available data in the UCSC
data base and about their structure. A good way to do this is to use
the table browser on the [UCSC web site](http://genome.ucsc.edu/cgi-bin/hgTables?command=start). 
Figure \@ref(fig:UCSC2) shows the table structure for the first gene model track, the known
UCSC genes, in the table browser. We can see that there are multiple
fields, some with genomic locations, other with additional data like
labels or identifiers. If we go back to the section about the
`GeneRegionTrack` class we remember that we need exactly this
type of information for the constructor function. So in order to take
the UCSC data and build an object of class `GeneRegionTrack` we
need a way to map them to the individual constructor arguments. This
is exactly what the `UcscTrack` meta-constructor is supposed
to do for us.

```{r UCSC2, echo=FALSE, fig.cap="A screen shot of a UCSC table browser view on the UCSC Known Genes track."}
include_graphics("ucsc2.png")
```

It needs to know about the track for which to extract the data (and
optionally one or several of the tables that make up the collective
track data, see `?UcscTrack` for details), about the genomic
range including the chromosome for which to extract data, about the
type of `r Biocpkg("Gviz")` track that we want to translate this data into, and about
the individual track columns and their counterparts in the respective
track class constructor. In our example, the track is called
`knownGene`, the track type to construct is
`GeneRegionTrack`, and the relevant columns are
`exonStarts`, `exonEnds`, `name` and `strand`,
which we will use as the start and end coordinates of the ranges and
for all the exon, transcript and gene identifiers. Here we make use of
the high flexibility of the `GeneRegionTrack` constructor in
the sense that the exon coordinates actually come in the form of a
comma-separated list, combining all the information for one transcript
in one row of the table. The function is smart enough to detect this
and to split the annotation regions accordingly. The full function
call to create the `GeneRegionTrack` from the UCSC data looks
like this:

```{r ucscTrack1, eval=FALSE}
from <- 65921878
to <- 65980988
knownGenes <- UcscTrack(genome = "mm9", chromosome = "chrX", 
                        track = "UCSC Genes", table="knownGene", 
                        from = from, to = to,
                        trackType = "GeneRegionTrack", 
                        rstarts = "exonStarts", rends = "exonEnds", 
                        gene = "name", symbol = "name", 
                        transcript = "name", strand = "strand", 
                        fill = "#8282d2", name = "UCSC Genes")
```

With a similar approach we can construct the next two gene model
tracks based on the `xenoRefGene` and `ensGene` data tables.

```{r ucscTrack2, eval=FALSE}
refGenes <- UcscTrack(genome = "mm9", chromosome = "chrX",
                      track="Other RefSeq", table = "xenoRefGene", 
                      from = from, to = to,
                      trackType = "GeneRegionTrack", 
                      rstarts = "exonStarts", rends = "exonEnds", 
                      gene = "name",  symbol = "name2", 
                      transcript = "name", strand = "strand",
                      fill = "#8282d2", stacking = "dense", 
                      name = "Other RefSeq")

ensGenes <- UcscTrack(genome = "mm9", chromosome = "chrX", 
                      track="Ensembl Genes", table = "ensGene", 
                      from = from, to = to,
                      trackType = "GeneRegionTrack", 
                      rstarts = "exonStarts", rends = "exonEnds",
                      gene = "name", symbol = "name2", 
                      transcript = "name", strand = "strand", 
                      fill = "#960000", name = "Ensembl Genes")
```

The CpG and SNP tracks are slightly different since a
`GeneRegionTrack` representation would not be particularly
useful. Instead, we can use `AnnotationTrack` objects as
containers. The overall process using the `UcscTrack`
meta-constructor remains the same.

```{r ucscTrack3, eval=FALSE}
cpgIslands <- UcscTrack(genome = "mm9", chromosome = "chrX", 
                        track="CpG Islands", table = "cpgIslandExt", 
                        from = from, to = to,
                        trackType = "AnnotationTrack", 
                        start = "chromStart", end = "chromEnd", 
                        id = "name", shape = "box", fill = "#006400", 
                        name = "CpG Islands")

snpLocations <-  UcscTrack(genome = "mm9", chromosome = "chrX", 
                           track="SNPs (128)", table = "snp128", 
                           from = from, to = to,
                           trackType = "AnnotationTrack", 
                           start = "chromStart", end = "chromEnd", 
                           id = "name", feature = "func", 
                           strand = "strand", shape = "box", 
                           stacking = "dense", fill = "black",
                           name = "SNPs")
```

Most of UCSC's `DataTrack`-like tracks are a little more
complex and represent a collection of several sub-tracks, with data
originating from multiple tables. To make sure that we get the correct
information we have to be a little bit more specific here and also
define the particular table on the UCSC data base to use.

```{r ucscTrack4, eval=FALSE}
conservation <- UcscTrack(genome = "mm9", chromosome = "chrX", 
                          track = "Conservation", 
                          table = "phyloP30wayPlacental",
                          from = from, to = to, trackType = "DataTrack", 
                          start = "start", end = "end", data = "value",
                          type = "hist", window = "auto", 
                          col.histogram = "darkblue", 
                          fill.histogram = "darkblue",
                          ylim = c(-3.7, 4), name = "Conservation")

gcContent <- UcscTrack(genome = "mm9", chromosome = "chrX", 
                       track = "GC Percent", table = "gc5Base",
                       from = from, to = to, trackType = "DataTrack", 
                       start = "start", end = "end", data = "value",
                       type = "hist", window = -1, windowSize = 1500, 
                       fill.histogram = "black", col.histogram = "black",
                       ylim = c(30, 70), name = "GC Percent")
```

To add some reference points we also need a genome axis and an
`IdeogramTrack` of the x chromosome.

```{r ucscTrack5, eval=FALSE}
axTrack <- GenomeAxisTrack()
idxTrack <- IdeogramTrack(genome="mm9", chromosome="chrX")
```

And finally we can plot all of our tracks.

```{r ucscTrackLoad, echo=FALSE, results='hide'}
data(ucscItems)
```

```{r ucscTrack6, results='hide', fig.width=7.5, fig.height=5.5}
plotTracks(list(idxTrack, axTrack, knownGenes, refGenes, ensGenes, 
                cpgIslands, gcContent, conservation, snpLocations), 
           from = from, to = to, showTitle = FALSE)
```

# Track highlighting and overlays

## Highlighting

Some features of a typical genome browser plot are not specific for
just a single track, but instead span multiple tracks. One prominent
example are highlighting regions. Assume you want to draw the
attention on the plot to a particular genomic stretch where the visual
clues across several or maybe even all tracks indicate a particular
interesting behavior. Because the individual tracks in the `r Biocpkg("Gviz")`
package are all represented by individual objects, there is no
straight forward way to define such a region. Here is where the
`HighlightingTrack` class comes into play. Objects from this
class are essentially just containers for one or several of the
previously described track objects, together with a very simple range
definition that is to be superimposed over the individual tracks once
they have been rendered. Going back to our final example from the
second chapter we could highlight two areas on the plot on the
`AnnotationTrack`, the `GeneRegionTrack` and the
`DataTrack` using the `HighlightTrack` constructor in
the following call:

```{r HighlightTrack, fig.width=7.5, fig.height=4}
ht <- HighlightTrack(trackList = list(atrack, grtrack, dtrack),
                     start = c(26705000, 26720000), width = 7000,
                     chromosome = 7)
plotTracks(list(itrack, gtrack, ht), from = lim[1], to = lim[2])
```

One can also build multiple `HighlightTrack` collections to
create gaps in the highlighting regions. For `IdeogramTracks`
we never plot highlighting ranges because they do not live on the same
x-axis coordinate system like the other tracks.

```{r HighlightTrack2, fig.width=7.5, fig.height=4}
ht1 <- HighlightTrack(trackList=list(itrack, gtrack, atrack), 
                      start = c(26705000, 26720000), width = 7000, 
                      chromosome = 7)
ht2 <- HighlightTrack(trackList = dtrack, start = c(26705000, 26720000),
                      width = 7000, chromosome = 7)
plotTracks(list(ht1, grtrack, ht2), from = lim[1], to = lim[2])
```

### Display parameters for HighlightTrack objects

For a complete listing of all the available display parameters please
see the table below or the man page of the `HighlightTrack`
class by typing in `?HighlightTrack` on the `R` command line.

```{r HighlightTrackClassTable, echo=FALSE, results='asis'}
addParTable(xtabDetails,"HighlightTrack", out_type = out_type)
```

## Overlays

For certain applications it can make sense to overlay multiple tracks
on the same area of the plot. For instance when dealing with multiple
data sets that do not necessarily share the same genomic coordinates
for their data points the design of the `DataTrack` class is
not optimal. To contrast these two data sets an overlay of two
separate `DataTrack` objects however can be quite powerful.The
`r Biocpkg("Gviz")` package allows for such overlay operations by means of the
`OverlayTrack` class. Similar to the `HighlightTrack`
class from the previous section this is merely a container for an
arbitrary number of other `r Biocpkg("Gviz")` track objects. For the purpose of an
instructive example we will generate a second `DataTrack`
object and combine it with the existing one from the second chapter.

```{r OverlayTrack, fig.width=7.5, fig.height=3}
dat <- runif(100, min = -2, max = 22)
dtrack2 <- DataTrack(data = dat, start = coords[-length(coords)], 
                end = coords[-1], chromosome = chr, genome = gen, 
                name = "Uniform2",  groups = factor("sample 2", 
                  levels = c("sample 1", "sample 2")), legend = TRUE)
displayPars(dtrack) <- list(groups = factor("sample 1", 
              levels = c("sample 1", "sample 2")), legend = TRUE)
ot <- OverlayTrack(trackList=list(dtrack2, dtrack))
ylims <- extendrange(range(c(values(dtrack), values(dtrack2))))
plotTracks(list(itrack, gtrack, ot), from = lim[1], to = lim[2], 
           ylim = ylims, type = c("smooth", "p"))
```

It is important to know that `r Biocpkg("Gviz")` is fully unaware of the intention
and any specifics of the track merging operation. Any track type can
be merged with any other track type, and there are no limitations on
the number of overlays. It is totally up to the user to design
reasonable merges and to control critical aspects of the plot, like
the y-axis scaling or the grouping in the above example. During
rendering, the software will only use the first track object in the
`trackList` container of `OverlayTracks` object to set
up the track title. Thus, the ranges on the y-axis are those that have
been automatically determined for the first of our two
`DataTracks`. Only by forcing a common y-range for both objects
do we actually plot the data from both tracks on a common scale. Had
we added another track object that is not a `DataTrack` is the
first element of our `trackList` there wouldn't even be an
axis. In a similar fashion, by nicely setting up the group levels for
the two `DataTracks` we can even ask for a legend to be added.

On devices that support it, alpha blending can be a useful tool to
tease out even more information out of track overlays, at least when
comparing just a small number of samples. The resulting transparency
effectively eliminates the problem of overplotting. The following
example will only work if this vignette has been built on a system
with alpha blending support.

```{r OverlayTrack2, fig.width=7.5, fig.height=3}
displayPars(dtrack) <- list(alpha.title = 1, alpha = 0.5)
displayPars(dtrack2) <- list(alpha.title = 1, alpha = 0.5)
ot <- OverlayTrack(trackList = list(dtrack, dtrack2))
plotTracks(list(itrack, gtrack, ot), from = lim[1], to = lim[2], 
           ylim = ylims, type = c("hist"), window = 30)
```



# Composite plots for multiple chromosomes

As mentioned in the introduction section, a set of `r Biocpkg("Gviz")` tracks has to
share the same chromosome when plotted, i.e., only a single chromosome
can be active during a given plotting operation. Consequently, we can
not directly create plots for multiple chromosomes in a single call to
the `plotTracks` function. However, since the underlying
graphical infrastructure of the `r Biocpkg("Gviz")` package uses grid graphics, we
can build our own composite plot using multiple consecutive
`plotTracks` calls. All we need to take care of is an
adequate layout structure to plot into, and we also need to tell
`plotTracks` not to clear the graphics device before
plotting, which can be achieved by setting the function's
`add` argument to `FALSE`. For details on how to create a
layout structure in the grid graphics system, please see the help page
at `?grid.layout)`.

We start by creating an `AnnotationTrack` objects and a
`DataTrack` object which both contain data for several
chromosomes.

```{r multPlot1}
chroms <- c("chr1", "chr2", "chr3", "chr4")
maTrack <- AnnotationTrack(range=GRanges(seqnames = chroms, 
    ranges = IRanges(start = 1,  width = c(100, 400, 200,1000)),
    strand = c("+", "+", "-", "+")), genome = "mm9", 
    chromosome = "chr1", name = "foo")

mdTrack <- DataTrack(
  range = GRanges(seqnames = rep(chroms, c(10, 40, 20, 100)),
                  ranges = IRanges(start = c(seq(1, 100, len = 10),
                                             seq(1, 400, len = 40), 
                                             seq(1, 200, len = 20),
                                             seq(1, 1000, len = 100)), 
                                   width = 9), values = runif(170)),
  data = "values", chromosome = "chr1", genome = "mm9", name = "bar")
```

Now we also want a genome axis and an `IdeogramTrack` object to indicate the genomic context.

```{r multPlot2}
mgTrack <- GenomeAxisTrack(scale = 50, labelPos = "below", exponent = 3)
chromosome(itrack) <- "chr1"
```

Finally, we build a layout in which the plots for each chromosome are
placed in a rectangular grid and repeatedly call
`plotTracks` for each chromosome.

```{r multPlot3, results='hide', fig.width=7.5, fig.height=4}
ncols <- 2
nrows <- length(chroms) %/% ncols
grid.newpage()
pushViewport(viewport(layout = grid.layout(nrows, ncols)))
for(i in seq_along(chroms)) {
  pushViewport(viewport(layout.pos.col = ((i - 1) %% ncols) + 1,
                        layout.pos.row = (((i) - 1) %/% ncols) + 1))
    plotTracks(list(itrack, maTrack, mdTrack, mgTrack), 
               chromosome = chroms[i], add = TRUE)
    popViewport(1)
}
```

Maybe an even more compact version of this would be to use the lattice
package for building the actual trellis, with `plotTracks`
as the panel function.

```{r multPlot4, results='hide', fig.width=7.5, fig.height=4}
library(lattice)
chroms <- data.frame(chromosome = chroms)
xyplot(1 ~ chromosome | chromosome, data = chroms, panel = function(x) {
  plotTracks(list(itrack , maTrack, mdTrack, mgTrack), 
             chromosome = x, add = TRUE, showId = FALSE) },
  scales = list(draw = FALSE), xlab = NULL, ylab = NULL)
```


# Bioconductor integration and file support {#bioconductor-integration}

This short section is supposed to give a very brief overview over the
different track classes in the `r Biocpkg("Gviz")` package and how those can be
constructed from the typical Bioconductor classes that deal with
genomic data. The list is by no means complete, and a closer look at
a track class' documentation should provide all the possible options.

```{r biocStruct1, echo=FALSE, results='asis'}
dt <- data.frame(`Gviz class` = rep(c("AnnotationTrack","GeneRegionTrack","DataTrack","SequenceTrack"), c(4,5,3,2)),
                 `Bioconductor class`=c("data.frame","IRanges","GRanges","GRangesList",
                                        "data.frame","IRanges","GRanges","GRangesList","TxDb",
                                        "data.frame","IRanges","GRanges",
                                        "DNAStringSet", "BSgenome"),
                 `Method`=c("Constructor","Constructor + additional arguments","Constructor or setAs method, additional data in metadata columns","Constructor or setAs method",
                            "Constructor","Constructor + additional arguments","Constructor or setAs method, additional data in metadata columns","Constructor or setAs method, additional data in metadata columns", "Constructor or setAs method",
                             "Constructor","Constructor + additional data matrix","Constructor or setAs method, numeric data in metadata columns",
                            "DNAStringSet & Constructor", "Constructor"), 
                 stringsAsFactors = FALSE, check.names=FALSE)
dt[duplicated(dt[,1]),1] <- "" 
if (out_type == "html") {
  kable(dt)
} else if (out_type == "latex") {
  kable(dt, "latex", booktabs=TRUE, longtable=TRUE)
}
```

We have seen in previous examples that the `r Biocpkg("Gviz")` package supports quite
a wide range of standard file types as inputs for the class'
constructor functions. In this section we summarize the file support
for the different track classes in one place.

```{r biocStruct2, echo=FALSE, results='asis'}
dt <- data.frame(`Gviz class`=rep(c("AnnotationTrack","GeneRegionTrack","DataTrack","SequenceTrack","AlignmentsTrack"), c(5, 4, 4, 2, 1)),
                 `File type`=c("BED","GFF","GFF2","GFF3","BAM",
                               "GTF","GFF","GFF2","GFF3",
                               "BedGraph","WIG","BigWig","BAM",
                               "FASTA","2Bit",
                               "BAM"),
                 `Extension`=c(".bed",".gff, .gff1, ",".gff2",".gff3",".bam",
                               ".gtf",".gff, .gff1",".gff2",".gff3",
                               ".bedGraph",".wig",".bigWig",".bam",
                               ".fa, .fasta",".2bit",
                               ".bam"),
                 `Streaming`=c("no","no","no","no","YES",
                               "no","no","no","no",
                               "no","no","YES","YES",
                               "YES","YES",
                               "YES"),
                 `Details`=c("Genomic locations from the mandatory `chrom`, `chromStart` and `chromEnd` fields, and optionally the strand from the strand field. If present, the information in the field is mapped to track item ids, and is mapped to track item feature type. All other fields are currently ignored.", "Only the following basic GFF fields are recognized: `seqname`, `start`, `end`, `strand`, (mapped to track item feature type) and `group` (to allow for track item grouping).", "Same as above, but feature grouping information may be provided either as `Group` or `Parent` attribute. Feature ids are mapped to one of the ID, Name or Alias attributes.", "Same as above, but feature grouping information has to be provided as the `Parent` attribute.", "Only start and end locations as well as the strand information for the reads are used. Read identifiers are used for track item grouping.", 
                             "A somewhat looser format definition for `gtf` files is applied here where gene, transcript and exon identifiers and names can be parsed from the `gene_id`, `gene_name`, `transcript_id`, `transcript_name`, `exon_id`, or `exon_id attributes`", "This only supports very limited item grouping and thus complete gene models can not be properly encoded.", "In most instances this is identical to the `GTF` standard and it could make sense to rename the file accordingly.", "The gene-to-transcript and transcript-to- exon relationships are encoded in the parent and `type` attributes and the parser tries to accommodate most of the existing `GFF3` variants.",
                             "", "", "", "Read coverage only is extracted from the bam file.",
                             "Streaming only possible if an index file is found in the same directory as the original fasta file.", "",
                             "Always needs an index file is found in the same directory as the original `BAM` file."),
                 stringsAsFactors = FALSE, check.names=FALSE)
dt[duplicated(dt[,1]),1] <- "" 
if (out_type == "html") {
  kable(dt)
} else if (out_type == "latex") {
  kable(dt, "latex", booktabs=TRUE, longtable=TRUE)
}
```

# SessionInfo

The following is the session info that generated this vignette:

```{r session-info}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UcscTrack.R
\name{UcscTrack}
\alias{UcscTrack}
\alias{clearSessionCache}
\title{Meta-constructor for Gviz tracks fetched directly from the various
UCSC data sources.}
\usage{
UcscTrack(
  track,
  table = NULL,
  trackType = c("AnnotationTrack", "GeneRegionTrack", "DataTrack", "GenomeAxisTrack"),
  genome,
  chromosome,
  name = NULL,
  from,
  to,
  ...
)
}
\arguments{
\item{track}{Character, the name of the track to fetch from UCSC. To find
out about available tracks please consult the online table browser at
\url{http://genome.ucsc.edu/cgi-bin/hgTables?command=start}.}

\item{table}{Character, the name of the table to fetch from UCSC, or
\code{NULL}, in which case the default selection of tables is used. To find
out about available tables for a given track please consult the online table
browser at \url{http://genome.ucsc.edu/cgi-bin/hgTables?command=start}.}

\item{trackType}{Character, one in \code{c("AnnotationTrack",
"GeneRegionTrack", "DataTrack", "GenomeAxisTrack")}. The function will try
to coerce the downloaded data in an object of this class. See below for
details.}

\item{genome}{Character, a valid USCS genome identifier for which to fetch
the data.}

\item{chromosome}{Character, a valid USCS character identifier for which to
fetch the data.}

\item{name}{Character, the name to use for the resulting track object.}

\item{from, to}{A range of genomic locations for which to fetch data.}

\item{\dots}{All additional named arguments are expected to be either
display parameters for the resulting objects, or character scalars of column
names in the downloaded UCSC data tables that are matched by name to
available arguments in the respective constructor functions as defined by
the \code{trackType} argument. See Details section for more information.}
}
\value{
An annotation track object as determined by \code{trackType}.
}
\description{
The UCSC data base provides a wealth of annotation information. This
function can be used to access UCSC, to retrieve the data available there
and to return it as an annotation track object amenable to plotting with
\code{\link{plotTracks}}.
}
\details{
\code{clearSessionCache} is can be called to remove all cached items from
the session which are generated when connecting with the UCSC data base.

The data stored at the UCSC data bases can be of different formats: gene or
transcript model data, simple annotation features like CpG Island locations
or SNPs, or numeric data like conservation or mapability. This function
presents a unified API to download all kinds of data and to map them back to
one of the annotation track objects defined in this package. The type of
object to hold the data has to be given in the \code{trackType} argument,
and subsequently the function passes all data on to the respective object
constructor. All additional named arguments are considered to be relevant
for the constructor of choice, and single character scalars are replaced by
the respective data columns in the downloaded UCSC tables if available. For
instance, assuming the table for track 'foo' contains the columns 'id',
'type', 'fromLoc' and 'toLoc', giving the feature identifier, type, start
end end location. In order to create an \code{\linkS4class{AnnotationTrack}}
object from that data, we have to pass the additional named arguments
\code{id="id"}, \code{feature="type"}, \code{start="fromLoc"} and
codeend="toLoc" to the \code{UcscTrack} function. The complete function call
could look like this:

\code{UcscTrack(track="foo", genome="mm9", chromosome=3, from=1000,
to=10000, trackType="AnnotationTrack", id="id", feature="type",
start="from", end="to")}

To reduce the bandwidth, some caching of the UCSC connection takes place. In
order to remove these cached session items, call \code{clearSessionCache}.

The \code{Gviz.ucscUrl} option controls which URL is being used to connect
to UCSC. For instance, one could switch to the European UCSC mirror by
calling \code{options(Gviz.ucscUrl="http://genome-euro.ucsc.edu/cgi-bin/"}.
}
\examples{
\dontrun{

## Create UcscTrack for Known Genes from mm9 genome
from <- 65921878
to <- 65980988
knownGenes <- UcscTrack(
    genome = "mm9", chromosome = "chrX", track = "knownGene",
    from = from, to = to, trackType = "GeneRegionTrack",
    rstarts = "exonStarts", rends = "exonEnds", gene = "name",
    symbol = "name", transcript = "name", strand = "strand",
    fill = "#8282d2", name = "UCSC Genes"
)
}

## if the UCSC is not accessible load prepared object
data(ucscItems)

## knownGenes is essentially GeneRegionTrack
knownGenes

## plotting
plotTracks(knownGenes, chromosome = "chrX", from = 65920688, to = 65960068)
}
\seealso{
\code{\linkS4class{AnnotationTrack}}

\code{\linkS4class{DataTrack}}

\code{\linkS4class{GeneRegionTrack}}

\code{\linkS4class{GenomeAxisTrack}}

\code{\link{plotTracks}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnnotationTrack-class.R
\docType{class}
\name{AnnotationTrack-class}
\alias{AnnotationTrack-class}
\alias{initialize,AnnotationTrack-method}
\alias{ReferenceAnnotationTrack-class}
\alias{initialize,ReferenceAnnotationTrack-method}
\alias{AnnotationTrack}
\alias{DetailsAnnotationTrack-class}
\alias{DetailsAnnotationTrack}
\alias{initialize,DetailsAnnotationTrack-method}
\alias{group,AnnotationTrack-method}
\alias{group<-,AnnotationTrack,character-method}
\alias{identifier,AnnotationTrack-method}
\alias{identifier<-,AnnotationTrack,character-method}
\alias{setStacks,AnnotationTrack-method}
\alias{consolidateTrack,AnnotationTrack-method}
\alias{collapseTrack,AnnotationTrack-method}
\alias{subset,AnnotationTrack-method}
\alias{subset,ReferenceAnnotationTrack-method}
\alias{drawGD,AnnotationTrack-method}
\alias{drawGD,DetailsAnnotationTrack-method}
\alias{show,AnnotationTrack-method}
\alias{show,ReferenceAnnotationTrack-method}
\title{AnnotationTrack class and methods}
\usage{
\S4method{initialize}{AnnotationTrack}(.Object, ...)

\S4method{initialize}{ReferenceAnnotationTrack}(
  .Object,
  stream,
  reference,
  mapping = list(),
  args = list(),
  defaults = list(),
  ...
)

AnnotationTrack(
  range = NULL,
  start = NULL,
  end = NULL,
  width = NULL,
  feature,
  group,
  id,
  strand,
  chromosome,
  genome,
  stacking = "squish",
  name = "AnnotationTrack",
  fun,
  selectFun,
  importFunction,
  stream = FALSE,
  ...
)

DetailsAnnotationTrack(...)

\S4method{initialize}{DetailsAnnotationTrack}(.Object, fun, selectFun, ...)

\S4method{group}{AnnotationTrack}(GdObject)

\S4method{group}{AnnotationTrack,character}(GdObject) <- value

\S4method{identifier}{AnnotationTrack}(GdObject, type = .dpOrDefault(GdObject, "groupAnnotation", "group"))

\S4method{identifier}{AnnotationTrack,character}(GdObject) <- value

\S4method{setStacks}{AnnotationTrack}(GdObject, recomputeRanges = TRUE)

\S4method{consolidateTrack}{AnnotationTrack}(
  GdObject,
  hasAxis = FALSE,
  hasTitle = .dpOrDefault(GdObject, "showTitle", TRUE),
  title.width = NULL,
  ...
)

\S4method{collapseTrack}{AnnotationTrack}(GdObject, diff = .pxResolution(coord = "x"), xrange)

\S4method{subset}{AnnotationTrack}(
  x,
  from = NULL,
  to = NULL,
  sort = FALSE,
  stacks = FALSE,
  use.defaults = TRUE,
  ...
)

\S4method{subset}{ReferenceAnnotationTrack}(x, from, to, chromosome, ...)

\S4method{drawGD}{AnnotationTrack}(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...)

\S4method{drawGD}{DetailsAnnotationTrack}(GdObject, minBase, maxBase, prepare = FALSE, ...)

\S4method{show}{AnnotationTrack}(object)

\S4method{show}{ReferenceAnnotationTrack}(object)
}
\arguments{
\item{\dots}{Additional items which will all be interpreted as further
display parameters. See \url{settings} and the "Display Parameters"
section below for details.}

\item{stream}{A logical flag indicating that the user-provided import
function can deal with indexed files and knows how to process the additional
\code{selection} argument when accessing the data on disk. This causes the
constructor to return a \code{ReferenceAnnotationTrack} object which will
grab the necessary data on the fly during each plotting operation.}

\item{range}{An optional meta argument to handle the different input types.
If the \code{range} argument is missing, all the relevant information to
create the object has to be provided as individual function arguments
(see below).

The different input options for \code{range} are:
\itemize{
\item A \code{GRanges} object: the genomic ranges for the
\code{Annotation} track as well as the optional additional metadata columns
\code{feature}, \code{group} and \code{id} (see description of the
individual function parameters below for details). Calling the constructor
on a \code{GRanges} object without further arguments, e.g.
\code{AnnotationTrack(range=obj)} is equivalent to calling the coerce method
\code{as(obj, "AnnotationTrack")}.
\item A \code{GRangesList} object: this is very similar to the previous
case, except that the grouping information that is part of the list
structure is preserved in the \code{AnnotationTrack}. I.e., all the elements
within one list item receive the same group id. For consistency, there is
also a coercion method from \code{GRangesLists} \code{as(obj,"AnnotationTrack")}.
\item An \code{\linkS4class{IRanges}} object: almost identical to the
\code{GRanges} case, except that the chromosome and strand information as
well as all additional metadata has to be provided in the separate
\code{chromosome}, \code{strand}, \code{feature}, \code{group} or \code{id}
arguments, because it can not be directly encoded in an \code{IRange}
object. Note that none of those inputs are mandatory, and if not provided
explicitly the more or less reasonable default values \code{chromosome=NA}
and \code{strand="*"} are used.
\item A \code{data.frame} object: the \code{data.frame} needs to contain
at least the two mandatory columns \code{start} and \code{end} with the
range coordinates. It may also contain a \code{chromosome} and a
\code{strand} column with the chromosome and strand information for each
range. If missing it will be drawn from the separate \code{chromosome} or
\code{strand} arguments. In addition, the \code{feature}, \code{group} and
\code{id} data can be provided as additional columns. The above comments
about potential default values also apply here.
\item A \code{character} scalar: in this case the value of the
\code{range} argument is considered to be a file path to an annotation file
on disk. A range of file types are supported by the \code{Gviz} package as
identified by the file extension. See the \code{importFunction}
documentation below for further details.
}}

\item{start, end, width}{Integer vectors, giving the start and the end end
coordinates for the individual track items, or their width. Two of the three
need to be specified, and have to be of equal length or of length one, in
which case this single value will be recycled. Otherwise, the usual R
recycling rules for vectors do not apply here.}

\item{feature}{Factor (or other vector that can be coerced into one), giving
the feature types for the individual track items. When plotting the track to
the device, if a display parameter with the same name as the value of
\code{feature} is set, this will be used as the track item's fill colour. See
\code{grouping} for details.  Needs to be of equal length as the
provided genomic coordinates, or of length 1.}

\item{group}{Factor (or other vector that can be coerced into one), giving
the group memberships for the individual track items. When plotting to the
device, all items in the same group will be connected.  See
\code{grouping} for details. Needs to be of equal length as the
provided genomic coordinates, or of length 1.}

\item{id}{Character vector of track item identifiers. When plotting to the
device, it's value will be used as the identifier tag if the display
parameter \code{showFeatureId=TRUE}. Needs to be of equal length as the
provided genomic ranges, or of length 1.}

\item{strand}{Character vector, the strand information for the individual
track items. It may be provided in the form \code{+} for the Watson strand,
\code{-} for the Crick strand or \code{*} for either one of the two. Needs
to be of equal length as the provided genomic coordinates, or of length 1.
Please note that grouped items need to be on the same strand, and erroneous
entries will result in casting of an error.}

\item{chromosome}{The chromosome on which the track's genomic ranges are
defined. A valid UCSC chromosome identifier if
\code{options(ucscChromosomeNames=TRUE)}. Please note that in this case only
syntactic checking takes place, i.e., the argument value needs to be an
integer, numeric character or a character of the form \code{chrx}, where
\code{x} may be any possible string. The user has to make sure that the
respective chromosome is indeed defined for the the track's genome. If not
provided here, the constructor will try to construct the chromosome
information based on the available inputs, and as a last resort will fall
back to the value \code{chrNA}. Please note that by definition all objects
in the \code{Gviz} package can only have a single active chromosome at a
time (although internally the information for more than one chromosome may
be present), and the user has to call the \verb{chromosome<-} replacement
method in order to change to a different active chromosome.}

\item{genome}{The genome on which the track's ranges are defined. Usually
this is a valid UCSC genome identifier, however this is not being formally
checked at this point. If not provided here the constructor will try to
extract this information from the provided input, and eventually will fall
back to the default value of \code{NA}.}

\item{stacking}{The stacking type for overlapping items of the track. One in
\code{c(hide, dense, squish, pack,full)}. Currently, only squish (make best
use of the available space), dense (no stacking, collapse overlapping
ranges), and hide (do not show any track items at all) are implemented.}

\item{name}{Character scalar of the track's name used in the title panel
when plotting.}

\item{fun}{A function that is being called for each entry in the
\code{AnnotationTrack} object. See section 'Details' and 'Examples' for
further information. When called internally by the plotting machinery, a
number of arguments are automatically passed on to this function, and the
user needs to make sure that they can all be digested (i.e., either have all
of them as formal named function arguments, or gobble up everything that is
not needed in \code{...}). These arguments are:
\itemize{
\item \code{start}: the genomic start coordinate of the range item.
\item \code{end}: the genomic end coordinates of the range item.
\item \code{strand}: the strand information for the range item.
\item \code{chromosome}: the chromosome of the range item.
\item \code{identifier}: the identifier of the range item, i.e., the result
of calling \code{identifier(DetailsAnnotationTrack, lowest=TRUE)}. Typically
those identifiers are passed on to the object constructor during
instantiation as the \code{id} argument.
\item \code{index}: a counter enumerating the ranges. The \code{AnnotationTrack} object
is sorted internally for visibility, and the \code{index} argument refers to
the index of plotting.
\item \code{GdObject}: a reference to the currently plotted \code{DetailsAnnotationTrack}
object.
\item \code{GdObject.original}: a reference to the \code{DetailsAnnotationTrack} before
any processing like item collapsing has taken place. Essentially, this is
the track object as it exists in your working environment.
}

Additional arguments can be passed to the plotting function by means of the
\code{detailsFunArgs} argument (see below). Note that the plot must use grid
graphics (e.g. function in the 'lattice' package or low-level grid
functions). To access a data object such a matrix or data frame within the
function you can either store it as a variable in the global environment or,
to avoid name space conflicts, you can make it part of the function
environment by means of a closure. Alternatively, you may want to
explicitely stick it into an environment or pass it along in the
\code{detailsFunArgs} list. To figure out in your custom plotting function
which annotation element is currently being plotted you can either use the
identifier which has to be unique for each range element, or you may want to
use the genomic position (start/end/strand/chromosome) e.g. if the data is
stored in a \code{GRanges} object.}

\item{selectFun}{A function that is being called for each entry in the
\code{AnnotationTrack} object with exactly the same arguments as in
\code{fun}. The purpose of this function is to decide for each track element
whether details should be drawn, and consequently it has to return a single
logical scalar. If the return value is \code{TRUE}, details will be drawn
for the item, if it is \code{FALSE}, the details strip for the item is
omitted.}

\item{importFunction}{A user-defined function to be used to import the data
from a file. This only applies when the \code{range} argument is a character
string with the path to the input data file. The function needs to accept an
argument \code{x} containing the file path and has to return a proper
\code{GRanges} object with all the necessary metadata columns set. A set of
default import functions is already implemented in the package for a number
of different file types, and one of these defaults will be picked
automatically based on the extension of the input file name. If the
extension can not be mapped to any of the existing import function, an error
is raised asking for a user-defined import function via this argument.
Currently the following file types can be imported with the default
functions: \code{gff}, \code{gff1}, \code{gff2}, \code{gff3}, \code{bed}, \code{bam}.}
}
\value{
The return value of the constructor function is a new object of class
\code{AnnotationTrack} or of class \code{DetailsAnnotationTrack}, depending
on the constructor arguments. Typically the user will not have to be
troubled with this distinction and can rely on the constructor to make the
right choice.
}
\description{
A fairly generic track object for arbitrary genomic range annotations, with
the option of grouped track items. The extended \code{DetailsAnnotationTrack}
provides a more flexible interface to add user-defined custom
information for each range.
}
\section{Functions}{
\itemize{
\item \code{initialize(AnnotationTrack)}: Show method.

\item \code{ReferenceAnnotationTrack-class}: The file-based version of the \code{AnnotationTrack-class}.

\item \code{initialize(ReferenceAnnotationTrack)}: Initialize.

\item \code{AnnotationTrack()}: Constructor function for
\code{AnnotationTrack-class}

\item \code{DetailsAnnotationTrack-class}: directly extends \code{AnnotationTrack.}

\item \code{DetailsAnnotationTrack()}: Constructor function for
\code{DetailsAnnotationTrack-class}

The \code{DetailsAnnotationTrack} class directly extends \code{AnnotationTrack.}
The purpose of this track type is to add an arbitrarily detailed plot
section (typically consisting of additional quantitative data) for each
range element of an \code{AnnotationTrack.} This allows a locus wide view of
annotation elements together with any kind of details per feature or element
that may for instance provide insight on how some complex quantitative
measurements change according to their position in a locus. If the
quantitative data is too complex for a \code{DataTrack} e.g. because it requires
extra space or a trellis-like representation, a \code{DetailsAnnotationTrack} can
be used instead. Example: An \code{AnnotationTrack} shows the positions of a
number of probes from a microarray, and you want a histogram of the signal
intensity distribution derived from all samples at each of these probe
location. Another example usage would be to show for each element of an
\code{AnnotationTrack} an xy-plot of the signal against some clinical measurement
such as blood pressure. The limitation for applications of this type of
track is basically only the available space of the device you are
plotting to.

This flexibility is possible by utilizing a simple function model
to perform all the detailed plotting. The functionality of this plotting
function fun is totally up to the user, and the function environment is
prepared in a way that all necessary information about the plotted
annotation feature is available. To restrict the details section to only
selected number of annotation features one can supply another function
\code{selectFun}, which decides for each feature separately whether details are
available or not. Finally, an arbitrary number of additional arguments can
be passed on to these two function by means of the \code{detailsFunArgs} display
parameter. This is expected to be a named list, and all list elements are
passed along to the plotting function fun and to the selector function
\code{selectFun} as additional named arguments. Please note that some argument
names like \code{start}, \code{end} or \code{identifier} are reserved and can not be used
in the \code{detailsFunArgs} list. For examples of plotting functions,
see the 'Examples' section.

\item \code{initialize(DetailsAnnotationTrack)}: Initialize.

\item \code{group(AnnotationTrack)}: extract the group membership for all track items.

\item \code{group(GdObject = AnnotationTrack) <- value}: replace the grouping information for track items.
The replacement value must be a factor of appropriate length or another
vector that can be coerced into such.

\item \code{identifier(AnnotationTrack)}: return track item identifiers.
Depending on the setting of the optional argument lowest, these are either
the group identifiers or the individual item identifiers.

\item \code{identifier(GdObject = AnnotationTrack) <- value}: Set the track item identifiers.
The replacement value has to be a character vector of appropriate length.
This always replaces the group-level identifiers, so essentially it is
similar to \verb{groups<-}.

\item \code{setStacks(AnnotationTrack)}: Recompute the stacks based on the available
space and on the object's track items and stacking settings.

\item \code{consolidateTrack(AnnotationTrack)}: Consolidate.
Determine whether there is group label annotation or not, and add this
information as the internal display parameter \code{.__hasAnno}. Precompute
the grouped ranges together with optional labels in order to determine
the correct plotting range later.

\item \code{collapseTrack(AnnotationTrack)}: preprocess the track before plotting.
This will collapse overlapping track items based on the available resolution
and increase the width and height of all track objects to a minimum value
to avoid rendering issues. See collapsing for details.

\item \code{subset(AnnotationTrack)}: subset a \code{AnnotationTrack} by coordinates
and sort if necessary.

\item \code{subset(ReferenceAnnotationTrack)}: subset a \code{ReferenceAnnotationTrack} by
coordinates and sort if necessary.

\item \code{drawGD(AnnotationTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{drawGD(DetailsAnnotationTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(AnnotationTrack)}: Show method.

\item \code{show(ReferenceAnnotationTrack)}: Show method.

}}
\section{Slots}{

\describe{
\item{\code{dp}}{Object of \code{DisplayPars-class}, the display settings controlling the
look and feel of a track. See settings for details on setting graphical
parameters for tracks.}

\item{\code{name}}{Object of class \code{character}, a human-readable name for the track
that will be used in the track's annotation panel if necessary.}

\item{\code{imageMap}}{Object of \code{ImageMap-class}, containing optional information
for an HTML image map. This will be created by the \code{drawGD} methods when the
track is plotted to a device and is usually not set by the user.}

\item{\code{range}}{Object of class \code{GRanges}, the genomic ranges of the track items
as well as additional annotation information in its \code{elementMetaData} slot.
Please not that the slot is actually implemented as a class union between
\code{GRanges} and IRanges to increase efficiency, for instance for DataTrack
objects. This usually does not concern the user.}

\item{\code{chromosome}}{Object of class \code{character}, the chromosome on which the
track is defined. There can only be a single chromosome for one track. For
certain subclasses, the space of allowed chromosome names is limited (e.g.,
only those chromosomes that exist for a particular genome). Throughout the
package, chromosome name have to be entered either as a single integer scalar
or as a character scalar of the form chrXYZ, where XYZ may be an arbitrary
character string.}

\item{\code{genome}}{Object of class \code{character}, the genome for which the track is
defined. For most sub-classes this has to be valid UCSC genome identifier,
however this may not always be formally checked upon object instantiation.}

\item{\code{stacking}}{Object of class \code{character}, the stacking type of overlapping
items on the final plot. One in \code{c(hide, dense, squish, pack,full)}.
Currently, only \code{hide} (do not show the track items at all), \code{squish} (make
best use of the available space) and \code{dense} (no stacking at all) are
implemented.}

\item{\code{stacks}}{Object of class \code{numeric}, holding the stack indices for each
track item. This slot is usually populated by calling the \code{setStacks} method
upon plotting, since the correct stacking is a function of the available
plotting space.}

\item{\code{fun}}{A function that is being called for each AnnotationTrack element to plot details.}

\item{\code{selectFun}}{A function that is being called for each AnnotationTrack element to decide whether details need to be plotted.}
}}

\section{Objects from the class}{


Objects can be created using the constructor function
\code{AnnotationTrack}.
}

\examples{
## An empty object
AnnotationTrack()

## Construct from individual arguments
st <- c(2000000, 2070000, 2100000, 2160000)
ed <- c(2050000, 2130000, 2150000, 2170000)
str <- c("-", "+", "-", "-")
gr <- c("Group1", "Group2", "Group1", "Group3")

annTrack <- AnnotationTrack(
    start = st, end = ed, strand = str, chromosome = 7,
    genome = "hg19", feature = "test", group = gr,
    id = paste("annTrack item", 1:4),
    name = "generic annotation", stacking = "squish"
)

## Or from a data.frame
df <- data.frame(
    start = st, end = ed, strand = str, id = paste("annTrack item", 1:4),
    feature = "test", group = gr
)
annTrack <- AnnotationTrack(
    range = df, genome = "hg19", chromosome = 7,
    name = "generic annotation", stacking = "squish"
)

## Or from a GRanges object
gr <- GRanges(
    seqnames = "chr7", range = IRanges(start = df$start, end = df$end),
    strand = str
)
genome(gr) <- "hg19"
mcols(gr) <- df[, -(1:3)]
annTrack <- AnnotationTrack(
    range = gr, name = "generic annotation",
    stacking = "squish"
)

## Finally from a GRangesList
grl <- split(gr, values(gr)$group)
AnnotationTrack(grl)
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(annTrack)

## Track names
names(annTrack)
names(annTrack) <- "foo"
plotTracks(annTrack)

## Subsetting and splitting
subTrack <- subset(annTrack, to = 2155000)
length(subTrack)
subTrack[1:2]
split(annTrack, c(1, 2, 1, 2))

## Accessors
start(annTrack)
end(annTrack)
width(annTrack)
position(annTrack)
width(subTrack) <- width(subTrack) + 1000

strand(annTrack)
strand(subTrack) <- "-"

chromosome(annTrack)
chromosome(subTrack) <- "chrX"

genome(annTrack)
genome(subTrack) <- "mm9"

range(annTrack)
ranges(annTrack)

## Annotation
identifier(annTrack)
identifier(annTrack, "lowest")
identifier(subTrack) <- "bar"

feature(annTrack)
feature(subTrack) <- "foo"

values(annTrack)

## Grouping
group(annTrack)
group(subTrack) <- "Group 1"
chromosome(subTrack) <- "chr7"
plotTracks(subTrack)

## Stacking
stacking(annTrack)
stacking(annTrack) <- "dense"
plotTracks(annTrack)

## coercion
as(annTrack, "data.frame")
as(annTrack, "UCSCData")

## HTML image map
coords(annTrack)
tags(annTrack)
annTrack <- plotTracks(annTrack)$foo
coords(annTrack)
tags(annTrack)

## DetailsAnnotationTrack
library(lattice) # need to use grid grapics

## generate two random distributions per row (probe/feature)
## the difference between the distributions increases from probe 1 to 4
m <- matrix(c(rgamma(400, 1)), ncol = 100)
m[, 51:100] <- m[, 51:100] + 0:3
## rownames must be accessible by AnnotationTrack element identifier
rownames(m) <- identifier(annTrack, "lowest")

## create a lattice density plot for the values (signals) of the two groups
## as the chart must be placed into a pre-set grid view port we have to use
## print without calling plot.new! Note, use a common prefix for all lattice.
## Avoid wasting space by removing y-axis decorations.

## Note, in this example 'm' will be found in the environment the 'details'
## function is defined in. To avoid overwriting 'm' you should use a closure
## or environment to access 'm'.
details <- function(identifier, ...) {
    d <- data.frame(signal = m[identifier, ], group = rep(c("grp1", "grp2"), each = 50))
    print(densityplot(~signal,
        group = group, data = d, main = identifier,
        scales = list(draw = FALSE, x = list(draw = TRUE)), ylab = "", xlab = "",
    ), newpage = FALSE, prefix = "plot")
}

deTrack <- AnnotationTrack(
    range = gr, genome = "hg19", chromosome = 7,
    name = "generic annotation with details per entry", stacking = "squish",
    fun = details, details.ratio = 1
)

plotTracks(deTrack)

set.seed(1234)
deTrack <- AnnotationTrack(
    range = gr, genome = "hg19", chromosome = 7,
    name = "generic annotation with details per entry",
    stacking = "squish", fun = details,
    details.ratio = 1, selectFun = function(...) {
        sample(c(FALSE, TRUE), 1)
    }
)

plotTracks(deTrack)
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne, Arne Mueller
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CustomTrack-class.R
\docType{class}
\name{CustomTrack-class}
\alias{CustomTrack-class}
\alias{initialize,CustomTrack-method}
\alias{CustomTrack}
\alias{drawGD,CustomTrack-method}
\alias{show,CustomTrack-method}
\title{CustomTrack class and methods}
\usage{
\S4method{initialize}{CustomTrack}(.Object, plottingFunction, variables, ...)

CustomTrack(
  plottingFunction = function(GdObject, prepare = FALSE, ...) {
 },
  variables = list(),
  name = "CustomTrack",
  ...
)

\S4method{drawGD}{CustomTrack}(GdObject, minBase, maxBase, prepare = FALSE, ...)

\S4method{show}{CustomTrack}(object)
}
\arguments{
\item{.Object}{.Object}

\item{plottingFunction}{A user-defined function to be executed once the
track coordinates have been properly set up. The function needs to accept
two mandatory arguments: \code{GdObject}, the \code{CustomTrack} object to
be plotted, and \code{prepare}, a logical flag indicating whether the
function has been called in preparation mode or in drawing mode. It also
needs to return the input \code{GdObject}, potentially with modifications.}

\item{variables}{A list of additional variables for the user-defined
plotting function.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters. See \code{settings} and the "Display Parameters"
section below for details.}

\item{name}{Character scalar of the track's name.}

\item{GdObject}{Object of \code{GdObject-class}.}

\item{minBase}{minBase}

\item{maxBase}{maxBase}

\item{prepare}{\code{logical}}

\item{object}{object}
}
\value{
The return value of the constructor function is a new object of class
\code{CustomTrack}.
}
\description{
A fully customizable track object to be populated via a user-defined
plotting function.
}
\details{
A track to allow for any sort of plotting, with the currently displayed
genomic location set. Essentially this acts as a simple callback into the
\code{Gviz} plotting machinery after all the track panels and coordinates
have been set up. It is entirely up to the user what to plot in the track,
or even to use the predefined coordinate system. The only prerequisite is
that all plotting operations need to utilize Grid graphics.
}
\section{Functions}{
\itemize{
\item \code{initialize(CustomTrack)}: Initialize.

\item \code{CustomTrack()}: Objects can be created using the constructor function.

\item \code{drawGD(CustomTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(CustomTrack)}: Show method.

}}
\examples{
## Object construction:

## An empty object
CustomTrack()
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{datasets}
\alias{datasets}
\alias{bmTrack}
\alias{cyp2b10}
\alias{idTrack}
\alias{biomTrack}
\alias{biomTrack2}
\alias{cpgIslands}
\alias{axTrack}
\alias{conservation}
\alias{ensGenes}
\alias{denseAnnTrack}
\alias{geneModels}
\alias{iTrack}
\alias{itrack}
\alias{idxTrack}
\alias{ideoTrack}
\alias{twoGroups}
\alias{from}
\alias{gcContent}
\alias{knownGenes}
\alias{refGenes}
\alias{snpLocations}
\alias{to}
\alias{ctrack}
\alias{geneDetails}
\alias{dtHoriz}
\alias{bmt}
\title{Data sets}
\description{
Some sample data sets used for the illustrative examples and the vignette.
}
\keyword{datasets}

\name{AlignedReadTrack-class}
\Rdversion{1.1}
\docType{class}
\alias{AlignedReadTrack-class}
\alias{AlignedReadTrack}
\alias{[,AlignedReadTrack-method}
\alias{[,AlignedReadTrack,ANY,ANY-method}
\alias{[,AlignedReadTrack,ANY,ANY,ANY-method}
\alias{coverage,AlignedReadTrack-method}
\alias{drawAxis,AlignedReadTrack-method}
\alias{drawGD,AlignedReadTrack-method}
\alias{drawGrid,AlignedReadTrack-method}
\alias{initialize,AlignedReadTrack-method}
\alias{setCoverage,AlignedReadTrack-method}
\alias{show,AlignedReadTrack-method}
\alias{split,AlignedReadTrack,ANY-method}
\alias{split,AlignedReadTrack-method}
\alias{subset,AlignedReadTrack-method}

\title{AlignedReadTrack class and methods (NOTE: THIS IS STILL IN
  DEVELOPMENT AND SUBJECT TO CHANGE)}

\description{

  A class to represent short sequences that have been aligned to a
  reference genome as they are typically generated in a next generation
  sequencing experiment.

}

\section{Objects from the Class}{

  Objects can be created using the constructor function \code{AlignedReadTrack}.

}


\usage{

AlignedReadTrack(range=NULL, start=NULL, end=NULL, width=NULL, chromosome, strand, genome,
stacking="squish", name="AlignedReadTrack", coverageOnly=FALSE, ...)

}

\arguments{

  \item{range}{An object of class \code{\linkS4class{GRanges}}, or a
    \code{data.frame} which will be coerced into one in which case it
    needs to contain at least the three columns:

    \describe{

      \item{}{\code{start}, \code{end}: the start and end coordinates
	for the track items.}

      \item{}{\code{strand}: the strand information for the track
	items. It may be provided in the form \code{+} for the Watson
	strand, \code{-} for the Crick strand or \code{*} for either one
	of the two.}

    }

    Alternatively, the \code{range} argument may be missing, in which
    case the relevant information has to be provided as individual
    function arguments (see below).

  }

  \item{start, end, width}{Integer vectors, giving the start and the end
    end coordinates for the individual track items, or their width. Two
    of the three need to be specified, and have to be of equal length or
    of length one, in which case this value will be recycled. Otherwise,
    the usual R recycling rules for vectors do not apply.}

  \item{strand}{Character vector, the strand information for the
    individual track items.  Needs to be of equal length as the
    \code{start, end} or \code{width} vectors, or of length 1. Please
    note that grouped items need to be on the same strand, and erroneous
    entries will result in casting of an error.}

  \item{chromosome}{The chromosome on which the track's genomic ranges
    are defined. A valid UCSC chromosome identifier. Please note that at
    this stage only syntactic checking takes place, i.e., the argument
    value needs to be a single integer, numeric character or a character
    of the form \code{chrx}, where \emph{x} may be any
    possible string. The user has to make sure that the respective
    chromosome is indeed defined for the the track's genome.}

  \item{genome}{The genome on which the track's ranges are
    defined. Usually this is a valid UCSC genome identifier, however
    this is not being formally checked at this point.}

  \item{stacking}{The stacking type for overlapping items of the
    track. One in \code{c(hide, dense, squish, pack,full)}. Currently,
    only hide (don't show the track items, squish (make best use of the
    available space) and dense (no stacking at all) are implemented.}

  \item{name}{Character scalar of the track's name used in the title
    panel when plotting.}

  \item{coverageOnly}{Instead of storing individual reads, just compute
    the coverage and store the resulting coverage vector.}

  \item{\dots}{Additional items which will all be interpreted as further
    display parameters.}

}

\value{

  The return value of the constructor function is a new object of class
  \code{AlignedReadTrack}.

}

\section{Slots}{

  \describe{

    \item{\code{coverage}:}{Object of class \code{"list"}, a list of
      coverage vectors for the plus strand, the minus strand and for
      both strands combined.}

    \item{\code{coverageOnly}:}{Object of class \code{"logical"}, flag
      to determine whether the object stores read locations or the coverage
      vectors only.}

     \item{\code{stacking}:}{Object of class \code{"character"},
      inherited from class \code{\linkS4class{StackedTrack}}}

    \item{\code{stacks}:}{Object of class \code{"environment"},
      inherited from class \code{\linkS4class{StackedTrack}}}

    \item{\code{range}:}{Object of class \code{\linkS4class{GRanges}},
      inherited from class \code{\linkS4class{RangeTrack}}}

    \item{\code{chromosome}:}{Object of class \code{"character"},
      inherited from class \code{\linkS4class{RangeTrack}} }

    \item{\code{genome}:}{Object of class \code{"character"}, inherited
      from class \code{\linkS4class{RangeTrack}}}

    \item{\code{dp}:}{Object of class
      \code{\linkS4class{DisplayPars}}, inherited from class
      \code{\linkS4class{GdObject}} }

    \item{\code{name}:}{Object of class \code{"character"}, inherited
      from class \code{\linkS4class{GdObject}}}

    \item{\code{imageMap}:}{Object of class
      \code{\linkS4class{ImageMap}}, inherited from class
      \code{\linkS4class{GdObject}}}

  }
}

\section{Extends}{

  Class \code{"\linkS4class{StackedTrack}"}, directly.

  Class \code{"\linkS4class{RangeTrack}"}, by class "StackedTrack",
  distance2.

  Class \code{"\linkS4class{GdObject}"}, by class "StackedTrack",
  distance3.

}

\section{Methods}{

  In the following code chunks, \code{obj} is considered to be an object
  of class \code{AlignedReadTrack}.

  \bold{\emph{Exported in the name space:}}

  \describe{

    \item{[}{\code{signature(x="AlignedReadTrack")}: subset the items in
      the \code{AlignedReadTrack}. This is essentially similar to
      subsetting of the \code{\linkS4class{GRanges}} object in the
      \code{range} slot. For most applications, the \code{subset} method
      may be more appropriate. The operation is only supported for
      objects that still contain all the read locations, i.e.,
      \code{coverageOnly=FALSE}.

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{i}: subsetting indices}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{obj[1:5]}}

      }
    }

    \item{subset}{\code{signature(x="AlignedReadTrack")}: subset a
      \code{AlignedReadTrack} by coordinates and sort if necessary.

      \emph{Usage:}

      \code{subset(x, from, to, sort=FALSE, stacks=FALSE)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{from}, \code{to}: the coordinates range to subset
	  to.}

	\item{}{\code{sort}: sort the object after subsetting. Usually
	  not necessary.}

	\item{}{\code{stacks}: recompute the stacking after subsetting
	  which can be expensive and is not always necessary.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{subset(obj, from=10, to=20)}}

	\item{}{\code{subset(obj, from=10, to=20, sort=TRUE, stacks=FALSE)}}

      }
    }

    \item{split}{\code{signature(x="AlignedReadTrack")}: split an
      \code{AlignedReadTrack} object by an appropriate factor vector (or
      another vector that can be coerced into one). The output of this
      operation is a list of \code{AlignedReadTrack} objects.

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{f}: the splitting factor.}

	\item{}{\code{\dots}: all further arguments are ignored.}

      }

      \emph{Usage:}

      \code{split(x, f, ...)}

      \emph{Examples:}

      \describe{

	\item{}{\code{split(obj, c("a", "a", "b", "c", "a"))}}

      }
    }

    \item{coverage}{\code{signature(x="AlignedReadTrack")}: return the
      coverage vector for one of the strands, or the combined vector.

      \emph{Usage:}

      \code{coverage(x, strand="*")}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{strand}: the selector for the strand, \code{+} for
	  the Watson strand, \code{-} for the Crick strand or \code{*}
	  for both strands.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{coveraget(obj)}}

	\item{}{\code{coverage(obj, strand="-")}}

      }
    }
  }

  \bold{\emph{Internal methods:}}

  \describe{


    \item{setCoverage}{\code{signature(GdObject="AlignedReadTrack")}:
      recompute the coverage on the plus and minus strand as well as for
      the combined strands and update the respective slot.

      \emph{Usage:}

      \code{setCoverage(GdObject)}

       \emph{Examples:}

      \describe{

	\item{}{\code{setCoverage(obj)}}

      }
    }

    \item{drawAxis}{\code{signature(GdObject="AlignedReadTrack")}: add a y-axis
      to the title panel of a track.

      \emph{Usage:}

      \code{drawAxis(GdObject, from, to, subset=FALSE, ...)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{from}, \code{to}: compute axis range from the data
	  within a certain coordinates range only.}

	\item{}{\code{subset}: subset the object prior to calculating
	  the axis ranges. Can be expensive and is not always needed.}

	\item{}{\code{\dots}: all further arguments are ignored.}

      }

      \emph{Examples:}

      \describe{

      	\item{}{\code{Gviz:::drawAxis(obj)}}

      }
    }

    \item{drawGD}{\code{signature(gdObject="AlignedReadTrack")}: plot the
      object to a graphics device. The return value of this method is
      the input object, potentially updated during the plotting
      operation. Internally, there are two modes in which the method can
      be called. Either in 'prepare' mode, in which case no plotting is
      done but the object is preprocessed based on the
      available space, or in 'plotting' mode, in which case the actual
      graphical output is created. Since subsetting of the object can be
      potentially costly, this can be switched off in case subsetting
      has already been performed before or is not necessary.

      \emph{Usage:}

      \code{drawGD(GdObject, minBase, maxBase, prepare=FALSE,
	subset=TRUE, ...)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{minBase}, \code{maxBase}: the coordinate range to
	  plot.}

	\item{}{\code{prepare}: run method in preparation or in
	  production mode.}

	\item{}{\code{subset}: subset the object to the visible region
	or skip the potentially expensive subsetting operation.}

	\item{}{\code{\dots}: all further arguments are ignored.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{Gviz:::drawGD(obj)}}

	\item{}{\code{Gviz:::drawGD(obj, minBase=1, maxBase=100)}}

	\item{}{\code{Gviz:::drawGD(obj, prepare=TRUE,
	    subset=FALSE)}}

      }
    }

    \item{drawGrid}{\code{signature(GdObject="AlignedReadTrack")}: superpose a grid on top of a track.

      \emph{Usage:}

      \code{drawGrid(GdObject, from, to)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{from}, \code{to}: draw grid within a certain
	  coordinates range. This needs to be supplied for the plotting
	  function to know the current genomic coordinates.}

      }

      \emph{Examples:}

      \describe{

      	\item{}{\code{Gviz:::drawGrid(obj, from=10, to=100)}}

      }
    }


    \item{initialize}{\code{signature(.Object="AligendReadTrack")}:
      initialize the object.}

    \item{show}{\code{signature(object="AlignedReadTrack")}: show a
      human-readable summary of the object.}

  }

  \bold{\emph{Inherited methods:}}

  \describe{

    \item{stacking}{\code{signature(GdObject="AlignedReadTrack")}: return
      the current stacking type.

      \emph{Usage:}

      \code{stacking(GdObject)}

      \emph{Examples:}

      \describe{

	\item{}{\code{stacking(obj)}}

      }
    }

    \item{stacking<-}{\code{signature(GdObject="AlignedReadTrack",
      value="character")}: set the object's stacking type to one in
      \code{c(hide, dense, squish, pack,full)}.

      \emph{Usage:}

      \code{stacking<-(GdObject, value)}

      \emph{Additional Arguments:}

	\describe{

	  \item{}{\code{value}: replacement value.}

	}

      \emph{Examples:}

      \describe{

	\item{}{\code{stacking(obj) <- "squish" }}

      }
    }

    \item{setStacks}{\code{signature(GdObject="AlignedReadTrack")}:
      recompute the stacks based on the available space and on the
      object's track items and stacking settings.

      \emph{Usage:}

      \code{setStacks(GdObject, from, to)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{from}, \code{to}: compute stacking within a
	  certain coordinates range. This needs to be supplied for the
	  plotting function to know the current genomic coordinates.}

      }

      \emph{Examples:}

      \describe{

      	\item{}{\code{Gviz:::setStacks(obj)}}

      }
    }

    \item{stacks}{\code{signature(GdObject="AlignedReadTrack")}: return
      the stack indices for each track item.

      \emph{Usage:}

      \code{stacks(GdObject)}

      \emph{Examples:}

      \describe{

      	\item{}{\code{Gviz:::stacks(obj)}}

      }
    }

    \item{chromosome}{\code{signature(GdObject="AlignedReadTrack")}:
      return the chromosome for which the track is defined.

      \emph{Usage:}

      \code{chromosome(GdObject)}

      \emph{Examples:}

      \describe{

	\item{}{\code{chromosome(obj)}}

      }
    }

    \item{chromosome<-}{\code{signature(GdObject="AlignedReadTrack")}:
      replace the value of the track's chromosome. This has to be a
      valid UCSC chromosome identifier or an integer or character
      scalar that can be reasonably coerced into one.

      \emph{Usage:}

      \code{chromosome<-(GdObject, value)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{value}: replacement value.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{chromosome(obj) <- "chr12"}}

      }
    }

    \item{start, end, width}{\code{signature(x="AlignedReadTrack")}: the
      start or end coordinates of the track items, or their width in
      genomic coordinates.

      \emph{Usage:}

      \code{start(x)}

      \code{end(x)}

      \code{width(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{start(obj)}}

	\item{}{\code{end(obj)}}

	\item{}{\code{width(obj)}}

      }
    }

    \item{start<-, end<-, width<-}{\code{signature(x="AlignedReadTrack")}:
      replace the start or end coordinates of the track items, or their
      width.

      \emph{Usage:}

      \code{start<-(x, value)}

      \code{end<-(x, value)}

      \code{width<-(x, value)}

      \emph{Additional Arguments:}

	\describe{

	  \item{}{\code{value}: replacement value.}

	}

      \emph{Examples:}

      \describe{

	\item{}{\code{start(obj) <- 1:10}}

	\item{}{\code{end(obj) <- 20:30}}

	\item{}{\code{width(obj) <- 1}}

      }
    }

    \item{position}{\code{signature(GdObject="AlignedReadTrack")}: the
      arithmetic mean of the track item's coordionates, i.e.,
      \code{(end(obj)-start(obj))/2}.

      \emph{Usage:}

      \code{position(GdObject)}

      \emph{Examples:}

      \describe{

	\item{}{\code{position(obj)}}

      }
    }

    \item{feature}{\code{signature(GdObject="AlignedReadTrack")}: return the
      grouping information for track items. For certain sub-classes,
      groups may be indicated by different color schemes when
      plotting. See \code{\link{grouping}} or
      \code{\linkS4class{AnnotationTrack}} and
      \code{\linkS4class{GeneRegionTrack}} for details.

      \emph{Usage:}

      \code{feature(GdObject)}

      \emph{Examples:}

      \describe{

	\item{}{\code{feature(obj)}}

      }
    }

    \item{feature<-}{\code{signature(gdObject="AlignedReadTrack",
      value="character")}: set the grouping information for track
      items. This has to be a factor vector (or another type of vector
      that can be coerced into one) of the same length as the number of
      items in the \code{AlignedReadTrack}. See \code{\link{grouping}} or
      \code{\linkS4class{AnnotationTrack}} and
      \code{\linkS4class{GeneRegionTrack}} for details.

      \emph{Usage:}

      \code{feature<-(GdObject, value)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{value}: replacement value.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{feature(obj) <- c("a", "a", "b", "c", "a")}}

      }
    }

    \item{genome}{\code{signature(x="AlignedReadTrack")}: return the track's genome.

      \emph{Usage:}

      \code{genome(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{genome(obj)}}

      }
    }

    \item{genome<-}{\code{signature(x="AlignedReadTrack")}: set the track's
      genome. Usually this has to be a valid UCSC identifier, however
      this is not formally enforced here.

      \emph{Usage:}

      \code{genome<-(x, value)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{value}: replacement value.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{genome(obj) <- "mm9"}}

      }
    }

    \item{length}{\code{signature(x="AlignedReadTrack")}: return the number
      of items in the track.

      \emph{Usage:}

      \code{length(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{length(obj)}}

      }
    }

    \item{range}{\code{signature(x="AlignedReadTrack")}: return the genomic
      coordinates for the track as an object of class
      \code{\linkS4class{IRanges}}.

      \emph{Usage:}

      \code{range(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{range(obj)}}

      }
    }

    \item{ranges}{\code{signature(x="AlignedReadTrack")}: return the genomic
      coordinates for the track along with all additional annotation
      information as an object of class \code{\linkS4class{GRanges}}.

      \emph{Usage:}

      \code{ranges(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{ranges(obj)}}

      }
    }

    \item{strand}{\code{signature(x="AlignedReadTrack")}: return a vector of
      strand specifiers for all track items, in the form '+' for the
      Watson strand, '-' for the Crick strand or '*' for either of the
      two.

      \emph{Usage:}

      \code{strand(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{strand(obj)}}

      }
    }

    \item{strand<-}{\code{signature(x="AlignedReadTrack")}: replace the
      strand information for the track items. The replacement value
      needs to be an appropriate scalar or vector of strand values.

      \emph{Usage:}

      \code{strand<-(x, value)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{value}: replacement value.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{strand(obj) <- "+"}}

      }
    }

    \item{values}{\code{signature(x="AlignedReadTrack")}: return all
      additional annotation information except for the genomic coordinates
      for the track items as a data.frame.

      \emph{Usage:}

      \code{values(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{values(obj)}}

      }
    }

    \item{coerce}{\code{signature(from="AlignedReadTrack",
	to="data.frame")}: coerce the \code{\linkS4class{GRanges}}
	object in the \code{range} slot into a regular data.frame.

       \emph{Examples:}

      \describe{

	\item{}{\code{as(obj, "data.frame")}}

      }
    }

    \item{displayPars}{\code{signature(x="AlignedReadTrack",
      name="character")}: list the value of the display parameter
      \code{name}. See \code{\link{settings}} for details on display
      parameters and customization.

      \emph{Usage:}

      \code{displayPars(x, name)}

      \emph{Examples:}

      \describe{

	\item{}{\code{displayPars(obj, "col")}}

      }
    }


    \item{displayPars}{\code{signature(x="AlignedReadTrack",
      name="missing")}: list the value of all available display
      parameters. See \code{\link{settings}} for details on display
      parameters and customization.

      \emph{Examples:}

      \describe{

	\item{}{\code{displayPars(obj)}}

      }
    }

    \item{getPar}{\code{signature(x="AlignedReadTrack", name="character")}:
      alias for the \code{displayPars} method. See
      \code{\link{settings}} for details on display parameters and
      customization.

      \emph{Usage:}

      \code{getPar(x, name)}

      \emph{Examples:}

      \describe{

	\item{}{\code{getPar(obj, "col")}}

      }
    }

    \item{getPar}{\code{signature(x="AlignedReadTrack", name="missing")}:
      alias for the \code{displayPars} method. See
      \code{\link{settings}} for details on display parameters and
      customization.

      \emph{Examples:}

      \describe{

	\item{}{\code{getPar(obj)}}

      }
    }

    \item{displayPars<-}{\code{signature(x="AlignedReadTrack",
      value="list")}: set display parameters using the values of the
      named list in \code{value}. See \code{\link{settings}} for details
      on display parameters and customization.

      \emph{Usage:}

      \code{displayPars<-(x, value)}

      \emph{Examples:}

      \describe{

	\item{}{\code{displayPars(obj) <- list(col="red", lwd=2)}}

      }
    }

    \item{setPar}{\code{signature(x="AlignedReadTrack", value="character")}:
      set the single display parameter \code{name} to \code{value}. Note
      that display parameters in the \code{AlignedReadTrack} class are
      pass-by-reference, so no re-assignmnet to the symbol \code{obj} is
      necessary. See \code{\link{settings}} for details on display
      parameters and customization.

      \emph{Usage:}

      \code{setPar(x, name, value)}

      \emph{Additional Arguments:}

      \describe{

	\item{}{\code{name}: the name of the display parameter to set.}

      }

      \emph{Examples:}

      \describe{

	\item{}{\code{setPar(obj, "col", "red")}}

      }
    }

    \item{setPar}{\code{signature(x="AlignedReadTrack", value="list")}: set
      display parameters by the values of the named list in
      \code{value}.  Note that display parameters in the
      \code{AlignedReadTrack} class are pass-by-reference, so no
      re-assignmnet to the symbol \code{obj} is necessary. See
      \code{\link{settings}} for details on display parameters and
      customization.


      \emph{Examples:}

      \describe{

	\item{}{\code{setPar(obj, list(col="red", lwd=2))}}

      }
    }

    \item{group}{\code{signature(GdObject="AlignedReadTrack")}: return
      grouping information for the individual items in the track. Unless
      overwritten in one of the sub-classes, this usualy returns
      \code{NULL}.

      \emph{Usage:}

      \code{group(GdObject)}

      \emph{Examples:}

      \describe{

	\item{}{\code{group(obj)}}

      }
    }

    \item{names}{\code{signature(x="AlignedReadTrack")}: return the value of
      the \code{name} slot.

      \emph{Usage:}

      \code{names(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{names(obj)}}

      }
    }

    \item{names<-}{\code{signature(x="AlignedReadTrack", value="character")}:
      set the value of the \code{name} slot.

      \emph{Usage:}

      \code{names<-(x, value)}

      \emph{Examples:}

      \describe{

	\item{}{\code{names(obj) <- "foo"}}

      }
    }

    \item{coords}{\code{signature(ImageMap="AlignedReadTrack")}: return the
      coordinates from the internal image map.

      \emph{Usage:}

      \code{coords(ImageMap)}

      \emph{Examples:}

      \describe{

	\item{}{\code{coords(obj)}}

      }
    }

    \item{tags}{\code{signature(x="AlignedReadTrack")}: return the tags from the
      internal image map.

      \emph{Usage:}

      \code{tags(x)}

      \emph{Examples:}

      \describe{

	\item{}{\code{tags(obj)}}

      }
    }
  }
}


\section{Display Parameters}{ 

  The following display parameters are set for objects of class
  \code{AlignedReadTrack} upon instantiation, unless one or more of
  them have already been set by one of the optional sub-class initializers,
  which always get precedence over these global defaults. See
  \code{\link{settings}} for details on setting graphical parameters
  for tracks. \describe{ 

    \item{}{\code{collapse=FALSE}: collapse overlapping ranges and
      aggregate the underlying data.} 

    \item{}{\code{detail="coverage"}: the amount of detail to plot the
      data. Either \code{coverage} to show the coverage only, or
      \code{reads} to show individual reads. For large data sets the
      latter can be very inefficient. Please note that \code{reads} is
      only available when the object has been created with option
      \code{coverageOnly=FALSE}.} 

    \item{}{\code{fill="#0080ff"}: the fill color for the coverage
      indicator.} 

    \item{}{\code{size=NULL}: the relative size of the track. Defaults
      to size selection based on the underlying data. Can be overridden
      in the \code{\link{plotTracks}} function.} 

    \item{}{\code{type="histogram"}: the plot type, one or several in
      \code{c("p","l", "b", "a", "s", "g", "r", "S", "smooth", "histogram",
      "mountain", "polygon", "h", "boxplot", "gradient", "heatmap",
      "horizon")}. See the 'Details' section in \code{\linkS4class{DataTrack}}
      for more information on the individual plotting types.} 

  } 

  Additional display parameters are being inherited from the respective
  parent classes. Note that not all of them may have an effect on the
  plotting of \code{AlignedReadTrack} objects. 

    \describe{ 

    \item{}{\code{\linkS4class{StackedTrack}}: 

      \describe{ 

        \item{}{\code{reverseStacking=FALSE}: Logical flag. Reverse
          the y-ordering of stacked items. I.e., features that are
          plotted on the bottom-most stacks will be moved to the top-most
          stack and vice versa.} 

        \item{}{\code{stackHeight=0.75}: Numeric between 0 and 1.
          Controls the vertical size and spacing between stacked
          elements. The number defines the proportion of the total
          available space for the stack that is used to draw the glyphs.
          E.g., a value of 0.5 means that half of the available vertical
          drawing space (for each stacking line) is used for the glyphs,
          and thus one quarter of the available space each is used for
          spacing above and below the glyph. Defaults to 0.75.} 

      } 

    } 

    \item{}{\code{\linkS4class{GdObject}}: 

      \describe{ 

        \item{}{\code{alpha=1}: Numeric scalar. The transparency for
          all track items.} 

        \item{}{\code{alpha.title=NULL}: Numeric scalar. The transparency
          for the title panel.} 

        \item{}{\code{background.legend="transparent"}: Integer or
          character scalar. The background color for the legend.} 

        \item{}{\code{background.panel="transparent"}: Integer or
          character scalar. The background color of the content panel.} 

        \item{}{\code{background.title="lightgray"}: Integer or character
          scalar. The background color for the title panel.} 

        \item{}{\code{cex=1}: Numeric scalar. The overall font expansion
          factor for all text and glyphs, unless a more specific
          definition exists.} 

        \item{}{\code{cex.axis=NULL}: Numeric scalar. The expansion
          factor for the axis annotation. Defaults to \code{NULL}, in
          which case it is automatically determined based on the
          available space.} 

        \item{}{\code{cex.title=NULL}: Numeric scalar. The expansion
          factor for the title panel. This effects the fontsize of both
          the title and the axis, if any. Defaults to \code{NULL},
          which means that the text size is automatically adjusted to
          the available space.} 

        \item{}{\code{col="#0080FF"}: Integer or character scalar.
          Default line color setting for all plotting elements, unless
          there is a more specific control defined elsewhere.} 

        \item{}{\code{col.axis="white"}: Integer or character scalar.
          The font and line color for the y axis, if any.} 

        \item{}{\code{col.border.title="white"}: Integer or character
          scalar. The border color for the title panels.} 

        \item{}{\code{col.frame="lightgray"}: Integer or character
          scalar. The line color used for the panel frame, if
          \code{frame==TRUE}} 

        \item{}{\code{col.grid="#808080"}: Integer or character scalar.
          Default line color for grid lines, both when \code{type=="g"}
          in \code{\link{DataTrack}}s and when display parameter
          \code{grid==TRUE}.} 

        \item{}{\code{col.line=NULL}: Integer or character scalar.
          Default colors for plot lines. Usually the same as the global
          \code{col} parameter.} 

        \item{}{\code{col.symbol=NULL}: Integer or character scalar.
          Default colors for plot symbols. Usually the same as the
          global \code{col} parameter.} 

        \item{}{\code{col.title="white"} \code{(Aliases: fontcolor.title)}:
          Integer or character scalar. The border color for the title panels} 

        \item{}{\code{fontcolor="black"}: Integer or character scalar.
          The font color for all text, unless a more specific definition
          exists.} 

        \item{}{\code{fontface=1}: Integer or character scalar. The
          font face for all text, unless a more specific definition exists.} 

        \item{}{\code{fontface.title=2}: Integer or character scalar.
          The font face for the title panels.} 

        \item{}{\code{fontfamily="sans"}: Integer or character scalar.
          The font family for all text, unless a more specific definition
          exists.} 

        \item{}{\code{fontfamily.title="sans"}: Integer or character
          scalar. The font family for the title panels.} 

        \item{}{\code{fontsize=12}: Numeric scalar. The font size for
          all text, unless a more specific definition exists.} 

        \item{}{\code{frame=FALSE}: Boolean. Draw a frame around the
          track when plotting.} 

        \item{}{\code{grid=FALSE}: Boolean, switching on/off the plotting
          of a grid.} 

        \item{}{\code{h=-1}: Integer scalar. Parameter controlling the
          number of horizontal grid lines, see \code{\link{panel.grid}}
          for details.} 

        \item{}{\code{lineheight=1}: Numeric scalar. The font line
          height for all text, unless a more specific definition exists.} 

        \item{}{\code{lty="solid"}: Numeric scalar. Default line type
          setting for all plotting elements, unless there is a more
          specific control defined elsewhere.} 

        \item{}{\code{lty.grid="solid"}: Integer or character scalar.
          Default line type for grid lines, both when \code{type=="g"}
          in \code{\link{DataTrack}}s and when display parameter
          \code{grid==TRUE}.} 

        \item{}{\code{lwd=1}: Numeric scalar. Default line width setting
          for all plotting elements, unless there is a more specific
          control defined elsewhere.} 

        \item{}{\code{lwd.border.title=1}: Integer scalar. The border
          width for the title panels.} 

        \item{}{\code{lwd.grid=1}: Numeric scalar. Default line width
          for grid lines, both when \code{type=="g"} in \code{\link{DataTrack}}s
          and when display parameter \code{grid==TRUE}.} 

        \item{}{\code{lwd.title=1}: Integer scalar. The border width
          for the title panels} 

        \item{}{\code{min.distance=1}: Numeric scalar. The minimum
          pixel distance before collapsing range items, only if
          \code{collapse==TRUE}. See \code{\link{collapsing}} for details.} 

        \item{}{\code{min.height=3}: Numeric scalar. The minimum range
          height in pixels to display. All ranges are expanded to this
          size in order to avoid rendering issues. See \code{\link{collapsing}}
          for details.} 

        \item{}{\code{min.width=1}: Numeric scalar. The minimum range
          width in pixels to display. All ranges are expanded to this
          size in order to avoid rendering issues. See \code{\link{collapsing}}
          for details.} 

        \item{}{\code{reverseStrand=FALSE}: Logical scalar. Set up the
          plotting coordinates in 3' -> 5' direction if \code{TRUE}.
          This will effectively mirror the plot on the vertical axis.} 

        \item{}{\code{rotation=0}: The rotation angle for all text
          unless a more specific definiton exists.} 

        \item{}{\code{rotation.title=90} \code{(Aliases: rotation.title)}:
          The rotation angle for the text in the title panel. Even
          though this can be adjusted, the automatic resizing of the
          title panel will currently not work, so use at own risk.} 

        \item{}{\code{showAxis=TRUE}: Boolean controlling whether to
          plot a y axis (only applies to track types where axes are
          implemented).} 

        \item{}{\code{showTitle=TRUE}: Boolean controlling whether to
          plot a title panel. Although this can be set individually
          for each track, in multi-track plots as created by
          \code{\link{plotTracks}} there will still be an empty
          placeholder in case any of the other tracks include a title.
          The same holds true for axes. Note that the the title panel
          background color could be set to transparent in order to
          completely hide it.} 

        \item{}{\code{v=-1}: Integer scalar. Parameter controlling the
          number of vertical grid lines, see \code{\link{panel.grid}}
          for details.} 

      } 

    } 

  } 

}

\author{Florian Hahne}

\seealso{

  \code{\linkS4class{AnnotationTrack}}

  \code{\linkS4class{DataTrack}}

  \code{\linkS4class{DisplayPars}}

  \code{\linkS4class{GRanges}}

  \code{\linkS4class{GdObject}}

  \code{\linkS4class{GeneRegionTrack}}

  \code{\linkS4class{IRanges}}

  \code{\linkS4class{ImageMap}}

  \code{\linkS4class{RangeTrack}}

  \code{\linkS4class{StackedTrack}}

  \code{\link{collapsing}}

  \code{\link{grouping}}

  \code{\link{panel.grid}}

  \code{\link{plotTracks}}

  \code{\link{settings}}

}

\examples{

## Construct from individual arguments
arTrack <- AlignedReadTrack(start=runif(1000, 100, 200), width=24,
genome="mm9", chromosome=7, strand=sample(c("+", "-"), 1000, TRUE))

\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if(!interactive())
{
font <- ps.options()$family
displayPars(arTrack) <- list(fontfamily=font, fontfamily.title=font)
}
}

## Plotting
plotTracks(arTrack)

## Track names
names(arTrack)
names(arTrack) <- "foo"
plotTracks(arTrack)

## Subsetting and splitting
subTrack <- subset(arTrack, from=110, to=130)
length(subTrack)
subTrack[1:2]
split(arTrack, strand(arTrack))

## Accessors
start(arTrack)
end(arTrack)
width(arTrack)
position(arTrack)
width(subTrack) <- 30

strand(arTrack)
strand(subTrack) <- "-"

chromosome(arTrack)
chromosome(subTrack) <- "chrX"

genome(arTrack)
genome(subTrack) <- "mm9"

range(arTrack)
ranges(arTrack)

coverage(arTrack)

## Annotation
values(arTrack)

## Stacking
stacking(arTrack)
stacking(arTrack) <- "dense"

## coercion
as(arTrack, "data.frame")

}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/settings.R
\name{settings}
\alias{settings}
\alias{addScheme}
\alias{getScheme}
\title{Setting display parameters to control the look and feel of the plots}
\arguments{
\item{scheme}{A named nested list of display parameters, where the first
level of nesting represents Gviz track object classes, and the second level
of nesting represents parameters.}

\item{name}{A character scalar with the scheme name.}
}
\value{
The \code{getScheme} function returns current scheme as a list structure.
}
\description{
The genome track plots in this package are all highly customizable by means
of so called 'display parameters'. This page highlights the use of these
parameters and list all available settings for the different track classes.
}
\details{
All of the package's track objects inherit the \code{dp} slot from the
\code{\linkS4class{GdObject}} parent class, which is the main container to
store an object's display parameters. Internally, the content of this slot
has to be an object of class \code{\linkS4class{DisplayPars}}, but the user
is usually not exposed to this low level implementation. Instead, there are
two main interaction points, namely the individual object constructor
functions and the final \code{\link{plotTracks}} function. In both cases,
all additional arguments that are not caught by any of the formally defined
function parameters are being interpreted as additional display parameters
and are automatically added to the aforementioned slot. The main difference
here is that display parameters that are passed on to the constructor
function are specific for an individual track object, whereas those supplied
to the \code{plotTracks} function will be applied to all the objects in the
plotting list. Not all display parameters have an effect on the plotting of
all track classes, and those will be silently ignored.

One can query the available display parameters for a given class as well as
their default values by calling the \code{\link{availableDisplayPars}}
function, or by inspecting the man pages of the individual track classes.
The structure of the classes defined in this package is hierarchical, and so
are the available display parameters, i.e., all objects inherit the
parameters defined in the commom \code{GdObject} parent class, and so on.

Once a track object has been created, the display parameters are still open
for modification. To this end, the \code{\linkS4class{DisplayPars}} replacement
method is available for all objects inheriting from class \code{GdObject}.
The method takes a named list of parameters as input, e.g.:

\code{displayPars(foo) <- list(col="red", lwd=2)}

In the same spirit, the currently set display parameters for the object
\code{foo} can be inferred using the \code{displayPars} method directly,
e.g.:

\code{displayPars(foo)}

For track objects inheriting from class
\code{\linkS4class{AnnotationTrack}}, display parameters that are not
formally defined in the class definition or in any of the parent classes are
considered to be valid R color identifiers that are used to distinguish
between different types of annotation features. For instance, the parameter
'miRNA' will be used to color all annotation features of class miRNA. The
annotation types can be set in the constructor function of the track object
via the \code{feature} argument. For most of the tracks that have been
inferred from one of the online repositories, this classification will
usually be downloaded along with the actual annotation data.

Users might find themselves changing the same parameters over and over
again, and it would make sense to register these modifications in a central
location once and for all. To this end the Gviz package supports display
parameter schemes. A scheme is essentially just a bunch of nested named
lists, where the names on the first level of nesting should correspond to
track class names, and the names on the second level to the display
parameters to set. The currently active schmeme can be changed by setting
the global option \code{Gviz.scheme}, and a new scheme can be registered by
using the \code{addScheme} function, providing both the list and the name
for the new scheme. The \code{getScheme} function is useful to get the
current scheme as a list structure, for instance to use as a skeleton for
your own custom scheme.

In order to make these settings persitant across R sessions one can create
one or several schemes in the global environment in the special object
\code{.GvizSchemes}, for instance by putting the necessary code in the
\code{.Rprofile} file. This object needs to be a named list of schemes, and
it will be collected when the Givz package loads. Its content is then
automatically added to the collection of available schemes.

Please note that because display parameters are stored with the track
objects, a scheme change only has an effect on those objects that are
created after the change has taken place.
}
\section{Display Parameters}{


\describe{

\item{GenomeAxisTrack:}{

\describe{

\item{\code{add35=FALSE}:}{ Logical scalar. Add 3' to 5' direction
indicators.}

\item{\code{add53=FALSE}:}{ Logical scalar. Add 5' to 3' direction
indicators.}

\item{\code{background.title="transparent"}:}{ Character scalar.  The
background color for the title panel. Defaults to omit the background.}

\item{\code{cex.id=0.7}:}{ Numeric scalar. The text size for the optional
range annotation.}

\item{\code{cex=0.8}:}{ Numeric scalar. The overall font expansion factor
for the axis annotation text.}

\item{\code{col.border.title="transparent"}:}{ Integer or character scalar.
The border color for the title panels.}

\item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
title panels.}

\item{\code{col.id="white"}:}{ Character scalar. The text color for the
optional range annotation.}

\item{\code{col.range="cornsilk4"}:}{ Character scalar. The border color for
highlighted regions on the axis.}

\item{\code{distFromAxis=1}:}{ Numeric scalar. Control the distance of the
axis annotation from the tick marks.}

\item{\code{exponent=NULL}:}{ Numeric scalar. The exponent for the axis
coordinates, e.g., 3 means mb, 6 means gb, etc. The default is to
automatically determine the optimal exponent.}

\item{\code{fill.range="cornsilk3"}:}{ Character scalar. The fill color for
highlighted regions on the axis.}

\item{\code{fontcolor="#808080"}:}{ Character scalar. The font color for the
axis annotation text.}

\item{\code{fontsize=10}:}{ Numeric scalar. Font size for the axis
annotation text in points.}

\item{\code{labelPos="alternating"}:}{ Character vector, one in
"alternating", "revAlternating", "above" or "below". The vertical
positioning of the axis labels. If \code{scale} is not \code{NULL}, the
possible values are "above", "below" and "beside".}

\item{\code{littleTicks=FALSE}:}{ Logical scalar. Add more fine-grained tick
marks.}

\item{\code{lwd=2}:}{ Numeric scalar. The line width for the axis
elementes.}

\item{\code{scale=NULL}:}{ Numeric scalar. If not \code{NULL} a small scale
is drawn instead of the full axis, if the value is between 0 and 1 it is
interpreted as a fraction of the current plotting region, otherwise as an
absolute length value in genomic coordinates.}

\item{\code{showId=FALSE}:}{ Logical scalar. Show the optional range
highlighting annotation.}

\item{\code{showTitle=FALSE}:}{ Logical scalar. Plot a title panel. Defaults
to omit the title panel.}

\item{\code{ticksAt=NULL}:}{ Numeric scalar. The exact x-position for
tickmarks (in base-pairs).}

\item{\code{size=NULL}:}{ Numeric scalar. The relative size of the track.
Can be overridden in the \code{\link{plotTracks}} function. Defaults to the
ideal size based on the other track settings.}

\item{\code{col="darkgray"}:}{ Character scalar. The color for the axis
lines and tickmarks.}

}

\bold{\emph{Inherited from class GdObject:}}

\describe{

\item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
items.}

\item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
title panel.}

\item{\code{background.panel="transparent"}:}{ Integer or character scalar.
The background color of the content panel.}

\item{\code{background.legend="transparent"}:}{ Integer or character scalar.
The background color for the legend.}

\item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
axis annotation. Defaults to \code{NULL}, in which case it is automatically
determined based on the available space.}

\item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
title panel. This effects the fontsize of both the title and the axis, if
any. Defaults to \code{NULL}, which means that the text size is
automatically adjusted to the available space.}

\item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
line color for the y axis, if any.}

\item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
color used for the panel frame, if \code{frame==TRUE}}

\item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
line color for grid lines, both when \code{type=="g"} in
\code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}

\item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
for plot lines. Usually the same as the global \code{col} parameter.}

\item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
for plot symbols. Usually the same as the global \code{col} parameter.}

\item{\code{col.title="white"}:}{ Integer or character scalar.  The border
color for the title panels}

\item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
content of the track to accomodate the minimum current device resolution.
See \code{\link{collapsing}} for details.}

\item{\code{fill="lightgray"}:}{ Integer or character scalar.  Default fill
color setting for all plotting elements, unless there is a more specific
control defined elsewhere.}

\item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
for the title panels.}

\item{\code{fontface=1}:}{ Integer or character scalar. The font face for
all text, unless a more specific definition exists.}

\item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
font family for the title panels.}

\item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
family for all text, unless a more specific definition exists.}

\item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
plotting.}

\item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
grid.}

\item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
horizontal grid lines, see \code{\link{panel.grid}} for details.}

\item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
text, unless a more specific definition exists.}

\item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
and when display parameter \code{grid==TRUE}.}

\item{\code{lty="solid"}:}{ Numeric scalar. Default line type setting for
all plotting elements, unless there is a more specific control defined
elsewhere.}

\item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
panels}

\item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
display parameter \code{grid==TRUE}.}

\item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
before collapsing range items, only if \code{collapse==TRUE}. See
\code{\link{collapsing}} for details.}

\item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
mirror the plot on the vertical axis.}

\item{\code{rotation.title=90}:}{ The rotation angle for the text in the
title panel. Even though this can be adjusted, the automatic resizing of the
title panel will currently not work, so use at own risk.}

\item{\code{rotation=0}:}{ The rotation angle for all text unless a more
specific definiton exists.}

\item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
(only applies to track types where axes are implemented).}

\item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
vertical grid lines, see \code{\link{panel.grid}} for details.}

}

}

\item{DataTrack:}{

\describe{

\item{\code{aggregateGroups=FALSE}:}{ Logical scalar. Aggregate the values
within a sample group using the aggregation funnction specified in the
\code{aggregation} parameter.}

\item{\code{aggregation="mean"}:}{ Function or character scalar.  Used to
aggregate values in windows or for collapsing overlapping items. The
function has to accept a numeric vector as a single input parameter and has
to return a numeric scalar with the aggregated value. Alternatively, one of
the predefined options \code{mean}, \code{median} \code{sum}, \code{min},
\code{max} or \code{extreme} can be supplied as a character scalar. Defaults
to \code{mean}.}

\item{\code{missingAsZero=TRUE}:}{ Logical scalar. Defines how the missing
values are treated in the aggregation procedure with running window. Setting
it to \code{TRUE} fills empty positions with zeros, which is default.
\code{FALSE} fills empty positions with \code{NA}.}

\item{\code{alpha.confint=0.3}:}{ Numeric scalar. The transparency for the
confidence intervalls in confint-type plots.}

\item{\code{amount=NULL}:}{ Numeric scalar. Amount of jittering in xy-type
plots. See \code{\link{panel.xyplot}} for details.}

\item{\code{baseline=NULL}:}{ Numeric scalar. Y-axis position of an optional
baseline. This parameter has a special meaning for mountain-type and
polygon-type plots, see the 'Details' section in
\code{\linkS4class{DataTrack}} for more information.}

\item{\code{box.legend=FALSE}:}{ Logical scalar. Draw a box around a
legend.}

\item{\code{box.ratio=1}:}{ Numeric scalar. Parameter controlling the
boxplot appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{box.width=NULL}:}{ Numeric scalar. Parameter controlling the
boxplot appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{grid=FALSE}:}{ Logical vector. Draw a line grid under the track
content.}

\item{\code{cex.legend=0.8}:}{ Numeric scalar. The size factor for the
legend text.}

\item{\code{cex.sampleNames=NULL}:}{ Numeric scalar. The size factor for the
sample names text in heatmap or horizon plots.  Defaults to an automatic
setting.}

\item{\code{cex=0.7}:}{ Numeric scalar. The default pixel size for plotting
symbols.}

\item{\code{coef=1.5}:}{ Numeric scalar. Parameter controlling the boxplot
appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{col.baseline=NULL}:}{ Character scalar. Color for the optional
baseline, defaults to the setting of \code{col}.}

\item{\code{col.confint=NA}:}{ Character vector. Border colors for the
confidence intervals for confint-type plots.}

\item{\code{col.boxplotFrame="#808080"}:}{ Character scalar.  Line color of
the frame around grouped boxplots.}

\item{\code{col.histogram="#808080"}:}{ Character scalar. Line color in
histogram-type plots.}

\item{\code{col.horizon=NA}:}{ The line color for the segments in the
\code{horizon}-type plot. See \code{\link{horizonplot}} for details.}

\item{\code{col.mountain=NULL}:}{ Character scalar. Line color in
mountain-type and polygon-type plots, defaults to the setting of
\code{col}.}

\item{\code{col.sampleNames="white"}:}{ Character or integer scalar. The
color used for the sample names in heatmap plots.}

\item{\code{col=c("#0080ff", "#ff00ff", "darkgreen", "#ff0000", "orange",
"#00ff00", "brown")}:}{ Character or integer vector.  The color used for all
line and symbol elements, unless there is a more specific control defined
elsewhere. Unless \code{groups} are specified, only the first color in the
vector is usually regarded.}

\item{\code{collapse=FALSE}:}{ Logical scalar. Collapse overlapping ranges
and aggregate the underlying data.}

\item{\code{degree=1}:}{ Numeric scalar. Parameter controlling the loess
calculation for smooth and mountain-type plots.  See
\code{\link{panel.loess}} for details.}

\item{\code{do.out=TRUE}:}{ Logical scalar. Parameter controlling the
boxplot appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{evaluation=50}:}{ Numeric scalar. Parameter controlling the
loess calculation for smooth and mountain-type plots. See
\code{\link{panel.loess}} for details.}

\item{\code{factor=0.5}:}{ Numeric scalar. Factor to control amount of
jittering in xy-type plots. See \code{\link{panel.xyplot}} for details.}

\item{\code{family="symmetric"}:}{ Character scalar. Parameter controlling
the loess calculation for smooth and mountain-type plots. See
\code{\link{panel.loess}} for details.}

\item{\code{fill.confint=NULL}:}{ Character vector. Fill colors for the
confidence intervals for confint-type plots.}

\item{\code{fill.histogram=NULL}:}{ Character scalar. Fill color in
histogram-type plots, defaults to the setting of \code{fill}.}

\item{\code{fill.horizon=c("#B41414", "#E03231", "#F7A99C", "#9FC8DC",
"#468CC8", "#0165B3")}:}{ The fill colors for the segments in the
\code{horizon}-type plot. This should be a vector of length six, where the
first three entries are the colors for positive changes, and the latter
three entries are the colors for negative changes. Defaults to a red-blue
color scheme. See \code{\link{horizonplot}} for details.}

\item{\code{fill.mountain=c("#CCFFFF", "#FFCCFF")}:}{ Character vector of
length 2. Fill color in mountain-type and polygon-type plots.}

\item{\code{fontface.legend=NULL}:}{ Integer or character scalar. The font
face for the legend text.}

\item{\code{fontfamily.legend=NULL}:}{ Integer or character scalar. The font
family for the legend text.}

\item{\code{fontsize.legend=NULL}:}{ Numeric scalar. The pixel size for the
legend text.}

\item{\code{fontcolor.legend="#808080"}:}{ Integer or character scalar. The
font color for the legend text.}

\item{\code{gradient=c("#F7FBFF", "#DEEBF7", "#C6DBEF", "#9ECAE1",
"#6BAED6", "#4292C6", "#2171B5", "#08519C", "#08306B")}:}{ Character vector.
The base colors for the \code{gradient} plotting type or the \code{heatmap}
type with a single group. When plotting heatmaps with more than one group,
the \code{col} parameter can be used to control the group color scheme,
however the gradient will always be from white to 'col' and thus does not
offer as much flexibility as this \code{gradient} parameter.}

\item{\code{groups=NULL}:}{ Vector coercable to a factor.  Optional sample
grouping. See 'Details' section in \code{\linkS4class{DataTrack}} for
further information.}

\item{\code{horizon.origin=0}:}{ The baseline relative to which changes are
indicated on the \code{horizon}-type plot. See \code{\link{horizonplot}} for
details.}

\item{\code{horizon.scale=NULL}:}{ The scale for each of the segments in the
\code{horizon}-type plot. Defaults to 1/3 of the absolute data range. See
\code{\link{horizonplot}} for details.}

\item{\code{jitter.x=FALSE}:}{ Logical scalar. Toggle on jittering on the x
axis in xy-type plots. See \code{\link{panel.xyplot}} for details.}

\item{\code{jitter.y=FALSE}:}{ Logical scalar. Toggle off jittering on the y
axis in xy-type plots. See \code{\link{panel.xyplot}} for details.}

\item{\code{levels.fos=NULL}:}{ Numeric scalar. Parameter controlling the
boxplot appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{legend=TRUE}:}{ Boolean triggering the addition of a legend to
the track to indicate groups. This only has an effect if at least two groups
are present.}

\item{\code{lineheight.legend=NULL}:}{ Numeric scalar. The line height for
the legend text.}

\item{\code{lty.baseline=NULL}:}{ Character or numeric scalar.  Line type of
the optional baseline, defaults to the setting of \code{lty}.}

\item{\code{lty.mountain=NULL}:}{ Character or numeric scalar.  Line type in
mountain-type and polygon-type plots, defaults to the setting of
\code{lty}.}

\item{\code{lwd.baseline=NULL}:}{ Numeric scalar. Line width of the optional
baseline, defaults to the setting of \code{lwd}.}

\item{\code{lwd.mountain=NULL}:}{ Numeric scalar. Line width in
mountain-type and polygon-type plots, defaults to the setting of
\code{lwd}.}

\item{\code{min.distance=0}:}{ Numeric scalar. The mimimum distance in pixel
below which to collapse ranges.}

\item{\code{na.rm=FALSE}:}{ Boolean controlling whether to discard all NA
values when plotting or to keep empty spaces for NAs}

\item{\code{ncolor=100}:}{ Integer scalar. The number of colors for the
'gradient' plotting type}

\item{\code{notch.frac=0.5}:}{ Numeric scalar. Parameter controlling the
boxplot appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{notch=FALSE}:}{ Logical scalar. Parameter controlling the
boxplot appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{pch=20}:}{ Integer scalar. The type of glyph used for plotting
symbols.}

\item{\code{separator=0}:}{ Numeric scalar. Number of pixels used to
separate individual samples in heatmap- and horizon-type plots.}

\item{\code{showColorBar=TRUE}:}{ Boolean. Indicate the data range color
mapping in the axis for 'heatmap' or 'gradient' types.}

\item{\code{showSampleNames=FALSE}:}{ Boolean. Display the names of the
individual samples in a heatmap or a horizon plot.}

\item{\code{size=NULL}:}{ Numeric scalar. The relative size of the track.
Can be overridden in the \code{\link{plotTracks}} function. By default the
size will be set automatically based on the selected plotting type.}

\item{\code{span=0.2}:}{ Numeric scalar. Parameter controlling the loess
calculation for smooth and mountain-type plots.  See
\code{\link{panel.loess}} for details.}

\item{\code{stackedBars=TRUE}:}{ Logical scalar. When there are several data
groups, draw the histogram-type plots as stacked barplots or grouped side by
side.}

\item{\code{stats=X[[i]]}:}{ Function. Parameter controlling the boxplot
appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{transformation=NULL}:}{ Function. Applied to the data matrix
prior to plotting or when calling the \code{score} method. The function
should accept exactly one input argument and its return value needs to be a
numeric vector which can be coerced back into a data matrix of identical
dimensionality as the input data.}

\item{\code{type="p"}:}{ Character vector. The plot type, one or several in
\code{p},\code{l}, \code{b}, \code{a}, \code{a_confint}, \code{s}, \code{g},
\code{r}, \code{S}, \code{confint}, \code{smooth}, \code{histogram},
\code{mountain}, \code{polygon}, \code{h}, \code{boxplot}, \code{gradient},
\code{heatmap}, \code{horizon}. See 'Details' section in
\code{\linkS4class{DataTrack}} for more information on the individual
plotting types.}

\item{\code{varwidth=FALSE}:}{ Logical scalar. Parameter controlling the
boxplot appearance. See \code{\link{panel.bwplot}} for details.}

\item{\code{window=NULL}:}{ Numeric or character scalar.  Aggregate the rows
values of the data matrix to \code{window} equally sized slices on the data
range using the method defined in \code{aggregation}. If negative, apply a
running window of size \code{windowSize} using the same aggregation method.
Alternatively, the special value \code{auto} causes the function to
determine the optimal window size to avoid overplotting, and \code{fixed}
uses fixed-size windows of size \code{windowSize}.}

\item{\code{windowSize=NULL}:}{ Numeric scalar. The size of the running
window when the value of \code{window} is negative.}

\item{\code{ylim=NULL}:}{ Numeric vector of length 2. The range of the
y-axis scale.}

\item{\code{yTicksAt=NULL}:}{ Numeric vector. The points at which y-axis
tick-marks are to be drawn. By default, when \code{NULL}, tickmark locations
are computed.}

}

\bold{\emph{Inherited from class GdObject:}}

\describe{

\item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
items.}

\item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
title panel.}

\item{\code{background.panel="transparent"}:}{ Integer or character scalar.
The background color of the content panel.}

\item{\code{background.title="lightgray"}:}{ Integer or character scalar.
The background color for the title panel.}

\item{\code{background.legend="transparent"}:}{ Integer or character scalar.
The background color for the legend.}

\item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
axis annotation. Defaults to \code{NULL}, in which case it is automatically
determined based on the available space.}

\item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
title panel. This effects the fontsize of both the title and the axis, if
any. Defaults to \code{NULL}, which means that the text size is
automatically adjusted to the available space.}

\item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
line color for the y axis, if any.}

\item{\code{col.border.title="white"}:}{ Integer or character scalar. The
border color for the title panels.}

\item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
color used for the panel frame, if \code{frame==TRUE}}

\item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
line color for grid lines, both when \code{type=="g"} in
\code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}

\item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
for plot lines. Usually the same as the global \code{col} parameter.}

\item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
for plot symbols. Usually the same as the global \code{col} parameter.}

\item{\code{col.title="white"}:}{ Integer or character scalar.  The border
color for the title panels}

\item{\code{fill="lightgray"}:}{ Integer or character scalar.  Default fill
color setting for all plotting elements, unless there is a more specific
control defined elsewhere.}

\item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
color for all text, unless a more specific definition exists.}

\item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
for the title panels.}

\item{\code{fontface=1}:}{ Integer or character scalar. The font face for
all text, unless a more specific definition exists.}

\item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
font family for the title panels.}

\item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
family for all text, unless a more specific definition exists.}

\item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
unless a more specific definition exists.}

\item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
plotting.}

\item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
horizontal grid lines, see \code{\link{panel.grid}} for details.}

\item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
text, unless a more specific definition exists.}

\item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
and when display parameter \code{grid==TRUE}.}

\item{\code{lty="solid"}:}{ Numeric scalar. Default line type setting for
all plotting elements, unless there is a more specific control defined
elsewhere.}

\item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
title panels.}

\item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
panels}

\item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
display parameter \code{grid==TRUE}.}

\item{\code{lwd=1}:}{ Numeric scalar. Default line width setting for all
plotting elements, unless there is a more specific control defined
elsewhere.}

\item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
mirror the plot on the vertical axis.}

\item{\code{rotation.title=90}:}{ The rotation angle for the text in the
title panel. Even though this can be adjusted, the automatic resizing of the
title panel will currently not work, so use at own risk.}

\item{\code{rotation=0}:}{ The rotation angle for all text unless a more
specific definiton exists.}

\item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
(only applies to track types where axes are implemented).}

\item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
panel. Although this can be set individually for each track, in multi-track
plots as created by \code{\link{plotTracks}} there will still be an empty
placeholder in case any of the other tracks include a title.  The same holds
true for axes. Note that the the title panel background color could be set
to transparent in order to completely hide it.}

\item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
vertical grid lines, see \code{\link{panel.grid}} for details.}

}

}

\item{IdeogramTrack:}{

\describe{

\item{\code{background.title="transparent"}:}{ Character scalar.  The
background color for the title panel. Defaults to omit the background.}

\item{\code{bevel=0.45}:}{ Numeric scalar, between 0 and 1.  The level of
smoothness for the two ends of the ideogram.}

\item{\code{centromereShape="triangle"}:}{ Character scalar.  The shape of
the centromere. Only "triangle" or "circle" is accepted. Default to
"triangle"}

\item{\code{cex.bands=0.7}:}{ Numeric scalar. The font expansion factor for
the chromosome band identifier text.}

\item{\code{cex=0.8}:}{ Numeric scalar. The overall font expansion factor
for the chromosome name text.}

\item{\code{col="red"}:}{ Character scalar. The border color used for the
highlighting of the currently displayed genomic region.}

\item{\code{col.border.title="transparent"}:}{ Integer or character scalar.
The border color for the title panels.}

\item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
title panels.}

\item{\code{fill="#FFE3E6"}:}{ Character scalar. The fill color used for the
highlighting of the currently displayed genomic region.}

\item{\code{fontface=1}:}{ Character scalar. The font face for the
chromosome name text.}

\item{\code{fontfamily="sans"}:}{ Character scalar. The font family for the
chromosome name text.}

\item{\code{fontcolor="#808080"}:}{ Character scalar. The font color for the
chromosome name text.}

\item{\code{fontsize=10}:}{ Numeric scalar. The font size for the chromosome
name text.}

\item{\code{outline=FALSE}:}{ Logical scalar. Add borders to the individual
chromosome staining bands.}

\item{\code{showBandId=FALSE}:}{ Logical scalar. Show the identifier for the
chromosome bands if there is space for it.}

\item{\code{lty=1}:}{ Character or integer scalar. The line type used for
the highlighting of the currently displayed genomic region.}

\item{\code{lwd=1}:}{ Numeric scalar. The line width used for the
highlighting of the currently displayed genomic region.}

\item{\code{showId=TRUE}:}{ Logical scalar. Indicate the chromosome name
next to the ideogram.}

\item{\code{showTitle=FALSE}:}{ Logical scalar. Plot a title panel. Defaults
to omit the title panel.}

\item{\code{size=NULL}:}{ Numeric scalar. The relative size of the track.
Defaults to automatic size setting. Can also be overridden in the
\code{\link{plotTracks}} function.}

}

\bold{\emph{Inherited from class GdObject:}}

\describe{

\item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
items.}

\item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
title panel.}

\item{\code{background.panel="transparent"}:}{ Integer or character scalar.
The background color of the content panel.}

\item{\code{background.legend="transparent"}:}{ Integer or character scalar.
The background color for the legend.}

\item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
axis annotation. Defaults to \code{NULL}, in which case it is automatically
determined based on the available space.}

\item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
title panel. This effects the fontsize of both the title and the axis, if
any. Defaults to \code{NULL}, which means that the text size is
automatically adjusted to the available space.}

\item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
line color for the y axis, if any.}

\item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
color used for the panel frame, if \code{frame==TRUE}}

\item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
line color for grid lines, both when \code{type=="g"} in
\code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}

\item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
for plot lines. Usually the same as the global \code{col} parameter.}

\item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
for plot symbols. Usually the same as the global \code{col} parameter.}

\item{\code{col.title="white"}:}{ Integer or character scalar.  The border
color for the title panels}

\item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
content of the track to accomodate the minimum current device resolution.
See \code{\link{collapsing}} for details.}

\item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
for the title panels.}

\item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
font family for the title panels.}

\item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
plotting.}

\item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
grid.}

\item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
horizontal grid lines, see \code{\link{panel.grid}} for details.}

\item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
text, unless a more specific definition exists.}

\item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
and when display parameter \code{grid==TRUE}.}

\item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
panels}

\item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
display parameter \code{grid==TRUE}.}

\item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
before collapsing range items, only if \code{collapse==TRUE}. See
\code{\link{collapsing}} for details.}

\item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
mirror the plot on the vertical axis.}

\item{\code{rotation.title=90}:}{ The rotation angle for the text in the
title panel. Even though this can be adjusted, the automatic resizing of the
title panel will currently not work, so use at own risk.}

\item{\code{rotation=0}:}{ The rotation angle for all text unless a more
specific definiton exists.}

\item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
(only applies to track types where axes are implemented).}

\item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
vertical grid lines, see \code{\link{panel.grid}} for details.}

}

}

\item{AnnotationTrack:}{

\describe{

\item{\code{arrowHeadWidth=30}:}{ Numeric scalar. The width of the arrow
head in pixels if \code{shape} is \code{fixedArrow}.}

\item{\code{arrowHeadMaxWidth=40}:}{ Numeric scalar. The maximum width of
the arrow head in pixels if \code{shape} is \code{arrow}.}

\item{\code{cex.group=0.6}:}{ Numeric scalar. The font expansion factor for
the group-level annotation.}

\item{\code{cex=1}:}{ Numeric scalar. The font expansion factor for item
identifiers.}

\item{\code{col.line="darkgray"}:}{ Character scalar. The color used for
connecting lines between grouped items. Defaults to a light gray, but if set
to \code{NULL} the same color as for the first item in the group is used.}

\item{\code{col="transparent"}:}{ Character or integer scalar.  The border
color for all track items.}

\item{\code{featureAnnotation=NULL}:}{ Character scalar. Add annotation
information to the individual track elements. This can be a value in
\code{id}, \code{group} or \code{feature}.  Defaults to \code{id}. Only
works if \code{showFeatureId} is not \code{FALSE}.}

\item{\code{fill="lightblue"}:}{ Character or integer scalar.  The fill
color for untyped items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{fontfamily.group="sans"}:}{ Character scalar. The font family
for the group-level annotation.}

\item{\code{fontcolor.group="#808080"}:}{ Character or integer scalar. The
font color for the group-level annotation.}

\item{\code{fontcolor.item="white"}:}{ Character or integer scalar. The font
color for item identifiers.}

\item{\code{fontface.group=2}:}{ Numeric scalar. The font face for the
group-level annotation.}

\item{\code{fontsize.group=12}:}{ Numeric scalar. The font size for the
group-level annotation.}

\item{\code{groupAnnotation=NULL}:}{ Character scalar. Add annotation
information as group labels. This can be a value in \code{id}, \code{group}
or \code{feature}. Defaults to \code{group}. Only works if \code{showId} is
not \code{FALSE}.}

\item{\code{just.group="left"}:}{ Character scalar. the justification of
group labels. Either \code{left}, \code{right}, \code{above} or
\code{below}.}

\item{\code{lex=1}:}{ Numeric scalar. The line expansion factor for all
track items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{lineheight=1}:}{ Numeric scalar. The font line height for item
identifiers.}

\item{\code{lty="solid"}:}{ Character or integer scalar. The line type for
all track items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{lwd=1}:}{ Integer scalar. The line width for all track items.
This is also used to connect grouped items. See \code{\link{grouping}} for
details.}

\item{\code{mergeGroups=FALSE}:}{ Logical scalar. Merge fully overlapping
groups if \code{collapse==TRUE}.}

\item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details. For feathered
bars indicating the strandedness of grouped items this also controls the
height of the arrow feathers.}

\item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{rotation=0}:}{ Numeric scalar. The degree of text rotation for
item identifiers.}

\item{\code{rotation.group=0}:}{ Numeric scalar. The degree of text rotation
for group labels.}

\item{\code{rotation.item=0}:}{ Numeric scalar. The degree of text rotation
for item identifiers.}

\item{\code{shape="arrow"}:}{ Character scalar. The shape in which to
display the track items. Currently only \code{box}, \code{arrow},
\code{fixedArrow}, \code{ellipse}, and \code{smallArrow} are implemented.}

\item{\code{showFeatureId=FALSE}:}{ Logical scalar. Control whether to plot
the individual track item identifiers.}

\item{\code{showId=FALSE}:}{ Logical scalar. Control whether to annotate
individual groups.}

\item{\code{showOverplotting=FALSE}:}{ Logical scalar. Use a color gradient
to show the amount of overplotting for collapsed items. This implies that
\code{collapse==TRUE}}

\item{\code{size=1}:}{ Numeric scalar. The relative size of the track. Can
be overridden in the \code{\link{plotTracks}} function.}

}

\bold{\emph{Inherited from class StackedTrack:}}

\describe{

\item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
vertical size and spacing between stacked elements. The number defines the
proportion of the total available space for the stack that is used to draw
the glyphs.  E.g., a value of 0.5 means that half of the available vertical
drawing space (for each stacking line) is used for the glyphs, and thus one
quarter of the available space each is used for spacing above and below the
glyph. Defaults to 0.75.}

\item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
of stacked items. I.e., features that are plotted on the bottom-most stacks
will be moved to the top-most stack and vice versa.}

}

\bold{\emph{Inherited from class GdObject:}}

\describe{

\item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
items.}

\item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
title panel.}

\item{\code{background.panel="transparent"}:}{ Integer or character scalar.
The background color of the content panel.}

\item{\code{background.title="lightgray"}:}{ Integer or character scalar.
The background color for the title panel.}

\item{\code{background.legend="transparent"}:}{ Integer or character scalar.
The background color for the legend.}

\item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
axis annotation. Defaults to \code{NULL}, in which case it is automatically
determined based on the available space.}

\item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
title panel. This effects the fontsize of both the title and the axis, if
any. Defaults to \code{NULL}, which means that the text size is
automatically adjusted to the available space.}

\item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
line color for the y axis, if any.}

\item{\code{col.border.title="white"}:}{ Integer or character scalar. The
border color for the title panels.}

\item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
color used for the panel frame, if \code{frame==TRUE}}

\item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
line color for grid lines, both when \code{type=="g"} in
\code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}

\item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
for plot symbols. Usually the same as the global \code{col} parameter.}

\item{\code{col.title="white"}:}{ Integer or character scalar.  The border
color for the title panels}

\item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
content of the track to accomodate the minimum current device resolution.
See \code{\link{collapsing}} for details.}

\item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
color for all text, unless a more specific definition exists.}

\item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
for the title panels.}

\item{\code{fontface=1}:}{ Integer or character scalar. The font face for
all text, unless a more specific definition exists.}

\item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
font family for the title panels.}

\item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
family for all text, unless a more specific definition exists.}

\item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
unless a more specific definition exists.}

\item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
plotting.}

\item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
grid.}

\item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
horizontal grid lines, see \code{\link{panel.grid}} for details.}

\item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
and when display parameter \code{grid==TRUE}.}

\item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
title panels.}

\item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
panels}

\item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
display parameter \code{grid==TRUE}.}

\item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
before collapsing range items, only if \code{collapse==TRUE}. See
\code{\link{collapsing}} for details.}

\item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
mirror the plot on the vertical axis.}

\item{\code{rotation.title=90}:}{ The rotation angle for the text in the
title panel. Even though this can be adjusted, the automatic resizing of the
title panel will currently not work, so use at own risk.}

\item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
(only applies to track types where axes are implemented).}

\item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
panel. Although this can be set individually for each track, in multi-track
plots as created by \code{\link{plotTracks}} there will still be an empty
placeholder in case any of the other tracks include a title.  The same holds
true for axes. Note that the the title panel background color could be set
to transparent in order to completely hide it.}

\item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
vertical grid lines, see \code{\link{panel.grid}} for details.}

}

}

\item{GeneRegionTrack:}{

\describe{

\item{\code{arrowHeadWidth=10}:}{ Numeric scalar. The width of the arrow
head in pixels if \code{shape} is \code{fixedArrow}.}

\item{\code{arrowHeadMaxWidth=20}:}{ Numeric scalar. The maximum width of
the arrow head in pixels if \code{shape} is \code{arrow}.}

\item{\code{col=NULL}:}{ Character or integer scalar. The border color for
all track items. Defaults to using the same color as in \code{fill}, also
taking into account different track \code{features}.}

\item{\code{collapseTranscripts=FALSE}:}{ Logical or character scalar. Can
be one in \code{gene}, \code{longest}, \code{shortest} or \code{meta}. Merge
all transcripts of the same gene into one single gene model. In the case of
\code{gene} (or \code{TRUE}), this will only keep the start location of the
first exon and the end location of the last exon from all transcripts of the
gene. For \code{shortest} and \code{longest}, only the longest or shortest
transcript model is retained.  For \code{meta}, a meta-transcript containing
the union of all exons is formed (essentially identical to the operation
\code{reduce(geneModel)}).}

\item{\code{exonAnnotation=NULL}:}{ Character scalar. Add annotation
information to the individual exon models. This can be a value in
\code{symbol}, \code{gene}, \code{transcript}, \code{exon} or
\code{feature}. Defaults to \code{exon}. Only works if \code{showExonId} is
not \code{FALSE}.}

\item{\code{fill="orange"}:}{ Character or integer scalar. The fill color
for untyped items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{min.distance=0}:}{ Numeric scalar. The minimum pixel distance
before collapsing range items, only if \code{collapse==TRUE}. See
\code{\link{collapsing}} for details. Note that a value larger than 0 may
lead to UTR regions being merged to CDS regions, which in most cases is not
particularly useful.}

\item{\code{shape=c("smallArrow", "box")}:}{ Character scalar.  The shape in
which to display the track items. Currently only \code{box}, \code{arrow},
\code{ellipse}, and \code{smallArrow} are implemented.}

\item{\code{showExonId=NULL}:}{ Logical scalar. Control whether to plot the
individual exon identifiers.}

\item{\code{thinBoxFeature=c("utr", "ncRNA", "utr3", "utr5", "3UTR",
"5UTR", "miRNA", "lincRNA", "three_prime_UTR", "five_prime_UTR")}:}{ Character
vector. A listing of feature types that should be drawn with thin boxes.
Typically those are non-coding elements.}

\item{\code{transcriptAnnotation=NULL}:}{ Character scalar.  Add annotation
information as transcript labels. This can be a value in \code{symbol},
\code{gene}, \code{transcript}, \code{exon} or \code{feature}. Defaults to
\code{symbol}.  Only works if \code{showId} is not \code{FALSE}.}

}

\bold{\emph{Inherited from class AnnotationTrack:}}

\describe{

\item{\code{cex.group=0.6}:}{ Numeric scalar. The font expansion factor for
the group-level annotation.}

\item{\code{cex=1}:}{ Numeric scalar. The font expansion factor for item
identifiers.}

\item{\code{col.line="darkgray"}:}{ Character scalar. The color used for
connecting lines between grouped items. Defaults to a light gray, but if set
to \code{NULL} the same color as for the first item in the group is used.}

\item{\code{featureAnnotation=NULL}:}{ Character scalar. Add annotation
information to the individual track elements. This can be a value in
\code{id}, \code{group} or \code{feature}.  Defaults to \code{id}. Only
works if \code{showFeatureId} is not \code{FALSE}.}

\item{\code{fontfamily.group="sans"}:}{ Character scalar. The font family
for the group-level annotation.}

\item{\code{fontcolor.group="#808080"}:}{ Character or integer scalar. The
font color for the group-level annotation.}

\item{\code{fontcolor.item="white"}:}{ Character or integer scalar. The font
color for item identifiers.}

\item{\code{fontface.group=2}:}{ Numeric scalar. The font face for the
group-level annotation.}

\item{\code{fontsize.group=12}:}{ Numeric scalar. The font size for the
group-level annotation.}

\item{\code{groupAnnotation=NULL}:}{ Character scalar. Add annotation
information as group labels. This can be a value in \code{id}, \code{group}
or \code{feature}. Defaults to \code{group}. Only works if \code{showId} is
not \code{FALSE}.}

\item{\code{just.group="left"}:}{ Character scalar. the justification of
group labels. Either \code{left}, \code{right}, \code{above} or
\code{below}.}

\item{\code{lex=1}:}{ Numeric scalar. The line expansion factor for all
track items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{lineheight=1}:}{ Numeric scalar. The font line height for item
identifiers.}

\item{\code{lty="solid"}:}{ Character or integer scalar. The line type for
all track items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{lwd=1}:}{ Integer scalar. The line width for all track items.
This is also used to connect grouped items. See \code{\link{grouping}} for
details.}

\item{\code{mergeGroups=FALSE}:}{ Logical scalar. Merge fully overlapping
groups if \code{collapse==TRUE}.}

\item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details. For feathered
bars indicating the strandedness of grouped items this also controls the
height of the arrow feathers.}

\item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{rotation=0}:}{ Numeric scalar. The degree of text rotation for
item identifiers.}

\item{\code{rotation.group=0}:}{ Numeric scalar. The degree of text rotation
for group labels.}

\item{\code{rotation.item=0}:}{ Numeric scalar. The degree of text rotation
for item identifiers.}

\item{\code{showFeatureId=FALSE}:}{ Logical scalar. Control whether to plot
the individual track item identifiers.}

\item{\code{showId=FALSE}:}{ Logical scalar. Control whether to annotate
individual groups.}

\item{\code{showOverplotting=FALSE}:}{ Logical scalar. Use a color gradient
to show the amount of overplotting for collapsed items. This implies that
\code{collapse==TRUE}}

\item{\code{size=1}:}{ Numeric scalar. The relative size of the track. Can
be overridden in the \code{\link{plotTracks}} function.}

}

\bold{\emph{Inherited from class StackedTrack:}}

\describe{

\item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
vertical size and spacing between stacked elements. The number defines the
proportion of the total available space for the stack that is used to draw
the glyphs.  E.g., a value of 0.5 means that half of the available vertical
drawing space (for each stacking line) is used for the glyphs, and thus one
quarter of the available space each is used for spacing above and below the
glyph. Defaults to 0.75.}

\item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
of stacked items. I.e., features that are plotted on the bottom-most stacks
will be moved to the top-most stack and vice versa.}

}

\bold{\emph{Inherited from class GdObject:}}

\describe{

\item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
items.}

\item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
title panel.}

\item{\code{background.panel="transparent"}:}{ Integer or character scalar.
The background color of the content panel.}

\item{\code{background.title="lightgray"}:}{ Integer or character scalar.
The background color for the title panel.}

\item{\code{background.legend="transparent"}:}{ Integer or character scalar.
The background color for the legend.}

\item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
axis annotation. Defaults to \code{NULL}, in which case it is automatically
determined based on the available space.}

\item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
title panel. This effects the fontsize of both the title and the axis, if
any. Defaults to \code{NULL}, which means that the text size is
automatically adjusted to the available space.}

\item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
line color for the y axis, if any.}

\item{\code{col.border.title="white"}:}{ Integer or character scalar. The
border color for the title panels.}

\item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
color used for the panel frame, if \code{frame==TRUE}}

\item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
line color for grid lines, both when \code{type=="g"} in
\code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}

\item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
for plot symbols. Usually the same as the global \code{col} parameter.}

\item{\code{col.title="white"}:}{ Integer or character scalar.  The border
color for the title panels}

\item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
content of the track to accomodate the minimum current device resolution.
See \code{\link{collapsing}} for details.}

\item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
color for all text, unless a more specific definition exists.}

\item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
for the title panels.}

\item{\code{fontface=1}:}{ Integer or character scalar. The font face for
all text, unless a more specific definition exists.}

\item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
font family for the title panels.}

\item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
family for all text, unless a more specific definition exists.}

\item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
unless a more specific definition exists.}

\item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
plotting.}

\item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
grid.}

\item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
horizontal grid lines, see \code{\link{panel.grid}} for details.}

\item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
and when display parameter \code{grid==TRUE}.}

\item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
title panels.}

\item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
panels}

\item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
display parameter \code{grid==TRUE}.}

\item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
mirror the plot on the vertical axis.}

\item{\code{rotation.title=90}:}{ The rotation angle for the text in the
title panel. Even though this can be adjusted, the automatic resizing of the
title panel will currently not work, so use at own risk.}

\item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
(only applies to track types where axes are implemented).}

\item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
panel. Although this can be set individually for each track, in multi-track
plots as created by \code{\link{plotTracks}} there will still be an empty
placeholder in case any of the other tracks include a title.  The same holds
true for axes. Note that the the title panel background color could be set
to transparent in order to completely hide it.}

\item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
vertical grid lines, see \code{\link{panel.grid}} for details.}

}

}

\item{BiomartGeneRegionTrack:}{

\describe{

\item{\code{C_segment="burlywood4"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'C_segment'.}

\item{\code{D_segment="lightblue"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'C_segment'.}

\item{\code{J_segment="dodgerblue2"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'C_segment'.}

\item{\code{Mt_rRNA="yellow"}:}{ Character or integer scalar.  Fill color
for annotation objects of type 'Mt_rRNA'.}

\item{\code{Mt_tRNA="darkgoldenrod"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'Mt_tRNA'.}

\item{\code{Mt_tRNA_pseudogene="darkgoldenrod1"}:}{ Character or integer
scalar. Fill color for annotation objects of type 'Mt_tRNA_pseudogene'.}

\item{\code{V_segment="aquamarine"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'V_segment'.}

\item{\code{miRNA="cornflowerblue"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'L_segment'.}

\item{\code{miRNA_pseudogene="cornsilk"}:}{ Character or integer scalar.
Fill color for annotation objects of type 'miRNA_pseudogene'.}

\item{\code{misc_RNA="cornsilk3"}:}{ Character or integer scalar. Fill color
for annotation objects of type 'misc_RNA'.}

\item{\code{misc_RNA_pseudogene="cornsilk4"}:}{ Character or integer scalar.
Fill color for annotation objects of type 'misc_RNA_pseudogene'.}

\item{\code{protein_coding="#FFD58A"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'protein_coding'.}

\item{\code{pseudogene="brown1"}:}{ Character or integer scalar.  Fill color
for annotation objects of type 'pseudogene'.}

\item{\code{rRNA="darkolivegreen1"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'rRNA'.}

\item{\code{rRNA_pseudogene="darkolivegreen"}:}{ Character or integer
scalar. Fill color for annotation objects of type 'rRNA_pseudogene'.}

\item{\code{retrotransposed="blueviolet"}:}{ Character or integer scalar.
Fill color for annotation objects of type 'retrotransposed'.}

\item{\code{scRNA="gold4"}:}{ Character or integer scalar. Fill color for
annotation objects of type 'scRNA'.}

\item{\code{scRNA_pseudogene="darkorange2"}:}{ Character or integer scalar.
Fill color for annotation objects of type 'scRNA_pseudogene'.}

\item{\code{snRNA="coral"}:}{ Character or integer scalar. Fill color for
annotation objects of type 'snRNA'.}

\item{\code{snRNA_pseudogene="coral3"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'snRNA_pseudogene'.}

\item{\code{snoRNA="cyan"}:}{ Character or integer scalar. Fill color for
annotation objects of type 'snoRNA'.}

\item{\code{snoRNA_pseudogene="cyan2"}:}{ Character or integer scalar. Fill
color for annotation objects of type 'snoRNA_pseudogene'.}

\item{\code{tRNA_pseudogene="antiquewhite3"}:}{ Character or integer scalar.
Fill color for annotation objects of type 'tRNA_pseudogene'.}

\item{\code{utr3="#FFD58A"}:}{ Character or integer scalar.  Fill color for
annotation objects of type 'utr3'.}

\item{\code{utr5="#FFD58A"}:}{ Character or integer scalar.  Fill color for
annotation objects of type 'utr5'.}

\item{\code{verbose=FALSE}:}{ Logical scalar. Report data loading events
from Bioamart or retrieval from cache.}

}

\bold{\emph{Inherited from class GeneRegionTrack:}}

\describe{

\item{\code{arrowHeadWidth=10}:}{ Numeric scalar. The width of the arrow
head in pixels if \code{shape} is \code{fixedArrow}.}

\item{\code{arrowHeadMaxWidth=20}:}{ Numeric scalar. The maximum width of
the arrow head in pixels if \code{shape} is \code{arrow}.}

\item{\code{col=NULL}:}{ Character or integer scalar. The border color for
all track items. Defaults to using the same color as in \code{fill}, also
taking into account different track \code{features}.}

\item{\code{collapseTranscripts=FALSE}:}{ Logical or character scalar. Can
be one in \code{gene}, \code{longest}, \code{shortest} or \code{meta}. Merge
all transcripts of the same gene into one single gene model. In the case of
\code{gene} (or \code{TRUE}), this will only keep the start location of the
first exon and the end location of the last exon from all transcripts of the
gene. For \code{shortest} and \code{longest}, only the longest or shortest
transcript model is retained.  For \code{meta}, a meta-transcript containing
the union of all exons is formed (essentially identical to the operation
\code{reduce(geneModel)}).}

\item{\code{exonAnnotation=NULL}:}{ Character scalar. Add annotation
information to the individual exon models. This can be a value in
\code{symbol}, \code{gene}, \code{transcript}, \code{exon} or
\code{feature}. Defaults to \code{exon}. Only works if \code{showExonId} is
not \code{FALSE}.}

\item{\code{fill="orange"}:}{ Character or integer scalar. The fill color
for untyped items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{min.distance=0}:}{ Numeric scalar. The minimum pixel distance
before collapsing range items, only if \code{collapse==TRUE}. See
\code{\link{collapsing}} for details. Note that a value larger than 0 may
lead to UTR regions being merged to CDS regions, which in most cases is not
particularly useful.}

\item{\code{shape=c("smallArrow", "box")}:}{ Character scalar.  The shape in
which to display the track items. Currently only \code{box}, \code{arrow},
\code{ellipse}, and \code{smallArrow} are implemented.}

\item{\code{showExonId=NULL}:}{ Logical scalar. Control whether to plot the
individual exon identifiers.}

\item{\code{thinBoxFeature=c("utr", "ncRNA", "utr3", "utr5", "3UTR",
"5UTR", "miRNA", "lincRNA", "three_prime_UTR", "five_prime_UTR")}:}{ Character
vector. A listing of feature types that should be drawn with thin boxes.
Typically those are non-coding elements.}

\item{\code{transcriptAnnotation=NULL}:}{ Character scalar.  Add annotation
information as transcript labels. This can be a value in \code{symbol},
\code{gene}, \code{transcript}, \code{exon} or \code{feature}. Defaults to
\code{symbol}.  Only works if \code{showId} is not \code{FALSE}.}

}

\bold{\emph{Inherited from class AnnotationTrack:}}

\describe{

\item{\code{cex.group=0.6}:}{ Numeric scalar. The font expansion factor for
the group-level annotation.}

\item{\code{cex=1}:}{ Numeric scalar. The font expansion factor for item
identifiers.}

\item{\code{col.line="darkgray"}:}{ Character scalar. The color used for
connecting lines between grouped items. Defaults to a light gray, but if set
to \code{NULL} the same color as for the first item in the group is used.}

\item{\code{featureAnnotation=NULL}:}{ Character scalar. Add annotation
information to the individual track elements. This can be a value in
\code{id}, \code{group} or \code{feature}.  Defaults to \code{id}. Only
works if \code{showFeatureId} is not \code{FALSE}.}

\item{\code{fontfamily.group="sans"}:}{ Character scalar. The font family
for the group-level annotation.}

\item{\code{fontcolor.group="#808080"}:}{ Character or integer scalar. The
font color for the group-level annotation.}

\item{\code{fontcolor.item="white"}:}{ Character or integer scalar. The font
color for item identifiers.}

\item{\code{fontface.group=2}:}{ Numeric scalar. The font face for the
group-level annotation.}

\item{\code{fontsize.group=12}:}{ Numeric scalar. The font size for the
group-level annotation.}

\item{\code{groupAnnotation=NULL}:}{ Character scalar. Add annotation
information as group labels. This can be a value in \code{id}, \code{group}
or \code{feature}. Defaults to \code{group}. Only works if \code{showId} is
not \code{FALSE}.}

\item{\code{just.group="left"}:}{ Character scalar. the justification of
group labels. Either \code{left}, \code{right}, \code{above} or
\code{below}.}

\item{\code{lex=1}:}{ Numeric scalar. The line expansion factor for all
track items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{lineheight=1}:}{ Numeric scalar. The font line height for item
identifiers.}

\item{\code{lty="solid"}:}{ Character or integer scalar. The line type for
all track items. This is also used to connect grouped items. See
\code{\link{grouping}} for details.}

\item{\code{lwd=1}:}{ Integer scalar. The line width for all track items.
This is also used to connect grouped items. See \code{\link{grouping}} for
details.}

\item{\code{mergeGroups=FALSE}:}{ Logical scalar. Merge fully overlapping
groups if \code{collapse==TRUE}.}

\item{\code{min.height=3}:}{ Numeric scalar. The minimum range height in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details. For feathered
bars indicating the strandedness of grouped items this also controls the
height of the arrow feathers.}

\item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{rotation=0}:}{ Numeric scalar. The degree of text rotation for
item identifiers.}

\item{\code{rotation.group=0}:}{ Numeric scalar. The degree of text rotation
for group labels.}

\item{\code{rotation.item=0}:}{ Numeric scalar. The degree of text rotation
for item identifiers.}

\item{\code{showFeatureId=FALSE}:}{ Logical scalar. Control whether to plot
the individual track item identifiers.}

\item{\code{showId=FALSE}:}{ Logical scalar. Control whether to annotate
individual groups.}

\item{\code{showOverplotting=FALSE}:}{ Logical scalar. Use a color gradient
to show the amount of overplotting for collapsed items. This implies that
\code{collapse==TRUE}}

\item{\code{size=1}:}{ Numeric scalar. The relative size of the track. Can
be overridden in the \code{\link{plotTracks}} function.}

}

\bold{\emph{Inherited from class StackedTrack:}}

\describe{

\item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
vertical size and spacing between stacked elements. The number defines the
proportion of the total available space for the stack that is used to draw
the glyphs.  E.g., a value of 0.5 means that half of the available vertical
drawing space (for each stacking line) is used for the glyphs, and thus one
quarter of the available space each is used for spacing above and below the
glyph. Defaults to 0.75.}

\item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
of stacked items. I.e., features that are plotted on the bottom-most stacks
will be moved to the top-most stack and vice versa.}

}

\bold{\emph{Inherited from class GdObject:}}

\describe{

\item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
items.}

\item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
title panel.}

\item{\code{background.panel="transparent"}:}{ Integer or character scalar.
The background color of the content panel.}

\item{\code{background.title="lightgray"}:}{ Integer or character scalar.
The background color for the title panel.}

\item{\code{background.legend="transparent"}:}{ Integer or character scalar.
The background color for the legend.}

\item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
axis annotation. Defaults to \code{NULL}, in which case it is automatically
determined based on the available space.}

\item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
title panel. This effects the fontsize of both the title and the axis, if
any. Defaults to \code{NULL}, which means that the text size is
automatically adjusted to the available space.}

\item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
line color for the y axis, if any.}

\item{\code{col.border.title="white"}:}{ Integer or character scalar. The
border color for the title panels.}

\item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
color used for the panel frame, if \code{frame==TRUE}}

\item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
line color for grid lines, both when \code{type=="g"} in
\code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}

\item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
for plot symbols. Usually the same as the global \code{col} parameter.}

\item{\code{col.title="white"}:}{ Integer or character scalar.  The border
color for the title panels}

\item{\code{collapse=TRUE}:}{ Boolean controlling whether to collapse the
content of the track to accomodate the minimum current device resolution.
See \code{\link{collapsing}} for details.}

\item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
color for all text, unless a more specific definition exists.}

\item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
for the title panels.}

\item{\code{fontface=1}:}{ Integer or character scalar. The font face for
all text, unless a more specific definition exists.}

\item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
font family for the title panels.}

\item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
family for all text, unless a more specific definition exists.}

\item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
unless a more specific definition exists.}

\item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
plotting.}

\item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
grid.}

\item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
horizontal grid lines, see \code{\link{panel.grid}} for details.}

\item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
and when display parameter \code{grid==TRUE}.}

\item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
title panels.}

\item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
panels}

\item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
display parameter \code{grid==TRUE}.}

\item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
mirror the plot on the vertical axis.}

\item{\code{rotation.title=90}:}{ The rotation angle for the text in the
title panel. Even though this can be adjusted, the automatic resizing of the
title panel will currently not work, so use at own risk.}

\item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
(only applies to track types where axes are implemented).}

\item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
panel. Although this can be set individually for each track, in multi-track
plots as created by \code{\link{plotTracks}} there will still be an empty
placeholder in case any of the other tracks include a title.  The same holds
true for axes. Note that the the title panel background color could be set
to transparent in order to completely hide it.}

\item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
vertical grid lines, see \code{\link{panel.grid}} for details.}

}

}

\item{AlignmentsTrack:}{

\describe{

\item{\code{alpha.reads=0.5}:}{ Numeric scalar between 0 and 1. The
transparency of the individual read icons. Can be used to indicate
overlapping regions in read pairs. Only on supported devices.}

\item{\code{alpha.mismatch=1}:}{ Numeric scalar between 0 and 1. The
transparency of the mismatch base information.}

\item{\code{cex=0.7}:}{ Numeric Scalar. The global character expansion
factor.}

\item{\code{cex.mismatch=NULL}:}{ Numeric Scalar. The character expansion
factor for the mismatch base letters.}

\item{\code{col.coverage=NULL}:}{ Integer or character scalar.  The line
color for the coverage profile.}

\item{\code{col.gap="#808080"}:}{ Integer or character scalar.  The color of
the line that is bridging the gap regions in gapped alignments.}

\item{\code{col.mates="#E0E0E0"}:}{ Integer or character scalar.  The color
of the line that is connecting two paired reads.}

\item{\code{col.deletion="#000000"}:}{ Integer or character scalar. The
color of the line that is bridging the deleted regions in alignments.}

\item{\code{col.insertion="#984EA3"}:}{ Integer or character scalar. The
color of the line that highlighting insertions in alignments.}

\item{\code{col.mismatch="#808080"}:}{ Integer or character scalar. The box
color around mismatch bases.}

\item{\code{col.reads=NULL}:}{ Integer or character scalar.  The box color
around reads.}

\item{\code{col.sashimi=NULL}:}{ Integer or character scalar.  The line
color for sashimi plots.}

\item{\code{col="#808080"}:}{ Integer or character scalar. The default color
of all line elements.}

\item{\code{collapse=FALSE}:}{ Logical scalar. Do not perform any collapsing
of overlapping elements. Currently not supported.}

\item{\code{coverageHeight=0.1}:}{ Numeric scalar. The height of the
coverage region of the track. Can either be a value between 0 and 1 in which
case it is taken as a relative height, or a positive value greater 1 in
which case it is interpreted as pixels.}

\item{\code{fill.coverage=NULL}:}{ Integer or character scalar.  The fill
color for the coverage profile.}

\item{\code{fill.reads=NULL}:}{ Integer or character scalar.  The fill color
for the read icons.}

\item{\code{fill="#BABABA"}:}{ Integer or character scalar.  The default
fill color of all plot elements.}

\item{\code{fontface.mismatch=2}:}{ Integer scalar. The font face for
mismatch bases.}

\item{\code{lty.coverage=NULL}:}{ Integer or character scalar.  The line
type of the coverage profile.}

\item{\code{lty.gap=NULL}:}{ Integer or character scalar. The type of the
line that is bridging the gap regions in gapped alignments.}

\item{\code{lty.mates=NULL}:}{ Integer or character scalar.  The type of the
line that is connecting two paired reads.}

\item{\code{lty.deletion=NULL}:}{ Integer or character scalar.  The type of
the line that is bridging the deleted regions in alignments.}

\item{\code{lty.insertion=NULL}:}{ Integer or character scalar.  The type of
the line that highlighting insertions in alignments.}

\item{\code{lty.mismatch=NULL}:}{ Integer or character scalar.  The box line
type around mismatch bases.}

\item{\code{lty.reads=NULL}:}{ Integer or character scalar.  The box line
type around mismatch reads.}

\item{\code{lty=1}:}{ Integer or character scalar. The default type of all
line elements.}

\item{\code{lwd.coverage=NULL}:}{ Integer or character scalar.  The line
width of the coverage profile.}

\item{\code{lwd.gap=NULL}:}{ Integer scalar. The width of the line that is
bridging the gap regions in gapped alignments.}

\item{\code{lwd.mates=NULL}:}{ Integer scalar. The width of the line that is
connecting two paired reads.}

\item{\code{lwd.deletion=NULL}:}{ Integer scalar. The width of the line that
is bridging the deleted regions in alignments.}

\item{\code{lwd.insertion=NULL}:}{ Integer scalar. The width of the line
that highlighting insertions in alignments.}

\item{\code{lwd.mismatch=NULL}:}{ Integer scalar. The box line width around
mismatch bases.}

\item{\code{lwd.reads=NULL}:}{ Integer scalar. The box line width around
reads.}

\item{\code{lwd.sashimiMax=10}:}{ Integer scalar. The maximal width of the
line in sashimi plots.}

\item{\code{lwd=1}:}{ Integer scalar. The default width of all line
elements.}

\item{\code{max.height=10}:}{ Integer scalar. The maximum height of an
individual read in pixels. Can be used in combination with \code{min.height}
to control the read and stacking appearance.}

\item{\code{min.height=5}:}{ Integer scalar. The minimum height of an
individual read in pixels. Can be used in combination with \code{max.height}
to control the read and stacking appearance.}

\item{\code{minCoverageHeight=50}:}{ Integer scalar. The minimum height of
the coverage section. Uselful in combination with a relative setting of
\code{coverageHeight}.}

\item{\code{minSashimiHeight=50}:}{ Integer scalar. The minimum height of
the sashimi section. Uselful in combination with a relative setting of
\code{sashimiHeight}.}

\item{\code{noLetters=FALSE}:}{ Logical scalar. Always plot colored boxes
for mismatch bases regardles of the available space.}

\item{\code{sashimiFilter=NULL}:}{ GRanges object. Only junctions which
overlap equally with \code{sashimiFilter} GRanges are shown. Default
\code{NULL}, no filtering.}

\item{\code{sashimiFilterTolerance=0}:}{ Integer scalar. Only used in
combination with \code{sashimiFilter}. It allows to include junctions whose
starts/ends are within specified distance from \code{sashimiFilter} GRanges.
This is useful for cases where the aligner did not place the junction reads
precisely. Default \code{0L} , no tolerance.}

\item{\code{sashimiHeight=0.1}:}{ Integer scalar. The height of the sashimi
part of the track. Can either be a value between 0 and 1 in which case it is
taken as a relative height, or a positive value greater 1 in which case it
is interpreted as pixels.}

\item{\code{sashimiScore=1}:}{ Integer scalar. The minimum number of reads
supporting the junction.}

\item{\code{sashimiStrand="*"}:}{ Integer scalar. Only reads which have the
specified strand are considered to count the junctions.}

\item{\code{sashimiTransformation=NULL}:}{ Function. Applied to the junction
score vector prior to plotting. The function should accept exactly one input
argument and its return value needs to be a numeric vector of identical
length as the input data.}

\item{\code{showIndels=FALSE}:}{ Logical scalar. Consider insertions and
deletions in coverage and pile-up. Default is \code{FALSE}. If set to
\code{TRUE} the deletions defined in CIGAR string are not considered in
coverage plot. The deletions are displayed as bridging lines in pile-up
track. Insertions are shown as vertical bars.}

\item{\code{showMismatches=TRUE}:}{ Logical scalar. Add mismatch
information, either as individual base letters or using color coded bars.
This implies that the reference sequence has been provided, either to the
class constructor or as part of the track list.}

\item{\code{size=NULL}:}{ Numeric scalar. The size of the track.  Defaults
to automatic sizing.}

\item{\code{transformation=NULL}:}{ Function. Applied to the coverage vector
prior to plotting. The function should accept exactly one input argument and
its return value needs to be a numeric Rle of identical length as the input
data.}

\item{\code{type=c("coverage", "pileup")}:}{ Character vactor.  The type of
information to plot. For \code{coverage} a coverage plot, potentially
augmented by base mismatch information, for \code{sashimi} a sashimi plot,
showing the juctions, and for \code{pileup} the pileups of the individual
reads. These three can be combined.}

}

\bold{\emph{Inherited from class StackedTrack:}}

\describe{

\item{\code{stackHeight=0.75}:}{ Numeric between 0 and 1.  Controls the
vertical size and spacing between stacked elements. The number defines the
proportion of the total available space for the stack that is used to draw
the glyphs.  E.g., a value of 0.5 means that half of the available vertical
drawing space (for each stacking line) is used for the glyphs, and thus one
quarter of the available space each is used for spacing above and below the
glyph. Defaults to 0.75.}

\item{\code{reverseStacking=FALSE}:}{ Logical flag. Reverse the y-ordering
of stacked items. I.e., features that are plotted on the bottom-most stacks
will be moved to the top-most stack and vice versa.}

}

\bold{\emph{Inherited from class GdObject:}}

\describe{

\item{\code{alpha=1}:}{ Numeric scalar. The transparency for all track
items.}

\item{\code{alpha.title=NULL}:}{ Numeric scalar. The transparency for the
title panel.}

\item{\code{background.panel="transparent"}:}{ Integer or character scalar.
The background color of the content panel.}

\item{\code{background.title="lightgray"}:}{ Integer or character scalar.
The background color for the title panel.}

\item{\code{background.legend="transparent"}:}{ Integer or character scalar.
The background color for the legend.}

\item{\code{cex.axis=NULL}:}{ Numeric scalar. The expansion factor for the
axis annotation. Defaults to \code{NULL}, in which case it is automatically
determined based on the available space.}

\item{\code{cex.title=NULL}:}{ Numeric scalar. The expansion factor for the
title panel. This effects the fontsize of both the title and the axis, if
any. Defaults to \code{NULL}, which means that the text size is
automatically adjusted to the available space.}

\item{\code{col.axis="white"}:}{ Integer or character scalar.  The font and
line color for the y axis, if any.}

\item{\code{col.border.title="white"}:}{ Integer or character scalar. The
border color for the title panels.}

\item{\code{col.frame="lightgray"}:}{ Integer or character scalar. The line
color used for the panel frame, if \code{frame==TRUE}}

\item{\code{col.grid="#808080"}:}{ Integer or character scalar.  Default
line color for grid lines, both when \code{type=="g"} in
\code{\linkS4class{DataTrack}}s and when display parameter \code{grid==TRUE}.}

\item{\code{col.line=NULL}:}{ Integer or character scalar.  Default colors
for plot lines. Usually the same as the global \code{col} parameter.}

\item{\code{col.symbol=NULL}:}{ Integer or character scalar.  Default colors
for plot symbols. Usually the same as the global \code{col} parameter.}

\item{\code{col.title="white"}:}{ Integer or character scalar.  The border
color for the title panels}

\item{\code{fontcolor="black"}:}{ Integer or character scalar.  The font
color for all text, unless a more specific definition exists.}

\item{\code{fontface.title=2}:}{ Integer or character scalar.  The font face
for the title panels.}

\item{\code{fontface=1}:}{ Integer or character scalar. The font face for
all text, unless a more specific definition exists.}

\item{\code{fontfamily.title="sans"}:}{ Integer or character scalar. The
font family for the title panels.}

\item{\code{fontfamily="sans"}:}{ Integer or character scalar.  The font
family for all text, unless a more specific definition exists.}

\item{\code{fontsize=12}:}{ Numeric scalar. The font size for all text,
unless a more specific definition exists.}

\item{\code{frame=FALSE}:}{ Boolean. Draw a frame around the track when
plotting.}

\item{\code{grid=FALSE}:}{ Boolean, switching on/off the plotting of a
grid.}

\item{\code{h=-1}:}{ Integer scalar. Parameter controlling the number of
horizontal grid lines, see \code{\link{panel.grid}} for details.}

\item{\code{lineheight=1}:}{ Numeric scalar. The font line height for all
text, unless a more specific definition exists.}

\item{\code{lty.grid="solid"}:}{ Integer or character scalar.  Default line
type for grid lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s
and when display parameter \code{grid==TRUE}.}

\item{\code{lwd.border.title=1}:}{ Integer scalar. The border width for the
title panels.}

\item{\code{lwd.title=1}:}{ Integer scalar. The border width for the title
panels}

\item{\code{lwd.grid=1}:}{ Numeric scalar. Default line width for grid
lines, both when \code{type=="g"} in \code{\linkS4class{DataTrack}}s and when
display parameter \code{grid==TRUE}.}

\item{\code{min.distance=1}:}{ Numeric scalar. The minimum pixel distance
before collapsing range items, only if \code{collapse==TRUE}. See
\code{\link{collapsing}} for details.}

\item{\code{min.width=1}:}{ Numeric scalar. The minimum range width in
pixels to display. All ranges are expanded to this size in order to avoid
rendering issues. See \code{\link{collapsing}} for details.}

\item{\code{reverseStrand=FALSE}:}{ Logical scalar. Set up the plotting
coordinates in 3' -> 5' direction if \code{TRUE}.  This will effectively
mirror the plot on the vertical axis.}

\item{\code{rotation.title=90}:}{ The rotation angle for the text in the
title panel. Even though this can be adjusted, the automatic resizing of the
title panel will currently not work, so use at own risk.}

\item{\code{rotation=0}:}{ The rotation angle for all text unless a more
specific definiton exists.}

\item{\code{showAxis=TRUE}:}{ Boolean controlling whether to plot a y axis
(only applies to track types where axes are implemented).}

\item{\code{showTitle=TRUE}:}{ Boolean controlling whether to plot a title
panel. Although this can be set individually for each track, in multi-track
plots as created by \code{\link{plotTracks}} there will still be an empty
placeholder in case any of the other tracks include a title.  The same holds
true for axes. Note that the the title panel background color could be set
to transparent in order to completely hide it.}

\item{\code{v=-1}:}{ Integer scalar. Parameter controlling the number of
vertical grid lines, see \code{\link{panel.grid}} for details.}

}

}

}
}

\examples{

## Which scheme is used?
getOption("Gviz.scheme")

## Change default settings for GeneRegionTrack
scheme <- getScheme()
scheme$GeneRegionTrack$fill <- "salmon"
scheme$GeneRegionTrack$col <- NULL
scheme$GeneRegionTrack$transcriptAnnotation <- "transcript"

## replace default scheme with myScheme
addScheme(scheme, "myScheme")
options(Gviz.scheme = "myScheme")
getOption("Gviz.scheme")

data(geneModels)
grtrack <- GeneRegionTrack(geneModels, genome = "hg19", chromosome = "chr7", name = "Gene Model")
plotTracks(grtrack)
}
\seealso{
\code{\linkS4class{AnnotationTrack}}

\code{\linkS4class{DataTrack}}

\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\link{availableDisplayPars}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{horizonplot}}

\code{\link{panel.bwplot}}

\code{\link{panel.grid}}

\code{\link{panel.loess}}

\code{\link{panel.xyplot}}

\code{\link{plotTracks}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GeneRegionTrack-class.R
\docType{class}
\name{GeneRegionTrack-class}
\alias{GeneRegionTrack-class}
\alias{initialize,GeneRegionTrack-method}
\alias{ReferenceGeneRegionTrack-class}
\alias{initialize,ReferenceGeneRegionTrack-method}
\alias{GeneRegionTrack}
\alias{gene,GeneRegionTrack-method}
\alias{gene<-,GeneRegionTrack,character-method}
\alias{symbol,GeneRegionTrack-method}
\alias{symbol<-,GeneRegionTrack,character-method}
\alias{transcript,GeneRegionTrack-method}
\alias{transcript<-,GeneRegionTrack,character-method}
\alias{exon,GeneRegionTrack-method}
\alias{exon<-,GeneRegionTrack,character-method}
\alias{group,GeneRegionTrack-method}
\alias{group<-,GeneRegionTrack,character-method}
\alias{identifier,GeneRegionTrack-method}
\alias{identifier<-,GeneRegionTrack,character-method}
\alias{subset,ReferenceGeneRegionTrack-method}
\alias{drawGD,GeneRegionTrack-method}
\alias{show,GeneRegionTrack-method}
\alias{show,ReferenceGeneRegionTrack-method}
\title{GeneRegionTrack class and methods}
\usage{
\S4method{initialize}{GeneRegionTrack}(.Object, start, end, ...)

\S4method{initialize}{ReferenceGeneRegionTrack}(
  .Object,
  stream,
  reference,
  mapping = list(),
  args = list(),
  defaults = list(),
  ...
)

GeneRegionTrack(
  range = NULL,
  rstarts = NULL,
  rends = NULL,
  rwidths = NULL,
  strand,
  feature,
  exon,
  transcript,
  gene,
  symbol,
  chromosome,
  genome,
  stacking = "squish",
  name = "GeneRegionTrack",
  start = NULL,
  end = NULL,
  importFunction,
  stream = FALSE,
  ...
)

\S4method{gene}{GeneRegionTrack}(GdObject)

\S4method{gene}{GeneRegionTrack,character}(GdObject) <- value

\S4method{symbol}{GeneRegionTrack}(GdObject)

\S4method{symbol}{GeneRegionTrack,character}(GdObject) <- value

\S4method{transcript}{GeneRegionTrack}(GdObject)

\S4method{transcript}{GeneRegionTrack,character}(GdObject) <- value

\S4method{exon}{GeneRegionTrack}(GdObject)

\S4method{exon}{GeneRegionTrack,character}(GdObject) <- value

\S4method{group}{GeneRegionTrack}(GdObject)

\S4method{group}{GeneRegionTrack,character}(GdObject) <- value

\S4method{identifier}{GeneRegionTrack}(
  GdObject,
  type = .dpOrDefault(GdObject, "transcriptAnnotation", "symbol")
)

\S4method{identifier}{GeneRegionTrack,character}(GdObject) <- value

\S4method{subset}{ReferenceGeneRegionTrack}(x, ...)

\S4method{drawGD}{GeneRegionTrack}(GdObject, ...)

\S4method{show}{GeneRegionTrack}(object)

\S4method{show}{ReferenceGeneRegionTrack}(object)
}
\arguments{
\item{.Object}{.Object}

\item{start, end}{An integer scalar with the genomic start or end coordinate
for the gene model range. If those are missing, the default value will
automatically be the smallest (or largest) value, respectively in
\code{rstarts} and \code{rends} for the currently active chromosome. When
building a \code{GeneRegionTrack} from a \code{TxDb} object, these arguments
can be used to subset the desired annotation data by genomic coordinates.
Please note this in that case the \code{chromosome} parameter must also be
set.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters. See \code{\link{settings}} and the "Display Parameters"
section below for details.}

\item{stream}{A logical flag indicating that the user-provided import
function can deal with indexed files and knows how to process the additional
\code{selection} argument when accessing the data on disk. This causes the
constructor to return a \code{ReferenceGeneRegionTrack} object which will
grab the necessary data on the fly during each plotting operation.}

\item{reference}{reference file}

\item{mapping}{mapping}

\item{args}{args}

\item{defaults}{\code{logical}}

\item{range}{An optional meta argument to handle the different input types. If the
\code{range} argument is missing, all the relevant information to create the
object has to be provided as individual function arguments (see below).

The different input options for \code{range} are:

\describe{

\item{A \code{TxDb} object:}{ all the necessary gene model information
including exon locations, transcript groupings and associated gene ids are
contained in \code{TxDb} objects, and the coercion between the two is almost
completely automated. If desired, the data to be fetched from the
\code{TxDb} object can be restricted using the constructor's
\code{chromosome}, \code{start} and \code{end} arguments. See below for
details. A direct coercion method \code{as(obj, "GeneRegionTrack")} is also
available. A nice added benefit of this input option is that the UTR and
coding region information that is part of the original \code{TxDb} object is
retained in the \code{GeneRegionTrack}.}

\item{A \code{GRanges} object:}{ the genomic ranges for the
\code{GeneRegion} track as well as the optional additional metadata columns
\code{feature}, \code{transcript}, \code{gene}, \code{exon} and
\code{symbol} (see description of the individual function parameters below
for details). Calling the constructor on a \code{GRanges} object without
further arguments, e.g.  \code{GeneRegionTrack(range=obj)} is equivalent to
calling the coerce method \code{as(obj, "GeneRegionTrack")}.}

\item{A \code{GRangesList} object:}{ this is very similar to the previous
case, except that the grouping information that is part of the list
structure is preserved in the \code{GeneRegionTrack}. I.e., all the elements
within one list item receive the same group id. For consistancy, there is
also a coercion method from \code{GRangesLists} \code{as(obj,
"GeneRegionTrack")}. Please note that unless the necessary information about
gene ids, symbols, etc. is present in the individual \code{GRanges} meta
data slots, the object will not be particularly useful, because all the
identifiers will be set to a common default value.}

\item{An \code{\linkS4class{IRanges}} object:}{ almost identical to the
\code{GRanges} case, except that the chromosome and strand information as
well as all additional data has to be provided in the separate
\code{chromosome}, \code{strand}, \code{feature}, \code{transcript},
\code{symbol}, \code{exon} or \code{gene} arguments, because it can not be
directly encoded in an \code{IRanges} object. Note that only the former two
are mandatory (if not provided explicitely the more or less reasonable
default values \code{chromosome=NA} and \code{strand=*} are used, but not
providing information about the gene-to-transcript relationship or the
human-readble symbols renders a lot of the class' functionality useles.}

\item{A \code{data.frame} object:}{ the \code{data.frame} needs to contain
at least the two mandatory columns \code{start} and \code{end} with the
range coordinates. It may also contain a \code{chromosome} and a
\code{strand} column with the chromosome and strand information for each
range. If missing, this information will be drawn from the constructor's
\code{chromosome} or \code{strand} arguments. In addition, the
\code{feature}, \code{exon}, \code{transcript}, \code{gene} and
\code{symbol} data can be provided as columns in the \code{data.frame}. The
above comments about potential default values also apply here.}

\item{A \code{character} scalar:}{ in this case the value of the
\code{range} argument is considered to be a file path to an annotation file
on disk. A range of file types are supported by the \code{Gviz} package as
identified by the file extension. See the \code{importFunction}
documentation below for further details.}

}}

\item{rstarts}{An integer vector of the start coordinates for the actual
gene model items, i.e., for the individual exons. The relationship between
exons is handled via the \code{gene} and \code{transcript} factors.
Alternatively, this can be a vector of comma-separated lists of integer
coordinates, one vector item for each transcript, and each comma-separated
element being the start location of a single exon within that transcript.
Those lists will be exploded upon object instantiation and all other
annotation arguments will be recycled accordingly to regenerate the
exon/transcript/gene relationship structure. This implies the approriate
number of items in all annotation and coordinates arguments.}

\item{rends}{An integer vector of the end coordinates for the actual gene
model items. Both \code{rstarts} and \code{rends} have to be of equal
length.}

\item{rwidths}{An integer vector of widths for the actual gene model items.
This can be used instead of either \code{rstarts} or \code{rends} to specify
the range coordinates.}

\item{strand}{Character vector, the strand information for the individual
track exons. It may be provided in the form \code{+} for the Watson strand,
\code{-} for the Crick strand or \code{*} for either one of the two. Please
note that all items within a single gene or transcript model need to be on
the same strand, and erroneous entries will result in casting of an error.}

\item{feature}{Factor (or other vector that can be coerced into one), giving
the feature types for the individual track exons.  When plotting the track
to the device, if a display parameter with the same name as the value of
\code{feature} is set, this will be used as the track item's fill color.
Additionally, the feature type defines whether an element in the
\code{GeneRegionTrack} is considered to be coding or non-coding. The details
section as well as the section about the \code{thinBoxFeature} display
parameter further below has more information on this. See also
\code{\link{grouping}} for details.}

\item{exon}{Character vector of exon identifiers. It's values will be used
as the identifier tag when plotting to the device if the display parameter
\code{showExonId=TRUE}.}

\item{transcript}{Factor (or other vector that can be coerced into one),
giving the transcript memberships for the individual track exons. All items
with the same transcript identifier will be visually connected when plotting
to the device.  See \code{\link{grouping}} for details. Will be used as
labels when \code{showId=TRUE}, and \code{geneSymbol=FALSE}.}

\item{gene}{Factor (or other vector that can be coerced into one), giving
the gene memberships for the individual track exons.}

\item{symbol}{A factor with human-readable gene name aliases which will be
used as labels when \code{showId=TRUE}, and \code{geneSymbol=TRUE}.}

\item{chromosome}{The chromosome on which the track's genomic ranges are
defined. A valid UCSC chromosome identifier if
\code{options(ucscChromosomeNames=TRUE)}. Please note that in this case only
syntactic checking takes place, i.e., the argument value needs to be an
integer, numeric character or a character of the form \code{chrx}, where
\code{x} may be any possible string. The user has to make sure that the
respective chromosome is indeed defined for the the track's genome. If not
provided here, the constructor will try to build the chromosome information
based on the available inputs, and as a last resort will fall back to the
value \code{chrNA}. Please note that by definition all objects in the
\code{Gviz} package can only have a single active chromosome at a time
(although internally the information for more than one chromosome may be
present), and the user has to call the \code{chromosome<-} replacement
method in order to change to a different active chromosome. When creating a
\code{GeneRegionTrack} from a \code{TxDb} object, the value of this
parameter can be used to subset the data to fetch only transcripts from a
single chromosome.}

\item{genome}{The genome on which the track's ranges are defined. Usually
this is a valid UCSC genome identifier, however this is not being formally
checked at this point. If not provided here the constructor will try to
extract this information from the provided inputs, and eventually will fall
back to the default value of \code{NA}.}

\item{stacking}{The stacking type for overlapping items of the track. One in
\code{c(hide, dense, squish, pack,full)}. Currently, only hide (don't show
the track items, squish (make best use of the available space) and dense (no
stacking at all) are implemented.}

\item{name}{Character scalar of the track's name used in the title panel
when plotting.}

\item{importFunction}{A user-defined function to be used to import the data
from a file. This only applies when the \code{range} argument is a character
string with the path to the input data file. The function needs to accept an
argument \code{x} containing the file path and has to return a proper
\code{GRanges} object with all the necessary metadata columns set. A set of
default import functions is already implemented in the package for a number
of different file types, and one of these defaults will be picked
automatically based on the extension of the input file name. If the
extension can not be mapped to any of the existing import function, an error
is raised asking for a user-defined import function via this argument.
Currently the following file types can be imported with the default
functions: \code{gff}, \code{gff1}, \code{gff2}, \code{gff3}, \code{gtf}.}

\item{GdObject}{Object of \code{GdObject-class}.}

\item{value}{Value to be set.}

\item{type}{type}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}

\item{object}{object}
}
\value{
The return value of the constructor function is a new object of class
\code{GeneRegionTrack}.
}
\description{
A class to hold gene model data for a genomic region.
}
\details{
A track containing all gene models in a particular region. The data are
usually fetched dynamially from an online data store, but it is also
possible to manully construct objects from local data. Connections to
particular online data sources should be implemented as sub-classes, and
\code{GeneRegionTrack} is just the commone denominator that is being used
for plotting later on. There are several levels of data associated to a
\code{GeneRegionTrack}:

\describe{

\item{exon level:}{identifiers are stored in the exon column of the
\code{\linkS4class{GRanges}} object in the \code{range} slot. Data may be
extracted using the \code{exon} method.}

\item{transcript level:}{identifiers are stored in the transcript column of
the \code{\linkS4class{GRanges}} object. Data may be extracted using the
\code{transcript} method.}

\item{gene level:}{identifiers are stored in the gene column of the
\code{\linkS4class{GRanges}} object, more human-readable versions in the
symbol column. Data may be extracted using the \code{gene} or the
\code{symbol} methods.}

\item{transcript-type level:}{information is stored in the feature column of
the \code{\linkS4class{GRanges}} object. If a display parameter of the same
name is specified, the software will use its value for the coloring.}

}

\code{GeneRegionTrack} objects also know about coding regions and non-coding
regions (e.g., UTRs) in a transcript, and will indicate those by using
different shapes (wide boxes for all coding regions, thinner boxes for
non-coding regions). This is archived by setting the \code{feature} values
of the object for non-coding elements to one of the options that are
provided in the \code{thinBoxFeature} display parameters. All other elements
are considered to be coding elements.
}
\section{Functions}{
\itemize{
\item \code{initialize(GeneRegionTrack)}: Initialize.

\item \code{ReferenceGeneRegionTrack-class}: The file-based version of the \code{GeneRegionTrack-class}.

\item \code{initialize(ReferenceGeneRegionTrack)}: Initialize.

\item \code{GeneRegionTrack()}: Constructor function for \code{GeneRegionTrack-class}.

\item \code{gene(GeneRegionTrack)}: Extract the gene identifiers for all
gene models.

\item \code{gene(GdObject = GeneRegionTrack) <- value}: Replace the gene identifiers for all
gene models.
The replacement value must be a character of appropriate length or another
vector that can be coerced into such.

\item \code{symbol(GeneRegionTrack)}: Extract the human-readble gene symbol
for all gene models.

\item \code{symbol(GdObject = GeneRegionTrack) <- value}: Replace the human-readable gene symbol
for all gene models.
The replacement value must be a character of appropriate length or another
vector that can be coerced into such.

\item \code{transcript(GeneRegionTrack)}: Extract the transcript identifiers for all
transcripts in the gene models.

\item \code{transcript(GdObject = GeneRegionTrack) <- value}: Replace the transcript identifiers for all
transcripts in the gene model. The replacement value must be a character of
appropriate length or another vector that can be coerced into such.

\item \code{exon(GeneRegionTrack)}: Extract the exon identifiers for all exons
in the gene models.

\item \code{exon(GdObject = GeneRegionTrack) <- value}: replace the exon identifiers for all exons
in the gene model. The replacement value must be a character of appropriate
length or another vector that can be coerced into such.

\item \code{group(GeneRegionTrack)}: extract the group membership for all track items.

\item \code{group(GdObject = GeneRegionTrack) <- value}: replace the grouping information for track items.
The replacement value must be a factor of appropriate length or another
vector that can be coerced into such.

\item \code{identifier(GeneRegionTrack)}: return track item identifiers.
Depending on the setting of the optional argument lowest, these are either
the group identifiers or the individual item identifiers.
export

\item \code{identifier(GdObject = GeneRegionTrack) <- value}: Set the track item identifiers.
The replacement value has to be a character vector of appropriate length.
This always replaces the group-level identifiers, so essentially it is
similar to \verb{groups<-}.

\item \code{subset(ReferenceGeneRegionTrack)}: Subset a GeneRegionTrack by coordinates
and sort if necessary.

\item \code{drawGD(GeneRegionTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(GeneRegionTrack)}: Show method.

\item \code{show(ReferenceGeneRegionTrack)}: Show method.

}}
\section{Objects from the class}{


Objects can be created using the constructor function
\code{GeneRegionTrack}.
}

\examples{


## The empty object
GeneRegionTrack()

## Load some sample data
data(cyp2b10)

## Construct the object
grTrack <- GeneRegionTrack(
    start = 26682683, end = 26711643,
    rstart = cyp2b10$start, rends = cyp2b10$end, chromosome = 7, genome = "mm9",
    transcript = cyp2b10$transcript, gene = cyp2b10$gene, symbol = cyp2b10$symbol,
    feature = cyp2b10$feature, exon = cyp2b10$exon,
    name = "Cyp2b10", strand = cyp2b10$strand
)

## Directly from the data.frame
grTrack <- GeneRegionTrack(cyp2b10)

## From a TxDb object
if (require(GenomicFeatures)) {
    samplefile <- system.file("extdata",
                              "hg19_knownGene_sample.sqlite",
                              package = "GenomicFeatures")
    txdb <- loadDb(samplefile)
    GeneRegionTrack(txdb)
    GeneRegionTrack(txdb, chromosome = "chr6", start = 35000000, end = 40000000)
}
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(grTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(grTrack)

## Track names
names(grTrack)
names(grTrack) <- "foo"
plotTracks(grTrack)

## Subsetting and splitting
subTrack <- subset(grTrack, from = 26700000, to = 26705000)
length(subTrack)
subTrack <- grTrack[transcript(grTrack) == "ENSMUST00000144140"]
split(grTrack, transcript(grTrack))

## Accessors
start(grTrack)
end(grTrack)
width(grTrack)
position(grTrack)
width(subTrack) <- width(subTrack) + 100

strand(grTrack)
strand(subTrack) <- "-"

chromosome(grTrack)
chromosome(subTrack) <- "chrX"

genome(grTrack)
genome(subTrack) <- "hg19"

range(grTrack)
ranges(grTrack)

## Annotation
identifier(grTrack)
identifier(grTrack, "lowest")
identifier(subTrack) <- "bar"

feature(grTrack)
feature(subTrack) <- "foo"

exon(grTrack)
exon(subTrack) <- letters[1:2]

gene(grTrack)
gene(subTrack) <- "bar"

symbol(grTrack)
symbol(subTrack) <- "foo"

transcript(grTrack)
transcript(subTrack) <- c("foo", "bar")
chromosome(subTrack) <- "chr7"
plotTracks(subTrack)

values(grTrack)

## Grouping
group(grTrack)
group(subTrack) <- "Group 1"
transcript(subTrack)
plotTracks(subTrack)

## Collapsing transcripts
plotTracks(grTrack,
    collapseTranscripts = TRUE, showId = TRUE,
    extend.left = 10000, shape = "arrow"
)

## Stacking
stacking(grTrack)
stacking(grTrack) <- "dense"
plotTracks(grTrack)

## coercion
as(grTrack, "data.frame")
as(grTrack, "UCSCData")

## HTML image map
coords(grTrack)
tags(grTrack)
grTrack <- plotTracks(grTrack)$foo
coords(grTrack)
tags(grTrack)
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne, Steve Lianoglou
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R, R/ReferenceTrack-class.R
\docType{class}
\name{availableDefaultMapping}
\alias{availableDefaultMapping}
\alias{ReferenceTrack-class}
\alias{initialize,ReferenceTrack-method}
\title{ReferenceTrack class and methods}
\usage{
availableDefaultMapping(file, trackType)

\S4method{initialize}{ReferenceTrack}(
  .Object,
  stream,
  reference,
  mapping = list(),
  args = list(),
  defaults = list()
)
}
\arguments{
\item{file}{A \code{character} scalar with a file name or just a file extension.}

\item{trackType}{A \code{character} scalar with one of the available track types
in the package.}

\item{.Object}{.Object}

\item{stream}{stream}

\item{reference}{reference}

\item{mapping}{mapping}

\item{args}{ars}

\item{defaults}{defaults}
}
\value{
Constructor functions of \code{AnnotationTrack}, \code{DataTrack}, \code{SequenceTrack}
and \verb{AlignmentsTrack`` can create a special object of corresponding }Reference*Track` subclass with pointer to the referenced
file.

A virtual class: No objects may be created from it.
}
\description{
A class allow for on-demand streaming of data off the file system.
}
\details{
The \code{availableDefaultMappings} function can be used to find out whether
the package defines a mapping scheme between one of the many supported input
file types and the metadata columns of the tracks' \code{GRanges} objects.
}
\section{Functions}{
\itemize{
\item \code{availableDefaultMapping()}: Function to find out whether the package
defines a mapping scheme between one of the many supported input file types
and the metadata columns of the tracks's \code{GRanges} objects.

\item \code{initialize(ReferenceTrack)}: Initialize.

}}
\section{Slots}{

\describe{
\item{\code{stream}}{Object of class function. The import function to stream data
of the file system. Needs to be able to handle the two mandatory arguments
\code{file} (a \code{character} containing a valid file path) and \code{selection}
(a \code{GRanges} object with the genomic region to plot).}

\item{\code{reference}}{Object of class "character", the path to the file
containing the data.}

\item{\code{mapping}}{Object of class \code{list}, a default mapping between the
metadata columns of the returned \code{GRanges} object from the import function
and the \code{elemenMetadata} columns that make up the final track object.}

\item{\code{args}}{Object of class \code{list}, the passed in constructor arguments
during object instantiation. Those will be needed when fetching the data
in order to fill all necessary slots.}

\item{\code{defaults}}{Object of class \code{list}, the relevant default values to be
used when neither mapping nor args provides the necessary information.}
}}

\examples{
# This is a reference class, below example from AlignmentsTrack

afrom <- 2960000
ato <- 3160000
alTrack <- AlignmentsTrack(system.file(
    package = "Gviz", "extdata",
    "gapped.bam"
), isPaired = TRUE)
plotTracks(alTrack, from = afrom, to = ato, chromosome = "chr12")
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OverlayTrack-class.R
\docType{class}
\name{OverlayTrack-class}
\alias{OverlayTrack-class}
\alias{initialize,OverlayTrack-method}
\alias{OverlayTrack}
\alias{displayPars<-,OverlayTrack,list-method}
\alias{length,OverlayTrack-method}
\alias{chromosome,OverlayTrack-method}
\alias{chromosome<-,OverlayTrack-method}
\alias{setStacks,OverlayTrack-method}
\alias{consolidateTrack,OverlayTrack-method}
\alias{subset,OverlayTrack-method}
\alias{drawGD,OverlayTrack-method}
\alias{show,OverlayTrack-method}
\title{OverlayTrack class and methods}
\usage{
\S4method{initialize}{OverlayTrack}(.Object, trackList, ...)

OverlayTrack(trackList = list(), name = "OverlayTrack", ...)

\S4method{displayPars}{OverlayTrack,list}(x, recursive = FALSE) <- value

\S4method{length}{OverlayTrack}(x)

\S4method{chromosome}{OverlayTrack}(GdObject)

\S4method{chromosome}{OverlayTrack}(GdObject) <- value

\S4method{setStacks}{OverlayTrack}(GdObject, ...)

\S4method{consolidateTrack}{OverlayTrack}(GdObject, chromosome, ...)

\S4method{subset}{OverlayTrack}(x, ...)

\S4method{drawGD}{OverlayTrack}(GdObject, ...)

\S4method{show}{OverlayTrack}(object)
}
\arguments{
\item{trackList}{A list of Gviz track objects that all have to inherit from
class \code{GdObject}.}

\item{\dots}{All additional parameters are ignored.}

\item{name}{Character scalar of the track's name. This is not really used
and only exists fro completeness.}
}
\value{
The return value of the constructor function is a new object of class
\code{OverlayTrack}.
}
\description{
A container for other track objects from the Gviz package that allows for
overlays of their content on the same region of the plot.
}
\details{
A track to conceptionally group other Gviz track objects into a meta track
in order to merge them into a single overlay visualization. Only the first
track in the supplied list will be inferred when setting up the track title
and axis, for all the other tracks only the panel content is plotted.
}
\section{Functions}{
\itemize{
\item \code{initialize(OverlayTrack)}: Initialize.

\item \code{OverlayTrack()}: Constructor function for \code{OverlayTrack-class}.

\item \code{displayPars(x = OverlayTrack) <- value}: set display parameters using the values of
the named list in value. See \code{\link{settings}} for details on
display parameters and customization.

\item \code{length(OverlayTrack)}: return the number of subtracks.

\item \code{chromosome(OverlayTrack)}: return the chromosome for which the track
is defined.

\item \code{chromosome(OverlayTrack) <- value}: replace the value of the track's chromosome.
This has to be a valid UCSC chromosome identifier or an integer or character
scalar that can be reasonably coerced into one.

\item \code{setStacks(OverlayTrack)}: recompute the stacks based on the available
space and on the object's track items and stacking settings.
This really just calls the setStacks methods for the contained
tracks and only exists for dispatching reasons.

\item \code{consolidateTrack(OverlayTrack)}: #' For a \code{OverlayTrack} apply the method on
each of the subtracks in the \code{trackList} slot

\item \code{subset(OverlayTrack)}: plot subset all the contained tracks in an
\code{OverlayTrack} by coordinates and sort if necessary.

\item \code{drawGD(OverlayTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(OverlayTrack)}: Show method.

}}
\section{Objects from the Class}{


Objects can be created using the constructor function \code{OverlayTrack}.
}

\examples{


## Object construction:
set.seed(123)
dat <- runif(100, min = -2, max = 22)
dt1 <- DataTrack(data = dat, start = sort(sample(200, 100)), width = 1, genome = "hg19")
dt2 <- DataTrack(data = dat, start = sort(sample(200, 100)), width = 1, genome = "hg19")
ot <- OverlayTrack(trackList = list(dt1, dt2))
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapsing.R
\name{collapsing}
\alias{collapsing}
\title{Dynamic content based on the available resolution}
\description{
When plotting features linearly along genomic coordinates one frequently
runs into the problem of too little resolution to adequately display all
details. Most genome browsers try to reasonably reduce the amount of detail
that is shown based on the current zoom level.
}
\details{
Most track classes in this package define an internal \code{collapseTrack}
method which tries to adjust the plotted content to the available
resolution, aims at reducing over-plotting and prevents rendering issues,
e.g. when lines are too thin to be plotted. This feature can be toggled on
or off using the \code{collapse} display parameter (see
\code{settings} for details on setting these parameters).

In the simplest case (for \code{AnnotationTrack} objects) this
involves expanding all shown features to a minimum pixel width and height
(using display parameters \code{min.width} and \code{min.heigh}) and
collapsing overlapping annotation items (as defined by the parameter
\code{min.distance} into one single item to prevent over-plotting.

For objects of class \code{DataTrack}, the data values
underlying collapsed regions will be summarized based on the \code{summary}
display parameter. See the class' documentation for more details.
}
\seealso{
\code{\linkS4class{AnnotationTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{settings}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NumericTrack-class.R
\docType{class}
\name{NumericTrack-class}
\alias{NumericTrack-class}
\alias{drawAxis,NumericTrack-method}
\alias{drawGrid,NumericTrack-method}
\title{NumericTrack class and methods}
\usage{
\S4method{drawAxis}{NumericTrack}(GdObject, from, to, ...)

\S4method{drawGrid}{NumericTrack}(GdObject, from, to)
}
\arguments{
\item{GdObject}{Object of \code{GdObject-class}.}

\item{from, to}{Numeric scalar, giving the range of genomic coordinates to
limit the tracks in. Note that \code{to} cannot be larger than \code{from.}}

\item{\dots}{Additional arguments.}
}
\value{
A virtual class: No objects may be created from it.
}
\description{
The virtual parent class for all track items in the Gviz package designed to
contain numeric data. This class merely exists for dispatching purpose.
}
\section{Functions}{
\itemize{
\item \code{drawAxis(NumericTrack)}: add a y-axis to the title panel of a track.

\item \code{drawGrid(NumericTrack)}: superpose a grid on top of a track.

}}
\section{Slots}{

\describe{
\item{\code{dp}}{Object of \code{DisplayPars-class}, the display settings controlling the
look and feel of a track. See settings for details on setting graphical
parameters for tracks.}

\item{\code{name}}{Object of class \code{character}, a human-readable name for the track
that will be used in the track's annotation panel if necessary.}

\item{\code{imageMap}}{Object of \code{ImageMap-class}, containing optional information
for an HTML image map. This will be created by the \code{drawGD} methods when the
track is plotted to a device and is usually not set by the user.}

\item{\code{range}}{Object of class \code{GRanges}, the genomic ranges of the track items
as well as additional annotation information in its \code{elementMetaData} slot.
Please not that the slot is actually implemented as a class union between
\code{GRanges} and IRanges to increase efficiency, for instance for DataTrack
objects. This usually does not concern the user.}

\item{\code{chromosome}}{Object of class \code{character}, the chromosome on which the
track is defined. There can only be a single chromosome for one track. For
certain subclasses, the space of allowed chromosome names is limited (e.g.,
only those chromosomes that exist for a particular genome). Throughout the
package, chromosome name have to be entered either as a single integer scalar
or as a character scalar of the form chrXYZ, where XYZ may be an arbitrary
character string.}

\item{\code{genome}}{Object of class \code{character}, the genome for which the track is
defined. For most sub-classes this has to be valid UCSC genome identifier,
however this may not always be formally checked upon object instantiation.}
}}

\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IdeogramTrack-class.R
\docType{class}
\name{IdeogramTrack-class}
\alias{IdeogramTrack-class}
\alias{IdeogramTrack}
\alias{drawGD,IdeogramTrack-method}
\alias{end,IdeogramTrack-method}
\alias{end<-,IdeogramTrack-method}
\alias{initialize,IdeogramTrack-method}
\alias{show,IdeogramTrack-method}
\alias{start,IdeogramTrack-method}
\alias{start<-,IdeogramTrack-method}
\alias{subset,IdeogramTrack-method}
\alias{width,IdeogramTrack-method}
\alias{width<-,IdeogramTrack-method}
\alias{length,IdeogramTrack-method}
\alias{[,IdeogramTrack-method}
\alias{[,IdeogramTrack,ANY,ANY-method}
\alias{[,IdeogramTrack,ANY,ANY,ANY-method}
\alias{chromosome<-,IdeogramTrack-method}
\alias{genome<-,IdeogramTrack-method}
\alias{position,IdeogramTrack-method}
\title{IdeogramTrack class and methods}
\arguments{
\item{chromosome}{The chromosome for which to create the ideogram. Has to be
a valid UCSC chromosome identifier of the form \code{chrx}, or a single
integer or numeric character unless
\code{option(ucscChromosomeNames=FALSE)}. The user has to make sure that the
respective chromosome is indeed defined for the the track's genome.}

\item{genome}{The genome on which to create the ideogram. This has to be a
valid UCSC genome identifier if the ideogram data is to be fetched from the
UCSC repository.}

\item{name}{Character scalar of the track's name used in the title panel
when plotting. Defaults to the selected chromosome.}

\item{bands}{A \code{data.frame} with the cytoband information for all
available chromosomes on the genome similar to the data that would be
fetched from UCSC. The table needs to contain the mandatory columns
\code{chrom}, \code{chromStart}, \code{chromEnd}, \code{name} and
\code{gieStain} with the chromosome name, cytoband start and end
coordinates, cytoband name and coloring information, respectively. This can
be used when no connection to the internet is available or when the cytoband
information has been cached locally to avoid the somewhat slow connection to
UCSC.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters.}
}
\value{
The return value of the constructor function is a new object of class
\code{IdeogramTrack}.
}
\description{
A class to represent the schematic display of a chromosome, also known as an
ideogram. The respective information is typically directly fetched from
UCSC.
}
\details{
Ideograms are schematic depictions of chromosomes, including chromosome band
information and centromere location. The relevant data for various species
is stored in the UCSC data base. The initializer method of the class will
automatically fetch the respective data for a given genome and chromosome
from UCSC and fill the appropriate object slots. When plotting
\code{IdeogramTracks}, the current genomic location is indicated on the
chromosome by a colored box.

The \code{Gviz.ucscUrl} option controls which URL is being used to connect
to UCSC. For instance, one could switch to the European UCSC mirror by
calling \code{options(Gviz.ucscUrl="http://genome-euro.ucsc.edu/cgi-bin/"}.
}
\note{
When fetching ideogram data from UCSC the results are cached for faster
acces. See \code{\link{clearSessionCache}} on details to delete these cached
items.
}
\section{Objects from the Class}{


Objects can be created using the constructor function \code{IdeogramTrack}.
}

\examples{
\dontshow{
## Load some sample data
data(idTrack)
}

## Construct the object
\dontrun{
idTrack <- IdeogramTrack(chromosome = 7, genome = "mm9")
}

\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(idTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(idTrack, from = 5000000, to = 9000000)

## Track names
names(idTrack)
names(idTrack) <- "foo"
plotTracks(idTrack, from = 5000000, to = 9000000)


## Accessors
chromosome(idTrack)
\dontrun{
chromosome(idTrack) <- "chrX"
}

genome(idTrack)
\dontrun{
genome(id) <- "hg19"
}

range(idTrack)
ranges(idTrack)

## Annotation
values(idTrack)

## coercion
as(idTrack, "data.frame")
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AlignmentsTrack-class.R
\docType{class}
\name{AlignmentsTrack-class}
\alias{AlignmentsTrack-class}
\alias{initialize,AlignmentsTrack-method}
\alias{ReferenceAlignmentsTrack-class}
\alias{initialize,ReferenceAlignmentsTrack-method}
\alias{AlignmentsTrack}
\alias{values,AlignmentsTrack-method}
\alias{chromosome<-,AlignmentsTrack-method}
\alias{stacks,AlignmentsTrack-method}
\alias{setStacks,AlignmentsTrack-method}
\alias{subset,AlignmentsTrack-method}
\alias{subset,ReferenceAlignmentsTrack-method}
\alias{drawAxis,AlignmentsTrack-method}
\alias{drawGD,AlignmentsTrack-method}
\alias{show,AlignmentsTrack-method}
\alias{show,ReferenceAlignmentsTrack-method}
\title{AlignmentsTrack class and methods}
\usage{
\S4method{initialize}{AlignmentsTrack}(
  .Object,
  stackRanges = GRanges(),
  stacks = numeric(),
  sequences = DNAStringSet(),
  referenceSequence = NULL,
  ...
)

\S4method{initialize}{ReferenceAlignmentsTrack}(
  .Object,
  stream,
  reference,
  mapping = list(),
  args = list(),
  defaults = list(),
  stacks = numeric(),
  stackRanges = GRanges(),
  sequences = Biostrings::DNAStringSet(),
  referenceSequence = NULL,
  ...
)

AlignmentsTrack(
  range = NULL,
  start = NULL,
  end = NULL,
  width = NULL,
  strand,
  chromosome,
  genome,
  stacking = "squish",
  id,
  cigar,
  mapq,
  flag = scanBamFlag(isUnmappedQuery = FALSE),
  isize,
  groupid,
  status,
  md,
  seqs,
  name = "AlignmentsTrack",
  isPaired = TRUE,
  importFunction,
  referenceSequence,
  ...
)

\S4method{values}{AlignmentsTrack}(x)

\S4method{chromosome}{AlignmentsTrack}(GdObject) <- value

\S4method{stacks}{AlignmentsTrack}(GdObject)

\S4method{setStacks}{AlignmentsTrack}(GdObject, ...)

\S4method{subset}{AlignmentsTrack}(x, from = NULL, to = NULL, stacks = FALSE, use.defaults = TRUE, ...)

\S4method{subset}{ReferenceAlignmentsTrack}(x, from, to, chromosome, ...)

\S4method{drawAxis}{AlignmentsTrack}(GdObject, ...)

\S4method{drawGD}{AlignmentsTrack}(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...)

\S4method{show}{AlignmentsTrack}(object)

\S4method{show}{ReferenceAlignmentsTrack}(object)
}
\arguments{
\item{.Object}{.Object}

\item{stackRanges}{stackRanges}

\item{stacks}{stacks}

\item{sequences}{sequences}

\item{referenceSequence}{An optional \code{\linkS4class{SequenceTrack}}
object containing the reference sequence against which the reads have been
aligned. This is only needed when mismatch information has to be added to
the plot (i.e., the \code{showMismatchs} display parameter is \code{TRUE})
because this is normally not encoded in the \code{BAM} file. If not provided
through this argument, the \code{\link{plotTracks}} function is smart enough
to detect the presence of a \code{\linkS4class{SequenceTrack}} object in the
track list and will use that as a reference sequence.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters. See \code{\link{settings}} and the "Display Parameters"
section below for details.}

\item{stream}{stream}

\item{reference}{reference}

\item{mapping}{mapping}

\item{args}{args}

\item{defaults}{defaults}

\item{range}{An optional meta argument to handle the different input types.
If the \code{range} argument is missing, all the relevant information to
create the object has to be provided as individual function arguments
(see below).

The different input options for \code{range} are:

\describe{

\item{A \code{character} string:}{ the path to a \code{BAM} file containing
the read alignments. To be precise, this will result in the instantiation of
a \code{ReferenceAlignmentsTrack} object, but for the user this
implementation detail should be of no concern.}

\item{A \code{GRanges} object:}{ the genomic ranges of the individual reads
as well as the optional additional metadata columns \code{id}, \code{cigar},
\code{mapq}, \code{flag}, \code{isize}, \code{groupid}, \code{status},
\code{md} and \code{seqs} (see description of the individual function
parameters below for details). Calling the constructor on a \code{GRanges}
object without further arguments, e.g. \code{AlignmentsTrack(range=obj)} is
equivalent to calling the coerce method \code{as(obj, "AlignmentsTrack")}.}

\item{An \code{\linkS4class{IRanges}} object:}{ almost identical to the
\code{GRanges} case, except that the chromosome and strand information as
well as all additional metadata has to be provided in the separate
\code{chromosome}, \code{strand}, \code{feature}, \code{group} or \code{id}
arguments, because it can not be directly encoded in an \code{IRanges}
object. Note that none of those inputs are mandatory, and if not provided
explicitely the more or less reasonable default values \code{chromosome=NA}
and \code{strand="*"} are used. }

\item{A \code{data.frame} object:}{ the \code{data.frame} needs to contain
at least the two mandatory columns \code{start} and \code{end} with the
range coordinates. It may also contain a \code{chromosome} and a
\code{strand} column with the chromosome and strand information for each
range. If missing it will be drawn from the separate \code{chromosome} or
\code{strand} arguments. In addition, the \code{id}, \code{cigar},
\code{mapq}, \code{flag}, \code{isize}, \code{groupid}, \code{status},
\code{md} and \code{seqs} data can be provided as additional columns. The
above comments about potential default values also apply here.}

}}

\item{start, end, width}{Integer vectors, giving the start and the end
coordinates for the individual track items, or their width. Two of the three
need to be specified, and have to be of equal length or of length one, in
which case this single value will be recycled. Otherwise, the usual R
recycling rules for vectors do not apply here.}

\item{strand}{Character vector, the strand information for the reads. It may
be provided in the form \code{+} for the Watson strand, \code{-} for the
Crick strand or \code{*} for either one of the two. Needs to be of equal
length as the provided genomic coordinates, or of length 1. Please note that
paired reads need to be on opposite strands, and erroneous entries will
result in casting of an error.}

\item{chromosome}{The chromosome on which the track's genomic ranges are
defined. A valid UCSC chromosome identifier if
\code{options(ucscChromosomeNames=TRUE)}. Please note that in this case only
syntactic checking takes place, i.e., the argument value needs to be an
integer, numeric character or a character of the form \code{chrx}, where
\code{x} may be any possible string. The user has to make sure that the
respective chromosome is indeed defined for the the track's genome. If not
provided here, the constructor will try to construct the chromosome
information based on the available inputs, and as a last resort will fall
back to the value \code{chrNA}. Please note that by definition all objects
in the \code{Gviz} package can only have a single active chromosome at a
time (although internally the information for more than one chromosome may
be present), and the user has to call the \code{chromosome<-} replacement
method in order to change to a different active chromosome.}

\item{genome}{The genome on which the track's ranges are defined. Usually
this is a valid UCSC genome identifier, however this is not being formally
checked at this point. If not provided here the constructor will try to
extract this information from the provided input, and eventually will fall
back to the default value of \code{NA}.}

\item{stacking}{The stacking type for overlapping items of the track. One in
\code{c(hide, dense, squish, pack, full)}. Currently, only squish (make best
use of the available space), dense (no stacking, collapse overlapping
ranges), and hide (do not show any track items at all) are implemented.}

\item{id}{Character vector of read identifiers. Those identifiers have to be
unique, i.e., each range representing a read needs to have a unique
\code{id}.}

\item{cigar}{A character vector of valid CIGAR strings describing details of
the alignment. Typically those include alignment gaps or insertions and
deletions, but also hard and soft clipped read regions. If missing, a fully
mapped read without gaps or indels is assumed. Needs to be of equal length
as the provided genomic coordinates, or of length 1.}

\item{mapq}{A numeric vector of read mapping qualities. Needs to be of equal
length as the provided genomic coordinates, or of length 1.}

\item{flag}{A named integer vector of length 2, as is produced by
Rsamtools::scanBamFlag(), used to filter out undesirable reads. If missing,
all mapped reads will be included.}

\item{isize}{A numeric vector of empirical insert sizes. This only applies
if the reads are paired. Needs to be of equal length as the provided genomic
coordinates, or of length 1. Currently not used.}

\item{groupid}{A factor (or vector than can be coerced into one) defining
the read pairs. Reads with the same \code{groupid} are considered to be
mates. Please note that each read group may only have one or two members.
Needs to be of equal length as the provided genomic coordinates, or of
length 1.}

\item{status}{A factor describing the mapping status of a read. Has to be
one in \code{mated}, \code{unmated} or \code{ambiguous}. Needs to be of
equal length as the provided genomic coordinates, or of length 1.}

\item{md}{A character vector describing the mapping details. This is
effectively and alternative to the CIGAR encoding and it removes the
dependency on a reference sequence to figure out read mismatches. Needs to
be of equal length as the provided genomic coordinates, or of length 1.
Currently not used.}

\item{seqs}{\code{DNAStringSet} of read sequences.}

\item{name}{Character scalar of the track's name used in the title panel
when plotting.}

\item{isPaired}{A logical scalar to determine whether the reads are paired
or not. While this may be used to render paired-end data as single-end, the
oppsite will typically not have any effect because the appropriate
\code{groupid} settings will not be present.  Thus setting \code{isPaired}
to \code{TRUE} can usually be used to autodetect the pairing state of the
input data.}

\item{importFunction}{A user-defined function to be used to import the data
from a file. This only applies when the \code{range} argument is a character
string with the path to the input data file. The function needs to accept an
argument \code{x} containing the file path and a second argument
\code{selection} with the desired plotting ranges. It has to return a proper
\code{GRanges} object with all the necessary metadata columns set. A single
default import function is already implemented in the package for \code{BAM}
files.}

\item{x}{x}

\item{GdObject}{Object of \code{GdObject-class}.}

\item{value}{value}

\item{from, to}{from,to}

\item{use.defaults}{\code{logical}}

\item{minBase, maxBase}{minBase,maxBase}

\item{prepare}{\code{logical}}

\item{subset}{\code{logical}}

\item{object}{object}
}
\value{
The return value of the constructor function is a new object of class
\code{AlignmentsTrack} or \code{ReferenceAlignmentsTrack}.
}
\description{
A class to represent short sequences that have been aligned to a reference
genome as they are typically generated in next generation sequencing
experiments.
}
\section{Functions}{
\itemize{
\item \code{initialize(AlignmentsTrack)}: Initialize.

\item \code{ReferenceAlignmentsTrack-class}: The file-based version of the \code{AlignmentsTrack-class}.

\item \code{initialize(ReferenceAlignmentsTrack)}: Initialize.

\item \code{AlignmentsTrack()}: Constructor for \code{AlignmentsTrack-class}.

\item \code{values(AlignmentsTrack)}: Return all additional annotation
information except for the genomic coordinates for the track items as
a \code{data.frame}.

\item \code{chromosome(AlignmentsTrack) <- value}: replace the value of the track's chromosome.
This has to be a valid UCSC chromosome identifier or an integer or character
scalar that can be reasonably coerced into one.

\item \code{stacks(AlignmentsTrack)}: return the stack indices for each track item.

\item \code{setStacks(AlignmentsTrack)}: recompute the stacks based on the available
space and on the object's track items and stacking settings.

\item \code{subset(AlignmentsTrack)}: Subset a \code{AlignmentsTrack} by coordinates
and sort if necessary.

\item \code{subset(ReferenceAlignmentsTrack)}: Subset a \code{ReferenceAlignmentsTrack} by coordinates
and sort if necessary.

\item \code{drawAxis(AlignmentsTrack)}: add a y-axis to the title panel of a track.

\item \code{drawGD(AlignmentsTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(AlignmentsTrack)}: Show method.

\item \code{show(ReferenceAlignmentsTrack)}: Show method.

}}
\section{Objects from the Class}{


Objects can be created using the constructor function
\code{AlignmentsTrack}.
}

\examples{
## Creating objects
afrom <- 2960000
ato <- 3160000
alTrack <- AlignmentsTrack(system.file(
    package = "Gviz", "extdata",
    "gapped.bam"
), isPaired = TRUE)
plotTracks(alTrack, from = afrom, to = ato, chromosome = "chr12")

## Omit the coverage or the pile-ups part
plotTracks(alTrack,
    from = afrom, to = ato, chromosome = "chr12",
    type = "coverage"
)
plotTracks(alTrack,
    from = afrom, to = ato, chromosome = "chr12",
    type = "pileup"
)

## Including sequence information with the constructor
if (require(BSgenome.Hsapiens.UCSC.hg19)) {
    strack <- SequenceTrack(Hsapiens, chromosome = "chr21")
    afrom <- 44945200
    ato <- 44947200
    alTrack <- AlignmentsTrack(system.file(
        package = "Gviz", "extdata",
        "snps.bam"
    ), isPaired = TRUE, referenceSequence = strack)
    plotTracks(alTrack, chromosome = "chr21", from = afrom, to = ato)

    ## Including sequence information in the track list
    alTrack <- AlignmentsTrack(system.file(
        package = "Gviz", "extdata",
        "snps.bam"
    ), isPaired = TRUE)
    plotTracks(c(alTrack, strack),
        chromosome = "chr21", from = 44946590,
        to = 44946660
    )
}
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SequenceTrack-class.R
\docType{class}
\name{SequenceTrack-class}
\alias{SequenceTrack-class}
\alias{initialize,SequenceTrack-method}
\alias{SequenceTrack}
\alias{RNASequenceTrack}
\alias{SequenceDNAStringSetTrack-class}
\alias{initialize,SequenceDNAStringSetTrack-method}
\alias{SequenceRNAStringSetTrack-class}
\alias{initialize,SequenceRNAStringSetTrack-method}
\alias{SequenceBSgenomeTrack-class}
\alias{initialize,SequenceBSgenomeTrack-method}
\alias{ReferenceSequenceTrack-class}
\alias{initialize,ReferenceSequenceTrack-method}
\alias{seqnames,SequenceTrack-method}
\alias{seqnames,SequenceBSgenomeTrack-method}
\alias{seqlevels,SequenceTrack-method}
\alias{seqlevels,SequenceBSgenomeTrack-method}
\alias{start,SequenceTrack-method}
\alias{end,SequenceTrack-method}
\alias{width,SequenceTrack-method}
\alias{length,SequenceTrack-method}
\alias{chromosome,SequenceTrack-method}
\alias{chromosome<-,SequenceTrack-method}
\alias{genome,SequenceTrack-method}
\alias{consolidateTrack,SequenceTrack-method}
\alias{drawGD,SequenceTrack-method}
\alias{show,SequenceBSgenomeTrack-method}
\alias{show,SequenceDNAStringSetTrack-method}
\alias{show,SequenceRNAStringSetTrack-method}
\alias{show,ReferenceSequenceTrack-method}
\title{SequenceTrack class and methods}
\usage{
\S4method{initialize}{SequenceTrack}(.Object, chromosome, genome, ...)

SequenceTrack(
  sequence,
  chromosome,
  genome,
  name = "SequenceTrack",
  importFunction,
  stream = FALSE,
  ...
)

RNASequenceTrack(
  sequence,
  chromosome,
  genome,
  name = "SequenceTrack",
  importFunction,
  stream = FALSE,
  ...
)

\S4method{initialize}{SequenceDNAStringSetTrack}(.Object, sequence, ...)

\S4method{initialize}{SequenceRNAStringSetTrack}(.Object, sequence, ...)

\S4method{initialize}{SequenceBSgenomeTrack}(.Object, sequence = NULL, ...)

\S4method{initialize}{ReferenceSequenceTrack}(.Object, stream, reference, ...)

\S4method{seqnames}{SequenceTrack}(x)

\S4method{seqnames}{SequenceBSgenomeTrack}(x)

\S4method{seqlevels}{SequenceTrack}(x)

\S4method{seqlevels}{SequenceBSgenomeTrack}(x)

\S4method{start}{SequenceTrack}(x)

\S4method{end}{SequenceTrack}(x)

\S4method{width}{SequenceTrack}(x)

\S4method{length}{SequenceTrack}(x)

\S4method{chromosome}{SequenceTrack}(GdObject)

\S4method{chromosome}{SequenceTrack}(GdObject) <- value

\S4method{genome}{SequenceTrack}(x)

\S4method{consolidateTrack}{SequenceTrack}(GdObject, chromosome, ...)

\S4method{drawGD}{SequenceTrack}(GdObject, minBase, maxBase, prepare = FALSE, ...)

\S4method{show}{SequenceBSgenomeTrack}(object)

\S4method{show}{SequenceDNAStringSetTrack}(object)

\S4method{show}{SequenceRNAStringSetTrack}(object)

\S4method{show}{ReferenceSequenceTrack}(object)
}
\arguments{
\item{.Object}{.Object}

\item{chromosome}{the currently active chromosome which may have to be set
for a \code{RangeTrack} or a \code{SequenceTrack} object
parameters}

\item{genome}{The genome on which the track's ranges are defined. Usually
this is a valid UCSC genome identifier, however this is not being formally
checked at this point. For a \code{SequenceBSgenomeTrack} object, the genome
information is extracted from the input \code{BSgenome} package. For a
\code{DNAStringSet} it has too be provided or the constructor will fall back
to the default value of \code{NA}.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters. See \code{\link{settings}} and the "Display Parameters"
section below for details.}

\item{sequence}{A meta argument to handle the different input types, making the construction
of a \code{SequenceTrack} as flexible as possible.

The different input options for \code{sequence} are:

\describe{

\item{An object of class \code{\linkS4class{DNAStringSet}}.}{ The individual
\code{\linkS4class{DNAString}}s are considered to be the different
chromosome sequences.}

\item{An object of class \code{\linkS4class{BSgenome}}.}{ The \code{Gviz}
package tries to follow the \code{BSgenome} philosophy in that the
respective chromosome sequences are only realized once they are first
accessed.}

\item{A \code{character} scalar:}{ in this case the value of the
\code{sequence} argument is considered to be a file path to an annotation
file on disk. A range of file types are supported by the \code{Gviz} package
as identified by the file extension. See the \code{importFunction}
documentation below for further details.}
}}

\item{name}{Character scalar of the track's name used in the title panel
when plotting.}

\item{importFunction}{A user-defined function to be used to import the
sequence data from a file. This only applies when the \code{sequence}
argument is a character string with the path to the input data file. The
function needs to accept an argument \code{file} containing the file path
and has to return a proper \code{DNAStringSet} object with the sequence
information per chromosome. A set of default import functions is already
implemented in the package for a number of different file types, and one of
these defaults will be picked automatically based on the extension of the
input file name. If the extension can not be mapped to any of the existing
import function, an error is raised asking for a user-defined import
function. Currently the following file types can be imported with the
default functions: \code{fa/fasta} and \code{2bit}.

Both file types support indexing by genomic coordinates, and it makes sense
to only load the part of the file that is needed for plotting. To this end,
the \code{Gviz} package defines the derived \code{ReferenceSequenceTrack}
class, which supports streaming data from the file system. The user
typically does not have to deal with this distinction but may rely on the
constructor function to make the right choice as long as the default import
functions are used. However, once a user-defined import function has been
provided and if this function adds support for indexed files, you will have
to make the constructor aware of this fact by setting the \code{stream}
argument to \code{TRUE}. Please note that in this case the import function
needs to accept a second mandatory argument \code{selection} which is a
\code{GRanges} object containing the dimensions of the plotted genomic
range. As before, the function has to return an appropriate
\code{DNAStringSet} object.}

\item{stream}{A logical flag indicating that the user-provided import
function can deal with indexed files and knows how to process the additional
\code{selection} argument when accessing the data on disk. This causes the
constructor to return a \code{ReferenceSequenceTrack} object which will grab
the necessary data on the fly during each plotting operation.}

\item{reference}{Name of the file (for streatming).}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}

\item{GdObject}{the input track object}

\item{value}{Value to be set.}

\item{minBase}{Start of the sequence.}

\item{maxBase}{End of the sequence.}

\item{prepare}{\code{logical}}

\item{object}{object}
}
\value{
The return value of the constructor function is a new object of class
\code{SequenceDNAStringSetTrack}, \code{SequenceBSgenomeTrack} ore
\code{ReferenceSequenceTrack}, depending on the constructor arguments.
Typically the user will not have to be troubled with this distinction and
can rely on the constructor to make the right choice.
}
\description{
A track class to represent genomic sequences. The three child classes
\code{SequenceDNAStringSetTrack}, \code{SequenceRNAStringSetTrack} and
\code{SequenceBSgenomeTrack} do most of the work, however in practise they
are of no particular relevance to the user.
}
\section{Functions}{
\itemize{
\item \code{initialize(SequenceTrack)}: Initialize.

\item \code{SequenceTrack()}: Constructor

\item \code{RNASequenceTrack()}: Constructor

\item \code{SequenceDNAStringSetTrack-class}: The \code{DNAStringSet}-based version of the \code{SequenceTrack-class}.

\item \code{initialize(SequenceDNAStringSetTrack)}: Initialize.

\item \code{SequenceRNAStringSetTrack-class}: The \code{RNAStringSet}-based version of the \code{SequenceTrack-class}.

\item \code{initialize(SequenceRNAStringSetTrack)}: Initialize \code{RNAStringSet}-based version of the \code{SequenceTrack-class}.

\item \code{SequenceBSgenomeTrack-class}: The \code{BSgenome}-based version of the \code{SequenceTrack-class}.

\item \code{initialize(SequenceBSgenomeTrack)}: Initialize.

\item \code{ReferenceSequenceTrack-class}: The file-based version of the \code{SequenceTrack-class}.

\item \code{initialize(ReferenceSequenceTrack)}: Initialize.

\item \code{seqnames(SequenceTrack)}: return the names (i.e., the chromosome)
of the sequences contained in the object.

\item \code{seqnames(SequenceBSgenomeTrack)}: return the names (i.e., the chromosome)
of the sequences contained in the object.

\item \code{seqlevels(SequenceTrack)}: return the names (i.e., the chromosome)
of the sequences contained in the object. Only those with length > 0.

\item \code{seqlevels(SequenceBSgenomeTrack)}: return the names (i.e., the chromosome)
of the sequences contained in the object. Only those with length > 0.

\item \code{start(SequenceTrack)}: return the start coordinates of the track
items.

\item \code{end(SequenceTrack)}: return the end coordinates of the track
items.

\item \code{width(SequenceTrack)}: return the with of the track items in
genomic coordinates.

\item \code{length(SequenceTrack)}: return the length of the sequence for
active chromosome.

\item \code{chromosome(SequenceTrack)}: return the chromosome for which the track
is defined.

\item \code{chromosome(SequenceTrack) <- value}: replace the value of the track's chromosome.
This has to be a valid UCSC chromosome identifier or an integer or character
scalar that can be reasonably coerced into one.

\item \code{genome(SequenceTrack)}: Set the track's genome.
Usually this has to be a valid UCSC identifier, however this is not
formally enforced here.

\item \code{consolidateTrack(SequenceTrack)}: Consolidate/
Determine whether there is \code{chromosome} settings or not, and add this information.

\item \code{drawGD(SequenceTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(SequenceBSgenomeTrack)}: Show method.

\item \code{show(SequenceDNAStringSetTrack)}: Show method.

\item \code{show(SequenceRNAStringSetTrack)}: Show method.

\item \code{show(ReferenceSequenceTrack)}: Show method.

}}
\section{Objects from the class}{


Objects can be created using the constructor function \code{SequenceTrack}.
}

\examples{
## An empty object
SequenceTrack()

## Construct from DNAStringSet
library(Biostrings)
letters <- c("A", "C", "T", "G", "N")
set.seed(999)
seqs <- DNAStringSet(c(chr1 = paste(sample(letters, 100000, TRUE),
    collapse = ""
), chr2 = paste(sample(letters, 200000, TRUE), collapse = "")))
sTrack <- SequenceTrack(seqs, genome = "hg19")
sTrack

## Construct from BSGenome object
if (require(BSgenome.Hsapiens.UCSC.hg19)) {
    sTrack <- SequenceTrack(Hsapiens)
    sTrack
}


## Set active chromosome
chromosome(sTrack)
chromosome(sTrack) <- "chr2"
head(seqnames(sTrack))
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(sTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
## Sequences
plotTracks(sTrack, from = 199970, to = 200000)
## Boxes
plotTracks(sTrack, from = 199800, to = 200000)
## Line
plotTracks(sTrack, from = 1, to = 200000)
## Force boxes
plotTracks(sTrack, from = 199970, to = 200000, noLetters = TRUE)
## Direction indicator
plotTracks(sTrack, from = 199970, to = 200000, add53 = TRUE)
## Sequence complement
plotTracks(sTrack, from = 199970, to = 200000, add53 = TRUE, complement = TRUE)
## Colors
plotTracks(sTrack, from = 199970, to = 200000, add53 = TRUE, fontcolor = c(
    A = 1,
    C = 1, G = 1, T = 1, N = 1
))

## Track names
names(sTrack)
names(sTrack) <- "foo"

## Accessors
genome(sTrack)
genome(sTrack) <- "mm9"
length(sTrack)

## Sequence extraction
subseq(sTrack, start = 100000, width = 20)
## beyond the stored sequence range
subseq(sTrack, start = length(sTrack), width = 20)
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ImageMap-class.R
\docType{class}
\name{ImageMap-class}
\alias{ImageMap-class}
\alias{coords}
\alias{coords,NULL-method}
\alias{coords,ImageMap-method}
\alias{tags}
\alias{tags,NULL-method}
\alias{tags,ImageMap-method}
\title{ImageMap: HTML image map information}
\usage{
coords(ImageMap, ...)

\S4method{coords}{`NULL`}(ImageMap)

\S4method{coords}{ImageMap}(ImageMap)

tags(ImageMap, ...)

\S4method{tags}{`NULL`}(ImageMap)

\S4method{tags}{ImageMap}(ImageMap)
}
\arguments{
\item{ImageMap}{Object of \code{ImageMap-class}, containing optional information
for an HTML image map.}
}
\value{
Returns the coordinates from the image map.

Returns the tags from the image map.
}
\description{
HTML image map information for annotation tracks.
}
\details{
Objects of the \code{ImageMap-class} are usually not created by the user,
hence the constructor function \code{ImageMap} is not exported in the name space.
}
\section{Functions}{
\itemize{
\item \code{coords()}: Generics for \code{coords}.

\item \code{coords(`NULL`)}: Returns the coordinates from the image map.

\item \code{coords(ImageMap)}: Returns the coordinates from the image map.

\item \code{tags()}: Generics for \code{tags}.

\item \code{tags(`NULL`)}: Returns the tags from the image map

\item \code{tags(ImageMap)}: Returns the tags from the image map

}}
\section{Slots}{

\describe{
\item{\code{coords}}{Object of class \code{matrix}, the image map coordinates. In the
order x bl, y bl, x tr, y tr. Row names are mandatory for the matrix
and have to be unique.}

\item{\code{tags}}{Object of class \code{list}, the individual HTML tags for the image map.
The value of each list item has to be a named character vector,
where the names must match back into the row names of the \code{coords} matrix}
}}

\examples{
## Not provided. This is an internal structure.

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StackedTrack-class.R
\docType{class}
\name{StackedTrack-class}
\alias{StackedTrack-class}
\alias{initialize,StackedTrack-method}
\alias{stacking,StackedTrack-method}
\alias{stacking<-,StackedTrack,character-method}
\alias{stacks,StackedTrack-method}
\alias{setStacks,StackedTrack-method}
\alias{consolidateTrack,StackedTrack-method}
\alias{[,StackedTrack,ANY,ANY,ANY-method}
\alias{subset,StackedTrack-method}
\alias{drawGD,StackedTrack-method}
\title{StackedTrack class and methods}
\usage{
\S4method{initialize}{StackedTrack}(.Object, stacking, ...)

\S4method{stacking}{StackedTrack}(GdObject)

\S4method{stacking}{StackedTrack,character}(GdObject) <- value

\S4method{stacks}{StackedTrack}(GdObject)

\S4method{setStacks}{StackedTrack}(GdObject, ...)

\S4method{consolidateTrack}{StackedTrack}(GdObject, ...)

\S4method{[}{StackedTrack,ANY,ANY,ANY}(x, i, j, ..., drop = TRUE)

\S4method{subset}{StackedTrack}(x, from = NULL, to = NULL, sort = FALSE, stacks = FALSE, ...)

\S4method{drawGD}{StackedTrack}(GdObject, ...)
}
\arguments{
\item{.Object}{.Object}

\item{stacking}{stacking}

\item{...}{Additional arguments.}

\item{GdObject}{Object of \code{GdObject-class}.}

\item{value}{Value to be set.}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}

\item{i}{Numeric scalar, index to subset.}

\item{j}{Numeric scalar, index to subset. Ignored.}

\item{drop}{\code{logical}, indicating if levels that do not occur should be dropped (if \code{f} is a factor).}

\item{from, to}{Numeric scalar, giving the range of genomic coordinates to
limit the tracks in. Note that \code{to} cannot be larger than \code{from.}}

\item{sort}{\code{logical}.}

\item{stacks}{\code{logical}. Set if stacking should  be preserved.}
}
\value{
A virtual Class: No objects may be created from it.
}
\description{
The virtual parent class for all track types in the Gviz package which
contain potentially overlapping annotation items that have to be stacked
when plotted.
}
\section{Functions}{
\itemize{
\item \code{initialize(StackedTrack)}: Initialize.

\item \code{stacking(StackedTrack)}: return the current stacking type.

\item \code{stacking(GdObject = StackedTrack) <- value}: set the object's stacking type to one in
\code{c(hide, dense, squish, pack,full)}.

\item \code{stacks(StackedTrack)}: return the stack indices for each track item.

\item \code{setStacks(StackedTrack)}: recompute the stacks based on the available
space and on the object's track items and stacking settings.

\item \code{consolidateTrack(StackedTrack)}: Consolidate.
a display parameter)

\item \code{x[i}: subset the items in the \code{StackedTrack} object.
This is essentially similar to subsetting of the \code{GRanges} object in the
range slot. For most applications, the subset method may be more appropriate.

\item \code{subset(StackedTrack)}: subset a \code{StackedTrack} by coordinates and sort if necessary.

\item \code{drawGD(StackedTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which
the method can be called. Either in 'prepare' mode, in which case no
plotting is done but the stacking information is updated based on the
available space, or in 'plotting' mode, in which case the actual graphical
output is created. Note that the method for this particular subclass is
usually called through inheritance and not particularly useful on its own.

}}
\section{Slots}{

\describe{
\item{\code{dp}}{Object of \code{DisplayPars-class}, the display settings controlling the
look and feel of a track. See settings for details on setting graphical
parameters for tracks.}

\item{\code{name}}{Object of class \code{character}, a human-readable name for the track
that will be used in the track's annotation panel if necessary.}

\item{\code{imageMap}}{Object of \code{ImageMap-class}, containing optional information
for an HTML image map. This will be created by the \code{drawGD} methods when the
track is plotted to a device and is usually not set by the user.}

\item{\code{range}}{Object of class \code{GRanges}, the genomic ranges of the track items
as well as additional annotation information in its \code{elementMetaData} slot.
Please not that the slot is actually implemented as a class union between
\code{GRanges} and IRanges to increase efficiency, for instance for DataTrack
objects. This usually does not concern the user.}

\item{\code{chromosome}}{Object of class \code{character}, the chromosome on which the
track is defined. There can only be a single chromosome for one track. For
certain subclasses, the space of allowed chromosome names is limited (e.g.,
only those chromosomes that exist for a particular genome). Throughout the
package, chromosome name have to be entered either as a single integer scalar
or as a character scalar of the form chrXYZ, where XYZ may be an arbitrary
character string.}

\item{\code{genome}}{Object of class \code{character}, the genome for which the track is
defined. For most sub-classes this has to be valid UCSC genome identifier,
however this may not always be formally checked upon object instantiation.}

\item{\code{stacking}}{Object of class \code{character}, the stacking type of overlapping
items on the final plot. One in \code{c(hide, dense, squish, pack,full)}.
Currently, only \code{hide} (do not show the track items at all), \code{squish} (make
best use of the available space) and \code{dense} (no stacking at all) are
implemented.}

\item{\code{stacks}}{Object of class \code{numeric}, holding the stack indices for each
track item. This slot is usually populated by calling the \code{setStacks} method
upon plotting, since the correct stacking is a function of the available
plotting space.}
}}

\examples{
## This is a reference class therefore we show below
## an example from AnnotationTrack

## An empty object
AnnotationTrack()

## Construct from individual arguments
st <- c(2000000, 2070000, 2100000, 2160000)
ed <- c(2050000, 2130000, 2150000, 2170000)
str <- c("-", "+", "-", "-")
gr <- c("Group1", "Group2", "Group1", "Group3")

annTrack <- AnnotationTrack(
    start = st, end = ed, strand = str, chromosome = 7,
    genome = "hg19", feature = "test", group = gr,
    id = paste("annTrack item", 1:4),
    name = "generic annotation", stacking = "squish"
)
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(annTrack)

## Stacking
stacking(annTrack)
stacking(annTrack) <- "dense"
plotTracks(annTrack)
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotTracks.R
\name{plotTracks}
\alias{plotTracks}
\title{The main plotting function for one or several Gviz tracks.}
\usage{
plotTracks(
  trackList,
  from = NULL,
  to = NULL,
  ...,
  sizes = NULL,
  panel.only = FALSE,
  extend.right = 0,
  extend.left = 0,
  title.width = NULL,
  add = FALSE,
  main,
  cex.main = 2,
  fontface.main = 2,
  col.main = "black",
  margin = 6,
  chromosome = NULL,
  innerMargin = 3
)
}
\arguments{
\item{trackList}{A list of Gviz track objects, all inheriting from
class \code{\linkS4class{GdObject}}. The tracks will all be drawn to the
same genomic coordinates, either as defined by the \code{from} and \code{to}
arguments if supplied, or by the maximum range across all individual items
in the list.}

\item{from, to}{Character scalar, giving the range of genomic coordinates to
draw the tracks in. Note that \code{to} cannot be larger than \code{from}.
If \code{NULL}, the plotting ranges are derived from the individual tracks.
See \code{extend.left} and \code{extend.right} below for the definition of
the final plotting ranges.}

\item{\dots}{Additional arguments which are all interpreted as display
parameters to tweak the appearance of the plot. These parameters are global,
meaning that they will be used for all tracks in the list where they
actually make sense, and they override the track-internal settings. See
\link{settings} for details on display parameters.}

\item{sizes}{A numeric vector of relative vertical sizes for the individual
tracks of length equal to the number of tracks in \code{trackList}, or
\code{NULL} to auto-detect the most appropriate vertical size proportions.}

\item{panel.only}{Logical flag, causing the tracks to be plotted as
lattice-like panel functions without resetting the plotting canvas and
omitting the title pane. This allows to embed tracks into a trellis layout.
Usually the function is called for a single track only when
\code{panel.only==TRUE}.}

\item{extend.right, extend.left}{Numeric scalar, extend the plotting range to
the right or to the left by a fixed number of bases. The final plotting
range is defined as \code{from-extend.left} to \code{to+extend.right}.}

\item{title.width}{A expansion factor for the width of the title panels.
This can be used to make more space, e.g. to accommodate for more detailed
data axes. The default is to use as much space as needed to fit all the
annotation text.}

\item{add}{Logical flag, add the plot to an existing plotting canvas without
re-initialising.}

\item{main}{Character scalar, the plots main header.}

\item{cex.main, fontface.main, col.main}{The fontface, color and expansion
factor settings for the main header.}

\item{margin}{The margin width to add to the plot in pixels.}

\item{chromosome}{Set the chromosome for all the tracks in the track list.}

\item{innerMargin}{The inner margin width to add to the plot in pixels.}
}
\value{
A list of Gviz tracks, each one augmented by the computed image map
coordinates in the \code{imageMap} slot, along with the additional \code{ImageMap}
object \code{titles} containing information about the title panels.
}
\description{
\code{plotTracks} is the main interface when plotting single track objects,
or lists of tracks linked together across the same genomic coordinates.
Essentially, the resulting plots are very similar to the graphical output of
the UCSC Genome Browser, except for all of the interactivity.
}
\details{
Gviz tracks are plotted in a vertically stacked layout. Each track
panel is split up into a title section containing the track name, as well as
an optional axis for tracks containing numeric data, and a data section
showing the actual data along genomic coordinates. In that sense, the output
is very similar to the UCSC Genome Browser.

The layout of the individual tracks is highly customizable though so called
"display parameters". See \code{\link{settings}} for details.

While plotting a track, the software automatically computes HTML image map
coordinates based on the current graphics device. These coordinates as well
as the associated annotation information can later be used to embed images
of the plots in semi-interactive HTML pages. See
\code{\linkS4class{ImageMap}} for details.
}
\examples{
## Create some tracks to plot
st <- c(2000000, 2070000, 2100000, 2160000)
ed <- c(2050000, 2130000, 2150000, 2170000)
str <- c("-", "+", "-", "-")
gr <- c("Group1", "Group2", "Group1", "Group3")
annTrack <- AnnotationTrack(
    start = st, end = ed, strand = str, chromosome = 7,
    genome = "hg19", feature = "test", group = gr,
    id = paste("annTrack item", 1:4),
    name = "annotation track foo",
    stacking = "squish"
)

ax <- GenomeAxisTrack()

dt <- DataTrack(
    start = seq(min(st), max(ed), len = 10), width = 18000,
    data = matrix(runif(40), nrow = 4), genome = "hg19", chromosome = 7,
    type = "histogram", name = "data track bar"
)
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
    displayPars(ax) <- list(fontfamily = font, fontfamily.title = font)
    displayPars(dt) <- list(fontfamily = font, fontfamily.title = font)
}
}


## Now plot the tracks
res <- plotTracks(list(ax, annTrack, dt))

## Plot only a subrange
res <- plotTracks(list(ax, annTrack, dt), from = 2080000, to = 2156000)

## Extend plotting ranges
res <- plotTracks(list(ax, annTrack, dt), extend.left = 200000, extend.right = 200000)

## Add a header
res <- plotTracks(list(ax, annTrack, dt),
    main = "A GenomGraphs plot",
    col.main = "darkgray"
)

## Change vertical size and title width
res <- plotTracks(list(ax, annTrack, dt), sizes = c(1, 1, 5))

names(annTrack) <- "foo"
res <- plotTracks(list(ax, annTrack), title.width = 0.6)

## Adding and lattice like plots
library(grid)
grid.newpage()
pushViewport(viewport(height = 0.5, y = 1, just = "top"))
grid.rect()
plotTracks(annTrack, add = TRUE)
popViewport(1)
pushViewport(viewport(height = 0.5, y = 0, just = "bottom"))
grid.rect()
plotTracks(dt, add = TRUE)
popViewport(1)
\dontrun{
library(lattice)
myPanel <- function(x, ...) {
    plotTracks(annTrack,
        panel.only = TRUE,
        from = min(x), to = max(x), shape = "box"
    )
}
a <- seq(1900000, 2250000, len = 40)
xyplot(b ~ a | c, data.frame(a = a, b = 1, c = cut(a, 4)),
    panel = myPanel,
    scales = list(x = "free")
)
}

}
\seealso{
\code{\linkS4class{GdObject}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{StackedTrack}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GdObject-class.R
\docType{class}
\name{GdObject-class}
\alias{GdObject-class}
\alias{initialize,GdObject-method}
\alias{setPar,GdObject,character-method}
\alias{setPar,GdObject,list-method}
\alias{displayPars<-,GdObject,list-method}
\alias{getPar,GdObject,character-method}
\alias{getPar,GdObject,missing-method}
\alias{displayPars,GdObject,character-method}
\alias{displayPars,GdObject,missing-method}
\alias{coords,GdObject-method}
\alias{tags,GdObject-method}
\alias{subset,GdObject-method}
\alias{names,GdObject-method}
\alias{names<-,GdObject,character-method}
\alias{group}
\alias{group<-}
\alias{group,GdObject-method}
\alias{imageMap}
\alias{imageMap,GdObject-method}
\alias{imageMap<-}
\alias{imageMap<-,GdObject,ImageMapOrNULL-method}
\alias{drawAxis}
\alias{drawAxis,GdObject-method}
\alias{drawGrid}
\alias{drawGD}
\alias{gene}
\alias{gene<-}
\alias{symbol}
\alias{symbol<-}
\alias{transcript}
\alias{transcript<-}
\alias{exon}
\alias{exon<-}
\alias{feature}
\alias{feature<-}
\alias{identifier}
\alias{identifier<-}
\alias{chromosome}
\alias{chromosome,GdObject-method}
\alias{chromosome<-}
\alias{chromosome<-,GdObject-method}
\alias{position}
\alias{genome,GdObject-method}
\alias{genome<-,GdObject-method}
\alias{consolidateTrack}
\alias{consolidateTrack,GdObject-method}
\alias{stacking}
\alias{stacking<-}
\alias{stacks}
\alias{setStacks}
\alias{setStacks,GdObject-method}
\alias{setCoverage}
\title{GdObject class and methods}
\usage{
\S4method{initialize}{GdObject}(.Object, name, ...)

\S4method{setPar}{GdObject,character}(x, name, value, interactive = TRUE)

\S4method{setPar}{GdObject,list}(x, value, interactive = TRUE)

\S4method{displayPars}{GdObject,list}(x, recursive = FALSE) <- value

\S4method{getPar}{GdObject,character}(x, name, asIs = FALSE)

\S4method{getPar}{GdObject,missing}(x, hideInternal = TRUE)

\S4method{displayPars}{GdObject,character}(x, name)

\S4method{displayPars}{GdObject,missing}(x, hideInternal = TRUE)

\S4method{coords}{GdObject}(ImageMap)

\S4method{tags}{GdObject}(ImageMap)

\S4method{subset}{GdObject}(x, ...)

\S4method{names}{GdObject}(x)

\S4method{names}{GdObject,character}(x) <- value

group(GdObject, ...)

group(GdObject) <- value

\S4method{group}{GdObject}(GdObject)

imageMap(GdObject, ...)

\S4method{imageMap}{GdObject}(GdObject)

imageMap(GdObject) <- value

\S4method{imageMap}{GdObject,ImageMapOrNULL}(GdObject) <- value

drawAxis(GdObject, ...)

\S4method{drawAxis}{GdObject}(GdObject, ...)

drawGrid(GdObject, ...)

drawGD(GdObject, ...)

gene(GdObject, ...)

gene(GdObject) <- value

symbol(GdObject, ...)

symbol(GdObject) <- value

transcript(GdObject, ...)

transcript(GdObject) <- value

exon(GdObject, ...)

exon(GdObject) <- value

feature(GdObject, ...)

feature(GdObject) <- value

identifier(GdObject, ...)

identifier(GdObject) <- value

chromosome(GdObject, ...)

\S4method{chromosome}{GdObject}(GdObject)

chromosome(GdObject) <- value

\S4method{chromosome}{GdObject}(GdObject) <- value

position(GdObject, ...)

\S4method{genome}{GdObject}(x)

\S4method{genome}{GdObject}(x) <- value

consolidateTrack(GdObject, ...)

\S4method{consolidateTrack}{GdObject}(GdObject, alpha, ...)

stacking(GdObject, ...)

stacking(GdObject) <- value

stacks(GdObject, ...)

setStacks(GdObject, ...)

\S4method{setStacks}{GdObject}(GdObject, ...)

setCoverage(GdObject, ...)
}
\arguments{
\item{name}{Name of the retrieved parameter.}

\item{\dots}{Additional arguments.}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}

\item{value}{Value to be set.}

\item{interactive}{\code{logical}}

\item{recursive}{\code{logical}}

\item{asIs}{\code{logical}}

\item{hideInternal}{\code{logical}}

\item{ImageMap}{Object of \code{ImageMap-class}, containing optional information
for an HTML image map.}

\item{GdObject}{Object of \code{GdObject-class}.}
}
\value{
A virtual class: No objects may be created from it.
}
\description{
The virtual parent class for all track items in the Gviz package.  This
class definition contains all the common entities that are needed for a
track to be plotted. During object instantiation for any of the sub-classes
inheriting from \code{GdObject}, this class' global initializer has to be
called in order to assure that all necessary settings are present.
}
\details{
\subsection{Display Parameters}{

The following display parameters are set for objects of class \code{GdObject}
upon instantiation, unless one or more of them have already been set by
one of the optional sub-class initializers, which always get precedence
over these global defaults. See \code{settings} for details on setting
graphical parameters for tracks.
\itemize{
\item \code{alpha=1} Numeric scalar. The transparency for all track items.
\item \code{alpha.title=NULL} Numeric scalar. The transparency for the title panel.
\item \code{background.legend="transparent"} Integer or character scalar.
The background colour for the legend.
\item \code{background.panel="transparent"} Integer or character scalar.
The background colour of the content panel.
\item \code{background.title="lightgray"} Integer or character scalar.
The background colour for the title panel.
\item \code{cex=1} Numeric scalar. The overall font expansion factor for all text
and glyphs, unless a more specific definition exists.
\item \code{cex.axis=NULL} Numeric scalar. The expansion factor for the axis
annotation. Defaults to \code{NULL}, in which case it is automatically determined
based on the available space.
\item \code{cex.title=NULL} Numeric scalar. The expansion factor for the title
panel. This effects the font size of both the title and the axis, if any.
Defaults to NULL, which means that the text size is automatically adjusted
to the available space.
\item \code{col="#0080FF"} Integer or character scalar. Default line colour setting
for all plotting elements, unless there is a more specific control defined
elsewhere.
\item \code{col.axis="white"} Integer or character scalar. The font and line colour
for the y axis, if any.
\item \code{col.border.title="white"} Integer or character scalar. The border
colour for the title panels.
\item \code{col.frame="lightgray"} Integer or character scalar. The line colour
used for the panel frame, if \code{frame==TRUE}
\item \code{col.grid="#808080"} Integer or character scalar. Default line colour
for grid lines, both when \code{type=="g"} in \code{DataTrack}s and when display
parameter \code{grid==TRUE}.
\item \code{col.line=NULL} Integer or character scalar. Default colours for plot
lines. Usually the same as the global col parameter.
\item \code{col.symbol=NULL} Integer or character scalar. Default colours for plot
symbols. Usually the same as the global col parameter.
\item \code{col.title="white"} (Aliases \code{fontcolour.title}) Integer or character
scalar. The border colour for the title panels
\item \code{collapse=TRUE} Boolean controlling whether to collapse the content of
the track to accommodate the minimum current device resolution.
See collapsing for details.
\item \code{fill="lightgray"} Integer or character scalar. Default fill colour
setting for all plotting elements, unless there is a more specific control
defined elsewhere.
\item \code{fontcolour="black"} Integer or character scalar. The font colour for
all text, unless a more specific definition exists.
\item \code{fontface=1} Integer or character scalar. The font face for all text,
unless a more specific definition exists.
\item \code{fontface.title=2} Integer or character scalar. The font face for the
title panels.
\item \code{fontfamily="sans"} Integer or character scalar. The font family for all
text, unless a more specific definition exists.
\item \code{fontfamily.title="sans"} Integer or character scalar. The font family
for the title panels.
\item \code{fontsize=12} Numeric scalar. The font size for all text, unless a more
specific definition exists.
\item \code{frame=FALSE} Boolean. Draw a frame around the track when plotting.
\item \code{grid=FALSE} Boolean, switching on/off the plotting of a grid.
\item \code{h=-1} Integer scalar. Parameter controlling the number of horizontal
grid lines, see panel.grid for details.
\item \code{lineheight=1} Numeric scalar. The font line height for all text, unless
a more specific definition exists.
\item \code{lty="solid"} Numeric scalar. Default line type setting for all plotting
elements, unless there is a more specific control defined elsewhere.
\item \code{lty.grid="solid"} Integer or character scalar. Default line type for
grid lines, both when \code{type=="g"} in \code{DataTrack}s and when display parameter
\code{grid==TRUE}.
\item \code{lwd=1} Numeric scalar. Default line width setting for all plotting
elements, unless there is a more specific control defined elsewhere.
\item \code{lwd.border.title=1} Integer scalar. The border width for the title
panels.
\item \code{lwd.grid=1} Numeric scalar. Default line width for grid lines, both
when \code{type=="g"} in \code{DataTrack}s and when display parameter \code{grid==TRUE}.
\item \code{lwd.title=1} Integer scalar. The border width for the title panels
\item \code{min.distance=1} Numeric scalar. The minimum pixel distance before
collapsing range items, only if \code{collapse==TRUE}. See collapsing for details.
\item \code{min.height=3} Numeric scalar. The minimum range height in pixels to
display. All ranges are expanded to this size in order to avoid rendering
issues. See collapsing for details.
\item \code{min.width=1} Numeric scalar. The minimum range width in pixels to
display. All ranges are expanded to this size in order to avoid rendering
issues. See collapsing for details.
\item \code{reverseStrand=FALSE} Logical scalar. Set up the plotting coordinates
in 3' -> 5' direction if \code{TRUE.} This will effectively mirror the plot
on the vertical axis.
\item \code{rotation=0} The rotation angle for all text unless a more specific
definition exists.
\item \code{rotation.title=90} (Aliases rotation.title) The rotation angle for
the text in the title panel. Even though this can be adjusted, the automatic
resizing of the title panel will currently not work, so use at own risk.
\item \code{showAxis=TRUE} Boolean controlling whether to plot a y axis (only
applies to track types where axes are implemented).
\item \code{showTitle=TRUE} Boolean controlling whether to plot a title panel.
Although this can be set individually for each track, in multi-track plots
as created by \code{plotTracks} there will still be an empty place holder in case
any of the other tracks include a title. The same holds true for axes. Note
that the the title panel background colour could be set to transparent in
order to completely hide it.
\item \code{size=1} Numeric scalar. The relative size of the track. Can be
overridden in the \code{plotTracks} function.
\item \code{v=-1} Integer scalar. Parameter controlling the number of vertical
grid lines, see panel.grid for details.
\item \code{...} additional display parameters are allowed. Those typically
take the value of a valid R colour descriptors. The parameter names will
later be matched to optional track item types as defined in the 'feature'
range attribute, and all tracks of the matched types are coloured
accordingly. See the documentation of the \code{GeneRegionTrack} and
\code{AnnotationTrack} classes as well as grouping for details.
}
}
}
\section{Functions}{
\itemize{
\item \code{initialize(GdObject)}: Initialize the object. This involves setting up a
new environment for the display parameters and filling it up with the current
settings. All arguments that have not been clobbered up by one of the
sub-class initializers are considered to be additional display parameters
and are also added to the environment. See \code{settings} for details on setting
graphical parameters for tracks.

\item \code{setPar(x = GdObject, value = character)}: set the single display parameter name to value.
Note that display parameters in the \code{GdObject-class} are pass-by-reference,
so no re-assignment to the symbol \code{obj} is necessary. See settings for
details on display parameters and customization.

\item \code{setPar(x = GdObject, value = list)}: set display parameters by the values of the named
list in value. Note that display parameters in the \code{GdObject-class} are
pass-by-reference, so no re-assignment to the symbol \code{obj} is necessary.
See settings for details on display parameters and customization.

\item \code{displayPars(x = GdObject) <- value}: set display parameters using the values of the
named list in \code{value}. See \code{settings} for details on display parameters
and customization.

\item \code{getPar(x = GdObject, name = character)}: alias for the \code{displayPars} method.
See \code{settings} for details on display parameters and customization.

\item \code{getPar(x = GdObject, name = missing)}: alias for the \code{displayPars} method.
See \code{settings} for details on display parameters and customization.

\item \code{displayPars(x = GdObject, name = character)}: list the value of the display parameter name.
See \code{settings} for details on display parameters and customization.

\item \code{displayPars(x = GdObject, name = missing)}: list the value of all available display parameters.
See \code{settings} for details on display parameters and customization.

\item \code{coords(GdObject)}: return the coordinates from the internal image map.

\item \code{tags(GdObject)}: return the tags from the internal image map.

\item \code{subset(GdObject)}: subset a \code{GdObject} by coordinates.
Most of the respective sub-classes inheriting from \code{GdObject} overwrite this
method, the default is to return the unaltered input object.

\item \code{names(GdObject)}: return the value of the \code{name} slot.

\item \code{names(x = GdObject) <- value}: set the value of the \code{name} slot.

\item \code{group()}: Generics for \code{group}.

\item \code{group(GdObject) <- value}: Generics for \verb{group<-}.

\item \code{group(GdObject)}: return grouping information for the individual
items in the track. Unless overwritten in one of the sub-classes,
this usually returns \code{NULL}.

\item \code{imageMap()}: Generics for \code{imageMap}.

\item \code{imageMap(GdObject)}: Extract the content of the \code{imageMap} slot.

\item \code{imageMap(GdObject) <- value}: Generics for \verb{imageMap<-}.

\item \code{imageMap(GdObject = GdObject) <- value}: Replace the content of the \code{imageMap} slot.

\item \code{drawAxis()}: Generics for \code{drawAxis}.

\item \code{drawAxis(GdObject)}: add a y-axis to the title panel of a track if
necessary. Unless overwritten in one of the sub-classes this usually
does not plot anything and returns \code{NULL}.

\item \code{drawGrid()}: Generics for \code{drawGrid}.

\item \code{drawGD()}: Generics for \code{drawGD}.

\item \code{gene()}: Generics for \code{gene}.

\item \code{gene(GdObject) <- value}: Generics for \verb{gene<-}.

\item \code{symbol()}: Generics for \code{symbol}.

\item \code{symbol(GdObject) <- value}: Generics for \verb{symbol<-}.

\item \code{transcript()}: Generics for \code{transcript}.

\item \code{transcript(GdObject) <- value}: Generics for \verb{transcript<-}.

\item \code{exon()}: Generics for \code{exon}.

\item \code{exon(GdObject) <- value}: Generics for \verb{exon<-}.

\item \code{feature()}: Generics for \code{feature}.

\item \code{feature(GdObject) <- value}: Generics for \verb{feature<-}.

\item \code{identifier()}: Generics for \code{identifier}.

\item \code{identifier(GdObject) <- value}: Generics for \verb{identifier<-}.

\item \code{chromosome()}: Generics for \code{chromosome}.

\item \code{chromosome(GdObject)}: return the chromosome for which the track is defined.

\item \code{chromosome(GdObject) <- value}: Generics for \code{chromosome}.

\item \code{chromosome(GdObject) <- value}: replace the value of the track's chromosome. This
has to be a valid UCSC chromosome identifier or an integer or character
scalar that can be reasonably coerced into one.

\item \code{position()}: Generics for \code{position}.

\item \code{genome(GdObject)}: return the track's genome.

\item \code{genome(GdObject) <- value}: set the track's genome. Usually this has to be a
valid UCSC identifier, however this is not formally enforced here.

\item \code{consolidateTrack()}: Generics for \code{consolidateTrack}.

\item \code{consolidateTrack(GdObject)}: Consolidate.
Determine whether there is \code{alpha} settings or not, and add this information
as the internal display parameter \code{.__hasAlphaSupport}.

\item \code{stacking()}: Generics for \code{stacking}.

\item \code{stacking(GdObject) <- value}: Generics for \verb{stacking<-}.

\item \code{stacks()}: Generics for \code{stacks}.

\item \code{setStacks()}: Generics for ``.

\item \code{setStacks(GdObject)}: set stacks.

\item \code{setCoverage()}: Generics for ``.

}}
\section{Slots}{

\describe{
\item{\code{dp}}{Object of \code{DisplayPars-class}, the display settings controlling the
look and feel of a track. See settings for details on setting graphical
parameters for tracks.}

\item{\code{name}}{Object of class \code{character}, a human-readable name for the track
that will be used in the track's annotation panel if necessary.}

\item{\code{imageMap}}{Object of \code{ImageMap-class}, containing optional information
for an HTML image map. This will be created by the \code{drawGD} methods when the
track is plotted to a device and is usually not set by the user.}
}}

\examples{
## This is a reference class therefore we show below
## an example from AnnotationTrack:

## An empty object
AnnotationTrack()

## Construct from individual arguments
st <- c(2000000, 2070000, 2100000, 2160000)
ed <- c(2050000, 2130000, 2150000, 2170000)
str <- c("-", "+", "-", "-")
gr <- c("Group1", "Group2", "Group1", "Group3")

annTrack <- AnnotationTrack(
    start = st, end = ed, strand = str, chromosome = 7,
    genome = "hg19", feature = "test", group = gr,
    id = paste("annTrack item", 1:4),
    name = "generic annotation", stacking = "squish"
)
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(annTrack)
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouping.R
\name{grouping}
\alias{grouping}
\title{Grouping of annotation features}
\description{
Many annotation tracks are actually composed of a number of grouped
sub-features, for instance exons in a gene model. This man page highlights
the use of grouping information to build informative annotation plots.
}
\details{
All track objects that inherit from class
\code{\linkS4class{AnnotationTrack}} support the grouping feature. The
information is usually passed on to the constructor function (for
\code{AnnotationTrack} via the \code{groups} argument and for
\code{\linkS4class{GeneRegionTrack}} objects via the \code{exon} argument)
or automatically downloaded from an online annotation repository
(\code{\linkS4class{BiomartGeneRegionTrack}}). Group membership is specified
by a factor vector with as many items as there are annotation items in the
track (i.e., the value of \code{length(track)}. Upon plotting, the grouped
annotation features are displayed together and will not be separated in the
stacking of track items.
}
\seealso{
\code{\linkS4class{AnnotationTrack}}

\code{\linkS4class{BiomartGeneRegionTrack}}

\code{\linkS4class{GeneRegionTrack}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gviz-defunct.R
\name{Gviz-defunct}
\alias{Gviz-defunct}
\title{Defunct functions in package \code{Gviz}}
\description{
These functions are defunct and no longer available.
\subsection{Defunct functions are:}{

(none)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gviz-deprecated.R
\name{Gviz-deprecated}
\alias{Gviz-deprecated}
\title{Deprecated functions in package \code{Gviz}}
\description{
These functions are provided for compatibility with older
versions of \code{Gviz} only, and will be defunct at the next release.
\subsection{The following functions are deprecated and will be made defunct (use the replacement indicated below):}{

(none)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BiomartGeneRegionTrack-class.R
\docType{class}
\name{BiomartGeneRegionTrack-class}
\alias{BiomartGeneRegionTrack-class}
\alias{initialize,BiomartGeneRegionTrack-method}
\alias{BiomartGeneRegionTrack}
\alias{subset,BiomartGeneRegionTrack-method}
\title{BiomartGeneRegionTrack class and methods}
\usage{
\S4method{initialize}{BiomartGeneRegionTrack}(
  .Object,
  start = NULL,
  end = NULL,
  biomart,
  filter = list(),
  range,
  genome = NULL,
  chromosome = NULL,
  strand = NULL,
  featureMap = NULL,
  symbol = NULL,
  gene = NULL,
  transcript = NULL,
  entrez = NULL,
  ...
)

BiomartGeneRegionTrack(
  start = NULL,
  end = NULL,
  biomart,
  chromosome = NULL,
  strand,
  genome = NULL,
  stacking = "squish",
  filters = list(),
  featureMap = NULL,
  name = "BiomartGeneRegionTrack",
  symbol = NULL,
  gene = NULL,
  entrez = NULL,
  transcript = NULL,
  ...
)

\S4method{subset}{BiomartGeneRegionTrack}(x, from, to, chromosome, use.defaults = TRUE, ...)
}
\arguments{
\item{.Object}{.Object}

\item{start}{An integer scalar with the genomic start coordinates for the
gene model range.}

\item{end}{An integer scalar with the genomic end coordinates for the gene
model range.}

\item{biomart}{An optional \code{\linkS4class{Mart}} object providing access
to the EBI Biomart webservice. As default the appropriate Ensembl data
source is selected based on the provided genome and chromosome.}

\item{filter}{filter}

\item{range}{range}

\item{genome}{The genome on which the track's ranges are defined. Usually
this is a valid UCSC genome identifier, however this is not being formally
checked at this point. If no mapping from genome to Biomart Ensembl data
source is possible, the \code{biomart} argument needs to be provided by the
user.}

\item{chromosome}{The chromosome on which the track's genomic ranges are
defined. A valid UCSC chromosome identifier. Please note that at this stage
only syntactic checking takes place, i.e., the argument value needs to be a
single integer, numeric character or a character of the form \code{chrx},
where \code{x} may be any possible string. The user has to make sure that
the respective chromosome is indeed defined for the the track's genome.}

\item{strand}{Character scalar, the strand for which to fetch gene
information from Biomart. One in \code{+}, \code{-}, or \code{+-}.}

\item{featureMap}{Named character vector or list to map between the fields
in the Biomart data base and the features as they are used to construct the
track. If multiple values are provided in a single list item, the package
will use the first one that is defined in the selected Biomart.}

\item{symbol, transcript, gene, entrez}{Character vector giving one or several
gene symbols, Ensembl transcript identifiers, Ensembl gene identifiers, or
ENTREZ gene identifiers, respectively. The genomic locus of their gene model
will be fetch from Biomart instead of providing explicit start and end
coordinates.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters. See \code{\link{settings}} and the "Display Parameters"
section below for details.}

\item{stacking}{The stacking type for overlapping items of the track. One in
\code{c(hide, dense, squish, pack,full)}. Currently, only hide (don't show
the track items, squish (make best use of the available space) and dense (no
stacking at all) are implemented.}

\item{filters}{A list of additional filters to be applied in the Biomart
query. See \code{\link{getBM}} for details.}

\item{name}{Character scalar of the track's name used in the title panel
when plotting.}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}

\item{from, to}{from, to}

\item{use.defaults}{\code{logical}}
}
\value{
The return value of the constructor function is a new object of class
\code{BiomartGeneRegionTrack}.
}
\description{
A class to hold gene model data for a genomic region fetched dynamically
from EBI's Biomart Ensembl data source.
}
\details{
A track containing all gene models in a particular region as fetched from
EBI's Biomart service. Usually the user does not have to take care of the
Biomart connection, which will be established automatically based on the
provided genome and chromosome information. However, for full flexibility a
valid \code{\linkS4class{Mart}} object may be passed on to the constructor.
Please note that this assumes a connection to one of the Ensembl gene data
sources, mapping the available query data back to the internal object slots.
}
\section{Functions}{
\itemize{
\item \code{initialize(BiomartGeneRegionTrack)}: Initialize.

\item \code{BiomartGeneRegionTrack()}: Constructor function for
\code{BiomartGeneRegionTrack-class}.

\item \code{subset(BiomartGeneRegionTrack)}: subset a \code{BiomartGeneRegionTrack}
by coordinates and sort if necessary.

}}
\section{Objects from the class}{


Objects can be created using the constructor function
\code{BiomartGeneRegionTrack}.
}

\examples{
\dontshow{
## Load some sample data
data(bmTrack)
}

## Construct the object
\dontrun{
bmTrack <- BiomartGeneRegionTrack(
    start = 26682683, end = 26711643,
    chromosome = 7, genome = "mm9"
)
}

\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(bmTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(bmTrack)

## Track names
names(bmTrack)
names(bmTrack) <- "foo"
plotTracks(bmTrack)

## Subsetting and splitting
subTrack <- subset(bmTrack, from = 26700000, to = 26705000)
length(subTrack)
subTrack <- bmTrack[transcript(bmTrack) == "ENSMUST00000144140"]
split(bmTrack, transcript(bmTrack))

## Accessors
start(bmTrack)
end(bmTrack)
width(bmTrack)
position(bmTrack)
width(subTrack) <- width(subTrack) + 100

strand(bmTrack)
strand(subTrack) <- "-"

chromosome(bmTrack)
chromosome(subTrack) <- "chrX"

genome(bmTrack)
genome(subTrack) <- "hg19"

range(bmTrack)
ranges(bmTrack)

## Annotation
identifier(bmTrack)
identifier(bmTrack, "lowest")
identifier(subTrack) <- "bar"

feature(bmTrack)
feature(subTrack) <- "foo"

exon(bmTrack)
exon(subTrack) <- letters[1:2]

gene(bmTrack)
gene(subTrack) <- "bar"

symbol(bmTrack)
symbol(subTrack) <- "foo"

transcript(bmTrack)
transcript(subTrack) <- c("foo", "bar")
chromosome(subTrack) <- "chr7"
plotTracks(subTrack)

values(bmTrack)

## Grouping
group(bmTrack)
group(subTrack) <- "Group 1"
transcript(subTrack)
plotTracks(subTrack)

## Stacking
stacking(bmTrack)
stacking(bmTrack) <- "dense"
plotTracks(bmTrack)

## coercion
as(bmTrack, "data.frame")
as(bmTrack, "UCSCData")

## HTML image map
coords(bmTrack)
tags(bmTrack)
bmTrack <- plotTracks(bmTrack)$foo
coords(bmTrack)
tags(bmTrack)
}
\references{
EBI Biomart webservice at \url{http://www.biomart.org}.
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RangeTrack-class.R
\docType{class}
\name{RangeTrack-class}
\alias{RangeTrack-class}
\alias{initialize,RangeTrack-method}
\alias{ranges,RangeTrack-method}
\alias{range,RangeTrack-method}
\alias{seqnames,RangeTrack-method}
\alias{seqlevels,RangeTrack-method}
\alias{seqinfo,RangeTrack-method}
\alias{genome,RangeTrack-method}
\alias{genome<-,RangeTrack-method}
\alias{chromosome,RangeTrack-method}
\alias{chromosome<-,RangeTrack-method}
\alias{start,RangeTrack-method}
\alias{start<-,RangeTrack-method}
\alias{end,RangeTrack-method}
\alias{end<-,RangeTrack-method}
\alias{width,RangeTrack-method}
\alias{width<-,RangeTrack-method}
\alias{min,RangeTrack-method}
\alias{max,RangeTrack-method}
\alias{length,RangeTrack-method}
\alias{strand,RangeTrack-method}
\alias{strand<-,RangeTrack,ANY-method}
\alias{position,RangeTrack-method}
\alias{[,RangeTrack,ANY,ANY,ANY-method}
\alias{subset,RangeTrack-method}
\alias{split,RangeTrack,ANY-method}
\alias{values,RangeTrack-method}
\alias{feature,RangeTrack-method}
\alias{feature<-,RangeTrack,character-method}
\alias{consolidateTrack,RangeTrack-method}
\title{RangeTrack class and methods}
\usage{
\S4method{initialize}{RangeTrack}(.Object, range, chromosome, genome, ...)

\S4method{ranges}{RangeTrack}(x)

\S4method{range}{RangeTrack}(x)

\S4method{seqnames}{RangeTrack}(x)

\S4method{seqlevels}{RangeTrack}(x)

\S4method{seqinfo}{RangeTrack}(x)

\S4method{genome}{RangeTrack}(x)

\S4method{genome}{RangeTrack}(x) <- value

\S4method{chromosome}{RangeTrack}(GdObject)

\S4method{chromosome}{RangeTrack}(GdObject) <- value

\S4method{start}{RangeTrack}(x)

\S4method{start}{RangeTrack}(x) <- value

\S4method{end}{RangeTrack}(x)

\S4method{end}{RangeTrack}(x) <- value

\S4method{width}{RangeTrack}(x)

\S4method{width}{RangeTrack}(x) <- value

\S4method{min}{RangeTrack}(x)

\S4method{max}{RangeTrack}(x)

\S4method{length}{RangeTrack}(x)

\S4method{strand}{RangeTrack}(x)

\S4method{strand}{RangeTrack,ANY}(x) <- value

\S4method{position}{RangeTrack}(GdObject, from = NULL, to = NULL, sort = FALSE, ...)

\S4method{[}{RangeTrack,ANY,ANY,ANY}(x, i, j, ..., drop = TRUE)

\S4method{subset}{RangeTrack}(
  x,
  from = NULL,
  to = NULL,
  sort = FALSE,
  drop = TRUE,
  use.defaults = TRUE,
  ...
)

\S4method{split}{RangeTrack,ANY}(x, f, drop = FALSE, ...)

\S4method{values}{RangeTrack}(x)

\S4method{feature}{RangeTrack}(GdObject)

\S4method{feature}{RangeTrack,character}(GdObject) <- value

\S4method{consolidateTrack}{RangeTrack}(GdObject, chromosome, ...)
}
\arguments{
\item{.Object}{.Object}

\item{range}{range}

\item{chromosome}{the currently active chromosome which may have to be set
for a \code{RangeTrack} or a \code{SequenceTrack} object
parameters}

\item{genome}{genome}

\item{...}{Additional arguments.}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}

\item{value}{Value to be set.}

\item{GdObject}{the input track object}

\item{from, to}{Numeric scalar, giving the range of genomic coordinates to
limit the tracks in. Note that \code{to} cannot be larger than \code{from.}}

\item{sort}{\code{logical}.}

\item{i}{Numeric scalar, index to subset.}

\item{j}{Numeric scalar, index to subset. Ignored.}

\item{drop}{\code{logical}, indicating if levels that do not occur should be dropped (if \code{f} is a factor).}

\item{use.defaults}{\code{logical}.}

\item{f}{\code{factor} in the sense that \code{as.factor(f)} defines the grouping,}
}
\value{
A virtual class: No objects may be created from it.
}
\description{
The virtual parent class for all track items in the Gviz package that
contain some form of genomic ranges (start, end, strand, chromosome
and the associated genome.)
}
\section{Functions}{
\itemize{
\item \code{initialize(RangeTrack)}: Initialize.

\item \code{ranges(RangeTrack)}: return the genomic coordinates for the track
along with all additional annotation information as an object of
class \code{GRanges.}

\item \code{range(RangeTrack)}: return the genomic coordinates for the
track as an object of class IRanges.

\item \code{seqnames(RangeTrack)}: return the track's seqnames.

\item \code{seqlevels(RangeTrack)}: return the track's seqlevels.

\item \code{seqinfo(RangeTrack)}: return the track's seqinfo.

\item \code{genome(RangeTrack)}: return the track's genome.

\item \code{genome(RangeTrack) <- value}: set the track's genome. Usually this has to
be a valid UCSC identifier, however this is not formally enforced here.

\item \code{chromosome(RangeTrack)}: return the chromosome for which the track is defined.

\item \code{chromosome(RangeTrack) <- value}: replace the value of the track's chromosome.
This has to be a valid UCSC chromosome identifier or an integer or character
scalar that can be reasonably coerced into one.

\item \code{start(RangeTrack)}: the start of the track items in genomic
coordinates.

\item \code{start(RangeTrack) <- value}: replace the start of the track items in
genomic coordinates.

\item \code{end(RangeTrack)}: the end of the track items in genomic
coordinates.

\item \code{end(RangeTrack) <- value}: replace the end of the track items in
genomic coordinates.

\item \code{width(RangeTrack)}: the width of the track items in genomic
coordinates.

\item \code{width(RangeTrack) <- value}: replace the width of the track items in genomic
coordinates.

\item \code{min(RangeTrack)}: return the start position for the leftmost range item.

\item \code{max(RangeTrack)}: return the end position for the rightmost range item.

\item \code{length(RangeTrack)}: return the number of items in the track.

\item \code{strand(RangeTrack)}: return a vector of strand specifiers for all
track items, in the form '+' for the Watson strand, '-' for the Crick
strand or '*' for either of the two.

\item \code{strand(x = RangeTrack) <- value}: replace the strand information for the track
items. The replacement value needs to be an appropriate scalar or vector
of strand values.

\item \code{position(RangeTrack)}: the arithmetic mean of the track item's
coordionates, i.e., \code{(end(obj)-start(obj))/2}.

\item \code{x[i}: subset the items in the \code{RangeTrack} object.
This is essentially similar to subsetting of the \code{GRanges} object in the
\code{range} slot. For most applications, the subset method may be more appropriate.

\item \code{subset(RangeTrack)}: subset a \code{RangeTrack} by coordinates and
sort if necessary.

\item \code{split(x = RangeTrack, f = ANY)}: split a \code{RangeTrack} object by an appropriate
factor vector (or another vector that can be coerced into one). The output
of this operation is a list of objects of the same class as the input
object, all inheriting from class \code{RangeTrack.}

\item \code{values(RangeTrack)}: return all additional annotation information
except for the genomic coordinates for the track items as a \code{data.frame}.

\item \code{feature(RangeTrack)}: return the grouping information for track
items. For certain sub-classes, groups may be indicated by different colour
schemes when plotting. See grouping or \code{AnnotationTrack} and
\code{GeneRegionTrack} for details.
@export

\item \code{feature(GdObject = RangeTrack) <- value}: set the grouping information for track items.
This has to be a factor vector (or another type of vector that can be
coerced into one) of the same length as the number of items in the
\code{RangeTrack.} See grouping or \code{AnnotationTrack} and \code{GeneRegionTrack} for
details.
@export

\item \code{consolidateTrack(RangeTrack)}: Consolidate.

}}
\section{Slots}{

\describe{
\item{\code{dp}}{Object of \code{DisplayPars-class}, the display settings controlling the
look and feel of a track. See settings for details on setting graphical
parameters for tracks.}

\item{\code{name}}{Object of class \code{character}, a human-readable name for the track
that will be used in the track's annotation panel if necessary.}

\item{\code{imageMap}}{Object of \code{ImageMap-class}, containing optional information
for an HTML image map. This will be created by the \code{drawGD} methods when the
track is plotted to a device and is usually not set by the user.}

\item{\code{range}}{Object of class \code{GRanges}, the genomic ranges of the track items
as well as additional annotation information in its \code{elementMetaData} slot.
Please not that the slot is actually implemented as a class union between
\code{GRanges} and IRanges to increase efficiency, for instance for DataTrack
objects. This usually does not concern the user.}

\item{\code{chromosome}}{Object of class \code{character}, the chromosome on which the
track is defined. There can only be a single chromosome for one track. For
certain subclasses, the space of allowed chromosome names is limited (e.g.,
only those chromosomes that exist for a particular genome). Throughout the
package, chromosome name have to be entered either as a single integer scalar
or as a character scalar of the form chrXYZ, where XYZ may be an arbitrary
character string.}

\item{\code{genome}}{Object of class \code{character}, the genome for which the track is
defined. For most sub-classes this has to be valid UCSC genome identifier,
however this may not always be formally checked upon object instantiation.}
}}

\examples{
## This is a reference class therefore we show below
## an example from AnnotationTrack

## An empty object
AnnotationTrack()

## Construct from individual arguments
st <- c(2000000, 2070000, 2100000, 2160000)
ed <- c(2050000, 2130000, 2150000, 2170000)
str <- c("-", "+", "-", "-")
gr <- c("Group1", "Group2", "Group1", "Group3")

annTrack <- AnnotationTrack(
    start = st, end = ed, strand = str, chromosome = 7,
    genome = "hg19", feature = "test", group = gr,
    id = paste("annTrack item", 1:4),
    name = "generic annotation", stacking = "squish"
)
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(annTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(annTrack)
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DisplayPars-class.R
\docType{class}
\name{DisplayPars-class}
\alias{DisplayPars-class}
\alias{.DisplayPars}
\alias{DisplayPars}
\alias{getPar}
\alias{getPar,DisplayPars,character-method}
\alias{getPar,DisplayPars,missing-method}
\alias{displayPars}
\alias{displayPars,DisplayPars,missing-method}
\alias{displayPars,DisplayPars,character-method}
\alias{as.list,DisplayPars-method}
\alias{setPar}
\alias{setPar,DisplayPars,list-method}
\alias{setPar,DisplayPars,character-method}
\alias{displayPars<-}
\alias{displayPars<-,DisplayPars,list-method}
\alias{show,DisplayPars-method}
\alias{as.list,InferredDisplayPars-method}
\alias{show,InferredDisplayPars-method}
\alias{availableDisplayPars}
\title{DisplayPars: A class to control the plotting parameters for GdObjects}
\usage{
DisplayPars(...)

getPar(x, name, ...)

\S4method{getPar}{DisplayPars,character}(x, name, asIs = FALSE)

\S4method{getPar}{DisplayPars,missing}(x, hideInternal = TRUE)

displayPars(x, name, ...)

\S4method{displayPars}{DisplayPars,missing}(x, hideInternal = TRUE)

\S4method{displayPars}{DisplayPars,character}(x, name)

\S4method{as.list}{DisplayPars}(x)

setPar(x, value, ...)

\S4method{setPar}{DisplayPars,list}(x, value, interactive = TRUE)

\S4method{setPar}{DisplayPars,character}(x, name, value, interactive = TRUE)

displayPars(x, recursive = FALSE) <- value

\S4method{displayPars}{DisplayPars,list}(x, recursive = FALSE) <- value

\S4method{show}{DisplayPars}(object)

\S4method{as.list}{InferredDisplayPars}(x)

\S4method{show}{InferredDisplayPars}(object)

availableDisplayPars(class)
}
\arguments{
\item{\dots}{All named arguments are stored in the object's environment as
individual parameters, regardless of their type.}

\item{x}{object to set the displayPar value on}

\item{name}{Name of the retrieved parameter.}

\item{asIs}{\code{logical}}

\item{hideInternal}{\code{logical}}

\item{value}{named value to be set}

\item{interactive}{\code{logical}}

\item{recursive}{\code{logical}}

\item{object}{object}

\item{class}{Either character scalar or object. Supported classes are:
\code{GdObject}, \code{GenomeAxisTrack}, \code{RangeTrack}, \code{NumericTrack}, \code{DataTrack},
\code{IdeogramTrack}, \code{StackedTrack}, \code{AnnotationTrack}, \code{DetailsAnnotationTrack},
\code{GeneRegionTrack}, \code{BiomartGeneRegionTrack}, \code{AlignmentsTrack},
\code{SequenceTrack}, \code{SequenceBSgenomeTrack}, \code{SequenceDNAStringSetTrack},
\code{SequenceRNAStringSetTrack}}
}
\value{
The return value of the constructor function is a new object of class
\code{DisplayPars}.

\code{availableDisplayPars} returns a list of the default display
parameters.
}
\description{
All tracks within this package are highly customizable. The
\code{DisplayPars} class facilitates this and provides a unified API to the
customization parameters.

The individual parameters in a \code{DisplayParameters} class are stored as
pointers in an environment. This has the upshot of not having to copy the
whole track object when changing parameters, and parameters can be updated
without the need to explicitly reassign the track to a symbol (i.e.,
updating of parameters happens in place). The downside is that upon copying
of track objects, the parameter environment needs to be re-instantiated.

Objects can be created using the constructor function \code{DisplayPars}.

The default display parameters for a track object class can be queried using
the \code{availableDisplayPars} function.
}
\section{Functions}{
\itemize{
\item \code{DisplayPars()}: Constructor function.

\item \code{getPar()}: Generics for \code{getPar}.

\item \code{getPar(x = DisplayPars, name = character)}: Alias for the \code{displayPars} method.

\item \code{getPar(x = DisplayPars, name = missing)}: Alias for the \code{displayPars} method.

\item \code{displayPars()}: Generics for \code{displayPars}.

\item \code{displayPars(x = DisplayPars, name = missing)}: Returns all available display parameters.

\item \code{displayPars(x = DisplayPars, name = character)}: Returns the value of a subset of display parameters, as identified by \code{name}.

\item \code{as.list(DisplayPars)}: Converts \code{DisplayPars} to \code{list}.

\item \code{setPar()}: Generics for \code{SetPar}.

SetPar generic function

\item \code{setPar(x = DisplayPars, value = list)}: Sets display parameters by the values of the
named \code{list} in value. Note that display parameters in the \code{DisplayPars-class}
are pass-by-reference, so no re-assignment to the symbol \code{obj} is necessary.

\item \code{setPar(x = DisplayPars, value = character)}: set the single display parameter name to value.
Note that display parameters in the DisplayPars class are pass-by-reference,
so no re-assignment to the symbol \code{obj} is necessary.

\item \code{displayPars(x, recursive = FALSE) <- value}: Generics for \verb{displayPars<-}.

\item \code{displayPars(x = DisplayPars) <- value}: Replaces or adds display parameters as provided by the named \code{list} items.

\item \code{show(DisplayPars)}: Show method.

\item \code{as.list(InferredDisplayPars)}: Return \code{InferredDisplayPars} as a \code{list}.

\item \code{show(InferredDisplayPars)}: Show method.

\item \code{availableDisplayPars()}: Get default display parameters.

}}
\section{Slots}{

\describe{
\item{\code{pars}}{an environment or a list containing parameter key value pairs.}
}}

\examples{
## Construct object
dp <- DisplayPars(col = "red", lwd = 2, transformation = log2)
dp

## Query parameters
displayPars(dp)
displayPars(dp, "col")
getPar(dp, c("col", "transformation"))

## Modify parameters
displayPars(dp) <- list(lty = 1, fontsize = 3)
setPar(dp, "pch", 20)
dp
## Default parameters
availableDisplayPars("GenomeAxisTrack")
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HighlightTrack-class.R
\docType{class}
\name{HighlightTrack-class}
\alias{HighlightTrack-class}
\alias{initialize,HighlightTrack-method}
\alias{HighlightTrack}
\alias{displayPars<-,HighlightTrack,list-method}
\alias{length,HighlightTrack-method}
\alias{chromosome<-,HighlightTrack-method}
\alias{setStacks,HighlightTrack-method}
\alias{consolidateTrack,HighlightTrack-method}
\alias{subset,HighlightTrack-method}
\alias{show,HighlightTrack-method}
\title{HighlightTrack class and methods}
\usage{
\S4method{initialize}{HighlightTrack}(.Object, trackList, ...)

HighlightTrack(
  trackList = list(),
  range = NULL,
  start = NULL,
  end = NULL,
  width = NULL,
  chromosome,
  genome,
  name = "HighlightTrack",
  ...
)

\S4method{displayPars}{HighlightTrack,list}(x, recursive = FALSE) <- value

\S4method{length}{HighlightTrack}(x)

\S4method{chromosome}{HighlightTrack}(GdObject) <- value

\S4method{setStacks}{HighlightTrack}(GdObject, ...)

\S4method{consolidateTrack}{HighlightTrack}(GdObject, chromosome, ...)

\S4method{subset}{HighlightTrack}(x, ...)

\S4method{show}{HighlightTrack}(object)
}
\arguments{
\item{.Object}{.Object}

\item{trackList}{A list of Gviz track objects that all have to inherit from
class \code{GdObject}.}

\item{\dots}{All additional parameters are ignored.}

\item{range}{An optional meta argument to handle the different input types. If the
\code{range} argument is missing, all the relevant information to create the
object has to be provided as individual function arguments (see below).

The different input options for \code{range} are:

\describe{

\item{A \code{GRanges} object:}{ the genomic ranges for the highlighting
regions.}

\item{An \code{\linkS4class{IRanges}} object:}{ almost identical to the
\code{GRanges} case, except that the chromosome information has to be
provided in the separate \code{chromosome} argument, because it can not be
directly encoded in an \code{IRanges} object.}

\item{A \code{data.frame} object:}{ the \code{data.frame} needs to contain
at least the two mandatory columns \code{start} and \code{end} with the
range coordinates. It may also contain a \code{chromosome} column with the
chromosome information for each range. If missing, this information will be
drawn from the constructor's \code{chromosome} argument.}

}}

\item{start, end}{An integer scalar with the genomic start or end coordinates
for the highlighting range. Can also be supplied as part of the \code{range}
argument.}

\item{width}{An integer vector of widths for highlighting ranges. This can
be used instead of either \code{start} or \code{end} to specify the range
coordinates.}

\item{chromosome}{The chromosome on which the track's genomic ranges are
defined. A valid UCSC chromosome identifier if
\code{options(ucscChromosomeNames=TRUE)}. Please note that in this case only
syntactic checking takes place, i.e., the argument value needs to be an
integer, numeric character or a character of the form \code{chrx}, where
\code{x} may be any possible string. The user has to make sure that the
respective chromosome is indeed defined for the the track's genome. If not
provided here, the constructor will try to build the chromosome information
based on the available inputs, and as a last resort will fall back to the
value \code{chrNA}. Please note that by definition all objects in the
\code{Gviz} package can only have a single active chromosome at a time
(although internally the information for more than one chromosome may be
present), and the user has to call the \code{chromosome<-} replacement
method in order to change to a different active chromosome.}

\item{genome}{The genome on which the track's ranges are defined. Usually
this is a valid UCSC genome identifier, however this is not being formally
checked at this point. If not provided here the constructor will try to
extract this information from the provided inputs, and eventually will fall
back to the default value of \code{NA}.}

\item{name}{Character scalar of the track's name. This is not really used
and only exists fro completeness.}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}

\item{recursive}{\code{logical}}

\item{value}{Value to be set.}

\item{GdObject}{Object of \code{GdObject-class}.}

\item{object}{object}
}
\value{
The return value of the constructor function is a new object of class
\code{HighlightTrack}.
}
\description{
A container for other track objects from the Gviz package that allows for
the addition of a common highlighting area across tracks.
}
\details{
A track to conceptionally group other Gviz track objects into a meta track
for the sole purpose of overlaying all the contained tracks with the same
highlighting region as defined by the objects genomic ranges. During
rendering the contained tracks will be treated as if they had been provided
to the \code{plotTracks} function as individual objects.
}
\section{Functions}{
\itemize{
\item \code{initialize(HighlightTrack)}: Initialize.

\item \code{HighlightTrack()}: Constructor function for
\code{HighlightTrack-class}.

\item \code{displayPars(x = HighlightTrack) <- value}: set display parameters using the values of
the named list in value. See \code{\link{settings}} for details on display
parameters and customization.

\item \code{length(HighlightTrack)}: return the number of subtracks.

\item \code{chromosome(HighlightTrack) <- value}: replace the value of the track's chromosome.
This has to be a valid UCSC chromosome identifier or an integer or character
scalar that can be reasonably coerced into one.

\item \code{setStacks(HighlightTrack)}: Rrecompute the stacks based on the available
space and on the object's track items and stacking settings.
This really just calls the \code{setStacks} methods for the contained tracks and
only exists for dispatching reasons.

\item \code{consolidateTrack(HighlightTrack)}: Consolidate
For a \code{HighlightTrack} apply the method on each of the subtracks in
the \code{trackList} slot

\item \code{subset(HighlightTrack)}: subset all the contained tracks in an HighlightTrack by coordinates and sort if necessary.

\item \code{show(HighlightTrack)}: Show method.

}}
\section{Objects from the Class}{


Objects can be created using the constructor function \code{HighlightTrack}.
}

\examples{
## Object construction:
set.seed(123)
dat <- runif(100, min = -2, max = 22)
gt <- GenomeAxisTrack()
dt <- DataTrack(data = dat, start = sort(sample(200, 100)), width = 1, genome = "hg19")

ht <- HighlightTrack(trackList = list(gt, dt))
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenomeAxisTrack-class.R
\docType{class}
\name{GenomeAxisTrack-class}
\alias{GenomeAxisTrack-class}
\alias{initialize,GenomeAxisTrack-method}
\alias{GenomeAxisTrack}
\alias{ranges,GenomeAxisTrack-method}
\alias{range,GenomeAxisTrack-method}
\alias{start,GenomeAxisTrack-method}
\alias{start<-,GenomeAxisTrack-method}
\alias{end,GenomeAxisTrack-method}
\alias{end<-,GenomeAxisTrack-method}
\alias{width,GenomeAxisTrack-method}
\alias{length,GenomeAxisTrack-method}
\alias{values,GenomeAxisTrack-method}
\alias{strand,GenomeAxisTrack-method}
\alias{collapseTrack,GenomeAxisTrack-method}
\alias{[,GenomeAxisTrack,ANY,ANY,ANY-method}
\alias{subset,GenomeAxisTrack-method}
\alias{drawGD,GenomeAxisTrack-method}
\alias{show,GenomeAxisTrack-method}
\title{GenomeAxisTrack class and methods}
\usage{
\S4method{initialize}{GenomeAxisTrack}(.Object, range, ids, ...)

GenomeAxisTrack(range = NULL, name = "Axis", id, ...)

\S4method{ranges}{GenomeAxisTrack}(x)

\S4method{range}{GenomeAxisTrack}(x)

\S4method{start}{GenomeAxisTrack}(x)

\S4method{start}{GenomeAxisTrack}(x) <- value

\S4method{end}{GenomeAxisTrack}(x)

\S4method{end}{GenomeAxisTrack}(x) <- value

\S4method{width}{GenomeAxisTrack}(x)

\S4method{length}{GenomeAxisTrack}(x)

\S4method{values}{GenomeAxisTrack}(x)

\S4method{strand}{GenomeAxisTrack}(x)

\S4method{collapseTrack}{GenomeAxisTrack}(
  GdObject,
  min.width = 1,
  min.distance = 0,
  collapse = TRUE,
  diff = .pxResolution(coord = "x"),
  xrange
)

\S4method{[}{GenomeAxisTrack,ANY,ANY,ANY}(x, i, j, ..., drop = TRUE)

\S4method{subset}{GenomeAxisTrack}(x, from = NULL, to = NULL, sort = FALSE, ...)

\S4method{drawGD}{GenomeAxisTrack}(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...)

\S4method{show}{GenomeAxisTrack}(object)
}
\arguments{
\item{range}{Optional object of class \code{GRanges} or \code{IRanges} containing
regions to be highlighted on the axis by coloured boxes.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters. See \code{settings} and the "Display Parameters"
section below for details.}

\item{name}{Character scalar of the track's name used in the title panel
when plotting.}

\item{id}{A character vector of the same length as \code{range} containing
identifiers for the ranges. If missing, the constructor will try to extract
the ids from \code{names(range)}.}

\item{x}{A valid track object class name, or the object itself, in which
case the class is derived directly from it.}
}
\value{
The return value of the constructor function is a new object of class
\code{GenomeAxisTrack}.

Objects can be created using the constructor function \code{GenomeAxisTrack}.
}
\description{
A class representing a customizable genomic axis.
}
\details{
A \code{GenomeAxisTrack} can be customized using the familiar display
parameters. By providing a \code{GRanges} or \code{IRanges} object to the
constructor, ranges on the axis can be further highlighted.

With the \code{scale} display parameter, a small scale indicator can be
shown instead of the entire genomic axis. The scale can either be provided
as a fraction of the plotting region (it will be rounded to the nearest
human readable absolute value) or as an absolute value and is always
displayed in bp, kb, mb or gb units. Note that most display parameters for
the \code{GenomeAxisTrack} are ignored when a scale is used instead of the
full axis. In particular, only the parameters \code{exponent}, \code{alpha},
\code{lwd}, \code{col}, \code{cex}, \code{distFromAxis} and \code{labelPos}
are used.
}
\section{Functions}{
\itemize{
\item \code{initialize(GenomeAxisTrack)}: Intialize.

\item \code{GenomeAxisTrack()}: Constructor

\item \code{ranges(GenomeAxisTrack)}: return the genomic coordinates for the
track along with all additional annotation information as an object of
class \code{GRanges}.

\item \code{range(GenomeAxisTrack)}: return the genomic coordinates for the
track as an object of class \code{IRanges}.
@export

\item \code{start(GenomeAxisTrack)}: return the start coordinates of the track
items.

\item \code{start(GenomeAxisTrack) <- value}: replace the start coordinates of the track
items.

\item \code{end(GenomeAxisTrack)}: return the end coordinates of the track
items.

\item \code{end(GenomeAxisTrack) <- value}: replace the end coordinates of the track
items.

\item \code{width(GenomeAxisTrack)}: return the with of the track items in
genomic coordinates.

\item \code{length(GenomeAxisTrack)}: return the number of items stored in the ranges slot.

\item \code{values(GenomeAxisTrack)}: return all additional annotation information
except for the genomic coordinates for the track items.

\item \code{strand(GenomeAxisTrack)}: return a vector of strand specifiers for
all track items, in the form '+' for the Watson strand, '-' for the Crick
strand or '*' for either of the two.

\item \code{collapseTrack(GenomeAxisTrack)}: preprocess the track before plotting.
This will collapse overlapping track items based on the available resolution
and increase the width and height of all track objects to a minimum value
to avoid rendering issues. See collapsing for details.

\item \code{x[i}: subset the items in the \code{GenomeAxisTrack} object.
This is essentially similar to subsetting of the \code{GRanges} object in the
\code{range} slot. For most applications, the subset method may be more appropriate.

\item \code{subset(GenomeAxisTrack)}: plot subset all the contained tracks in an
\code{GenomeAxisTrack} by coordinates and sort if necessary.

\item \code{drawGD(GenomeAxisTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(GenomeAxisTrack)}: Show method.

}}
\examples{
## Construct object
axTrack <- GenomeAxisTrack(
    name = "Axis",
    range <- IRanges(start = c(100, 300, 800), end = c(150, 400, 1000))
)
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(axTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}


## Plotting
plotTracks(axTrack, from = 0, to = 1100)

## Track names
names(axTrack)
names(axTrack) <- "foo"

## Subsetting and splitting
subTrack <- subset(axTrack, from = 0, to = 500)
length(subTrack)
subTrack[1]
split(axTrack, c(1, 1, 2))

## Accessors
start(axTrack)
end(axTrack)
width(axTrack)

strand(axTrack)

range(axTrack)
ranges(axTrack)

## Annotation
values(axTrack)

## Grouping
group(axTrack)

## HTML image map
coords(axTrack)
tags(axTrack)
axTrack <- plotTracks(axTrack)$foo
coords(axTrack)
tags(axTrack)

## adding an axis to another track
data(cyp2b10)
grTrack <- GeneRegionTrack(
    start = 26682683, end = 26711643,
    rstart = cyp2b10$start, rends = cyp2b10$end, chromosome = 7, genome = "mm9",
    transcript = cyp2b10$transcript, gene = cyp2b10$gene, symbol = cyp2b10$symbol,
    name = "Cyp2b10", strand = cyp2b10$strand
)

plotTracks(list(grTrack, GenomeAxisTrack()))
plotTracks(list(grTrack, GenomeAxisTrack(scale = 0.1)))
plotTracks(list(grTrack, GenomeAxisTrack(scale = 5000)))
plotTracks(list(grTrack, GenomeAxisTrack(scale = 0.5, labelPos = "below")))
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DataTrack-class.R
\docType{class}
\name{DataTrack-class}
\alias{DataTrack-class}
\alias{initialize,DataTrack-method}
\alias{ReferenceDataTrack-class}
\alias{initialize,ReferenceDataTrack-method}
\alias{DataTrack}
\alias{values,DataTrack-method}
\alias{values<-,DataTrack-method}
\alias{strand,DataTrack-method}
\alias{strand<-,DataTrack,ANY-method}
\alias{split,DataTrack,ANY-method}
\alias{feature,DataTrack-method}
\alias{feature<-,DataTrack,character-method}
\alias{collapseTrack,DataTrack-method}
\alias{[,DataTrack,ANY,ANY,ANY-method}
\alias{subset,DataTrack-method}
\alias{subset,ReferenceDataTrack-method}
\alias{drawAxis,DataTrack-method}
\alias{drawGD,DataTrack-method}
\alias{show,DataTrack-method}
\alias{show,ReferenceDataTrack-method}
\title{DataTrack class and methods}
\usage{
\S4method{initialize}{DataTrack}(.Object, data = matrix(), strand, ...)

\S4method{initialize}{ReferenceDataTrack}(
  .Object,
  stream,
  reference,
  mapping = list(),
  args = list(),
  defaults = list(),
  ...
)

DataTrack(
  range = NULL,
  start = NULL,
  end = NULL,
  width = NULL,
  data,
  chromosome,
  strand,
  genome,
  name = "DataTrack",
  importFunction,
  stream = FALSE,
  ...
)

\S4method{values}{DataTrack}(x, all = FALSE)

\S4method{values}{DataTrack}(x) <- value

\S4method{strand}{DataTrack}(x)

\S4method{strand}{DataTrack,ANY}(x) <- value

\S4method{split}{DataTrack,ANY}(x, f, drop = FALSE, ...)

\S4method{feature}{DataTrack}(GdObject)

\S4method{feature}{DataTrack,character}(GdObject) <- value

\S4method{collapseTrack}{DataTrack}(GdObject, diff = .pxResolution(coord = "x"), xrange)

\S4method{[}{DataTrack,ANY,ANY,ANY}(x, i, j, ..., drop = FALSE)

\S4method{subset}{DataTrack}(
  x,
  from = NULL,
  to = NULL,
  sort = FALSE,
  drop = TRUE,
  use.defaults = TRUE,
  ...
)

\S4method{subset}{ReferenceDataTrack}(x, from, to, chromosome, ...)

\S4method{drawAxis}{DataTrack}(GdObject, ...)

\S4method{drawGD}{DataTrack}(GdObject, minBase, maxBase, prepare = FALSE, subset = TRUE, ...)

\S4method{show}{DataTrack}(object)

\S4method{show}{ReferenceDataTrack}(object)
}
\arguments{
\item{.Object}{.Object}

\item{data}{A numeric matrix of data points with the number of columns equal
to the number of coordinates in \code{range}, or a numeric vector of
appropriate length that will be coerced into such a one-row matrix. Each
individual row is supposed to contain data for a given sample, where the
coordinates for each single observation are constant across samples.
Depending on the plotting type of the data (see 'Details' and 'Display
Parameters' sections), sample grouping or data aggregation may be available.
Alternatively, this can be a character vector of column names that point
into the element metadata of the \code{range} object for subsetting.
Naturally, this is only supported when the \code{range} argument is of class
\code{GRanges}.}

\item{strand}{Character vector, the strand information for the individual
track items. Currently this has to be unique for the whole track and doesn't
really have any visible consequences, but we might decide to make
\code{DataTracks} strand-specific at a later stage.}

\item{\dots}{Additional items which will all be interpreted as further
display parameters.}

\item{stream}{A logical flag indicating that the user-provided import
function can deal with indexed files and knows how to process the additional
\code{selection} argument when accessing the data on disk. This causes the
constructor to return a \code{ReferenceDataTrack} object which will grab the
necessary data on the fly during each plotting operation.}

\item{range}{An optional meta argument to handle the different input types.
If the \code{range} argument is missing, all the relevant information to create
the object has to be provided as individual function arguments (see below).

The different input options for \code{range} are:

\describe{

\item{A \code{GRanges} object:}{ essentially all the necessary information
to create a \code{DataTrack} can be contained in a single \code{GRanges}
object. The track's coordinates are taken from the \code{start}, \code{end}
and \code{seqnames} slots, the genome information from the genome slot, and
the numeric data values can be extracted from additional metadata columns
columns (please note that non-numeric columns are being ignored with a
warning). As a matter of fact, calling the constructor on a \code{GRanges}
object without further arguments, e.g.  \code{DataTrack(range=obj)} is
equivalent to calling the coerce method \code{as(obj, "DataTrack")}.
Alternatively, the \code{GRanges} object may only contain the coordinate
information, in which case the numeric data part is expected to be present
in the separate \code{data} argument, and the ranges have to match the
dimensions of the data matrix. If \code{data} is not \code{NULL}, this will
always take precedence over anything defined in the \code{range} argument.
See below for details.}

\item{An \code{\linkS4class{IRanges}} object:}{ this is very similar to the
above case, except that the numeric data part now always has to be provided
in the separate \code{data} argument. Also the chromosome information must
be provided in the \code{chromosome} argument, because neither of the two
can be directly encoded in an \code{IRange} object.}

\item{A \code{data.frame} object:}{ the \code{data.frame} needs to contain
at least the two mandatory columns \code{start} and \code{end} with the
range coordinates. It may also contain a \code{chromosome} column with the
chromosome information for each range. If missing it will be drawn from the
separate \code{chromosome} argument. All additional numeric columns will be
interpreted as data columns, unless the \code{data} argument is explicitely
provided.}

\item{A \code{character} scalar:}{ in this case the value of the
\code{range} argument is considered to be a file path to an annotation file
on disk. A range of file types are supported by the \code{Gviz} package as
identified by the file extension. See the \code{importFunction}
documentation below for further details.}

}}

\item{start, end, width}{Integer vectors, giving the start and the end end
coordinates for the individual track items, or their width. Two of the three
need to be specified, and have to be of equal length or of length one, in
which case the single value will be recycled accordingly. Otherwise, the
usual R recycling rules for vectors do not apply and the function will cast
an error.}

\item{chromosome}{The chromosome on which the track's genomic ranges are
defined. A valid UCSC chromosome identifier if
\code{options(ucscChromosomeNames=TRUE)}. Please note that in this case only
syntactic checking takes place, i.e., the argument value needs to be an
integer, numeric character or a character of the form \code{chrx}, where
\code{x} may be any possible string. The user has to make sure that the
respective chromosome is indeed defined for the the track's genome. If not
provided here, the constructor will try to construct the chromosome
information based on the available inputs, and as a last resort will fall
back to the value \code{chrNA}. Please note that by definition all objects
in the \code{Gviz} package can only have a single active chromosome at a
time (although internally the information for more than one chromosome may
be present), and the user has to call the \code{chromosome<-} replacement
method in order to change to a different active chromosome.}

\item{genome}{The genome on which the track's ranges are defined. Usually
this is a valid UCSC genome identifier, however this is not being formally
checked at this point. If not provided here the constructor will try to
extract this information from the provided input, and eventually will fall
back to the default value of \code{NA}.}

\item{name}{Character scalar of the track's name used in the title panel
when plotting.}

\item{importFunction}{A user-defined function to be used to import the data
from a file. This only applies when the \code{range} argument is a character
string with the path to the input data file. The function needs to accept an
argument \code{file} containing the file path and has to return a proper
\code{GRanges} object with the data part attached as numeric metadata
columns. Essentially the process is equivalent to constructing a
\code{DataTrack} directly from a \code{GRanges} object in that non-numeric
columns will be dropped, and further subsetting can be archived by means of
the \code{data} argument. A set of default import functions is already
implemented in the package for a number of different file types, and one of
these defaults will be picked automatically based on the extension of the
input file name. If the extension can not be mapped to any of the existing
import function, an error is raised asking for a user-defined import
function. Currently the following file types can be imported with the
default functions: \code{wig}, \code{bigWig/bw}, \code{bedGraph} and
\code{bam}.

Some file types support indexing by genomic coordinates (e.g., \code{bigWig}
and \code{bam}), and it makes sense to only load the part of the file that
is needed for plotting. To this end, the \code{Gviz} package defines the
derived \code{ReferenceDataTrack} class, which supports streaming data from
the file system. The user typically does not have to deal with this
distinction but may rely on the constructor function to make the right
choice as long as the default import functions are used. However, once a
user-defined import function has been provided and if this function adds
support for indexed files, you will have to make the constructor aware of
this fact by setting the \code{stream} argument to \code{TRUE}. Please note
that in this case the import function needs to accept a second mandatory
argument \code{selection} which is a \code{GRanges} object containing the
dimensions of the plotted genomic range. As before, the function has to
return an appropriate \code{GRanges} object.}

\item{value}{Value to be set.}

\item{GdObject}{Object of \code{GdObject-class}.}
}
\value{
The return value of the constructor function is a new object of class
\code{DataTrack} or \code{ReferenceDataTrack}.
}
\description{
A class to store numeric data values along genomic coordinates. Multiple
samples as well as sample groupings are supported, with the restriction of
equal genomic coordinates for a single observation across samples.
}
\details{
Depending on the setting of the \code{type} display parameter, the data can
be plotted in various different forms as well as combinations thereof.
Supported plotting types are:

\describe{

\item{\code{p}:}{ simple xy-plot.}

\item{\code{l}:}{ lines plot. In the case of multiple samples this plotting
type is not overly usefull since the points in the data matrix are connected
in column-wise order. Type \code{a} might be more appropriate in these
situations.}

\item{\code{b}:}{ combination of xy-plot and lines plot.}

\item{\code{a}:}{ lines plot of the column-wise average values.}

\item{\code{s}:}{ sort and connect data points along the x-axis}

\item{\code{S}:}{ sort and connect data points along the y-axis}

\item{\code{g}:}{ add grid lines. To ensure a consitant look and feel across
multiple tracks, grid lines should preferentially be added by using the
\code{grid} display parameter.}

\item{\code{r}:}{ add a regression line to the plot.}

\item{\code{h}:}{ histogram-like vertical lines centered in the middle of
the coordinate ranges.}

\item{\code{smooth}:}{ add a loess fit to the plot. The following display
parameters can be used to control the loess calculation: \code{span, degree,
family, evaluation}. See \code{\link{panel.loess}} for details.}

\item{\code{histogram}:}{ plot data as a histogram, where the width of the
histogram bars reflects the width of the genomic ranges in the \code{range}
slot.}

\item{\code{mountain}:}{ plot a smoothed version of the data relative to a
baseline, as defined by the \code{baseline} display parameter. The following
display parameters can be used to control the smoothing: \code{span, degree,
family, evaluation}. See \code{\link{panel.loess}} for details. The layout
of the plot can be further customized via the following display parameters:
\code{col.mountain, lwd.mountain, lty.mountain, fill.mountain}.}

\item{\code{polygon}:}{ plot data as a polygon (similar to
\code{mountain}-type but without smoothing). Data are plotted relative to a
baseline, as defined by the \code{baseline} display parameter. The layout of
the plot can be further customized via the following display parameters:
\code{col.mountain, lwd.mountain, lty.mountain, fill.mountain}.}

\item{\code{boxplot}:}{ plot the data as box-and-whisker plots. The layout
of the plot can be further customized via the following display parameters:
\code{box.ratio, box.width, varwidt, notch, notch.frac, levels.fos, stats,
coef, do.out}. See \code{\link{panel.bwplot}} for details.}

\item{\code{gradient}:}{ collapse the data across samples and plot this
average value as a color-coded gradient. Essenitally this is similar to the
heatmap-type plot of a single sample. The layout of the plot can be further
customized via the display parameters \code{ncolor} and \code{gradient}
which control the number of gradient colors as well as the gradient base
colors, respectively.}

\item{\code{heatmap}:}{ plot the color-coded values for all samples in the
form of a heatmap. The data for individual samples can be visually separated
by setting the \code{separator} display parameter. It's value is taken as
the amount of spacing in pixels in between two heatmap rows. The layout of
the plot can be further customized via the display parameters \code{ncolor}
and \code{gradient} which control the number of gradient colors as well as
the gradient base colors, respectively.}

\item{\code{horizon}:}{ plot continuous data by cutting the y range into
segments and overplotting them with color representing the magnitude and
direction of deviation. This is particularly useful when comparing multiple
samples, in which case the horizon strips are stacked. See
\code{\link{horizonplot}} for details. Please note that the \code{origin}
and \code{horizonscale} arguments of the Lattice \code{horizonplot} function
are available as display parameters \code{horizon.origin} and
\code{horizon.scale}.}

}

For some of the above plotting-types the \code{groups} display parameter can
be used to indicate sample sub-groupings. Its value is supposed to be a
factor vector of similar length as the number of samples. In most cases, the
groups are shown in different plotting colors and data aggregation
operations are done in a stratified fashion.

The \code{window} display parameter can be used to aggregate the data prior
to plotting. Its value is taken as the number of equal-sized windows along
the genomic coordinates of the track for which to compute average values.
The special value \code{auto} can be used to automatically determine a
reasonable number of windows which can be particularly useful when plotting
very large genomic regions with many data points.

The \code{aggregation} parameter can be set to define the aggregation
function to be used when averaging in windows or across collapsed items. It
takes the form of either a function which should condense a numeric vector
into a single number, or one of the predefined options as character scalars
\code{"mean"}, \code{"median"} or \code{"sum"} for mean, median or
summation, respectively. Defaults to computing mean values for each sample.
Note that the predefined options can be much faster because they are
optimized to work on large numeric tables.
}
\section{Functions}{
\itemize{
\item \code{initialize(DataTrack)}: Initialize.

\item \code{ReferenceDataTrack-class}: The file-based version of the \code{DataTrack-class}.

\item \code{initialize(ReferenceDataTrack)}: Initialize.

\item \code{DataTrack()}: Constructor function for \code{DataTrack-class}

\item \code{values(DataTrack)}: return the raw data values of the object, i.e.,
the data matrix in the \code{data} slot.

\item \code{values(DataTrack) <- value}: Replace the data matrix in the \code{data} slot.

\item \code{strand(DataTrack)}: return a vector of strand specifiers for all
track items, in the form '+' for the Watson strand, '-' for the Crick
strand or '*' for either of the two.

\item \code{strand(x = DataTrack) <- value}: replace the strand information for the track items.
The replacement value needs to be an appropriate scalar or vector of strand values.

\item \code{split(x = DataTrack, f = ANY)}: Split a \code{DataTrack} object by an appropriate
factor vector (or another vector that can be coerced into one).
The output of this operation is a list of \code{DataTrack} objects.

\item \code{feature(DataTrack)}: returns NULL since there is no grouping
information for the ranges in a \code{DataTrack}.

\item \code{feature(GdObject = DataTrack) <- value}: this return the unaltered input object since
there is no grouping information for the ranges in a \code{DataTrack}.

\item \code{collapseTrack(DataTrack)}: preprocess the track before plotting.
This will collapse overlapping track items based on the available resolution
and increase the width and height of all track objects to a minimum value
to avoid rendering issues. See collapsing for details.

\item \code{x[i}: subset the items in the \code{DataTrack} object.
This is essentially similar to subsetting of the \code{GRanges} object in the
\code{range} slot. For most applications, the subset method may be more appropriate.

\item \code{subset(DataTrack)}: Subset a \code{DataTrack} by coordinates
and sort if necessary.

\item \code{subset(ReferenceDataTrack)}: Subset a \code{ReferenceDataTrack} by coordinates
and sort if necessary.

\item \code{drawAxis(DataTrack)}: add a y-axis to the title panel of a track.

\item \code{drawGD(DataTrack)}: plot the object to a graphics device.
The return value of this method is the input object, potentially updated
during the plotting operation. Internally, there are two modes in which the
method can be called. Either in 'prepare' mode, in which case no plotting is
done but the object is preprocessed based on the available space, or in
'plotting' mode, in which case the actual graphical output is created.
Since subsetting of the object can be potentially costly, this can be
switched off in case subsetting has already been performed before or
is not necessary.

\item \code{show(DataTrack)}: Show method.

\item \code{show(ReferenceDataTrack)}: Show method.

}}
\section{Objects from the class}{


Objects can be created using the constructor function \code{DataTrack}.
}

\examples{
## Object construction:

## An empty object
DataTrack()

## from individual arguments
dat <- matrix(runif(400), nrow = 4)
dtTrack <- DataTrack(
    start = seq(1, 1000, len = 100), width = 10, data = dat,
    chromosome = 1, genome = "mm9", name = "random data"
)

## from GRanges
library(GenomicRanges)
gr <- GRanges(seqnames = "chr1", ranges = IRanges(seq(1, 1000, len = 100),
    width = 10
))
values(gr) <- t(dat)
dtTrack <- DataTrack(range = gr, genome = "mm9", name = "random data")

## from IRanges
dtTrack <- DataTrack(
    range = ranges(gr), data = dat, genome = "mm9",
    name = "random data", chromosome = 1
)

## from a data.frame
df <- as.data.frame(gr)
colnames(df)[1] <- "chromosome"
dtTrack <- DataTrack(range = df, genome = "mm9", name = "random data")
\dontshow{
## For some annoying reason the postscript device does not know about
## the sans font
if (!interactive()) {
    font <- ps.options()$family
    displayPars(dtTrack) <- list(fontfamily = font, fontfamily.title = font)
}
}

## Plotting
plotTracks(dtTrack)

## Track names
names(dtTrack)
names(dtTrack) <- "foo"
plotTracks(dtTrack)

## Subsetting and splitting
subTrack <- subset(dtTrack, from = 100, to = 300)
length(subTrack)
subTrack[1:2, ]
subTrack[, 1:2]
split(dtTrack, rep(1:2, each = 50))

## Accessors
start(dtTrack)
end(dtTrack)
width(dtTrack)
position(dtTrack)
width(subTrack) <- width(subTrack) - 5

strand(dtTrack)
strand(subTrack) <- "-"

chromosome(dtTrack)
chromosome(subTrack) <- "chrX"

genome(dtTrack)
genome(subTrack) <- "mm9"

range(dtTrack)
ranges(dtTrack)

## Data
values(dtTrack)
score(dtTrack)

## coercion
as(dtTrack, "data.frame")
}
\seealso{
\code{\linkS4class{DisplayPars}}

\code{\linkS4class{GdObject}}

\code{\linkS4class{GRanges}}

\code{\linkS4class{HighlightTrack}}

\code{\linkS4class{ImageMap}}

\code{\linkS4class{IRanges}}

\code{\linkS4class{RangeTrack}}

\code{\linkS4class{DataTrack}}

\code{\link{collapsing}}

\code{\link{grouping}}

\code{\link{panel.grid}}

\code{\link{plotTracks}}

\code{\link{settings}}
}
\author{
Florian Hahne
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exportTracks.R
\name{exportTracks}
\alias{exportTracks}
\title{Export Gviz tracks into an annotation file representation.}
\usage{
exportTracks(tracks, range, chromosome, file)
}
\arguments{
\item{tracks}{A list of annotation track objects to be exported into a
single BED file.}

\item{range}{A numeric vector or length 2. The genomic range to display when
opening the file in a browser.}

\item{chromosome}{The chromosome to display when opening the file in a browser.}

\item{file}{Character, the path to the file to write into.}
}
\value{
The function is called for its side effect of writing to a file.
}
\description{
This function is still a bit experimental. Write all tracks provided as a
list \code{tracks} into a single BED file. So far only BED export is supported.
}
\examples{
## export AnnotationTrack to BED file
at <- AnnotationTrack(start = seq(1, 10), width = 1, chromosome = "chr1")
exportTracks(list(at),
    range = c(1, 10), chromosome = "chr1",
    file = paste0(tempfile(), ".bed")
)
}
\author{
Florian Hahne
}
